"use strict";
(self["webpackChunkclimb_onyx_gui_extension"] = self["webpackChunkclimb_onyx_gui_extension"] || []).push([["vendors-node_modules_climb-onyx-gui_dist_climb-onyx-gui_js"],{

/***/ "./node_modules/climb-onyx-gui/dist/climb-onyx-gui.js":
/*!************************************************************!*\
  !*** ./node_modules/climb-onyx-gui/dist/climb-onyx-gui.js ***!
  \************************************************************/
/***/ ((__unused_webpack_module, __webpack_exports__, __webpack_require__) => {

__webpack_require__.r(__webpack_exports__);
/* harmony export */ __webpack_require__.d(__webpack_exports__, {
/* harmony export */   "default": () => (/* binding */ I_)
/* harmony export */ });
/* harmony import */ var react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(/*! react/jsx-runtime */ "./node_modules/react/jsx-runtime.js");
/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1__ = __webpack_require__(/*! react */ "webpack/sharing/consume/default/react");
/* harmony import */ var react__WEBPACK_IMPORTED_MODULE_1___default = /*#__PURE__*/__webpack_require__.n(react__WEBPACK_IMPORTED_MODULE_1__);
var gE = (a, r, u) => {
  if (!r.has(a))
    throw TypeError("Cannot " + u);
};
var $ = (a, r, u) => (gE(a, r, "read from private field"), u ? u.call(a) : r.get(a)), ut = (a, r, u) => {
  if (r.has(a))
    throw TypeError("Cannot add the same private member more than once");
  r instanceof WeakSet ? r.add(a) : r.set(a, u);
}, Ye = (a, r, u, f) => (gE(a, r, "write to private field"), f ? f.call(a, u) : r.set(a, u), u);
var B0 = (a, r, u, f) => ({
  set _(c) {
    Ye(a, r, c, u);
  },
  get _() {
    return $(a, r, f);
  }
}), Mt = (a, r, u) => (gE(a, r, "access private method"), u);



function ym(a) {
  return a && a.__esModule && Object.prototype.hasOwnProperty.call(a, "default") ? a.default : a;
}
var zO = { exports: {} };
/*!
	Copyright (c) 2018 Jed Watson.
	Licensed under the MIT License (MIT), see
	http://jedwatson.github.io/classnames
*/
(function(a) {
  (function() {
    var r = {}.hasOwnProperty;
    function u() {
      for (var v = "", h = 0; h < arguments.length; h++) {
        var C = arguments[h];
        C && (v = c(v, f(C)));
      }
      return v;
    }
    function f(v) {
      if (typeof v == "string" || typeof v == "number")
        return v;
      if (typeof v != "object")
        return "";
      if (Array.isArray(v))
        return u.apply(null, v);
      if (v.toString !== Object.prototype.toString && !v.toString.toString().includes("[native code]"))
        return v.toString();
      var h = "";
      for (var C in v)
        r.call(v, C) && v[C] && (h = c(h, C));
      return h;
    }
    function c(v, h) {
      return h ? v ? v + " " + h : v + h : v;
    }
    a.exports ? (u.default = u, a.exports = u) : window.classNames = u;
  })();
})(zO);
var hB = zO.exports;
const Ke = /* @__PURE__ */ ym(hB);
var XE = { exports: {} }, V0 = { exports: {} }, Qt = {};
/** @license React v16.13.1
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var KI;
function mB() {
  if (KI)
    return Qt;
  KI = 1;
  var a = typeof Symbol == "function" && Symbol.for, r = a ? Symbol.for("react.element") : 60103, u = a ? Symbol.for("react.portal") : 60106, f = a ? Symbol.for("react.fragment") : 60107, c = a ? Symbol.for("react.strict_mode") : 60108, v = a ? Symbol.for("react.profiler") : 60114, h = a ? Symbol.for("react.provider") : 60109, C = a ? Symbol.for("react.context") : 60110, b = a ? Symbol.for("react.async_mode") : 60111, E = a ? Symbol.for("react.concurrent_mode") : 60111, w = a ? Symbol.for("react.forward_ref") : 60112, R = a ? Symbol.for("react.suspense") : 60113, A = a ? Symbol.for("react.suspense_list") : 60120, D = a ? Symbol.for("react.memo") : 60115, L = a ? Symbol.for("react.lazy") : 60116, M = a ? Symbol.for("react.block") : 60121, k = a ? Symbol.for("react.fundamental") : 60117, B = a ? Symbol.for("react.responder") : 60118, Y = a ? Symbol.for("react.scope") : 60119;
  function z(V) {
    if (typeof V == "object" && V !== null) {
      var Q = V.$$typeof;
      switch (Q) {
        case r:
          switch (V = V.type, V) {
            case b:
            case E:
            case f:
            case v:
            case c:
            case R:
              return V;
            default:
              switch (V = V && V.$$typeof, V) {
                case C:
                case w:
                case L:
                case D:
                case h:
                  return V;
                default:
                  return Q;
              }
          }
        case u:
          return Q;
      }
    }
  }
  function Z(V) {
    return z(V) === E;
  }
  return Qt.AsyncMode = b, Qt.ConcurrentMode = E, Qt.ContextConsumer = C, Qt.ContextProvider = h, Qt.Element = r, Qt.ForwardRef = w, Qt.Fragment = f, Qt.Lazy = L, Qt.Memo = D, Qt.Portal = u, Qt.Profiler = v, Qt.StrictMode = c, Qt.Suspense = R, Qt.isAsyncMode = function(V) {
    return Z(V) || z(V) === b;
  }, Qt.isConcurrentMode = Z, Qt.isContextConsumer = function(V) {
    return z(V) === C;
  }, Qt.isContextProvider = function(V) {
    return z(V) === h;
  }, Qt.isElement = function(V) {
    return typeof V == "object" && V !== null && V.$$typeof === r;
  }, Qt.isForwardRef = function(V) {
    return z(V) === w;
  }, Qt.isFragment = function(V) {
    return z(V) === f;
  }, Qt.isLazy = function(V) {
    return z(V) === L;
  }, Qt.isMemo = function(V) {
    return z(V) === D;
  }, Qt.isPortal = function(V) {
    return z(V) === u;
  }, Qt.isProfiler = function(V) {
    return z(V) === v;
  }, Qt.isStrictMode = function(V) {
    return z(V) === c;
  }, Qt.isSuspense = function(V) {
    return z(V) === R;
  }, Qt.isValidElementType = function(V) {
    return typeof V == "string" || typeof V == "function" || V === f || V === E || V === v || V === c || V === R || V === A || typeof V == "object" && V !== null && (V.$$typeof === L || V.$$typeof === D || V.$$typeof === h || V.$$typeof === C || V.$$typeof === w || V.$$typeof === k || V.$$typeof === B || V.$$typeof === Y || V.$$typeof === M);
  }, Qt.typeOf = z, Qt;
}
var Jt = {};
/** @license React v16.13.1
 * react-is.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var qI;
function gB() {
  return qI || (qI = 1,  true && function() {
    var a = typeof Symbol == "function" && Symbol.for, r = a ? Symbol.for("react.element") : 60103, u = a ? Symbol.for("react.portal") : 60106, f = a ? Symbol.for("react.fragment") : 60107, c = a ? Symbol.for("react.strict_mode") : 60108, v = a ? Symbol.for("react.profiler") : 60114, h = a ? Symbol.for("react.provider") : 60109, C = a ? Symbol.for("react.context") : 60110, b = a ? Symbol.for("react.async_mode") : 60111, E = a ? Symbol.for("react.concurrent_mode") : 60111, w = a ? Symbol.for("react.forward_ref") : 60112, R = a ? Symbol.for("react.suspense") : 60113, A = a ? Symbol.for("react.suspense_list") : 60120, D = a ? Symbol.for("react.memo") : 60115, L = a ? Symbol.for("react.lazy") : 60116, M = a ? Symbol.for("react.block") : 60121, k = a ? Symbol.for("react.fundamental") : 60117, B = a ? Symbol.for("react.responder") : 60118, Y = a ? Symbol.for("react.scope") : 60119;
    function z(Ce) {
      return typeof Ce == "string" || typeof Ce == "function" || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
      Ce === f || Ce === E || Ce === v || Ce === c || Ce === R || Ce === A || typeof Ce == "object" && Ce !== null && (Ce.$$typeof === L || Ce.$$typeof === D || Ce.$$typeof === h || Ce.$$typeof === C || Ce.$$typeof === w || Ce.$$typeof === k || Ce.$$typeof === B || Ce.$$typeof === Y || Ce.$$typeof === M);
    }
    function Z(Ce) {
      if (typeof Ce == "object" && Ce !== null) {
        var Ot = Ce.$$typeof;
        switch (Ot) {
          case r:
            var _t = Ce.type;
            switch (_t) {
              case b:
              case E:
              case f:
              case v:
              case c:
              case R:
                return _t;
              default:
                var Et = _t && _t.$$typeof;
                switch (Et) {
                  case C:
                  case w:
                  case L:
                  case D:
                  case h:
                    return Et;
                  default:
                    return Ot;
                }
            }
          case u:
            return Ot;
        }
      }
    }
    var V = b, Q = E, P = C, U = h, se = r, Se = w, he = f, xe = L, ye = D, Me = u, We = v, Ee = c, tt = R, ue = !1;
    function we(Ce) {
      return ue || (ue = !0, console.warn("The ReactIs.isAsyncMode() alias has been deprecated, and will be removed in React 17+. Update your code to use ReactIs.isConcurrentMode() instead. It has the exact same API.")), W(Ce) || Z(Ce) === b;
    }
    function W(Ce) {
      return Z(Ce) === E;
    }
    function ce(Ce) {
      return Z(Ce) === C;
    }
    function ge(Ce) {
      return Z(Ce) === h;
    }
    function Ie(Ce) {
      return typeof Ce == "object" && Ce !== null && Ce.$$typeof === r;
    }
    function Ae(Ce) {
      return Z(Ce) === w;
    }
    function De(Ce) {
      return Z(Ce) === f;
    }
    function Oe(Ce) {
      return Z(Ce) === L;
    }
    function Ue(Ce) {
      return Z(Ce) === D;
    }
    function He(Ce) {
      return Z(Ce) === u;
    }
    function nt(Ce) {
      return Z(Ce) === v;
    }
    function vt(Ce) {
      return Z(Ce) === c;
    }
    function At(Ce) {
      return Z(Ce) === R;
    }
    Jt.AsyncMode = V, Jt.ConcurrentMode = Q, Jt.ContextConsumer = P, Jt.ContextProvider = U, Jt.Element = se, Jt.ForwardRef = Se, Jt.Fragment = he, Jt.Lazy = xe, Jt.Memo = ye, Jt.Portal = Me, Jt.Profiler = We, Jt.StrictMode = Ee, Jt.Suspense = tt, Jt.isAsyncMode = we, Jt.isConcurrentMode = W, Jt.isContextConsumer = ce, Jt.isContextProvider = ge, Jt.isElement = Ie, Jt.isForwardRef = Ae, Jt.isFragment = De, Jt.isLazy = Oe, Jt.isMemo = Ue, Jt.isPortal = He, Jt.isProfiler = nt, Jt.isStrictMode = vt, Jt.isSuspense = At, Jt.isValidElementType = z, Jt.typeOf = Z;
  }()), Jt;
}
var eA;
function Cx() {
  return eA || (eA = 1,  false ? 0 : V0.exports = gB()), V0.exports;
}
/*
object-assign
(c) Sindre Sorhus
@license MIT
*/
var yE, tA;
function yB() {
  if (tA)
    return yE;
  tA = 1;
  var a = Object.getOwnPropertySymbols, r = Object.prototype.hasOwnProperty, u = Object.prototype.propertyIsEnumerable;
  function f(v) {
    if (v == null)
      throw new TypeError("Object.assign cannot be called with null or undefined");
    return Object(v);
  }
  function c() {
    try {
      if (!Object.assign)
        return !1;
      var v = new String("abc");
      if (v[5] = "de", Object.getOwnPropertyNames(v)[0] === "5")
        return !1;
      for (var h = {}, C = 0; C < 10; C++)
        h["_" + String.fromCharCode(C)] = C;
      var b = Object.getOwnPropertyNames(h).map(function(w) {
        return h[w];
      });
      if (b.join("") !== "0123456789")
        return !1;
      var E = {};
      return "abcdefghijklmnopqrst".split("").forEach(function(w) {
        E[w] = w;
      }), Object.keys(Object.assign({}, E)).join("") === "abcdefghijklmnopqrst";
    } catch {
      return !1;
    }
  }
  return yE = c() ? Object.assign : function(v, h) {
    for (var C, b = f(v), E, w = 1; w < arguments.length; w++) {
      C = Object(arguments[w]);
      for (var R in C)
        r.call(C, R) && (b[R] = C[R]);
      if (a) {
        E = a(C);
        for (var A = 0; A < E.length; A++)
          u.call(C, E[A]) && (b[E[A]] = C[E[A]]);
      }
    }
    return b;
  }, yE;
}
var bE, nA;
function Sx() {
  if (nA)
    return bE;
  nA = 1;
  var a = "SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED";
  return bE = a, bE;
}
var CE, rA;
function GO() {
  return rA || (rA = 1, CE = Function.call.bind(Object.prototype.hasOwnProperty)), CE;
}
var SE, aA;
function bB() {
  if (aA)
    return SE;
  aA = 1;
  var a = function() {
  };
  if (true) {
    var r = Sx(), u = {}, f = GO();
    a = function(v) {
      var h = "Warning: " + v;
      typeof console < "u" && console.error(h);
      try {
        throw new Error(h);
      } catch {
      }
    };
  }
  function c(v, h, C, b, E) {
    if (true) {
      for (var w in v)
        if (f(v, w)) {
          var R;
          try {
            if (typeof v[w] != "function") {
              var A = Error(
                (b || "React class") + ": " + C + " type `" + w + "` is invalid; it must be a function, usually from the `prop-types` package, but received `" + typeof v[w] + "`.This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`."
              );
              throw A.name = "Invariant Violation", A;
            }
            R = v[w](h, w, b, C, null, r);
          } catch (L) {
            R = L;
          }
          if (R && !(R instanceof Error) && a(
            (b || "React class") + ": type specification of " + C + " `" + w + "` is invalid; the type checker function must return `null` or an `Error` but returned a " + typeof R + ". You may have forgotten to pass an argument to the type checker creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and shape all require an argument)."
          ), R instanceof Error && !(R.message in u)) {
            u[R.message] = !0;
            var D = E ? E() : "";
            a(
              "Failed " + C + " type: " + R.message + (D ?? "")
            );
          }
        }
    }
  }
  return c.resetWarningCache = function() {
     true && (u = {});
  }, SE = c, SE;
}
var EE, iA;
function CB() {
  if (iA)
    return EE;
  iA = 1;
  var a = Cx(), r = yB(), u = Sx(), f = GO(), c = bB(), v = function() {
  };
   true && (v = function(C) {
    var b = "Warning: " + C;
    typeof console < "u" && console.error(b);
    try {
      throw new Error(b);
    } catch {
    }
  });
  function h() {
    return null;
  }
  return EE = function(C, b) {
    var E = typeof Symbol == "function" && Symbol.iterator, w = "@@iterator";
    function R(W) {
      var ce = W && (E && W[E] || W[w]);
      if (typeof ce == "function")
        return ce;
    }
    var A = "<<anonymous>>", D = {
      array: B("array"),
      bigint: B("bigint"),
      bool: B("boolean"),
      func: B("function"),
      number: B("number"),
      object: B("object"),
      string: B("string"),
      symbol: B("symbol"),
      any: Y(),
      arrayOf: z,
      element: Z(),
      elementType: V(),
      instanceOf: Q,
      node: Se(),
      objectOf: U,
      oneOf: P,
      oneOfType: se,
      shape: xe,
      exact: ye
    };
    function L(W, ce) {
      return W === ce ? W !== 0 || 1 / W === 1 / ce : W !== W && ce !== ce;
    }
    function M(W, ce) {
      this.message = W, this.data = ce && typeof ce == "object" ? ce : {}, this.stack = "";
    }
    M.prototype = Error.prototype;
    function k(W) {
      if (true)
        var ce = {}, ge = 0;
      function Ie(De, Oe, Ue, He, nt, vt, At) {
        if (He = He || A, vt = vt || Ue, At !== u) {
          if (b) {
            var Ce = new Error(
              "Calling PropTypes validators directly is not supported by the `prop-types` package. Use `PropTypes.checkPropTypes()` to call them. Read more at http://fb.me/use-check-prop-types"
            );
            throw Ce.name = "Invariant Violation", Ce;
          } else if ( true && typeof console < "u") {
            var Ot = He + ":" + Ue;
            !ce[Ot] && // Avoid spamming the console because they are often not actionable except for lib authors
            ge < 3 && (v(
              "You are manually calling a React.PropTypes validation function for the `" + vt + "` prop on `" + He + "`. This is deprecated and will throw in the standalone `prop-types` package. You may be seeing this warning due to a third-party PropTypes library. See https://fb.me/react-warning-dont-call-proptypes for details."
            ), ce[Ot] = !0, ge++);
          }
        }
        return Oe[Ue] == null ? De ? Oe[Ue] === null ? new M("The " + nt + " `" + vt + "` is marked as required " + ("in `" + He + "`, but its value is `null`.")) : new M("The " + nt + " `" + vt + "` is marked as required in " + ("`" + He + "`, but its value is `undefined`.")) : null : W(Oe, Ue, He, nt, vt);
      }
      var Ae = Ie.bind(null, !1);
      return Ae.isRequired = Ie.bind(null, !0), Ae;
    }
    function B(W) {
      function ce(ge, Ie, Ae, De, Oe, Ue) {
        var He = ge[Ie], nt = Ee(He);
        if (nt !== W) {
          var vt = tt(He);
          return new M(
            "Invalid " + De + " `" + Oe + "` of type " + ("`" + vt + "` supplied to `" + Ae + "`, expected ") + ("`" + W + "`."),
            { expectedType: W }
          );
        }
        return null;
      }
      return k(ce);
    }
    function Y() {
      return k(h);
    }
    function z(W) {
      function ce(ge, Ie, Ae, De, Oe) {
        if (typeof W != "function")
          return new M("Property `" + Oe + "` of component `" + Ae + "` has invalid PropType notation inside arrayOf.");
        var Ue = ge[Ie];
        if (!Array.isArray(Ue)) {
          var He = Ee(Ue);
          return new M("Invalid " + De + " `" + Oe + "` of type " + ("`" + He + "` supplied to `" + Ae + "`, expected an array."));
        }
        for (var nt = 0; nt < Ue.length; nt++) {
          var vt = W(Ue, nt, Ae, De, Oe + "[" + nt + "]", u);
          if (vt instanceof Error)
            return vt;
        }
        return null;
      }
      return k(ce);
    }
    function Z() {
      function W(ce, ge, Ie, Ae, De) {
        var Oe = ce[ge];
        if (!C(Oe)) {
          var Ue = Ee(Oe);
          return new M("Invalid " + Ae + " `" + De + "` of type " + ("`" + Ue + "` supplied to `" + Ie + "`, expected a single ReactElement."));
        }
        return null;
      }
      return k(W);
    }
    function V() {
      function W(ce, ge, Ie, Ae, De) {
        var Oe = ce[ge];
        if (!a.isValidElementType(Oe)) {
          var Ue = Ee(Oe);
          return new M("Invalid " + Ae + " `" + De + "` of type " + ("`" + Ue + "` supplied to `" + Ie + "`, expected a single ReactElement type."));
        }
        return null;
      }
      return k(W);
    }
    function Q(W) {
      function ce(ge, Ie, Ae, De, Oe) {
        if (!(ge[Ie] instanceof W)) {
          var Ue = W.name || A, He = we(ge[Ie]);
          return new M("Invalid " + De + " `" + Oe + "` of type " + ("`" + He + "` supplied to `" + Ae + "`, expected ") + ("instance of `" + Ue + "`."));
        }
        return null;
      }
      return k(ce);
    }
    function P(W) {
      if (!Array.isArray(W))
        return  true && (arguments.length > 1 ? v(
          "Invalid arguments supplied to oneOf, expected an array, got " + arguments.length + " arguments. A common mistake is to write oneOf(x, y, z) instead of oneOf([x, y, z])."
        ) : v("Invalid argument supplied to oneOf, expected an array.")), h;
      function ce(ge, Ie, Ae, De, Oe) {
        for (var Ue = ge[Ie], He = 0; He < W.length; He++)
          if (L(Ue, W[He]))
            return null;
        var nt = JSON.stringify(W, function(At, Ce) {
          var Ot = tt(Ce);
          return Ot === "symbol" ? String(Ce) : Ce;
        });
        return new M("Invalid " + De + " `" + Oe + "` of value `" + String(Ue) + "` " + ("supplied to `" + Ae + "`, expected one of " + nt + "."));
      }
      return k(ce);
    }
    function U(W) {
      function ce(ge, Ie, Ae, De, Oe) {
        if (typeof W != "function")
          return new M("Property `" + Oe + "` of component `" + Ae + "` has invalid PropType notation inside objectOf.");
        var Ue = ge[Ie], He = Ee(Ue);
        if (He !== "object")
          return new M("Invalid " + De + " `" + Oe + "` of type " + ("`" + He + "` supplied to `" + Ae + "`, expected an object."));
        for (var nt in Ue)
          if (f(Ue, nt)) {
            var vt = W(Ue, nt, Ae, De, Oe + "." + nt, u);
            if (vt instanceof Error)
              return vt;
          }
        return null;
      }
      return k(ce);
    }
    function se(W) {
      if (!Array.isArray(W))
        return  true && v("Invalid argument supplied to oneOfType, expected an instance of array."), h;
      for (var ce = 0; ce < W.length; ce++) {
        var ge = W[ce];
        if (typeof ge != "function")
          return v(
            "Invalid argument supplied to oneOfType. Expected an array of check functions, but received " + ue(ge) + " at index " + ce + "."
          ), h;
      }
      function Ie(Ae, De, Oe, Ue, He) {
        for (var nt = [], vt = 0; vt < W.length; vt++) {
          var At = W[vt], Ce = At(Ae, De, Oe, Ue, He, u);
          if (Ce == null)
            return null;
          Ce.data && f(Ce.data, "expectedType") && nt.push(Ce.data.expectedType);
        }
        var Ot = nt.length > 0 ? ", expected one of type [" + nt.join(", ") + "]" : "";
        return new M("Invalid " + Ue + " `" + He + "` supplied to " + ("`" + Oe + "`" + Ot + "."));
      }
      return k(Ie);
    }
    function Se() {
      function W(ce, ge, Ie, Ae, De) {
        return Me(ce[ge]) ? null : new M("Invalid " + Ae + " `" + De + "` supplied to " + ("`" + Ie + "`, expected a ReactNode."));
      }
      return k(W);
    }
    function he(W, ce, ge, Ie, Ae) {
      return new M(
        (W || "React class") + ": " + ce + " type `" + ge + "." + Ie + "` is invalid; it must be a function, usually from the `prop-types` package, but received `" + Ae + "`."
      );
    }
    function xe(W) {
      function ce(ge, Ie, Ae, De, Oe) {
        var Ue = ge[Ie], He = Ee(Ue);
        if (He !== "object")
          return new M("Invalid " + De + " `" + Oe + "` of type `" + He + "` " + ("supplied to `" + Ae + "`, expected `object`."));
        for (var nt in W) {
          var vt = W[nt];
          if (typeof vt != "function")
            return he(Ae, De, Oe, nt, tt(vt));
          var At = vt(Ue, nt, Ae, De, Oe + "." + nt, u);
          if (At)
            return At;
        }
        return null;
      }
      return k(ce);
    }
    function ye(W) {
      function ce(ge, Ie, Ae, De, Oe) {
        var Ue = ge[Ie], He = Ee(Ue);
        if (He !== "object")
          return new M("Invalid " + De + " `" + Oe + "` of type `" + He + "` " + ("supplied to `" + Ae + "`, expected `object`."));
        var nt = r({}, ge[Ie], W);
        for (var vt in nt) {
          var At = W[vt];
          if (f(W, vt) && typeof At != "function")
            return he(Ae, De, Oe, vt, tt(At));
          if (!At)
            return new M(
              "Invalid " + De + " `" + Oe + "` key `" + vt + "` supplied to `" + Ae + "`.\nBad object: " + JSON.stringify(ge[Ie], null, "  ") + `
Valid keys: ` + JSON.stringify(Object.keys(W), null, "  ")
            );
          var Ce = At(Ue, vt, Ae, De, Oe + "." + vt, u);
          if (Ce)
            return Ce;
        }
        return null;
      }
      return k(ce);
    }
    function Me(W) {
      switch (typeof W) {
        case "number":
        case "string":
        case "undefined":
          return !0;
        case "boolean":
          return !W;
        case "object":
          if (Array.isArray(W))
            return W.every(Me);
          if (W === null || C(W))
            return !0;
          var ce = R(W);
          if (ce) {
            var ge = ce.call(W), Ie;
            if (ce !== W.entries) {
              for (; !(Ie = ge.next()).done; )
                if (!Me(Ie.value))
                  return !1;
            } else
              for (; !(Ie = ge.next()).done; ) {
                var Ae = Ie.value;
                if (Ae && !Me(Ae[1]))
                  return !1;
              }
          } else
            return !1;
          return !0;
        default:
          return !1;
      }
    }
    function We(W, ce) {
      return W === "symbol" ? !0 : ce ? ce["@@toStringTag"] === "Symbol" || typeof Symbol == "function" && ce instanceof Symbol : !1;
    }
    function Ee(W) {
      var ce = typeof W;
      return Array.isArray(W) ? "array" : W instanceof RegExp ? "object" : We(ce, W) ? "symbol" : ce;
    }
    function tt(W) {
      if (typeof W > "u" || W === null)
        return "" + W;
      var ce = Ee(W);
      if (ce === "object") {
        if (W instanceof Date)
          return "date";
        if (W instanceof RegExp)
          return "regexp";
      }
      return ce;
    }
    function ue(W) {
      var ce = tt(W);
      switch (ce) {
        case "array":
        case "object":
          return "an " + ce;
        case "boolean":
        case "date":
        case "regexp":
          return "a " + ce;
        default:
          return ce;
      }
    }
    function we(W) {
      return !W.constructor || !W.constructor.name ? A : W.constructor.name;
    }
    return D.checkPropTypes = c, D.resetWarningCache = c.resetWarningCache, D.PropTypes = D, D;
  }, EE;
}
var xE, oA;
function SB() {
  if (oA)
    return xE;
  oA = 1;
  var a = Sx();
  function r() {
  }
  function u() {
  }
  return u.resetWarningCache = r, xE = function() {
    function f(h, C, b, E, w, R) {
      if (R !== a) {
        var A = new Error(
          "Calling PropTypes validators directly is not supported by the `prop-types` package. Use PropTypes.checkPropTypes() to call them. Read more at http://fb.me/use-check-prop-types"
        );
        throw A.name = "Invariant Violation", A;
      }
    }
    f.isRequired = f;
    function c() {
      return f;
    }
    var v = {
      array: f,
      bigint: f,
      bool: f,
      func: f,
      number: f,
      object: f,
      string: f,
      symbol: f,
      any: f,
      arrayOf: c,
      element: f,
      elementType: f,
      instanceOf: c,
      node: f,
      objectOf: c,
      oneOf: c,
      oneOfType: c,
      shape: c,
      exact: c,
      checkPropTypes: u,
      resetWarningCache: r
    };
    return v.PropTypes = v, v;
  }, xE;
}
if (true) {
  var EB = Cx(), xB = !0;
  XE.exports = CB()(EB.isElement, xB);
} else
  {}
var wB = XE.exports;
const gt = /* @__PURE__ */ ym(wB), TB = {
  /**
   * Specify whether the feedback is for valid or invalid fields
   *
   * @type {('valid'|'invalid')}
   */
  type: gt.string,
  /** Display feedback as a tooltip. */
  tooltip: gt.bool,
  as: gt.elementType
}, xb = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  ({
    as: a = "div",
    className: r,
    type: u = "valid",
    tooltip: f = !1,
    ...c
  }, v) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(a, {
    ...c,
    ref: v,
    className: Ke(r, `${u}-${f ? "tooltip" : "feedback"}`)
  })
);
xb.displayName = "Feedback";
xb.propTypes = TB;
const _u = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext({}), _O = ["xxl", "xl", "lg", "md", "sm", "xs"], jO = "xs", wb = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext({
  prefixes: {},
  breakpoints: _O,
  minBreakpoint: jO
});
function ft(a, r) {
  const {
    prefixes: u
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(wb);
  return a || u[r] || r;
}
function Ex() {
  const {
    breakpoints: a
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(wb);
  return a;
}
function xx() {
  const {
    minBreakpoint: a
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(wb);
  return a;
}
function RB() {
  const {
    dir: a
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(wb);
  return a === "rtl";
}
const wx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  id: a,
  bsPrefix: r,
  className: u,
  type: f = "checkbox",
  isValid: c = !1,
  isInvalid: v = !1,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: h = "input",
  ...C
}, b) => {
  const {
    controlId: E
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  return r = ft(r, "form-check-input"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(h, {
    ...C,
    ref: b,
    type: f,
    id: a || E,
    className: Ke(u, r, c && "is-valid", v && "is-invalid")
  });
});
wx.displayName = "FormCheckInput";
const eb = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  htmlFor: u,
  ...f
}, c) => {
  const {
    controlId: v
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  return a = ft(a, "form-check-label"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("label", {
    ...f,
    ref: c,
    htmlFor: u || v,
    className: Ke(r, a)
  });
});
eb.displayName = "FormCheckLabel";
function IB(a, r) {
  return react__WEBPACK_IMPORTED_MODULE_1__.Children.toArray(a).some((u) => /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.isValidElement(u) && u.type === r);
}
const UO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  id: a,
  bsPrefix: r,
  bsSwitchPrefix: u,
  inline: f = !1,
  reverse: c = !1,
  disabled: v = !1,
  isValid: h = !1,
  isInvalid: C = !1,
  feedbackTooltip: b = !1,
  feedback: E,
  feedbackType: w,
  className: R,
  style: A,
  title: D = "",
  type: L = "checkbox",
  label: M,
  children: k,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: B = "input",
  ...Y
}, z) => {
  r = ft(r, "form-check"), u = ft(u, "form-switch");
  const {
    controlId: Z
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    controlId: a || Z
  }), [Z, a]), Q = !k && M != null && M !== !1 || IB(k, eb), P = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(wx, {
    ...Y,
    type: L === "switch" ? "checkbox" : L,
    ref: z,
    isValid: h,
    isInvalid: C,
    disabled: v,
    as: B
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_u.Provider, {
    value: V,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
      style: A,
      className: Ke(R, Q && r, f && `${r}-inline`, c && `${r}-reverse`, L === "switch" && u),
      children: k || /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
        children: [P, Q && /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(eb, {
          title: D,
          children: M
        }), E && /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(xb, {
          type: w,
          tooltip: b,
          children: E
        })]
      })
    })
  });
});
UO.displayName = "FormCheck";
const tb = Object.assign(UO, {
  Input: wx,
  Label: eb
});
var AB = "development" !== "production", WO = function() {
};
if (AB) {
  var OB = function(r, u) {
    var f = arguments.length;
    u = new Array(f > 1 ? f - 1 : 0);
    for (var c = 1; c < f; c++)
      u[c - 1] = arguments[c];
    var v = 0, h = "Warning: " + r.replace(/%s/g, function() {
      return u[v++];
    });
    typeof console < "u" && console.error(h);
    try {
      throw new Error(h);
    } catch {
    }
  };
  WO = function(a, r, u) {
    var f = arguments.length;
    u = new Array(f > 2 ? f - 2 : 0);
    for (var c = 2; c < f; c++)
      u[c - 2] = arguments[c];
    if (r === void 0)
      throw new Error(
        "`warning(condition, format, ...args)` requires a warning message argument"
      );
    a || OB.apply(null, [r].concat(u));
  };
}
var DB = WO;
const Tb = /* @__PURE__ */ ym(DB), XO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  type: r,
  size: u,
  htmlSize: f,
  id: c,
  className: v,
  isValid: h = !1,
  isInvalid: C = !1,
  plaintext: b,
  readOnly: E,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: w = "input",
  ...R
}, A) => {
  const {
    controlId: D
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  return a = ft(a, "form-control"),  true && Tb(D == null || !c, "`controlId` is ignored on `<FormControl>` when `id` is specified."), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(w, {
    ...R,
    type: r,
    size: f,
    ref: A,
    readOnly: E,
    id: c || D,
    className: Ke(v, b ? `${a}-plaintext` : a, u && `${a}-${u}`, r === "color" && `${a}-color`, h && "is-valid", C && "is-invalid")
  });
});
XO.displayName = "FormControl";
const NB = Object.assign(XO, {
  Feedback: xb
}), ZO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  ...f
}, c) => (r = ft(r, "form-floating"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
ZO.displayName = "FormFloating";
const Tx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  controlId: a,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: r = "div",
  ...u
}, f) => {
  const c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    controlId: a
  }), [a]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_u.Provider, {
    value: c,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(r, {
      ...u,
      ref: f
    })
  });
});
Tx.displayName = "FormGroup";
function kB({
  as: a,
  bsPrefix: r,
  className: u,
  ...f
}) {
  r = ft(r, "col");
  const c = Ex(), v = xx(), h = [], C = [];
  return c.forEach((b) => {
    const E = f[b];
    delete f[b];
    let w, R, A;
    typeof E == "object" && E != null ? {
      span: w,
      offset: R,
      order: A
    } = E : w = E;
    const D = b !== v ? `-${b}` : "";
    w && h.push(w === !0 ? `${r}${D}` : `${r}${D}-${w}`), A != null && C.push(`order${D}-${A}`), R != null && C.push(`offset${D}-${R}`);
  }), [{
    ...f,
    className: Ke(u, ...h, ...C)
  }, {
    as: a,
    bsPrefix: r,
    spans: h
  }];
}
const Vc = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  (a, r) => {
    const [{
      className: u,
      ...f
    }, {
      as: c = "div",
      bsPrefix: v,
      spans: h
    }] = kB(a);
    return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(c, {
      ...f,
      ref: r,
      className: Ke(u, !h.length && v)
    });
  }
);
Vc.displayName = "Col";
const YO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: a = "label",
  bsPrefix: r,
  column: u = !1,
  visuallyHidden: f = !1,
  className: c,
  htmlFor: v,
  ...h
}, C) => {
  const {
    controlId: b
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  r = ft(r, "form-label");
  let E = "col-form-label";
  typeof u == "string" && (E = `${E} ${E}-${u}`);
  const w = Ke(c, r, f && "visually-hidden", u && E);
  return  true && Tb(b == null || !v, "`controlId` is ignored on `<FormLabel>` when `htmlFor` is specified."), v = v || b, u ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, {
    ref: C,
    as: "label",
    className: w,
    htmlFor: v,
    ...h
  }) : (
    // eslint-disable-next-line jsx-a11y/label-has-for, jsx-a11y/label-has-associated-control
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(a, {
      ref: C,
      className: w,
      htmlFor: v,
      ...h
    })
  );
});
YO.displayName = "FormLabel";
const $O = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  id: u,
  ...f
}, c) => {
  const {
    controlId: v
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  return a = ft(a, "form-range"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("input", {
    ...f,
    type: "range",
    ref: c,
    className: Ke(r, a),
    id: u || v
  });
});
$O.displayName = "FormRange";
const QO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  size: r,
  htmlSize: u,
  className: f,
  isValid: c = !1,
  isInvalid: v = !1,
  id: h,
  ...C
}, b) => {
  const {
    controlId: E
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(_u);
  return a = ft(a, "form-select"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("select", {
    ...C,
    size: u,
    ref: b,
    className: Ke(f, a, r && `${a}-${r}`, c && "is-valid", v && "is-invalid"),
    id: h || E
  });
});
QO.displayName = "FormSelect";
const JO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  ({
    bsPrefix: a,
    className: r,
    as: u = "small",
    muted: f,
    ...c
  }, v) => (a = ft(a, "form-text"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
    ...c,
    ref: v,
    className: Ke(r, a, f && "text-muted")
  }))
);
JO.displayName = "FormText";
const KO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(tb, {
  ...a,
  ref: r,
  type: "switch"
}));
KO.displayName = "Switch";
const MB = Object.assign(KO, {
  Input: tb.Input,
  Label: tb.Label
}), qO = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  children: u,
  controlId: f,
  label: c,
  ...v
}, h) => (a = ft(a, "form-floating"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Tx, {
  ref: h,
  className: Ke(r, a),
  controlId: f,
  ...v,
  children: [u, /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("label", {
    htmlFor: f,
    children: c
  })]
})));
qO.displayName = "FloatingLabel";
const LB = {
  /**
   * The Form `ref` will be forwarded to the underlying element,
   * which means, unless it's rendered `as` a composite component,
   * it will be a DOM node, when resolved.
   *
   * @type {ReactRef}
   * @alias ref
   */
  _ref: gt.any,
  /**
   * Mark a form as having been validated. Setting it to `true` will
   * toggle any validation styles on the forms elements.
   */
  validated: gt.bool,
  as: gt.elementType
}, Rx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  validated: r,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: u = "form",
  ...f
}, c) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ...f,
  ref: c,
  className: Ke(a, r && "was-validated")
}));
Rx.displayName = "Form";
Rx.propTypes = LB;
const Ix = Object.assign(Rx, {
  Group: Tx,
  Control: NB,
  Floating: ZO,
  Check: tb,
  Switch: MB,
  Label: YO,
  Text: JO,
  Range: $O,
  Select: QO,
  FloatingLabel: qO
});
function st() {
  return st = Object.assign ? Object.assign.bind() : function(a) {
    for (var r = 1; r < arguments.length; r++) {
      var u = arguments[r];
      for (var f in u)
        Object.prototype.hasOwnProperty.call(u, f) && (a[f] = u[f]);
    }
    return a;
  }, st.apply(this, arguments);
}
function Ax(a, r) {
  if (a == null)
    return {};
  var u = {};
  for (var f in a)
    if (Object.prototype.hasOwnProperty.call(a, f)) {
      if (r.indexOf(f) >= 0)
        continue;
      u[f] = a[f];
    }
  return u;
}
var FB = function(a, r, u, f, c, v, h, C) {
  if ( true && r === void 0)
    throw new Error("invariant requires an error message argument");
  if (!a) {
    var b;
    if (r === void 0)
      b = new Error(
        "Minified exception occurred; use the non-minified dev environment for the full error message and additional helpful warnings."
      );
    else {
      var E = [u, f, c, v, h, C], w = 0;
      b = new Error(
        r.replace(/%s/g, function() {
          return E[w++];
        })
      ), b.name = "Invariant Violation";
    }
    throw b.framesToPop = 1, b;
  }
}, BB = FB;
const uA = /* @__PURE__ */ ym(BB);
function lA(a) {
  return "default" + a.charAt(0).toUpperCase() + a.substr(1);
}
function VB(a) {
  var r = PB(a, "string");
  return typeof r == "symbol" ? r : String(r);
}
function PB(a, r) {
  if (typeof a != "object" || a === null)
    return a;
  var u = a[Symbol.toPrimitive];
  if (u !== void 0) {
    var f = u.call(a, r);
    if (typeof f != "object")
      return f;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return String(a);
}
function HB(a, r, u) {
  var f = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(a !== void 0), c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(r), v = c[0], h = c[1], C = a !== void 0, b = f.current;
  return f.current = C, !C && b && v !== r && h(r), [C ? a : v, (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(E) {
    for (var w = arguments.length, R = new Array(w > 1 ? w - 1 : 0), A = 1; A < w; A++)
      R[A - 1] = arguments[A];
    u && u.apply(void 0, [E].concat(R)), h(E);
  }, [u])];
}
function Ox(a, r) {
  return Object.keys(r).reduce(function(u, f) {
    var c, v = u, h = v[lA(f)], C = v[f], b = Ax(v, [lA(f), f].map(VB)), E = r[f], w = HB(C, h, a[E]), R = w[0], A = w[1];
    return st({}, b, (c = {}, c[f] = R, c[E] = A, c));
  }, a);
}
function nb(a, r) {
  return nb = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(f, c) {
    return f.__proto__ = c, f;
  }, nb(a, r);
}
function zB(a, r) {
  a.prototype = Object.create(r.prototype), a.prototype.constructor = a, nb(a, r);
}
function GB(a) {
  const r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(a);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    r.current = a;
  }, [a]), r;
}
function Un(a) {
  const r = GB(a);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(...u) {
    return r.current && r.current(...u);
  }, [r]);
}
const Rb = (a) => /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((r, u) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
  ...r,
  ref: u,
  className: Ke(r.className, a)
})), eD = Rb("h4");
eD.displayName = "DivStyledAsH4";
const tD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = eD,
  ...f
}, c) => (r = ft(r, "alert-heading"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
tD.displayName = "AlertHeading";
function _B() {
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(null);
}
function jB(a, r, u, f = !1) {
  const c = Un(u);
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    const v = typeof a == "function" ? a() : a;
    return v.addEventListener(r, c, f), () => v.removeEventListener(r, c, f);
  }, [a]);
}
function nD() {
  const a = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!0), r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(() => a.current);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => (a.current = !0, () => {
    a.current = !1;
  }), []), r.current;
}
function rD(a) {
  const r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    r.current = a;
  }), r.current;
}
const UB = typeof __webpack_require__.g < "u" && // @ts-ignore
__webpack_require__.g.navigator && // @ts-ignore
__webpack_require__.g.navigator.product === "ReactNative", WB = typeof document < "u", rb = WB || UB ? react__WEBPACK_IMPORTED_MODULE_1__.useLayoutEffect : react__WEBPACK_IMPORTED_MODULE_1__.useEffect, XB = ["as", "disabled"];
function ZB(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
function YB(a) {
  return !a || a.trim() === "#";
}
function Dx({
  tagName: a,
  disabled: r,
  href: u,
  target: f,
  rel: c,
  role: v,
  onClick: h,
  tabIndex: C = 0,
  type: b
}) {
  a || (u != null || f != null || c != null ? a = "a" : a = "button");
  const E = {
    tagName: a
  };
  if (a === "button")
    return [{
      type: b || "button",
      disabled: r
    }, E];
  const w = (A) => {
    if ((r || a === "a" && YB(u)) && A.preventDefault(), r) {
      A.stopPropagation();
      return;
    }
    h == null || h(A);
  }, R = (A) => {
    A.key === " " && (A.preventDefault(), w(A));
  };
  return a === "a" && (u || (u = "#"), r && (u = void 0)), [{
    role: v ?? "button",
    // explicitly undefined so that it overrides the props disabled in a spread
    // e.g. <Tag {...props} {...hookProps} />
    disabled: void 0,
    tabIndex: r ? void 0 : C,
    href: u,
    target: a === "a" ? f : void 0,
    "aria-disabled": r || void 0,
    rel: a === "a" ? c : void 0,
    onClick: w,
    onKeyDown: R
  }, E];
}
const Nx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  let {
    as: u,
    disabled: f
  } = a, c = ZB(a, XB);
  const [v, {
    tagName: h
  }] = Dx(Object.assign({
    tagName: u,
    disabled: f
  }, c));
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(h, Object.assign({}, c, v, {
    ref: r
  }));
});
Nx.displayName = "Button";
const $B = ["onKeyDown"];
function QB(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
function JB(a) {
  return !a || a.trim() === "#";
}
const bm = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  let {
    onKeyDown: u
  } = a, f = QB(a, $B);
  const [c] = Dx(Object.assign({
    tagName: "a"
  }, f)), v = Un((h) => {
    c.onKeyDown(h), u == null || u(h);
  });
  return JB(f.href) || f.role === "button" ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("a", Object.assign({
    ref: r
  }, f, c, {
    onKeyDown: v
  })) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("a", Object.assign({
    ref: r
  }, f, {
    onKeyDown: u
  }));
});
bm.displayName = "Anchor";
const aD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = bm,
  ...f
}, c) => (r = ft(r, "alert-link"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
aD.displayName = "AlertLink";
var ZE = { exports: {} }, Ga = {}, P0 = { exports: {} }, wE = {};
/**
 * @license React
 * scheduler.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var sA;
function KB() {
  return sA || (sA = 1, function(a) {
    function r(ue, we) {
      var W = ue.length;
      ue.push(we);
      e:
        for (; 0 < W; ) {
          var ce = W - 1 >>> 1, ge = ue[ce];
          if (0 < c(ge, we))
            ue[ce] = we, ue[W] = ge, W = ce;
          else
            break e;
        }
    }
    function u(ue) {
      return ue.length === 0 ? null : ue[0];
    }
    function f(ue) {
      if (ue.length === 0)
        return null;
      var we = ue[0], W = ue.pop();
      if (W !== we) {
        ue[0] = W;
        e:
          for (var ce = 0, ge = ue.length, Ie = ge >>> 1; ce < Ie; ) {
            var Ae = 2 * (ce + 1) - 1, De = ue[Ae], Oe = Ae + 1, Ue = ue[Oe];
            if (0 > c(De, W))
              Oe < ge && 0 > c(Ue, De) ? (ue[ce] = Ue, ue[Oe] = W, ce = Oe) : (ue[ce] = De, ue[Ae] = W, ce = Ae);
            else if (Oe < ge && 0 > c(Ue, W))
              ue[ce] = Ue, ue[Oe] = W, ce = Oe;
            else
              break e;
          }
      }
      return we;
    }
    function c(ue, we) {
      var W = ue.sortIndex - we.sortIndex;
      return W !== 0 ? W : ue.id - we.id;
    }
    if (typeof performance == "object" && typeof performance.now == "function") {
      var v = performance;
      a.unstable_now = function() {
        return v.now();
      };
    } else {
      var h = Date, C = h.now();
      a.unstable_now = function() {
        return h.now() - C;
      };
    }
    var b = [], E = [], w = 1, R = null, A = 3, D = !1, L = !1, M = !1, k = typeof setTimeout == "function" ? setTimeout : null, B = typeof clearTimeout == "function" ? clearTimeout : null, Y = typeof setImmediate < "u" ? setImmediate : null;
    typeof navigator < "u" && navigator.scheduling !== void 0 && navigator.scheduling.isInputPending !== void 0 && navigator.scheduling.isInputPending.bind(navigator.scheduling);
    function z(ue) {
      for (var we = u(E); we !== null; ) {
        if (we.callback === null)
          f(E);
        else if (we.startTime <= ue)
          f(E), we.sortIndex = we.expirationTime, r(b, we);
        else
          break;
        we = u(E);
      }
    }
    function Z(ue) {
      if (M = !1, z(ue), !L)
        if (u(b) !== null)
          L = !0, Ee(V);
        else {
          var we = u(E);
          we !== null && tt(Z, we.startTime - ue);
        }
    }
    function V(ue, we) {
      L = !1, M && (M = !1, B(U), U = -1), D = !0;
      var W = A;
      try {
        for (z(we), R = u(b); R !== null && (!(R.expirationTime > we) || ue && !he()); ) {
          var ce = R.callback;
          if (typeof ce == "function") {
            R.callback = null, A = R.priorityLevel;
            var ge = ce(R.expirationTime <= we);
            we = a.unstable_now(), typeof ge == "function" ? R.callback = ge : R === u(b) && f(b), z(we);
          } else
            f(b);
          R = u(b);
        }
        if (R !== null)
          var Ie = !0;
        else {
          var Ae = u(E);
          Ae !== null && tt(Z, Ae.startTime - we), Ie = !1;
        }
        return Ie;
      } finally {
        R = null, A = W, D = !1;
      }
    }
    var Q = !1, P = null, U = -1, se = 5, Se = -1;
    function he() {
      return !(a.unstable_now() - Se < se);
    }
    function xe() {
      if (P !== null) {
        var ue = a.unstable_now();
        Se = ue;
        var we = !0;
        try {
          we = P(!0, ue);
        } finally {
          we ? ye() : (Q = !1, P = null);
        }
      } else
        Q = !1;
    }
    var ye;
    if (typeof Y == "function")
      ye = function() {
        Y(xe);
      };
    else if (typeof MessageChannel < "u") {
      var Me = new MessageChannel(), We = Me.port2;
      Me.port1.onmessage = xe, ye = function() {
        We.postMessage(null);
      };
    } else
      ye = function() {
        k(xe, 0);
      };
    function Ee(ue) {
      P = ue, Q || (Q = !0, ye());
    }
    function tt(ue, we) {
      U = k(function() {
        ue(a.unstable_now());
      }, we);
    }
    a.unstable_IdlePriority = 5, a.unstable_ImmediatePriority = 1, a.unstable_LowPriority = 4, a.unstable_NormalPriority = 3, a.unstable_Profiling = null, a.unstable_UserBlockingPriority = 2, a.unstable_cancelCallback = function(ue) {
      ue.callback = null;
    }, a.unstable_continueExecution = function() {
      L || D || (L = !0, Ee(V));
    }, a.unstable_forceFrameRate = function(ue) {
      0 > ue || 125 < ue ? console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported") : se = 0 < ue ? Math.floor(1e3 / ue) : 5;
    }, a.unstable_getCurrentPriorityLevel = function() {
      return A;
    }, a.unstable_getFirstCallbackNode = function() {
      return u(b);
    }, a.unstable_next = function(ue) {
      switch (A) {
        case 1:
        case 2:
        case 3:
          var we = 3;
          break;
        default:
          we = A;
      }
      var W = A;
      A = we;
      try {
        return ue();
      } finally {
        A = W;
      }
    }, a.unstable_pauseExecution = function() {
    }, a.unstable_requestPaint = function() {
    }, a.unstable_runWithPriority = function(ue, we) {
      switch (ue) {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
          break;
        default:
          ue = 3;
      }
      var W = A;
      A = ue;
      try {
        return we();
      } finally {
        A = W;
      }
    }, a.unstable_scheduleCallback = function(ue, we, W) {
      var ce = a.unstable_now();
      switch (typeof W == "object" && W !== null ? (W = W.delay, W = typeof W == "number" && 0 < W ? ce + W : ce) : W = ce, ue) {
        case 1:
          var ge = -1;
          break;
        case 2:
          ge = 250;
          break;
        case 5:
          ge = 1073741823;
          break;
        case 4:
          ge = 1e4;
          break;
        default:
          ge = 5e3;
      }
      return ge = W + ge, ue = { id: w++, callback: we, priorityLevel: ue, startTime: W, expirationTime: ge, sortIndex: -1 }, W > ce ? (ue.sortIndex = W, r(E, ue), u(b) === null && ue === u(E) && (M ? (B(U), U = -1) : M = !0, tt(Z, W - ce))) : (ue.sortIndex = ge, r(b, ue), L || D || (L = !0, Ee(V))), ue;
    }, a.unstable_shouldYield = he, a.unstable_wrapCallback = function(ue) {
      var we = A;
      return function() {
        var W = A;
        A = we;
        try {
          return ue.apply(this, arguments);
        } finally {
          A = W;
        }
      };
    };
  }(wE)), wE;
}
var TE = {};
/**
 * @license React
 * scheduler.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var cA;
function qB() {
  return cA || (cA = 1, function(a) {
     true && function() {
      typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u" && typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart == "function" && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(new Error());
      var r = !1, u = !1, f = 5;
      function c(ke, rt) {
        var wt = ke.length;
        ke.push(rt), C(ke, rt, wt);
      }
      function v(ke) {
        return ke.length === 0 ? null : ke[0];
      }
      function h(ke) {
        if (ke.length === 0)
          return null;
        var rt = ke[0], wt = ke.pop();
        return wt !== rt && (ke[0] = wt, b(ke, wt, 0)), rt;
      }
      function C(ke, rt, wt) {
        for (var Kt = wt; Kt > 0; ) {
          var En = Kt - 1 >>> 1, pr = ke[En];
          if (E(pr, rt) > 0)
            ke[En] = rt, ke[Kt] = pr, Kt = En;
          else
            return;
        }
      }
      function b(ke, rt, wt) {
        for (var Kt = wt, En = ke.length, pr = En >>> 1; Kt < pr; ) {
          var Bn = (Kt + 1) * 2 - 1, ya = ke[Bn], vn = Bn + 1, fi = ke[vn];
          if (E(ya, rt) < 0)
            vn < En && E(fi, ya) < 0 ? (ke[Kt] = fi, ke[vn] = rt, Kt = vn) : (ke[Kt] = ya, ke[Bn] = rt, Kt = Bn);
          else if (vn < En && E(fi, rt) < 0)
            ke[Kt] = fi, ke[vn] = rt, Kt = vn;
          else
            return;
        }
      }
      function E(ke, rt) {
        var wt = ke.sortIndex - rt.sortIndex;
        return wt !== 0 ? wt : ke.id - rt.id;
      }
      var w = 1, R = 2, A = 3, D = 4, L = 5;
      function M(ke, rt) {
      }
      var k = typeof performance == "object" && typeof performance.now == "function";
      if (k) {
        var B = performance;
        a.unstable_now = function() {
          return B.now();
        };
      } else {
        var Y = Date, z = Y.now();
        a.unstable_now = function() {
          return Y.now() - z;
        };
      }
      var Z = 1073741823, V = -1, Q = 250, P = 5e3, U = 1e4, se = Z, Se = [], he = [], xe = 1, ye = null, Me = A, We = !1, Ee = !1, tt = !1, ue = typeof setTimeout == "function" ? setTimeout : null, we = typeof clearTimeout == "function" ? clearTimeout : null, W = typeof setImmediate < "u" ? setImmediate : null;
      typeof navigator < "u" && navigator.scheduling !== void 0 && navigator.scheduling.isInputPending !== void 0 && navigator.scheduling.isInputPending.bind(navigator.scheduling);
      function ce(ke) {
        for (var rt = v(he); rt !== null; ) {
          if (rt.callback === null)
            h(he);
          else if (rt.startTime <= ke)
            h(he), rt.sortIndex = rt.expirationTime, c(Se, rt);
          else
            return;
          rt = v(he);
        }
      }
      function ge(ke) {
        if (tt = !1, ce(ke), !Ee)
          if (v(Se) !== null)
            Ee = !0, $a(Ie);
          else {
            var rt = v(he);
            rt !== null && tr(ge, rt.startTime - ke);
          }
      }
      function Ie(ke, rt) {
        Ee = !1, tt && (tt = !1, Br()), We = !0;
        var wt = Me;
        try {
          var Kt;
          if (!u)
            return Ae(ke, rt);
        } finally {
          ye = null, Me = wt, We = !1;
        }
      }
      function Ae(ke, rt) {
        var wt = rt;
        for (ce(wt), ye = v(Se); ye !== null && !r && !(ye.expirationTime > wt && (!ke || so())); ) {
          var Kt = ye.callback;
          if (typeof Kt == "function") {
            ye.callback = null, Me = ye.priorityLevel;
            var En = ye.expirationTime <= wt, pr = Kt(En);
            wt = a.unstable_now(), typeof pr == "function" ? ye.callback = pr : ye === v(Se) && h(Se), ce(wt);
          } else
            h(Se);
          ye = v(Se);
        }
        if (ye !== null)
          return !0;
        var Bn = v(he);
        return Bn !== null && tr(ge, Bn.startTime - wt), !1;
      }
      function De(ke, rt) {
        switch (ke) {
          case w:
          case R:
          case A:
          case D:
          case L:
            break;
          default:
            ke = A;
        }
        var wt = Me;
        Me = ke;
        try {
          return rt();
        } finally {
          Me = wt;
        }
      }
      function Oe(ke) {
        var rt;
        switch (Me) {
          case w:
          case R:
          case A:
            rt = A;
            break;
          default:
            rt = Me;
            break;
        }
        var wt = Me;
        Me = rt;
        try {
          return ke();
        } finally {
          Me = wt;
        }
      }
      function Ue(ke) {
        var rt = Me;
        return function() {
          var wt = Me;
          Me = rt;
          try {
            return ke.apply(this, arguments);
          } finally {
            Me = wt;
          }
        };
      }
      function He(ke, rt, wt) {
        var Kt = a.unstable_now(), En;
        if (typeof wt == "object" && wt !== null) {
          var pr = wt.delay;
          typeof pr == "number" && pr > 0 ? En = Kt + pr : En = Kt;
        } else
          En = Kt;
        var Bn;
        switch (ke) {
          case w:
            Bn = V;
            break;
          case R:
            Bn = Q;
            break;
          case L:
            Bn = se;
            break;
          case D:
            Bn = U;
            break;
          case A:
          default:
            Bn = P;
            break;
        }
        var ya = En + Bn, vn = {
          id: xe++,
          callback: rt,
          priorityLevel: ke,
          startTime: En,
          expirationTime: ya,
          sortIndex: -1
        };
        return En > Kt ? (vn.sortIndex = En, c(he, vn), v(Se) === null && vn === v(he) && (tt ? Br() : tt = !0, tr(ge, En - Kt))) : (vn.sortIndex = ya, c(Se, vn), !Ee && !We && (Ee = !0, $a(Ie))), vn;
      }
      function nt() {
      }
      function vt() {
        !Ee && !We && (Ee = !0, $a(Ie));
      }
      function At() {
        return v(Se);
      }
      function Ce(ke) {
        ke.callback = null;
      }
      function Ot() {
        return Me;
      }
      var _t = !1, Et = null, er = -1, In = f, Ya = -1;
      function so() {
        var ke = a.unstable_now() - Ya;
        return !(ke < In);
      }
      function Sr() {
      }
      function ta(ke) {
        if (ke < 0 || ke > 125) {
          console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported");
          return;
        }
        ke > 0 ? In = Math.floor(1e3 / ke) : In = f;
      }
      var ci = function() {
        if (Et !== null) {
          var ke = a.unstable_now();
          Ya = ke;
          var rt = !0, wt = !0;
          try {
            wt = Et(rt, ke);
          } finally {
            wt ? Fr() : (_t = !1, Et = null);
          }
        } else
          _t = !1;
      }, Fr;
      if (typeof W == "function")
        Fr = function() {
          W(ci);
        };
      else if (typeof MessageChannel < "u") {
        var ga = new MessageChannel(), na = ga.port2;
        ga.port1.onmessage = ci, Fr = function() {
          na.postMessage(null);
        };
      } else
        Fr = function() {
          ue(ci, 0);
        };
      function $a(ke) {
        Et = ke, _t || (_t = !0, Fr());
      }
      function tr(ke, rt) {
        er = ue(function() {
          ke(a.unstable_now());
        }, rt);
      }
      function Br() {
        we(er), er = -1;
      }
      var ps = Sr, co = null;
      a.unstable_IdlePriority = L, a.unstable_ImmediatePriority = w, a.unstable_LowPriority = D, a.unstable_NormalPriority = A, a.unstable_Profiling = co, a.unstable_UserBlockingPriority = R, a.unstable_cancelCallback = Ce, a.unstable_continueExecution = vt, a.unstable_forceFrameRate = ta, a.unstable_getCurrentPriorityLevel = Ot, a.unstable_getFirstCallbackNode = At, a.unstable_next = Oe, a.unstable_pauseExecution = nt, a.unstable_requestPaint = ps, a.unstable_runWithPriority = De, a.unstable_scheduleCallback = He, a.unstable_shouldYield = so, a.unstable_wrapCallback = Ue, typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u" && typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop == "function" && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(new Error());
    }();
  }(TE)), TE;
}
var fA;
function iD() {
  return fA || (fA = 1,  false ? 0 : P0.exports = qB()), P0.exports;
}
/**
 * @license React
 * react-dom.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var dA;
function eV() {
  if (dA)
    return Ga;
  dA = 1;
  var a = (react__WEBPACK_IMPORTED_MODULE_1___default()), r = iD();
  function u(n) {
    for (var i = "https://reactjs.org/docs/error-decoder.html?invariant=" + n, s = 1; s < arguments.length; s++)
      i += "&args[]=" + encodeURIComponent(arguments[s]);
    return "Minified React error #" + n + "; visit " + i + " for the full message or use the non-minified dev environment for full errors and additional helpful warnings.";
  }
  var f = /* @__PURE__ */ new Set(), c = {};
  function v(n, i) {
    h(n, i), h(n + "Capture", i);
  }
  function h(n, i) {
    for (c[n] = i, n = 0; n < i.length; n++)
      f.add(i[n]);
  }
  var C = !(typeof window > "u" || typeof window.document > "u" || typeof window.document.createElement > "u"), b = Object.prototype.hasOwnProperty, E = /^[:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD][:A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF\u200C-\u200D\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD\-.0-9\u00B7\u0300-\u036F\u203F-\u2040]*$/, w = {}, R = {};
  function A(n) {
    return b.call(R, n) ? !0 : b.call(w, n) ? !1 : E.test(n) ? R[n] = !0 : (w[n] = !0, !1);
  }
  function D(n, i, s, p) {
    if (s !== null && s.type === 0)
      return !1;
    switch (typeof i) {
      case "function":
      case "symbol":
        return !0;
      case "boolean":
        return p ? !1 : s !== null ? !s.acceptsBooleans : (n = n.toLowerCase().slice(0, 5), n !== "data-" && n !== "aria-");
      default:
        return !1;
    }
  }
  function L(n, i, s, p) {
    if (i === null || typeof i > "u" || D(n, i, s, p))
      return !0;
    if (p)
      return !1;
    if (s !== null)
      switch (s.type) {
        case 3:
          return !i;
        case 4:
          return i === !1;
        case 5:
          return isNaN(i);
        case 6:
          return isNaN(i) || 1 > i;
      }
    return !1;
  }
  function M(n, i, s, p, g, S, I) {
    this.acceptsBooleans = i === 2 || i === 3 || i === 4, this.attributeName = p, this.attributeNamespace = g, this.mustUseProperty = s, this.propertyName = n, this.type = i, this.sanitizeURL = S, this.removeEmptyString = I;
  }
  var k = {};
  "children dangerouslySetInnerHTML defaultValue defaultChecked innerHTML suppressContentEditableWarning suppressHydrationWarning style".split(" ").forEach(function(n) {
    k[n] = new M(n, 0, !1, n, null, !1, !1);
  }), [["acceptCharset", "accept-charset"], ["className", "class"], ["htmlFor", "for"], ["httpEquiv", "http-equiv"]].forEach(function(n) {
    var i = n[0];
    k[i] = new M(i, 1, !1, n[1], null, !1, !1);
  }), ["contentEditable", "draggable", "spellCheck", "value"].forEach(function(n) {
    k[n] = new M(n, 2, !1, n.toLowerCase(), null, !1, !1);
  }), ["autoReverse", "externalResourcesRequired", "focusable", "preserveAlpha"].forEach(function(n) {
    k[n] = new M(n, 2, !1, n, null, !1, !1);
  }), "allowFullScreen async autoFocus autoPlay controls default defer disabled disablePictureInPicture disableRemotePlayback formNoValidate hidden loop noModule noValidate open playsInline readOnly required reversed scoped seamless itemScope".split(" ").forEach(function(n) {
    k[n] = new M(n, 3, !1, n.toLowerCase(), null, !1, !1);
  }), ["checked", "multiple", "muted", "selected"].forEach(function(n) {
    k[n] = new M(n, 3, !0, n, null, !1, !1);
  }), ["capture", "download"].forEach(function(n) {
    k[n] = new M(n, 4, !1, n, null, !1, !1);
  }), ["cols", "rows", "size", "span"].forEach(function(n) {
    k[n] = new M(n, 6, !1, n, null, !1, !1);
  }), ["rowSpan", "start"].forEach(function(n) {
    k[n] = new M(n, 5, !1, n.toLowerCase(), null, !1, !1);
  });
  var B = /[\-:]([a-z])/g;
  function Y(n) {
    return n[1].toUpperCase();
  }
  "accent-height alignment-baseline arabic-form baseline-shift cap-height clip-path clip-rule color-interpolation color-interpolation-filters color-profile color-rendering dominant-baseline enable-background fill-opacity fill-rule flood-color flood-opacity font-family font-size font-size-adjust font-stretch font-style font-variant font-weight glyph-name glyph-orientation-horizontal glyph-orientation-vertical horiz-adv-x horiz-origin-x image-rendering letter-spacing lighting-color marker-end marker-mid marker-start overline-position overline-thickness paint-order panose-1 pointer-events rendering-intent shape-rendering stop-color stop-opacity strikethrough-position strikethrough-thickness stroke-dasharray stroke-dashoffset stroke-linecap stroke-linejoin stroke-miterlimit stroke-opacity stroke-width text-anchor text-decoration text-rendering underline-position underline-thickness unicode-bidi unicode-range units-per-em v-alphabetic v-hanging v-ideographic v-mathematical vector-effect vert-adv-y vert-origin-x vert-origin-y word-spacing writing-mode xmlns:xlink x-height".split(" ").forEach(function(n) {
    var i = n.replace(
      B,
      Y
    );
    k[i] = new M(i, 1, !1, n, null, !1, !1);
  }), "xlink:actuate xlink:arcrole xlink:role xlink:show xlink:title xlink:type".split(" ").forEach(function(n) {
    var i = n.replace(B, Y);
    k[i] = new M(i, 1, !1, n, "http://www.w3.org/1999/xlink", !1, !1);
  }), ["xml:base", "xml:lang", "xml:space"].forEach(function(n) {
    var i = n.replace(B, Y);
    k[i] = new M(i, 1, !1, n, "http://www.w3.org/XML/1998/namespace", !1, !1);
  }), ["tabIndex", "crossOrigin"].forEach(function(n) {
    k[n] = new M(n, 1, !1, n.toLowerCase(), null, !1, !1);
  }), k.xlinkHref = new M("xlinkHref", 1, !1, "xlink:href", "http://www.w3.org/1999/xlink", !0, !1), ["src", "href", "action", "formAction"].forEach(function(n) {
    k[n] = new M(n, 1, !1, n.toLowerCase(), null, !0, !0);
  });
  function z(n, i, s, p) {
    var g = k.hasOwnProperty(i) ? k[i] : null;
    (g !== null ? g.type !== 0 : p || !(2 < i.length) || i[0] !== "o" && i[0] !== "O" || i[1] !== "n" && i[1] !== "N") && (L(i, s, g, p) && (s = null), p || g === null ? A(i) && (s === null ? n.removeAttribute(i) : n.setAttribute(i, "" + s)) : g.mustUseProperty ? n[g.propertyName] = s === null ? g.type === 3 ? !1 : "" : s : (i = g.attributeName, p = g.attributeNamespace, s === null ? n.removeAttribute(i) : (g = g.type, s = g === 3 || g === 4 && s === !0 ? "" : "" + s, p ? n.setAttributeNS(p, i, s) : n.setAttribute(i, s))));
  }
  var Z = a.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED, V = Symbol.for("react.element"), Q = Symbol.for("react.portal"), P = Symbol.for("react.fragment"), U = Symbol.for("react.strict_mode"), se = Symbol.for("react.profiler"), Se = Symbol.for("react.provider"), he = Symbol.for("react.context"), xe = Symbol.for("react.forward_ref"), ye = Symbol.for("react.suspense"), Me = Symbol.for("react.suspense_list"), We = Symbol.for("react.memo"), Ee = Symbol.for("react.lazy"), tt = Symbol.for("react.offscreen"), ue = Symbol.iterator;
  function we(n) {
    return n === null || typeof n != "object" ? null : (n = ue && n[ue] || n["@@iterator"], typeof n == "function" ? n : null);
  }
  var W = Object.assign, ce;
  function ge(n) {
    if (ce === void 0)
      try {
        throw Error();
      } catch (s) {
        var i = s.stack.trim().match(/\n( *(at )?)/);
        ce = i && i[1] || "";
      }
    return `
` + ce + n;
  }
  var Ie = !1;
  function Ae(n, i) {
    if (!n || Ie)
      return "";
    Ie = !0;
    var s = Error.prepareStackTrace;
    Error.prepareStackTrace = void 0;
    try {
      if (i)
        if (i = function() {
          throw Error();
        }, Object.defineProperty(i.prototype, "props", { set: function() {
          throw Error();
        } }), typeof Reflect == "object" && Reflect.construct) {
          try {
            Reflect.construct(i, []);
          } catch (ne) {
            var p = ne;
          }
          Reflect.construct(n, [], i);
        } else {
          try {
            i.call();
          } catch (ne) {
            p = ne;
          }
          n.call(i.prototype);
        }
      else {
        try {
          throw Error();
        } catch (ne) {
          p = ne;
        }
        n();
      }
    } catch (ne) {
      if (ne && p && typeof ne.stack == "string") {
        for (var g = ne.stack.split(`
`), S = p.stack.split(`
`), I = g.length - 1, F = S.length - 1; 1 <= I && 0 <= F && g[I] !== S[F]; )
          F--;
        for (; 1 <= I && 0 <= F; I--, F--)
          if (g[I] !== S[F]) {
            if (I !== 1 || F !== 1)
              do
                if (I--, F--, 0 > F || g[I] !== S[F]) {
                  var H = `
` + g[I].replace(" at new ", " at ");
                  return n.displayName && H.includes("<anonymous>") && (H = H.replace("<anonymous>", n.displayName)), H;
                }
              while (1 <= I && 0 <= F);
            break;
          }
      }
    } finally {
      Ie = !1, Error.prepareStackTrace = s;
    }
    return (n = n ? n.displayName || n.name : "") ? ge(n) : "";
  }
  function De(n) {
    switch (n.tag) {
      case 5:
        return ge(n.type);
      case 16:
        return ge("Lazy");
      case 13:
        return ge("Suspense");
      case 19:
        return ge("SuspenseList");
      case 0:
      case 2:
      case 15:
        return n = Ae(n.type, !1), n;
      case 11:
        return n = Ae(n.type.render, !1), n;
      case 1:
        return n = Ae(n.type, !0), n;
      default:
        return "";
    }
  }
  function Oe(n) {
    if (n == null)
      return null;
    if (typeof n == "function")
      return n.displayName || n.name || null;
    if (typeof n == "string")
      return n;
    switch (n) {
      case P:
        return "Fragment";
      case Q:
        return "Portal";
      case se:
        return "Profiler";
      case U:
        return "StrictMode";
      case ye:
        return "Suspense";
      case Me:
        return "SuspenseList";
    }
    if (typeof n == "object")
      switch (n.$$typeof) {
        case he:
          return (n.displayName || "Context") + ".Consumer";
        case Se:
          return (n._context.displayName || "Context") + ".Provider";
        case xe:
          var i = n.render;
          return n = n.displayName, n || (n = i.displayName || i.name || "", n = n !== "" ? "ForwardRef(" + n + ")" : "ForwardRef"), n;
        case We:
          return i = n.displayName || null, i !== null ? i : Oe(n.type) || "Memo";
        case Ee:
          i = n._payload, n = n._init;
          try {
            return Oe(n(i));
          } catch {
          }
      }
    return null;
  }
  function Ue(n) {
    var i = n.type;
    switch (n.tag) {
      case 24:
        return "Cache";
      case 9:
        return (i.displayName || "Context") + ".Consumer";
      case 10:
        return (i._context.displayName || "Context") + ".Provider";
      case 18:
        return "DehydratedFragment";
      case 11:
        return n = i.render, n = n.displayName || n.name || "", i.displayName || (n !== "" ? "ForwardRef(" + n + ")" : "ForwardRef");
      case 7:
        return "Fragment";
      case 5:
        return i;
      case 4:
        return "Portal";
      case 3:
        return "Root";
      case 6:
        return "Text";
      case 16:
        return Oe(i);
      case 8:
        return i === U ? "StrictMode" : "Mode";
      case 22:
        return "Offscreen";
      case 12:
        return "Profiler";
      case 21:
        return "Scope";
      case 13:
        return "Suspense";
      case 19:
        return "SuspenseList";
      case 25:
        return "TracingMarker";
      case 1:
      case 0:
      case 17:
      case 2:
      case 14:
      case 15:
        if (typeof i == "function")
          return i.displayName || i.name || null;
        if (typeof i == "string")
          return i;
    }
    return null;
  }
  function He(n) {
    switch (typeof n) {
      case "boolean":
      case "number":
      case "string":
      case "undefined":
        return n;
      case "object":
        return n;
      default:
        return "";
    }
  }
  function nt(n) {
    var i = n.type;
    return (n = n.nodeName) && n.toLowerCase() === "input" && (i === "checkbox" || i === "radio");
  }
  function vt(n) {
    var i = nt(n) ? "checked" : "value", s = Object.getOwnPropertyDescriptor(n.constructor.prototype, i), p = "" + n[i];
    if (!n.hasOwnProperty(i) && typeof s < "u" && typeof s.get == "function" && typeof s.set == "function") {
      var g = s.get, S = s.set;
      return Object.defineProperty(n, i, { configurable: !0, get: function() {
        return g.call(this);
      }, set: function(I) {
        p = "" + I, S.call(this, I);
      } }), Object.defineProperty(n, i, { enumerable: s.enumerable }), { getValue: function() {
        return p;
      }, setValue: function(I) {
        p = "" + I;
      }, stopTracking: function() {
        n._valueTracker = null, delete n[i];
      } };
    }
  }
  function At(n) {
    n._valueTracker || (n._valueTracker = vt(n));
  }
  function Ce(n) {
    if (!n)
      return !1;
    var i = n._valueTracker;
    if (!i)
      return !0;
    var s = i.getValue(), p = "";
    return n && (p = nt(n) ? n.checked ? "true" : "false" : n.value), n = p, n !== s ? (i.setValue(n), !0) : !1;
  }
  function Ot(n) {
    if (n = n || (typeof document < "u" ? document : void 0), typeof n > "u")
      return null;
    try {
      return n.activeElement || n.body;
    } catch {
      return n.body;
    }
  }
  function _t(n, i) {
    var s = i.checked;
    return W({}, i, { defaultChecked: void 0, defaultValue: void 0, value: void 0, checked: s ?? n._wrapperState.initialChecked });
  }
  function Et(n, i) {
    var s = i.defaultValue == null ? "" : i.defaultValue, p = i.checked != null ? i.checked : i.defaultChecked;
    s = He(i.value != null ? i.value : s), n._wrapperState = { initialChecked: p, initialValue: s, controlled: i.type === "checkbox" || i.type === "radio" ? i.checked != null : i.value != null };
  }
  function er(n, i) {
    i = i.checked, i != null && z(n, "checked", i, !1);
  }
  function In(n, i) {
    er(n, i);
    var s = He(i.value), p = i.type;
    if (s != null)
      p === "number" ? (s === 0 && n.value === "" || n.value != s) && (n.value = "" + s) : n.value !== "" + s && (n.value = "" + s);
    else if (p === "submit" || p === "reset") {
      n.removeAttribute("value");
      return;
    }
    i.hasOwnProperty("value") ? so(n, i.type, s) : i.hasOwnProperty("defaultValue") && so(n, i.type, He(i.defaultValue)), i.checked == null && i.defaultChecked != null && (n.defaultChecked = !!i.defaultChecked);
  }
  function Ya(n, i, s) {
    if (i.hasOwnProperty("value") || i.hasOwnProperty("defaultValue")) {
      var p = i.type;
      if (!(p !== "submit" && p !== "reset" || i.value !== void 0 && i.value !== null))
        return;
      i = "" + n._wrapperState.initialValue, s || i === n.value || (n.value = i), n.defaultValue = i;
    }
    s = n.name, s !== "" && (n.name = ""), n.defaultChecked = !!n._wrapperState.initialChecked, s !== "" && (n.name = s);
  }
  function so(n, i, s) {
    (i !== "number" || Ot(n.ownerDocument) !== n) && (s == null ? n.defaultValue = "" + n._wrapperState.initialValue : n.defaultValue !== "" + s && (n.defaultValue = "" + s));
  }
  var Sr = Array.isArray;
  function ta(n, i, s, p) {
    if (n = n.options, i) {
      i = {};
      for (var g = 0; g < s.length; g++)
        i["$" + s[g]] = !0;
      for (s = 0; s < n.length; s++)
        g = i.hasOwnProperty("$" + n[s].value), n[s].selected !== g && (n[s].selected = g), g && p && (n[s].defaultSelected = !0);
    } else {
      for (s = "" + He(s), i = null, g = 0; g < n.length; g++) {
        if (n[g].value === s) {
          n[g].selected = !0, p && (n[g].defaultSelected = !0);
          return;
        }
        i !== null || n[g].disabled || (i = n[g]);
      }
      i !== null && (i.selected = !0);
    }
  }
  function ci(n, i) {
    if (i.dangerouslySetInnerHTML != null)
      throw Error(u(91));
    return W({}, i, { value: void 0, defaultValue: void 0, children: "" + n._wrapperState.initialValue });
  }
  function Fr(n, i) {
    var s = i.value;
    if (s == null) {
      if (s = i.children, i = i.defaultValue, s != null) {
        if (i != null)
          throw Error(u(92));
        if (Sr(s)) {
          if (1 < s.length)
            throw Error(u(93));
          s = s[0];
        }
        i = s;
      }
      i == null && (i = ""), s = i;
    }
    n._wrapperState = { initialValue: He(s) };
  }
  function ga(n, i) {
    var s = He(i.value), p = He(i.defaultValue);
    s != null && (s = "" + s, s !== n.value && (n.value = s), i.defaultValue == null && n.defaultValue !== s && (n.defaultValue = s)), p != null && (n.defaultValue = "" + p);
  }
  function na(n) {
    var i = n.textContent;
    i === n._wrapperState.initialValue && i !== "" && i !== null && (n.value = i);
  }
  function $a(n) {
    switch (n) {
      case "svg":
        return "http://www.w3.org/2000/svg";
      case "math":
        return "http://www.w3.org/1998/Math/MathML";
      default:
        return "http://www.w3.org/1999/xhtml";
    }
  }
  function tr(n, i) {
    return n == null || n === "http://www.w3.org/1999/xhtml" ? $a(i) : n === "http://www.w3.org/2000/svg" && i === "foreignObject" ? "http://www.w3.org/1999/xhtml" : n;
  }
  var Br, ps = function(n) {
    return typeof MSApp < "u" && MSApp.execUnsafeLocalFunction ? function(i, s, p, g) {
      MSApp.execUnsafeLocalFunction(function() {
        return n(i, s, p, g);
      });
    } : n;
  }(function(n, i) {
    if (n.namespaceURI !== "http://www.w3.org/2000/svg" || "innerHTML" in n)
      n.innerHTML = i;
    else {
      for (Br = Br || document.createElement("div"), Br.innerHTML = "<svg>" + i.valueOf().toString() + "</svg>", i = Br.firstChild; n.firstChild; )
        n.removeChild(n.firstChild);
      for (; i.firstChild; )
        n.appendChild(i.firstChild);
    }
  });
  function co(n, i) {
    if (i) {
      var s = n.firstChild;
      if (s && s === n.lastChild && s.nodeType === 3) {
        s.nodeValue = i;
        return;
      }
    }
    n.textContent = i;
  }
  var ke = {
    animationIterationCount: !0,
    aspectRatio: !0,
    borderImageOutset: !0,
    borderImageSlice: !0,
    borderImageWidth: !0,
    boxFlex: !0,
    boxFlexGroup: !0,
    boxOrdinalGroup: !0,
    columnCount: !0,
    columns: !0,
    flex: !0,
    flexGrow: !0,
    flexPositive: !0,
    flexShrink: !0,
    flexNegative: !0,
    flexOrder: !0,
    gridArea: !0,
    gridRow: !0,
    gridRowEnd: !0,
    gridRowSpan: !0,
    gridRowStart: !0,
    gridColumn: !0,
    gridColumnEnd: !0,
    gridColumnSpan: !0,
    gridColumnStart: !0,
    fontWeight: !0,
    lineClamp: !0,
    lineHeight: !0,
    opacity: !0,
    order: !0,
    orphans: !0,
    tabSize: !0,
    widows: !0,
    zIndex: !0,
    zoom: !0,
    fillOpacity: !0,
    floodOpacity: !0,
    stopOpacity: !0,
    strokeDasharray: !0,
    strokeDashoffset: !0,
    strokeMiterlimit: !0,
    strokeOpacity: !0,
    strokeWidth: !0
  }, rt = ["Webkit", "ms", "Moz", "O"];
  Object.keys(ke).forEach(function(n) {
    rt.forEach(function(i) {
      i = i + n.charAt(0).toUpperCase() + n.substring(1), ke[i] = ke[n];
    });
  });
  function wt(n, i, s) {
    return i == null || typeof i == "boolean" || i === "" ? "" : s || typeof i != "number" || i === 0 || ke.hasOwnProperty(n) && ke[n] ? ("" + i).trim() : i + "px";
  }
  function Kt(n, i) {
    n = n.style;
    for (var s in i)
      if (i.hasOwnProperty(s)) {
        var p = s.indexOf("--") === 0, g = wt(s, i[s], p);
        s === "float" && (s = "cssFloat"), p ? n.setProperty(s, g) : n[s] = g;
      }
  }
  var En = W({ menuitem: !0 }, { area: !0, base: !0, br: !0, col: !0, embed: !0, hr: !0, img: !0, input: !0, keygen: !0, link: !0, meta: !0, param: !0, source: !0, track: !0, wbr: !0 });
  function pr(n, i) {
    if (i) {
      if (En[n] && (i.children != null || i.dangerouslySetInnerHTML != null))
        throw Error(u(137, n));
      if (i.dangerouslySetInnerHTML != null) {
        if (i.children != null)
          throw Error(u(60));
        if (typeof i.dangerouslySetInnerHTML != "object" || !("__html" in i.dangerouslySetInnerHTML))
          throw Error(u(61));
      }
      if (i.style != null && typeof i.style != "object")
        throw Error(u(62));
    }
  }
  function Bn(n, i) {
    if (n.indexOf("-") === -1)
      return typeof i.is == "string";
    switch (n) {
      case "annotation-xml":
      case "color-profile":
      case "font-face":
      case "font-face-src":
      case "font-face-uri":
      case "font-face-format":
      case "font-face-name":
      case "missing-glyph":
        return !1;
      default:
        return !0;
    }
  }
  var ya = null;
  function vn(n) {
    return n = n.target || n.srcElement || window, n.correspondingUseElement && (n = n.correspondingUseElement), n.nodeType === 3 ? n.parentNode : n;
  }
  var fi = null, An = null, hn = null;
  function Im(n) {
    if (n = Ps(n)) {
      if (typeof fi != "function")
        throw Error(u(280));
      var i = n.stateNode;
      i && (i = qe(i), fi(n.stateNode, n.type, i));
    }
  }
  function Wc(n) {
    An ? hn ? hn.push(n) : hn = [n] : An = n;
  }
  function Xc() {
    if (An) {
      var n = An, i = hn;
      if (hn = An = null, Im(n), i)
        for (n = 0; n < i.length; n++)
          Im(i[n]);
    }
  }
  function Am(n, i) {
    return n(i);
  }
  function Om() {
  }
  var Zc = !1;
  function xp(n, i, s) {
    if (Zc)
      return n(i, s);
    Zc = !0;
    try {
      return Am(n, i, s);
    } finally {
      Zc = !1, (An !== null || hn !== null) && (Om(), Xc());
    }
  }
  function vs(n, i) {
    var s = n.stateNode;
    if (s === null)
      return null;
    var p = qe(s);
    if (p === null)
      return null;
    s = p[i];
    e:
      switch (i) {
        case "onClick":
        case "onClickCapture":
        case "onDoubleClick":
        case "onDoubleClickCapture":
        case "onMouseDown":
        case "onMouseDownCapture":
        case "onMouseMove":
        case "onMouseMoveCapture":
        case "onMouseUp":
        case "onMouseUpCapture":
        case "onMouseEnter":
          (p = !p.disabled) || (n = n.type, p = !(n === "button" || n === "input" || n === "select" || n === "textarea")), n = !p;
          break e;
        default:
          n = !1;
      }
    if (n)
      return null;
    if (s && typeof s != "function")
      throw Error(u(231, i, typeof s));
    return s;
  }
  var Yc = !1;
  if (C)
    try {
      var Qo = {};
      Object.defineProperty(Qo, "passive", { get: function() {
        Yc = !0;
      } }), window.addEventListener("test", Qo, Qo), window.removeEventListener("test", Qo, Qo);
    } catch {
      Yc = !1;
    }
  function Xu(n, i, s, p, g, S, I, F, H) {
    var ne = Array.prototype.slice.call(arguments, 3);
    try {
      i.apply(s, ne);
    } catch (pe) {
      this.onError(pe);
    }
  }
  var Qa = !1, ki = null, Zu = !1, hs = null, wp = { onError: function(n) {
    Qa = !0, ki = n;
  } };
  function Tp(n, i, s, p, g, S, I, F, H) {
    Qa = !1, ki = null, Xu.apply(wp, arguments);
  }
  function Yu(n, i, s, p, g, S, I, F, H) {
    if (Tp.apply(this, arguments), Qa) {
      if (Qa) {
        var ne = ki;
        Qa = !1, ki = null;
      } else
        throw Error(u(198));
      Zu || (Zu = !0, hs = ne);
    }
  }
  function di(n) {
    var i = n, s = n;
    if (n.alternate)
      for (; i.return; )
        i = i.return;
    else {
      n = i;
      do
        i = n, i.flags & 4098 && (s = i.return), n = i.return;
      while (n);
    }
    return i.tag === 3 ? s : null;
  }
  function ms(n) {
    if (n.tag === 13) {
      var i = n.memoizedState;
      if (i === null && (n = n.alternate, n !== null && (i = n.memoizedState)), i !== null)
        return i.dehydrated;
    }
    return null;
  }
  function $u(n) {
    if (di(n) !== n)
      throw Error(u(188));
  }
  function Er(n) {
    var i = n.alternate;
    if (!i) {
      if (i = di(n), i === null)
        throw Error(u(188));
      return i !== n ? null : n;
    }
    for (var s = n, p = i; ; ) {
      var g = s.return;
      if (g === null)
        break;
      var S = g.alternate;
      if (S === null) {
        if (p = g.return, p !== null) {
          s = p;
          continue;
        }
        break;
      }
      if (g.child === S.child) {
        for (S = g.child; S; ) {
          if (S === s)
            return $u(g), n;
          if (S === p)
            return $u(g), i;
          S = S.sibling;
        }
        throw Error(u(188));
      }
      if (s.return !== p.return)
        s = g, p = S;
      else {
        for (var I = !1, F = g.child; F; ) {
          if (F === s) {
            I = !0, s = g, p = S;
            break;
          }
          if (F === p) {
            I = !0, p = g, s = S;
            break;
          }
          F = F.sibling;
        }
        if (!I) {
          for (F = S.child; F; ) {
            if (F === s) {
              I = !0, s = S, p = g;
              break;
            }
            if (F === p) {
              I = !0, p = S, s = g;
              break;
            }
            F = F.sibling;
          }
          if (!I)
            throw Error(u(189));
        }
      }
      if (s.alternate !== p)
        throw Error(u(190));
    }
    if (s.tag !== 3)
      throw Error(u(188));
    return s.stateNode.current === s ? n : i;
  }
  function Dm(n) {
    return n = Er(n), n !== null ? Nm(n) : null;
  }
  function Nm(n) {
    if (n.tag === 5 || n.tag === 6)
      return n;
    for (n = n.child; n !== null; ) {
      var i = Nm(n);
      if (i !== null)
        return i;
      n = n.sibling;
    }
    return null;
  }
  var Rp = r.unstable_scheduleCallback, km = r.unstable_cancelCallback, zb = r.unstable_shouldYield, Gb = r.unstable_requestPaint, mn = r.unstable_now, _b = r.unstable_getCurrentPriorityLevel, Mi = r.unstable_ImmediatePriority, Tt = r.unstable_UserBlockingPriority, fo = r.unstable_NormalPriority, Mm = r.unstable_LowPriority, Ip = r.unstable_IdlePriority, gs = null, Ja = null;
  function Lm(n) {
    if (Ja && typeof Ja.onCommitFiberRoot == "function")
      try {
        Ja.onCommitFiberRoot(gs, n, void 0, (n.current.flags & 128) === 128);
      } catch {
      }
  }
  var ba = Math.clz32 ? Math.clz32 : jb, Fm = Math.log, Bm = Math.LN2;
  function jb(n) {
    return n >>>= 0, n === 0 ? 32 : 31 - (Fm(n) / Bm | 0) | 0;
  }
  var $c = 64, Qu = 4194304;
  function Jo(n) {
    switch (n & -n) {
      case 1:
        return 1;
      case 2:
        return 2;
      case 4:
        return 4;
      case 8:
        return 8;
      case 16:
        return 16;
      case 32:
        return 32;
      case 64:
      case 128:
      case 256:
      case 512:
      case 1024:
      case 2048:
      case 4096:
      case 8192:
      case 16384:
      case 32768:
      case 65536:
      case 131072:
      case 262144:
      case 524288:
      case 1048576:
      case 2097152:
        return n & 4194240;
      case 4194304:
      case 8388608:
      case 16777216:
      case 33554432:
      case 67108864:
        return n & 130023424;
      case 134217728:
        return 134217728;
      case 268435456:
        return 268435456;
      case 536870912:
        return 536870912;
      case 1073741824:
        return 1073741824;
      default:
        return n;
    }
  }
  function Ka(n, i) {
    var s = n.pendingLanes;
    if (s === 0)
      return 0;
    var p = 0, g = n.suspendedLanes, S = n.pingedLanes, I = s & 268435455;
    if (I !== 0) {
      var F = I & ~g;
      F !== 0 ? p = Jo(F) : (S &= I, S !== 0 && (p = Jo(S)));
    } else
      I = s & ~g, I !== 0 ? p = Jo(I) : S !== 0 && (p = Jo(S));
    if (p === 0)
      return 0;
    if (i !== 0 && i !== p && !(i & g) && (g = p & -p, S = i & -i, g >= S || g === 16 && (S & 4194240) !== 0))
      return i;
    if (p & 4 && (p |= s & 16), i = n.entangledLanes, i !== 0)
      for (n = n.entanglements, i &= p; 0 < i; )
        s = 31 - ba(i), g = 1 << s, p |= n[s], i &= ~g;
    return p;
  }
  function Ap(n, i) {
    switch (n) {
      case 1:
      case 2:
      case 4:
        return i + 250;
      case 8:
      case 16:
      case 32:
      case 64:
      case 128:
      case 256:
      case 512:
      case 1024:
      case 2048:
      case 4096:
      case 8192:
      case 16384:
      case 32768:
      case 65536:
      case 131072:
      case 262144:
      case 524288:
      case 1048576:
      case 2097152:
        return i + 5e3;
      case 4194304:
      case 8388608:
      case 16777216:
      case 33554432:
      case 67108864:
        return -1;
      case 134217728:
      case 268435456:
      case 536870912:
      case 1073741824:
        return -1;
      default:
        return -1;
    }
  }
  function Qc(n, i) {
    for (var s = n.suspendedLanes, p = n.pingedLanes, g = n.expirationTimes, S = n.pendingLanes; 0 < S; ) {
      var I = 31 - ba(S), F = 1 << I, H = g[I];
      H === -1 ? (!(F & s) || F & p) && (g[I] = Ap(F, i)) : H <= i && (n.expiredLanes |= F), S &= ~F;
    }
  }
  function Op(n) {
    return n = n.pendingLanes & -1073741825, n !== 0 ? n : n & 1073741824 ? 1073741824 : 0;
  }
  function Jc() {
    var n = $c;
    return $c <<= 1, !($c & 4194240) && ($c = 64), n;
  }
  function Dp(n) {
    for (var i = [], s = 0; 31 > s; s++)
      i.push(n);
    return i;
  }
  function Ko(n, i, s) {
    n.pendingLanes |= i, i !== 536870912 && (n.suspendedLanes = 0, n.pingedLanes = 0), n = n.eventTimes, i = 31 - ba(i), n[i] = s;
  }
  function Ub(n, i) {
    var s = n.pendingLanes & ~i;
    n.pendingLanes = i, n.suspendedLanes = 0, n.pingedLanes = 0, n.expiredLanes &= i, n.mutableReadLanes &= i, n.entangledLanes &= i, i = n.entanglements;
    var p = n.eventTimes;
    for (n = n.expirationTimes; 0 < s; ) {
      var g = 31 - ba(s), S = 1 << g;
      i[g] = 0, p[g] = -1, n[g] = -1, s &= ~S;
    }
  }
  function ys(n, i) {
    var s = n.entangledLanes |= i;
    for (n = n.entanglements; s; ) {
      var p = 31 - ba(s), g = 1 << p;
      g & i | n[p] & i && (n[p] |= i), s &= ~g;
    }
  }
  var jt = 0;
  function Np(n) {
    return n &= -n, 1 < n ? 4 < n ? n & 268435455 ? 16 : 536870912 : 4 : 1;
  }
  var Vm, Kc, Ut, Pm, kp, bt = !1, bs = [], On = null, Ca = null, Sa = null, Cs = /* @__PURE__ */ new Map(), Vn = /* @__PURE__ */ new Map(), Zt = [], Wb = "mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset submit".split(" ");
  function qa(n, i) {
    switch (n) {
      case "focusin":
      case "focusout":
        On = null;
        break;
      case "dragenter":
      case "dragleave":
        Ca = null;
        break;
      case "mouseover":
      case "mouseout":
        Sa = null;
        break;
      case "pointerover":
      case "pointerout":
        Cs.delete(i.pointerId);
        break;
      case "gotpointercapture":
      case "lostpointercapture":
        Vn.delete(i.pointerId);
    }
  }
  function vr(n, i, s, p, g, S) {
    return n === null || n.nativeEvent !== S ? (n = { blockedOn: i, domEventName: s, eventSystemFlags: p, nativeEvent: S, targetContainers: [g] }, i !== null && (i = Ps(i), i !== null && Kc(i)), n) : (n.eventSystemFlags |= p, i = n.targetContainers, g !== null && i.indexOf(g) === -1 && i.push(g), n);
  }
  function po(n, i, s, p, g) {
    switch (i) {
      case "focusin":
        return On = vr(On, n, i, s, p, g), !0;
      case "dragenter":
        return Ca = vr(Ca, n, i, s, p, g), !0;
      case "mouseover":
        return Sa = vr(Sa, n, i, s, p, g), !0;
      case "pointerover":
        var S = g.pointerId;
        return Cs.set(S, vr(Cs.get(S) || null, n, i, s, p, g)), !0;
      case "gotpointercapture":
        return S = g.pointerId, Vn.set(S, vr(Vn.get(S) || null, n, i, s, p, g)), !0;
    }
    return !1;
  }
  function Hm(n) {
    var i = xa(n.target);
    if (i !== null) {
      var s = di(i);
      if (s !== null) {
        if (i = s.tag, i === 13) {
          if (i = ms(s), i !== null) {
            n.blockedOn = i, kp(n.priority, function() {
              Ut(s);
            });
            return;
          }
        } else if (i === 3 && s.stateNode.current.memoizedState.isDehydrated) {
          n.blockedOn = s.tag === 3 ? s.stateNode.containerInfo : null;
          return;
        }
      }
    }
    n.blockedOn = null;
  }
  function Ju(n) {
    if (n.blockedOn !== null)
      return !1;
    for (var i = n.targetContainers; 0 < i.length; ) {
      var s = tf(n.domEventName, n.eventSystemFlags, i[0], n.nativeEvent);
      if (s === null) {
        s = n.nativeEvent;
        var p = new s.constructor(s.type, s);
        ya = p, s.target.dispatchEvent(p), ya = null;
      } else
        return i = Ps(s), i !== null && Kc(i), n.blockedOn = s, !1;
      i.shift();
    }
    return !0;
  }
  function Mp(n, i, s) {
    Ju(n) && s.delete(i);
  }
  function zm() {
    bt = !1, On !== null && Ju(On) && (On = null), Ca !== null && Ju(Ca) && (Ca = null), Sa !== null && Ju(Sa) && (Sa = null), Cs.forEach(Mp), Vn.forEach(Mp);
  }
  function Ss(n, i) {
    n.blockedOn === i && (n.blockedOn = null, bt || (bt = !0, r.unstable_scheduleCallback(r.unstable_NormalPriority, zm)));
  }
  function Es(n) {
    function i(g) {
      return Ss(g, n);
    }
    if (0 < bs.length) {
      Ss(bs[0], n);
      for (var s = 1; s < bs.length; s++) {
        var p = bs[s];
        p.blockedOn === n && (p.blockedOn = null);
      }
    }
    for (On !== null && Ss(On, n), Ca !== null && Ss(Ca, n), Sa !== null && Ss(Sa, n), Cs.forEach(i), Vn.forEach(i), s = 0; s < Zt.length; s++)
      p = Zt[s], p.blockedOn === n && (p.blockedOn = null);
    for (; 0 < Zt.length && (s = Zt[0], s.blockedOn === null); )
      Hm(s), s.blockedOn === null && Zt.shift();
  }
  var Ku = Z.ReactCurrentBatchConfig, qo = !0;
  function Gm(n, i, s, p) {
    var g = jt, S = Ku.transition;
    Ku.transition = null;
    try {
      jt = 1, ef(n, i, s, p);
    } finally {
      jt = g, Ku.transition = S;
    }
  }
  function qc(n, i, s, p) {
    var g = jt, S = Ku.transition;
    Ku.transition = null;
    try {
      jt = 4, ef(n, i, s, p);
    } finally {
      jt = g, Ku.transition = S;
    }
  }
  function ef(n, i, s, p) {
    if (qo) {
      var g = tf(n, i, s, p);
      if (g === null)
        vf(n, i, p, xs, s), qa(n, p);
      else if (po(g, n, i, s, p))
        p.stopPropagation();
      else if (qa(n, p), i & 4 && -1 < Wb.indexOf(n)) {
        for (; g !== null; ) {
          var S = Ps(g);
          if (S !== null && Vm(S), S = tf(n, i, s, p), S === null && vf(n, i, p, xs, s), S === g)
            break;
          g = S;
        }
        g !== null && p.stopPropagation();
      } else
        vf(n, i, p, null, s);
    }
  }
  var xs = null;
  function tf(n, i, s, p) {
    if (xs = null, n = vn(p), n = xa(n), n !== null)
      if (i = di(n), i === null)
        n = null;
      else if (s = i.tag, s === 13) {
        if (n = ms(i), n !== null)
          return n;
        n = null;
      } else if (s === 3) {
        if (i.stateNode.current.memoizedState.isDehydrated)
          return i.tag === 3 ? i.stateNode.containerInfo : null;
        n = null;
      } else
        i !== n && (n = null);
    return xs = n, null;
  }
  function Lp(n) {
    switch (n) {
      case "cancel":
      case "click":
      case "close":
      case "contextmenu":
      case "copy":
      case "cut":
      case "auxclick":
      case "dblclick":
      case "dragend":
      case "dragstart":
      case "drop":
      case "focusin":
      case "focusout":
      case "input":
      case "invalid":
      case "keydown":
      case "keypress":
      case "keyup":
      case "mousedown":
      case "mouseup":
      case "paste":
      case "pause":
      case "play":
      case "pointercancel":
      case "pointerdown":
      case "pointerup":
      case "ratechange":
      case "reset":
      case "resize":
      case "seeked":
      case "submit":
      case "touchcancel":
      case "touchend":
      case "touchstart":
      case "volumechange":
      case "change":
      case "selectionchange":
      case "textInput":
      case "compositionstart":
      case "compositionend":
      case "compositionupdate":
      case "beforeblur":
      case "afterblur":
      case "beforeinput":
      case "blur":
      case "fullscreenchange":
      case "focus":
      case "hashchange":
      case "popstate":
      case "select":
      case "selectstart":
        return 1;
      case "drag":
      case "dragenter":
      case "dragexit":
      case "dragleave":
      case "dragover":
      case "mousemove":
      case "mouseout":
      case "mouseover":
      case "pointermove":
      case "pointerout":
      case "pointerover":
      case "scroll":
      case "toggle":
      case "touchmove":
      case "wheel":
      case "mouseenter":
      case "mouseleave":
      case "pointerenter":
      case "pointerleave":
        return 4;
      case "message":
        switch (_b()) {
          case Mi:
            return 1;
          case Tt:
            return 4;
          case fo:
          case Mm:
            return 16;
          case Ip:
            return 536870912;
          default:
            return 16;
        }
      default:
        return 16;
    }
  }
  var Li = null, ws = null, Ts = null;
  function Fp() {
    if (Ts)
      return Ts;
    var n, i = ws, s = i.length, p, g = "value" in Li ? Li.value : Li.textContent, S = g.length;
    for (n = 0; n < s && i[n] === g[n]; n++)
      ;
    var I = s - n;
    for (p = 1; p <= I && i[s - p] === g[S - p]; p++)
      ;
    return Ts = g.slice(n, 1 < p ? 1 - p : void 0);
  }
  function qu(n) {
    var i = n.keyCode;
    return "charCode" in n ? (n = n.charCode, n === 0 && i === 13 && (n = 13)) : n = i, n === 10 && (n = 13), 32 <= n || n === 13 ? n : 0;
  }
  function Rs() {
    return !0;
  }
  function _m() {
    return !1;
  }
  function ra(n) {
    function i(s, p, g, S, I) {
      this._reactName = s, this._targetInst = g, this.type = p, this.nativeEvent = S, this.target = I, this.currentTarget = null;
      for (var F in n)
        n.hasOwnProperty(F) && (s = n[F], this[F] = s ? s(S) : S[F]);
      return this.isDefaultPrevented = (S.defaultPrevented != null ? S.defaultPrevented : S.returnValue === !1) ? Rs : _m, this.isPropagationStopped = _m, this;
    }
    return W(i.prototype, { preventDefault: function() {
      this.defaultPrevented = !0;
      var s = this.nativeEvent;
      s && (s.preventDefault ? s.preventDefault() : typeof s.returnValue != "unknown" && (s.returnValue = !1), this.isDefaultPrevented = Rs);
    }, stopPropagation: function() {
      var s = this.nativeEvent;
      s && (s.stopPropagation ? s.stopPropagation() : typeof s.cancelBubble != "unknown" && (s.cancelBubble = !0), this.isPropagationStopped = Rs);
    }, persist: function() {
    }, isPersistent: Rs }), i;
  }
  var vo = { eventPhase: 0, bubbles: 0, cancelable: 0, timeStamp: function(n) {
    return n.timeStamp || Date.now();
  }, defaultPrevented: 0, isTrusted: 0 }, nf = ra(vo), el = W({}, vo, { view: 0, detail: 0 }), jm = ra(el), rf, Bp, Is, Wn = W({}, el, { screenX: 0, screenY: 0, clientX: 0, clientY: 0, pageX: 0, pageY: 0, ctrlKey: 0, shiftKey: 0, altKey: 0, metaKey: 0, getModifierState: zp, button: 0, buttons: 0, relatedTarget: function(n) {
    return n.relatedTarget === void 0 ? n.fromElement === n.srcElement ? n.toElement : n.fromElement : n.relatedTarget;
  }, movementX: function(n) {
    return "movementX" in n ? n.movementX : (n !== Is && (Is && n.type === "mousemove" ? (rf = n.screenX - Is.screenX, Bp = n.screenY - Is.screenY) : Bp = rf = 0, Is = n), rf);
  }, movementY: function(n) {
    return "movementY" in n ? n.movementY : Bp;
  } }), af = ra(Wn), Um = W({}, Wn, { dataTransfer: 0 }), Wm = ra(Um), Xb = W({}, el, { relatedTarget: 0 }), ho = ra(Xb), Vp = W({}, vo, { animationName: 0, elapsedTime: 0, pseudoElement: 0 }), Xm = ra(Vp), Zb = W({}, vo, { clipboardData: function(n) {
    return "clipboardData" in n ? n.clipboardData : window.clipboardData;
  } }), Yb = ra(Zb), $b = W({}, vo, { data: 0 }), Pp = ra($b), Hp = {
    Esc: "Escape",
    Spacebar: " ",
    Left: "ArrowLeft",
    Up: "ArrowUp",
    Right: "ArrowRight",
    Down: "ArrowDown",
    Del: "Delete",
    Win: "OS",
    Menu: "ContextMenu",
    Apps: "ContextMenu",
    Scroll: "ScrollLock",
    MozPrintableKey: "Unidentified"
  }, Zm = {
    8: "Backspace",
    9: "Tab",
    12: "Clear",
    13: "Enter",
    16: "Shift",
    17: "Control",
    18: "Alt",
    19: "Pause",
    20: "CapsLock",
    27: "Escape",
    32: " ",
    33: "PageUp",
    34: "PageDown",
    35: "End",
    36: "Home",
    37: "ArrowLeft",
    38: "ArrowUp",
    39: "ArrowRight",
    40: "ArrowDown",
    45: "Insert",
    46: "Delete",
    112: "F1",
    113: "F2",
    114: "F3",
    115: "F4",
    116: "F5",
    117: "F6",
    118: "F7",
    119: "F8",
    120: "F9",
    121: "F10",
    122: "F11",
    123: "F12",
    144: "NumLock",
    145: "ScrollLock",
    224: "Meta"
  }, Ym = { Alt: "altKey", Control: "ctrlKey", Meta: "metaKey", Shift: "shiftKey" };
  function $m(n) {
    var i = this.nativeEvent;
    return i.getModifierState ? i.getModifierState(n) : (n = Ym[n]) ? !!i[n] : !1;
  }
  function zp() {
    return $m;
  }
  var Fi = W({}, el, { key: function(n) {
    if (n.key) {
      var i = Hp[n.key] || n.key;
      if (i !== "Unidentified")
        return i;
    }
    return n.type === "keypress" ? (n = qu(n), n === 13 ? "Enter" : String.fromCharCode(n)) : n.type === "keydown" || n.type === "keyup" ? Zm[n.keyCode] || "Unidentified" : "";
  }, code: 0, location: 0, ctrlKey: 0, shiftKey: 0, altKey: 0, metaKey: 0, repeat: 0, locale: 0, getModifierState: zp, charCode: function(n) {
    return n.type === "keypress" ? qu(n) : 0;
  }, keyCode: function(n) {
    return n.type === "keydown" || n.type === "keyup" ? n.keyCode : 0;
  }, which: function(n) {
    return n.type === "keypress" ? qu(n) : n.type === "keydown" || n.type === "keyup" ? n.keyCode : 0;
  } }), Qb = ra(Fi), Gp = W({}, Wn, { pointerId: 0, width: 0, height: 0, pressure: 0, tangentialPressure: 0, tiltX: 0, tiltY: 0, twist: 0, pointerType: 0, isPrimary: 0 }), of = ra(Gp), _p = W({}, el, { touches: 0, targetTouches: 0, changedTouches: 0, altKey: 0, metaKey: 0, ctrlKey: 0, shiftKey: 0, getModifierState: zp }), Jb = ra(_p), uf = W({}, vo, { propertyName: 0, elapsedTime: 0, pseudoElement: 0 }), Qm = ra(uf), Vr = W({}, Wn, {
    deltaX: function(n) {
      return "deltaX" in n ? n.deltaX : "wheelDeltaX" in n ? -n.wheelDeltaX : 0;
    },
    deltaY: function(n) {
      return "deltaY" in n ? n.deltaY : "wheelDeltaY" in n ? -n.wheelDeltaY : "wheelDelta" in n ? -n.wheelDelta : 0;
    },
    deltaZ: 0,
    deltaMode: 0
  }), Bi = ra(Vr), Dn = [9, 13, 27, 32], ei = C && "CompositionEvent" in window, eu = null;
  C && "documentMode" in document && (eu = document.documentMode);
  var lf = C && "TextEvent" in window && !eu, Jm = C && (!ei || eu && 8 < eu && 11 >= eu), tl = " ", Km = !1;
  function qm(n, i) {
    switch (n) {
      case "keyup":
        return Dn.indexOf(i.keyCode) !== -1;
      case "keydown":
        return i.keyCode !== 229;
      case "keypress":
      case "mousedown":
      case "focusout":
        return !0;
      default:
        return !1;
    }
  }
  function sf(n) {
    return n = n.detail, typeof n == "object" && "data" in n ? n.data : null;
  }
  var nl = !1;
  function Kb(n, i) {
    switch (n) {
      case "compositionend":
        return sf(i);
      case "keypress":
        return i.which !== 32 ? null : (Km = !0, tl);
      case "textInput":
        return n = i.data, n === tl && Km ? null : n;
      default:
        return null;
    }
  }
  function qb(n, i) {
    if (nl)
      return n === "compositionend" || !ei && qm(n, i) ? (n = Fp(), Ts = ws = Li = null, nl = !1, n) : null;
    switch (n) {
      case "paste":
        return null;
      case "keypress":
        if (!(i.ctrlKey || i.altKey || i.metaKey) || i.ctrlKey && i.altKey) {
          if (i.char && 1 < i.char.length)
            return i.char;
          if (i.which)
            return String.fromCharCode(i.which);
        }
        return null;
      case "compositionend":
        return Jm && i.locale !== "ko" ? null : i.data;
      default:
        return null;
    }
  }
  var eg = { color: !0, date: !0, datetime: !0, "datetime-local": !0, email: !0, month: !0, number: !0, password: !0, range: !0, search: !0, tel: !0, text: !0, time: !0, url: !0, week: !0 };
  function tg(n) {
    var i = n && n.nodeName && n.nodeName.toLowerCase();
    return i === "input" ? !!eg[n.type] : i === "textarea";
  }
  function ng(n, i, s, p) {
    Wc(p), i = Fs(i, "onChange"), 0 < i.length && (s = new nf("onChange", "change", null, s, p), n.push({ event: s, listeners: i }));
  }
  var As = null, rl = null;
  function al(n) {
    pf(n, 0);
  }
  function il(n) {
    var i = ul(n);
    if (Ce(i))
      return n;
  }
  function rg(n, i) {
    if (n === "change")
      return i;
  }
  var jp = !1;
  if (C) {
    var Up;
    if (C) {
      var Wp = "oninput" in document;
      if (!Wp) {
        var ag = document.createElement("div");
        ag.setAttribute("oninput", "return;"), Wp = typeof ag.oninput == "function";
      }
      Up = Wp;
    } else
      Up = !1;
    jp = Up && (!document.documentMode || 9 < document.documentMode);
  }
  function ig() {
    As && (As.detachEvent("onpropertychange", og), rl = As = null);
  }
  function og(n) {
    if (n.propertyName === "value" && il(rl)) {
      var i = [];
      ng(i, rl, n, vn(n)), xp(al, i);
    }
  }
  function eC(n, i, s) {
    n === "focusin" ? (ig(), As = i, rl = s, As.attachEvent("onpropertychange", og)) : n === "focusout" && ig();
  }
  function tC(n) {
    if (n === "selectionchange" || n === "keyup" || n === "keydown")
      return il(rl);
  }
  function nC(n, i) {
    if (n === "click")
      return il(i);
  }
  function ug(n, i) {
    if (n === "input" || n === "change")
      return il(i);
  }
  function rC(n, i) {
    return n === i && (n !== 0 || 1 / n === 1 / i) || n !== n && i !== i;
  }
  var Ea = typeof Object.is == "function" ? Object.is : rC;
  function Os(n, i) {
    if (Ea(n, i))
      return !0;
    if (typeof n != "object" || n === null || typeof i != "object" || i === null)
      return !1;
    var s = Object.keys(n), p = Object.keys(i);
    if (s.length !== p.length)
      return !1;
    for (p = 0; p < s.length; p++) {
      var g = s[p];
      if (!b.call(i, g) || !Ea(n[g], i[g]))
        return !1;
    }
    return !0;
  }
  function lg(n) {
    for (; n && n.firstChild; )
      n = n.firstChild;
    return n;
  }
  function sg(n, i) {
    var s = lg(n);
    n = 0;
    for (var p; s; ) {
      if (s.nodeType === 3) {
        if (p = n + s.textContent.length, n <= i && p >= i)
          return { node: s, offset: i - n };
        n = p;
      }
      e: {
        for (; s; ) {
          if (s.nextSibling) {
            s = s.nextSibling;
            break e;
          }
          s = s.parentNode;
        }
        s = void 0;
      }
      s = lg(s);
    }
  }
  function cg(n, i) {
    return n && i ? n === i ? !0 : n && n.nodeType === 3 ? !1 : i && i.nodeType === 3 ? cg(n, i.parentNode) : "contains" in n ? n.contains(i) : n.compareDocumentPosition ? !!(n.compareDocumentPosition(i) & 16) : !1 : !1;
  }
  function cf() {
    for (var n = window, i = Ot(); i instanceof n.HTMLIFrameElement; ) {
      try {
        var s = typeof i.contentWindow.location.href == "string";
      } catch {
        s = !1;
      }
      if (s)
        n = i.contentWindow;
      else
        break;
      i = Ot(n.document);
    }
    return i;
  }
  function Vi(n) {
    var i = n && n.nodeName && n.nodeName.toLowerCase();
    return i && (i === "input" && (n.type === "text" || n.type === "search" || n.type === "tel" || n.type === "url" || n.type === "password") || i === "textarea" || n.contentEditable === "true");
  }
  function ff(n) {
    var i = cf(), s = n.focusedElem, p = n.selectionRange;
    if (i !== s && s && s.ownerDocument && cg(s.ownerDocument.documentElement, s)) {
      if (p !== null && Vi(s)) {
        if (i = p.start, n = p.end, n === void 0 && (n = i), "selectionStart" in s)
          s.selectionStart = i, s.selectionEnd = Math.min(n, s.value.length);
        else if (n = (i = s.ownerDocument || document) && i.defaultView || window, n.getSelection) {
          n = n.getSelection();
          var g = s.textContent.length, S = Math.min(p.start, g);
          p = p.end === void 0 ? S : Math.min(p.end, g), !n.extend && S > p && (g = p, p = S, S = g), g = sg(s, S);
          var I = sg(
            s,
            p
          );
          g && I && (n.rangeCount !== 1 || n.anchorNode !== g.node || n.anchorOffset !== g.offset || n.focusNode !== I.node || n.focusOffset !== I.offset) && (i = i.createRange(), i.setStart(g.node, g.offset), n.removeAllRanges(), S > p ? (n.addRange(i), n.extend(I.node, I.offset)) : (i.setEnd(I.node, I.offset), n.addRange(i)));
        }
      }
      for (i = [], n = s; n = n.parentNode; )
        n.nodeType === 1 && i.push({ element: n, left: n.scrollLeft, top: n.scrollTop });
      for (typeof s.focus == "function" && s.focus(), s = 0; s < i.length; s++)
        n = i[s], n.element.scrollLeft = n.left, n.element.scrollTop = n.top;
    }
  }
  var fg = C && "documentMode" in document && 11 >= document.documentMode, ti = null, Xp = null, Ds = null, Zp = !1;
  function dg(n, i, s) {
    var p = s.window === s ? s.document : s.nodeType === 9 ? s : s.ownerDocument;
    Zp || ti == null || ti !== Ot(p) || (p = ti, "selectionStart" in p && Vi(p) ? p = { start: p.selectionStart, end: p.selectionEnd } : (p = (p.ownerDocument && p.ownerDocument.defaultView || window).getSelection(), p = { anchorNode: p.anchorNode, anchorOffset: p.anchorOffset, focusNode: p.focusNode, focusOffset: p.focusOffset }), Ds && Os(Ds, p) || (Ds = p, p = Fs(Xp, "onSelect"), 0 < p.length && (i = new nf("onSelect", "select", null, i, s), n.push({ event: i, listeners: p }), i.target = ti)));
  }
  function df(n, i) {
    var s = {};
    return s[n.toLowerCase()] = i.toLowerCase(), s["Webkit" + n] = "webkit" + i, s["Moz" + n] = "moz" + i, s;
  }
  var tu = { animationend: df("Animation", "AnimationEnd"), animationiteration: df("Animation", "AnimationIteration"), animationstart: df("Animation", "AnimationStart"), transitionend: df("Transition", "TransitionEnd") }, Yp = {}, $p = {};
  C && ($p = document.createElement("div").style, "AnimationEvent" in window || (delete tu.animationend.animation, delete tu.animationiteration.animation, delete tu.animationstart.animation), "TransitionEvent" in window || delete tu.transitionend.transition);
  function Xn(n) {
    if (Yp[n])
      return Yp[n];
    if (!tu[n])
      return n;
    var i = tu[n], s;
    for (s in i)
      if (i.hasOwnProperty(s) && s in $p)
        return Yp[n] = i[s];
    return n;
  }
  var Qp = Xn("animationend"), pg = Xn("animationiteration"), vg = Xn("animationstart"), hg = Xn("transitionend"), mg = /* @__PURE__ */ new Map(), gg = "abort auxClick cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");
  function Pi(n, i) {
    mg.set(n, i), v(i, [n]);
  }
  for (var Ns = 0; Ns < gg.length; Ns++) {
    var nu = gg[Ns], aC = nu.toLowerCase(), ks = nu[0].toUpperCase() + nu.slice(1);
    Pi(aC, "on" + ks);
  }
  Pi(Qp, "onAnimationEnd"), Pi(pg, "onAnimationIteration"), Pi(vg, "onAnimationStart"), Pi("dblclick", "onDoubleClick"), Pi("focusin", "onFocus"), Pi("focusout", "onBlur"), Pi(hg, "onTransitionEnd"), h("onMouseEnter", ["mouseout", "mouseover"]), h("onMouseLeave", ["mouseout", "mouseover"]), h("onPointerEnter", ["pointerout", "pointerover"]), h("onPointerLeave", ["pointerout", "pointerover"]), v("onChange", "change click focusin focusout input keydown keyup selectionchange".split(" ")), v("onSelect", "focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")), v("onBeforeInput", ["compositionend", "keypress", "textInput", "paste"]), v("onCompositionEnd", "compositionend focusout keydown keypress keyup mousedown".split(" ")), v("onCompositionStart", "compositionstart focusout keydown keypress keyup mousedown".split(" ")), v("onCompositionUpdate", "compositionupdate focusout keydown keypress keyup mousedown".split(" "));
  var Ms = "abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "), iC = new Set("cancel close invalid load scroll toggle".split(" ").concat(Ms));
  function yg(n, i, s) {
    var p = n.type || "unknown-event";
    n.currentTarget = s, Yu(p, i, void 0, n), n.currentTarget = null;
  }
  function pf(n, i) {
    i = (i & 4) !== 0;
    for (var s = 0; s < n.length; s++) {
      var p = n[s], g = p.event;
      p = p.listeners;
      e: {
        var S = void 0;
        if (i)
          for (var I = p.length - 1; 0 <= I; I--) {
            var F = p[I], H = F.instance, ne = F.currentTarget;
            if (F = F.listener, H !== S && g.isPropagationStopped())
              break e;
            yg(g, F, ne), S = H;
          }
        else
          for (I = 0; I < p.length; I++) {
            if (F = p[I], H = F.instance, ne = F.currentTarget, F = F.listener, H !== S && g.isPropagationStopped())
              break e;
            yg(g, F, ne), S = H;
          }
      }
    }
    if (Zu)
      throw n = hs, Zu = !1, hs = null, n;
  }
  function qt(n, i) {
    var s = i[rv];
    s === void 0 && (s = i[rv] = /* @__PURE__ */ new Set());
    var p = n + "__bubble";
    s.has(p) || (bg(i, n, 2, !1), s.add(p));
  }
  function mo(n, i, s) {
    var p = 0;
    i && (p |= 4), bg(s, n, p, i);
  }
  var Hi = "_reactListening" + Math.random().toString(36).slice(2);
  function ol(n) {
    if (!n[Hi]) {
      n[Hi] = !0, f.forEach(function(s) {
        s !== "selectionchange" && (iC.has(s) || mo(s, !1, n), mo(s, !0, n));
      });
      var i = n.nodeType === 9 ? n : n.ownerDocument;
      i === null || i[Hi] || (i[Hi] = !0, mo("selectionchange", !1, i));
    }
  }
  function bg(n, i, s, p) {
    switch (Lp(i)) {
      case 1:
        var g = Gm;
        break;
      case 4:
        g = qc;
        break;
      default:
        g = ef;
    }
    s = g.bind(null, i, s, n), g = void 0, !Yc || i !== "touchstart" && i !== "touchmove" && i !== "wheel" || (g = !0), p ? g !== void 0 ? n.addEventListener(i, s, { capture: !0, passive: g }) : n.addEventListener(i, s, !0) : g !== void 0 ? n.addEventListener(i, s, { passive: g }) : n.addEventListener(i, s, !1);
  }
  function vf(n, i, s, p, g) {
    var S = p;
    if (!(i & 1) && !(i & 2) && p !== null)
      e:
        for (; ; ) {
          if (p === null)
            return;
          var I = p.tag;
          if (I === 3 || I === 4) {
            var F = p.stateNode.containerInfo;
            if (F === g || F.nodeType === 8 && F.parentNode === g)
              break;
            if (I === 4)
              for (I = p.return; I !== null; ) {
                var H = I.tag;
                if ((H === 3 || H === 4) && (H = I.stateNode.containerInfo, H === g || H.nodeType === 8 && H.parentNode === g))
                  return;
                I = I.return;
              }
            for (; F !== null; ) {
              if (I = xa(F), I === null)
                return;
              if (H = I.tag, H === 5 || H === 6) {
                p = S = I;
                continue e;
              }
              F = F.parentNode;
            }
          }
          p = p.return;
        }
    xp(function() {
      var ne = S, pe = vn(s), ve = [];
      e: {
        var de = mg.get(n);
        if (de !== void 0) {
          var Fe = nf, ze = n;
          switch (n) {
            case "keypress":
              if (qu(s) === 0)
                break e;
            case "keydown":
            case "keyup":
              Fe = Qb;
              break;
            case "focusin":
              ze = "focus", Fe = ho;
              break;
            case "focusout":
              ze = "blur", Fe = ho;
              break;
            case "beforeblur":
            case "afterblur":
              Fe = ho;
              break;
            case "click":
              if (s.button === 2)
                break e;
            case "auxclick":
            case "dblclick":
            case "mousedown":
            case "mousemove":
            case "mouseup":
            case "mouseout":
            case "mouseover":
            case "contextmenu":
              Fe = af;
              break;
            case "drag":
            case "dragend":
            case "dragenter":
            case "dragexit":
            case "dragleave":
            case "dragover":
            case "dragstart":
            case "drop":
              Fe = Wm;
              break;
            case "touchcancel":
            case "touchend":
            case "touchmove":
            case "touchstart":
              Fe = Jb;
              break;
            case Qp:
            case pg:
            case vg:
              Fe = Xm;
              break;
            case hg:
              Fe = Qm;
              break;
            case "scroll":
              Fe = jm;
              break;
            case "wheel":
              Fe = Bi;
              break;
            case "copy":
            case "cut":
            case "paste":
              Fe = Yb;
              break;
            case "gotpointercapture":
            case "lostpointercapture":
            case "pointercancel":
            case "pointerdown":
            case "pointermove":
            case "pointerout":
            case "pointerover":
            case "pointerup":
              Fe = of;
          }
          var je = (i & 4) !== 0, Tn = !je && n === "scroll", J = je ? de !== null ? de + "Capture" : null : de;
          je = [];
          for (var _ = ne, ee; _ !== null; ) {
            ee = _;
            var be = ee.stateNode;
            if (ee.tag === 5 && be !== null && (ee = be, J !== null && (be = vs(_, J), be != null && je.push(Ls(_, be, ee)))), Tn)
              break;
            _ = _.return;
          }
          0 < je.length && (de = new Fe(de, ze, null, s, pe), ve.push({ event: de, listeners: je }));
        }
      }
      if (!(i & 7)) {
        e: {
          if (de = n === "mouseover" || n === "pointerover", Fe = n === "mouseout" || n === "pointerout", de && s !== ya && (ze = s.relatedTarget || s.fromElement) && (xa(ze) || ze[zi]))
            break e;
          if ((Fe || de) && (de = pe.window === pe ? pe : (de = pe.ownerDocument) ? de.defaultView || de.parentWindow : window, Fe ? (ze = s.relatedTarget || s.toElement, Fe = ne, ze = ze ? xa(ze) : null, ze !== null && (Tn = di(ze), ze !== Tn || ze.tag !== 5 && ze.tag !== 6) && (ze = null)) : (Fe = null, ze = ne), Fe !== ze)) {
            if (je = af, be = "onMouseLeave", J = "onMouseEnter", _ = "mouse", (n === "pointerout" || n === "pointerover") && (je = of, be = "onPointerLeave", J = "onPointerEnter", _ = "pointer"), Tn = Fe == null ? de : ul(Fe), ee = ze == null ? de : ul(ze), de = new je(be, _ + "leave", Fe, s, pe), de.target = Tn, de.relatedTarget = ee, be = null, xa(pe) === ne && (je = new je(J, _ + "enter", ze, s, pe), je.target = ee, je.relatedTarget = Tn, be = je), Tn = be, Fe && ze)
              t: {
                for (je = Fe, J = ze, _ = 0, ee = je; ee; ee = ru(ee))
                  _++;
                for (ee = 0, be = J; be; be = ru(be))
                  ee++;
                for (; 0 < _ - ee; )
                  je = ru(je), _--;
                for (; 0 < ee - _; )
                  J = ru(J), ee--;
                for (; _--; ) {
                  if (je === J || J !== null && je === J.alternate)
                    break t;
                  je = ru(je), J = ru(J);
                }
                je = null;
              }
            else
              je = null;
            Fe !== null && Jp(ve, de, Fe, je, !1), ze !== null && Tn !== null && Jp(ve, Tn, ze, je, !0);
          }
        }
        e: {
          if (de = ne ? ul(ne) : window, Fe = de.nodeName && de.nodeName.toLowerCase(), Fe === "select" || Fe === "input" && de.type === "file")
            var Ze = rg;
          else if (tg(de))
            if (jp)
              Ze = ug;
            else {
              Ze = tC;
              var at = eC;
            }
          else
            (Fe = de.nodeName) && Fe.toLowerCase() === "input" && (de.type === "checkbox" || de.type === "radio") && (Ze = nC);
          if (Ze && (Ze = Ze(n, ne))) {
            ng(ve, Ze, s, pe);
            break e;
          }
          at && at(n, de, ne), n === "focusout" && (at = de._wrapperState) && at.controlled && de.type === "number" && so(de, "number", de.value);
        }
        switch (at = ne ? ul(ne) : window, n) {
          case "focusin":
            (tg(at) || at.contentEditable === "true") && (ti = at, Xp = ne, Ds = null);
            break;
          case "focusout":
            Ds = Xp = ti = null;
            break;
          case "mousedown":
            Zp = !0;
            break;
          case "contextmenu":
          case "mouseup":
          case "dragend":
            Zp = !1, dg(ve, s, pe);
            break;
          case "selectionchange":
            if (fg)
              break;
          case "keydown":
          case "keyup":
            dg(ve, s, pe);
        }
        var Ge;
        if (ei)
          e: {
            switch (n) {
              case "compositionstart":
                var ot = "onCompositionStart";
                break e;
              case "compositionend":
                ot = "onCompositionEnd";
                break e;
              case "compositionupdate":
                ot = "onCompositionUpdate";
                break e;
            }
            ot = void 0;
          }
        else
          nl ? qm(n, s) && (ot = "onCompositionEnd") : n === "keydown" && s.keyCode === 229 && (ot = "onCompositionStart");
        ot && (Jm && s.locale !== "ko" && (nl || ot !== "onCompositionStart" ? ot === "onCompositionEnd" && nl && (Ge = Fp()) : (Li = pe, ws = "value" in Li ? Li.value : Li.textContent, nl = !0)), at = Fs(ne, ot), 0 < at.length && (ot = new Pp(ot, n, null, s, pe), ve.push({ event: ot, listeners: at }), Ge ? ot.data = Ge : (Ge = sf(s), Ge !== null && (ot.data = Ge)))), (Ge = lf ? Kb(n, s) : qb(n, s)) && (ne = Fs(ne, "onBeforeInput"), 0 < ne.length && (pe = new Pp("onBeforeInput", "beforeinput", null, s, pe), ve.push({ event: pe, listeners: ne }), pe.data = Ge));
      }
      pf(ve, i);
    });
  }
  function Ls(n, i, s) {
    return { instance: n, listener: i, currentTarget: s };
  }
  function Fs(n, i) {
    for (var s = i + "Capture", p = []; n !== null; ) {
      var g = n, S = g.stateNode;
      g.tag === 5 && S !== null && (g = S, S = vs(n, s), S != null && p.unshift(Ls(n, S, g)), S = vs(n, i), S != null && p.push(Ls(n, S, g))), n = n.return;
    }
    return p;
  }
  function ru(n) {
    if (n === null)
      return null;
    do
      n = n.return;
    while (n && n.tag !== 5);
    return n || null;
  }
  function Jp(n, i, s, p, g) {
    for (var S = i._reactName, I = []; s !== null && s !== p; ) {
      var F = s, H = F.alternate, ne = F.stateNode;
      if (H !== null && H === p)
        break;
      F.tag === 5 && ne !== null && (F = ne, g ? (H = vs(s, S), H != null && I.unshift(Ls(s, H, F))) : g || (H = vs(s, S), H != null && I.push(Ls(s, H, F)))), s = s.return;
    }
    I.length !== 0 && n.push({ event: i, listeners: I });
  }
  var Kp = /\r\n?/g, oC = /\u0000|\uFFFD/g;
  function qp(n) {
    return (typeof n == "string" ? n : "" + n).replace(Kp, `
`).replace(oC, "");
  }
  function hf(n, i, s) {
    if (i = qp(i), qp(n) !== i && s)
      throw Error(u(425));
  }
  function mf() {
  }
  var ev = null, au = null;
  function Bs(n, i) {
    return n === "textarea" || n === "noscript" || typeof i.children == "string" || typeof i.children == "number" || typeof i.dangerouslySetInnerHTML == "object" && i.dangerouslySetInnerHTML !== null && i.dangerouslySetInnerHTML.__html != null;
  }
  var iu = typeof setTimeout == "function" ? setTimeout : void 0, Cg = typeof clearTimeout == "function" ? clearTimeout : void 0, tv = typeof Promise == "function" ? Promise : void 0, nv = typeof queueMicrotask == "function" ? queueMicrotask : typeof tv < "u" ? function(n) {
    return tv.resolve(null).then(n).catch(uC);
  } : iu;
  function uC(n) {
    setTimeout(function() {
      throw n;
    });
  }
  function go(n, i) {
    var s = i, p = 0;
    do {
      var g = s.nextSibling;
      if (n.removeChild(s), g && g.nodeType === 8)
        if (s = g.data, s === "/$") {
          if (p === 0) {
            n.removeChild(g), Es(i);
            return;
          }
          p--;
        } else
          s !== "$" && s !== "$?" && s !== "$!" || p++;
      s = g;
    } while (s);
    Es(i);
  }
  function ni(n) {
    for (; n != null; n = n.nextSibling) {
      var i = n.nodeType;
      if (i === 1 || i === 3)
        break;
      if (i === 8) {
        if (i = n.data, i === "$" || i === "$!" || i === "$?")
          break;
        if (i === "/$")
          return null;
      }
    }
    return n;
  }
  function Vs(n) {
    n = n.previousSibling;
    for (var i = 0; n; ) {
      if (n.nodeType === 8) {
        var s = n.data;
        if (s === "$" || s === "$!" || s === "$?") {
          if (i === 0)
            return n;
          i--;
        } else
          s === "/$" && i++;
      }
      n = n.previousSibling;
    }
    return null;
  }
  var yo = Math.random().toString(36).slice(2), pi = "__reactFiber$" + yo, ou = "__reactProps$" + yo, zi = "__reactContainer$" + yo, rv = "__reactEvents$" + yo, lC = "__reactListeners$" + yo, av = "__reactHandles$" + yo;
  function xa(n) {
    var i = n[pi];
    if (i)
      return i;
    for (var s = n.parentNode; s; ) {
      if (i = s[zi] || s[pi]) {
        if (s = i.alternate, i.child !== null || s !== null && s.child !== null)
          for (n = Vs(n); n !== null; ) {
            if (s = n[pi])
              return s;
            n = Vs(n);
          }
        return i;
      }
      n = s, s = n.parentNode;
    }
    return null;
  }
  function Ps(n) {
    return n = n[pi] || n[zi], !n || n.tag !== 5 && n.tag !== 6 && n.tag !== 13 && n.tag !== 3 ? null : n;
  }
  function ul(n) {
    if (n.tag === 5 || n.tag === 6)
      return n.stateNode;
    throw Error(u(33));
  }
  function qe(n) {
    return n[ou] || null;
  }
  var bo = [], nn = -1;
  function St(n) {
    return { current: n };
  }
  function Pt(n) {
    0 > nn || (n.current = bo[nn], bo[nn] = null, nn--);
  }
  function Ht(n, i) {
    nn++, bo[nn] = n.current, n.current = i;
  }
  var hi = {}, dt = St(hi), gn = St(!1), Pr = hi;
  function wa(n, i) {
    var s = n.type.contextTypes;
    if (!s)
      return hi;
    var p = n.stateNode;
    if (p && p.__reactInternalMemoizedUnmaskedChildContext === i)
      return p.__reactInternalMemoizedMaskedChildContext;
    var g = {}, S;
    for (S in s)
      g[S] = i[S];
    return p && (n = n.stateNode, n.__reactInternalMemoizedUnmaskedChildContext = i, n.__reactInternalMemoizedMaskedChildContext = g), g;
  }
  function un(n) {
    return n = n.childContextTypes, n != null;
  }
  function Ta() {
    Pt(gn), Pt(dt);
  }
  function Co(n, i, s) {
    if (dt.current !== hi)
      throw Error(u(168));
    Ht(dt, i), Ht(gn, s);
  }
  function Hs(n, i, s) {
    var p = n.stateNode;
    if (i = i.childContextTypes, typeof p.getChildContext != "function")
      return s;
    p = p.getChildContext();
    for (var g in p)
      if (!(g in i))
        throw Error(u(108, Ue(n) || "Unknown", g));
    return W({}, s, p);
  }
  function gf(n) {
    return n = (n = n.stateNode) && n.__reactInternalMemoizedMergedChildContext || hi, Pr = dt.current, Ht(dt, n), Ht(gn, gn.current), !0;
  }
  function Sg(n, i, s) {
    var p = n.stateNode;
    if (!p)
      throw Error(u(169));
    s ? (n = Hs(n, i, Pr), p.__reactInternalMemoizedMergedChildContext = n, Pt(gn), Pt(dt), Ht(dt, n)) : Pt(gn), Ht(gn, s);
  }
  var aa = null, Zn = !1, zs = !1;
  function iv(n) {
    aa === null ? aa = [n] : aa.push(n);
  }
  function ov(n) {
    Zn = !0, iv(n);
  }
  function Hr() {
    if (!zs && aa !== null) {
      zs = !0;
      var n = 0, i = jt;
      try {
        var s = aa;
        for (jt = 1; n < s.length; n++) {
          var p = s[n];
          do
            p = p(!0);
          while (p !== null);
        }
        aa = null, Zn = !1;
      } catch (g) {
        throw aa !== null && (aa = aa.slice(n + 1)), Rp(Mi, Hr), g;
      } finally {
        jt = i, zs = !1;
      }
    }
    return null;
  }
  var So = [], zr = 0, uu = null, ll = 0, Gr = [], hr = 0, Ra = null, nr = 1, Gi = "";
  function ia(n, i) {
    So[zr++] = ll, So[zr++] = uu, uu = n, ll = i;
  }
  function uv(n, i, s) {
    Gr[hr++] = nr, Gr[hr++] = Gi, Gr[hr++] = Ra, Ra = n;
    var p = nr;
    n = Gi;
    var g = 32 - ba(p) - 1;
    p &= ~(1 << g), s += 1;
    var S = 32 - ba(i) + g;
    if (30 < S) {
      var I = g - g % 5;
      S = (p & (1 << I) - 1).toString(32), p >>= I, g -= I, nr = 1 << 32 - ba(i) + g | s << g | p, Gi = S + n;
    } else
      nr = 1 << S | s << g | p, Gi = n;
  }
  function yf(n) {
    n.return !== null && (ia(n, 1), uv(n, 1, 0));
  }
  function lv(n) {
    for (; n === uu; )
      uu = So[--zr], So[zr] = null, ll = So[--zr], So[zr] = null;
    for (; n === Ra; )
      Ra = Gr[--hr], Gr[hr] = null, Gi = Gr[--hr], Gr[hr] = null, nr = Gr[--hr], Gr[hr] = null;
  }
  var oa = null, _r = null, rn = !1, Ia = null;
  function sv(n, i) {
    var s = Fa(5, null, null, 0);
    s.elementType = "DELETED", s.stateNode = i, s.return = n, i = n.deletions, i === null ? (n.deletions = [s], n.flags |= 16) : i.push(s);
  }
  function Eg(n, i) {
    switch (n.tag) {
      case 5:
        var s = n.type;
        return i = i.nodeType !== 1 || s.toLowerCase() !== i.nodeName.toLowerCase() ? null : i, i !== null ? (n.stateNode = i, oa = n, _r = ni(i.firstChild), !0) : !1;
      case 6:
        return i = n.pendingProps === "" || i.nodeType !== 3 ? null : i, i !== null ? (n.stateNode = i, oa = n, _r = null, !0) : !1;
      case 13:
        return i = i.nodeType !== 8 ? null : i, i !== null ? (s = Ra !== null ? { id: nr, overflow: Gi } : null, n.memoizedState = { dehydrated: i, treeContext: s, retryLane: 1073741824 }, s = Fa(18, null, null, 0), s.stateNode = i, s.return = n, n.child = s, oa = n, _r = null, !0) : !1;
      default:
        return !1;
    }
  }
  function bf(n) {
    return (n.mode & 1) !== 0 && (n.flags & 128) === 0;
  }
  function Cf(n) {
    if (rn) {
      var i = _r;
      if (i) {
        var s = i;
        if (!Eg(n, i)) {
          if (bf(n))
            throw Error(u(418));
          i = ni(s.nextSibling);
          var p = oa;
          i && Eg(n, i) ? sv(p, s) : (n.flags = n.flags & -4097 | 2, rn = !1, oa = n);
        }
      } else {
        if (bf(n))
          throw Error(u(418));
        n.flags = n.flags & -4097 | 2, rn = !1, oa = n;
      }
    }
  }
  function xg(n) {
    for (n = n.return; n !== null && n.tag !== 5 && n.tag !== 3 && n.tag !== 13; )
      n = n.return;
    oa = n;
  }
  function Sf(n) {
    if (n !== oa)
      return !1;
    if (!rn)
      return xg(n), rn = !0, !1;
    var i;
    if ((i = n.tag !== 3) && !(i = n.tag !== 5) && (i = n.type, i = i !== "head" && i !== "body" && !Bs(n.type, n.memoizedProps)), i && (i = _r)) {
      if (bf(n))
        throw wg(), Error(u(418));
      for (; i; )
        sv(n, i), i = ni(i.nextSibling);
    }
    if (xg(n), n.tag === 13) {
      if (n = n.memoizedState, n = n !== null ? n.dehydrated : null, !n)
        throw Error(u(317));
      e: {
        for (n = n.nextSibling, i = 0; n; ) {
          if (n.nodeType === 8) {
            var s = n.data;
            if (s === "/$") {
              if (i === 0) {
                _r = ni(n.nextSibling);
                break e;
              }
              i--;
            } else
              s !== "$" && s !== "$!" && s !== "$?" || i++;
          }
          n = n.nextSibling;
        }
        _r = null;
      }
    } else
      _r = oa ? ni(n.stateNode.nextSibling) : null;
    return !0;
  }
  function wg() {
    for (var n = _r; n; )
      n = ni(n.nextSibling);
  }
  function cn() {
    _r = oa = null, rn = !1;
  }
  function cv(n) {
    Ia === null ? Ia = [n] : Ia.push(n);
  }
  var Ef = Z.ReactCurrentBatchConfig;
  function lu(n, i, s) {
    if (n = s.ref, n !== null && typeof n != "function" && typeof n != "object") {
      if (s._owner) {
        if (s = s._owner, s) {
          if (s.tag !== 1)
            throw Error(u(309));
          var p = s.stateNode;
        }
        if (!p)
          throw Error(u(147, n));
        var g = p, S = "" + n;
        return i !== null && i.ref !== null && typeof i.ref == "function" && i.ref._stringRef === S ? i.ref : (i = function(I) {
          var F = g.refs;
          I === null ? delete F[S] : F[S] = I;
        }, i._stringRef = S, i);
      }
      if (typeof n != "string")
        throw Error(u(284));
      if (!s._owner)
        throw Error(u(290, n));
    }
    return n;
  }
  function mi(n, i) {
    throw n = Object.prototype.toString.call(i), Error(u(31, n === "[object Object]" ? "object with keys {" + Object.keys(i).join(", ") + "}" : n));
  }
  function Tg(n) {
    var i = n._init;
    return i(n._payload);
  }
  function xf(n) {
    function i(J, _) {
      if (n) {
        var ee = J.deletions;
        ee === null ? (J.deletions = [_], J.flags |= 16) : ee.push(_);
      }
    }
    function s(J, _) {
      if (!n)
        return null;
      for (; _ !== null; )
        i(J, _), _ = _.sibling;
      return null;
    }
    function p(J, _) {
      for (J = /* @__PURE__ */ new Map(); _ !== null; )
        _.key !== null ? J.set(_.key, _) : J.set(_.index, _), _ = _.sibling;
      return J;
    }
    function g(J, _) {
      return J = Oo(J, _), J.index = 0, J.sibling = null, J;
    }
    function S(J, _, ee) {
      return J.index = ee, n ? (ee = J.alternate, ee !== null ? (ee = ee.index, ee < _ ? (J.flags |= 2, _) : ee) : (J.flags |= 2, _)) : (J.flags |= 1048576, _);
    }
    function I(J) {
      return n && J.alternate === null && (J.flags |= 2), J;
    }
    function F(J, _, ee, be) {
      return _ === null || _.tag !== 6 ? (_ = cd(ee, J.mode, be), _.return = J, _) : (_ = g(_, ee), _.return = J, _);
    }
    function H(J, _, ee, be) {
      var Ze = ee.type;
      return Ze === P ? pe(J, _, ee.props.children, be, ee.key) : _ !== null && (_.elementType === Ze || typeof Ze == "object" && Ze !== null && Ze.$$typeof === Ee && Tg(Ze) === _.type) ? (be = g(_, ee.props), be.ref = lu(J, _, ee), be.return = J, be) : (be = ld(ee.type, ee.key, ee.props, null, J.mode, be), be.ref = lu(J, _, ee), be.return = J, be);
    }
    function ne(J, _, ee, be) {
      return _ === null || _.tag !== 4 || _.stateNode.containerInfo !== ee.containerInfo || _.stateNode.implementation !== ee.implementation ? (_ = ac(ee, J.mode, be), _.return = J, _) : (_ = g(_, ee.children || []), _.return = J, _);
    }
    function pe(J, _, ee, be, Ze) {
      return _ === null || _.tag !== 7 ? (_ = xu(ee, J.mode, be, Ze), _.return = J, _) : (_ = g(_, ee), _.return = J, _);
    }
    function ve(J, _, ee) {
      if (typeof _ == "string" && _ !== "" || typeof _ == "number")
        return _ = cd("" + _, J.mode, ee), _.return = J, _;
      if (typeof _ == "object" && _ !== null) {
        switch (_.$$typeof) {
          case V:
            return ee = ld(_.type, _.key, _.props, null, J.mode, ee), ee.ref = lu(J, null, _), ee.return = J, ee;
          case Q:
            return _ = ac(_, J.mode, ee), _.return = J, _;
          case Ee:
            var be = _._init;
            return ve(J, be(_._payload), ee);
        }
        if (Sr(_) || we(_))
          return _ = xu(_, J.mode, ee, null), _.return = J, _;
        mi(J, _);
      }
      return null;
    }
    function de(J, _, ee, be) {
      var Ze = _ !== null ? _.key : null;
      if (typeof ee == "string" && ee !== "" || typeof ee == "number")
        return Ze !== null ? null : F(J, _, "" + ee, be);
      if (typeof ee == "object" && ee !== null) {
        switch (ee.$$typeof) {
          case V:
            return ee.key === Ze ? H(J, _, ee, be) : null;
          case Q:
            return ee.key === Ze ? ne(J, _, ee, be) : null;
          case Ee:
            return Ze = ee._init, de(
              J,
              _,
              Ze(ee._payload),
              be
            );
        }
        if (Sr(ee) || we(ee))
          return Ze !== null ? null : pe(J, _, ee, be, null);
        mi(J, ee);
      }
      return null;
    }
    function Fe(J, _, ee, be, Ze) {
      if (typeof be == "string" && be !== "" || typeof be == "number")
        return J = J.get(ee) || null, F(_, J, "" + be, Ze);
      if (typeof be == "object" && be !== null) {
        switch (be.$$typeof) {
          case V:
            return J = J.get(be.key === null ? ee : be.key) || null, H(_, J, be, Ze);
          case Q:
            return J = J.get(be.key === null ? ee : be.key) || null, ne(_, J, be, Ze);
          case Ee:
            var at = be._init;
            return Fe(J, _, ee, at(be._payload), Ze);
        }
        if (Sr(be) || we(be))
          return J = J.get(ee) || null, pe(_, J, be, Ze, null);
        mi(_, be);
      }
      return null;
    }
    function ze(J, _, ee, be) {
      for (var Ze = null, at = null, Ge = _, ot = _ = 0, _n = null; Ge !== null && ot < ee.length; ot++) {
        Ge.index > ot ? (_n = Ge, Ge = null) : _n = Ge.sibling;
        var Ft = de(J, Ge, ee[ot], be);
        if (Ft === null) {
          Ge === null && (Ge = _n);
          break;
        }
        n && Ge && Ft.alternate === null && i(J, Ge), _ = S(Ft, _, ot), at === null ? Ze = Ft : at.sibling = Ft, at = Ft, Ge = _n;
      }
      if (ot === ee.length)
        return s(J, Ge), rn && ia(J, ot), Ze;
      if (Ge === null) {
        for (; ot < ee.length; ot++)
          Ge = ve(J, ee[ot], be), Ge !== null && (_ = S(Ge, _, ot), at === null ? Ze = Ge : at.sibling = Ge, at = Ge);
        return rn && ia(J, ot), Ze;
      }
      for (Ge = p(J, Ge); ot < ee.length; ot++)
        _n = Fe(Ge, J, ot, ee[ot], be), _n !== null && (n && _n.alternate !== null && Ge.delete(_n.key === null ? ot : _n.key), _ = S(_n, _, ot), at === null ? Ze = _n : at.sibling = _n, at = _n);
      return n && Ge.forEach(function(Yi) {
        return i(J, Yi);
      }), rn && ia(J, ot), Ze;
    }
    function je(J, _, ee, be) {
      var Ze = we(ee);
      if (typeof Ze != "function")
        throw Error(u(150));
      if (ee = Ze.call(ee), ee == null)
        throw Error(u(151));
      for (var at = Ze = null, Ge = _, ot = _ = 0, _n = null, Ft = ee.next(); Ge !== null && !Ft.done; ot++, Ft = ee.next()) {
        Ge.index > ot ? (_n = Ge, Ge = null) : _n = Ge.sibling;
        var Yi = de(J, Ge, Ft.value, be);
        if (Yi === null) {
          Ge === null && (Ge = _n);
          break;
        }
        n && Ge && Yi.alternate === null && i(J, Ge), _ = S(Yi, _, ot), at === null ? Ze = Yi : at.sibling = Yi, at = Yi, Ge = _n;
      }
      if (Ft.done)
        return s(
          J,
          Ge
        ), rn && ia(J, ot), Ze;
      if (Ge === null) {
        for (; !Ft.done; ot++, Ft = ee.next())
          Ft = ve(J, Ft.value, be), Ft !== null && (_ = S(Ft, _, ot), at === null ? Ze = Ft : at.sibling = Ft, at = Ft);
        return rn && ia(J, ot), Ze;
      }
      for (Ge = p(J, Ge); !Ft.done; ot++, Ft = ee.next())
        Ft = Fe(Ge, J, ot, Ft.value, be), Ft !== null && (n && Ft.alternate !== null && Ge.delete(Ft.key === null ? ot : Ft.key), _ = S(Ft, _, ot), at === null ? Ze = Ft : at.sibling = Ft, at = Ft);
      return n && Ge.forEach(function(RC) {
        return i(J, RC);
      }), rn && ia(J, ot), Ze;
    }
    function Tn(J, _, ee, be) {
      if (typeof ee == "object" && ee !== null && ee.type === P && ee.key === null && (ee = ee.props.children), typeof ee == "object" && ee !== null) {
        switch (ee.$$typeof) {
          case V:
            e: {
              for (var Ze = ee.key, at = _; at !== null; ) {
                if (at.key === Ze) {
                  if (Ze = ee.type, Ze === P) {
                    if (at.tag === 7) {
                      s(J, at.sibling), _ = g(at, ee.props.children), _.return = J, J = _;
                      break e;
                    }
                  } else if (at.elementType === Ze || typeof Ze == "object" && Ze !== null && Ze.$$typeof === Ee && Tg(Ze) === at.type) {
                    s(J, at.sibling), _ = g(at, ee.props), _.ref = lu(J, at, ee), _.return = J, J = _;
                    break e;
                  }
                  s(J, at);
                  break;
                } else
                  i(J, at);
                at = at.sibling;
              }
              ee.type === P ? (_ = xu(ee.props.children, J.mode, be, ee.key), _.return = J, J = _) : (be = ld(ee.type, ee.key, ee.props, null, J.mode, be), be.ref = lu(J, _, ee), be.return = J, J = be);
            }
            return I(J);
          case Q:
            e: {
              for (at = ee.key; _ !== null; ) {
                if (_.key === at)
                  if (_.tag === 4 && _.stateNode.containerInfo === ee.containerInfo && _.stateNode.implementation === ee.implementation) {
                    s(J, _.sibling), _ = g(_, ee.children || []), _.return = J, J = _;
                    break e;
                  } else {
                    s(J, _);
                    break;
                  }
                else
                  i(J, _);
                _ = _.sibling;
              }
              _ = ac(ee, J.mode, be), _.return = J, J = _;
            }
            return I(J);
          case Ee:
            return at = ee._init, Tn(J, _, at(ee._payload), be);
        }
        if (Sr(ee))
          return ze(J, _, ee, be);
        if (we(ee))
          return je(J, _, ee, be);
        mi(J, ee);
      }
      return typeof ee == "string" && ee !== "" || typeof ee == "number" ? (ee = "" + ee, _ !== null && _.tag === 6 ? (s(J, _.sibling), _ = g(_, ee), _.return = J, J = _) : (s(J, _), _ = cd(ee, J.mode, be), _.return = J, J = _), I(J)) : s(J, _);
    }
    return Tn;
  }
  var sl = xf(!0), Rg = xf(!1), _i = St(null), Pn = null, Te = null, Aa = null;
  function ua() {
    Aa = Te = Pn = null;
  }
  function fv(n) {
    var i = _i.current;
    Pt(_i), n._currentValue = i;
  }
  function dv(n, i, s) {
    for (; n !== null; ) {
      var p = n.alternate;
      if ((n.childLanes & i) !== i ? (n.childLanes |= i, p !== null && (p.childLanes |= i)) : p !== null && (p.childLanes & i) !== i && (p.childLanes |= i), n === s)
        break;
      n = n.return;
    }
  }
  function cl(n, i) {
    Pn = n, Aa = Te = null, n = n.dependencies, n !== null && n.firstContext !== null && (n.lanes & i && (Wr = !0), n.firstContext = null);
  }
  function Oa(n) {
    var i = n._currentValue;
    if (Aa !== n)
      if (n = { context: n, memoizedValue: i, next: null }, Te === null) {
        if (Pn === null)
          throw Error(u(308));
        Te = n, Pn.dependencies = { lanes: 0, firstContext: n };
      } else
        Te = Te.next = n;
    return i;
  }
  var su = null;
  function Nn(n) {
    su === null ? su = [n] : su.push(n);
  }
  function Ig(n, i, s, p) {
    var g = i.interleaved;
    return g === null ? (s.next = s, Nn(i)) : (s.next = g.next, g.next = s), i.interleaved = s, ji(n, p);
  }
  function ji(n, i) {
    n.lanes |= i;
    var s = n.alternate;
    for (s !== null && (s.lanes |= i), s = n, n = n.return; n !== null; )
      n.childLanes |= i, s = n.alternate, s !== null && (s.childLanes |= i), s = n, n = n.return;
    return s.tag === 3 ? s.stateNode : null;
  }
  var Eo = !1;
  function wf(n) {
    n.updateQueue = { baseState: n.memoizedState, firstBaseUpdate: null, lastBaseUpdate: null, shared: { pending: null, interleaved: null, lanes: 0 }, effects: null };
  }
  function fl(n, i) {
    n = n.updateQueue, i.updateQueue === n && (i.updateQueue = { baseState: n.baseState, firstBaseUpdate: n.firstBaseUpdate, lastBaseUpdate: n.lastBaseUpdate, shared: n.shared, effects: n.effects });
  }
  function jr(n, i) {
    return { eventTime: n, lane: i, tag: 0, payload: null, callback: null, next: null };
  }
  function xo(n, i, s) {
    var p = n.updateQueue;
    if (p === null)
      return null;
    if (p = p.shared, Rt & 2) {
      var g = p.pending;
      return g === null ? i.next = i : (i.next = g.next, g.next = i), p.pending = i, ji(n, s);
    }
    return g = p.interleaved, g === null ? (i.next = i, Nn(p)) : (i.next = g.next, g.next = i), p.interleaved = i, ji(n, s);
  }
  function Tf(n, i, s) {
    if (i = i.updateQueue, i !== null && (i = i.shared, (s & 4194240) !== 0)) {
      var p = i.lanes;
      p &= n.pendingLanes, s |= p, i.lanes = s, ys(n, s);
    }
  }
  function Ag(n, i) {
    var s = n.updateQueue, p = n.alternate;
    if (p !== null && (p = p.updateQueue, s === p)) {
      var g = null, S = null;
      if (s = s.firstBaseUpdate, s !== null) {
        do {
          var I = { eventTime: s.eventTime, lane: s.lane, tag: s.tag, payload: s.payload, callback: s.callback, next: null };
          S === null ? g = S = I : S = S.next = I, s = s.next;
        } while (s !== null);
        S === null ? g = S = i : S = S.next = i;
      } else
        g = S = i;
      s = { baseState: p.baseState, firstBaseUpdate: g, lastBaseUpdate: S, shared: p.shared, effects: p.effects }, n.updateQueue = s;
      return;
    }
    n = s.lastBaseUpdate, n === null ? s.firstBaseUpdate = i : n.next = i, s.lastBaseUpdate = i;
  }
  function Rf(n, i, s, p) {
    var g = n.updateQueue;
    Eo = !1;
    var S = g.firstBaseUpdate, I = g.lastBaseUpdate, F = g.shared.pending;
    if (F !== null) {
      g.shared.pending = null;
      var H = F, ne = H.next;
      H.next = null, I === null ? S = ne : I.next = ne, I = H;
      var pe = n.alternate;
      pe !== null && (pe = pe.updateQueue, F = pe.lastBaseUpdate, F !== I && (F === null ? pe.firstBaseUpdate = ne : F.next = ne, pe.lastBaseUpdate = H));
    }
    if (S !== null) {
      var ve = g.baseState;
      I = 0, pe = ne = H = null, F = S;
      do {
        var de = F.lane, Fe = F.eventTime;
        if ((p & de) === de) {
          pe !== null && (pe = pe.next = {
            eventTime: Fe,
            lane: 0,
            tag: F.tag,
            payload: F.payload,
            callback: F.callback,
            next: null
          });
          e: {
            var ze = n, je = F;
            switch (de = i, Fe = s, je.tag) {
              case 1:
                if (ze = je.payload, typeof ze == "function") {
                  ve = ze.call(Fe, ve, de);
                  break e;
                }
                ve = ze;
                break e;
              case 3:
                ze.flags = ze.flags & -65537 | 128;
              case 0:
                if (ze = je.payload, de = typeof ze == "function" ? ze.call(Fe, ve, de) : ze, de == null)
                  break e;
                ve = W({}, ve, de);
                break e;
              case 2:
                Eo = !0;
            }
          }
          F.callback !== null && F.lane !== 0 && (n.flags |= 64, de = g.effects, de === null ? g.effects = [F] : de.push(F));
        } else
          Fe = { eventTime: Fe, lane: de, tag: F.tag, payload: F.payload, callback: F.callback, next: null }, pe === null ? (ne = pe = Fe, H = ve) : pe = pe.next = Fe, I |= de;
        if (F = F.next, F === null) {
          if (F = g.shared.pending, F === null)
            break;
          de = F, F = de.next, de.next = null, g.lastBaseUpdate = de, g.shared.pending = null;
        }
      } while (!0);
      if (pe === null && (H = ve), g.baseState = H, g.firstBaseUpdate = ne, g.lastBaseUpdate = pe, i = g.shared.interleaved, i !== null) {
        g = i;
        do
          I |= g.lane, g = g.next;
        while (g !== i);
      } else
        S === null && (g.shared.lanes = 0);
      bu |= I, n.lanes = I, n.memoizedState = ve;
    }
  }
  function Og(n, i, s) {
    if (n = i.effects, i.effects = null, n !== null)
      for (i = 0; i < n.length; i++) {
        var p = n[i], g = p.callback;
        if (g !== null) {
          if (p.callback = null, p = s, typeof g != "function")
            throw Error(u(191, g));
          g.call(p);
        }
      }
  }
  var Gs = {}, ri = St(Gs), dl = St(Gs), _s = St(Gs);
  function cu(n) {
    if (n === Gs)
      throw Error(u(174));
    return n;
  }
  function pv(n, i) {
    switch (Ht(_s, i), Ht(dl, n), Ht(ri, Gs), n = i.nodeType, n) {
      case 9:
      case 11:
        i = (i = i.documentElement) ? i.namespaceURI : tr(null, "");
        break;
      default:
        n = n === 8 ? i.parentNode : i, i = n.namespaceURI || null, n = n.tagName, i = tr(i, n);
    }
    Pt(ri), Ht(ri, i);
  }
  function pl() {
    Pt(ri), Pt(dl), Pt(_s);
  }
  function Dg(n) {
    cu(_s.current);
    var i = cu(ri.current), s = tr(i, n.type);
    i !== s && (Ht(dl, n), Ht(ri, s));
  }
  function vv(n) {
    dl.current === n && (Pt(ri), Pt(dl));
  }
  var ln = St(0);
  function If(n) {
    for (var i = n; i !== null; ) {
      if (i.tag === 13) {
        var s = i.memoizedState;
        if (s !== null && (s = s.dehydrated, s === null || s.data === "$?" || s.data === "$!"))
          return i;
      } else if (i.tag === 19 && i.memoizedProps.revealOrder !== void 0) {
        if (i.flags & 128)
          return i;
      } else if (i.child !== null) {
        i.child.return = i, i = i.child;
        continue;
      }
      if (i === n)
        break;
      for (; i.sibling === null; ) {
        if (i.return === null || i.return === n)
          return null;
        i = i.return;
      }
      i.sibling.return = i.return, i = i.sibling;
    }
    return null;
  }
  var Af = [];
  function hv() {
    for (var n = 0; n < Af.length; n++)
      Af[n]._workInProgressVersionPrimary = null;
    Af.length = 0;
  }
  var Of = Z.ReactCurrentDispatcher, js = Z.ReactCurrentBatchConfig, Xe = 0, $e = null, pt = null, xt = null, la = !1, vl = !1, Us = 0, sC = 0;
  function mr() {
    throw Error(u(321));
  }
  function Ws(n, i) {
    if (i === null)
      return !1;
    for (var s = 0; s < i.length && s < n.length; s++)
      if (!Ea(n[s], i[s]))
        return !1;
    return !0;
  }
  function fe(n, i, s, p, g, S) {
    if (Xe = S, $e = i, i.memoizedState = null, i.updateQueue = null, i.lanes = 0, Of.current = n === null || n.memoizedState === null ? cC : tn, n = s(p, g), vl) {
      S = 0;
      do {
        if (vl = !1, Us = 0, 25 <= S)
          throw Error(u(301));
        S += 1, xt = pt = null, i.updateQueue = null, Of.current = Uf, n = s(p, g);
      } while (vl);
    }
    if (Of.current = gr, i = pt !== null && pt.next !== null, Xe = 0, xt = pt = $e = null, la = !1, i)
      throw Error(u(300));
    return n;
  }
  function kn() {
    var n = Us !== 0;
    return Us = 0, n;
  }
  function Je() {
    var n = { memoizedState: null, baseState: null, baseQueue: null, queue: null, next: null };
    return xt === null ? $e.memoizedState = xt = n : xt = xt.next = n, xt;
  }
  function rr() {
    if (pt === null) {
      var n = $e.alternate;
      n = n !== null ? n.memoizedState : null;
    } else
      n = pt.next;
    var i = xt === null ? $e.memoizedState : xt.next;
    if (i !== null)
      xt = i, pt = n;
    else {
      if (n === null)
        throw Error(u(310));
      pt = n, n = { memoizedState: pt.memoizedState, baseState: pt.baseState, baseQueue: pt.baseQueue, queue: pt.queue, next: null }, xt === null ? $e.memoizedState = xt = n : xt = xt.next = n;
    }
    return xt;
  }
  function sa(n, i) {
    return typeof i == "function" ? i(n) : i;
  }
  function Ui(n) {
    var i = rr(), s = i.queue;
    if (s === null)
      throw Error(u(311));
    s.lastRenderedReducer = n;
    var p = pt, g = p.baseQueue, S = s.pending;
    if (S !== null) {
      if (g !== null) {
        var I = g.next;
        g.next = S.next, S.next = I;
      }
      p.baseQueue = g = S, s.pending = null;
    }
    if (g !== null) {
      S = g.next, p = p.baseState;
      var F = I = null, H = null, ne = S;
      do {
        var pe = ne.lane;
        if ((Xe & pe) === pe)
          H !== null && (H = H.next = { lane: 0, action: ne.action, hasEagerState: ne.hasEagerState, eagerState: ne.eagerState, next: null }), p = ne.hasEagerState ? ne.eagerState : n(p, ne.action);
        else {
          var ve = {
            lane: pe,
            action: ne.action,
            hasEagerState: ne.hasEagerState,
            eagerState: ne.eagerState,
            next: null
          };
          H === null ? (F = H = ve, I = p) : H = H.next = ve, $e.lanes |= pe, bu |= pe;
        }
        ne = ne.next;
      } while (ne !== null && ne !== S);
      H === null ? I = p : H.next = F, Ea(p, i.memoizedState) || (Wr = !0), i.memoizedState = p, i.baseState = I, i.baseQueue = H, s.lastRenderedState = p;
    }
    if (n = s.interleaved, n !== null) {
      g = n;
      do
        S = g.lane, $e.lanes |= S, bu |= S, g = g.next;
      while (g !== n);
    } else
      g === null && (s.lanes = 0);
    return [i.memoizedState, s.dispatch];
  }
  function Da(n) {
    var i = rr(), s = i.queue;
    if (s === null)
      throw Error(u(311));
    s.lastRenderedReducer = n;
    var p = s.dispatch, g = s.pending, S = i.memoizedState;
    if (g !== null) {
      s.pending = null;
      var I = g = g.next;
      do
        S = n(S, I.action), I = I.next;
      while (I !== g);
      Ea(S, i.memoizedState) || (Wr = !0), i.memoizedState = S, i.baseQueue === null && (i.baseState = S), s.lastRenderedState = S;
    }
    return [S, p];
  }
  function hl() {
  }
  function fu(n, i) {
    var s = $e, p = rr(), g = i(), S = !Ea(p.memoizedState, g);
    if (S && (p.memoizedState = g, Wr = !0), p = p.queue, Xs(Nf.bind(null, s, p, n), [n]), p.getSnapshot !== i || S || xt !== null && xt.memoizedState.tag & 1) {
      if (s.flags |= 2048, du(9, Df.bind(null, s, p, g, i), void 0, null), bn === null)
        throw Error(u(349));
      Xe & 30 || ml(s, i, g);
    }
    return g;
  }
  function ml(n, i, s) {
    n.flags |= 16384, n = { getSnapshot: i, value: s }, i = $e.updateQueue, i === null ? (i = { lastEffect: null, stores: null }, $e.updateQueue = i, i.stores = [n]) : (s = i.stores, s === null ? i.stores = [n] : s.push(n));
  }
  function Df(n, i, s, p) {
    i.value = s, i.getSnapshot = p, kf(i) && Mf(n);
  }
  function Nf(n, i, s) {
    return s(function() {
      kf(i) && Mf(n);
    });
  }
  function kf(n) {
    var i = n.getSnapshot;
    n = n.value;
    try {
      var s = i();
      return !Ea(n, s);
    } catch {
      return !0;
    }
  }
  function Mf(n) {
    var i = ji(n, 1);
    i !== null && fn(i, n, 1, -1);
  }
  function Lf(n) {
    var i = Je();
    return typeof n == "function" && (n = n()), i.memoizedState = i.baseState = n, n = { pending: null, interleaved: null, lanes: 0, dispatch: null, lastRenderedReducer: sa, lastRenderedState: n }, i.queue = n, n = n.dispatch = Zs.bind(null, $e, n), [i.memoizedState, n];
  }
  function du(n, i, s, p) {
    return n = { tag: n, create: i, destroy: s, deps: p, next: null }, i = $e.updateQueue, i === null ? (i = { lastEffect: null, stores: null }, $e.updateQueue = i, i.lastEffect = n.next = n) : (s = i.lastEffect, s === null ? i.lastEffect = n.next = n : (p = s.next, s.next = n, n.next = p, i.lastEffect = n)), n;
  }
  function Ff() {
    return rr().memoizedState;
  }
  function gl(n, i, s, p) {
    var g = Je();
    $e.flags |= n, g.memoizedState = du(1 | i, s, void 0, p === void 0 ? null : p);
  }
  function yl(n, i, s, p) {
    var g = rr();
    p = p === void 0 ? null : p;
    var S = void 0;
    if (pt !== null) {
      var I = pt.memoizedState;
      if (S = I.destroy, p !== null && Ws(p, I.deps)) {
        g.memoizedState = du(i, s, S, p);
        return;
      }
    }
    $e.flags |= n, g.memoizedState = du(1 | i, s, S, p);
  }
  function Bf(n, i) {
    return gl(8390656, 8, n, i);
  }
  function Xs(n, i) {
    return yl(2048, 8, n, i);
  }
  function Vf(n, i) {
    return yl(4, 2, n, i);
  }
  function Pf(n, i) {
    return yl(4, 4, n, i);
  }
  function Hf(n, i) {
    if (typeof i == "function")
      return n = n(), i(n), function() {
        i(null);
      };
    if (i != null)
      return n = n(), i.current = n, function() {
        i.current = null;
      };
  }
  function zf(n, i, s) {
    return s = s != null ? s.concat([n]) : null, yl(4, 4, Hf.bind(null, i, n), s);
  }
  function bl() {
  }
  function pu(n, i) {
    var s = rr();
    i = i === void 0 ? null : i;
    var p = s.memoizedState;
    return p !== null && i !== null && Ws(i, p[1]) ? p[0] : (s.memoizedState = [n, i], n);
  }
  function Gf(n, i) {
    var s = rr();
    i = i === void 0 ? null : i;
    var p = s.memoizedState;
    return p !== null && i !== null && Ws(i, p[1]) ? p[0] : (n = n(), s.memoizedState = [n, i], n);
  }
  function _f(n, i, s) {
    return Xe & 21 ? (Ea(s, i) || (s = Jc(), $e.lanes |= s, bu |= s, n.baseState = !0), i) : (n.baseState && (n.baseState = !1, Wr = !0), n.memoizedState = s);
  }
  function mv(n, i) {
    var s = jt;
    jt = s !== 0 && 4 > s ? s : 4, n(!0);
    var p = js.transition;
    js.transition = {};
    try {
      n(!1), i();
    } finally {
      jt = s, js.transition = p;
    }
  }
  function jf() {
    return rr().memoizedState;
  }
  function Ng(n, i, s) {
    var p = Zi(n);
    if (s = { lane: p, action: s, hasEagerState: !1, eagerState: null, next: null }, gv(n))
      Cl(i, s);
    else if (s = Ig(n, i, s, p), s !== null) {
      var g = Qn();
      fn(s, n, p, g), wo(s, i, p);
    }
  }
  function Zs(n, i, s) {
    var p = Zi(n), g = { lane: p, action: s, hasEagerState: !1, eagerState: null, next: null };
    if (gv(n))
      Cl(i, g);
    else {
      var S = n.alternate;
      if (n.lanes === 0 && (S === null || S.lanes === 0) && (S = i.lastRenderedReducer, S !== null))
        try {
          var I = i.lastRenderedState, F = S(I, s);
          if (g.hasEagerState = !0, g.eagerState = F, Ea(F, I)) {
            var H = i.interleaved;
            H === null ? (g.next = g, Nn(i)) : (g.next = H.next, H.next = g), i.interleaved = g;
            return;
          }
        } catch {
        } finally {
        }
      s = Ig(n, i, g, p), s !== null && (g = Qn(), fn(s, n, p, g), wo(s, i, p));
    }
  }
  function gv(n) {
    var i = n.alternate;
    return n === $e || i !== null && i === $e;
  }
  function Cl(n, i) {
    vl = la = !0;
    var s = n.pending;
    s === null ? i.next = i : (i.next = s.next, s.next = i), n.pending = i;
  }
  function wo(n, i, s) {
    if (s & 4194240) {
      var p = i.lanes;
      p &= n.pendingLanes, s |= p, i.lanes = s, ys(n, s);
    }
  }
  var gr = { readContext: Oa, useCallback: mr, useContext: mr, useEffect: mr, useImperativeHandle: mr, useInsertionEffect: mr, useLayoutEffect: mr, useMemo: mr, useReducer: mr, useRef: mr, useState: mr, useDebugValue: mr, useDeferredValue: mr, useTransition: mr, useMutableSource: mr, useSyncExternalStore: mr, useId: mr, unstable_isNewReconciler: !1 }, cC = { readContext: Oa, useCallback: function(n, i) {
    return Je().memoizedState = [n, i === void 0 ? null : i], n;
  }, useContext: Oa, useEffect: Bf, useImperativeHandle: function(n, i, s) {
    return s = s != null ? s.concat([n]) : null, gl(
      4194308,
      4,
      Hf.bind(null, i, n),
      s
    );
  }, useLayoutEffect: function(n, i) {
    return gl(4194308, 4, n, i);
  }, useInsertionEffect: function(n, i) {
    return gl(4, 2, n, i);
  }, useMemo: function(n, i) {
    var s = Je();
    return i = i === void 0 ? null : i, n = n(), s.memoizedState = [n, i], n;
  }, useReducer: function(n, i, s) {
    var p = Je();
    return i = s !== void 0 ? s(i) : i, p.memoizedState = p.baseState = i, n = { pending: null, interleaved: null, lanes: 0, dispatch: null, lastRenderedReducer: n, lastRenderedState: i }, p.queue = n, n = n.dispatch = Ng.bind(null, $e, n), [p.memoizedState, n];
  }, useRef: function(n) {
    var i = Je();
    return n = { current: n }, i.memoizedState = n;
  }, useState: Lf, useDebugValue: bl, useDeferredValue: function(n) {
    return Je().memoizedState = n;
  }, useTransition: function() {
    var n = Lf(!1), i = n[0];
    return n = mv.bind(null, n[1]), Je().memoizedState = n, [i, n];
  }, useMutableSource: function() {
  }, useSyncExternalStore: function(n, i, s) {
    var p = $e, g = Je();
    if (rn) {
      if (s === void 0)
        throw Error(u(407));
      s = s();
    } else {
      if (s = i(), bn === null)
        throw Error(u(349));
      Xe & 30 || ml(p, i, s);
    }
    g.memoizedState = s;
    var S = { value: s, getSnapshot: i };
    return g.queue = S, Bf(Nf.bind(
      null,
      p,
      S,
      n
    ), [n]), p.flags |= 2048, du(9, Df.bind(null, p, S, s, i), void 0, null), s;
  }, useId: function() {
    var n = Je(), i = bn.identifierPrefix;
    if (rn) {
      var s = Gi, p = nr;
      s = (p & ~(1 << 32 - ba(p) - 1)).toString(32) + s, i = ":" + i + "R" + s, s = Us++, 0 < s && (i += "H" + s.toString(32)), i += ":";
    } else
      s = sC++, i = ":" + i + "r" + s.toString(32) + ":";
    return n.memoizedState = i;
  }, unstable_isNewReconciler: !1 }, tn = {
    readContext: Oa,
    useCallback: pu,
    useContext: Oa,
    useEffect: Xs,
    useImperativeHandle: zf,
    useInsertionEffect: Vf,
    useLayoutEffect: Pf,
    useMemo: Gf,
    useReducer: Ui,
    useRef: Ff,
    useState: function() {
      return Ui(sa);
    },
    useDebugValue: bl,
    useDeferredValue: function(n) {
      var i = rr();
      return _f(i, pt.memoizedState, n);
    },
    useTransition: function() {
      var n = Ui(sa)[0], i = rr().memoizedState;
      return [n, i];
    },
    useMutableSource: hl,
    useSyncExternalStore: fu,
    useId: jf,
    unstable_isNewReconciler: !1
  }, Uf = { readContext: Oa, useCallback: pu, useContext: Oa, useEffect: Xs, useImperativeHandle: zf, useInsertionEffect: Vf, useLayoutEffect: Pf, useMemo: Gf, useReducer: Da, useRef: Ff, useState: function() {
    return Da(sa);
  }, useDebugValue: bl, useDeferredValue: function(n) {
    var i = rr();
    return pt === null ? i.memoizedState = n : _f(i, pt.memoizedState, n);
  }, useTransition: function() {
    var n = Da(sa)[0], i = rr().memoizedState;
    return [n, i];
  }, useMutableSource: hl, useSyncExternalStore: fu, useId: jf, unstable_isNewReconciler: !1 };
  function Ur(n, i) {
    if (n && n.defaultProps) {
      i = W({}, i), n = n.defaultProps;
      for (var s in n)
        i[s] === void 0 && (i[s] = n[s]);
      return i;
    }
    return i;
  }
  function vu(n, i, s, p) {
    i = n.memoizedState, s = s(p, i), s = s == null ? i : W({}, i, s), n.memoizedState = s, n.lanes === 0 && (n.updateQueue.baseState = s);
  }
  var hu = { isMounted: function(n) {
    return (n = n._reactInternals) ? di(n) === n : !1;
  }, enqueueSetState: function(n, i, s) {
    n = n._reactInternals;
    var p = Qn(), g = Zi(n), S = jr(p, g);
    S.payload = i, s != null && (S.callback = s), i = xo(n, S, g), i !== null && (fn(i, n, g, p), Tf(i, n, g));
  }, enqueueReplaceState: function(n, i, s) {
    n = n._reactInternals;
    var p = Qn(), g = Zi(n), S = jr(p, g);
    S.tag = 1, S.payload = i, s != null && (S.callback = s), i = xo(n, S, g), i !== null && (fn(i, n, g, p), Tf(i, n, g));
  }, enqueueForceUpdate: function(n, i) {
    n = n._reactInternals;
    var s = Qn(), p = Zi(n), g = jr(s, p);
    g.tag = 2, i != null && (g.callback = i), i = xo(n, g, p), i !== null && (fn(i, n, p, s), Tf(i, n, p));
  } };
  function kg(n, i, s, p, g, S, I) {
    return n = n.stateNode, typeof n.shouldComponentUpdate == "function" ? n.shouldComponentUpdate(p, S, I) : i.prototype && i.prototype.isPureReactComponent ? !Os(s, p) || !Os(g, S) : !0;
  }
  function Mg(n, i, s) {
    var p = !1, g = hi, S = i.contextType;
    return typeof S == "object" && S !== null ? S = Oa(S) : (g = un(i) ? Pr : dt.current, p = i.contextTypes, S = (p = p != null) ? wa(n, g) : hi), i = new i(s, S), n.memoizedState = i.state !== null && i.state !== void 0 ? i.state : null, i.updater = hu, n.stateNode = i, i._reactInternals = n, p && (n = n.stateNode, n.__reactInternalMemoizedUnmaskedChildContext = g, n.__reactInternalMemoizedMaskedChildContext = S), i;
  }
  function Lg(n, i, s, p) {
    n = i.state, typeof i.componentWillReceiveProps == "function" && i.componentWillReceiveProps(s, p), typeof i.UNSAFE_componentWillReceiveProps == "function" && i.UNSAFE_componentWillReceiveProps(s, p), i.state !== n && hu.enqueueReplaceState(i, i.state, null);
  }
  function yv(n, i, s, p) {
    var g = n.stateNode;
    g.props = s, g.state = n.memoizedState, g.refs = {}, wf(n);
    var S = i.contextType;
    typeof S == "object" && S !== null ? g.context = Oa(S) : (S = un(i) ? Pr : dt.current, g.context = wa(n, S)), g.state = n.memoizedState, S = i.getDerivedStateFromProps, typeof S == "function" && (vu(n, i, S, s), g.state = n.memoizedState), typeof i.getDerivedStateFromProps == "function" || typeof g.getSnapshotBeforeUpdate == "function" || typeof g.UNSAFE_componentWillMount != "function" && typeof g.componentWillMount != "function" || (i = g.state, typeof g.componentWillMount == "function" && g.componentWillMount(), typeof g.UNSAFE_componentWillMount == "function" && g.UNSAFE_componentWillMount(), i !== g.state && hu.enqueueReplaceState(g, g.state, null), Rf(n, s, g, p), g.state = n.memoizedState), typeof g.componentDidMount == "function" && (n.flags |= 4194308);
  }
  function To(n, i) {
    try {
      var s = "", p = i;
      do
        s += De(p), p = p.return;
      while (p);
      var g = s;
    } catch (S) {
      g = `
Error generating stack: ` + S.message + `
` + S.stack;
    }
    return { value: n, source: i, stack: g, digest: null };
  }
  function bv(n, i, s) {
    return { value: n, source: null, stack: s ?? null, digest: i ?? null };
  }
  function Ys(n, i) {
    try {
      console.error(i.value);
    } catch (s) {
      setTimeout(function() {
        throw s;
      });
    }
  }
  var Fg = typeof WeakMap == "function" ? WeakMap : Map;
  function Bg(n, i, s) {
    s = jr(-1, s), s.tag = 3, s.payload = { element: null };
    var p = i.value;
    return s.callback = function() {
      nd || (nd = !0, Av = p), Ys(n, i);
    }, s;
  }
  function Vg(n, i, s) {
    s = jr(-1, s), s.tag = 3;
    var p = n.type.getDerivedStateFromError;
    if (typeof p == "function") {
      var g = i.value;
      s.payload = function() {
        return p(g);
      }, s.callback = function() {
        Ys(n, i);
      };
    }
    var S = n.stateNode;
    return S !== null && typeof S.componentDidCatch == "function" && (s.callback = function() {
      Ys(n, i), typeof p != "function" && (Ma === null ? Ma = /* @__PURE__ */ new Set([this]) : Ma.add(this));
      var I = i.stack;
      this.componentDidCatch(i.value, { componentStack: I !== null ? I : "" });
    }), s;
  }
  function $s(n, i, s) {
    var p = n.pingCache;
    if (p === null) {
      p = n.pingCache = new Fg();
      var g = /* @__PURE__ */ new Set();
      p.set(i, g);
    } else
      g = p.get(i), g === void 0 && (g = /* @__PURE__ */ new Set(), p.set(i, g));
    g.has(s) || (g.add(s), n = CC.bind(null, n, i, s), i.then(n, n));
  }
  function Pg(n) {
    do {
      var i;
      if ((i = n.tag === 13) && (i = n.memoizedState, i = i !== null ? i.dehydrated !== null : !0), i)
        return n;
      n = n.return;
    } while (n !== null);
    return null;
  }
  function Cv(n, i, s, p, g) {
    return n.mode & 1 ? (n.flags |= 65536, n.lanes = g, n) : (n === i ? n.flags |= 65536 : (n.flags |= 128, s.flags |= 131072, s.flags &= -52805, s.tag === 1 && (s.alternate === null ? s.tag = 17 : (i = jr(-1, 1), i.tag = 2, xo(s, i, 1))), s.lanes |= 1), n);
  }
  var Hg = Z.ReactCurrentOwner, Wr = !1;
  function xn(n, i, s, p) {
    i.child = n === null ? Rg(i, null, s, p) : sl(i, n.child, s, p);
  }
  function Sl(n, i, s, p, g) {
    s = s.render;
    var S = i.ref;
    return cl(i, g), p = fe(n, i, s, p, S, g), s = kn(), n !== null && !Wr ? (i.updateQueue = n.updateQueue, i.flags &= -2053, n.lanes &= ~g, wn(n, i, g)) : (rn && s && yf(i), i.flags |= 1, xn(n, i, p, g), i.child);
  }
  function Ro(n, i, s, p, g) {
    if (n === null) {
      var S = s.type;
      return typeof S == "function" && !Mv(S) && S.defaultProps === void 0 && s.compare === null && s.defaultProps === void 0 ? (i.tag = 15, i.type = S, Wf(n, i, S, p, g)) : (n = ld(s.type, null, p, i, i.mode, g), n.ref = i.ref, n.return = i, i.child = n);
    }
    if (S = n.child, !(n.lanes & g)) {
      var I = S.memoizedProps;
      if (s = s.compare, s = s !== null ? s : Os, s(I, p) && n.ref === i.ref)
        return wn(n, i, g);
    }
    return i.flags |= 1, n = Oo(S, p), n.ref = i.ref, n.return = i, i.child = n;
  }
  function Wf(n, i, s, p, g) {
    if (n !== null) {
      var S = n.memoizedProps;
      if (Os(S, p) && n.ref === i.ref)
        if (Wr = !1, i.pendingProps = p = S, (n.lanes & g) !== 0)
          n.flags & 131072 && (Wr = !0);
        else
          return i.lanes = n.lanes, wn(n, i, g);
    }
    return Ct(n, i, s, p, g);
  }
  function Xr(n, i, s) {
    var p = i.pendingProps, g = p.children, S = n !== null ? n.memoizedState : null;
    if (p.mode === "hidden")
      if (!(i.mode & 1))
        i.memoizedState = { baseLanes: 0, cachePool: null, transitions: null }, Ht(Nl, Zr), Zr |= s;
      else {
        if (!(s & 1073741824))
          return n = S !== null ? S.baseLanes | s : s, i.lanes = i.childLanes = 1073741824, i.memoizedState = { baseLanes: n, cachePool: null, transitions: null }, i.updateQueue = null, Ht(Nl, Zr), Zr |= n, null;
        i.memoizedState = { baseLanes: 0, cachePool: null, transitions: null }, p = S !== null ? S.baseLanes : s, Ht(Nl, Zr), Zr |= p;
      }
    else
      S !== null ? (p = S.baseLanes | s, i.memoizedState = null) : p = s, Ht(Nl, Zr), Zr |= p;
    return xn(n, i, g, s), i.child;
  }
  function mu(n, i) {
    var s = i.ref;
    (n === null && s !== null || n !== null && n.ref !== s) && (i.flags |= 512, i.flags |= 2097152);
  }
  function Ct(n, i, s, p, g) {
    var S = un(s) ? Pr : dt.current;
    return S = wa(i, S), cl(i, g), s = fe(n, i, s, p, S, g), p = kn(), n !== null && !Wr ? (i.updateQueue = n.updateQueue, i.flags &= -2053, n.lanes &= ~g, wn(n, i, g)) : (rn && p && yf(i), i.flags |= 1, xn(n, i, s, g), i.child);
  }
  function Qs(n, i, s, p, g) {
    if (un(s)) {
      var S = !0;
      gf(i);
    } else
      S = !1;
    if (cl(i, g), i.stateNode === null)
      Ks(n, i), Mg(i, s, p), yv(i, s, p, g), p = !0;
    else if (n === null) {
      var I = i.stateNode, F = i.memoizedProps;
      I.props = F;
      var H = I.context, ne = s.contextType;
      typeof ne == "object" && ne !== null ? ne = Oa(ne) : (ne = un(s) ? Pr : dt.current, ne = wa(i, ne));
      var pe = s.getDerivedStateFromProps, ve = typeof pe == "function" || typeof I.getSnapshotBeforeUpdate == "function";
      ve || typeof I.UNSAFE_componentWillReceiveProps != "function" && typeof I.componentWillReceiveProps != "function" || (F !== p || H !== ne) && Lg(i, I, p, ne), Eo = !1;
      var de = i.memoizedState;
      I.state = de, Rf(i, p, I, g), H = i.memoizedState, F !== p || de !== H || gn.current || Eo ? (typeof pe == "function" && (vu(i, s, pe, p), H = i.memoizedState), (F = Eo || kg(i, s, F, p, de, H, ne)) ? (ve || typeof I.UNSAFE_componentWillMount != "function" && typeof I.componentWillMount != "function" || (typeof I.componentWillMount == "function" && I.componentWillMount(), typeof I.UNSAFE_componentWillMount == "function" && I.UNSAFE_componentWillMount()), typeof I.componentDidMount == "function" && (i.flags |= 4194308)) : (typeof I.componentDidMount == "function" && (i.flags |= 4194308), i.memoizedProps = p, i.memoizedState = H), I.props = p, I.state = H, I.context = ne, p = F) : (typeof I.componentDidMount == "function" && (i.flags |= 4194308), p = !1);
    } else {
      I = i.stateNode, fl(n, i), F = i.memoizedProps, ne = i.type === i.elementType ? F : Ur(i.type, F), I.props = ne, ve = i.pendingProps, de = I.context, H = s.contextType, typeof H == "object" && H !== null ? H = Oa(H) : (H = un(s) ? Pr : dt.current, H = wa(i, H));
      var Fe = s.getDerivedStateFromProps;
      (pe = typeof Fe == "function" || typeof I.getSnapshotBeforeUpdate == "function") || typeof I.UNSAFE_componentWillReceiveProps != "function" && typeof I.componentWillReceiveProps != "function" || (F !== ve || de !== H) && Lg(i, I, p, H), Eo = !1, de = i.memoizedState, I.state = de, Rf(i, p, I, g);
      var ze = i.memoizedState;
      F !== ve || de !== ze || gn.current || Eo ? (typeof Fe == "function" && (vu(i, s, Fe, p), ze = i.memoizedState), (ne = Eo || kg(i, s, ne, p, de, ze, H) || !1) ? (pe || typeof I.UNSAFE_componentWillUpdate != "function" && typeof I.componentWillUpdate != "function" || (typeof I.componentWillUpdate == "function" && I.componentWillUpdate(p, ze, H), typeof I.UNSAFE_componentWillUpdate == "function" && I.UNSAFE_componentWillUpdate(p, ze, H)), typeof I.componentDidUpdate == "function" && (i.flags |= 4), typeof I.getSnapshotBeforeUpdate == "function" && (i.flags |= 1024)) : (typeof I.componentDidUpdate != "function" || F === n.memoizedProps && de === n.memoizedState || (i.flags |= 4), typeof I.getSnapshotBeforeUpdate != "function" || F === n.memoizedProps && de === n.memoizedState || (i.flags |= 1024), i.memoizedProps = p, i.memoizedState = ze), I.props = p, I.state = ze, I.context = H, p = ne) : (typeof I.componentDidUpdate != "function" || F === n.memoizedProps && de === n.memoizedState || (i.flags |= 4), typeof I.getSnapshotBeforeUpdate != "function" || F === n.memoizedProps && de === n.memoizedState || (i.flags |= 1024), p = !1);
    }
    return Xf(n, i, s, p, S, g);
  }
  function Xf(n, i, s, p, g, S) {
    mu(n, i);
    var I = (i.flags & 128) !== 0;
    if (!p && !I)
      return g && Sg(i, s, !1), wn(n, i, S);
    p = i.stateNode, Hg.current = i;
    var F = I && typeof s.getDerivedStateFromError != "function" ? null : p.render();
    return i.flags |= 1, n !== null && I ? (i.child = sl(i, n.child, null, S), i.child = sl(i, null, F, S)) : xn(n, i, F, S), i.memoizedState = p.state, g && Sg(i, s, !0), i.child;
  }
  function fC(n) {
    var i = n.stateNode;
    i.pendingContext ? Co(n, i.pendingContext, i.pendingContext !== i.context) : i.context && Co(n, i.context, !1), pv(n, i.containerInfo);
  }
  function zg(n, i, s, p, g) {
    return cn(), cv(g), i.flags |= 256, xn(n, i, s, p), i.child;
  }
  var Js = { dehydrated: null, treeContext: null, retryLane: 0 };
  function gu(n) {
    return { baseLanes: n, cachePool: null, transitions: null };
  }
  function Gg(n, i, s) {
    var p = i.pendingProps, g = ln.current, S = !1, I = (i.flags & 128) !== 0, F;
    if ((F = I) || (F = n !== null && n.memoizedState === null ? !1 : (g & 2) !== 0), F ? (S = !0, i.flags &= -129) : (n === null || n.memoizedState !== null) && (g |= 1), Ht(ln, g & 1), n === null)
      return Cf(i), n = i.memoizedState, n !== null && (n = n.dehydrated, n !== null) ? (i.mode & 1 ? n.data === "$!" ? i.lanes = 8 : i.lanes = 1073741824 : i.lanes = 1, null) : (I = p.children, n = p.fallback, S ? (p = i.mode, S = i.child, I = { mode: "hidden", children: I }, !(p & 1) && S !== null ? (S.childLanes = 0, S.pendingProps = I) : S = sd(I, p, 0, null), n = xu(n, p, s, null), S.return = i, n.return = i, S.sibling = n, i.child = S, i.child.memoizedState = gu(s), i.memoizedState = Js, n) : Zf(i, I));
    if (g = n.memoizedState, g !== null && (F = g.dehydrated, F !== null))
      return Sv(n, i, I, p, F, g, s);
    if (S) {
      S = p.fallback, I = i.mode, g = n.child, F = g.sibling;
      var H = { mode: "hidden", children: p.children };
      return !(I & 1) && i.child !== g ? (p = i.child, p.childLanes = 0, p.pendingProps = H, i.deletions = null) : (p = Oo(g, H), p.subtreeFlags = g.subtreeFlags & 14680064), F !== null ? S = Oo(F, S) : (S = xu(S, I, s, null), S.flags |= 2), S.return = i, p.return = i, p.sibling = S, i.child = p, p = S, S = i.child, I = n.child.memoizedState, I = I === null ? gu(s) : { baseLanes: I.baseLanes | s, cachePool: null, transitions: I.transitions }, S.memoizedState = I, S.childLanes = n.childLanes & ~s, i.memoizedState = Js, p;
    }
    return S = n.child, n = S.sibling, p = Oo(S, { mode: "visible", children: p.children }), !(i.mode & 1) && (p.lanes = s), p.return = i, p.sibling = null, n !== null && (s = i.deletions, s === null ? (i.deletions = [n], i.flags |= 16) : s.push(n)), i.child = p, i.memoizedState = null, p;
  }
  function Zf(n, i) {
    return i = sd({ mode: "visible", children: i }, n.mode, 0, null), i.return = n, n.child = i;
  }
  function Yf(n, i, s, p) {
    return p !== null && cv(p), sl(i, n.child, null, s), n = Zf(i, i.pendingProps.children), n.flags |= 2, i.memoizedState = null, n;
  }
  function Sv(n, i, s, p, g, S, I) {
    if (s)
      return i.flags & 256 ? (i.flags &= -257, p = bv(Error(u(422))), Yf(n, i, I, p)) : i.memoizedState !== null ? (i.child = n.child, i.flags |= 128, null) : (S = p.fallback, g = i.mode, p = sd({ mode: "visible", children: p.children }, g, 0, null), S = xu(S, g, I, null), S.flags |= 2, p.return = i, S.return = i, p.sibling = S, i.child = p, i.mode & 1 && sl(i, n.child, null, I), i.child.memoizedState = gu(I), i.memoizedState = Js, S);
    if (!(i.mode & 1))
      return Yf(n, i, I, null);
    if (g.data === "$!") {
      if (p = g.nextSibling && g.nextSibling.dataset, p)
        var F = p.dgst;
      return p = F, S = Error(u(419)), p = bv(S, p, void 0), Yf(n, i, I, p);
    }
    if (F = (I & n.childLanes) !== 0, Wr || F) {
      if (p = bn, p !== null) {
        switch (I & -I) {
          case 4:
            g = 2;
            break;
          case 16:
            g = 8;
            break;
          case 64:
          case 128:
          case 256:
          case 512:
          case 1024:
          case 2048:
          case 4096:
          case 8192:
          case 16384:
          case 32768:
          case 65536:
          case 131072:
          case 262144:
          case 524288:
          case 1048576:
          case 2097152:
          case 4194304:
          case 8388608:
          case 16777216:
          case 33554432:
          case 67108864:
            g = 32;
            break;
          case 536870912:
            g = 268435456;
            break;
          default:
            g = 0;
        }
        g = g & (p.suspendedLanes | I) ? 0 : g, g !== 0 && g !== S.retryLane && (S.retryLane = g, ji(n, g), fn(p, n, g, -1));
      }
      return rc(), p = bv(Error(u(421))), Yf(n, i, I, p);
    }
    return g.data === "$?" ? (i.flags |= 128, i.child = n.child, i = kv.bind(null, n), g._reactRetry = i, null) : (n = S.treeContext, _r = ni(g.nextSibling), oa = i, rn = !0, Ia = null, n !== null && (Gr[hr++] = nr, Gr[hr++] = Gi, Gr[hr++] = Ra, nr = n.id, Gi = n.overflow, Ra = i), i = Zf(i, p.children), i.flags |= 4096, i);
  }
  function _g(n, i, s) {
    n.lanes |= i;
    var p = n.alternate;
    p !== null && (p.lanes |= i), dv(n.return, i, s);
  }
  function $f(n, i, s, p, g) {
    var S = n.memoizedState;
    S === null ? n.memoizedState = { isBackwards: i, rendering: null, renderingStartTime: 0, last: p, tail: s, tailMode: g } : (S.isBackwards = i, S.rendering = null, S.renderingStartTime = 0, S.last = p, S.tail = s, S.tailMode = g);
  }
  function Ev(n, i, s) {
    var p = i.pendingProps, g = p.revealOrder, S = p.tail;
    if (xn(n, i, p.children, s), p = ln.current, p & 2)
      p = p & 1 | 2, i.flags |= 128;
    else {
      if (n !== null && n.flags & 128)
        e:
          for (n = i.child; n !== null; ) {
            if (n.tag === 13)
              n.memoizedState !== null && _g(n, s, i);
            else if (n.tag === 19)
              _g(n, s, i);
            else if (n.child !== null) {
              n.child.return = n, n = n.child;
              continue;
            }
            if (n === i)
              break e;
            for (; n.sibling === null; ) {
              if (n.return === null || n.return === i)
                break e;
              n = n.return;
            }
            n.sibling.return = n.return, n = n.sibling;
          }
      p &= 1;
    }
    if (Ht(ln, p), !(i.mode & 1))
      i.memoizedState = null;
    else
      switch (g) {
        case "forwards":
          for (s = i.child, g = null; s !== null; )
            n = s.alternate, n !== null && If(n) === null && (g = s), s = s.sibling;
          s = g, s === null ? (g = i.child, i.child = null) : (g = s.sibling, s.sibling = null), $f(i, !1, g, s, S);
          break;
        case "backwards":
          for (s = null, g = i.child, i.child = null; g !== null; ) {
            if (n = g.alternate, n !== null && If(n) === null) {
              i.child = g;
              break;
            }
            n = g.sibling, g.sibling = s, s = g, g = n;
          }
          $f(i, !0, s, null, S);
          break;
        case "together":
          $f(i, !1, null, null, void 0);
          break;
        default:
          i.memoizedState = null;
      }
    return i.child;
  }
  function Ks(n, i) {
    !(i.mode & 1) && n !== null && (n.alternate = null, i.alternate = null, i.flags |= 2);
  }
  function wn(n, i, s) {
    if (n !== null && (i.dependencies = n.dependencies), bu |= i.lanes, !(s & i.childLanes))
      return null;
    if (n !== null && i.child !== n.child)
      throw Error(u(153));
    if (i.child !== null) {
      for (n = i.child, s = Oo(n, n.pendingProps), i.child = s, s.return = i; n.sibling !== null; )
        n = n.sibling, s = s.sibling = Oo(n, n.pendingProps), s.return = i;
      s.sibling = null;
    }
    return i.child;
  }
  function Wi(n, i, s) {
    switch (i.tag) {
      case 3:
        fC(i), cn();
        break;
      case 5:
        Dg(i);
        break;
      case 1:
        un(i.type) && gf(i);
        break;
      case 4:
        pv(i, i.stateNode.containerInfo);
        break;
      case 10:
        var p = i.type._context, g = i.memoizedProps.value;
        Ht(_i, p._currentValue), p._currentValue = g;
        break;
      case 13:
        if (p = i.memoizedState, p !== null)
          return p.dehydrated !== null ? (Ht(ln, ln.current & 1), i.flags |= 128, null) : s & i.child.childLanes ? Gg(n, i, s) : (Ht(ln, ln.current & 1), n = wn(n, i, s), n !== null ? n.sibling : null);
        Ht(ln, ln.current & 1);
        break;
      case 19:
        if (p = (s & i.childLanes) !== 0, n.flags & 128) {
          if (p)
            return Ev(n, i, s);
          i.flags |= 128;
        }
        if (g = i.memoizedState, g !== null && (g.rendering = null, g.tail = null, g.lastEffect = null), Ht(ln, ln.current), p)
          break;
        return null;
      case 22:
      case 23:
        return i.lanes = 0, Xr(n, i, s);
    }
    return wn(n, i, s);
  }
  var gi, El, xl, Na;
  gi = function(n, i) {
    for (var s = i.child; s !== null; ) {
      if (s.tag === 5 || s.tag === 6)
        n.appendChild(s.stateNode);
      else if (s.tag !== 4 && s.child !== null) {
        s.child.return = s, s = s.child;
        continue;
      }
      if (s === i)
        break;
      for (; s.sibling === null; ) {
        if (s.return === null || s.return === i)
          return;
        s = s.return;
      }
      s.sibling.return = s.return, s = s.sibling;
    }
  }, El = function() {
  }, xl = function(n, i, s, p) {
    var g = n.memoizedProps;
    if (g !== p) {
      n = i.stateNode, cu(ri.current);
      var S = null;
      switch (s) {
        case "input":
          g = _t(n, g), p = _t(n, p), S = [];
          break;
        case "select":
          g = W({}, g, { value: void 0 }), p = W({}, p, { value: void 0 }), S = [];
          break;
        case "textarea":
          g = ci(n, g), p = ci(n, p), S = [];
          break;
        default:
          typeof g.onClick != "function" && typeof p.onClick == "function" && (n.onclick = mf);
      }
      pr(s, p);
      var I;
      s = null;
      for (ne in g)
        if (!p.hasOwnProperty(ne) && g.hasOwnProperty(ne) && g[ne] != null)
          if (ne === "style") {
            var F = g[ne];
            for (I in F)
              F.hasOwnProperty(I) && (s || (s = {}), s[I] = "");
          } else
            ne !== "dangerouslySetInnerHTML" && ne !== "children" && ne !== "suppressContentEditableWarning" && ne !== "suppressHydrationWarning" && ne !== "autoFocus" && (c.hasOwnProperty(ne) ? S || (S = []) : (S = S || []).push(ne, null));
      for (ne in p) {
        var H = p[ne];
        if (F = g != null ? g[ne] : void 0, p.hasOwnProperty(ne) && H !== F && (H != null || F != null))
          if (ne === "style")
            if (F) {
              for (I in F)
                !F.hasOwnProperty(I) || H && H.hasOwnProperty(I) || (s || (s = {}), s[I] = "");
              for (I in H)
                H.hasOwnProperty(I) && F[I] !== H[I] && (s || (s = {}), s[I] = H[I]);
            } else
              s || (S || (S = []), S.push(
                ne,
                s
              )), s = H;
          else
            ne === "dangerouslySetInnerHTML" ? (H = H ? H.__html : void 0, F = F ? F.__html : void 0, H != null && F !== H && (S = S || []).push(ne, H)) : ne === "children" ? typeof H != "string" && typeof H != "number" || (S = S || []).push(ne, "" + H) : ne !== "suppressContentEditableWarning" && ne !== "suppressHydrationWarning" && (c.hasOwnProperty(ne) ? (H != null && ne === "onScroll" && qt("scroll", n), S || F === H || (S = [])) : (S = S || []).push(ne, H));
      }
      s && (S = S || []).push("style", s);
      var ne = S;
      (i.updateQueue = ne) && (i.flags |= 4);
    }
  }, Na = function(n, i, s, p) {
    s !== p && (i.flags |= 4);
  };
  function yn(n, i) {
    if (!rn)
      switch (n.tailMode) {
        case "hidden":
          i = n.tail;
          for (var s = null; i !== null; )
            i.alternate !== null && (s = i), i = i.sibling;
          s === null ? n.tail = null : s.sibling = null;
          break;
        case "collapsed":
          s = n.tail;
          for (var p = null; s !== null; )
            s.alternate !== null && (p = s), s = s.sibling;
          p === null ? i || n.tail === null ? n.tail = null : n.tail.sibling = null : p.sibling = null;
      }
  }
  function yr(n) {
    var i = n.alternate !== null && n.alternate.child === n.child, s = 0, p = 0;
    if (i)
      for (var g = n.child; g !== null; )
        s |= g.lanes | g.childLanes, p |= g.subtreeFlags & 14680064, p |= g.flags & 14680064, g.return = n, g = g.sibling;
    else
      for (g = n.child; g !== null; )
        s |= g.lanes | g.childLanes, p |= g.subtreeFlags, p |= g.flags, g.return = n, g = g.sibling;
    return n.subtreeFlags |= p, n.childLanes = s, i;
  }
  function dC(n, i, s) {
    var p = i.pendingProps;
    switch (lv(i), i.tag) {
      case 2:
      case 16:
      case 15:
      case 0:
      case 11:
      case 7:
      case 8:
      case 12:
      case 9:
      case 14:
        return yr(i), null;
      case 1:
        return un(i.type) && Ta(), yr(i), null;
      case 3:
        return p = i.stateNode, pl(), Pt(gn), Pt(dt), hv(), p.pendingContext && (p.context = p.pendingContext, p.pendingContext = null), (n === null || n.child === null) && (Sf(i) ? i.flags |= 4 : n === null || n.memoizedState.isDehydrated && !(i.flags & 256) || (i.flags |= 1024, Ia !== null && (Ov(Ia), Ia = null))), El(n, i), yr(i), null;
      case 5:
        vv(i);
        var g = cu(_s.current);
        if (s = i.type, n !== null && i.stateNode != null)
          xl(n, i, s, p, g), n.ref !== i.ref && (i.flags |= 512, i.flags |= 2097152);
        else {
          if (!p) {
            if (i.stateNode === null)
              throw Error(u(166));
            return yr(i), null;
          }
          if (n = cu(ri.current), Sf(i)) {
            p = i.stateNode, s = i.type;
            var S = i.memoizedProps;
            switch (p[pi] = i, p[ou] = S, n = (i.mode & 1) !== 0, s) {
              case "dialog":
                qt("cancel", p), qt("close", p);
                break;
              case "iframe":
              case "object":
              case "embed":
                qt("load", p);
                break;
              case "video":
              case "audio":
                for (g = 0; g < Ms.length; g++)
                  qt(Ms[g], p);
                break;
              case "source":
                qt("error", p);
                break;
              case "img":
              case "image":
              case "link":
                qt(
                  "error",
                  p
                ), qt("load", p);
                break;
              case "details":
                qt("toggle", p);
                break;
              case "input":
                Et(p, S), qt("invalid", p);
                break;
              case "select":
                p._wrapperState = { wasMultiple: !!S.multiple }, qt("invalid", p);
                break;
              case "textarea":
                Fr(p, S), qt("invalid", p);
            }
            pr(s, S), g = null;
            for (var I in S)
              if (S.hasOwnProperty(I)) {
                var F = S[I];
                I === "children" ? typeof F == "string" ? p.textContent !== F && (S.suppressHydrationWarning !== !0 && hf(p.textContent, F, n), g = ["children", F]) : typeof F == "number" && p.textContent !== "" + F && (S.suppressHydrationWarning !== !0 && hf(
                  p.textContent,
                  F,
                  n
                ), g = ["children", "" + F]) : c.hasOwnProperty(I) && F != null && I === "onScroll" && qt("scroll", p);
              }
            switch (s) {
              case "input":
                At(p), Ya(p, S, !0);
                break;
              case "textarea":
                At(p), na(p);
                break;
              case "select":
              case "option":
                break;
              default:
                typeof S.onClick == "function" && (p.onclick = mf);
            }
            p = g, i.updateQueue = p, p !== null && (i.flags |= 4);
          } else {
            I = g.nodeType === 9 ? g : g.ownerDocument, n === "http://www.w3.org/1999/xhtml" && (n = $a(s)), n === "http://www.w3.org/1999/xhtml" ? s === "script" ? (n = I.createElement("div"), n.innerHTML = "<script><\/script>", n = n.removeChild(n.firstChild)) : typeof p.is == "string" ? n = I.createElement(s, { is: p.is }) : (n = I.createElement(s), s === "select" && (I = n, p.multiple ? I.multiple = !0 : p.size && (I.size = p.size))) : n = I.createElementNS(n, s), n[pi] = i, n[ou] = p, gi(n, i, !1, !1), i.stateNode = n;
            e: {
              switch (I = Bn(s, p), s) {
                case "dialog":
                  qt("cancel", n), qt("close", n), g = p;
                  break;
                case "iframe":
                case "object":
                case "embed":
                  qt("load", n), g = p;
                  break;
                case "video":
                case "audio":
                  for (g = 0; g < Ms.length; g++)
                    qt(Ms[g], n);
                  g = p;
                  break;
                case "source":
                  qt("error", n), g = p;
                  break;
                case "img":
                case "image":
                case "link":
                  qt(
                    "error",
                    n
                  ), qt("load", n), g = p;
                  break;
                case "details":
                  qt("toggle", n), g = p;
                  break;
                case "input":
                  Et(n, p), g = _t(n, p), qt("invalid", n);
                  break;
                case "option":
                  g = p;
                  break;
                case "select":
                  n._wrapperState = { wasMultiple: !!p.multiple }, g = W({}, p, { value: void 0 }), qt("invalid", n);
                  break;
                case "textarea":
                  Fr(n, p), g = ci(n, p), qt("invalid", n);
                  break;
                default:
                  g = p;
              }
              pr(s, g), F = g;
              for (S in F)
                if (F.hasOwnProperty(S)) {
                  var H = F[S];
                  S === "style" ? Kt(n, H) : S === "dangerouslySetInnerHTML" ? (H = H ? H.__html : void 0, H != null && ps(n, H)) : S === "children" ? typeof H == "string" ? (s !== "textarea" || H !== "") && co(n, H) : typeof H == "number" && co(n, "" + H) : S !== "suppressContentEditableWarning" && S !== "suppressHydrationWarning" && S !== "autoFocus" && (c.hasOwnProperty(S) ? H != null && S === "onScroll" && qt("scroll", n) : H != null && z(n, S, H, I));
                }
              switch (s) {
                case "input":
                  At(n), Ya(n, p, !1);
                  break;
                case "textarea":
                  At(n), na(n);
                  break;
                case "option":
                  p.value != null && n.setAttribute("value", "" + He(p.value));
                  break;
                case "select":
                  n.multiple = !!p.multiple, S = p.value, S != null ? ta(n, !!p.multiple, S, !1) : p.defaultValue != null && ta(
                    n,
                    !!p.multiple,
                    p.defaultValue,
                    !0
                  );
                  break;
                default:
                  typeof g.onClick == "function" && (n.onclick = mf);
              }
              switch (s) {
                case "button":
                case "input":
                case "select":
                case "textarea":
                  p = !!p.autoFocus;
                  break e;
                case "img":
                  p = !0;
                  break e;
                default:
                  p = !1;
              }
            }
            p && (i.flags |= 4);
          }
          i.ref !== null && (i.flags |= 512, i.flags |= 2097152);
        }
        return yr(i), null;
      case 6:
        if (n && i.stateNode != null)
          Na(n, i, n.memoizedProps, p);
        else {
          if (typeof p != "string" && i.stateNode === null)
            throw Error(u(166));
          if (s = cu(_s.current), cu(ri.current), Sf(i)) {
            if (p = i.stateNode, s = i.memoizedProps, p[pi] = i, (S = p.nodeValue !== s) && (n = oa, n !== null))
              switch (n.tag) {
                case 3:
                  hf(p.nodeValue, s, (n.mode & 1) !== 0);
                  break;
                case 5:
                  n.memoizedProps.suppressHydrationWarning !== !0 && hf(p.nodeValue, s, (n.mode & 1) !== 0);
              }
            S && (i.flags |= 4);
          } else
            p = (s.nodeType === 9 ? s : s.ownerDocument).createTextNode(p), p[pi] = i, i.stateNode = p;
        }
        return yr(i), null;
      case 13:
        if (Pt(ln), p = i.memoizedState, n === null || n.memoizedState !== null && n.memoizedState.dehydrated !== null) {
          if (rn && _r !== null && i.mode & 1 && !(i.flags & 128))
            wg(), cn(), i.flags |= 98560, S = !1;
          else if (S = Sf(i), p !== null && p.dehydrated !== null) {
            if (n === null) {
              if (!S)
                throw Error(u(318));
              if (S = i.memoizedState, S = S !== null ? S.dehydrated : null, !S)
                throw Error(u(317));
              S[pi] = i;
            } else
              cn(), !(i.flags & 128) && (i.memoizedState = null), i.flags |= 4;
            yr(i), S = !1;
          } else
            Ia !== null && (Ov(Ia), Ia = null), S = !0;
          if (!S)
            return i.flags & 65536 ? i : null;
        }
        return i.flags & 128 ? (i.lanes = s, i) : (p = p !== null, p !== (n !== null && n.memoizedState !== null) && p && (i.child.flags |= 8192, i.mode & 1 && (n === null || ln.current & 1 ? zn === 0 && (zn = 3) : rc())), i.updateQueue !== null && (i.flags |= 4), yr(i), null);
      case 4:
        return pl(), El(n, i), n === null && ol(i.stateNode.containerInfo), yr(i), null;
      case 10:
        return fv(i.type._context), yr(i), null;
      case 17:
        return un(i.type) && Ta(), yr(i), null;
      case 19:
        if (Pt(ln), S = i.memoizedState, S === null)
          return yr(i), null;
        if (p = (i.flags & 128) !== 0, I = S.rendering, I === null)
          if (p)
            yn(S, !1);
          else {
            if (zn !== 0 || n !== null && n.flags & 128)
              for (n = i.child; n !== null; ) {
                if (I = If(n), I !== null) {
                  for (i.flags |= 128, yn(S, !1), p = I.updateQueue, p !== null && (i.updateQueue = p, i.flags |= 4), i.subtreeFlags = 0, p = s, s = i.child; s !== null; )
                    S = s, n = p, S.flags &= 14680066, I = S.alternate, I === null ? (S.childLanes = 0, S.lanes = n, S.child = null, S.subtreeFlags = 0, S.memoizedProps = null, S.memoizedState = null, S.updateQueue = null, S.dependencies = null, S.stateNode = null) : (S.childLanes = I.childLanes, S.lanes = I.lanes, S.child = I.child, S.subtreeFlags = 0, S.deletions = null, S.memoizedProps = I.memoizedProps, S.memoizedState = I.memoizedState, S.updateQueue = I.updateQueue, S.type = I.type, n = I.dependencies, S.dependencies = n === null ? null : { lanes: n.lanes, firstContext: n.firstContext }), s = s.sibling;
                  return Ht(ln, ln.current & 1 | 2), i.child;
                }
                n = n.sibling;
              }
            S.tail !== null && mn() > Ml && (i.flags |= 128, p = !0, yn(S, !1), i.lanes = 4194304);
          }
        else {
          if (!p)
            if (n = If(I), n !== null) {
              if (i.flags |= 128, p = !0, s = n.updateQueue, s !== null && (i.updateQueue = s, i.flags |= 4), yn(S, !0), S.tail === null && S.tailMode === "hidden" && !I.alternate && !rn)
                return yr(i), null;
            } else
              2 * mn() - S.renderingStartTime > Ml && s !== 1073741824 && (i.flags |= 128, p = !0, yn(S, !1), i.lanes = 4194304);
          S.isBackwards ? (I.sibling = i.child, i.child = I) : (s = S.last, s !== null ? s.sibling = I : i.child = I, S.last = I);
        }
        return S.tail !== null ? (i = S.tail, S.rendering = i, S.tail = i.sibling, S.renderingStartTime = mn(), i.sibling = null, s = ln.current, Ht(ln, p ? s & 1 | 2 : s & 1), i) : (yr(i), null);
      case 22:
      case 23:
        return od(), p = i.memoizedState !== null, n !== null && n.memoizedState !== null !== p && (i.flags |= 8192), p && i.mode & 1 ? Zr & 1073741824 && (yr(i), i.subtreeFlags & 6 && (i.flags |= 8192)) : yr(i), null;
      case 24:
        return null;
      case 25:
        return null;
    }
    throw Error(u(156, i.tag));
  }
  function pC(n, i) {
    switch (lv(i), i.tag) {
      case 1:
        return un(i.type) && Ta(), n = i.flags, n & 65536 ? (i.flags = n & -65537 | 128, i) : null;
      case 3:
        return pl(), Pt(gn), Pt(dt), hv(), n = i.flags, n & 65536 && !(n & 128) ? (i.flags = n & -65537 | 128, i) : null;
      case 5:
        return vv(i), null;
      case 13:
        if (Pt(ln), n = i.memoizedState, n !== null && n.dehydrated !== null) {
          if (i.alternate === null)
            throw Error(u(340));
          cn();
        }
        return n = i.flags, n & 65536 ? (i.flags = n & -65537 | 128, i) : null;
      case 19:
        return Pt(ln), null;
      case 4:
        return pl(), null;
      case 10:
        return fv(i.type._context), null;
      case 22:
      case 23:
        return od(), null;
      case 24:
        return null;
      default:
        return null;
    }
  }
  var wl = !1, ar = !1, Qf = typeof WeakSet == "function" ? WeakSet : Set, Pe = null;
  function Tl(n, i) {
    var s = n.ref;
    if (s !== null)
      if (typeof s == "function")
        try {
          s(null);
        } catch (p) {
          Cn(n, i, p);
        }
      else
        s.current = null;
  }
  function xv(n, i, s) {
    try {
      s();
    } catch (p) {
      Cn(n, i, p);
    }
  }
  var Jf = !1;
  function vC(n, i) {
    if (ev = qo, n = cf(), Vi(n)) {
      if ("selectionStart" in n)
        var s = { start: n.selectionStart, end: n.selectionEnd };
      else
        e: {
          s = (s = n.ownerDocument) && s.defaultView || window;
          var p = s.getSelection && s.getSelection();
          if (p && p.rangeCount !== 0) {
            s = p.anchorNode;
            var g = p.anchorOffset, S = p.focusNode;
            p = p.focusOffset;
            try {
              s.nodeType, S.nodeType;
            } catch {
              s = null;
              break e;
            }
            var I = 0, F = -1, H = -1, ne = 0, pe = 0, ve = n, de = null;
            t:
              for (; ; ) {
                for (var Fe; ve !== s || g !== 0 && ve.nodeType !== 3 || (F = I + g), ve !== S || p !== 0 && ve.nodeType !== 3 || (H = I + p), ve.nodeType === 3 && (I += ve.nodeValue.length), (Fe = ve.firstChild) !== null; )
                  de = ve, ve = Fe;
                for (; ; ) {
                  if (ve === n)
                    break t;
                  if (de === s && ++ne === g && (F = I), de === S && ++pe === p && (H = I), (Fe = ve.nextSibling) !== null)
                    break;
                  ve = de, de = ve.parentNode;
                }
                ve = Fe;
              }
            s = F === -1 || H === -1 ? null : { start: F, end: H };
          } else
            s = null;
        }
      s = s || { start: 0, end: 0 };
    } else
      s = null;
    for (au = { focusedElem: n, selectionRange: s }, qo = !1, Pe = i; Pe !== null; )
      if (i = Pe, n = i.child, (i.subtreeFlags & 1028) !== 0 && n !== null)
        n.return = i, Pe = n;
      else
        for (; Pe !== null; ) {
          i = Pe;
          try {
            var ze = i.alternate;
            if (i.flags & 1024)
              switch (i.tag) {
                case 0:
                case 11:
                case 15:
                  break;
                case 1:
                  if (ze !== null) {
                    var je = ze.memoizedProps, Tn = ze.memoizedState, J = i.stateNode, _ = J.getSnapshotBeforeUpdate(i.elementType === i.type ? je : Ur(i.type, je), Tn);
                    J.__reactInternalSnapshotBeforeUpdate = _;
                  }
                  break;
                case 3:
                  var ee = i.stateNode.containerInfo;
                  ee.nodeType === 1 ? ee.textContent = "" : ee.nodeType === 9 && ee.documentElement && ee.removeChild(ee.documentElement);
                  break;
                case 5:
                case 6:
                case 4:
                case 17:
                  break;
                default:
                  throw Error(u(163));
              }
          } catch (be) {
            Cn(i, i.return, be);
          }
          if (n = i.sibling, n !== null) {
            n.return = i.return, Pe = n;
            break;
          }
          Pe = i.return;
        }
    return ze = Jf, Jf = !1, ze;
  }
  function Rl(n, i, s) {
    var p = i.updateQueue;
    if (p = p !== null ? p.lastEffect : null, p !== null) {
      var g = p = p.next;
      do {
        if ((g.tag & n) === n) {
          var S = g.destroy;
          g.destroy = void 0, S !== void 0 && xv(i, s, S);
        }
        g = g.next;
      } while (g !== p);
    }
  }
  function Kf(n, i) {
    if (i = i.updateQueue, i = i !== null ? i.lastEffect : null, i !== null) {
      var s = i = i.next;
      do {
        if ((s.tag & n) === n) {
          var p = s.create;
          s.destroy = p();
        }
        s = s.next;
      } while (s !== i);
    }
  }
  function qf(n) {
    var i = n.ref;
    if (i !== null) {
      var s = n.stateNode;
      switch (n.tag) {
        case 5:
          n = s;
          break;
        default:
          n = s;
      }
      typeof i == "function" ? i(n) : i.current = n;
    }
  }
  function jg(n) {
    var i = n.alternate;
    i !== null && (n.alternate = null, jg(i)), n.child = null, n.deletions = null, n.sibling = null, n.tag === 5 && (i = n.stateNode, i !== null && (delete i[pi], delete i[ou], delete i[rv], delete i[lC], delete i[av])), n.stateNode = null, n.return = null, n.dependencies = null, n.memoizedProps = null, n.memoizedState = null, n.pendingProps = null, n.stateNode = null, n.updateQueue = null;
  }
  function wv(n) {
    return n.tag === 5 || n.tag === 3 || n.tag === 4;
  }
  function Ug(n) {
    e:
      for (; ; ) {
        for (; n.sibling === null; ) {
          if (n.return === null || wv(n.return))
            return null;
          n = n.return;
        }
        for (n.sibling.return = n.return, n = n.sibling; n.tag !== 5 && n.tag !== 6 && n.tag !== 18; ) {
          if (n.flags & 2 || n.child === null || n.tag === 4)
            continue e;
          n.child.return = n, n = n.child;
        }
        if (!(n.flags & 2))
          return n.stateNode;
      }
  }
  function qs(n, i, s) {
    var p = n.tag;
    if (p === 5 || p === 6)
      n = n.stateNode, i ? s.nodeType === 8 ? s.parentNode.insertBefore(n, i) : s.insertBefore(n, i) : (s.nodeType === 8 ? (i = s.parentNode, i.insertBefore(n, s)) : (i = s, i.appendChild(n)), s = s._reactRootContainer, s != null || i.onclick !== null || (i.onclick = mf));
    else if (p !== 4 && (n = n.child, n !== null))
      for (qs(n, i, s), n = n.sibling; n !== null; )
        qs(n, i, s), n = n.sibling;
  }
  function Il(n, i, s) {
    var p = n.tag;
    if (p === 5 || p === 6)
      n = n.stateNode, i ? s.insertBefore(n, i) : s.appendChild(n);
    else if (p !== 4 && (n = n.child, n !== null))
      for (Il(n, i, s), n = n.sibling; n !== null; )
        Il(n, i, s), n = n.sibling;
  }
  var sn = null, Yn = !1;
  function xr(n, i, s) {
    for (s = s.child; s !== null; )
      Al(n, i, s), s = s.sibling;
  }
  function Al(n, i, s) {
    if (Ja && typeof Ja.onCommitFiberUnmount == "function")
      try {
        Ja.onCommitFiberUnmount(gs, s);
      } catch {
      }
    switch (s.tag) {
      case 5:
        ar || Tl(s, i);
      case 6:
        var p = sn, g = Yn;
        sn = null, xr(n, i, s), sn = p, Yn = g, sn !== null && (Yn ? (n = sn, s = s.stateNode, n.nodeType === 8 ? n.parentNode.removeChild(s) : n.removeChild(s)) : sn.removeChild(s.stateNode));
        break;
      case 18:
        sn !== null && (Yn ? (n = sn, s = s.stateNode, n.nodeType === 8 ? go(n.parentNode, s) : n.nodeType === 1 && go(n, s), Es(n)) : go(sn, s.stateNode));
        break;
      case 4:
        p = sn, g = Yn, sn = s.stateNode.containerInfo, Yn = !0, xr(n, i, s), sn = p, Yn = g;
        break;
      case 0:
      case 11:
      case 14:
      case 15:
        if (!ar && (p = s.updateQueue, p !== null && (p = p.lastEffect, p !== null))) {
          g = p = p.next;
          do {
            var S = g, I = S.destroy;
            S = S.tag, I !== void 0 && (S & 2 || S & 4) && xv(s, i, I), g = g.next;
          } while (g !== p);
        }
        xr(n, i, s);
        break;
      case 1:
        if (!ar && (Tl(s, i), p = s.stateNode, typeof p.componentWillUnmount == "function"))
          try {
            p.props = s.memoizedProps, p.state = s.memoizedState, p.componentWillUnmount();
          } catch (F) {
            Cn(s, i, F);
          }
        xr(n, i, s);
        break;
      case 21:
        xr(n, i, s);
        break;
      case 22:
        s.mode & 1 ? (ar = (p = ar) || s.memoizedState !== null, xr(n, i, s), ar = p) : xr(n, i, s);
        break;
      default:
        xr(n, i, s);
    }
  }
  function Ol(n) {
    var i = n.updateQueue;
    if (i !== null) {
      n.updateQueue = null;
      var s = n.stateNode;
      s === null && (s = n.stateNode = new Qf()), i.forEach(function(p) {
        var g = SC.bind(null, n, p);
        s.has(p) || (s.add(p), p.then(g, g));
      });
    }
  }
  function $n(n, i) {
    var s = i.deletions;
    if (s !== null)
      for (var p = 0; p < s.length; p++) {
        var g = s[p];
        try {
          var S = n, I = i, F = I;
          e:
            for (; F !== null; ) {
              switch (F.tag) {
                case 5:
                  sn = F.stateNode, Yn = !1;
                  break e;
                case 3:
                  sn = F.stateNode.containerInfo, Yn = !0;
                  break e;
                case 4:
                  sn = F.stateNode.containerInfo, Yn = !0;
                  break e;
              }
              F = F.return;
            }
          if (sn === null)
            throw Error(u(160));
          Al(S, I, g), sn = null, Yn = !1;
          var H = g.alternate;
          H !== null && (H.return = null), g.return = null;
        } catch (ne) {
          Cn(g, i, ne);
        }
      }
    if (i.subtreeFlags & 12854)
      for (i = i.child; i !== null; )
        Wg(i, n), i = i.sibling;
  }
  function Wg(n, i) {
    var s = n.alternate, p = n.flags;
    switch (n.tag) {
      case 0:
      case 11:
      case 14:
      case 15:
        if ($n(i, n), yi(n), p & 4) {
          try {
            Rl(3, n, n.return), Kf(3, n);
          } catch (je) {
            Cn(n, n.return, je);
          }
          try {
            Rl(5, n, n.return);
          } catch (je) {
            Cn(n, n.return, je);
          }
        }
        break;
      case 1:
        $n(i, n), yi(n), p & 512 && s !== null && Tl(s, s.return);
        break;
      case 5:
        if ($n(i, n), yi(n), p & 512 && s !== null && Tl(s, s.return), n.flags & 32) {
          var g = n.stateNode;
          try {
            co(g, "");
          } catch (je) {
            Cn(n, n.return, je);
          }
        }
        if (p & 4 && (g = n.stateNode, g != null)) {
          var S = n.memoizedProps, I = s !== null ? s.memoizedProps : S, F = n.type, H = n.updateQueue;
          if (n.updateQueue = null, H !== null)
            try {
              F === "input" && S.type === "radio" && S.name != null && er(g, S), Bn(F, I);
              var ne = Bn(F, S);
              for (I = 0; I < H.length; I += 2) {
                var pe = H[I], ve = H[I + 1];
                pe === "style" ? Kt(g, ve) : pe === "dangerouslySetInnerHTML" ? ps(g, ve) : pe === "children" ? co(g, ve) : z(g, pe, ve, ne);
              }
              switch (F) {
                case "input":
                  In(g, S);
                  break;
                case "textarea":
                  ga(g, S);
                  break;
                case "select":
                  var de = g._wrapperState.wasMultiple;
                  g._wrapperState.wasMultiple = !!S.multiple;
                  var Fe = S.value;
                  Fe != null ? ta(g, !!S.multiple, Fe, !1) : de !== !!S.multiple && (S.defaultValue != null ? ta(
                    g,
                    !!S.multiple,
                    S.defaultValue,
                    !0
                  ) : ta(g, !!S.multiple, S.multiple ? [] : "", !1));
              }
              g[ou] = S;
            } catch (je) {
              Cn(n, n.return, je);
            }
        }
        break;
      case 6:
        if ($n(i, n), yi(n), p & 4) {
          if (n.stateNode === null)
            throw Error(u(162));
          g = n.stateNode, S = n.memoizedProps;
          try {
            g.nodeValue = S;
          } catch (je) {
            Cn(n, n.return, je);
          }
        }
        break;
      case 3:
        if ($n(i, n), yi(n), p & 4 && s !== null && s.memoizedState.isDehydrated)
          try {
            Es(i.containerInfo);
          } catch (je) {
            Cn(n, n.return, je);
          }
        break;
      case 4:
        $n(i, n), yi(n);
        break;
      case 13:
        $n(i, n), yi(n), g = n.child, g.flags & 8192 && (S = g.memoizedState !== null, g.stateNode.isHidden = S, !S || g.alternate !== null && g.alternate.memoizedState !== null || (td = mn())), p & 4 && Ol(n);
        break;
      case 22:
        if (pe = s !== null && s.memoizedState !== null, n.mode & 1 ? (ar = (ne = ar) || pe, $n(i, n), ar = ne) : $n(i, n), yi(n), p & 8192) {
          if (ne = n.memoizedState !== null, (n.stateNode.isHidden = ne) && !pe && n.mode & 1)
            for (Pe = n, pe = n.child; pe !== null; ) {
              for (ve = Pe = pe; Pe !== null; ) {
                switch (de = Pe, Fe = de.child, de.tag) {
                  case 0:
                  case 11:
                  case 14:
                  case 15:
                    Rl(4, de, de.return);
                    break;
                  case 1:
                    Tl(de, de.return);
                    var ze = de.stateNode;
                    if (typeof ze.componentWillUnmount == "function") {
                      p = de, s = de.return;
                      try {
                        i = p, ze.props = i.memoizedProps, ze.state = i.memoizedState, ze.componentWillUnmount();
                      } catch (je) {
                        Cn(p, s, je);
                      }
                    }
                    break;
                  case 5:
                    Tl(de, de.return);
                    break;
                  case 22:
                    if (de.memoizedState !== null) {
                      Xg(ve);
                      continue;
                    }
                }
                Fe !== null ? (Fe.return = de, Pe = Fe) : Xg(ve);
              }
              pe = pe.sibling;
            }
          e:
            for (pe = null, ve = n; ; ) {
              if (ve.tag === 5) {
                if (pe === null) {
                  pe = ve;
                  try {
                    g = ve.stateNode, ne ? (S = g.style, typeof S.setProperty == "function" ? S.setProperty("display", "none", "important") : S.display = "none") : (F = ve.stateNode, H = ve.memoizedProps.style, I = H != null && H.hasOwnProperty("display") ? H.display : null, F.style.display = wt("display", I));
                  } catch (je) {
                    Cn(n, n.return, je);
                  }
                }
              } else if (ve.tag === 6) {
                if (pe === null)
                  try {
                    ve.stateNode.nodeValue = ne ? "" : ve.memoizedProps;
                  } catch (je) {
                    Cn(n, n.return, je);
                  }
              } else if ((ve.tag !== 22 && ve.tag !== 23 || ve.memoizedState === null || ve === n) && ve.child !== null) {
                ve.child.return = ve, ve = ve.child;
                continue;
              }
              if (ve === n)
                break e;
              for (; ve.sibling === null; ) {
                if (ve.return === null || ve.return === n)
                  break e;
                pe === ve && (pe = null), ve = ve.return;
              }
              pe === ve && (pe = null), ve.sibling.return = ve.return, ve = ve.sibling;
            }
        }
        break;
      case 19:
        $n(i, n), yi(n), p & 4 && Ol(n);
        break;
      case 21:
        break;
      default:
        $n(
          i,
          n
        ), yi(n);
    }
  }
  function yi(n) {
    var i = n.flags;
    if (i & 2) {
      try {
        e: {
          for (var s = n.return; s !== null; ) {
            if (wv(s)) {
              var p = s;
              break e;
            }
            s = s.return;
          }
          throw Error(u(160));
        }
        switch (p.tag) {
          case 5:
            var g = p.stateNode;
            p.flags & 32 && (co(g, ""), p.flags &= -33);
            var S = Ug(n);
            Il(n, S, g);
            break;
          case 3:
          case 4:
            var I = p.stateNode.containerInfo, F = Ug(n);
            qs(n, F, I);
            break;
          default:
            throw Error(u(161));
        }
      } catch (H) {
        Cn(n, n.return, H);
      }
      n.flags &= -3;
    }
    i & 4096 && (n.flags &= -4097);
  }
  function hC(n, i, s) {
    Pe = n, Tv(n);
  }
  function Tv(n, i, s) {
    for (var p = (n.mode & 1) !== 0; Pe !== null; ) {
      var g = Pe, S = g.child;
      if (g.tag === 22 && p) {
        var I = g.memoizedState !== null || wl;
        if (!I) {
          var F = g.alternate, H = F !== null && F.memoizedState !== null || ar;
          F = wl;
          var ne = ar;
          if (wl = I, (ar = H) && !ne)
            for (Pe = g; Pe !== null; )
              I = Pe, H = I.child, I.tag === 22 && I.memoizedState !== null ? Rv(g) : H !== null ? (H.return = I, Pe = H) : Rv(g);
          for (; S !== null; )
            Pe = S, Tv(S), S = S.sibling;
          Pe = g, wl = F, ar = ne;
        }
        Dl(n);
      } else
        g.subtreeFlags & 8772 && S !== null ? (S.return = g, Pe = S) : Dl(n);
    }
  }
  function Dl(n) {
    for (; Pe !== null; ) {
      var i = Pe;
      if (i.flags & 8772) {
        var s = i.alternate;
        try {
          if (i.flags & 8772)
            switch (i.tag) {
              case 0:
              case 11:
              case 15:
                ar || Kf(5, i);
                break;
              case 1:
                var p = i.stateNode;
                if (i.flags & 4 && !ar)
                  if (s === null)
                    p.componentDidMount();
                  else {
                    var g = i.elementType === i.type ? s.memoizedProps : Ur(i.type, s.memoizedProps);
                    p.componentDidUpdate(g, s.memoizedState, p.__reactInternalSnapshotBeforeUpdate);
                  }
                var S = i.updateQueue;
                S !== null && Og(i, S, p);
                break;
              case 3:
                var I = i.updateQueue;
                if (I !== null) {
                  if (s = null, i.child !== null)
                    switch (i.child.tag) {
                      case 5:
                        s = i.child.stateNode;
                        break;
                      case 1:
                        s = i.child.stateNode;
                    }
                  Og(i, I, s);
                }
                break;
              case 5:
                var F = i.stateNode;
                if (s === null && i.flags & 4) {
                  s = F;
                  var H = i.memoizedProps;
                  switch (i.type) {
                    case "button":
                    case "input":
                    case "select":
                    case "textarea":
                      H.autoFocus && s.focus();
                      break;
                    case "img":
                      H.src && (s.src = H.src);
                  }
                }
                break;
              case 6:
                break;
              case 4:
                break;
              case 12:
                break;
              case 13:
                if (i.memoizedState === null) {
                  var ne = i.alternate;
                  if (ne !== null) {
                    var pe = ne.memoizedState;
                    if (pe !== null) {
                      var ve = pe.dehydrated;
                      ve !== null && Es(ve);
                    }
                  }
                }
                break;
              case 19:
              case 17:
              case 21:
              case 22:
              case 23:
              case 25:
                break;
              default:
                throw Error(u(163));
            }
          ar || i.flags & 512 && qf(i);
        } catch (de) {
          Cn(i, i.return, de);
        }
      }
      if (i === n) {
        Pe = null;
        break;
      }
      if (s = i.sibling, s !== null) {
        s.return = i.return, Pe = s;
        break;
      }
      Pe = i.return;
    }
  }
  function Xg(n) {
    for (; Pe !== null; ) {
      var i = Pe;
      if (i === n) {
        Pe = null;
        break;
      }
      var s = i.sibling;
      if (s !== null) {
        s.return = i.return, Pe = s;
        break;
      }
      Pe = i.return;
    }
  }
  function Rv(n) {
    for (; Pe !== null; ) {
      var i = Pe;
      try {
        switch (i.tag) {
          case 0:
          case 11:
          case 15:
            var s = i.return;
            try {
              Kf(4, i);
            } catch (H) {
              Cn(i, s, H);
            }
            break;
          case 1:
            var p = i.stateNode;
            if (typeof p.componentDidMount == "function") {
              var g = i.return;
              try {
                p.componentDidMount();
              } catch (H) {
                Cn(i, g, H);
              }
            }
            var S = i.return;
            try {
              qf(i);
            } catch (H) {
              Cn(i, S, H);
            }
            break;
          case 5:
            var I = i.return;
            try {
              qf(i);
            } catch (H) {
              Cn(i, I, H);
            }
        }
      } catch (H) {
        Cn(i, i.return, H);
      }
      if (i === n) {
        Pe = null;
        break;
      }
      var F = i.sibling;
      if (F !== null) {
        F.return = i.return, Pe = F;
        break;
      }
      Pe = i.return;
    }
  }
  var mC = Math.ceil, yu = Z.ReactCurrentDispatcher, ed = Z.ReactCurrentOwner, ka = Z.ReactCurrentBatchConfig, Rt = 0, bn = null, an = null, Hn = 0, Zr = 0, Nl = St(0), zn = 0, ec = null, bu = 0, kl = 0, Iv = 0, Io = null, br = null, td = 0, Ml = 1 / 0, Xi = null, nd = !1, Av = null, Ma = null, Ll = !1, La = null, rd = 0, tc = 0, ad = null, nc = -1, Cu = 0;
  function Qn() {
    return Rt & 6 ? mn() : nc !== -1 ? nc : nc = mn();
  }
  function Zi(n) {
    return n.mode & 1 ? Rt & 2 && Hn !== 0 ? Hn & -Hn : Ef.transition !== null ? (Cu === 0 && (Cu = Jc()), Cu) : (n = jt, n !== 0 || (n = window.event, n = n === void 0 ? 16 : Lp(n.type)), n) : 1;
  }
  function fn(n, i, s, p) {
    if (50 < tc)
      throw tc = 0, ad = null, Error(u(185));
    Ko(n, s, p), (!(Rt & 2) || n !== bn) && (n === bn && (!(Rt & 2) && (kl |= s), zn === 4 && bi(n, Hn)), Gn(n, p), s === 1 && Rt === 0 && !(i.mode & 1) && (Ml = mn() + 500, Zn && Hr()));
  }
  function Gn(n, i) {
    var s = n.callbackNode;
    Qc(n, i);
    var p = Ka(n, n === bn ? Hn : 0);
    if (p === 0)
      s !== null && km(s), n.callbackNode = null, n.callbackPriority = 0;
    else if (i = p & -p, n.callbackPriority !== i) {
      if (s != null && km(s), i === 1)
        n.tag === 0 ? ov(Fl.bind(null, n)) : iv(Fl.bind(null, n)), nv(function() {
          !(Rt & 6) && Hr();
        }), s = null;
      else {
        switch (Np(p)) {
          case 1:
            s = Mi;
            break;
          case 4:
            s = Tt;
            break;
          case 16:
            s = fo;
            break;
          case 536870912:
            s = Ip;
            break;
          default:
            s = fo;
        }
        s = ey(s, id.bind(null, n));
      }
      n.callbackPriority = i, n.callbackNode = s;
    }
  }
  function id(n, i) {
    if (nc = -1, Cu = 0, Rt & 6)
      throw Error(u(327));
    var s = n.callbackNode;
    if (Bl() && n.callbackNode !== s)
      return null;
    var p = Ka(n, n === bn ? Hn : 0);
    if (p === 0)
      return null;
    if (p & 30 || p & n.expiredLanes || i)
      i = ud(n, p);
    else {
      i = p;
      var g = Rt;
      Rt |= 2;
      var S = Yg();
      (bn !== n || Hn !== i) && (Xi = null, Ml = mn() + 500, Eu(n, i));
      do
        try {
          yC();
          break;
        } catch (F) {
          Zg(n, F);
        }
      while (!0);
      ua(), yu.current = S, Rt = g, an !== null ? i = 0 : (bn = null, Hn = 0, i = zn);
    }
    if (i !== 0) {
      if (i === 2 && (g = Op(n), g !== 0 && (p = g, i = Su(n, g))), i === 1)
        throw s = ec, Eu(n, 0), bi(n, p), Gn(n, mn()), s;
      if (i === 6)
        bi(n, p);
      else {
        if (g = n.current.alternate, !(p & 30) && !Dv(g) && (i = ud(n, p), i === 2 && (S = Op(n), S !== 0 && (p = S, i = Su(n, S))), i === 1))
          throw s = ec, Eu(n, 0), bi(n, p), Gn(n, mn()), s;
        switch (n.finishedWork = g, n.finishedLanes = p, i) {
          case 0:
          case 1:
            throw Error(u(345));
          case 2:
            Ao(n, br, Xi);
            break;
          case 3:
            if (bi(n, p), (p & 130023424) === p && (i = td + 500 - mn(), 10 < i)) {
              if (Ka(n, 0) !== 0)
                break;
              if (g = n.suspendedLanes, (g & p) !== p) {
                Qn(), n.pingedLanes |= n.suspendedLanes & g;
                break;
              }
              n.timeoutHandle = iu(Ao.bind(null, n, br, Xi), i);
              break;
            }
            Ao(n, br, Xi);
            break;
          case 4:
            if (bi(n, p), (p & 4194240) === p)
              break;
            for (i = n.eventTimes, g = -1; 0 < p; ) {
              var I = 31 - ba(p);
              S = 1 << I, I = i[I], I > g && (g = I), p &= ~S;
            }
            if (p = g, p = mn() - p, p = (120 > p ? 120 : 480 > p ? 480 : 1080 > p ? 1080 : 1920 > p ? 1920 : 3e3 > p ? 3e3 : 4320 > p ? 4320 : 1960 * mC(p / 1960)) - p, 10 < p) {
              n.timeoutHandle = iu(Ao.bind(null, n, br, Xi), p);
              break;
            }
            Ao(n, br, Xi);
            break;
          case 5:
            Ao(n, br, Xi);
            break;
          default:
            throw Error(u(329));
        }
      }
    }
    return Gn(n, mn()), n.callbackNode === s ? id.bind(null, n) : null;
  }
  function Su(n, i) {
    var s = Io;
    return n.current.memoizedState.isDehydrated && (Eu(n, i).flags |= 256), n = ud(n, i), n !== 2 && (i = br, br = s, i !== null && Ov(i)), n;
  }
  function Ov(n) {
    br === null ? br = n : br.push.apply(br, n);
  }
  function Dv(n) {
    for (var i = n; ; ) {
      if (i.flags & 16384) {
        var s = i.updateQueue;
        if (s !== null && (s = s.stores, s !== null))
          for (var p = 0; p < s.length; p++) {
            var g = s[p], S = g.getSnapshot;
            g = g.value;
            try {
              if (!Ea(S(), g))
                return !1;
            } catch {
              return !1;
            }
          }
      }
      if (s = i.child, i.subtreeFlags & 16384 && s !== null)
        s.return = i, i = s;
      else {
        if (i === n)
          break;
        for (; i.sibling === null; ) {
          if (i.return === null || i.return === n)
            return !0;
          i = i.return;
        }
        i.sibling.return = i.return, i = i.sibling;
      }
    }
    return !0;
  }
  function bi(n, i) {
    for (i &= ~Iv, i &= ~kl, n.suspendedLanes |= i, n.pingedLanes &= ~i, n = n.expirationTimes; 0 < i; ) {
      var s = 31 - ba(i), p = 1 << s;
      n[s] = -1, i &= ~p;
    }
  }
  function Fl(n) {
    if (Rt & 6)
      throw Error(u(327));
    Bl();
    var i = Ka(n, 0);
    if (!(i & 1))
      return Gn(n, mn()), null;
    var s = ud(n, i);
    if (n.tag !== 0 && s === 2) {
      var p = Op(n);
      p !== 0 && (i = p, s = Su(n, p));
    }
    if (s === 1)
      throw s = ec, Eu(n, 0), bi(n, i), Gn(n, mn()), s;
    if (s === 6)
      throw Error(u(345));
    return n.finishedWork = n.current.alternate, n.finishedLanes = i, Ao(n, br, Xi), Gn(n, mn()), null;
  }
  function Nv(n, i) {
    var s = Rt;
    Rt |= 1;
    try {
      return n(i);
    } finally {
      Rt = s, Rt === 0 && (Ml = mn() + 500, Zn && Hr());
    }
  }
  function Ci(n) {
    La !== null && La.tag === 0 && !(Rt & 6) && Bl();
    var i = Rt;
    Rt |= 1;
    var s = ka.transition, p = jt;
    try {
      if (ka.transition = null, jt = 1, n)
        return n();
    } finally {
      jt = p, ka.transition = s, Rt = i, !(Rt & 6) && Hr();
    }
  }
  function od() {
    Zr = Nl.current, Pt(Nl);
  }
  function Eu(n, i) {
    n.finishedWork = null, n.finishedLanes = 0;
    var s = n.timeoutHandle;
    if (s !== -1 && (n.timeoutHandle = -1, Cg(s)), an !== null)
      for (s = an.return; s !== null; ) {
        var p = s;
        switch (lv(p), p.tag) {
          case 1:
            p = p.type.childContextTypes, p != null && Ta();
            break;
          case 3:
            pl(), Pt(gn), Pt(dt), hv();
            break;
          case 5:
            vv(p);
            break;
          case 4:
            pl();
            break;
          case 13:
            Pt(ln);
            break;
          case 19:
            Pt(ln);
            break;
          case 10:
            fv(p.type._context);
            break;
          case 22:
          case 23:
            od();
        }
        s = s.return;
      }
    if (bn = n, an = n = Oo(n.current, null), Hn = Zr = i, zn = 0, ec = null, Iv = kl = bu = 0, br = Io = null, su !== null) {
      for (i = 0; i < su.length; i++)
        if (s = su[i], p = s.interleaved, p !== null) {
          s.interleaved = null;
          var g = p.next, S = s.pending;
          if (S !== null) {
            var I = S.next;
            S.next = g, p.next = I;
          }
          s.pending = p;
        }
      su = null;
    }
    return n;
  }
  function Zg(n, i) {
    do {
      var s = an;
      try {
        if (ua(), Of.current = gr, la) {
          for (var p = $e.memoizedState; p !== null; ) {
            var g = p.queue;
            g !== null && (g.pending = null), p = p.next;
          }
          la = !1;
        }
        if (Xe = 0, xt = pt = $e = null, vl = !1, Us = 0, ed.current = null, s === null || s.return === null) {
          zn = 1, ec = i, an = null;
          break;
        }
        e: {
          var S = n, I = s.return, F = s, H = i;
          if (i = Hn, F.flags |= 32768, H !== null && typeof H == "object" && typeof H.then == "function") {
            var ne = H, pe = F, ve = pe.tag;
            if (!(pe.mode & 1) && (ve === 0 || ve === 11 || ve === 15)) {
              var de = pe.alternate;
              de ? (pe.updateQueue = de.updateQueue, pe.memoizedState = de.memoizedState, pe.lanes = de.lanes) : (pe.updateQueue = null, pe.memoizedState = null);
            }
            var Fe = Pg(I);
            if (Fe !== null) {
              Fe.flags &= -257, Cv(Fe, I, F, S, i), Fe.mode & 1 && $s(S, ne, i), i = Fe, H = ne;
              var ze = i.updateQueue;
              if (ze === null) {
                var je = /* @__PURE__ */ new Set();
                je.add(H), i.updateQueue = je;
              } else
                ze.add(H);
              break e;
            } else {
              if (!(i & 1)) {
                $s(S, ne, i), rc();
                break e;
              }
              H = Error(u(426));
            }
          } else if (rn && F.mode & 1) {
            var Tn = Pg(I);
            if (Tn !== null) {
              !(Tn.flags & 65536) && (Tn.flags |= 256), Cv(Tn, I, F, S, i), cv(To(H, F));
              break e;
            }
          }
          S = H = To(H, F), zn !== 4 && (zn = 2), Io === null ? Io = [S] : Io.push(S), S = I;
          do {
            switch (S.tag) {
              case 3:
                S.flags |= 65536, i &= -i, S.lanes |= i;
                var J = Bg(S, H, i);
                Ag(S, J);
                break e;
              case 1:
                F = H;
                var _ = S.type, ee = S.stateNode;
                if (!(S.flags & 128) && (typeof _.getDerivedStateFromError == "function" || ee !== null && typeof ee.componentDidCatch == "function" && (Ma === null || !Ma.has(ee)))) {
                  S.flags |= 65536, i &= -i, S.lanes |= i;
                  var be = Vg(S, F, i);
                  Ag(S, be);
                  break e;
                }
            }
            S = S.return;
          } while (S !== null);
        }
        Qg(s);
      } catch (Ze) {
        i = Ze, an === s && s !== null && (an = s = s.return);
        continue;
      }
      break;
    } while (!0);
  }
  function Yg() {
    var n = yu.current;
    return yu.current = gr, n === null ? gr : n;
  }
  function rc() {
    (zn === 0 || zn === 3 || zn === 2) && (zn = 4), bn === null || !(bu & 268435455) && !(kl & 268435455) || bi(bn, Hn);
  }
  function ud(n, i) {
    var s = Rt;
    Rt |= 2;
    var p = Yg();
    (bn !== n || Hn !== i) && (Xi = null, Eu(n, i));
    do
      try {
        gC();
        break;
      } catch (g) {
        Zg(n, g);
      }
    while (!0);
    if (ua(), Rt = s, yu.current = p, an !== null)
      throw Error(u(261));
    return bn = null, Hn = 0, zn;
  }
  function gC() {
    for (; an !== null; )
      $g(an);
  }
  function yC() {
    for (; an !== null && !zb(); )
      $g(an);
  }
  function $g(n) {
    var i = qg(n.alternate, n, Zr);
    n.memoizedProps = n.pendingProps, i === null ? Qg(n) : an = i, ed.current = null;
  }
  function Qg(n) {
    var i = n;
    do {
      var s = i.alternate;
      if (n = i.return, i.flags & 32768) {
        if (s = pC(s, i), s !== null) {
          s.flags &= 32767, an = s;
          return;
        }
        if (n !== null)
          n.flags |= 32768, n.subtreeFlags = 0, n.deletions = null;
        else {
          zn = 6, an = null;
          return;
        }
      } else if (s = dC(s, i, Zr), s !== null) {
        an = s;
        return;
      }
      if (i = i.sibling, i !== null) {
        an = i;
        return;
      }
      an = i = n;
    } while (i !== null);
    zn === 0 && (zn = 5);
  }
  function Ao(n, i, s) {
    var p = jt, g = ka.transition;
    try {
      ka.transition = null, jt = 1, bC(n, i, s, p);
    } finally {
      ka.transition = g, jt = p;
    }
    return null;
  }
  function bC(n, i, s, p) {
    do
      Bl();
    while (La !== null);
    if (Rt & 6)
      throw Error(u(327));
    s = n.finishedWork;
    var g = n.finishedLanes;
    if (s === null)
      return null;
    if (n.finishedWork = null, n.finishedLanes = 0, s === n.current)
      throw Error(u(177));
    n.callbackNode = null, n.callbackPriority = 0;
    var S = s.lanes | s.childLanes;
    if (Ub(n, S), n === bn && (an = bn = null, Hn = 0), !(s.subtreeFlags & 2064) && !(s.flags & 2064) || Ll || (Ll = !0, ey(fo, function() {
      return Bl(), null;
    })), S = (s.flags & 15990) !== 0, s.subtreeFlags & 15990 || S) {
      S = ka.transition, ka.transition = null;
      var I = jt;
      jt = 1;
      var F = Rt;
      Rt |= 4, ed.current = null, vC(n, s), Wg(s, n), ff(au), qo = !!ev, au = ev = null, n.current = s, hC(s), Gb(), Rt = F, jt = I, ka.transition = S;
    } else
      n.current = s;
    if (Ll && (Ll = !1, La = n, rd = g), S = n.pendingLanes, S === 0 && (Ma = null), Lm(s.stateNode), Gn(n, mn()), i !== null)
      for (p = n.onRecoverableError, s = 0; s < i.length; s++)
        g = i[s], p(g.value, { componentStack: g.stack, digest: g.digest });
    if (nd)
      throw nd = !1, n = Av, Av = null, n;
    return rd & 1 && n.tag !== 0 && Bl(), S = n.pendingLanes, S & 1 ? n === ad ? tc++ : (tc = 0, ad = n) : tc = 0, Hr(), null;
  }
  function Bl() {
    if (La !== null) {
      var n = Np(rd), i = ka.transition, s = jt;
      try {
        if (ka.transition = null, jt = 16 > n ? 16 : n, La === null)
          var p = !1;
        else {
          if (n = La, La = null, rd = 0, Rt & 6)
            throw Error(u(331));
          var g = Rt;
          for (Rt |= 4, Pe = n.current; Pe !== null; ) {
            var S = Pe, I = S.child;
            if (Pe.flags & 16) {
              var F = S.deletions;
              if (F !== null) {
                for (var H = 0; H < F.length; H++) {
                  var ne = F[H];
                  for (Pe = ne; Pe !== null; ) {
                    var pe = Pe;
                    switch (pe.tag) {
                      case 0:
                      case 11:
                      case 15:
                        Rl(8, pe, S);
                    }
                    var ve = pe.child;
                    if (ve !== null)
                      ve.return = pe, Pe = ve;
                    else
                      for (; Pe !== null; ) {
                        pe = Pe;
                        var de = pe.sibling, Fe = pe.return;
                        if (jg(pe), pe === ne) {
                          Pe = null;
                          break;
                        }
                        if (de !== null) {
                          de.return = Fe, Pe = de;
                          break;
                        }
                        Pe = Fe;
                      }
                  }
                }
                var ze = S.alternate;
                if (ze !== null) {
                  var je = ze.child;
                  if (je !== null) {
                    ze.child = null;
                    do {
                      var Tn = je.sibling;
                      je.sibling = null, je = Tn;
                    } while (je !== null);
                  }
                }
                Pe = S;
              }
            }
            if (S.subtreeFlags & 2064 && I !== null)
              I.return = S, Pe = I;
            else
              e:
                for (; Pe !== null; ) {
                  if (S = Pe, S.flags & 2048)
                    switch (S.tag) {
                      case 0:
                      case 11:
                      case 15:
                        Rl(9, S, S.return);
                    }
                  var J = S.sibling;
                  if (J !== null) {
                    J.return = S.return, Pe = J;
                    break e;
                  }
                  Pe = S.return;
                }
          }
          var _ = n.current;
          for (Pe = _; Pe !== null; ) {
            I = Pe;
            var ee = I.child;
            if (I.subtreeFlags & 2064 && ee !== null)
              ee.return = I, Pe = ee;
            else
              e:
                for (I = _; Pe !== null; ) {
                  if (F = Pe, F.flags & 2048)
                    try {
                      switch (F.tag) {
                        case 0:
                        case 11:
                        case 15:
                          Kf(9, F);
                      }
                    } catch (Ze) {
                      Cn(F, F.return, Ze);
                    }
                  if (F === I) {
                    Pe = null;
                    break e;
                  }
                  var be = F.sibling;
                  if (be !== null) {
                    be.return = F.return, Pe = be;
                    break e;
                  }
                  Pe = F.return;
                }
          }
          if (Rt = g, Hr(), Ja && typeof Ja.onPostCommitFiberRoot == "function")
            try {
              Ja.onPostCommitFiberRoot(gs, n);
            } catch {
            }
          p = !0;
        }
        return p;
      } finally {
        jt = s, ka.transition = i;
      }
    }
    return !1;
  }
  function Jg(n, i, s) {
    i = To(s, i), i = Bg(n, i, 1), n = xo(n, i, 1), i = Qn(), n !== null && (Ko(n, 1, i), Gn(n, i));
  }
  function Cn(n, i, s) {
    if (n.tag === 3)
      Jg(n, n, s);
    else
      for (; i !== null; ) {
        if (i.tag === 3) {
          Jg(i, n, s);
          break;
        } else if (i.tag === 1) {
          var p = i.stateNode;
          if (typeof i.type.getDerivedStateFromError == "function" || typeof p.componentDidCatch == "function" && (Ma === null || !Ma.has(p))) {
            n = To(s, n), n = Vg(i, n, 1), i = xo(i, n, 1), n = Qn(), i !== null && (Ko(i, 1, n), Gn(i, n));
            break;
          }
        }
        i = i.return;
      }
  }
  function CC(n, i, s) {
    var p = n.pingCache;
    p !== null && p.delete(i), i = Qn(), n.pingedLanes |= n.suspendedLanes & s, bn === n && (Hn & s) === s && (zn === 4 || zn === 3 && (Hn & 130023424) === Hn && 500 > mn() - td ? Eu(n, 0) : Iv |= s), Gn(n, i);
  }
  function Kg(n, i) {
    i === 0 && (n.mode & 1 ? (i = Qu, Qu <<= 1, !(Qu & 130023424) && (Qu = 4194304)) : i = 1);
    var s = Qn();
    n = ji(n, i), n !== null && (Ko(n, i, s), Gn(n, s));
  }
  function kv(n) {
    var i = n.memoizedState, s = 0;
    i !== null && (s = i.retryLane), Kg(n, s);
  }
  function SC(n, i) {
    var s = 0;
    switch (n.tag) {
      case 13:
        var p = n.stateNode, g = n.memoizedState;
        g !== null && (s = g.retryLane);
        break;
      case 19:
        p = n.stateNode;
        break;
      default:
        throw Error(u(314));
    }
    p !== null && p.delete(i), Kg(n, s);
  }
  var qg;
  qg = function(n, i, s) {
    if (n !== null)
      if (n.memoizedProps !== i.pendingProps || gn.current)
        Wr = !0;
      else {
        if (!(n.lanes & s) && !(i.flags & 128))
          return Wr = !1, Wi(n, i, s);
        Wr = !!(n.flags & 131072);
      }
    else
      Wr = !1, rn && i.flags & 1048576 && uv(i, ll, i.index);
    switch (i.lanes = 0, i.tag) {
      case 2:
        var p = i.type;
        Ks(n, i), n = i.pendingProps;
        var g = wa(i, dt.current);
        cl(i, s), g = fe(null, i, p, n, g, s);
        var S = kn();
        return i.flags |= 1, typeof g == "object" && g !== null && typeof g.render == "function" && g.$$typeof === void 0 ? (i.tag = 1, i.memoizedState = null, i.updateQueue = null, un(p) ? (S = !0, gf(i)) : S = !1, i.memoizedState = g.state !== null && g.state !== void 0 ? g.state : null, wf(i), g.updater = hu, i.stateNode = g, g._reactInternals = i, yv(i, p, n, s), i = Xf(null, i, p, !0, S, s)) : (i.tag = 0, rn && S && yf(i), xn(null, i, g, s), i = i.child), i;
      case 16:
        p = i.elementType;
        e: {
          switch (Ks(n, i), n = i.pendingProps, g = p._init, p = g(p._payload), i.type = p, g = i.tag = EC(p), n = Ur(p, n), g) {
            case 0:
              i = Ct(null, i, p, n, s);
              break e;
            case 1:
              i = Qs(null, i, p, n, s);
              break e;
            case 11:
              i = Sl(null, i, p, n, s);
              break e;
            case 14:
              i = Ro(null, i, p, Ur(p.type, n), s);
              break e;
          }
          throw Error(u(
            306,
            p,
            ""
          ));
        }
        return i;
      case 0:
        return p = i.type, g = i.pendingProps, g = i.elementType === p ? g : Ur(p, g), Ct(n, i, p, g, s);
      case 1:
        return p = i.type, g = i.pendingProps, g = i.elementType === p ? g : Ur(p, g), Qs(n, i, p, g, s);
      case 3:
        e: {
          if (fC(i), n === null)
            throw Error(u(387));
          p = i.pendingProps, S = i.memoizedState, g = S.element, fl(n, i), Rf(i, p, null, s);
          var I = i.memoizedState;
          if (p = I.element, S.isDehydrated)
            if (S = { element: p, isDehydrated: !1, cache: I.cache, pendingSuspenseBoundaries: I.pendingSuspenseBoundaries, transitions: I.transitions }, i.updateQueue.baseState = S, i.memoizedState = S, i.flags & 256) {
              g = To(Error(u(423)), i), i = zg(n, i, p, s, g);
              break e;
            } else if (p !== g) {
              g = To(Error(u(424)), i), i = zg(n, i, p, s, g);
              break e;
            } else
              for (_r = ni(i.stateNode.containerInfo.firstChild), oa = i, rn = !0, Ia = null, s = Rg(i, null, p, s), i.child = s; s; )
                s.flags = s.flags & -3 | 4096, s = s.sibling;
          else {
            if (cn(), p === g) {
              i = wn(n, i, s);
              break e;
            }
            xn(n, i, p, s);
          }
          i = i.child;
        }
        return i;
      case 5:
        return Dg(i), n === null && Cf(i), p = i.type, g = i.pendingProps, S = n !== null ? n.memoizedProps : null, I = g.children, Bs(p, g) ? I = null : S !== null && Bs(p, S) && (i.flags |= 32), mu(n, i), xn(n, i, I, s), i.child;
      case 6:
        return n === null && Cf(i), null;
      case 13:
        return Gg(n, i, s);
      case 4:
        return pv(i, i.stateNode.containerInfo), p = i.pendingProps, n === null ? i.child = sl(i, null, p, s) : xn(n, i, p, s), i.child;
      case 11:
        return p = i.type, g = i.pendingProps, g = i.elementType === p ? g : Ur(p, g), Sl(n, i, p, g, s);
      case 7:
        return xn(n, i, i.pendingProps, s), i.child;
      case 8:
        return xn(n, i, i.pendingProps.children, s), i.child;
      case 12:
        return xn(n, i, i.pendingProps.children, s), i.child;
      case 10:
        e: {
          if (p = i.type._context, g = i.pendingProps, S = i.memoizedProps, I = g.value, Ht(_i, p._currentValue), p._currentValue = I, S !== null)
            if (Ea(S.value, I)) {
              if (S.children === g.children && !gn.current) {
                i = wn(n, i, s);
                break e;
              }
            } else
              for (S = i.child, S !== null && (S.return = i); S !== null; ) {
                var F = S.dependencies;
                if (F !== null) {
                  I = S.child;
                  for (var H = F.firstContext; H !== null; ) {
                    if (H.context === p) {
                      if (S.tag === 1) {
                        H = jr(-1, s & -s), H.tag = 2;
                        var ne = S.updateQueue;
                        if (ne !== null) {
                          ne = ne.shared;
                          var pe = ne.pending;
                          pe === null ? H.next = H : (H.next = pe.next, pe.next = H), ne.pending = H;
                        }
                      }
                      S.lanes |= s, H = S.alternate, H !== null && (H.lanes |= s), dv(
                        S.return,
                        s,
                        i
                      ), F.lanes |= s;
                      break;
                    }
                    H = H.next;
                  }
                } else if (S.tag === 10)
                  I = S.type === i.type ? null : S.child;
                else if (S.tag === 18) {
                  if (I = S.return, I === null)
                    throw Error(u(341));
                  I.lanes |= s, F = I.alternate, F !== null && (F.lanes |= s), dv(I, s, i), I = S.sibling;
                } else
                  I = S.child;
                if (I !== null)
                  I.return = S;
                else
                  for (I = S; I !== null; ) {
                    if (I === i) {
                      I = null;
                      break;
                    }
                    if (S = I.sibling, S !== null) {
                      S.return = I.return, I = S;
                      break;
                    }
                    I = I.return;
                  }
                S = I;
              }
          xn(n, i, g.children, s), i = i.child;
        }
        return i;
      case 9:
        return g = i.type, p = i.pendingProps.children, cl(i, s), g = Oa(g), p = p(g), i.flags |= 1, xn(n, i, p, s), i.child;
      case 14:
        return p = i.type, g = Ur(p, i.pendingProps), g = Ur(p.type, g), Ro(n, i, p, g, s);
      case 15:
        return Wf(n, i, i.type, i.pendingProps, s);
      case 17:
        return p = i.type, g = i.pendingProps, g = i.elementType === p ? g : Ur(p, g), Ks(n, i), i.tag = 1, un(p) ? (n = !0, gf(i)) : n = !1, cl(i, s), Mg(i, p, g), yv(i, p, g, s), Xf(null, i, p, !0, n, s);
      case 19:
        return Ev(n, i, s);
      case 22:
        return Xr(n, i, s);
    }
    throw Error(u(156, i.tag));
  };
  function ey(n, i) {
    return Rp(n, i);
  }
  function ty(n, i, s, p) {
    this.tag = n, this.key = s, this.sibling = this.child = this.return = this.stateNode = this.type = this.elementType = null, this.index = 0, this.ref = null, this.pendingProps = i, this.dependencies = this.memoizedState = this.updateQueue = this.memoizedProps = null, this.mode = p, this.subtreeFlags = this.flags = 0, this.deletions = null, this.childLanes = this.lanes = 0, this.alternate = null;
  }
  function Fa(n, i, s, p) {
    return new ty(n, i, s, p);
  }
  function Mv(n) {
    return n = n.prototype, !(!n || !n.isReactComponent);
  }
  function EC(n) {
    if (typeof n == "function")
      return Mv(n) ? 1 : 0;
    if (n != null) {
      if (n = n.$$typeof, n === xe)
        return 11;
      if (n === We)
        return 14;
    }
    return 2;
  }
  function Oo(n, i) {
    var s = n.alternate;
    return s === null ? (s = Fa(n.tag, i, n.key, n.mode), s.elementType = n.elementType, s.type = n.type, s.stateNode = n.stateNode, s.alternate = n, n.alternate = s) : (s.pendingProps = i, s.type = n.type, s.flags = 0, s.subtreeFlags = 0, s.deletions = null), s.flags = n.flags & 14680064, s.childLanes = n.childLanes, s.lanes = n.lanes, s.child = n.child, s.memoizedProps = n.memoizedProps, s.memoizedState = n.memoizedState, s.updateQueue = n.updateQueue, i = n.dependencies, s.dependencies = i === null ? null : { lanes: i.lanes, firstContext: i.firstContext }, s.sibling = n.sibling, s.index = n.index, s.ref = n.ref, s;
  }
  function ld(n, i, s, p, g, S) {
    var I = 2;
    if (p = n, typeof n == "function")
      Mv(n) && (I = 1);
    else if (typeof n == "string")
      I = 5;
    else
      e:
        switch (n) {
          case P:
            return xu(s.children, g, S, i);
          case U:
            I = 8, g |= 8;
            break;
          case se:
            return n = Fa(12, s, i, g | 2), n.elementType = se, n.lanes = S, n;
          case ye:
            return n = Fa(13, s, i, g), n.elementType = ye, n.lanes = S, n;
          case Me:
            return n = Fa(19, s, i, g), n.elementType = Me, n.lanes = S, n;
          case tt:
            return sd(s, g, S, i);
          default:
            if (typeof n == "object" && n !== null)
              switch (n.$$typeof) {
                case Se:
                  I = 10;
                  break e;
                case he:
                  I = 9;
                  break e;
                case xe:
                  I = 11;
                  break e;
                case We:
                  I = 14;
                  break e;
                case Ee:
                  I = 16, p = null;
                  break e;
              }
            throw Error(u(130, n == null ? n : typeof n, ""));
        }
    return i = Fa(I, s, i, g), i.elementType = n, i.type = p, i.lanes = S, i;
  }
  function xu(n, i, s, p) {
    return n = Fa(7, n, p, i), n.lanes = s, n;
  }
  function sd(n, i, s, p) {
    return n = Fa(22, n, p, i), n.elementType = tt, n.lanes = s, n.stateNode = { isHidden: !1 }, n;
  }
  function cd(n, i, s) {
    return n = Fa(6, n, null, i), n.lanes = s, n;
  }
  function ac(n, i, s) {
    return i = Fa(4, n.children !== null ? n.children : [], n.key, i), i.lanes = s, i.stateNode = { containerInfo: n.containerInfo, pendingChildren: null, implementation: n.implementation }, i;
  }
  function ic(n, i, s, p, g) {
    this.tag = i, this.containerInfo = n, this.finishedWork = this.pingCache = this.current = this.pendingChildren = null, this.timeoutHandle = -1, this.callbackNode = this.pendingContext = this.context = null, this.callbackPriority = 0, this.eventTimes = Dp(0), this.expirationTimes = Dp(-1), this.entangledLanes = this.finishedLanes = this.mutableReadLanes = this.expiredLanes = this.pingedLanes = this.suspendedLanes = this.pendingLanes = 0, this.entanglements = Dp(0), this.identifierPrefix = p, this.onRecoverableError = g, this.mutableSourceEagerHydrationData = null;
  }
  function Lv(n, i, s, p, g, S, I, F, H) {
    return n = new ic(n, i, s, F, H), i === 1 ? (i = 1, S === !0 && (i |= 8)) : i = 0, S = Fa(3, null, null, i), n.current = S, S.stateNode = n, S.memoizedState = { element: p, isDehydrated: s, cache: null, transitions: null, pendingSuspenseBoundaries: null }, wf(S), n;
  }
  function ny(n, i, s) {
    var p = 3 < arguments.length && arguments[3] !== void 0 ? arguments[3] : null;
    return { $$typeof: Q, key: p == null ? null : "" + p, children: n, containerInfo: i, implementation: s };
  }
  function Fv(n) {
    if (!n)
      return hi;
    n = n._reactInternals;
    e: {
      if (di(n) !== n || n.tag !== 1)
        throw Error(u(170));
      var i = n;
      do {
        switch (i.tag) {
          case 3:
            i = i.stateNode.context;
            break e;
          case 1:
            if (un(i.type)) {
              i = i.stateNode.__reactInternalMemoizedMergedChildContext;
              break e;
            }
        }
        i = i.return;
      } while (i !== null);
      throw Error(u(171));
    }
    if (n.tag === 1) {
      var s = n.type;
      if (un(s))
        return Hs(n, s, i);
    }
    return i;
  }
  function Bv(n, i, s, p, g, S, I, F, H) {
    return n = Lv(s, p, !0, n, g, S, I, F, H), n.context = Fv(null), s = n.current, p = Qn(), g = Zi(s), S = jr(p, g), S.callback = i ?? null, xo(s, S, g), n.current.lanes = g, Ko(n, g, p), Gn(n, p), n;
  }
  function fd(n, i, s, p) {
    var g = i.current, S = Qn(), I = Zi(g);
    return s = Fv(s), i.context === null ? i.context = s : i.pendingContext = s, i = jr(S, I), i.payload = { element: n }, p = p === void 0 ? null : p, p !== null && (i.callback = p), n = xo(g, i, I), n !== null && (fn(n, g, I, S), Tf(n, g, I)), I;
  }
  function oc(n) {
    if (n = n.current, !n.child)
      return null;
    switch (n.child.tag) {
      case 5:
        return n.child.stateNode;
      default:
        return n.child.stateNode;
    }
  }
  function ry(n, i) {
    if (n = n.memoizedState, n !== null && n.dehydrated !== null) {
      var s = n.retryLane;
      n.retryLane = s !== 0 && s < i ? s : i;
    }
  }
  function Vv(n, i) {
    ry(n, i), (n = n.alternate) && ry(n, i);
  }
  function xC() {
    return null;
  }
  var Pv = typeof reportError == "function" ? reportError : function(n) {
    console.error(n);
  };
  function dd(n) {
    this._internalRoot = n;
  }
  uc.prototype.render = dd.prototype.render = function(n) {
    var i = this._internalRoot;
    if (i === null)
      throw Error(u(409));
    fd(n, i, null, null);
  }, uc.prototype.unmount = dd.prototype.unmount = function() {
    var n = this._internalRoot;
    if (n !== null) {
      this._internalRoot = null;
      var i = n.containerInfo;
      Ci(function() {
        fd(null, n, null, null);
      }), i[zi] = null;
    }
  };
  function uc(n) {
    this._internalRoot = n;
  }
  uc.prototype.unstable_scheduleHydration = function(n) {
    if (n) {
      var i = Pm();
      n = { blockedOn: null, target: n, priority: i };
      for (var s = 0; s < Zt.length && i !== 0 && i < Zt[s].priority; s++)
        ;
      Zt.splice(s, 0, n), s === 0 && Hm(n);
    }
  };
  function Do(n) {
    return !(!n || n.nodeType !== 1 && n.nodeType !== 9 && n.nodeType !== 11);
  }
  function pd(n) {
    return !(!n || n.nodeType !== 1 && n.nodeType !== 9 && n.nodeType !== 11 && (n.nodeType !== 8 || n.nodeValue !== " react-mount-point-unstable "));
  }
  function ay() {
  }
  function wC(n, i, s, p, g) {
    if (g) {
      if (typeof p == "function") {
        var S = p;
        p = function() {
          var ne = oc(I);
          S.call(ne);
        };
      }
      var I = Bv(i, p, n, 0, null, !1, !1, "", ay);
      return n._reactRootContainer = I, n[zi] = I.current, ol(n.nodeType === 8 ? n.parentNode : n), Ci(), I;
    }
    for (; g = n.lastChild; )
      n.removeChild(g);
    if (typeof p == "function") {
      var F = p;
      p = function() {
        var ne = oc(H);
        F.call(ne);
      };
    }
    var H = Lv(n, 0, !1, null, null, !1, !1, "", ay);
    return n._reactRootContainer = H, n[zi] = H.current, ol(n.nodeType === 8 ? n.parentNode : n), Ci(function() {
      fd(i, H, s, p);
    }), H;
  }
  function vd(n, i, s, p, g) {
    var S = s._reactRootContainer;
    if (S) {
      var I = S;
      if (typeof g == "function") {
        var F = g;
        g = function() {
          var H = oc(I);
          F.call(H);
        };
      }
      fd(i, I, n, g);
    } else
      I = wC(s, i, n, g, p);
    return oc(I);
  }
  Vm = function(n) {
    switch (n.tag) {
      case 3:
        var i = n.stateNode;
        if (i.current.memoizedState.isDehydrated) {
          var s = Jo(i.pendingLanes);
          s !== 0 && (ys(i, s | 1), Gn(i, mn()), !(Rt & 6) && (Ml = mn() + 500, Hr()));
        }
        break;
      case 13:
        Ci(function() {
          var p = ji(n, 1);
          if (p !== null) {
            var g = Qn();
            fn(p, n, 1, g);
          }
        }), Vv(n, 1);
    }
  }, Kc = function(n) {
    if (n.tag === 13) {
      var i = ji(n, 134217728);
      if (i !== null) {
        var s = Qn();
        fn(i, n, 134217728, s);
      }
      Vv(n, 134217728);
    }
  }, Ut = function(n) {
    if (n.tag === 13) {
      var i = Zi(n), s = ji(n, i);
      if (s !== null) {
        var p = Qn();
        fn(s, n, i, p);
      }
      Vv(n, i);
    }
  }, Pm = function() {
    return jt;
  }, kp = function(n, i) {
    var s = jt;
    try {
      return jt = n, i();
    } finally {
      jt = s;
    }
  }, fi = function(n, i, s) {
    switch (i) {
      case "input":
        if (In(n, s), i = s.name, s.type === "radio" && i != null) {
          for (s = n; s.parentNode; )
            s = s.parentNode;
          for (s = s.querySelectorAll("input[name=" + JSON.stringify("" + i) + '][type="radio"]'), i = 0; i < s.length; i++) {
            var p = s[i];
            if (p !== n && p.form === n.form) {
              var g = qe(p);
              if (!g)
                throw Error(u(90));
              Ce(p), In(p, g);
            }
          }
        }
        break;
      case "textarea":
        ga(n, s);
        break;
      case "select":
        i = s.value, i != null && ta(n, !!s.multiple, i, !1);
    }
  }, Am = Nv, Om = Ci;
  var TC = { usingClientEntryPoint: !1, Events: [Ps, ul, qe, Wc, Xc, Nv] }, lc = { findFiberByHostInstance: xa, bundleType: 0, version: "18.3.1", rendererPackageName: "react-dom" }, iy = { bundleType: lc.bundleType, version: lc.version, rendererPackageName: lc.rendererPackageName, rendererConfig: lc.rendererConfig, overrideHookState: null, overrideHookStateDeletePath: null, overrideHookStateRenamePath: null, overrideProps: null, overridePropsDeletePath: null, overridePropsRenamePath: null, setErrorHandler: null, setSuspenseHandler: null, scheduleUpdate: null, currentDispatcherRef: Z.ReactCurrentDispatcher, findHostInstanceByFiber: function(n) {
    return n = Dm(n), n === null ? null : n.stateNode;
  }, findFiberByHostInstance: lc.findFiberByHostInstance || xC, findHostInstancesForRefresh: null, scheduleRefresh: null, scheduleRoot: null, setRefreshHandler: null, getCurrentFiber: null, reconcilerVersion: "18.3.1-next-f1338f8080-20240426" };
  if (typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u") {
    var hd = __REACT_DEVTOOLS_GLOBAL_HOOK__;
    if (!hd.isDisabled && hd.supportsFiber)
      try {
        gs = hd.inject(iy), Ja = hd;
      } catch {
      }
  }
  return Ga.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = TC, Ga.createPortal = function(n, i) {
    var s = 2 < arguments.length && arguments[2] !== void 0 ? arguments[2] : null;
    if (!Do(i))
      throw Error(u(200));
    return ny(n, i, null, s);
  }, Ga.createRoot = function(n, i) {
    if (!Do(n))
      throw Error(u(299));
    var s = !1, p = "", g = Pv;
    return i != null && (i.unstable_strictMode === !0 && (s = !0), i.identifierPrefix !== void 0 && (p = i.identifierPrefix), i.onRecoverableError !== void 0 && (g = i.onRecoverableError)), i = Lv(n, 1, !1, null, null, s, !1, p, g), n[zi] = i.current, ol(n.nodeType === 8 ? n.parentNode : n), new dd(i);
  }, Ga.findDOMNode = function(n) {
    if (n == null)
      return null;
    if (n.nodeType === 1)
      return n;
    var i = n._reactInternals;
    if (i === void 0)
      throw typeof n.render == "function" ? Error(u(188)) : (n = Object.keys(n).join(","), Error(u(268, n)));
    return n = Dm(i), n = n === null ? null : n.stateNode, n;
  }, Ga.flushSync = function(n) {
    return Ci(n);
  }, Ga.hydrate = function(n, i, s) {
    if (!pd(i))
      throw Error(u(200));
    return vd(null, n, i, !0, s);
  }, Ga.hydrateRoot = function(n, i, s) {
    if (!Do(n))
      throw Error(u(405));
    var p = s != null && s.hydratedSources || null, g = !1, S = "", I = Pv;
    if (s != null && (s.unstable_strictMode === !0 && (g = !0), s.identifierPrefix !== void 0 && (S = s.identifierPrefix), s.onRecoverableError !== void 0 && (I = s.onRecoverableError)), i = Bv(i, null, n, 1, s ?? null, g, !1, S, I), n[zi] = i.current, ol(n), p)
      for (n = 0; n < p.length; n++)
        s = p[n], g = s._getVersion, g = g(s._source), i.mutableSourceEagerHydrationData == null ? i.mutableSourceEagerHydrationData = [s, g] : i.mutableSourceEagerHydrationData.push(
          s,
          g
        );
    return new uc(i);
  }, Ga.render = function(n, i, s) {
    if (!pd(i))
      throw Error(u(200));
    return vd(null, n, i, !1, s);
  }, Ga.unmountComponentAtNode = function(n) {
    if (!pd(n))
      throw Error(u(40));
    return n._reactRootContainer ? (Ci(function() {
      vd(null, null, n, !1, function() {
        n._reactRootContainer = null, n[zi] = null;
      });
    }), !0) : !1;
  }, Ga.unstable_batchedUpdates = Nv, Ga.unstable_renderSubtreeIntoContainer = function(n, i, s, p) {
    if (!pd(s))
      throw Error(u(200));
    if (n == null || n._reactInternals === void 0)
      throw Error(u(38));
    return vd(n, i, s, !1, p);
  }, Ga.version = "18.3.1-next-f1338f8080-20240426", Ga;
}
var _a = {};
/**
 * @license React
 * react-dom.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
var pA;
function tV() {
  return pA || (pA = 1,  true && function() {
    typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u" && typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart == "function" && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStart(new Error());
    var a = (react__WEBPACK_IMPORTED_MODULE_1___default()), r = iD(), u = a.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED, f = !1;
    function c(e) {
      f = e;
    }
    function v(e) {
      if (!f) {
        for (var t = arguments.length, o = new Array(t > 1 ? t - 1 : 0), l = 1; l < t; l++)
          o[l - 1] = arguments[l];
        C("warn", e, o);
      }
    }
    function h(e) {
      if (!f) {
        for (var t = arguments.length, o = new Array(t > 1 ? t - 1 : 0), l = 1; l < t; l++)
          o[l - 1] = arguments[l];
        C("error", e, o);
      }
    }
    function C(e, t, o) {
      {
        var l = u.ReactDebugCurrentFrame, d = l.getStackAddendum();
        d !== "" && (t += "%s", o = o.concat([d]));
        var m = o.map(function(y) {
          return String(y);
        });
        m.unshift("Warning: " + t), Function.prototype.apply.call(console[e], console, m);
      }
    }
    var b = 0, E = 1, w = 2, R = 3, A = 4, D = 5, L = 6, M = 7, k = 8, B = 9, Y = 10, z = 11, Z = 12, V = 13, Q = 14, P = 15, U = 16, se = 17, Se = 18, he = 19, xe = 21, ye = 22, Me = 23, We = 24, Ee = 25, tt = !0, ue = !1, we = !1, W = !1, ce = !1, ge = !0, Ie = !1, Ae = !0, De = !0, Oe = !0, Ue = !0, He = /* @__PURE__ */ new Set(), nt = {}, vt = {};
    function At(e, t) {
      Ce(e, t), Ce(e + "Capture", t);
    }
    function Ce(e, t) {
      nt[e] && h("EventRegistry: More than one plugin attempted to publish the same registration name, `%s`.", e), nt[e] = t;
      {
        var o = e.toLowerCase();
        vt[o] = e, e === "onDoubleClick" && (vt.ondblclick = e);
      }
      for (var l = 0; l < t.length; l++)
        He.add(t[l]);
    }
    var Ot = typeof window < "u" && typeof window.document < "u" && typeof window.document.createElement < "u", _t = Object.prototype.hasOwnProperty;
    function Et(e) {
      {
        var t = typeof Symbol == "function" && Symbol.toStringTag, o = t && e[Symbol.toStringTag] || e.constructor.name || "Object";
        return o;
      }
    }
    function er(e) {
      try {
        return In(e), !1;
      } catch {
        return !0;
      }
    }
    function In(e) {
      return "" + e;
    }
    function Ya(e, t) {
      if (er(e))
        return h("The provided `%s` attribute is an unsupported type %s. This value must be coerced to a string before before using it here.", t, Et(e)), In(e);
    }
    function so(e) {
      if (er(e))
        return h("The provided key is an unsupported type %s. This value must be coerced to a string before before using it here.", Et(e)), In(e);
    }
    function Sr(e, t) {
      if (er(e))
        return h("The provided `%s` prop is an unsupported type %s. This value must be coerced to a string before before using it here.", t, Et(e)), In(e);
    }
    function ta(e, t) {
      if (er(e))
        return h("The provided `%s` CSS property is an unsupported type %s. This value must be coerced to a string before before using it here.", t, Et(e)), In(e);
    }
    function ci(e) {
      if (er(e))
        return h("The provided HTML markup uses a value of unsupported type %s. This value must be coerced to a string before before using it here.", Et(e)), In(e);
    }
    function Fr(e) {
      if (er(e))
        return h("Form field values (value, checked, defaultValue, or defaultChecked props) must be strings, not %s. This value must be coerced to a string before before using it here.", Et(e)), In(e);
    }
    var ga = 0, na = 1, $a = 2, tr = 3, Br = 4, ps = 5, co = 6, ke = ":A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD", rt = ke + "\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040", wt = new RegExp("^[" + ke + "][" + rt + "]*$"), Kt = {}, En = {};
    function pr(e) {
      return _t.call(En, e) ? !0 : _t.call(Kt, e) ? !1 : wt.test(e) ? (En[e] = !0, !0) : (Kt[e] = !0, h("Invalid attribute name: `%s`", e), !1);
    }
    function Bn(e, t, o) {
      return t !== null ? t.type === ga : o ? !1 : e.length > 2 && (e[0] === "o" || e[0] === "O") && (e[1] === "n" || e[1] === "N");
    }
    function ya(e, t, o, l) {
      if (o !== null && o.type === ga)
        return !1;
      switch (typeof t) {
        case "function":
        case "symbol":
          return !0;
        case "boolean": {
          if (l)
            return !1;
          if (o !== null)
            return !o.acceptsBooleans;
          var d = e.toLowerCase().slice(0, 5);
          return d !== "data-" && d !== "aria-";
        }
        default:
          return !1;
      }
    }
    function vn(e, t, o, l) {
      if (t === null || typeof t > "u" || ya(e, t, o, l))
        return !0;
      if (l)
        return !1;
      if (o !== null)
        switch (o.type) {
          case tr:
            return !t;
          case Br:
            return t === !1;
          case ps:
            return isNaN(t);
          case co:
            return isNaN(t) || t < 1;
        }
      return !1;
    }
    function fi(e) {
      return hn.hasOwnProperty(e) ? hn[e] : null;
    }
    function An(e, t, o, l, d, m, y) {
      this.acceptsBooleans = t === $a || t === tr || t === Br, this.attributeName = l, this.attributeNamespace = d, this.mustUseProperty = o, this.propertyName = e, this.type = t, this.sanitizeURL = m, this.removeEmptyString = y;
    }
    var hn = {}, Im = [
      "children",
      "dangerouslySetInnerHTML",
      // TODO: This prevents the assignment of defaultValue to regular
      // elements (not just inputs). Now that ReactDOMInput assigns to the
      // defaultValue property -- do we need this?
      "defaultValue",
      "defaultChecked",
      "innerHTML",
      "suppressContentEditableWarning",
      "suppressHydrationWarning",
      "style"
    ];
    Im.forEach(function(e) {
      hn[e] = new An(
        e,
        ga,
        !1,
        // mustUseProperty
        e,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [["acceptCharset", "accept-charset"], ["className", "class"], ["htmlFor", "for"], ["httpEquiv", "http-equiv"]].forEach(function(e) {
      var t = e[0], o = e[1];
      hn[t] = new An(
        t,
        na,
        !1,
        // mustUseProperty
        o,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), ["contentEditable", "draggable", "spellCheck", "value"].forEach(function(e) {
      hn[e] = new An(
        e,
        $a,
        !1,
        // mustUseProperty
        e.toLowerCase(),
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), ["autoReverse", "externalResourcesRequired", "focusable", "preserveAlpha"].forEach(function(e) {
      hn[e] = new An(
        e,
        $a,
        !1,
        // mustUseProperty
        e,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "allowFullScreen",
      "async",
      // Note: there is a special case that prevents it from being written to the DOM
      // on the client side because the browsers are inconsistent. Instead we call focus().
      "autoFocus",
      "autoPlay",
      "controls",
      "default",
      "defer",
      "disabled",
      "disablePictureInPicture",
      "disableRemotePlayback",
      "formNoValidate",
      "hidden",
      "loop",
      "noModule",
      "noValidate",
      "open",
      "playsInline",
      "readOnly",
      "required",
      "reversed",
      "scoped",
      "seamless",
      // Microdata
      "itemScope"
    ].forEach(function(e) {
      hn[e] = new An(
        e,
        tr,
        !1,
        // mustUseProperty
        e.toLowerCase(),
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "checked",
      // Note: `option.selected` is not updated if `select.multiple` is
      // disabled with `removeAttribute`. We have special logic for handling this.
      "multiple",
      "muted",
      "selected"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      hn[e] = new An(
        e,
        tr,
        !0,
        // mustUseProperty
        e,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "capture",
      "download"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      hn[e] = new An(
        e,
        Br,
        !1,
        // mustUseProperty
        e,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "cols",
      "rows",
      "size",
      "span"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      hn[e] = new An(
        e,
        co,
        !1,
        // mustUseProperty
        e,
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), ["rowSpan", "start"].forEach(function(e) {
      hn[e] = new An(
        e,
        ps,
        !1,
        // mustUseProperty
        e.toLowerCase(),
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    });
    var Wc = /[\-\:]([a-z])/g, Xc = function(e) {
      return e[1].toUpperCase();
    };
    [
      "accent-height",
      "alignment-baseline",
      "arabic-form",
      "baseline-shift",
      "cap-height",
      "clip-path",
      "clip-rule",
      "color-interpolation",
      "color-interpolation-filters",
      "color-profile",
      "color-rendering",
      "dominant-baseline",
      "enable-background",
      "fill-opacity",
      "fill-rule",
      "flood-color",
      "flood-opacity",
      "font-family",
      "font-size",
      "font-size-adjust",
      "font-stretch",
      "font-style",
      "font-variant",
      "font-weight",
      "glyph-name",
      "glyph-orientation-horizontal",
      "glyph-orientation-vertical",
      "horiz-adv-x",
      "horiz-origin-x",
      "image-rendering",
      "letter-spacing",
      "lighting-color",
      "marker-end",
      "marker-mid",
      "marker-start",
      "overline-position",
      "overline-thickness",
      "paint-order",
      "panose-1",
      "pointer-events",
      "rendering-intent",
      "shape-rendering",
      "stop-color",
      "stop-opacity",
      "strikethrough-position",
      "strikethrough-thickness",
      "stroke-dasharray",
      "stroke-dashoffset",
      "stroke-linecap",
      "stroke-linejoin",
      "stroke-miterlimit",
      "stroke-opacity",
      "stroke-width",
      "text-anchor",
      "text-decoration",
      "text-rendering",
      "underline-position",
      "underline-thickness",
      "unicode-bidi",
      "unicode-range",
      "units-per-em",
      "v-alphabetic",
      "v-hanging",
      "v-ideographic",
      "v-mathematical",
      "vector-effect",
      "vert-adv-y",
      "vert-origin-x",
      "vert-origin-y",
      "word-spacing",
      "writing-mode",
      "xmlns:xlink",
      "x-height"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      var t = e.replace(Wc, Xc);
      hn[t] = new An(
        t,
        na,
        !1,
        // mustUseProperty
        e,
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "xlink:actuate",
      "xlink:arcrole",
      "xlink:role",
      "xlink:show",
      "xlink:title",
      "xlink:type"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      var t = e.replace(Wc, Xc);
      hn[t] = new An(
        t,
        na,
        !1,
        // mustUseProperty
        e,
        "http://www.w3.org/1999/xlink",
        !1,
        // sanitizeURL
        !1
      );
    }), [
      "xml:base",
      "xml:lang",
      "xml:space"
      // NOTE: if you add a camelCased prop to this list,
      // you'll need to set attributeName to name.toLowerCase()
      // instead in the assignment below.
    ].forEach(function(e) {
      var t = e.replace(Wc, Xc);
      hn[t] = new An(
        t,
        na,
        !1,
        // mustUseProperty
        e,
        "http://www.w3.org/XML/1998/namespace",
        !1,
        // sanitizeURL
        !1
      );
    }), ["tabIndex", "crossOrigin"].forEach(function(e) {
      hn[e] = new An(
        e,
        na,
        !1,
        // mustUseProperty
        e.toLowerCase(),
        // attributeName
        null,
        // attributeNamespace
        !1,
        // sanitizeURL
        !1
      );
    });
    var Am = "xlinkHref";
    hn[Am] = new An(
      "xlinkHref",
      na,
      !1,
      // mustUseProperty
      "xlink:href",
      "http://www.w3.org/1999/xlink",
      !0,
      // sanitizeURL
      !1
    ), ["src", "href", "action", "formAction"].forEach(function(e) {
      hn[e] = new An(
        e,
        na,
        !1,
        // mustUseProperty
        e.toLowerCase(),
        // attributeName
        null,
        // attributeNamespace
        !0,
        // sanitizeURL
        !0
      );
    });
    var Om = /^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*\:/i, Zc = !1;
    function xp(e) {
      !Zc && Om.test(e) && (Zc = !0, h("A future version of React will block javascript: URLs as a security precaution. Use event handlers instead if you can. If you need to generate unsafe HTML try using dangerouslySetInnerHTML instead. React was passed %s.", JSON.stringify(e)));
    }
    function vs(e, t, o, l) {
      if (l.mustUseProperty) {
        var d = l.propertyName;
        return e[d];
      } else {
        Ya(o, t), l.sanitizeURL && xp("" + o);
        var m = l.attributeName, y = null;
        if (l.type === Br) {
          if (e.hasAttribute(m)) {
            var x = e.getAttribute(m);
            return x === "" ? !0 : vn(t, o, l, !1) ? x : x === "" + o ? o : x;
          }
        } else if (e.hasAttribute(m)) {
          if (vn(t, o, l, !1))
            return e.getAttribute(m);
          if (l.type === tr)
            return o;
          y = e.getAttribute(m);
        }
        return vn(t, o, l, !1) ? y === null ? o : y : y === "" + o ? o : y;
      }
    }
    function Yc(e, t, o, l) {
      {
        if (!pr(t))
          return;
        if (!e.hasAttribute(t))
          return o === void 0 ? void 0 : null;
        var d = e.getAttribute(t);
        return Ya(o, t), d === "" + o ? o : d;
      }
    }
    function Qo(e, t, o, l) {
      var d = fi(t);
      if (!Bn(t, d, l)) {
        if (vn(t, o, d, l) && (o = null), l || d === null) {
          if (pr(t)) {
            var m = t;
            o === null ? e.removeAttribute(m) : (Ya(o, t), e.setAttribute(m, "" + o));
          }
          return;
        }
        var y = d.mustUseProperty;
        if (y) {
          var x = d.propertyName;
          if (o === null) {
            var T = d.type;
            e[x] = T === tr ? !1 : "";
          } else
            e[x] = o;
          return;
        }
        var O = d.attributeName, N = d.attributeNamespace;
        if (o === null)
          e.removeAttribute(O);
        else {
          var j = d.type, G;
          j === tr || j === Br && o === !0 ? G = "" : (Ya(o, O), G = "" + o, d.sanitizeURL && xp(G.toString())), N ? e.setAttributeNS(N, O, G) : e.setAttribute(O, G);
        }
      }
    }
    var Xu = Symbol.for("react.element"), Qa = Symbol.for("react.portal"), ki = Symbol.for("react.fragment"), Zu = Symbol.for("react.strict_mode"), hs = Symbol.for("react.profiler"), wp = Symbol.for("react.provider"), Tp = Symbol.for("react.context"), Yu = Symbol.for("react.forward_ref"), di = Symbol.for("react.suspense"), ms = Symbol.for("react.suspense_list"), $u = Symbol.for("react.memo"), Er = Symbol.for("react.lazy"), Dm = Symbol.for("react.scope"), Nm = Symbol.for("react.debug_trace_mode"), Rp = Symbol.for("react.offscreen"), km = Symbol.for("react.legacy_hidden"), zb = Symbol.for("react.cache"), Gb = Symbol.for("react.tracing_marker"), mn = Symbol.iterator, _b = "@@iterator";
    function Mi(e) {
      if (e === null || typeof e != "object")
        return null;
      var t = mn && e[mn] || e[_b];
      return typeof t == "function" ? t : null;
    }
    var Tt = Object.assign, fo = 0, Mm, Ip, gs, Ja, Lm, ba, Fm;
    function Bm() {
    }
    Bm.__reactDisabledLog = !0;
    function jb() {
      {
        if (fo === 0) {
          Mm = console.log, Ip = console.info, gs = console.warn, Ja = console.error, Lm = console.group, ba = console.groupCollapsed, Fm = console.groupEnd;
          var e = {
            configurable: !0,
            enumerable: !0,
            value: Bm,
            writable: !0
          };
          Object.defineProperties(console, {
            info: e,
            log: e,
            warn: e,
            error: e,
            group: e,
            groupCollapsed: e,
            groupEnd: e
          });
        }
        fo++;
      }
    }
    function $c() {
      {
        if (fo--, fo === 0) {
          var e = {
            configurable: !0,
            enumerable: !0,
            writable: !0
          };
          Object.defineProperties(console, {
            log: Tt({}, e, {
              value: Mm
            }),
            info: Tt({}, e, {
              value: Ip
            }),
            warn: Tt({}, e, {
              value: gs
            }),
            error: Tt({}, e, {
              value: Ja
            }),
            group: Tt({}, e, {
              value: Lm
            }),
            groupCollapsed: Tt({}, e, {
              value: ba
            }),
            groupEnd: Tt({}, e, {
              value: Fm
            })
          });
        }
        fo < 0 && h("disabledDepth fell below zero. This is a bug in React. Please file an issue.");
      }
    }
    var Qu = u.ReactCurrentDispatcher, Jo;
    function Ka(e, t, o) {
      {
        if (Jo === void 0)
          try {
            throw Error();
          } catch (d) {
            var l = d.stack.trim().match(/\n( *(at )?)/);
            Jo = l && l[1] || "";
          }
        return `
` + Jo + e;
      }
    }
    var Ap = !1, Qc;
    {
      var Op = typeof WeakMap == "function" ? WeakMap : Map;
      Qc = new Op();
    }
    function Jc(e, t) {
      if (!e || Ap)
        return "";
      {
        var o = Qc.get(e);
        if (o !== void 0)
          return o;
      }
      var l;
      Ap = !0;
      var d = Error.prepareStackTrace;
      Error.prepareStackTrace = void 0;
      var m;
      m = Qu.current, Qu.current = null, jb();
      try {
        if (t) {
          var y = function() {
            throw Error();
          };
          if (Object.defineProperty(y.prototype, "props", {
            set: function() {
              throw Error();
            }
          }), typeof Reflect == "object" && Reflect.construct) {
            try {
              Reflect.construct(y, []);
            } catch (re) {
              l = re;
            }
            Reflect.construct(e, [], y);
          } else {
            try {
              y.call();
            } catch (re) {
              l = re;
            }
            e.call(y.prototype);
          }
        } else {
          try {
            throw Error();
          } catch (re) {
            l = re;
          }
          e();
        }
      } catch (re) {
        if (re && l && typeof re.stack == "string") {
          for (var x = re.stack.split(`
`), T = l.stack.split(`
`), O = x.length - 1, N = T.length - 1; O >= 1 && N >= 0 && x[O] !== T[N]; )
            N--;
          for (; O >= 1 && N >= 0; O--, N--)
            if (x[O] !== T[N]) {
              if (O !== 1 || N !== 1)
                do
                  if (O--, N--, N < 0 || x[O] !== T[N]) {
                    var j = `
` + x[O].replace(" at new ", " at ");
                    return e.displayName && j.includes("<anonymous>") && (j = j.replace("<anonymous>", e.displayName)), typeof e == "function" && Qc.set(e, j), j;
                  }
                while (O >= 1 && N >= 0);
              break;
            }
        }
      } finally {
        Ap = !1, Qu.current = m, $c(), Error.prepareStackTrace = d;
      }
      var G = e ? e.displayName || e.name : "", te = G ? Ka(G) : "";
      return typeof e == "function" && Qc.set(e, te), te;
    }
    function Dp(e, t, o) {
      return Jc(e, !0);
    }
    function Ko(e, t, o) {
      return Jc(e, !1);
    }
    function Ub(e) {
      var t = e.prototype;
      return !!(t && t.isReactComponent);
    }
    function ys(e, t, o) {
      if (e == null)
        return "";
      if (typeof e == "function")
        return Jc(e, Ub(e));
      if (typeof e == "string")
        return Ka(e);
      switch (e) {
        case di:
          return Ka("Suspense");
        case ms:
          return Ka("SuspenseList");
      }
      if (typeof e == "object")
        switch (e.$$typeof) {
          case Yu:
            return Ko(e.render);
          case $u:
            return ys(e.type, t, o);
          case Er: {
            var l = e, d = l._payload, m = l._init;
            try {
              return ys(m(d), t, o);
            } catch {
            }
          }
        }
      return "";
    }
    function jt(e) {
      switch (e._debugOwner && e._debugOwner.type, e._debugSource, e.tag) {
        case D:
          return Ka(e.type);
        case U:
          return Ka("Lazy");
        case V:
          return Ka("Suspense");
        case he:
          return Ka("SuspenseList");
        case b:
        case w:
        case P:
          return Ko(e.type);
        case z:
          return Ko(e.type.render);
        case E:
          return Dp(e.type);
        default:
          return "";
      }
    }
    function Np(e) {
      try {
        var t = "", o = e;
        do
          t += jt(o), o = o.return;
        while (o);
        return t;
      } catch (l) {
        return `
Error generating stack: ` + l.message + `
` + l.stack;
      }
    }
    function Vm(e, t, o) {
      var l = e.displayName;
      if (l)
        return l;
      var d = t.displayName || t.name || "";
      return d !== "" ? o + "(" + d + ")" : o;
    }
    function Kc(e) {
      return e.displayName || "Context";
    }
    function Ut(e) {
      if (e == null)
        return null;
      if (typeof e.tag == "number" && h("Received an unexpected object in getComponentNameFromType(). This is likely a bug in React. Please file an issue."), typeof e == "function")
        return e.displayName || e.name || null;
      if (typeof e == "string")
        return e;
      switch (e) {
        case ki:
          return "Fragment";
        case Qa:
          return "Portal";
        case hs:
          return "Profiler";
        case Zu:
          return "StrictMode";
        case di:
          return "Suspense";
        case ms:
          return "SuspenseList";
      }
      if (typeof e == "object")
        switch (e.$$typeof) {
          case Tp:
            var t = e;
            return Kc(t) + ".Consumer";
          case wp:
            var o = e;
            return Kc(o._context) + ".Provider";
          case Yu:
            return Vm(e, e.render, "ForwardRef");
          case $u:
            var l = e.displayName || null;
            return l !== null ? l : Ut(e.type) || "Memo";
          case Er: {
            var d = e, m = d._payload, y = d._init;
            try {
              return Ut(y(m));
            } catch {
              return null;
            }
          }
        }
      return null;
    }
    function Pm(e, t, o) {
      var l = t.displayName || t.name || "";
      return e.displayName || (l !== "" ? o + "(" + l + ")" : o);
    }
    function kp(e) {
      return e.displayName || "Context";
    }
    function bt(e) {
      var t = e.tag, o = e.type;
      switch (t) {
        case We:
          return "Cache";
        case B:
          var l = o;
          return kp(l) + ".Consumer";
        case Y:
          var d = o;
          return kp(d._context) + ".Provider";
        case Se:
          return "DehydratedFragment";
        case z:
          return Pm(o, o.render, "ForwardRef");
        case M:
          return "Fragment";
        case D:
          return o;
        case A:
          return "Portal";
        case R:
          return "Root";
        case L:
          return "Text";
        case U:
          return Ut(o);
        case k:
          return o === Zu ? "StrictMode" : "Mode";
        case ye:
          return "Offscreen";
        case Z:
          return "Profiler";
        case xe:
          return "Scope";
        case V:
          return "Suspense";
        case he:
          return "SuspenseList";
        case Ee:
          return "TracingMarker";
        case E:
        case b:
        case se:
        case w:
        case Q:
        case P:
          if (typeof o == "function")
            return o.displayName || o.name || null;
          if (typeof o == "string")
            return o;
          break;
      }
      return null;
    }
    var bs = u.ReactDebugCurrentFrame, On = null, Ca = !1;
    function Sa() {
      {
        if (On === null)
          return null;
        var e = On._debugOwner;
        if (e !== null && typeof e < "u")
          return bt(e);
      }
      return null;
    }
    function Cs() {
      return On === null ? "" : Np(On);
    }
    function Vn() {
      bs.getCurrentStack = null, On = null, Ca = !1;
    }
    function Zt(e) {
      bs.getCurrentStack = e === null ? null : Cs, On = e, Ca = !1;
    }
    function Wb() {
      return On;
    }
    function qa(e) {
      Ca = e;
    }
    function vr(e) {
      return "" + e;
    }
    function po(e) {
      switch (typeof e) {
        case "boolean":
        case "number":
        case "string":
        case "undefined":
          return e;
        case "object":
          return Fr(e), e;
        default:
          return "";
      }
    }
    var Hm = {
      button: !0,
      checkbox: !0,
      image: !0,
      hidden: !0,
      radio: !0,
      reset: !0,
      submit: !0
    };
    function Ju(e, t) {
      Hm[t.type] || t.onChange || t.onInput || t.readOnly || t.disabled || t.value == null || h("You provided a `value` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultValue`. Otherwise, set either `onChange` or `readOnly`."), t.onChange || t.readOnly || t.disabled || t.checked == null || h("You provided a `checked` prop to a form field without an `onChange` handler. This will render a read-only field. If the field should be mutable use `defaultChecked`. Otherwise, set either `onChange` or `readOnly`.");
    }
    function Mp(e) {
      var t = e.type, o = e.nodeName;
      return o && o.toLowerCase() === "input" && (t === "checkbox" || t === "radio");
    }
    function zm(e) {
      return e._valueTracker;
    }
    function Ss(e) {
      e._valueTracker = null;
    }
    function Es(e) {
      var t = "";
      return e && (Mp(e) ? t = e.checked ? "true" : "false" : t = e.value), t;
    }
    function Ku(e) {
      var t = Mp(e) ? "checked" : "value", o = Object.getOwnPropertyDescriptor(e.constructor.prototype, t);
      Fr(e[t]);
      var l = "" + e[t];
      if (!(e.hasOwnProperty(t) || typeof o > "u" || typeof o.get != "function" || typeof o.set != "function")) {
        var d = o.get, m = o.set;
        Object.defineProperty(e, t, {
          configurable: !0,
          get: function() {
            return d.call(this);
          },
          set: function(x) {
            Fr(x), l = "" + x, m.call(this, x);
          }
        }), Object.defineProperty(e, t, {
          enumerable: o.enumerable
        });
        var y = {
          getValue: function() {
            return l;
          },
          setValue: function(x) {
            Fr(x), l = "" + x;
          },
          stopTracking: function() {
            Ss(e), delete e[t];
          }
        };
        return y;
      }
    }
    function qo(e) {
      zm(e) || (e._valueTracker = Ku(e));
    }
    function Gm(e) {
      if (!e)
        return !1;
      var t = zm(e);
      if (!t)
        return !0;
      var o = t.getValue(), l = Es(e);
      return l !== o ? (t.setValue(l), !0) : !1;
    }
    function qc(e) {
      if (e = e || (typeof document < "u" ? document : void 0), typeof e > "u")
        return null;
      try {
        return e.activeElement || e.body;
      } catch {
        return e.body;
      }
    }
    var ef = !1, xs = !1, tf = !1, Lp = !1;
    function Li(e) {
      var t = e.type === "checkbox" || e.type === "radio";
      return t ? e.checked != null : e.value != null;
    }
    function ws(e, t) {
      var o = e, l = t.checked, d = Tt({}, t, {
        defaultChecked: void 0,
        defaultValue: void 0,
        value: void 0,
        checked: l ?? o._wrapperState.initialChecked
      });
      return d;
    }
    function Ts(e, t) {
      Ju("input", t), t.checked !== void 0 && t.defaultChecked !== void 0 && !xs && (h("%s contains an input of type %s with both checked and defaultChecked props. Input elements must be either controlled or uncontrolled (specify either the checked prop, or the defaultChecked prop, but not both). Decide between using a controlled or uncontrolled input element and remove one of these props. More info: https://reactjs.org/link/controlled-components", Sa() || "A component", t.type), xs = !0), t.value !== void 0 && t.defaultValue !== void 0 && !ef && (h("%s contains an input of type %s with both value and defaultValue props. Input elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled input element and remove one of these props. More info: https://reactjs.org/link/controlled-components", Sa() || "A component", t.type), ef = !0);
      var o = e, l = t.defaultValue == null ? "" : t.defaultValue;
      o._wrapperState = {
        initialChecked: t.checked != null ? t.checked : t.defaultChecked,
        initialValue: po(t.value != null ? t.value : l),
        controlled: Li(t)
      };
    }
    function Fp(e, t) {
      var o = e, l = t.checked;
      l != null && Qo(o, "checked", l, !1);
    }
    function qu(e, t) {
      var o = e;
      {
        var l = Li(t);
        !o._wrapperState.controlled && l && !Lp && (h("A component is changing an uncontrolled input to be controlled. This is likely caused by the value changing from undefined to a defined value, which should not happen. Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components"), Lp = !0), o._wrapperState.controlled && !l && !tf && (h("A component is changing a controlled input to be uncontrolled. This is likely caused by the value changing from a defined to undefined, which should not happen. Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://reactjs.org/link/controlled-components"), tf = !0);
      }
      Fp(e, t);
      var d = po(t.value), m = t.type;
      if (d != null)
        m === "number" ? (d === 0 && o.value === "" || // We explicitly want to coerce to number here if possible.
        // eslint-disable-next-line
        o.value != d) && (o.value = vr(d)) : o.value !== vr(d) && (o.value = vr(d));
      else if (m === "submit" || m === "reset") {
        o.removeAttribute("value");
        return;
      }
      t.hasOwnProperty("value") ? vo(o, t.type, d) : t.hasOwnProperty("defaultValue") && vo(o, t.type, po(t.defaultValue)), t.checked == null && t.defaultChecked != null && (o.defaultChecked = !!t.defaultChecked);
    }
    function Rs(e, t, o) {
      var l = e;
      if (t.hasOwnProperty("value") || t.hasOwnProperty("defaultValue")) {
        var d = t.type, m = d === "submit" || d === "reset";
        if (m && (t.value === void 0 || t.value === null))
          return;
        var y = vr(l._wrapperState.initialValue);
        o || y !== l.value && (l.value = y), l.defaultValue = y;
      }
      var x = l.name;
      x !== "" && (l.name = ""), l.defaultChecked = !l.defaultChecked, l.defaultChecked = !!l._wrapperState.initialChecked, x !== "" && (l.name = x);
    }
    function _m(e, t) {
      var o = e;
      qu(o, t), ra(o, t);
    }
    function ra(e, t) {
      var o = t.name;
      if (t.type === "radio" && o != null) {
        for (var l = e; l.parentNode; )
          l = l.parentNode;
        Ya(o, "name");
        for (var d = l.querySelectorAll("input[name=" + JSON.stringify("" + o) + '][type="radio"]'), m = 0; m < d.length; m++) {
          var y = d[m];
          if (!(y === e || y.form !== e.form)) {
            var x = Ey(y);
            if (!x)
              throw new Error("ReactDOMInput: Mixing React and non-React radio inputs with the same `name` is not supported.");
            Gm(y), qu(y, x);
          }
        }
      }
    }
    function vo(e, t, o) {
      // Focused number inputs synchronize on blur. See ChangeEventPlugin.js
      (t !== "number" || qc(e.ownerDocument) !== e) && (o == null ? e.defaultValue = vr(e._wrapperState.initialValue) : e.defaultValue !== vr(o) && (e.defaultValue = vr(o)));
    }
    var nf = !1, el = !1, jm = !1;
    function rf(e, t) {
      t.value == null && (typeof t.children == "object" && t.children !== null ? a.Children.forEach(t.children, function(o) {
        o != null && (typeof o == "string" || typeof o == "number" || el || (el = !0, h("Cannot infer the option value of complex children. Pass a `value` prop or use a plain string as children to <option>.")));
      }) : t.dangerouslySetInnerHTML != null && (jm || (jm = !0, h("Pass a `value` prop if you set dangerouslyInnerHTML so React knows which value should be selected.")))), t.selected != null && !nf && (h("Use the `defaultValue` or `value` props on <select> instead of setting `selected` on <option>."), nf = !0);
    }
    function Bp(e, t) {
      t.value != null && e.setAttribute("value", vr(po(t.value)));
    }
    var Is = Array.isArray;
    function Wn(e) {
      return Is(e);
    }
    var af;
    af = !1;
    function Um() {
      var e = Sa();
      return e ? `

Check the render method of \`` + e + "`." : "";
    }
    var Wm = ["value", "defaultValue"];
    function Xb(e) {
      {
        Ju("select", e);
        for (var t = 0; t < Wm.length; t++) {
          var o = Wm[t];
          if (e[o] != null) {
            var l = Wn(e[o]);
            e.multiple && !l ? h("The `%s` prop supplied to <select> must be an array if `multiple` is true.%s", o, Um()) : !e.multiple && l && h("The `%s` prop supplied to <select> must be a scalar value if `multiple` is false.%s", o, Um());
          }
        }
      }
    }
    function ho(e, t, o, l) {
      var d = e.options;
      if (t) {
        for (var m = o, y = {}, x = 0; x < m.length; x++)
          y["$" + m[x]] = !0;
        for (var T = 0; T < d.length; T++) {
          var O = y.hasOwnProperty("$" + d[T].value);
          d[T].selected !== O && (d[T].selected = O), O && l && (d[T].defaultSelected = !0);
        }
      } else {
        for (var N = vr(po(o)), j = null, G = 0; G < d.length; G++) {
          if (d[G].value === N) {
            d[G].selected = !0, l && (d[G].defaultSelected = !0);
            return;
          }
          j === null && !d[G].disabled && (j = d[G]);
        }
        j !== null && (j.selected = !0);
      }
    }
    function Vp(e, t) {
      return Tt({}, t, {
        value: void 0
      });
    }
    function Xm(e, t) {
      var o = e;
      Xb(t), o._wrapperState = {
        wasMultiple: !!t.multiple
      }, t.value !== void 0 && t.defaultValue !== void 0 && !af && (h("Select elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled select element and remove one of these props. More info: https://reactjs.org/link/controlled-components"), af = !0);
    }
    function Zb(e, t) {
      var o = e;
      o.multiple = !!t.multiple;
      var l = t.value;
      l != null ? ho(o, !!t.multiple, l, !1) : t.defaultValue != null && ho(o, !!t.multiple, t.defaultValue, !0);
    }
    function Yb(e, t) {
      var o = e, l = o._wrapperState.wasMultiple;
      o._wrapperState.wasMultiple = !!t.multiple;
      var d = t.value;
      d != null ? ho(o, !!t.multiple, d, !1) : l !== !!t.multiple && (t.defaultValue != null ? ho(o, !!t.multiple, t.defaultValue, !0) : ho(o, !!t.multiple, t.multiple ? [] : "", !1));
    }
    function $b(e, t) {
      var o = e, l = t.value;
      l != null && ho(o, !!t.multiple, l, !1);
    }
    var Pp = !1;
    function Hp(e, t) {
      var o = e;
      if (t.dangerouslySetInnerHTML != null)
        throw new Error("`dangerouslySetInnerHTML` does not make sense on <textarea>.");
      var l = Tt({}, t, {
        value: void 0,
        defaultValue: void 0,
        children: vr(o._wrapperState.initialValue)
      });
      return l;
    }
    function Zm(e, t) {
      var o = e;
      Ju("textarea", t), t.value !== void 0 && t.defaultValue !== void 0 && !Pp && (h("%s contains a textarea with both value and defaultValue props. Textarea elements must be either controlled or uncontrolled (specify either the value prop, or the defaultValue prop, but not both). Decide between using a controlled or uncontrolled textarea and remove one of these props. More info: https://reactjs.org/link/controlled-components", Sa() || "A component"), Pp = !0);
      var l = t.value;
      if (l == null) {
        var d = t.children, m = t.defaultValue;
        if (d != null) {
          h("Use the `defaultValue` or `value` props instead of setting children on <textarea>.");
          {
            if (m != null)
              throw new Error("If you supply `defaultValue` on a <textarea>, do not pass children.");
            if (Wn(d)) {
              if (d.length > 1)
                throw new Error("<textarea> can only have at most one child.");
              d = d[0];
            }
            m = d;
          }
        }
        m == null && (m = ""), l = m;
      }
      o._wrapperState = {
        initialValue: po(l)
      };
    }
    function Ym(e, t) {
      var o = e, l = po(t.value), d = po(t.defaultValue);
      if (l != null) {
        var m = vr(l);
        m !== o.value && (o.value = m), t.defaultValue == null && o.defaultValue !== m && (o.defaultValue = m);
      }
      d != null && (o.defaultValue = vr(d));
    }
    function $m(e, t) {
      var o = e, l = o.textContent;
      l === o._wrapperState.initialValue && l !== "" && l !== null && (o.value = l);
    }
    function zp(e, t) {
      Ym(e, t);
    }
    var Fi = "http://www.w3.org/1999/xhtml", Qb = "http://www.w3.org/1998/Math/MathML", Gp = "http://www.w3.org/2000/svg";
    function of(e) {
      switch (e) {
        case "svg":
          return Gp;
        case "math":
          return Qb;
        default:
          return Fi;
      }
    }
    function _p(e, t) {
      return e == null || e === Fi ? of(t) : e === Gp && t === "foreignObject" ? Fi : e;
    }
    var Jb = function(e) {
      return typeof MSApp < "u" && MSApp.execUnsafeLocalFunction ? function(t, o, l, d) {
        MSApp.execUnsafeLocalFunction(function() {
          return e(t, o, l, d);
        });
      } : e;
    }, uf, Qm = Jb(function(e, t) {
      if (e.namespaceURI === Gp && !("innerHTML" in e)) {
        uf = uf || document.createElement("div"), uf.innerHTML = "<svg>" + t.valueOf().toString() + "</svg>";
        for (var o = uf.firstChild; e.firstChild; )
          e.removeChild(e.firstChild);
        for (; o.firstChild; )
          e.appendChild(o.firstChild);
        return;
      }
      e.innerHTML = t;
    }), Vr = 1, Bi = 3, Dn = 8, ei = 9, eu = 11, lf = function(e, t) {
      if (t) {
        var o = e.firstChild;
        if (o && o === e.lastChild && o.nodeType === Bi) {
          o.nodeValue = t;
          return;
        }
      }
      e.textContent = t;
    }, Jm = {
      animation: ["animationDelay", "animationDirection", "animationDuration", "animationFillMode", "animationIterationCount", "animationName", "animationPlayState", "animationTimingFunction"],
      background: ["backgroundAttachment", "backgroundClip", "backgroundColor", "backgroundImage", "backgroundOrigin", "backgroundPositionX", "backgroundPositionY", "backgroundRepeat", "backgroundSize"],
      backgroundPosition: ["backgroundPositionX", "backgroundPositionY"],
      border: ["borderBottomColor", "borderBottomStyle", "borderBottomWidth", "borderImageOutset", "borderImageRepeat", "borderImageSlice", "borderImageSource", "borderImageWidth", "borderLeftColor", "borderLeftStyle", "borderLeftWidth", "borderRightColor", "borderRightStyle", "borderRightWidth", "borderTopColor", "borderTopStyle", "borderTopWidth"],
      borderBlockEnd: ["borderBlockEndColor", "borderBlockEndStyle", "borderBlockEndWidth"],
      borderBlockStart: ["borderBlockStartColor", "borderBlockStartStyle", "borderBlockStartWidth"],
      borderBottom: ["borderBottomColor", "borderBottomStyle", "borderBottomWidth"],
      borderColor: ["borderBottomColor", "borderLeftColor", "borderRightColor", "borderTopColor"],
      borderImage: ["borderImageOutset", "borderImageRepeat", "borderImageSlice", "borderImageSource", "borderImageWidth"],
      borderInlineEnd: ["borderInlineEndColor", "borderInlineEndStyle", "borderInlineEndWidth"],
      borderInlineStart: ["borderInlineStartColor", "borderInlineStartStyle", "borderInlineStartWidth"],
      borderLeft: ["borderLeftColor", "borderLeftStyle", "borderLeftWidth"],
      borderRadius: ["borderBottomLeftRadius", "borderBottomRightRadius", "borderTopLeftRadius", "borderTopRightRadius"],
      borderRight: ["borderRightColor", "borderRightStyle", "borderRightWidth"],
      borderStyle: ["borderBottomStyle", "borderLeftStyle", "borderRightStyle", "borderTopStyle"],
      borderTop: ["borderTopColor", "borderTopStyle", "borderTopWidth"],
      borderWidth: ["borderBottomWidth", "borderLeftWidth", "borderRightWidth", "borderTopWidth"],
      columnRule: ["columnRuleColor", "columnRuleStyle", "columnRuleWidth"],
      columns: ["columnCount", "columnWidth"],
      flex: ["flexBasis", "flexGrow", "flexShrink"],
      flexFlow: ["flexDirection", "flexWrap"],
      font: ["fontFamily", "fontFeatureSettings", "fontKerning", "fontLanguageOverride", "fontSize", "fontSizeAdjust", "fontStretch", "fontStyle", "fontVariant", "fontVariantAlternates", "fontVariantCaps", "fontVariantEastAsian", "fontVariantLigatures", "fontVariantNumeric", "fontVariantPosition", "fontWeight", "lineHeight"],
      fontVariant: ["fontVariantAlternates", "fontVariantCaps", "fontVariantEastAsian", "fontVariantLigatures", "fontVariantNumeric", "fontVariantPosition"],
      gap: ["columnGap", "rowGap"],
      grid: ["gridAutoColumns", "gridAutoFlow", "gridAutoRows", "gridTemplateAreas", "gridTemplateColumns", "gridTemplateRows"],
      gridArea: ["gridColumnEnd", "gridColumnStart", "gridRowEnd", "gridRowStart"],
      gridColumn: ["gridColumnEnd", "gridColumnStart"],
      gridColumnGap: ["columnGap"],
      gridGap: ["columnGap", "rowGap"],
      gridRow: ["gridRowEnd", "gridRowStart"],
      gridRowGap: ["rowGap"],
      gridTemplate: ["gridTemplateAreas", "gridTemplateColumns", "gridTemplateRows"],
      listStyle: ["listStyleImage", "listStylePosition", "listStyleType"],
      margin: ["marginBottom", "marginLeft", "marginRight", "marginTop"],
      marker: ["markerEnd", "markerMid", "markerStart"],
      mask: ["maskClip", "maskComposite", "maskImage", "maskMode", "maskOrigin", "maskPositionX", "maskPositionY", "maskRepeat", "maskSize"],
      maskPosition: ["maskPositionX", "maskPositionY"],
      outline: ["outlineColor", "outlineStyle", "outlineWidth"],
      overflow: ["overflowX", "overflowY"],
      padding: ["paddingBottom", "paddingLeft", "paddingRight", "paddingTop"],
      placeContent: ["alignContent", "justifyContent"],
      placeItems: ["alignItems", "justifyItems"],
      placeSelf: ["alignSelf", "justifySelf"],
      textDecoration: ["textDecorationColor", "textDecorationLine", "textDecorationStyle"],
      textEmphasis: ["textEmphasisColor", "textEmphasisStyle"],
      transition: ["transitionDelay", "transitionDuration", "transitionProperty", "transitionTimingFunction"],
      wordWrap: ["overflowWrap"]
    }, tl = {
      animationIterationCount: !0,
      aspectRatio: !0,
      borderImageOutset: !0,
      borderImageSlice: !0,
      borderImageWidth: !0,
      boxFlex: !0,
      boxFlexGroup: !0,
      boxOrdinalGroup: !0,
      columnCount: !0,
      columns: !0,
      flex: !0,
      flexGrow: !0,
      flexPositive: !0,
      flexShrink: !0,
      flexNegative: !0,
      flexOrder: !0,
      gridArea: !0,
      gridRow: !0,
      gridRowEnd: !0,
      gridRowSpan: !0,
      gridRowStart: !0,
      gridColumn: !0,
      gridColumnEnd: !0,
      gridColumnSpan: !0,
      gridColumnStart: !0,
      fontWeight: !0,
      lineClamp: !0,
      lineHeight: !0,
      opacity: !0,
      order: !0,
      orphans: !0,
      tabSize: !0,
      widows: !0,
      zIndex: !0,
      zoom: !0,
      // SVG-related properties
      fillOpacity: !0,
      floodOpacity: !0,
      stopOpacity: !0,
      strokeDasharray: !0,
      strokeDashoffset: !0,
      strokeMiterlimit: !0,
      strokeOpacity: !0,
      strokeWidth: !0
    };
    function Km(e, t) {
      return e + t.charAt(0).toUpperCase() + t.substring(1);
    }
    var qm = ["Webkit", "ms", "Moz", "O"];
    Object.keys(tl).forEach(function(e) {
      qm.forEach(function(t) {
        tl[Km(t, e)] = tl[e];
      });
    });
    function sf(e, t, o) {
      var l = t == null || typeof t == "boolean" || t === "";
      return l ? "" : !o && typeof t == "number" && t !== 0 && !(tl.hasOwnProperty(e) && tl[e]) ? t + "px" : (ta(t, e), ("" + t).trim());
    }
    var nl = /([A-Z])/g, Kb = /^ms-/;
    function qb(e) {
      return e.replace(nl, "-$1").toLowerCase().replace(Kb, "-ms-");
    }
    var eg = function() {
    };
    {
      var tg = /^(?:webkit|moz|o)[A-Z]/, ng = /^-ms-/, As = /-(.)/g, rl = /;\s*$/, al = {}, il = {}, rg = !1, jp = !1, Up = function(e) {
        return e.replace(As, function(t, o) {
          return o.toUpperCase();
        });
      }, Wp = function(e) {
        al.hasOwnProperty(e) && al[e] || (al[e] = !0, h(
          "Unsupported style property %s. Did you mean %s?",
          e,
          // As Andi Smith suggests
          // (http://www.andismith.com/blog/2012/02/modernizr-prefixed/), an `-ms` prefix
          // is converted to lowercase `ms`.
          Up(e.replace(ng, "ms-"))
        ));
      }, ag = function(e) {
        al.hasOwnProperty(e) && al[e] || (al[e] = !0, h("Unsupported vendor-prefixed style property %s. Did you mean %s?", e, e.charAt(0).toUpperCase() + e.slice(1)));
      }, ig = function(e, t) {
        il.hasOwnProperty(t) && il[t] || (il[t] = !0, h(`Style property values shouldn't contain a semicolon. Try "%s: %s" instead.`, e, t.replace(rl, "")));
      }, og = function(e, t) {
        rg || (rg = !0, h("`NaN` is an invalid value for the `%s` css style property.", e));
      }, eC = function(e, t) {
        jp || (jp = !0, h("`Infinity` is an invalid value for the `%s` css style property.", e));
      };
      eg = function(e, t) {
        e.indexOf("-") > -1 ? Wp(e) : tg.test(e) ? ag(e) : rl.test(t) && ig(e, t), typeof t == "number" && (isNaN(t) ? og(e, t) : isFinite(t) || eC(e, t));
      };
    }
    var tC = eg;
    function nC(e) {
      {
        var t = "", o = "";
        for (var l in e)
          if (e.hasOwnProperty(l)) {
            var d = e[l];
            if (d != null) {
              var m = l.indexOf("--") === 0;
              t += o + (m ? l : qb(l)) + ":", t += sf(l, d, m), o = ";";
            }
          }
        return t || null;
      }
    }
    function ug(e, t) {
      var o = e.style;
      for (var l in t)
        if (t.hasOwnProperty(l)) {
          var d = l.indexOf("--") === 0;
          d || tC(l, t[l]);
          var m = sf(l, t[l], d);
          l === "float" && (l = "cssFloat"), d ? o.setProperty(l, m) : o[l] = m;
        }
    }
    function rC(e) {
      return e == null || typeof e == "boolean" || e === "";
    }
    function Ea(e) {
      var t = {};
      for (var o in e)
        for (var l = Jm[o] || [o], d = 0; d < l.length; d++)
          t[l[d]] = o;
      return t;
    }
    function Os(e, t) {
      {
        if (!t)
          return;
        var o = Ea(e), l = Ea(t), d = {};
        for (var m in o) {
          var y = o[m], x = l[m];
          if (x && y !== x) {
            var T = y + "," + x;
            if (d[T])
              continue;
            d[T] = !0, h("%s a style property during rerender (%s) when a conflicting property is set (%s) can lead to styling bugs. To avoid this, don't mix shorthand and non-shorthand properties for the same value; instead, replace the shorthand with separate values.", rC(e[y]) ? "Removing" : "Updating", y, x);
          }
        }
      }
    }
    var lg = {
      area: !0,
      base: !0,
      br: !0,
      col: !0,
      embed: !0,
      hr: !0,
      img: !0,
      input: !0,
      keygen: !0,
      link: !0,
      meta: !0,
      param: !0,
      source: !0,
      track: !0,
      wbr: !0
      // NOTE: menuitem's close tag should be omitted, but that causes problems.
    }, sg = Tt({
      menuitem: !0
    }, lg), cg = "__html";
    function cf(e, t) {
      if (t) {
        if (sg[e] && (t.children != null || t.dangerouslySetInnerHTML != null))
          throw new Error(e + " is a void element tag and must neither have `children` nor use `dangerouslySetInnerHTML`.");
        if (t.dangerouslySetInnerHTML != null) {
          if (t.children != null)
            throw new Error("Can only set one of `children` or `props.dangerouslySetInnerHTML`.");
          if (typeof t.dangerouslySetInnerHTML != "object" || !(cg in t.dangerouslySetInnerHTML))
            throw new Error("`props.dangerouslySetInnerHTML` must be in the form `{__html: ...}`. Please visit https://reactjs.org/link/dangerously-set-inner-html for more information.");
        }
        if (!t.suppressContentEditableWarning && t.contentEditable && t.children != null && h("A component is `contentEditable` and contains `children` managed by React. It is now your responsibility to guarantee that none of those nodes are unexpectedly modified or duplicated. This is probably not intentional."), t.style != null && typeof t.style != "object")
          throw new Error("The `style` prop expects a mapping from style properties to values, not a string. For example, style={{marginRight: spacing + 'em'}} when using JSX.");
      }
    }
    function Vi(e, t) {
      if (e.indexOf("-") === -1)
        return typeof t.is == "string";
      switch (e) {
        case "annotation-xml":
        case "color-profile":
        case "font-face":
        case "font-face-src":
        case "font-face-uri":
        case "font-face-format":
        case "font-face-name":
        case "missing-glyph":
          return !1;
        default:
          return !0;
      }
    }
    var ff = {
      // HTML
      accept: "accept",
      acceptcharset: "acceptCharset",
      "accept-charset": "acceptCharset",
      accesskey: "accessKey",
      action: "action",
      allowfullscreen: "allowFullScreen",
      alt: "alt",
      as: "as",
      async: "async",
      autocapitalize: "autoCapitalize",
      autocomplete: "autoComplete",
      autocorrect: "autoCorrect",
      autofocus: "autoFocus",
      autoplay: "autoPlay",
      autosave: "autoSave",
      capture: "capture",
      cellpadding: "cellPadding",
      cellspacing: "cellSpacing",
      challenge: "challenge",
      charset: "charSet",
      checked: "checked",
      children: "children",
      cite: "cite",
      class: "className",
      classid: "classID",
      classname: "className",
      cols: "cols",
      colspan: "colSpan",
      content: "content",
      contenteditable: "contentEditable",
      contextmenu: "contextMenu",
      controls: "controls",
      controlslist: "controlsList",
      coords: "coords",
      crossorigin: "crossOrigin",
      dangerouslysetinnerhtml: "dangerouslySetInnerHTML",
      data: "data",
      datetime: "dateTime",
      default: "default",
      defaultchecked: "defaultChecked",
      defaultvalue: "defaultValue",
      defer: "defer",
      dir: "dir",
      disabled: "disabled",
      disablepictureinpicture: "disablePictureInPicture",
      disableremoteplayback: "disableRemotePlayback",
      download: "download",
      draggable: "draggable",
      enctype: "encType",
      enterkeyhint: "enterKeyHint",
      for: "htmlFor",
      form: "form",
      formmethod: "formMethod",
      formaction: "formAction",
      formenctype: "formEncType",
      formnovalidate: "formNoValidate",
      formtarget: "formTarget",
      frameborder: "frameBorder",
      headers: "headers",
      height: "height",
      hidden: "hidden",
      high: "high",
      href: "href",
      hreflang: "hrefLang",
      htmlfor: "htmlFor",
      httpequiv: "httpEquiv",
      "http-equiv": "httpEquiv",
      icon: "icon",
      id: "id",
      imagesizes: "imageSizes",
      imagesrcset: "imageSrcSet",
      innerhtml: "innerHTML",
      inputmode: "inputMode",
      integrity: "integrity",
      is: "is",
      itemid: "itemID",
      itemprop: "itemProp",
      itemref: "itemRef",
      itemscope: "itemScope",
      itemtype: "itemType",
      keyparams: "keyParams",
      keytype: "keyType",
      kind: "kind",
      label: "label",
      lang: "lang",
      list: "list",
      loop: "loop",
      low: "low",
      manifest: "manifest",
      marginwidth: "marginWidth",
      marginheight: "marginHeight",
      max: "max",
      maxlength: "maxLength",
      media: "media",
      mediagroup: "mediaGroup",
      method: "method",
      min: "min",
      minlength: "minLength",
      multiple: "multiple",
      muted: "muted",
      name: "name",
      nomodule: "noModule",
      nonce: "nonce",
      novalidate: "noValidate",
      open: "open",
      optimum: "optimum",
      pattern: "pattern",
      placeholder: "placeholder",
      playsinline: "playsInline",
      poster: "poster",
      preload: "preload",
      profile: "profile",
      radiogroup: "radioGroup",
      readonly: "readOnly",
      referrerpolicy: "referrerPolicy",
      rel: "rel",
      required: "required",
      reversed: "reversed",
      role: "role",
      rows: "rows",
      rowspan: "rowSpan",
      sandbox: "sandbox",
      scope: "scope",
      scoped: "scoped",
      scrolling: "scrolling",
      seamless: "seamless",
      selected: "selected",
      shape: "shape",
      size: "size",
      sizes: "sizes",
      span: "span",
      spellcheck: "spellCheck",
      src: "src",
      srcdoc: "srcDoc",
      srclang: "srcLang",
      srcset: "srcSet",
      start: "start",
      step: "step",
      style: "style",
      summary: "summary",
      tabindex: "tabIndex",
      target: "target",
      title: "title",
      type: "type",
      usemap: "useMap",
      value: "value",
      width: "width",
      wmode: "wmode",
      wrap: "wrap",
      // SVG
      about: "about",
      accentheight: "accentHeight",
      "accent-height": "accentHeight",
      accumulate: "accumulate",
      additive: "additive",
      alignmentbaseline: "alignmentBaseline",
      "alignment-baseline": "alignmentBaseline",
      allowreorder: "allowReorder",
      alphabetic: "alphabetic",
      amplitude: "amplitude",
      arabicform: "arabicForm",
      "arabic-form": "arabicForm",
      ascent: "ascent",
      attributename: "attributeName",
      attributetype: "attributeType",
      autoreverse: "autoReverse",
      azimuth: "azimuth",
      basefrequency: "baseFrequency",
      baselineshift: "baselineShift",
      "baseline-shift": "baselineShift",
      baseprofile: "baseProfile",
      bbox: "bbox",
      begin: "begin",
      bias: "bias",
      by: "by",
      calcmode: "calcMode",
      capheight: "capHeight",
      "cap-height": "capHeight",
      clip: "clip",
      clippath: "clipPath",
      "clip-path": "clipPath",
      clippathunits: "clipPathUnits",
      cliprule: "clipRule",
      "clip-rule": "clipRule",
      color: "color",
      colorinterpolation: "colorInterpolation",
      "color-interpolation": "colorInterpolation",
      colorinterpolationfilters: "colorInterpolationFilters",
      "color-interpolation-filters": "colorInterpolationFilters",
      colorprofile: "colorProfile",
      "color-profile": "colorProfile",
      colorrendering: "colorRendering",
      "color-rendering": "colorRendering",
      contentscripttype: "contentScriptType",
      contentstyletype: "contentStyleType",
      cursor: "cursor",
      cx: "cx",
      cy: "cy",
      d: "d",
      datatype: "datatype",
      decelerate: "decelerate",
      descent: "descent",
      diffuseconstant: "diffuseConstant",
      direction: "direction",
      display: "display",
      divisor: "divisor",
      dominantbaseline: "dominantBaseline",
      "dominant-baseline": "dominantBaseline",
      dur: "dur",
      dx: "dx",
      dy: "dy",
      edgemode: "edgeMode",
      elevation: "elevation",
      enablebackground: "enableBackground",
      "enable-background": "enableBackground",
      end: "end",
      exponent: "exponent",
      externalresourcesrequired: "externalResourcesRequired",
      fill: "fill",
      fillopacity: "fillOpacity",
      "fill-opacity": "fillOpacity",
      fillrule: "fillRule",
      "fill-rule": "fillRule",
      filter: "filter",
      filterres: "filterRes",
      filterunits: "filterUnits",
      floodopacity: "floodOpacity",
      "flood-opacity": "floodOpacity",
      floodcolor: "floodColor",
      "flood-color": "floodColor",
      focusable: "focusable",
      fontfamily: "fontFamily",
      "font-family": "fontFamily",
      fontsize: "fontSize",
      "font-size": "fontSize",
      fontsizeadjust: "fontSizeAdjust",
      "font-size-adjust": "fontSizeAdjust",
      fontstretch: "fontStretch",
      "font-stretch": "fontStretch",
      fontstyle: "fontStyle",
      "font-style": "fontStyle",
      fontvariant: "fontVariant",
      "font-variant": "fontVariant",
      fontweight: "fontWeight",
      "font-weight": "fontWeight",
      format: "format",
      from: "from",
      fx: "fx",
      fy: "fy",
      g1: "g1",
      g2: "g2",
      glyphname: "glyphName",
      "glyph-name": "glyphName",
      glyphorientationhorizontal: "glyphOrientationHorizontal",
      "glyph-orientation-horizontal": "glyphOrientationHorizontal",
      glyphorientationvertical: "glyphOrientationVertical",
      "glyph-orientation-vertical": "glyphOrientationVertical",
      glyphref: "glyphRef",
      gradienttransform: "gradientTransform",
      gradientunits: "gradientUnits",
      hanging: "hanging",
      horizadvx: "horizAdvX",
      "horiz-adv-x": "horizAdvX",
      horizoriginx: "horizOriginX",
      "horiz-origin-x": "horizOriginX",
      ideographic: "ideographic",
      imagerendering: "imageRendering",
      "image-rendering": "imageRendering",
      in2: "in2",
      in: "in",
      inlist: "inlist",
      intercept: "intercept",
      k1: "k1",
      k2: "k2",
      k3: "k3",
      k4: "k4",
      k: "k",
      kernelmatrix: "kernelMatrix",
      kernelunitlength: "kernelUnitLength",
      kerning: "kerning",
      keypoints: "keyPoints",
      keysplines: "keySplines",
      keytimes: "keyTimes",
      lengthadjust: "lengthAdjust",
      letterspacing: "letterSpacing",
      "letter-spacing": "letterSpacing",
      lightingcolor: "lightingColor",
      "lighting-color": "lightingColor",
      limitingconeangle: "limitingConeAngle",
      local: "local",
      markerend: "markerEnd",
      "marker-end": "markerEnd",
      markerheight: "markerHeight",
      markermid: "markerMid",
      "marker-mid": "markerMid",
      markerstart: "markerStart",
      "marker-start": "markerStart",
      markerunits: "markerUnits",
      markerwidth: "markerWidth",
      mask: "mask",
      maskcontentunits: "maskContentUnits",
      maskunits: "maskUnits",
      mathematical: "mathematical",
      mode: "mode",
      numoctaves: "numOctaves",
      offset: "offset",
      opacity: "opacity",
      operator: "operator",
      order: "order",
      orient: "orient",
      orientation: "orientation",
      origin: "origin",
      overflow: "overflow",
      overlineposition: "overlinePosition",
      "overline-position": "overlinePosition",
      overlinethickness: "overlineThickness",
      "overline-thickness": "overlineThickness",
      paintorder: "paintOrder",
      "paint-order": "paintOrder",
      panose1: "panose1",
      "panose-1": "panose1",
      pathlength: "pathLength",
      patterncontentunits: "patternContentUnits",
      patterntransform: "patternTransform",
      patternunits: "patternUnits",
      pointerevents: "pointerEvents",
      "pointer-events": "pointerEvents",
      points: "points",
      pointsatx: "pointsAtX",
      pointsaty: "pointsAtY",
      pointsatz: "pointsAtZ",
      prefix: "prefix",
      preservealpha: "preserveAlpha",
      preserveaspectratio: "preserveAspectRatio",
      primitiveunits: "primitiveUnits",
      property: "property",
      r: "r",
      radius: "radius",
      refx: "refX",
      refy: "refY",
      renderingintent: "renderingIntent",
      "rendering-intent": "renderingIntent",
      repeatcount: "repeatCount",
      repeatdur: "repeatDur",
      requiredextensions: "requiredExtensions",
      requiredfeatures: "requiredFeatures",
      resource: "resource",
      restart: "restart",
      result: "result",
      results: "results",
      rotate: "rotate",
      rx: "rx",
      ry: "ry",
      scale: "scale",
      security: "security",
      seed: "seed",
      shaperendering: "shapeRendering",
      "shape-rendering": "shapeRendering",
      slope: "slope",
      spacing: "spacing",
      specularconstant: "specularConstant",
      specularexponent: "specularExponent",
      speed: "speed",
      spreadmethod: "spreadMethod",
      startoffset: "startOffset",
      stddeviation: "stdDeviation",
      stemh: "stemh",
      stemv: "stemv",
      stitchtiles: "stitchTiles",
      stopcolor: "stopColor",
      "stop-color": "stopColor",
      stopopacity: "stopOpacity",
      "stop-opacity": "stopOpacity",
      strikethroughposition: "strikethroughPosition",
      "strikethrough-position": "strikethroughPosition",
      strikethroughthickness: "strikethroughThickness",
      "strikethrough-thickness": "strikethroughThickness",
      string: "string",
      stroke: "stroke",
      strokedasharray: "strokeDasharray",
      "stroke-dasharray": "strokeDasharray",
      strokedashoffset: "strokeDashoffset",
      "stroke-dashoffset": "strokeDashoffset",
      strokelinecap: "strokeLinecap",
      "stroke-linecap": "strokeLinecap",
      strokelinejoin: "strokeLinejoin",
      "stroke-linejoin": "strokeLinejoin",
      strokemiterlimit: "strokeMiterlimit",
      "stroke-miterlimit": "strokeMiterlimit",
      strokewidth: "strokeWidth",
      "stroke-width": "strokeWidth",
      strokeopacity: "strokeOpacity",
      "stroke-opacity": "strokeOpacity",
      suppresscontenteditablewarning: "suppressContentEditableWarning",
      suppresshydrationwarning: "suppressHydrationWarning",
      surfacescale: "surfaceScale",
      systemlanguage: "systemLanguage",
      tablevalues: "tableValues",
      targetx: "targetX",
      targety: "targetY",
      textanchor: "textAnchor",
      "text-anchor": "textAnchor",
      textdecoration: "textDecoration",
      "text-decoration": "textDecoration",
      textlength: "textLength",
      textrendering: "textRendering",
      "text-rendering": "textRendering",
      to: "to",
      transform: "transform",
      typeof: "typeof",
      u1: "u1",
      u2: "u2",
      underlineposition: "underlinePosition",
      "underline-position": "underlinePosition",
      underlinethickness: "underlineThickness",
      "underline-thickness": "underlineThickness",
      unicode: "unicode",
      unicodebidi: "unicodeBidi",
      "unicode-bidi": "unicodeBidi",
      unicoderange: "unicodeRange",
      "unicode-range": "unicodeRange",
      unitsperem: "unitsPerEm",
      "units-per-em": "unitsPerEm",
      unselectable: "unselectable",
      valphabetic: "vAlphabetic",
      "v-alphabetic": "vAlphabetic",
      values: "values",
      vectoreffect: "vectorEffect",
      "vector-effect": "vectorEffect",
      version: "version",
      vertadvy: "vertAdvY",
      "vert-adv-y": "vertAdvY",
      vertoriginx: "vertOriginX",
      "vert-origin-x": "vertOriginX",
      vertoriginy: "vertOriginY",
      "vert-origin-y": "vertOriginY",
      vhanging: "vHanging",
      "v-hanging": "vHanging",
      videographic: "vIdeographic",
      "v-ideographic": "vIdeographic",
      viewbox: "viewBox",
      viewtarget: "viewTarget",
      visibility: "visibility",
      vmathematical: "vMathematical",
      "v-mathematical": "vMathematical",
      vocab: "vocab",
      widths: "widths",
      wordspacing: "wordSpacing",
      "word-spacing": "wordSpacing",
      writingmode: "writingMode",
      "writing-mode": "writingMode",
      x1: "x1",
      x2: "x2",
      x: "x",
      xchannelselector: "xChannelSelector",
      xheight: "xHeight",
      "x-height": "xHeight",
      xlinkactuate: "xlinkActuate",
      "xlink:actuate": "xlinkActuate",
      xlinkarcrole: "xlinkArcrole",
      "xlink:arcrole": "xlinkArcrole",
      xlinkhref: "xlinkHref",
      "xlink:href": "xlinkHref",
      xlinkrole: "xlinkRole",
      "xlink:role": "xlinkRole",
      xlinkshow: "xlinkShow",
      "xlink:show": "xlinkShow",
      xlinktitle: "xlinkTitle",
      "xlink:title": "xlinkTitle",
      xlinktype: "xlinkType",
      "xlink:type": "xlinkType",
      xmlbase: "xmlBase",
      "xml:base": "xmlBase",
      xmllang: "xmlLang",
      "xml:lang": "xmlLang",
      xmlns: "xmlns",
      "xml:space": "xmlSpace",
      xmlnsxlink: "xmlnsXlink",
      "xmlns:xlink": "xmlnsXlink",
      xmlspace: "xmlSpace",
      y1: "y1",
      y2: "y2",
      y: "y",
      ychannelselector: "yChannelSelector",
      z: "z",
      zoomandpan: "zoomAndPan"
    }, fg = {
      "aria-current": 0,
      // state
      "aria-description": 0,
      "aria-details": 0,
      "aria-disabled": 0,
      // state
      "aria-hidden": 0,
      // state
      "aria-invalid": 0,
      // state
      "aria-keyshortcuts": 0,
      "aria-label": 0,
      "aria-roledescription": 0,
      // Widget Attributes
      "aria-autocomplete": 0,
      "aria-checked": 0,
      "aria-expanded": 0,
      "aria-haspopup": 0,
      "aria-level": 0,
      "aria-modal": 0,
      "aria-multiline": 0,
      "aria-multiselectable": 0,
      "aria-orientation": 0,
      "aria-placeholder": 0,
      "aria-pressed": 0,
      "aria-readonly": 0,
      "aria-required": 0,
      "aria-selected": 0,
      "aria-sort": 0,
      "aria-valuemax": 0,
      "aria-valuemin": 0,
      "aria-valuenow": 0,
      "aria-valuetext": 0,
      // Live Region Attributes
      "aria-atomic": 0,
      "aria-busy": 0,
      "aria-live": 0,
      "aria-relevant": 0,
      // Drag-and-Drop Attributes
      "aria-dropeffect": 0,
      "aria-grabbed": 0,
      // Relationship Attributes
      "aria-activedescendant": 0,
      "aria-colcount": 0,
      "aria-colindex": 0,
      "aria-colspan": 0,
      "aria-controls": 0,
      "aria-describedby": 0,
      "aria-errormessage": 0,
      "aria-flowto": 0,
      "aria-labelledby": 0,
      "aria-owns": 0,
      "aria-posinset": 0,
      "aria-rowcount": 0,
      "aria-rowindex": 0,
      "aria-rowspan": 0,
      "aria-setsize": 0
    }, ti = {}, Xp = new RegExp("^(aria)-[" + rt + "]*$"), Ds = new RegExp("^(aria)[A-Z][" + rt + "]*$");
    function Zp(e, t) {
      {
        if (_t.call(ti, t) && ti[t])
          return !0;
        if (Ds.test(t)) {
          var o = "aria-" + t.slice(4).toLowerCase(), l = fg.hasOwnProperty(o) ? o : null;
          if (l == null)
            return h("Invalid ARIA attribute `%s`. ARIA attributes follow the pattern aria-* and must be lowercase.", t), ti[t] = !0, !0;
          if (t !== l)
            return h("Invalid ARIA attribute `%s`. Did you mean `%s`?", t, l), ti[t] = !0, !0;
        }
        if (Xp.test(t)) {
          var d = t.toLowerCase(), m = fg.hasOwnProperty(d) ? d : null;
          if (m == null)
            return ti[t] = !0, !1;
          if (t !== m)
            return h("Unknown ARIA attribute `%s`. Did you mean `%s`?", t, m), ti[t] = !0, !0;
        }
      }
      return !0;
    }
    function dg(e, t) {
      {
        var o = [];
        for (var l in t) {
          var d = Zp(e, l);
          d || o.push(l);
        }
        var m = o.map(function(y) {
          return "`" + y + "`";
        }).join(", ");
        o.length === 1 ? h("Invalid aria prop %s on <%s> tag. For details, see https://reactjs.org/link/invalid-aria-props", m, e) : o.length > 1 && h("Invalid aria props %s on <%s> tag. For details, see https://reactjs.org/link/invalid-aria-props", m, e);
      }
    }
    function df(e, t) {
      Vi(e, t) || dg(e, t);
    }
    var tu = !1;
    function Yp(e, t) {
      {
        if (e !== "input" && e !== "textarea" && e !== "select")
          return;
        t != null && t.value === null && !tu && (tu = !0, e === "select" && t.multiple ? h("`value` prop on `%s` should not be null. Consider using an empty array when `multiple` is set to `true` to clear the component or `undefined` for uncontrolled components.", e) : h("`value` prop on `%s` should not be null. Consider using an empty string to clear the component or `undefined` for uncontrolled components.", e));
      }
    }
    var $p = function() {
    };
    {
      var Xn = {}, Qp = /^on./, pg = /^on[^A-Z]/, vg = new RegExp("^(aria)-[" + rt + "]*$"), hg = new RegExp("^(aria)[A-Z][" + rt + "]*$");
      $p = function(e, t, o, l) {
        if (_t.call(Xn, t) && Xn[t])
          return !0;
        var d = t.toLowerCase();
        if (d === "onfocusin" || d === "onfocusout")
          return h("React uses onFocus and onBlur instead of onFocusIn and onFocusOut. All React events are normalized to bubble, so onFocusIn and onFocusOut are not needed/supported by React."), Xn[t] = !0, !0;
        if (l != null) {
          var m = l.registrationNameDependencies, y = l.possibleRegistrationNames;
          if (m.hasOwnProperty(t))
            return !0;
          var x = y.hasOwnProperty(d) ? y[d] : null;
          if (x != null)
            return h("Invalid event handler property `%s`. Did you mean `%s`?", t, x), Xn[t] = !0, !0;
          if (Qp.test(t))
            return h("Unknown event handler property `%s`. It will be ignored.", t), Xn[t] = !0, !0;
        } else if (Qp.test(t))
          return pg.test(t) && h("Invalid event handler property `%s`. React events use the camelCase naming convention, for example `onClick`.", t), Xn[t] = !0, !0;
        if (vg.test(t) || hg.test(t))
          return !0;
        if (d === "innerhtml")
          return h("Directly setting property `innerHTML` is not permitted. For more information, lookup documentation on `dangerouslySetInnerHTML`."), Xn[t] = !0, !0;
        if (d === "aria")
          return h("The `aria` attribute is reserved for future use in React. Pass individual `aria-` attributes instead."), Xn[t] = !0, !0;
        if (d === "is" && o !== null && o !== void 0 && typeof o != "string")
          return h("Received a `%s` for a string attribute `is`. If this is expected, cast the value to a string.", typeof o), Xn[t] = !0, !0;
        if (typeof o == "number" && isNaN(o))
          return h("Received NaN for the `%s` attribute. If this is expected, cast the value to a string.", t), Xn[t] = !0, !0;
        var T = fi(t), O = T !== null && T.type === ga;
        if (ff.hasOwnProperty(d)) {
          var N = ff[d];
          if (N !== t)
            return h("Invalid DOM property `%s`. Did you mean `%s`?", t, N), Xn[t] = !0, !0;
        } else if (!O && t !== d)
          return h("React does not recognize the `%s` prop on a DOM element. If you intentionally want it to appear in the DOM as a custom attribute, spell it as lowercase `%s` instead. If you accidentally passed it from a parent component, remove it from the DOM element.", t, d), Xn[t] = !0, !0;
        return typeof o == "boolean" && ya(t, o, T, !1) ? (o ? h('Received `%s` for a non-boolean attribute `%s`.\n\nIf you want to write it to the DOM, pass a string instead: %s="%s" or %s={value.toString()}.', o, t, t, o, t) : h('Received `%s` for a non-boolean attribute `%s`.\n\nIf you want to write it to the DOM, pass a string instead: %s="%s" or %s={value.toString()}.\n\nIf you used to conditionally omit it with %s={condition && value}, pass %s={condition ? value : undefined} instead.', o, t, t, o, t, t, t), Xn[t] = !0, !0) : O ? !0 : ya(t, o, T, !1) ? (Xn[t] = !0, !1) : ((o === "false" || o === "true") && T !== null && T.type === tr && (h("Received the string `%s` for the boolean attribute `%s`. %s Did you mean %s={%s}?", o, t, o === "false" ? "The browser will interpret it as a truthy value." : 'Although this works, it will not work as expected if you pass the string "false".', t, o), Xn[t] = !0), !0);
      };
    }
    var mg = function(e, t, o) {
      {
        var l = [];
        for (var d in t) {
          var m = $p(e, d, t[d], o);
          m || l.push(d);
        }
        var y = l.map(function(x) {
          return "`" + x + "`";
        }).join(", ");
        l.length === 1 ? h("Invalid value for prop %s on <%s> tag. Either remove it from the element, or pass a string or number value to keep it in the DOM. For details, see https://reactjs.org/link/attribute-behavior ", y, e) : l.length > 1 && h("Invalid values for props %s on <%s> tag. Either remove them from the element, or pass a string or number value to keep them in the DOM. For details, see https://reactjs.org/link/attribute-behavior ", y, e);
      }
    };
    function gg(e, t, o) {
      Vi(e, t) || mg(e, t, o);
    }
    var Pi = 1, Ns = 2, nu = 4, aC = Pi | Ns | nu, ks = null;
    function Ms(e) {
      ks !== null && h("Expected currently replaying event to be null. This error is likely caused by a bug in React. Please file an issue."), ks = e;
    }
    function iC() {
      ks === null && h("Expected currently replaying event to not be null. This error is likely caused by a bug in React. Please file an issue."), ks = null;
    }
    function yg(e) {
      return e === ks;
    }
    function pf(e) {
      var t = e.target || e.srcElement || window;
      return t.correspondingUseElement && (t = t.correspondingUseElement), t.nodeType === Bi ? t.parentNode : t;
    }
    var qt = null, mo = null, Hi = null;
    function ol(e) {
      var t = Hl(e);
      if (t) {
        if (typeof qt != "function")
          throw new Error("setRestoreImplementation() needs to be called to handle a target for controlled events. This error is likely caused by a bug in React. Please file an issue.");
        var o = t.stateNode;
        if (o) {
          var l = Ey(o);
          qt(t.stateNode, t.type, l);
        }
      }
    }
    function bg(e) {
      qt = e;
    }
    function vf(e) {
      mo ? Hi ? Hi.push(e) : Hi = [e] : mo = e;
    }
    function Ls() {
      return mo !== null || Hi !== null;
    }
    function Fs() {
      if (mo) {
        var e = mo, t = Hi;
        if (mo = null, Hi = null, ol(e), t)
          for (var o = 0; o < t.length; o++)
            ol(t[o]);
      }
    }
    var ru = function(e, t) {
      return e(t);
    }, Jp = function() {
    }, Kp = !1;
    function oC() {
      var e = Ls();
      e && (Jp(), Fs());
    }
    function qp(e, t, o) {
      if (Kp)
        return e(t, o);
      Kp = !0;
      try {
        return ru(e, t, o);
      } finally {
        Kp = !1, oC();
      }
    }
    function hf(e, t, o) {
      ru = e, Jp = o;
    }
    function mf(e) {
      return e === "button" || e === "input" || e === "select" || e === "textarea";
    }
    function ev(e, t, o) {
      switch (e) {
        case "onClick":
        case "onClickCapture":
        case "onDoubleClick":
        case "onDoubleClickCapture":
        case "onMouseDown":
        case "onMouseDownCapture":
        case "onMouseMove":
        case "onMouseMoveCapture":
        case "onMouseUp":
        case "onMouseUpCapture":
        case "onMouseEnter":
          return !!(o.disabled && mf(t));
        default:
          return !1;
      }
    }
    function au(e, t) {
      var o = e.stateNode;
      if (o === null)
        return null;
      var l = Ey(o);
      if (l === null)
        return null;
      var d = l[t];
      if (ev(t, e.type, l))
        return null;
      if (d && typeof d != "function")
        throw new Error("Expected `" + t + "` listener to be a function, instead got a value of `" + typeof d + "` type.");
      return d;
    }
    var Bs = !1;
    if (Ot)
      try {
        var iu = {};
        Object.defineProperty(iu, "passive", {
          get: function() {
            Bs = !0;
          }
        }), window.addEventListener("test", iu, iu), window.removeEventListener("test", iu, iu);
      } catch {
        Bs = !1;
      }
    function Cg(e, t, o, l, d, m, y, x, T) {
      var O = Array.prototype.slice.call(arguments, 3);
      try {
        t.apply(o, O);
      } catch (N) {
        this.onError(N);
      }
    }
    var tv = Cg;
    if (typeof window < "u" && typeof window.dispatchEvent == "function" && typeof document < "u" && typeof document.createEvent == "function") {
      var nv = document.createElement("react");
      tv = function(t, o, l, d, m, y, x, T, O) {
        if (typeof document > "u" || document === null)
          throw new Error("The `document` global was defined when React was initialized, but is not defined anymore. This can happen in a test environment if a component schedules an update from an asynchronous callback, but the test has already finished running. To solve this, you can either unmount the component at the end of your test (and ensure that any asynchronous operations get canceled in `componentWillUnmount`), or you can change the test itself to be asynchronous.");
        var N = document.createEvent("Event"), j = !1, G = !0, te = window.event, re = Object.getOwnPropertyDescriptor(window, "event");
        function ie() {
          nv.removeEventListener(oe, et, !1), typeof window.event < "u" && window.hasOwnProperty("event") && (window.event = te);
        }
        var Ne = Array.prototype.slice.call(arguments, 3);
        function et() {
          j = !0, ie(), o.apply(l, Ne), G = !1;
        }
        var Qe, Nt = !1, It = !1;
        function K(q) {
          if (Qe = q.error, Nt = !0, Qe === null && q.colno === 0 && q.lineno === 0 && (It = !0), q.defaultPrevented && Qe != null && typeof Qe == "object")
            try {
              Qe._suppressLogging = !0;
            } catch {
            }
        }
        var oe = "react-" + (t || "invokeguardedcallback");
        if (window.addEventListener("error", K), nv.addEventListener(oe, et, !1), N.initEvent(oe, !1, !1), nv.dispatchEvent(N), re && Object.defineProperty(window, "event", re), j && G && (Nt ? It && (Qe = new Error("A cross-origin error was thrown. React doesn't have access to the actual error object in development. See https://reactjs.org/link/crossorigin-error for more information.")) : Qe = new Error(`An error was thrown inside one of your components, but React doesn't know what it was. This is likely due to browser flakiness. React does its best to preserve the "Pause on exceptions" behavior of the DevTools, which requires some DEV-mode only tricks. It's possible that these don't work in your browser. Try triggering the error in production mode, or switching to a modern browser. If you suspect that this is actually an issue with React, please file an issue.`), this.onError(Qe)), window.removeEventListener("error", K), !j)
          return ie(), Cg.apply(this, arguments);
      };
    }
    var uC = tv, go = !1, ni = null, Vs = !1, yo = null, pi = {
      onError: function(e) {
        go = !0, ni = e;
      }
    };
    function ou(e, t, o, l, d, m, y, x, T) {
      go = !1, ni = null, uC.apply(pi, arguments);
    }
    function zi(e, t, o, l, d, m, y, x, T) {
      if (ou.apply(this, arguments), go) {
        var O = av();
        Vs || (Vs = !0, yo = O);
      }
    }
    function rv() {
      if (Vs) {
        var e = yo;
        throw Vs = !1, yo = null, e;
      }
    }
    function lC() {
      return go;
    }
    function av() {
      if (go) {
        var e = ni;
        return go = !1, ni = null, e;
      } else
        throw new Error("clearCaughtError was called but no error was captured. This error is likely caused by a bug in React. Please file an issue.");
    }
    function xa(e) {
      return e._reactInternals;
    }
    function Ps(e) {
      return e._reactInternals !== void 0;
    }
    function ul(e, t) {
      e._reactInternals = t;
    }
    var qe = (
      /*                      */
      0
    ), bo = (
      /*                */
      1
    ), nn = (
      /*                    */
      2
    ), St = (
      /*                       */
      4
    ), Pt = (
      /*                */
      16
    ), Ht = (
      /*                 */
      32
    ), hi = (
      /*                     */
      64
    ), dt = (
      /*                   */
      128
    ), gn = (
      /*            */
      256
    ), Pr = (
      /*                          */
      512
    ), wa = (
      /*                     */
      1024
    ), un = (
      /*                      */
      2048
    ), Ta = (
      /*                    */
      4096
    ), Co = (
      /*                   */
      8192
    ), Hs = (
      /*             */
      16384
    ), gf = un | St | hi | Pr | wa | Hs, Sg = (
      /*               */
      32767
    ), aa = (
      /*                   */
      32768
    ), Zn = (
      /*                */
      65536
    ), zs = (
      /* */
      131072
    ), iv = (
      /*                       */
      1048576
    ), ov = (
      /*                    */
      2097152
    ), Hr = (
      /*                 */
      4194304
    ), So = (
      /*                */
      8388608
    ), zr = (
      /*               */
      16777216
    ), uu = (
      /*              */
      33554432
    ), ll = (
      // TODO: Remove Update flag from before mutation phase by re-landing Visibility
      // flag logic (see #20043)
      St | wa | 0
    ), Gr = nn | St | Pt | Ht | Pr | Ta | Co, hr = St | hi | Pr | Co, Ra = un | Pt, nr = Hr | So | ov, Gi = u.ReactCurrentOwner;
    function ia(e) {
      var t = e, o = e;
      if (e.alternate)
        for (; t.return; )
          t = t.return;
      else {
        var l = t;
        do
          t = l, (t.flags & (nn | Ta)) !== qe && (o = t.return), l = t.return;
        while (l);
      }
      return t.tag === R ? o : null;
    }
    function uv(e) {
      if (e.tag === V) {
        var t = e.memoizedState;
        if (t === null) {
          var o = e.alternate;
          o !== null && (t = o.memoizedState);
        }
        if (t !== null)
          return t.dehydrated;
      }
      return null;
    }
    function yf(e) {
      return e.tag === R ? e.stateNode.containerInfo : null;
    }
    function lv(e) {
      return ia(e) === e;
    }
    function oa(e) {
      {
        var t = Gi.current;
        if (t !== null && t.tag === E) {
          var o = t, l = o.stateNode;
          l._warnedAboutRefsInRender || h("%s is accessing isMounted inside its render() function. render() should be a pure function of props and state. It should never access something that requires stale data from the previous render, such as refs. Move this logic to componentDidMount and componentDidUpdate instead.", bt(o) || "A component"), l._warnedAboutRefsInRender = !0;
        }
      }
      var d = xa(e);
      return d ? ia(d) === d : !1;
    }
    function _r(e) {
      if (ia(e) !== e)
        throw new Error("Unable to find node on an unmounted component.");
    }
    function rn(e) {
      var t = e.alternate;
      if (!t) {
        var o = ia(e);
        if (o === null)
          throw new Error("Unable to find node on an unmounted component.");
        return o !== e ? null : e;
      }
      for (var l = e, d = t; ; ) {
        var m = l.return;
        if (m === null)
          break;
        var y = m.alternate;
        if (y === null) {
          var x = m.return;
          if (x !== null) {
            l = d = x;
            continue;
          }
          break;
        }
        if (m.child === y.child) {
          for (var T = m.child; T; ) {
            if (T === l)
              return _r(m), e;
            if (T === d)
              return _r(m), t;
            T = T.sibling;
          }
          throw new Error("Unable to find node on an unmounted component.");
        }
        if (l.return !== d.return)
          l = m, d = y;
        else {
          for (var O = !1, N = m.child; N; ) {
            if (N === l) {
              O = !0, l = m, d = y;
              break;
            }
            if (N === d) {
              O = !0, d = m, l = y;
              break;
            }
            N = N.sibling;
          }
          if (!O) {
            for (N = y.child; N; ) {
              if (N === l) {
                O = !0, l = y, d = m;
                break;
              }
              if (N === d) {
                O = !0, d = y, l = m;
                break;
              }
              N = N.sibling;
            }
            if (!O)
              throw new Error("Child was not found in either parent set. This indicates a bug in React related to the return pointer. Please file an issue.");
          }
        }
        if (l.alternate !== d)
          throw new Error("Return fibers should always be each others' alternates. This error is likely caused by a bug in React. Please file an issue.");
      }
      if (l.tag !== R)
        throw new Error("Unable to find node on an unmounted component.");
      return l.stateNode.current === l ? e : t;
    }
    function Ia(e) {
      var t = rn(e);
      return t !== null ? sv(t) : null;
    }
    function sv(e) {
      if (e.tag === D || e.tag === L)
        return e;
      for (var t = e.child; t !== null; ) {
        var o = sv(t);
        if (o !== null)
          return o;
        t = t.sibling;
      }
      return null;
    }
    function Eg(e) {
      var t = rn(e);
      return t !== null ? bf(t) : null;
    }
    function bf(e) {
      if (e.tag === D || e.tag === L)
        return e;
      for (var t = e.child; t !== null; ) {
        if (t.tag !== A) {
          var o = bf(t);
          if (o !== null)
            return o;
        }
        t = t.sibling;
      }
      return null;
    }
    var Cf = r.unstable_scheduleCallback, xg = r.unstable_cancelCallback, Sf = r.unstable_shouldYield, wg = r.unstable_requestPaint, cn = r.unstable_now, cv = r.unstable_getCurrentPriorityLevel, Ef = r.unstable_ImmediatePriority, lu = r.unstable_UserBlockingPriority, mi = r.unstable_NormalPriority, Tg = r.unstable_LowPriority, xf = r.unstable_IdlePriority, sl = r.unstable_yieldValue, Rg = r.unstable_setDisableYieldValue, _i = null, Pn = null, Te = null, Aa = !1, ua = typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u";
    function fv(e) {
      if (typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ > "u")
        return !1;
      var t = __REACT_DEVTOOLS_GLOBAL_HOOK__;
      if (t.isDisabled)
        return !0;
      if (!t.supportsFiber)
        return h("The installed version of React DevTools is too old and will not work with the current version of React. Please update React DevTools. https://reactjs.org/link/react-devtools"), !0;
      try {
        De && (e = Tt({}, e, {
          getLaneLabelMap: ji,
          injectProfilingHooks: Ig
        })), _i = t.inject(e), Pn = t;
      } catch (o) {
        h("React instrumentation encountered an error: %s.", o);
      }
      return !!t.checkDCE;
    }
    function dv(e, t) {
      if (Pn && typeof Pn.onScheduleFiberRoot == "function")
        try {
          Pn.onScheduleFiberRoot(_i, e, t);
        } catch (o) {
          Aa || (Aa = !0, h("React instrumentation encountered an error: %s", o));
        }
    }
    function cl(e, t) {
      if (Pn && typeof Pn.onCommitFiberRoot == "function")
        try {
          var o = (e.current.flags & dt) === dt;
          if (Oe) {
            var l;
            switch (t) {
              case wn:
                l = Ef;
                break;
              case Wi:
                l = lu;
                break;
              case gi:
                l = mi;
                break;
              case El:
                l = xf;
                break;
              default:
                l = mi;
                break;
            }
            Pn.onCommitFiberRoot(_i, e, l, o);
          }
        } catch (d) {
          Aa || (Aa = !0, h("React instrumentation encountered an error: %s", d));
        }
    }
    function Oa(e) {
      if (Pn && typeof Pn.onPostCommitFiberRoot == "function")
        try {
          Pn.onPostCommitFiberRoot(_i, e);
        } catch (t) {
          Aa || (Aa = !0, h("React instrumentation encountered an error: %s", t));
        }
    }
    function su(e) {
      if (Pn && typeof Pn.onCommitFiberUnmount == "function")
        try {
          Pn.onCommitFiberUnmount(_i, e);
        } catch (t) {
          Aa || (Aa = !0, h("React instrumentation encountered an error: %s", t));
        }
    }
    function Nn(e) {
      if (typeof sl == "function" && (Rg(e), c(e)), Pn && typeof Pn.setStrictMode == "function")
        try {
          Pn.setStrictMode(_i, e);
        } catch (t) {
          Aa || (Aa = !0, h("React instrumentation encountered an error: %s", t));
        }
    }
    function Ig(e) {
      Te = e;
    }
    function ji() {
      {
        for (var e = /* @__PURE__ */ new Map(), t = 1, o = 0; o < Ws; o++) {
          var l = cC(t);
          e.set(t, l), t *= 2;
        }
        return e;
      }
    }
    function Eo(e) {
      Te !== null && typeof Te.markCommitStarted == "function" && Te.markCommitStarted(e);
    }
    function wf() {
      Te !== null && typeof Te.markCommitStopped == "function" && Te.markCommitStopped();
    }
    function fl(e) {
      Te !== null && typeof Te.markComponentRenderStarted == "function" && Te.markComponentRenderStarted(e);
    }
    function jr() {
      Te !== null && typeof Te.markComponentRenderStopped == "function" && Te.markComponentRenderStopped();
    }
    function xo(e) {
      Te !== null && typeof Te.markComponentPassiveEffectMountStarted == "function" && Te.markComponentPassiveEffectMountStarted(e);
    }
    function Tf() {
      Te !== null && typeof Te.markComponentPassiveEffectMountStopped == "function" && Te.markComponentPassiveEffectMountStopped();
    }
    function Ag(e) {
      Te !== null && typeof Te.markComponentPassiveEffectUnmountStarted == "function" && Te.markComponentPassiveEffectUnmountStarted(e);
    }
    function Rf() {
      Te !== null && typeof Te.markComponentPassiveEffectUnmountStopped == "function" && Te.markComponentPassiveEffectUnmountStopped();
    }
    function Og(e) {
      Te !== null && typeof Te.markComponentLayoutEffectMountStarted == "function" && Te.markComponentLayoutEffectMountStarted(e);
    }
    function Gs() {
      Te !== null && typeof Te.markComponentLayoutEffectMountStopped == "function" && Te.markComponentLayoutEffectMountStopped();
    }
    function ri(e) {
      Te !== null && typeof Te.markComponentLayoutEffectUnmountStarted == "function" && Te.markComponentLayoutEffectUnmountStarted(e);
    }
    function dl() {
      Te !== null && typeof Te.markComponentLayoutEffectUnmountStopped == "function" && Te.markComponentLayoutEffectUnmountStopped();
    }
    function _s(e, t, o) {
      Te !== null && typeof Te.markComponentErrored == "function" && Te.markComponentErrored(e, t, o);
    }
    function cu(e, t, o) {
      Te !== null && typeof Te.markComponentSuspended == "function" && Te.markComponentSuspended(e, t, o);
    }
    function pv(e) {
      Te !== null && typeof Te.markLayoutEffectsStarted == "function" && Te.markLayoutEffectsStarted(e);
    }
    function pl() {
      Te !== null && typeof Te.markLayoutEffectsStopped == "function" && Te.markLayoutEffectsStopped();
    }
    function Dg(e) {
      Te !== null && typeof Te.markPassiveEffectsStarted == "function" && Te.markPassiveEffectsStarted(e);
    }
    function vv() {
      Te !== null && typeof Te.markPassiveEffectsStopped == "function" && Te.markPassiveEffectsStopped();
    }
    function ln(e) {
      Te !== null && typeof Te.markRenderStarted == "function" && Te.markRenderStarted(e);
    }
    function If() {
      Te !== null && typeof Te.markRenderYielded == "function" && Te.markRenderYielded();
    }
    function Af() {
      Te !== null && typeof Te.markRenderStopped == "function" && Te.markRenderStopped();
    }
    function hv(e) {
      Te !== null && typeof Te.markRenderScheduled == "function" && Te.markRenderScheduled(e);
    }
    function Of(e, t) {
      Te !== null && typeof Te.markForceUpdateScheduled == "function" && Te.markForceUpdateScheduled(e, t);
    }
    function js(e, t) {
      Te !== null && typeof Te.markStateUpdateScheduled == "function" && Te.markStateUpdateScheduled(e, t);
    }
    var Xe = (
      /*                         */
      0
    ), $e = (
      /*                 */
      1
    ), pt = (
      /*                    */
      2
    ), xt = (
      /*               */
      8
    ), la = (
      /*              */
      16
    ), vl = Math.clz32 ? Math.clz32 : mr, Us = Math.log, sC = Math.LN2;
    function mr(e) {
      var t = e >>> 0;
      return t === 0 ? 32 : 31 - (Us(t) / sC | 0) | 0;
    }
    var Ws = 31, fe = (
      /*                        */
      0
    ), kn = (
      /*                          */
      0
    ), Je = (
      /*                        */
      1
    ), rr = (
      /*    */
      2
    ), sa = (
      /*             */
      4
    ), Ui = (
      /*            */
      8
    ), Da = (
      /*                     */
      16
    ), hl = (
      /*                */
      32
    ), fu = (
      /*                       */
      4194240
    ), ml = (
      /*                        */
      64
    ), Df = (
      /*                        */
      128
    ), Nf = (
      /*                        */
      256
    ), kf = (
      /*                        */
      512
    ), Mf = (
      /*                        */
      1024
    ), Lf = (
      /*                        */
      2048
    ), du = (
      /*                        */
      4096
    ), Ff = (
      /*                        */
      8192
    ), gl = (
      /*                        */
      16384
    ), yl = (
      /*                       */
      32768
    ), Bf = (
      /*                       */
      65536
    ), Xs = (
      /*                       */
      131072
    ), Vf = (
      /*                       */
      262144
    ), Pf = (
      /*                       */
      524288
    ), Hf = (
      /*                       */
      1048576
    ), zf = (
      /*                       */
      2097152
    ), bl = (
      /*                            */
      130023424
    ), pu = (
      /*                             */
      4194304
    ), Gf = (
      /*                             */
      8388608
    ), _f = (
      /*                             */
      16777216
    ), mv = (
      /*                             */
      33554432
    ), jf = (
      /*                             */
      67108864
    ), Ng = pu, Zs = (
      /*          */
      134217728
    ), gv = (
      /*                          */
      268435455
    ), Cl = (
      /*               */
      268435456
    ), wo = (
      /*                        */
      536870912
    ), gr = (
      /*                   */
      1073741824
    );
    function cC(e) {
      {
        if (e & Je)
          return "Sync";
        if (e & rr)
          return "InputContinuousHydration";
        if (e & sa)
          return "InputContinuous";
        if (e & Ui)
          return "DefaultHydration";
        if (e & Da)
          return "Default";
        if (e & hl)
          return "TransitionHydration";
        if (e & fu)
          return "Transition";
        if (e & bl)
          return "Retry";
        if (e & Zs)
          return "SelectiveHydration";
        if (e & Cl)
          return "IdleHydration";
        if (e & wo)
          return "Idle";
        if (e & gr)
          return "Offscreen";
      }
    }
    var tn = -1, Uf = ml, Ur = pu;
    function vu(e) {
      switch (xn(e)) {
        case Je:
          return Je;
        case rr:
          return rr;
        case sa:
          return sa;
        case Ui:
          return Ui;
        case Da:
          return Da;
        case hl:
          return hl;
        case ml:
        case Df:
        case Nf:
        case kf:
        case Mf:
        case Lf:
        case du:
        case Ff:
        case gl:
        case yl:
        case Bf:
        case Xs:
        case Vf:
        case Pf:
        case Hf:
        case zf:
          return e & fu;
        case pu:
        case Gf:
        case _f:
        case mv:
        case jf:
          return e & bl;
        case Zs:
          return Zs;
        case Cl:
          return Cl;
        case wo:
          return wo;
        case gr:
          return gr;
        default:
          return h("Should have found matching lanes. This is a bug in React."), e;
      }
    }
    function hu(e, t) {
      var o = e.pendingLanes;
      if (o === fe)
        return fe;
      var l = fe, d = e.suspendedLanes, m = e.pingedLanes, y = o & gv;
      if (y !== fe) {
        var x = y & ~d;
        if (x !== fe)
          l = vu(x);
        else {
          var T = y & m;
          T !== fe && (l = vu(T));
        }
      } else {
        var O = o & ~d;
        O !== fe ? l = vu(O) : m !== fe && (l = vu(m));
      }
      if (l === fe)
        return fe;
      if (t !== fe && t !== l && // If we already suspended with a delay, then interrupting is fine. Don't
      // bother waiting until the root is complete.
      (t & d) === fe) {
        var N = xn(l), j = xn(t);
        if (
          // Tests whether the next lane is equal or lower priority than the wip
          // one. This works because the bits decrease in priority as you go left.
          N >= j || // Default priority updates should not interrupt transition updates. The
          // only difference between default updates and transition updates is that
          // default updates do not support refresh transitions.
          N === Da && (j & fu) !== fe
        )
          return t;
      }
      (l & sa) !== fe && (l |= o & Da);
      var G = e.entangledLanes;
      if (G !== fe)
        for (var te = e.entanglements, re = l & G; re > 0; ) {
          var ie = Ro(re), Ne = 1 << ie;
          l |= te[ie], re &= ~Ne;
        }
      return l;
    }
    function kg(e, t) {
      for (var o = e.eventTimes, l = tn; t > 0; ) {
        var d = Ro(t), m = 1 << d, y = o[d];
        y > l && (l = y), t &= ~m;
      }
      return l;
    }
    function Mg(e, t) {
      switch (e) {
        case Je:
        case rr:
        case sa:
          return t + 250;
        case Ui:
        case Da:
        case hl:
        case ml:
        case Df:
        case Nf:
        case kf:
        case Mf:
        case Lf:
        case du:
        case Ff:
        case gl:
        case yl:
        case Bf:
        case Xs:
        case Vf:
        case Pf:
        case Hf:
        case zf:
          return t + 5e3;
        case pu:
        case Gf:
        case _f:
        case mv:
        case jf:
          return tn;
        case Zs:
        case Cl:
        case wo:
        case gr:
          return tn;
        default:
          return h("Should have found matching lanes. This is a bug in React."), tn;
      }
    }
    function Lg(e, t) {
      for (var o = e.pendingLanes, l = e.suspendedLanes, d = e.pingedLanes, m = e.expirationTimes, y = o; y > 0; ) {
        var x = Ro(y), T = 1 << x, O = m[x];
        O === tn ? ((T & l) === fe || (T & d) !== fe) && (m[x] = Mg(T, t)) : O <= t && (e.expiredLanes |= T), y &= ~T;
      }
    }
    function yv(e) {
      return vu(e.pendingLanes);
    }
    function To(e) {
      var t = e.pendingLanes & ~gr;
      return t !== fe ? t : t & gr ? gr : fe;
    }
    function bv(e) {
      return (e & Je) !== fe;
    }
    function Ys(e) {
      return (e & gv) !== fe;
    }
    function Fg(e) {
      return (e & bl) === e;
    }
    function Bg(e) {
      var t = Je | sa | Da;
      return (e & t) === fe;
    }
    function Vg(e) {
      return (e & fu) === e;
    }
    function $s(e, t) {
      var o = rr | sa | Ui | Da;
      return (t & o) !== fe;
    }
    function Pg(e, t) {
      return (t & e.expiredLanes) !== fe;
    }
    function Cv(e) {
      return (e & fu) !== fe;
    }
    function Hg() {
      var e = Uf;
      return Uf <<= 1, (Uf & fu) === fe && (Uf = ml), e;
    }
    function Wr() {
      var e = Ur;
      return Ur <<= 1, (Ur & bl) === fe && (Ur = pu), e;
    }
    function xn(e) {
      return e & -e;
    }
    function Sl(e) {
      return xn(e);
    }
    function Ro(e) {
      return 31 - vl(e);
    }
    function Wf(e) {
      return Ro(e);
    }
    function Xr(e, t) {
      return (e & t) !== fe;
    }
    function mu(e, t) {
      return (e & t) === t;
    }
    function Ct(e, t) {
      return e | t;
    }
    function Qs(e, t) {
      return e & ~t;
    }
    function Xf(e, t) {
      return e & t;
    }
    function fC(e) {
      return e;
    }
    function zg(e, t) {
      return e !== kn && e < t ? e : t;
    }
    function Js(e) {
      for (var t = [], o = 0; o < Ws; o++)
        t.push(e);
      return t;
    }
    function gu(e, t, o) {
      e.pendingLanes |= t, t !== wo && (e.suspendedLanes = fe, e.pingedLanes = fe);
      var l = e.eventTimes, d = Wf(t);
      l[d] = o;
    }
    function Gg(e, t) {
      e.suspendedLanes |= t, e.pingedLanes &= ~t;
      for (var o = e.expirationTimes, l = t; l > 0; ) {
        var d = Ro(l), m = 1 << d;
        o[d] = tn, l &= ~m;
      }
    }
    function Zf(e, t, o) {
      e.pingedLanes |= e.suspendedLanes & t;
    }
    function Yf(e, t) {
      var o = e.pendingLanes & ~t;
      e.pendingLanes = t, e.suspendedLanes = fe, e.pingedLanes = fe, e.expiredLanes &= t, e.mutableReadLanes &= t, e.entangledLanes &= t;
      for (var l = e.entanglements, d = e.eventTimes, m = e.expirationTimes, y = o; y > 0; ) {
        var x = Ro(y), T = 1 << x;
        l[x] = fe, d[x] = tn, m[x] = tn, y &= ~T;
      }
    }
    function Sv(e, t) {
      for (var o = e.entangledLanes |= t, l = e.entanglements, d = o; d; ) {
        var m = Ro(d), y = 1 << m;
        // Is this one of the newly entangled lanes?
        y & t | // Is this lane transitively entangled with the newly entangled lanes?
        l[m] & t && (l[m] |= t), d &= ~y;
      }
    }
    function _g(e, t) {
      var o = xn(t), l;
      switch (o) {
        case sa:
          l = rr;
          break;
        case Da:
          l = Ui;
          break;
        case ml:
        case Df:
        case Nf:
        case kf:
        case Mf:
        case Lf:
        case du:
        case Ff:
        case gl:
        case yl:
        case Bf:
        case Xs:
        case Vf:
        case Pf:
        case Hf:
        case zf:
        case pu:
        case Gf:
        case _f:
        case mv:
        case jf:
          l = hl;
          break;
        case wo:
          l = Cl;
          break;
        default:
          l = kn;
          break;
      }
      return (l & (e.suspendedLanes | t)) !== kn ? kn : l;
    }
    function $f(e, t, o) {
      if (ua)
        for (var l = e.pendingUpdatersLaneMap; o > 0; ) {
          var d = Wf(o), m = 1 << d, y = l[d];
          y.add(t), o &= ~m;
        }
    }
    function Ev(e, t) {
      if (ua)
        for (var o = e.pendingUpdatersLaneMap, l = e.memoizedUpdaters; t > 0; ) {
          var d = Wf(t), m = 1 << d, y = o[d];
          y.size > 0 && (y.forEach(function(x) {
            var T = x.alternate;
            (T === null || !l.has(T)) && l.add(x);
          }), y.clear()), t &= ~m;
        }
    }
    function Ks(e, t) {
      return null;
    }
    var wn = Je, Wi = sa, gi = Da, El = wo, xl = kn;
    function Na() {
      return xl;
    }
    function yn(e) {
      xl = e;
    }
    function yr(e, t) {
      var o = xl;
      try {
        return xl = e, t();
      } finally {
        xl = o;
      }
    }
    function dC(e, t) {
      return e !== 0 && e < t ? e : t;
    }
    function pC(e, t) {
      return e > t ? e : t;
    }
    function wl(e, t) {
      return e !== 0 && e < t;
    }
    function ar(e) {
      var t = xn(e);
      return wl(wn, t) ? wl(Wi, t) ? Ys(t) ? gi : El : Wi : wn;
    }
    function Qf(e) {
      var t = e.current.memoizedState;
      return t.isDehydrated;
    }
    var Pe;
    function Tl(e) {
      Pe = e;
    }
    function xv(e) {
      Pe(e);
    }
    var Jf;
    function vC(e) {
      Jf = e;
    }
    var Rl;
    function Kf(e) {
      Rl = e;
    }
    var qf;
    function jg(e) {
      qf = e;
    }
    var wv;
    function Ug(e) {
      wv = e;
    }
    var qs = !1, Il = [], sn = null, Yn = null, xr = null, Al = /* @__PURE__ */ new Map(), Ol = /* @__PURE__ */ new Map(), $n = [], Wg = [
      "mousedown",
      "mouseup",
      "touchcancel",
      "touchend",
      "touchstart",
      "auxclick",
      "dblclick",
      "pointercancel",
      "pointerdown",
      "pointerup",
      "dragend",
      "dragstart",
      "drop",
      "compositionend",
      "compositionstart",
      "keydown",
      "keypress",
      "keyup",
      "input",
      "textInput",
      // Intentionally camelCase
      "copy",
      "cut",
      "paste",
      "click",
      "change",
      "contextmenu",
      "reset",
      "submit"
    ];
    function yi(e) {
      return Wg.indexOf(e) > -1;
    }
    function hC(e, t, o, l, d) {
      return {
        blockedOn: e,
        domEventName: t,
        eventSystemFlags: o,
        nativeEvent: d,
        targetContainers: [l]
      };
    }
    function Tv(e, t) {
      switch (e) {
        case "focusin":
        case "focusout":
          sn = null;
          break;
        case "dragenter":
        case "dragleave":
          Yn = null;
          break;
        case "mouseover":
        case "mouseout":
          xr = null;
          break;
        case "pointerover":
        case "pointerout": {
          var o = t.pointerId;
          Al.delete(o);
          break;
        }
        case "gotpointercapture":
        case "lostpointercapture": {
          var l = t.pointerId;
          Ol.delete(l);
          break;
        }
      }
    }
    function Dl(e, t, o, l, d, m) {
      if (e === null || e.nativeEvent !== m) {
        var y = hC(t, o, l, d, m);
        if (t !== null) {
          var x = Hl(t);
          x !== null && Jf(x);
        }
        return y;
      }
      e.eventSystemFlags |= l;
      var T = e.targetContainers;
      return d !== null && T.indexOf(d) === -1 && T.push(d), e;
    }
    function Xg(e, t, o, l, d) {
      switch (t) {
        case "focusin": {
          var m = d;
          return sn = Dl(sn, e, t, o, l, m), !0;
        }
        case "dragenter": {
          var y = d;
          return Yn = Dl(Yn, e, t, o, l, y), !0;
        }
        case "mouseover": {
          var x = d;
          return xr = Dl(xr, e, t, o, l, x), !0;
        }
        case "pointerover": {
          var T = d, O = T.pointerId;
          return Al.set(O, Dl(Al.get(O) || null, e, t, o, l, T)), !0;
        }
        case "gotpointercapture": {
          var N = d, j = N.pointerId;
          return Ol.set(j, Dl(Ol.get(j) || null, e, t, o, l, N)), !0;
        }
      }
      return !1;
    }
    function Rv(e) {
      var t = fc(e.target);
      if (t !== null) {
        var o = ia(t);
        if (o !== null) {
          var l = o.tag;
          if (l === V) {
            var d = uv(o);
            if (d !== null) {
              e.blockedOn = d, wv(e.priority, function() {
                Rl(o);
              });
              return;
            }
          } else if (l === R) {
            var m = o.stateNode;
            if (Qf(m)) {
              e.blockedOn = yf(o);
              return;
            }
          }
        }
      }
      e.blockedOn = null;
    }
    function mC(e) {
      for (var t = qf(), o = {
        blockedOn: null,
        target: e,
        priority: t
      }, l = 0; l < $n.length && wl(t, $n[l].priority); l++)
        ;
      $n.splice(l, 0, o), l === 0 && Rv(o);
    }
    function yu(e) {
      if (e.blockedOn !== null)
        return !1;
      for (var t = e.targetContainers; t.length > 0; ) {
        var o = t[0], l = br(e.domEventName, e.eventSystemFlags, o, e.nativeEvent);
        if (l === null) {
          var d = e.nativeEvent, m = new d.constructor(d.type, d);
          Ms(m), d.target.dispatchEvent(m), iC();
        } else {
          var y = Hl(l);
          return y !== null && Jf(y), e.blockedOn = l, !1;
        }
        t.shift();
      }
      return !0;
    }
    function ed(e, t, o) {
      yu(e) && o.delete(t);
    }
    function ka() {
      qs = !1, sn !== null && yu(sn) && (sn = null), Yn !== null && yu(Yn) && (Yn = null), xr !== null && yu(xr) && (xr = null), Al.forEach(ed), Ol.forEach(ed);
    }
    function Rt(e, t) {
      e.blockedOn === t && (e.blockedOn = null, qs || (qs = !0, r.unstable_scheduleCallback(r.unstable_NormalPriority, ka)));
    }
    function bn(e) {
      if (Il.length > 0) {
        Rt(Il[0], e);
        for (var t = 1; t < Il.length; t++) {
          var o = Il[t];
          o.blockedOn === e && (o.blockedOn = null);
        }
      }
      sn !== null && Rt(sn, e), Yn !== null && Rt(Yn, e), xr !== null && Rt(xr, e);
      var l = function(x) {
        return Rt(x, e);
      };
      Al.forEach(l), Ol.forEach(l);
      for (var d = 0; d < $n.length; d++) {
        var m = $n[d];
        m.blockedOn === e && (m.blockedOn = null);
      }
      for (; $n.length > 0; ) {
        var y = $n[0];
        if (y.blockedOn !== null)
          break;
        Rv(y), y.blockedOn === null && $n.shift();
      }
    }
    var an = u.ReactCurrentBatchConfig, Hn = !0;
    function Zr(e) {
      Hn = !!e;
    }
    function Nl() {
      return Hn;
    }
    function zn(e, t, o) {
      var l = td(t), d;
      switch (l) {
        case wn:
          d = ec;
          break;
        case Wi:
          d = bu;
          break;
        case gi:
        default:
          d = kl;
          break;
      }
      return d.bind(null, t, o, e);
    }
    function ec(e, t, o, l) {
      var d = Na(), m = an.transition;
      an.transition = null;
      try {
        yn(wn), kl(e, t, o, l);
      } finally {
        yn(d), an.transition = m;
      }
    }
    function bu(e, t, o, l) {
      var d = Na(), m = an.transition;
      an.transition = null;
      try {
        yn(Wi), kl(e, t, o, l);
      } finally {
        yn(d), an.transition = m;
      }
    }
    function kl(e, t, o, l) {
      Hn && Iv(e, t, o, l);
    }
    function Iv(e, t, o, l) {
      var d = br(e, t, o, l);
      if (d === null) {
        MC(e, t, l, Io, o), Tv(e, l);
        return;
      }
      if (Xg(d, e, t, o, l)) {
        l.stopPropagation();
        return;
      }
      if (Tv(e, l), t & nu && yi(e)) {
        for (; d !== null; ) {
          var m = Hl(d);
          m !== null && xv(m);
          var y = br(e, t, o, l);
          if (y === null && MC(e, t, l, Io, o), y === d)
            break;
          d = y;
        }
        d !== null && l.stopPropagation();
        return;
      }
      MC(e, t, l, null, o);
    }
    var Io = null;
    function br(e, t, o, l) {
      Io = null;
      var d = pf(l), m = fc(d);
      if (m !== null) {
        var y = ia(m);
        if (y === null)
          m = null;
        else {
          var x = y.tag;
          if (x === V) {
            var T = uv(y);
            if (T !== null)
              return T;
            m = null;
          } else if (x === R) {
            var O = y.stateNode;
            if (Qf(O))
              return yf(y);
            m = null;
          } else
            y !== m && (m = null);
        }
      }
      return Io = m, null;
    }
    function td(e) {
      switch (e) {
        case "cancel":
        case "click":
        case "close":
        case "contextmenu":
        case "copy":
        case "cut":
        case "auxclick":
        case "dblclick":
        case "dragend":
        case "dragstart":
        case "drop":
        case "focusin":
        case "focusout":
        case "input":
        case "invalid":
        case "keydown":
        case "keypress":
        case "keyup":
        case "mousedown":
        case "mouseup":
        case "paste":
        case "pause":
        case "play":
        case "pointercancel":
        case "pointerdown":
        case "pointerup":
        case "ratechange":
        case "reset":
        case "resize":
        case "seeked":
        case "submit":
        case "touchcancel":
        case "touchend":
        case "touchstart":
        case "volumechange":
        case "change":
        case "selectionchange":
        case "textInput":
        case "compositionstart":
        case "compositionend":
        case "compositionupdate":
        case "beforeblur":
        case "afterblur":
        case "beforeinput":
        case "blur":
        case "fullscreenchange":
        case "focus":
        case "hashchange":
        case "popstate":
        case "select":
        case "selectstart":
          return wn;
        case "drag":
        case "dragenter":
        case "dragexit":
        case "dragleave":
        case "dragover":
        case "mousemove":
        case "mouseout":
        case "mouseover":
        case "pointermove":
        case "pointerout":
        case "pointerover":
        case "scroll":
        case "toggle":
        case "touchmove":
        case "wheel":
        case "mouseenter":
        case "mouseleave":
        case "pointerenter":
        case "pointerleave":
          return Wi;
        case "message": {
          var t = cv();
          switch (t) {
            case Ef:
              return wn;
            case lu:
              return Wi;
            case mi:
            case Tg:
              return gi;
            case xf:
              return El;
            default:
              return gi;
          }
        }
        default:
          return gi;
      }
    }
    function Ml(e, t, o) {
      return e.addEventListener(t, o, !1), o;
    }
    function Xi(e, t, o) {
      return e.addEventListener(t, o, !0), o;
    }
    function nd(e, t, o, l) {
      return e.addEventListener(t, o, {
        capture: !0,
        passive: l
      }), o;
    }
    function Av(e, t, o, l) {
      return e.addEventListener(t, o, {
        passive: l
      }), o;
    }
    var Ma = null, Ll = null, La = null;
    function rd(e) {
      return Ma = e, Ll = nc(), !0;
    }
    function tc() {
      Ma = null, Ll = null, La = null;
    }
    function ad() {
      if (La)
        return La;
      var e, t = Ll, o = t.length, l, d = nc(), m = d.length;
      for (e = 0; e < o && t[e] === d[e]; e++)
        ;
      var y = o - e;
      for (l = 1; l <= y && t[o - l] === d[m - l]; l++)
        ;
      var x = l > 1 ? 1 - l : void 0;
      return La = d.slice(e, x), La;
    }
    function nc() {
      return "value" in Ma ? Ma.value : Ma.textContent;
    }
    function Cu(e) {
      var t, o = e.keyCode;
      return "charCode" in e ? (t = e.charCode, t === 0 && o === 13 && (t = 13)) : t = o, t === 10 && (t = 13), t >= 32 || t === 13 ? t : 0;
    }
    function Qn() {
      return !0;
    }
    function Zi() {
      return !1;
    }
    function fn(e) {
      function t(o, l, d, m, y) {
        this._reactName = o, this._targetInst = d, this.type = l, this.nativeEvent = m, this.target = y, this.currentTarget = null;
        for (var x in e)
          if (e.hasOwnProperty(x)) {
            var T = e[x];
            T ? this[x] = T(m) : this[x] = m[x];
          }
        var O = m.defaultPrevented != null ? m.defaultPrevented : m.returnValue === !1;
        return O ? this.isDefaultPrevented = Qn : this.isDefaultPrevented = Zi, this.isPropagationStopped = Zi, this;
      }
      return Tt(t.prototype, {
        preventDefault: function() {
          this.defaultPrevented = !0;
          var o = this.nativeEvent;
          o && (o.preventDefault ? o.preventDefault() : typeof o.returnValue != "unknown" && (o.returnValue = !1), this.isDefaultPrevented = Qn);
        },
        stopPropagation: function() {
          var o = this.nativeEvent;
          o && (o.stopPropagation ? o.stopPropagation() : typeof o.cancelBubble != "unknown" && (o.cancelBubble = !0), this.isPropagationStopped = Qn);
        },
        /**
         * We release all dispatched `SyntheticEvent`s after each event loop, adding
         * them back into the pool. This allows a way to hold onto a reference that
         * won't be added back into the pool.
         */
        persist: function() {
        },
        /**
         * Checks if this event should be released back into the pool.
         *
         * @return {boolean} True if this should not be released, false otherwise.
         */
        isPersistent: Qn
      }), t;
    }
    var Gn = {
      eventPhase: 0,
      bubbles: 0,
      cancelable: 0,
      timeStamp: function(e) {
        return e.timeStamp || Date.now();
      },
      defaultPrevented: 0,
      isTrusted: 0
    }, id = fn(Gn), Su = Tt({}, Gn, {
      view: 0,
      detail: 0
    }), Ov = fn(Su), Dv, bi, Fl;
    function Nv(e) {
      e !== Fl && (Fl && e.type === "mousemove" ? (Dv = e.screenX - Fl.screenX, bi = e.screenY - Fl.screenY) : (Dv = 0, bi = 0), Fl = e);
    }
    var Ci = Tt({}, Su, {
      screenX: 0,
      screenY: 0,
      clientX: 0,
      clientY: 0,
      pageX: 0,
      pageY: 0,
      ctrlKey: 0,
      shiftKey: 0,
      altKey: 0,
      metaKey: 0,
      getModifierState: kv,
      button: 0,
      buttons: 0,
      relatedTarget: function(e) {
        return e.relatedTarget === void 0 ? e.fromElement === e.srcElement ? e.toElement : e.fromElement : e.relatedTarget;
      },
      movementX: function(e) {
        return "movementX" in e ? e.movementX : (Nv(e), Dv);
      },
      movementY: function(e) {
        return "movementY" in e ? e.movementY : bi;
      }
    }), od = fn(Ci), Eu = Tt({}, Ci, {
      dataTransfer: 0
    }), Zg = fn(Eu), Yg = Tt({}, Su, {
      relatedTarget: 0
    }), rc = fn(Yg), ud = Tt({}, Gn, {
      animationName: 0,
      elapsedTime: 0,
      pseudoElement: 0
    }), gC = fn(ud), yC = Tt({}, Gn, {
      clipboardData: function(e) {
        return "clipboardData" in e ? e.clipboardData : window.clipboardData;
      }
    }), $g = fn(yC), Qg = Tt({}, Gn, {
      data: 0
    }), Ao = fn(Qg), bC = Ao, Bl = {
      Esc: "Escape",
      Spacebar: " ",
      Left: "ArrowLeft",
      Up: "ArrowUp",
      Right: "ArrowRight",
      Down: "ArrowDown",
      Del: "Delete",
      Win: "OS",
      Menu: "ContextMenu",
      Apps: "ContextMenu",
      Scroll: "ScrollLock",
      MozPrintableKey: "Unidentified"
    }, Jg = {
      8: "Backspace",
      9: "Tab",
      12: "Clear",
      13: "Enter",
      16: "Shift",
      17: "Control",
      18: "Alt",
      19: "Pause",
      20: "CapsLock",
      27: "Escape",
      32: " ",
      33: "PageUp",
      34: "PageDown",
      35: "End",
      36: "Home",
      37: "ArrowLeft",
      38: "ArrowUp",
      39: "ArrowRight",
      40: "ArrowDown",
      45: "Insert",
      46: "Delete",
      112: "F1",
      113: "F2",
      114: "F3",
      115: "F4",
      116: "F5",
      117: "F6",
      118: "F7",
      119: "F8",
      120: "F9",
      121: "F10",
      122: "F11",
      123: "F12",
      144: "NumLock",
      145: "ScrollLock",
      224: "Meta"
    };
    function Cn(e) {
      if (e.key) {
        var t = Bl[e.key] || e.key;
        if (t !== "Unidentified")
          return t;
      }
      if (e.type === "keypress") {
        var o = Cu(e);
        return o === 13 ? "Enter" : String.fromCharCode(o);
      }
      return e.type === "keydown" || e.type === "keyup" ? Jg[e.keyCode] || "Unidentified" : "";
    }
    var CC = {
      Alt: "altKey",
      Control: "ctrlKey",
      Meta: "metaKey",
      Shift: "shiftKey"
    };
    function Kg(e) {
      var t = this, o = t.nativeEvent;
      if (o.getModifierState)
        return o.getModifierState(e);
      var l = CC[e];
      return l ? !!o[l] : !1;
    }
    function kv(e) {
      return Kg;
    }
    var SC = Tt({}, Su, {
      key: Cn,
      code: 0,
      location: 0,
      ctrlKey: 0,
      shiftKey: 0,
      altKey: 0,
      metaKey: 0,
      repeat: 0,
      locale: 0,
      getModifierState: kv,
      // Legacy Interface
      charCode: function(e) {
        return e.type === "keypress" ? Cu(e) : 0;
      },
      keyCode: function(e) {
        return e.type === "keydown" || e.type === "keyup" ? e.keyCode : 0;
      },
      which: function(e) {
        return e.type === "keypress" ? Cu(e) : e.type === "keydown" || e.type === "keyup" ? e.keyCode : 0;
      }
    }), qg = fn(SC), ey = Tt({}, Ci, {
      pointerId: 0,
      width: 0,
      height: 0,
      pressure: 0,
      tangentialPressure: 0,
      tiltX: 0,
      tiltY: 0,
      twist: 0,
      pointerType: 0,
      isPrimary: 0
    }), ty = fn(ey), Fa = Tt({}, Su, {
      touches: 0,
      targetTouches: 0,
      changedTouches: 0,
      altKey: 0,
      metaKey: 0,
      ctrlKey: 0,
      shiftKey: 0,
      getModifierState: kv
    }), Mv = fn(Fa), EC = Tt({}, Gn, {
      propertyName: 0,
      elapsedTime: 0,
      pseudoElement: 0
    }), Oo = fn(EC), ld = Tt({}, Ci, {
      deltaX: function(e) {
        return "deltaX" in e ? e.deltaX : (
          // Fallback to `wheelDeltaX` for Webkit and normalize (right is positive).
          "wheelDeltaX" in e ? -e.wheelDeltaX : 0
        );
      },
      deltaY: function(e) {
        return "deltaY" in e ? e.deltaY : (
          // Fallback to `wheelDeltaY` for Webkit and normalize (down is positive).
          "wheelDeltaY" in e ? -e.wheelDeltaY : (
            // Fallback to `wheelDelta` for IE<9 and normalize (down is positive).
            "wheelDelta" in e ? -e.wheelDelta : 0
          )
        );
      },
      deltaZ: 0,
      // Browsers without "deltaMode" is reporting in raw wheel delta where one
      // notch on the scroll is always +/- 120, roughly equivalent to pixels.
      // A good approximation of DOM_DELTA_LINE (1) is 5% of viewport size or
      // ~40 pixels, for DOM_DELTA_SCREEN (2) it is 87.5% of viewport size.
      deltaMode: 0
    }), xu = fn(ld), sd = [9, 13, 27, 32], cd = 229, ac = Ot && "CompositionEvent" in window, ic = null;
    Ot && "documentMode" in document && (ic = document.documentMode);
    var Lv = Ot && "TextEvent" in window && !ic, ny = Ot && (!ac || ic && ic > 8 && ic <= 11), Fv = 32, Bv = String.fromCharCode(Fv);
    function fd() {
      At("onBeforeInput", ["compositionend", "keypress", "textInput", "paste"]), At("onCompositionEnd", ["compositionend", "focusout", "keydown", "keypress", "keyup", "mousedown"]), At("onCompositionStart", ["compositionstart", "focusout", "keydown", "keypress", "keyup", "mousedown"]), At("onCompositionUpdate", ["compositionupdate", "focusout", "keydown", "keypress", "keyup", "mousedown"]);
    }
    var oc = !1;
    function ry(e) {
      return (e.ctrlKey || e.altKey || e.metaKey) && // ctrlKey && altKey is equivalent to AltGr, and is not a command.
      !(e.ctrlKey && e.altKey);
    }
    function Vv(e) {
      switch (e) {
        case "compositionstart":
          return "onCompositionStart";
        case "compositionend":
          return "onCompositionEnd";
        case "compositionupdate":
          return "onCompositionUpdate";
      }
    }
    function xC(e, t) {
      return e === "keydown" && t.keyCode === cd;
    }
    function Pv(e, t) {
      switch (e) {
        case "keyup":
          return sd.indexOf(t.keyCode) !== -1;
        case "keydown":
          return t.keyCode !== cd;
        case "keypress":
        case "mousedown":
        case "focusout":
          return !0;
        default:
          return !1;
      }
    }
    function dd(e) {
      var t = e.detail;
      return typeof t == "object" && "data" in t ? t.data : null;
    }
    function uc(e) {
      return e.locale === "ko";
    }
    var Do = !1;
    function pd(e, t, o, l, d) {
      var m, y;
      if (ac ? m = Vv(t) : Do ? Pv(t, l) && (m = "onCompositionEnd") : xC(t, l) && (m = "onCompositionStart"), !m)
        return null;
      ny && !uc(l) && (!Do && m === "onCompositionStart" ? Do = rd(d) : m === "onCompositionEnd" && Do && (y = ad()));
      var x = sy(o, m);
      if (x.length > 0) {
        var T = new Ao(m, t, null, l, d);
        if (e.push({
          event: T,
          listeners: x
        }), y)
          T.data = y;
        else {
          var O = dd(l);
          O !== null && (T.data = O);
        }
      }
    }
    function ay(e, t) {
      switch (e) {
        case "compositionend":
          return dd(t);
        case "keypress":
          var o = t.which;
          return o !== Fv ? null : (oc = !0, Bv);
        case "textInput":
          var l = t.data;
          return l === Bv && oc ? null : l;
        default:
          return null;
      }
    }
    function wC(e, t) {
      if (Do) {
        if (e === "compositionend" || !ac && Pv(e, t)) {
          var o = ad();
          return tc(), Do = !1, o;
        }
        return null;
      }
      switch (e) {
        case "paste":
          return null;
        case "keypress":
          if (!ry(t)) {
            if (t.char && t.char.length > 1)
              return t.char;
            if (t.which)
              return String.fromCharCode(t.which);
          }
          return null;
        case "compositionend":
          return ny && !uc(t) ? null : t.data;
        default:
          return null;
      }
    }
    function vd(e, t, o, l, d) {
      var m;
      if (Lv ? m = ay(t, l) : m = wC(t, l), !m)
        return null;
      var y = sy(o, "onBeforeInput");
      if (y.length > 0) {
        var x = new bC("onBeforeInput", "beforeinput", null, l, d);
        e.push({
          event: x,
          listeners: y
        }), x.data = m;
      }
    }
    function TC(e, t, o, l, d, m, y) {
      pd(e, t, o, l, d), vd(e, t, o, l, d);
    }
    var lc = {
      color: !0,
      date: !0,
      datetime: !0,
      "datetime-local": !0,
      email: !0,
      month: !0,
      number: !0,
      password: !0,
      range: !0,
      search: !0,
      tel: !0,
      text: !0,
      time: !0,
      url: !0,
      week: !0
    };
    function iy(e) {
      var t = e && e.nodeName && e.nodeName.toLowerCase();
      return t === "input" ? !!lc[e.type] : t === "textarea";
    }
    /**
     * Checks if an event is supported in the current execution environment.
     *
     * NOTE: This will not work correctly for non-generic events such as `change`,
     * `reset`, `load`, `error`, and `select`.
     *
     * Borrows from Modernizr.
     *
     * @param {string} eventNameSuffix Event name, e.g. "click".
     * @return {boolean} True if the event is supported.
     * @internal
     * @license Modernizr 3.0.0pre (Custom Build) | MIT
     */
    function hd(e) {
      if (!Ot)
        return !1;
      var t = "on" + e, o = t in document;
      if (!o) {
        var l = document.createElement("div");
        l.setAttribute(t, "return;"), o = typeof l[t] == "function";
      }
      return o;
    }
    function n() {
      At("onChange", ["change", "click", "focusin", "focusout", "input", "keydown", "keyup", "selectionchange"]);
    }
    function i(e, t, o, l) {
      vf(l);
      var d = sy(t, "onChange");
      if (d.length > 0) {
        var m = new id("onChange", "change", null, o, l);
        e.push({
          event: m,
          listeners: d
        });
      }
    }
    var s = null, p = null;
    function g(e) {
      var t = e.nodeName && e.nodeName.toLowerCase();
      return t === "select" || t === "input" && e.type === "file";
    }
    function S(e) {
      var t = [];
      i(t, p, e, pf(e)), qp(I, t);
    }
    function I(e) {
      Ow(e, 0);
    }
    function F(e) {
      var t = Sd(e);
      if (Gm(t))
        return e;
    }
    function H(e, t) {
      if (e === "change")
        return t;
    }
    var ne = !1;
    Ot && (ne = hd("input") && (!document.documentMode || document.documentMode > 9));
    function pe(e, t) {
      s = e, p = t, s.attachEvent("onpropertychange", de);
    }
    function ve() {
      s && (s.detachEvent("onpropertychange", de), s = null, p = null);
    }
    function de(e) {
      e.propertyName === "value" && F(p) && S(e);
    }
    function Fe(e, t, o) {
      e === "focusin" ? (ve(), pe(t, o)) : e === "focusout" && ve();
    }
    function ze(e, t) {
      if (e === "selectionchange" || e === "keyup" || e === "keydown")
        return F(p);
    }
    function je(e) {
      var t = e.nodeName;
      return t && t.toLowerCase() === "input" && (e.type === "checkbox" || e.type === "radio");
    }
    function Tn(e, t) {
      if (e === "click")
        return F(t);
    }
    function J(e, t) {
      if (e === "input" || e === "change")
        return F(t);
    }
    function _(e) {
      var t = e._wrapperState;
      !t || !t.controlled || e.type !== "number" || vo(e, "number", e.value);
    }
    function ee(e, t, o, l, d, m, y) {
      var x = o ? Sd(o) : window, T, O;
      if (g(x) ? T = H : iy(x) ? ne ? T = J : (T = ze, O = Fe) : je(x) && (T = Tn), T) {
        var N = T(t, o);
        if (N) {
          i(e, N, l, d);
          return;
        }
      }
      O && O(t, x, o), t === "focusout" && _(x);
    }
    function be() {
      Ce("onMouseEnter", ["mouseout", "mouseover"]), Ce("onMouseLeave", ["mouseout", "mouseover"]), Ce("onPointerEnter", ["pointerout", "pointerover"]), Ce("onPointerLeave", ["pointerout", "pointerover"]);
    }
    function Ze(e, t, o, l, d, m, y) {
      var x = t === "mouseover" || t === "pointerover", T = t === "mouseout" || t === "pointerout";
      if (x && !yg(l)) {
        var O = l.relatedTarget || l.fromElement;
        if (O && (fc(O) || Kv(O)))
          return;
      }
      if (!(!T && !x)) {
        var N;
        if (d.window === d)
          N = d;
        else {
          var j = d.ownerDocument;
          j ? N = j.defaultView || j.parentWindow : N = window;
        }
        var G, te;
        if (T) {
          var re = l.relatedTarget || l.toElement;
          if (G = o, te = re ? fc(re) : null, te !== null) {
            var ie = ia(te);
            (te !== ie || te.tag !== D && te.tag !== L) && (te = null);
          }
        } else
          G = null, te = o;
        if (G !== te) {
          var Ne = od, et = "onMouseLeave", Qe = "onMouseEnter", Nt = "mouse";
          (t === "pointerout" || t === "pointerover") && (Ne = ty, et = "onPointerLeave", Qe = "onPointerEnter", Nt = "pointer");
          var It = G == null ? N : Sd(G), K = te == null ? N : Sd(te), oe = new Ne(et, Nt + "leave", G, l, d);
          oe.target = It, oe.relatedTarget = K;
          var q = null, me = fc(d);
          if (me === o) {
            var Ve = new Ne(Qe, Nt + "enter", te, l, d);
            Ve.target = K, Ve.relatedTarget = It, q = Ve;
          }
          j2(e, oe, q, G, te);
        }
      }
    }
    function at(e, t) {
      return e === t && (e !== 0 || 1 / e === 1 / t) || e !== e && t !== t;
    }
    var Ge = typeof Object.is == "function" ? Object.is : at;
    function ot(e, t) {
      if (Ge(e, t))
        return !0;
      if (typeof e != "object" || e === null || typeof t != "object" || t === null)
        return !1;
      var o = Object.keys(e), l = Object.keys(t);
      if (o.length !== l.length)
        return !1;
      for (var d = 0; d < o.length; d++) {
        var m = o[d];
        if (!_t.call(t, m) || !Ge(e[m], t[m]))
          return !1;
      }
      return !0;
    }
    function _n(e) {
      for (; e && e.firstChild; )
        e = e.firstChild;
      return e;
    }
    function Ft(e) {
      for (; e; ) {
        if (e.nextSibling)
          return e.nextSibling;
        e = e.parentNode;
      }
    }
    function Yi(e, t) {
      for (var o = _n(e), l = 0, d = 0; o; ) {
        if (o.nodeType === Bi) {
          if (d = l + o.textContent.length, l <= t && d >= t)
            return {
              node: o,
              offset: t - l
            };
          l = d;
        }
        o = _n(Ft(o));
      }
    }
    function RC(e) {
      var t = e.ownerDocument, o = t && t.defaultView || window, l = o.getSelection && o.getSelection();
      if (!l || l.rangeCount === 0)
        return null;
      var d = l.anchorNode, m = l.anchorOffset, y = l.focusNode, x = l.focusOffset;
      try {
        d.nodeType, y.nodeType;
      } catch {
        return null;
      }
      return x2(e, d, m, y, x);
    }
    function x2(e, t, o, l, d) {
      var m = 0, y = -1, x = -1, T = 0, O = 0, N = e, j = null;
      e:
        for (; ; ) {
          for (var G = null; N === t && (o === 0 || N.nodeType === Bi) && (y = m + o), N === l && (d === 0 || N.nodeType === Bi) && (x = m + d), N.nodeType === Bi && (m += N.nodeValue.length), (G = N.firstChild) !== null; )
            j = N, N = G;
          for (; ; ) {
            if (N === e)
              break e;
            if (j === t && ++T === o && (y = m), j === l && ++O === d && (x = m), (G = N.nextSibling) !== null)
              break;
            N = j, j = N.parentNode;
          }
          N = G;
        }
      return y === -1 || x === -1 ? null : {
        start: y,
        end: x
      };
    }
    function w2(e, t) {
      var o = e.ownerDocument || document, l = o && o.defaultView || window;
      if (l.getSelection) {
        var d = l.getSelection(), m = e.textContent.length, y = Math.min(t.start, m), x = t.end === void 0 ? y : Math.min(t.end, m);
        if (!d.extend && y > x) {
          var T = x;
          x = y, y = T;
        }
        var O = Yi(e, y), N = Yi(e, x);
        if (O && N) {
          if (d.rangeCount === 1 && d.anchorNode === O.node && d.anchorOffset === O.offset && d.focusNode === N.node && d.focusOffset === N.offset)
            return;
          var j = o.createRange();
          j.setStart(O.node, O.offset), d.removeAllRanges(), y > x ? (d.addRange(j), d.extend(N.node, N.offset)) : (j.setEnd(N.node, N.offset), d.addRange(j));
        }
      }
    }
    function gw(e) {
      return e && e.nodeType === Bi;
    }
    function yw(e, t) {
      return !e || !t ? !1 : e === t ? !0 : gw(e) ? !1 : gw(t) ? yw(e, t.parentNode) : "contains" in e ? e.contains(t) : e.compareDocumentPosition ? !!(e.compareDocumentPosition(t) & 16) : !1;
    }
    function T2(e) {
      return e && e.ownerDocument && yw(e.ownerDocument.documentElement, e);
    }
    function R2(e) {
      try {
        return typeof e.contentWindow.location.href == "string";
      } catch {
        return !1;
      }
    }
    function bw() {
      for (var e = window, t = qc(); t instanceof e.HTMLIFrameElement; ) {
        if (R2(t))
          e = t.contentWindow;
        else
          return t;
        t = qc(e.document);
      }
      return t;
    }
    function IC(e) {
      var t = e && e.nodeName && e.nodeName.toLowerCase();
      return t && (t === "input" && (e.type === "text" || e.type === "search" || e.type === "tel" || e.type === "url" || e.type === "password") || t === "textarea" || e.contentEditable === "true");
    }
    function I2() {
      var e = bw();
      return {
        focusedElem: e,
        selectionRange: IC(e) ? O2(e) : null
      };
    }
    function A2(e) {
      var t = bw(), o = e.focusedElem, l = e.selectionRange;
      if (t !== o && T2(o)) {
        l !== null && IC(o) && D2(o, l);
        for (var d = [], m = o; m = m.parentNode; )
          m.nodeType === Vr && d.push({
            element: m,
            left: m.scrollLeft,
            top: m.scrollTop
          });
        typeof o.focus == "function" && o.focus();
        for (var y = 0; y < d.length; y++) {
          var x = d[y];
          x.element.scrollLeft = x.left, x.element.scrollTop = x.top;
        }
      }
    }
    function O2(e) {
      var t;
      return "selectionStart" in e ? t = {
        start: e.selectionStart,
        end: e.selectionEnd
      } : t = RC(e), t || {
        start: 0,
        end: 0
      };
    }
    function D2(e, t) {
      var o = t.start, l = t.end;
      l === void 0 && (l = o), "selectionStart" in e ? (e.selectionStart = o, e.selectionEnd = Math.min(l, e.value.length)) : w2(e, t);
    }
    var N2 = Ot && "documentMode" in document && document.documentMode <= 11;
    function k2() {
      At("onSelect", ["focusout", "contextmenu", "dragend", "focusin", "keydown", "keyup", "mousedown", "mouseup", "selectionchange"]);
    }
    var md = null, AC = null, Hv = null, OC = !1;
    function M2(e) {
      if ("selectionStart" in e && IC(e))
        return {
          start: e.selectionStart,
          end: e.selectionEnd
        };
      var t = e.ownerDocument && e.ownerDocument.defaultView || window, o = t.getSelection();
      return {
        anchorNode: o.anchorNode,
        anchorOffset: o.anchorOffset,
        focusNode: o.focusNode,
        focusOffset: o.focusOffset
      };
    }
    function L2(e) {
      return e.window === e ? e.document : e.nodeType === ei ? e : e.ownerDocument;
    }
    function Cw(e, t, o) {
      var l = L2(o);
      if (!(OC || md == null || md !== qc(l))) {
        var d = M2(md);
        if (!Hv || !ot(Hv, d)) {
          Hv = d;
          var m = sy(AC, "onSelect");
          if (m.length > 0) {
            var y = new id("onSelect", "select", null, t, o);
            e.push({
              event: y,
              listeners: m
            }), y.target = md;
          }
        }
      }
    }
    function F2(e, t, o, l, d, m, y) {
      var x = o ? Sd(o) : window;
      switch (t) {
        case "focusin":
          (iy(x) || x.contentEditable === "true") && (md = x, AC = o, Hv = null);
          break;
        case "focusout":
          md = null, AC = null, Hv = null;
          break;
        case "mousedown":
          OC = !0;
          break;
        case "contextmenu":
        case "mouseup":
        case "dragend":
          OC = !1, Cw(e, l, d);
          break;
        case "selectionchange":
          if (N2)
            break;
        case "keydown":
        case "keyup":
          Cw(e, l, d);
      }
    }
    function oy(e, t) {
      var o = {};
      return o[e.toLowerCase()] = t.toLowerCase(), o["Webkit" + e] = "webkit" + t, o["Moz" + e] = "moz" + t, o;
    }
    var gd = {
      animationend: oy("Animation", "AnimationEnd"),
      animationiteration: oy("Animation", "AnimationIteration"),
      animationstart: oy("Animation", "AnimationStart"),
      transitionend: oy("Transition", "TransitionEnd")
    }, DC = {}, Sw = {};
    Ot && (Sw = document.createElement("div").style, "AnimationEvent" in window || (delete gd.animationend.animation, delete gd.animationiteration.animation, delete gd.animationstart.animation), "TransitionEvent" in window || delete gd.transitionend.transition);
    function uy(e) {
      if (DC[e])
        return DC[e];
      if (!gd[e])
        return e;
      var t = gd[e];
      for (var o in t)
        if (t.hasOwnProperty(o) && o in Sw)
          return DC[e] = t[o];
      return e;
    }
    var Ew = uy("animationend"), xw = uy("animationiteration"), ww = uy("animationstart"), Tw = uy("transitionend"), Rw = /* @__PURE__ */ new Map(), Iw = ["abort", "auxClick", "cancel", "canPlay", "canPlayThrough", "click", "close", "contextMenu", "copy", "cut", "drag", "dragEnd", "dragEnter", "dragExit", "dragLeave", "dragOver", "dragStart", "drop", "durationChange", "emptied", "encrypted", "ended", "error", "gotPointerCapture", "input", "invalid", "keyDown", "keyPress", "keyUp", "load", "loadedData", "loadedMetadata", "loadStart", "lostPointerCapture", "mouseDown", "mouseMove", "mouseOut", "mouseOver", "mouseUp", "paste", "pause", "play", "playing", "pointerCancel", "pointerDown", "pointerMove", "pointerOut", "pointerOver", "pointerUp", "progress", "rateChange", "reset", "resize", "seeked", "seeking", "stalled", "submit", "suspend", "timeUpdate", "touchCancel", "touchEnd", "touchStart", "volumeChange", "scroll", "toggle", "touchMove", "waiting", "wheel"];
    function Vl(e, t) {
      Rw.set(e, t), At(t, [e]);
    }
    function B2() {
      for (var e = 0; e < Iw.length; e++) {
        var t = Iw[e], o = t.toLowerCase(), l = t[0].toUpperCase() + t.slice(1);
        Vl(o, "on" + l);
      }
      Vl(Ew, "onAnimationEnd"), Vl(xw, "onAnimationIteration"), Vl(ww, "onAnimationStart"), Vl("dblclick", "onDoubleClick"), Vl("focusin", "onFocus"), Vl("focusout", "onBlur"), Vl(Tw, "onTransitionEnd");
    }
    function V2(e, t, o, l, d, m, y) {
      var x = Rw.get(t);
      if (x !== void 0) {
        var T = id, O = t;
        switch (t) {
          case "keypress":
            if (Cu(l) === 0)
              return;
          case "keydown":
          case "keyup":
            T = qg;
            break;
          case "focusin":
            O = "focus", T = rc;
            break;
          case "focusout":
            O = "blur", T = rc;
            break;
          case "beforeblur":
          case "afterblur":
            T = rc;
            break;
          case "click":
            if (l.button === 2)
              return;
          case "auxclick":
          case "dblclick":
          case "mousedown":
          case "mousemove":
          case "mouseup":
          case "mouseout":
          case "mouseover":
          case "contextmenu":
            T = od;
            break;
          case "drag":
          case "dragend":
          case "dragenter":
          case "dragexit":
          case "dragleave":
          case "dragover":
          case "dragstart":
          case "drop":
            T = Zg;
            break;
          case "touchcancel":
          case "touchend":
          case "touchmove":
          case "touchstart":
            T = Mv;
            break;
          case Ew:
          case xw:
          case ww:
            T = gC;
            break;
          case Tw:
            T = Oo;
            break;
          case "scroll":
            T = Ov;
            break;
          case "wheel":
            T = xu;
            break;
          case "copy":
          case "cut":
          case "paste":
            T = $g;
            break;
          case "gotpointercapture":
          case "lostpointercapture":
          case "pointercancel":
          case "pointerdown":
          case "pointermove":
          case "pointerout":
          case "pointerover":
          case "pointerup":
            T = ty;
            break;
        }
        var N = (m & nu) !== 0;
        {
          var j = !N && // TODO: ideally, we'd eventually add all events from
          // nonDelegatedEvents list in DOMPluginEventSystem.
          // Then we can remove this special list.
          // This is a breaking change that can wait until React 18.
          t === "scroll", G = G2(o, x, l.type, N, j);
          if (G.length > 0) {
            var te = new T(x, O, null, l, d);
            e.push({
              event: te,
              listeners: G
            });
          }
        }
      }
    }
    B2(), be(), n(), k2(), fd();
    function P2(e, t, o, l, d, m, y) {
      V2(e, t, o, l, d, m);
      var x = (m & aC) === 0;
      x && (Ze(e, t, o, l, d), ee(e, t, o, l, d), F2(e, t, o, l, d), TC(e, t, o, l, d));
    }
    var zv = ["abort", "canplay", "canplaythrough", "durationchange", "emptied", "encrypted", "ended", "error", "loadeddata", "loadedmetadata", "loadstart", "pause", "play", "playing", "progress", "ratechange", "resize", "seeked", "seeking", "stalled", "suspend", "timeupdate", "volumechange", "waiting"], NC = new Set(["cancel", "close", "invalid", "load", "scroll", "toggle"].concat(zv));
    function Aw(e, t, o) {
      var l = e.type || "unknown-event";
      e.currentTarget = o, zi(l, t, void 0, e), e.currentTarget = null;
    }
    function H2(e, t, o) {
      var l;
      if (o)
        for (var d = t.length - 1; d >= 0; d--) {
          var m = t[d], y = m.instance, x = m.currentTarget, T = m.listener;
          if (y !== l && e.isPropagationStopped())
            return;
          Aw(e, T, x), l = y;
        }
      else
        for (var O = 0; O < t.length; O++) {
          var N = t[O], j = N.instance, G = N.currentTarget, te = N.listener;
          if (j !== l && e.isPropagationStopped())
            return;
          Aw(e, te, G), l = j;
        }
    }
    function Ow(e, t) {
      for (var o = (t & nu) !== 0, l = 0; l < e.length; l++) {
        var d = e[l], m = d.event, y = d.listeners;
        H2(m, y, o);
      }
      rv();
    }
    function z2(e, t, o, l, d) {
      var m = pf(o), y = [];
      P2(y, e, l, o, m, t), Ow(y, t);
    }
    function dn(e, t) {
      NC.has(e) || h('Did not expect a listenToNonDelegatedEvent() call for "%s". This is a bug in React. Please file an issue.', e);
      var o = !1, l = mM(t), d = U2(e);
      l.has(d) || (Dw(t, e, Ns, o), l.add(d));
    }
    function kC(e, t, o) {
      NC.has(e) && !t && h('Did not expect a listenToNativeEvent() call for "%s" in the bubble phase. This is a bug in React. Please file an issue.', e);
      var l = 0;
      t && (l |= nu), Dw(o, e, l, t);
    }
    var ly = "_reactListening" + Math.random().toString(36).slice(2);
    function Gv(e) {
      if (!e[ly]) {
        e[ly] = !0, He.forEach(function(o) {
          o !== "selectionchange" && (NC.has(o) || kC(o, !1, e), kC(o, !0, e));
        });
        var t = e.nodeType === ei ? e : e.ownerDocument;
        t !== null && (t[ly] || (t[ly] = !0, kC("selectionchange", !1, t)));
      }
    }
    function Dw(e, t, o, l, d) {
      var m = zn(e, t, o), y = void 0;
      Bs && (t === "touchstart" || t === "touchmove" || t === "wheel") && (y = !0), e = e, l ? y !== void 0 ? nd(e, t, m, y) : Xi(e, t, m) : y !== void 0 ? Av(e, t, m, y) : Ml(e, t, m);
    }
    function Nw(e, t) {
      return e === t || e.nodeType === Dn && e.parentNode === t;
    }
    function MC(e, t, o, l, d) {
      var m = l;
      if (!(t & Pi) && !(t & Ns)) {
        var y = d;
        if (l !== null) {
          var x = l;
          e:
            for (; ; ) {
              if (x === null)
                return;
              var T = x.tag;
              if (T === R || T === A) {
                var O = x.stateNode.containerInfo;
                if (Nw(O, y))
                  break;
                if (T === A)
                  for (var N = x.return; N !== null; ) {
                    var j = N.tag;
                    if (j === R || j === A) {
                      var G = N.stateNode.containerInfo;
                      if (Nw(G, y))
                        return;
                    }
                    N = N.return;
                  }
                for (; O !== null; ) {
                  var te = fc(O);
                  if (te === null)
                    return;
                  var re = te.tag;
                  if (re === D || re === L) {
                    x = m = te;
                    continue e;
                  }
                  O = O.parentNode;
                }
              }
              x = x.return;
            }
        }
      }
      qp(function() {
        return z2(e, t, o, m);
      });
    }
    function _v(e, t, o) {
      return {
        instance: e,
        listener: t,
        currentTarget: o
      };
    }
    function G2(e, t, o, l, d, m) {
      for (var y = t !== null ? t + "Capture" : null, x = l ? y : t, T = [], O = e, N = null; O !== null; ) {
        var j = O, G = j.stateNode, te = j.tag;
        if (te === D && G !== null && (N = G, x !== null)) {
          var re = au(O, x);
          re != null && T.push(_v(O, re, N));
        }
        if (d)
          break;
        O = O.return;
      }
      return T;
    }
    function sy(e, t) {
      for (var o = t + "Capture", l = [], d = e; d !== null; ) {
        var m = d, y = m.stateNode, x = m.tag;
        if (x === D && y !== null) {
          var T = y, O = au(d, o);
          O != null && l.unshift(_v(d, O, T));
          var N = au(d, t);
          N != null && l.push(_v(d, N, T));
        }
        d = d.return;
      }
      return l;
    }
    function yd(e) {
      if (e === null)
        return null;
      do
        e = e.return;
      while (e && e.tag !== D);
      return e || null;
    }
    function _2(e, t) {
      for (var o = e, l = t, d = 0, m = o; m; m = yd(m))
        d++;
      for (var y = 0, x = l; x; x = yd(x))
        y++;
      for (; d - y > 0; )
        o = yd(o), d--;
      for (; y - d > 0; )
        l = yd(l), y--;
      for (var T = d; T--; ) {
        if (o === l || l !== null && o === l.alternate)
          return o;
        o = yd(o), l = yd(l);
      }
      return null;
    }
    function kw(e, t, o, l, d) {
      for (var m = t._reactName, y = [], x = o; x !== null && x !== l; ) {
        var T = x, O = T.alternate, N = T.stateNode, j = T.tag;
        if (O !== null && O === l)
          break;
        if (j === D && N !== null) {
          var G = N;
          if (d) {
            var te = au(x, m);
            te != null && y.unshift(_v(x, te, G));
          } else if (!d) {
            var re = au(x, m);
            re != null && y.push(_v(x, re, G));
          }
        }
        x = x.return;
      }
      y.length !== 0 && e.push({
        event: t,
        listeners: y
      });
    }
    function j2(e, t, o, l, d) {
      var m = l && d ? _2(l, d) : null;
      l !== null && kw(e, t, l, m, !1), d !== null && o !== null && kw(e, o, d, m, !0);
    }
    function U2(e, t) {
      return e + "__bubble";
    }
    var Ba = !1, jv = "dangerouslySetInnerHTML", cy = "suppressContentEditableWarning", Pl = "suppressHydrationWarning", Mw = "autoFocus", sc = "children", cc = "style", fy = "__html", LC, dy, Uv, Lw, py, Fw, Bw;
    LC = {
      // There are working polyfills for <dialog>. Let people use it.
      dialog: !0,
      // Electron ships a custom <webview> tag to display external web content in
      // an isolated frame and process.
      // This tag is not present in non Electron environments such as JSDom which
      // is often used for testing purposes.
      // @see https://electronjs.org/docs/api/webview-tag
      webview: !0
    }, dy = function(e, t) {
      df(e, t), Yp(e, t), gg(e, t, {
        registrationNameDependencies: nt,
        possibleRegistrationNames: vt
      });
    }, Fw = Ot && !document.documentMode, Uv = function(e, t, o) {
      if (!Ba) {
        var l = vy(o), d = vy(t);
        d !== l && (Ba = !0, h("Prop `%s` did not match. Server: %s Client: %s", e, JSON.stringify(d), JSON.stringify(l)));
      }
    }, Lw = function(e) {
      if (!Ba) {
        Ba = !0;
        var t = [];
        e.forEach(function(o) {
          t.push(o);
        }), h("Extra attributes from the server: %s", t);
      }
    }, py = function(e, t) {
      t === !1 ? h("Expected `%s` listener to be a function, instead got `false`.\n\nIf you used to conditionally omit it with %s={condition && value}, pass %s={condition ? value : undefined} instead.", e, e, e) : h("Expected `%s` listener to be a function, instead got a value of `%s` type.", e, typeof t);
    }, Bw = function(e, t) {
      var o = e.namespaceURI === Fi ? e.ownerDocument.createElement(e.tagName) : e.ownerDocument.createElementNS(e.namespaceURI, e.tagName);
      return o.innerHTML = t, o.innerHTML;
    };
    var W2 = /\r\n?/g, X2 = /\u0000|\uFFFD/g;
    function vy(e) {
      ci(e);
      var t = typeof e == "string" ? e : "" + e;
      return t.replace(W2, `
`).replace(X2, "");
    }
    function hy(e, t, o, l) {
      var d = vy(t), m = vy(e);
      if (m !== d && (l && (Ba || (Ba = !0, h('Text content did not match. Server: "%s" Client: "%s"', m, d))), o && tt))
        throw new Error("Text content does not match server-rendered HTML.");
    }
    function Vw(e) {
      return e.nodeType === ei ? e : e.ownerDocument;
    }
    function Z2() {
    }
    function my(e) {
      e.onclick = Z2;
    }
    function Y2(e, t, o, l, d) {
      for (var m in l)
        if (l.hasOwnProperty(m)) {
          var y = l[m];
          if (m === cc)
            y && Object.freeze(y), ug(t, y);
          else if (m === jv) {
            var x = y ? y[fy] : void 0;
            x != null && Qm(t, x);
          } else if (m === sc)
            if (typeof y == "string") {
              var T = e !== "textarea" || y !== "";
              T && lf(t, y);
            } else
              typeof y == "number" && lf(t, "" + y);
          else
            m === cy || m === Pl || m === Mw || (nt.hasOwnProperty(m) ? y != null && (typeof y != "function" && py(m, y), m === "onScroll" && dn("scroll", t)) : y != null && Qo(t, m, y, d));
        }
    }
    function $2(e, t, o, l) {
      for (var d = 0; d < t.length; d += 2) {
        var m = t[d], y = t[d + 1];
        m === cc ? ug(e, y) : m === jv ? Qm(e, y) : m === sc ? lf(e, y) : Qo(e, m, y, l);
      }
    }
    function Q2(e, t, o, l) {
      var d, m = Vw(o), y, x = l;
      if (x === Fi && (x = of(e)), x === Fi) {
        if (d = Vi(e, t), !d && e !== e.toLowerCase() && h("<%s /> is using incorrect casing. Use PascalCase for React components, or lowercase for HTML elements.", e), e === "script") {
          var T = m.createElement("div");
          T.innerHTML = "<script><\/script>";
          var O = T.firstChild;
          y = T.removeChild(O);
        } else if (typeof t.is == "string")
          y = m.createElement(e, {
            is: t.is
          });
        else if (y = m.createElement(e), e === "select") {
          var N = y;
          t.multiple ? N.multiple = !0 : t.size && (N.size = t.size);
        }
      } else
        y = m.createElementNS(x, e);
      return x === Fi && !d && Object.prototype.toString.call(y) === "[object HTMLUnknownElement]" && !_t.call(LC, e) && (LC[e] = !0, h("The tag <%s> is unrecognized in this browser. If you meant to render a React component, start its name with an uppercase letter.", e)), y;
    }
    function J2(e, t) {
      return Vw(t).createTextNode(e);
    }
    function K2(e, t, o, l) {
      var d = Vi(t, o);
      dy(t, o);
      var m;
      switch (t) {
        case "dialog":
          dn("cancel", e), dn("close", e), m = o;
          break;
        case "iframe":
        case "object":
        case "embed":
          dn("load", e), m = o;
          break;
        case "video":
        case "audio":
          for (var y = 0; y < zv.length; y++)
            dn(zv[y], e);
          m = o;
          break;
        case "source":
          dn("error", e), m = o;
          break;
        case "img":
        case "image":
        case "link":
          dn("error", e), dn("load", e), m = o;
          break;
        case "details":
          dn("toggle", e), m = o;
          break;
        case "input":
          Ts(e, o), m = ws(e, o), dn("invalid", e);
          break;
        case "option":
          rf(e, o), m = o;
          break;
        case "select":
          Xm(e, o), m = Vp(e, o), dn("invalid", e);
          break;
        case "textarea":
          Zm(e, o), m = Hp(e, o), dn("invalid", e);
          break;
        default:
          m = o;
      }
      switch (cf(t, m), Y2(t, e, l, m, d), t) {
        case "input":
          qo(e), Rs(e, o, !1);
          break;
        case "textarea":
          qo(e), $m(e);
          break;
        case "option":
          Bp(e, o);
          break;
        case "select":
          Zb(e, o);
          break;
        default:
          typeof m.onClick == "function" && my(e);
          break;
      }
    }
    function q2(e, t, o, l, d) {
      dy(t, l);
      var m = null, y, x;
      switch (t) {
        case "input":
          y = ws(e, o), x = ws(e, l), m = [];
          break;
        case "select":
          y = Vp(e, o), x = Vp(e, l), m = [];
          break;
        case "textarea":
          y = Hp(e, o), x = Hp(e, l), m = [];
          break;
        default:
          y = o, x = l, typeof y.onClick != "function" && typeof x.onClick == "function" && my(e);
          break;
      }
      cf(t, x);
      var T, O, N = null;
      for (T in y)
        if (!(x.hasOwnProperty(T) || !y.hasOwnProperty(T) || y[T] == null))
          if (T === cc) {
            var j = y[T];
            for (O in j)
              j.hasOwnProperty(O) && (N || (N = {}), N[O] = "");
          } else
            T === jv || T === sc || T === cy || T === Pl || T === Mw || (nt.hasOwnProperty(T) ? m || (m = []) : (m = m || []).push(T, null));
      for (T in x) {
        var G = x[T], te = y != null ? y[T] : void 0;
        if (!(!x.hasOwnProperty(T) || G === te || G == null && te == null))
          if (T === cc)
            if (G && Object.freeze(G), te) {
              for (O in te)
                te.hasOwnProperty(O) && (!G || !G.hasOwnProperty(O)) && (N || (N = {}), N[O] = "");
              for (O in G)
                G.hasOwnProperty(O) && te[O] !== G[O] && (N || (N = {}), N[O] = G[O]);
            } else
              N || (m || (m = []), m.push(T, N)), N = G;
          else if (T === jv) {
            var re = G ? G[fy] : void 0, ie = te ? te[fy] : void 0;
            re != null && ie !== re && (m = m || []).push(T, re);
          } else
            T === sc ? (typeof G == "string" || typeof G == "number") && (m = m || []).push(T, "" + G) : T === cy || T === Pl || (nt.hasOwnProperty(T) ? (G != null && (typeof G != "function" && py(T, G), T === "onScroll" && dn("scroll", e)), !m && te !== G && (m = [])) : (m = m || []).push(T, G));
      }
      return N && (Os(N, x[cc]), (m = m || []).push(cc, N)), m;
    }
    function ek(e, t, o, l, d) {
      o === "input" && d.type === "radio" && d.name != null && Fp(e, d);
      var m = Vi(o, l), y = Vi(o, d);
      switch ($2(e, t, m, y), o) {
        case "input":
          qu(e, d);
          break;
        case "textarea":
          Ym(e, d);
          break;
        case "select":
          Yb(e, d);
          break;
      }
    }
    function tk(e) {
      {
        var t = e.toLowerCase();
        return ff.hasOwnProperty(t) && ff[t] || null;
      }
    }
    function nk(e, t, o, l, d, m, y) {
      var x, T;
      switch (x = Vi(t, o), dy(t, o), t) {
        case "dialog":
          dn("cancel", e), dn("close", e);
          break;
        case "iframe":
        case "object":
        case "embed":
          dn("load", e);
          break;
        case "video":
        case "audio":
          for (var O = 0; O < zv.length; O++)
            dn(zv[O], e);
          break;
        case "source":
          dn("error", e);
          break;
        case "img":
        case "image":
        case "link":
          dn("error", e), dn("load", e);
          break;
        case "details":
          dn("toggle", e);
          break;
        case "input":
          Ts(e, o), dn("invalid", e);
          break;
        case "option":
          rf(e, o);
          break;
        case "select":
          Xm(e, o), dn("invalid", e);
          break;
        case "textarea":
          Zm(e, o), dn("invalid", e);
          break;
      }
      cf(t, o);
      {
        T = /* @__PURE__ */ new Set();
        for (var N = e.attributes, j = 0; j < N.length; j++) {
          var G = N[j].name.toLowerCase();
          switch (G) {
            case "value":
              break;
            case "checked":
              break;
            case "selected":
              break;
            default:
              T.add(N[j].name);
          }
        }
      }
      var te = null;
      for (var re in o)
        if (o.hasOwnProperty(re)) {
          var ie = o[re];
          if (re === sc)
            typeof ie == "string" ? e.textContent !== ie && (o[Pl] !== !0 && hy(e.textContent, ie, m, y), te = [sc, ie]) : typeof ie == "number" && e.textContent !== "" + ie && (o[Pl] !== !0 && hy(e.textContent, ie, m, y), te = [sc, "" + ie]);
          else if (nt.hasOwnProperty(re))
            ie != null && (typeof ie != "function" && py(re, ie), re === "onScroll" && dn("scroll", e));
          else if (y && // Convince Flow we've calculated it (it's DEV-only in this method.)
          typeof x == "boolean") {
            var Ne = void 0, et = x && Ie ? null : fi(re);
            if (o[Pl] !== !0) {
              if (!(re === cy || re === Pl || // Controlled attributes are not validated
              // TODO: Only ignore them on controlled tags.
              re === "value" || re === "checked" || re === "selected")) {
                if (re === jv) {
                  var Qe = e.innerHTML, Nt = ie ? ie[fy] : void 0;
                  if (Nt != null) {
                    var It = Bw(e, Nt);
                    It !== Qe && Uv(re, Qe, It);
                  }
                } else if (re === cc) {
                  if (T.delete(re), Fw) {
                    var K = nC(ie);
                    Ne = e.getAttribute("style"), K !== Ne && Uv(re, Ne, K);
                  }
                } else if (x && !Ie)
                  T.delete(re.toLowerCase()), Ne = Yc(e, re, ie), ie !== Ne && Uv(re, Ne, ie);
                else if (!Bn(re, et, x) && !vn(re, ie, et, x)) {
                  var oe = !1;
                  if (et !== null)
                    T.delete(et.attributeName), Ne = vs(e, re, ie, et);
                  else {
                    var q = l;
                    if (q === Fi && (q = of(t)), q === Fi)
                      T.delete(re.toLowerCase());
                    else {
                      var me = tk(re);
                      me !== null && me !== re && (oe = !0, T.delete(me)), T.delete(re);
                    }
                    Ne = Yc(e, re, ie);
                  }
                  var Ve = Ie;
                  !Ve && ie !== Ne && !oe && Uv(re, Ne, ie);
                }
              }
            }
          }
        }
      switch (y && // $FlowFixMe - Should be inferred as not undefined.
      T.size > 0 && o[Pl] !== !0 && Lw(T), t) {
        case "input":
          qo(e), Rs(e, o, !0);
          break;
        case "textarea":
          qo(e), $m(e);
          break;
        case "select":
        case "option":
          break;
        default:
          typeof o.onClick == "function" && my(e);
          break;
      }
      return te;
    }
    function rk(e, t, o) {
      var l = e.nodeValue !== t;
      return l;
    }
    function FC(e, t) {
      {
        if (Ba)
          return;
        Ba = !0, h("Did not expect server HTML to contain a <%s> in <%s>.", t.nodeName.toLowerCase(), e.nodeName.toLowerCase());
      }
    }
    function BC(e, t) {
      {
        if (Ba)
          return;
        Ba = !0, h('Did not expect server HTML to contain the text node "%s" in <%s>.', t.nodeValue, e.nodeName.toLowerCase());
      }
    }
    function VC(e, t, o) {
      {
        if (Ba)
          return;
        Ba = !0, h("Expected server HTML to contain a matching <%s> in <%s>.", t, e.nodeName.toLowerCase());
      }
    }
    function PC(e, t) {
      {
        if (t === "" || Ba)
          return;
        Ba = !0, h('Expected server HTML to contain a matching text node for "%s" in <%s>.', t, e.nodeName.toLowerCase());
      }
    }
    function ak(e, t, o) {
      switch (t) {
        case "input":
          _m(e, o);
          return;
        case "textarea":
          zp(e, o);
          return;
        case "select":
          $b(e, o);
          return;
      }
    }
    var Wv = function() {
    }, Xv = function() {
    };
    {
      var ik = ["address", "applet", "area", "article", "aside", "base", "basefont", "bgsound", "blockquote", "body", "br", "button", "caption", "center", "col", "colgroup", "dd", "details", "dir", "div", "dl", "dt", "embed", "fieldset", "figcaption", "figure", "footer", "form", "frame", "frameset", "h1", "h2", "h3", "h4", "h5", "h6", "head", "header", "hgroup", "hr", "html", "iframe", "img", "input", "isindex", "li", "link", "listing", "main", "marquee", "menu", "menuitem", "meta", "nav", "noembed", "noframes", "noscript", "object", "ol", "p", "param", "plaintext", "pre", "script", "section", "select", "source", "style", "summary", "table", "tbody", "td", "template", "textarea", "tfoot", "th", "thead", "title", "tr", "track", "ul", "wbr", "xmp"], Pw = [
        "applet",
        "caption",
        "html",
        "table",
        "td",
        "th",
        "marquee",
        "object",
        "template",
        // https://html.spec.whatwg.org/multipage/syntax.html#html-integration-point
        // TODO: Distinguish by namespace here -- for <title>, including it here
        // errs on the side of fewer warnings
        "foreignObject",
        "desc",
        "title"
      ], ok = Pw.concat(["button"]), uk = ["dd", "dt", "li", "option", "optgroup", "p", "rp", "rt"], Hw = {
        current: null,
        formTag: null,
        aTagInScope: null,
        buttonTagInScope: null,
        nobrTagInScope: null,
        pTagInButtonScope: null,
        listItemTagAutoclosing: null,
        dlItemTagAutoclosing: null
      };
      Xv = function(e, t) {
        var o = Tt({}, e || Hw), l = {
          tag: t
        };
        return Pw.indexOf(t) !== -1 && (o.aTagInScope = null, o.buttonTagInScope = null, o.nobrTagInScope = null), ok.indexOf(t) !== -1 && (o.pTagInButtonScope = null), ik.indexOf(t) !== -1 && t !== "address" && t !== "div" && t !== "p" && (o.listItemTagAutoclosing = null, o.dlItemTagAutoclosing = null), o.current = l, t === "form" && (o.formTag = l), t === "a" && (o.aTagInScope = l), t === "button" && (o.buttonTagInScope = l), t === "nobr" && (o.nobrTagInScope = l), t === "p" && (o.pTagInButtonScope = l), t === "li" && (o.listItemTagAutoclosing = l), (t === "dd" || t === "dt") && (o.dlItemTagAutoclosing = l), o;
      };
      var lk = function(e, t) {
        switch (t) {
          case "select":
            return e === "option" || e === "optgroup" || e === "#text";
          case "optgroup":
            return e === "option" || e === "#text";
          case "option":
            return e === "#text";
          case "tr":
            return e === "th" || e === "td" || e === "style" || e === "script" || e === "template";
          case "tbody":
          case "thead":
          case "tfoot":
            return e === "tr" || e === "style" || e === "script" || e === "template";
          case "colgroup":
            return e === "col" || e === "template";
          case "table":
            return e === "caption" || e === "colgroup" || e === "tbody" || e === "tfoot" || e === "thead" || e === "style" || e === "script" || e === "template";
          case "head":
            return e === "base" || e === "basefont" || e === "bgsound" || e === "link" || e === "meta" || e === "title" || e === "noscript" || e === "noframes" || e === "style" || e === "script" || e === "template";
          case "html":
            return e === "head" || e === "body" || e === "frameset";
          case "frameset":
            return e === "frame";
          case "#document":
            return e === "html";
        }
        switch (e) {
          case "h1":
          case "h2":
          case "h3":
          case "h4":
          case "h5":
          case "h6":
            return t !== "h1" && t !== "h2" && t !== "h3" && t !== "h4" && t !== "h5" && t !== "h6";
          case "rp":
          case "rt":
            return uk.indexOf(t) === -1;
          case "body":
          case "caption":
          case "col":
          case "colgroup":
          case "frameset":
          case "frame":
          case "head":
          case "html":
          case "tbody":
          case "td":
          case "tfoot":
          case "th":
          case "thead":
          case "tr":
            return t == null;
        }
        return !0;
      }, sk = function(e, t) {
        switch (e) {
          case "address":
          case "article":
          case "aside":
          case "blockquote":
          case "center":
          case "details":
          case "dialog":
          case "dir":
          case "div":
          case "dl":
          case "fieldset":
          case "figcaption":
          case "figure":
          case "footer":
          case "header":
          case "hgroup":
          case "main":
          case "menu":
          case "nav":
          case "ol":
          case "p":
          case "section":
          case "summary":
          case "ul":
          case "pre":
          case "listing":
          case "table":
          case "hr":
          case "xmp":
          case "h1":
          case "h2":
          case "h3":
          case "h4":
          case "h5":
          case "h6":
            return t.pTagInButtonScope;
          case "form":
            return t.formTag || t.pTagInButtonScope;
          case "li":
            return t.listItemTagAutoclosing;
          case "dd":
          case "dt":
            return t.dlItemTagAutoclosing;
          case "button":
            return t.buttonTagInScope;
          case "a":
            return t.aTagInScope;
          case "nobr":
            return t.nobrTagInScope;
        }
        return null;
      }, zw = {};
      Wv = function(e, t, o) {
        o = o || Hw;
        var l = o.current, d = l && l.tag;
        t != null && (e != null && h("validateDOMNesting: when childText is passed, childTag should be null"), e = "#text");
        var m = lk(e, d) ? null : l, y = m ? null : sk(e, o), x = m || y;
        if (x) {
          var T = x.tag, O = !!m + "|" + e + "|" + T;
          if (!zw[O]) {
            zw[O] = !0;
            var N = e, j = "";
            if (e === "#text" ? /\S/.test(t) ? N = "Text nodes" : (N = "Whitespace text nodes", j = " Make sure you don't have any extra whitespace between tags on each line of your source code.") : N = "<" + e + ">", m) {
              var G = "";
              T === "table" && e === "tr" && (G += " Add a <tbody>, <thead> or <tfoot> to your code to match the DOM tree generated by the browser."), h("validateDOMNesting(...): %s cannot appear as a child of <%s>.%s%s", N, T, j, G);
            } else
              h("validateDOMNesting(...): %s cannot appear as a descendant of <%s>.", N, T);
          }
        }
      };
    }
    var gy = "suppressHydrationWarning", yy = "$", by = "/$", Zv = "$?", Yv = "$!", ck = "style", HC = null, zC = null;
    function fk(e) {
      var t, o, l = e.nodeType;
      switch (l) {
        case ei:
        case eu: {
          t = l === ei ? "#document" : "#fragment";
          var d = e.documentElement;
          o = d ? d.namespaceURI : _p(null, "");
          break;
        }
        default: {
          var m = l === Dn ? e.parentNode : e, y = m.namespaceURI || null;
          t = m.tagName, o = _p(y, t);
          break;
        }
      }
      {
        var x = t.toLowerCase(), T = Xv(null, x);
        return {
          namespace: o,
          ancestorInfo: T
        };
      }
    }
    function dk(e, t, o) {
      {
        var l = e, d = _p(l.namespace, t), m = Xv(l.ancestorInfo, t);
        return {
          namespace: d,
          ancestorInfo: m
        };
      }
    }
    function S_(e) {
      return e;
    }
    function pk(e) {
      HC = Nl(), zC = I2();
      var t = null;
      return Zr(!1), t;
    }
    function vk(e) {
      A2(zC), Zr(HC), HC = null, zC = null;
    }
    function hk(e, t, o, l, d) {
      var m;
      {
        var y = l;
        if (Wv(e, null, y.ancestorInfo), typeof t.children == "string" || typeof t.children == "number") {
          var x = "" + t.children, T = Xv(y.ancestorInfo, e);
          Wv(null, x, T);
        }
        m = y.namespace;
      }
      var O = Q2(e, t, o, m);
      return Jv(d, O), YC(O, t), O;
    }
    function mk(e, t) {
      e.appendChild(t);
    }
    function gk(e, t, o, l, d) {
      switch (K2(e, t, o, l), t) {
        case "button":
        case "input":
        case "select":
        case "textarea":
          return !!o.autoFocus;
        case "img":
          return !0;
        default:
          return !1;
      }
    }
    function yk(e, t, o, l, d, m) {
      {
        var y = m;
        if (typeof l.children != typeof o.children && (typeof l.children == "string" || typeof l.children == "number")) {
          var x = "" + l.children, T = Xv(y.ancestorInfo, t);
          Wv(null, x, T);
        }
      }
      return q2(e, t, o, l);
    }
    function GC(e, t) {
      return e === "textarea" || e === "noscript" || typeof t.children == "string" || typeof t.children == "number" || typeof t.dangerouslySetInnerHTML == "object" && t.dangerouslySetInnerHTML !== null && t.dangerouslySetInnerHTML.__html != null;
    }
    function bk(e, t, o, l) {
      {
        var d = o;
        Wv(null, e, d.ancestorInfo);
      }
      var m = J2(e, t);
      return Jv(l, m), m;
    }
    function Ck() {
      var e = window.event;
      return e === void 0 ? gi : td(e.type);
    }
    var _C = typeof setTimeout == "function" ? setTimeout : void 0, Sk = typeof clearTimeout == "function" ? clearTimeout : void 0, jC = -1, Gw = typeof Promise == "function" ? Promise : void 0, Ek = typeof queueMicrotask == "function" ? queueMicrotask : typeof Gw < "u" ? function(e) {
      return Gw.resolve(null).then(e).catch(xk);
    } : _C;
    function xk(e) {
      setTimeout(function() {
        throw e;
      });
    }
    function wk(e, t, o, l) {
      switch (t) {
        case "button":
        case "input":
        case "select":
        case "textarea":
          o.autoFocus && e.focus();
          return;
        case "img": {
          o.src && (e.src = o.src);
          return;
        }
      }
    }
    function Tk(e, t, o, l, d, m) {
      ek(e, t, o, l, d), YC(e, d);
    }
    function _w(e) {
      lf(e, "");
    }
    function Rk(e, t, o) {
      e.nodeValue = o;
    }
    function Ik(e, t) {
      e.appendChild(t);
    }
    function Ak(e, t) {
      var o;
      e.nodeType === Dn ? (o = e.parentNode, o.insertBefore(t, e)) : (o = e, o.appendChild(t));
      var l = e._reactRootContainer;
      l == null && o.onclick === null && my(o);
    }
    function Ok(e, t, o) {
      e.insertBefore(t, o);
    }
    function Dk(e, t, o) {
      e.nodeType === Dn ? e.parentNode.insertBefore(t, o) : e.insertBefore(t, o);
    }
    function Nk(e, t) {
      e.removeChild(t);
    }
    function kk(e, t) {
      e.nodeType === Dn ? e.parentNode.removeChild(t) : e.removeChild(t);
    }
    function UC(e, t) {
      var o = t, l = 0;
      do {
        var d = o.nextSibling;
        if (e.removeChild(o), d && d.nodeType === Dn) {
          var m = d.data;
          if (m === by)
            if (l === 0) {
              e.removeChild(d), bn(t);
              return;
            } else
              l--;
          else
            (m === yy || m === Zv || m === Yv) && l++;
        }
        o = d;
      } while (o);
      bn(t);
    }
    function Mk(e, t) {
      e.nodeType === Dn ? UC(e.parentNode, t) : e.nodeType === Vr && UC(e, t), bn(e);
    }
    function Lk(e) {
      e = e;
      var t = e.style;
      typeof t.setProperty == "function" ? t.setProperty("display", "none", "important") : t.display = "none";
    }
    function Fk(e) {
      e.nodeValue = "";
    }
    function Bk(e, t) {
      e = e;
      var o = t[ck], l = o != null && o.hasOwnProperty("display") ? o.display : null;
      e.style.display = sf("display", l);
    }
    function Vk(e, t) {
      e.nodeValue = t;
    }
    function Pk(e) {
      e.nodeType === Vr ? e.textContent = "" : e.nodeType === ei && e.documentElement && e.removeChild(e.documentElement);
    }
    function Hk(e, t, o) {
      return e.nodeType !== Vr || t.toLowerCase() !== e.nodeName.toLowerCase() ? null : e;
    }
    function zk(e, t) {
      return t === "" || e.nodeType !== Bi ? null : e;
    }
    function Gk(e) {
      return e.nodeType !== Dn ? null : e;
    }
    function jw(e) {
      return e.data === Zv;
    }
    function WC(e) {
      return e.data === Yv;
    }
    function _k(e) {
      var t = e.nextSibling && e.nextSibling.dataset, o, l, d;
      return t && (o = t.dgst, l = t.msg, d = t.stck), {
        message: l,
        digest: o,
        stack: d
      };
    }
    function jk(e, t) {
      e._reactRetry = t;
    }
    function Cy(e) {
      for (; e != null; e = e.nextSibling) {
        var t = e.nodeType;
        if (t === Vr || t === Bi)
          break;
        if (t === Dn) {
          var o = e.data;
          if (o === yy || o === Yv || o === Zv)
            break;
          if (o === by)
            return null;
        }
      }
      return e;
    }
    function $v(e) {
      return Cy(e.nextSibling);
    }
    function Uk(e) {
      return Cy(e.firstChild);
    }
    function Wk(e) {
      return Cy(e.firstChild);
    }
    function Xk(e) {
      return Cy(e.nextSibling);
    }
    function Zk(e, t, o, l, d, m, y) {
      Jv(m, e), YC(e, o);
      var x;
      {
        var T = d;
        x = T.namespace;
      }
      var O = (m.mode & $e) !== Xe;
      return nk(e, t, o, x, l, O, y);
    }
    function Yk(e, t, o, l) {
      return Jv(o, e), o.mode & $e, rk(e, t);
    }
    function $k(e, t) {
      Jv(t, e);
    }
    function Qk(e) {
      for (var t = e.nextSibling, o = 0; t; ) {
        if (t.nodeType === Dn) {
          var l = t.data;
          if (l === by) {
            if (o === 0)
              return $v(t);
            o--;
          } else
            (l === yy || l === Yv || l === Zv) && o++;
        }
        t = t.nextSibling;
      }
      return null;
    }
    function Uw(e) {
      for (var t = e.previousSibling, o = 0; t; ) {
        if (t.nodeType === Dn) {
          var l = t.data;
          if (l === yy || l === Yv || l === Zv) {
            if (o === 0)
              return t;
            o--;
          } else
            l === by && o++;
        }
        t = t.previousSibling;
      }
      return null;
    }
    function Jk(e) {
      bn(e);
    }
    function Kk(e) {
      bn(e);
    }
    function qk(e) {
      return e !== "head" && e !== "body";
    }
    function eM(e, t, o, l) {
      var d = !0;
      hy(t.nodeValue, o, l, d);
    }
    function tM(e, t, o, l, d, m) {
      if (t[gy] !== !0) {
        var y = !0;
        hy(l.nodeValue, d, m, y);
      }
    }
    function nM(e, t) {
      t.nodeType === Vr ? FC(e, t) : t.nodeType === Dn || BC(e, t);
    }
    function rM(e, t) {
      {
        var o = e.parentNode;
        o !== null && (t.nodeType === Vr ? FC(o, t) : t.nodeType === Dn || BC(o, t));
      }
    }
    function aM(e, t, o, l, d) {
      (d || t[gy] !== !0) && (l.nodeType === Vr ? FC(o, l) : l.nodeType === Dn || BC(o, l));
    }
    function iM(e, t, o) {
      VC(e, t);
    }
    function oM(e, t) {
      PC(e, t);
    }
    function uM(e, t, o) {
      {
        var l = e.parentNode;
        l !== null && VC(l, t);
      }
    }
    function lM(e, t) {
      {
        var o = e.parentNode;
        o !== null && PC(o, t);
      }
    }
    function sM(e, t, o, l, d, m) {
      (m || t[gy] !== !0) && VC(o, l);
    }
    function cM(e, t, o, l, d) {
      (d || t[gy] !== !0) && PC(o, l);
    }
    function fM(e) {
      h("An error occurred during hydration. The server HTML was replaced with client content in <%s>.", e.nodeName.toLowerCase());
    }
    function dM(e) {
      Gv(e);
    }
    var bd = Math.random().toString(36).slice(2), Cd = "__reactFiber$" + bd, XC = "__reactProps$" + bd, Qv = "__reactContainer$" + bd, ZC = "__reactEvents$" + bd, pM = "__reactListeners$" + bd, vM = "__reactHandles$" + bd;
    function hM(e) {
      delete e[Cd], delete e[XC], delete e[ZC], delete e[pM], delete e[vM];
    }
    function Jv(e, t) {
      t[Cd] = e;
    }
    function Sy(e, t) {
      t[Qv] = e;
    }
    function Ww(e) {
      e[Qv] = null;
    }
    function Kv(e) {
      return !!e[Qv];
    }
    function fc(e) {
      var t = e[Cd];
      if (t)
        return t;
      for (var o = e.parentNode; o; ) {
        if (t = o[Qv] || o[Cd], t) {
          var l = t.alternate;
          if (t.child !== null || l !== null && l.child !== null)
            for (var d = Uw(e); d !== null; ) {
              var m = d[Cd];
              if (m)
                return m;
              d = Uw(d);
            }
          return t;
        }
        e = o, o = e.parentNode;
      }
      return null;
    }
    function Hl(e) {
      var t = e[Cd] || e[Qv];
      return t && (t.tag === D || t.tag === L || t.tag === V || t.tag === R) ? t : null;
    }
    function Sd(e) {
      if (e.tag === D || e.tag === L)
        return e.stateNode;
      throw new Error("getNodeFromInstance: Invalid argument.");
    }
    function Ey(e) {
      return e[XC] || null;
    }
    function YC(e, t) {
      e[XC] = t;
    }
    function mM(e) {
      var t = e[ZC];
      return t === void 0 && (t = e[ZC] = /* @__PURE__ */ new Set()), t;
    }
    var Xw = {}, Zw = u.ReactDebugCurrentFrame;
    function xy(e) {
      if (e) {
        var t = e._owner, o = ys(e.type, e._source, t ? t.type : null);
        Zw.setExtraStackFrame(o);
      } else
        Zw.setExtraStackFrame(null);
    }
    function $i(e, t, o, l, d) {
      {
        var m = Function.call.bind(_t);
        for (var y in e)
          if (m(e, y)) {
            var x = void 0;
            try {
              if (typeof e[y] != "function") {
                var T = Error((l || "React class") + ": " + o + " type `" + y + "` is invalid; it must be a function, usually from the `prop-types` package, but received `" + typeof e[y] + "`.This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`.");
                throw T.name = "Invariant Violation", T;
              }
              x = e[y](t, y, l, o, null, "SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED");
            } catch (O) {
              x = O;
            }
            x && !(x instanceof Error) && (xy(d), h("%s: type specification of %s `%s` is invalid; the type checker function must return `null` or an `Error` but returned a %s. You may have forgotten to pass an argument to the type checker creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and shape all require an argument).", l || "React class", o, y, typeof x), xy(null)), x instanceof Error && !(x.message in Xw) && (Xw[x.message] = !0, xy(d), h("Failed %s type: %s", o, x.message), xy(null));
          }
      }
    }
    var $C = [], wy;
    wy = [];
    var wu = -1;
    function zl(e) {
      return {
        current: e
      };
    }
    function Yr(e, t) {
      if (wu < 0) {
        h("Unexpected pop.");
        return;
      }
      t !== wy[wu] && h("Unexpected Fiber popped."), e.current = $C[wu], $C[wu] = null, wy[wu] = null, wu--;
    }
    function $r(e, t, o) {
      wu++, $C[wu] = e.current, wy[wu] = o, e.current = t;
    }
    var QC;
    QC = {};
    var ai = {};
    Object.freeze(ai);
    var Tu = zl(ai), No = zl(!1), JC = ai;
    function Ed(e, t, o) {
      return o && ko(t) ? JC : Tu.current;
    }
    function Yw(e, t, o) {
      {
        var l = e.stateNode;
        l.__reactInternalMemoizedUnmaskedChildContext = t, l.__reactInternalMemoizedMaskedChildContext = o;
      }
    }
    function xd(e, t) {
      {
        var o = e.type, l = o.contextTypes;
        if (!l)
          return ai;
        var d = e.stateNode;
        if (d && d.__reactInternalMemoizedUnmaskedChildContext === t)
          return d.__reactInternalMemoizedMaskedChildContext;
        var m = {};
        for (var y in l)
          m[y] = t[y];
        {
          var x = bt(e) || "Unknown";
          $i(l, m, "context", x);
        }
        return d && Yw(e, t, m), m;
      }
    }
    function Ty() {
      return No.current;
    }
    function ko(e) {
      {
        var t = e.childContextTypes;
        return t != null;
      }
    }
    function Ry(e) {
      Yr(No, e), Yr(Tu, e);
    }
    function KC(e) {
      Yr(No, e), Yr(Tu, e);
    }
    function $w(e, t, o) {
      {
        if (Tu.current !== ai)
          throw new Error("Unexpected context found on stack. This error is likely caused by a bug in React. Please file an issue.");
        $r(Tu, t, e), $r(No, o, e);
      }
    }
    function Qw(e, t, o) {
      {
        var l = e.stateNode, d = t.childContextTypes;
        if (typeof l.getChildContext != "function") {
          {
            var m = bt(e) || "Unknown";
            QC[m] || (QC[m] = !0, h("%s.childContextTypes is specified but there is no getChildContext() method on the instance. You can either define getChildContext() on %s or remove childContextTypes from it.", m, m));
          }
          return o;
        }
        var y = l.getChildContext();
        for (var x in y)
          if (!(x in d))
            throw new Error((bt(e) || "Unknown") + '.getChildContext(): key "' + x + '" is not defined in childContextTypes.');
        {
          var T = bt(e) || "Unknown";
          $i(d, y, "child context", T);
        }
        return Tt({}, o, y);
      }
    }
    function Iy(e) {
      {
        var t = e.stateNode, o = t && t.__reactInternalMemoizedMergedChildContext || ai;
        return JC = Tu.current, $r(Tu, o, e), $r(No, No.current, e), !0;
      }
    }
    function Jw(e, t, o) {
      {
        var l = e.stateNode;
        if (!l)
          throw new Error("Expected to have an instance by this point. This error is likely caused by a bug in React. Please file an issue.");
        if (o) {
          var d = Qw(e, t, JC);
          l.__reactInternalMemoizedMergedChildContext = d, Yr(No, e), Yr(Tu, e), $r(Tu, d, e), $r(No, o, e);
        } else
          Yr(No, e), $r(No, o, e);
      }
    }
    function gM(e) {
      {
        if (!lv(e) || e.tag !== E)
          throw new Error("Expected subtree parent to be a mounted class component. This error is likely caused by a bug in React. Please file an issue.");
        var t = e;
        do {
          switch (t.tag) {
            case R:
              return t.stateNode.context;
            case E: {
              var o = t.type;
              if (ko(o))
                return t.stateNode.__reactInternalMemoizedMergedChildContext;
              break;
            }
          }
          t = t.return;
        } while (t !== null);
        throw new Error("Found unexpected detached subtree parent. This error is likely caused by a bug in React. Please file an issue.");
      }
    }
    var Gl = 0, Ay = 1, Ru = null, qC = !1, e1 = !1;
    function Kw(e) {
      Ru === null ? Ru = [e] : Ru.push(e);
    }
    function yM(e) {
      qC = !0, Kw(e);
    }
    function qw() {
      qC && _l();
    }
    function _l() {
      if (!e1 && Ru !== null) {
        e1 = !0;
        var e = 0, t = Na();
        try {
          var o = !0, l = Ru;
          for (yn(wn); e < l.length; e++) {
            var d = l[e];
            do
              d = d(o);
            while (d !== null);
          }
          Ru = null, qC = !1;
        } catch (m) {
          throw Ru !== null && (Ru = Ru.slice(e + 1)), Cf(Ef, _l), m;
        } finally {
          yn(t), e1 = !1;
        }
      }
      return null;
    }
    var wd = [], Td = 0, Oy = null, Dy = 0, Si = [], Ei = 0, dc = null, Iu = 1, Au = "";
    function bM(e) {
      return vc(), (e.flags & iv) !== qe;
    }
    function CM(e) {
      return vc(), Dy;
    }
    function SM() {
      var e = Au, t = Iu, o = t & ~EM(t);
      return o.toString(32) + e;
    }
    function pc(e, t) {
      vc(), wd[Td++] = Dy, wd[Td++] = Oy, Oy = e, Dy = t;
    }
    function eT(e, t, o) {
      vc(), Si[Ei++] = Iu, Si[Ei++] = Au, Si[Ei++] = dc, dc = e;
      var l = Iu, d = Au, m = Ny(l) - 1, y = l & ~(1 << m), x = o + 1, T = Ny(t) + m;
      if (T > 30) {
        var O = m - m % 5, N = (1 << O) - 1, j = (y & N).toString(32), G = y >> O, te = m - O, re = Ny(t) + te, ie = x << te, Ne = ie | G, et = j + d;
        Iu = 1 << re | Ne, Au = et;
      } else {
        var Qe = x << m, Nt = Qe | y, It = d;
        Iu = 1 << T | Nt, Au = It;
      }
    }
    function t1(e) {
      vc();
      var t = e.return;
      if (t !== null) {
        var o = 1, l = 0;
        pc(e, o), eT(e, o, l);
      }
    }
    function Ny(e) {
      return 32 - vl(e);
    }
    function EM(e) {
      return 1 << Ny(e) - 1;
    }
    function n1(e) {
      for (; e === Oy; )
        Oy = wd[--Td], wd[Td] = null, Dy = wd[--Td], wd[Td] = null;
      for (; e === dc; )
        dc = Si[--Ei], Si[Ei] = null, Au = Si[--Ei], Si[Ei] = null, Iu = Si[--Ei], Si[Ei] = null;
    }
    function xM() {
      return vc(), dc !== null ? {
        id: Iu,
        overflow: Au
      } : null;
    }
    function wM(e, t) {
      vc(), Si[Ei++] = Iu, Si[Ei++] = Au, Si[Ei++] = dc, Iu = t.id, Au = t.overflow, dc = e;
    }
    function vc() {
      Tr() || h("Expected to be hydrating. This is a bug in React. Please file an issue.");
    }
    var wr = null, xi = null, Qi = !1, hc = !1, jl = null;
    function TM() {
      Qi && h("We should not be hydrating here. This is a bug in React. Please file a bug.");
    }
    function tT() {
      hc = !0;
    }
    function RM() {
      return hc;
    }
    function IM(e) {
      var t = e.stateNode.containerInfo;
      return xi = Wk(t), wr = e, Qi = !0, jl = null, hc = !1, !0;
    }
    function AM(e, t, o) {
      return xi = Xk(t), wr = e, Qi = !0, jl = null, hc = !1, o !== null && wM(e, o), !0;
    }
    function nT(e, t) {
      switch (e.tag) {
        case R: {
          nM(e.stateNode.containerInfo, t);
          break;
        }
        case D: {
          var o = (e.mode & $e) !== Xe;
          aM(
            e.type,
            e.memoizedProps,
            e.stateNode,
            t,
            // TODO: Delete this argument when we remove the legacy root API.
            o
          );
          break;
        }
        case V: {
          var l = e.memoizedState;
          l.dehydrated !== null && rM(l.dehydrated, t);
          break;
        }
      }
    }
    function rT(e, t) {
      nT(e, t);
      var o = k4();
      o.stateNode = t, o.return = e;
      var l = e.deletions;
      l === null ? (e.deletions = [o], e.flags |= Pt) : l.push(o);
    }
    function r1(e, t) {
      {
        if (hc)
          return;
        switch (e.tag) {
          case R: {
            var o = e.stateNode.containerInfo;
            switch (t.tag) {
              case D:
                var l = t.type;
                t.pendingProps, iM(o, l);
                break;
              case L:
                var d = t.pendingProps;
                oM(o, d);
                break;
            }
            break;
          }
          case D: {
            var m = e.type, y = e.memoizedProps, x = e.stateNode;
            switch (t.tag) {
              case D: {
                var T = t.type, O = t.pendingProps, N = (e.mode & $e) !== Xe;
                sM(
                  m,
                  y,
                  x,
                  T,
                  O,
                  // TODO: Delete this argument when we remove the legacy root API.
                  N
                );
                break;
              }
              case L: {
                var j = t.pendingProps, G = (e.mode & $e) !== Xe;
                cM(
                  m,
                  y,
                  x,
                  j,
                  // TODO: Delete this argument when we remove the legacy root API.
                  G
                );
                break;
              }
            }
            break;
          }
          case V: {
            var te = e.memoizedState, re = te.dehydrated;
            if (re !== null)
              switch (t.tag) {
                case D:
                  var ie = t.type;
                  t.pendingProps, uM(re, ie);
                  break;
                case L:
                  var Ne = t.pendingProps;
                  lM(re, Ne);
                  break;
              }
            break;
          }
          default:
            return;
        }
      }
    }
    function aT(e, t) {
      t.flags = t.flags & ~Ta | nn, r1(e, t);
    }
    function iT(e, t) {
      switch (e.tag) {
        case D: {
          var o = e.type;
          e.pendingProps;
          var l = Hk(t, o);
          return l !== null ? (e.stateNode = l, wr = e, xi = Uk(l), !0) : !1;
        }
        case L: {
          var d = e.pendingProps, m = zk(t, d);
          return m !== null ? (e.stateNode = m, wr = e, xi = null, !0) : !1;
        }
        case V: {
          var y = Gk(t);
          if (y !== null) {
            var x = {
              dehydrated: y,
              treeContext: xM(),
              retryLane: gr
            };
            e.memoizedState = x;
            var T = M4(y);
            return T.return = e, e.child = T, wr = e, xi = null, !0;
          }
          return !1;
        }
        default:
          return !1;
      }
    }
    function a1(e) {
      return (e.mode & $e) !== Xe && (e.flags & dt) === qe;
    }
    function i1(e) {
      throw new Error("Hydration failed because the initial UI does not match what was rendered on the server.");
    }
    function o1(e) {
      if (Qi) {
        var t = xi;
        if (!t) {
          a1(e) && (r1(wr, e), i1()), aT(wr, e), Qi = !1, wr = e;
          return;
        }
        var o = t;
        if (!iT(e, t)) {
          a1(e) && (r1(wr, e), i1()), t = $v(o);
          var l = wr;
          if (!t || !iT(e, t)) {
            aT(wr, e), Qi = !1, wr = e;
            return;
          }
          rT(l, o);
        }
      }
    }
    function OM(e, t, o) {
      var l = e.stateNode, d = !hc, m = Zk(l, e.type, e.memoizedProps, t, o, e, d);
      return e.updateQueue = m, m !== null;
    }
    function DM(e) {
      var t = e.stateNode, o = e.memoizedProps, l = Yk(t, o, e);
      if (l) {
        var d = wr;
        if (d !== null)
          switch (d.tag) {
            case R: {
              var m = d.stateNode.containerInfo, y = (d.mode & $e) !== Xe;
              eM(
                m,
                t,
                o,
                // TODO: Delete this argument when we remove the legacy root API.
                y
              );
              break;
            }
            case D: {
              var x = d.type, T = d.memoizedProps, O = d.stateNode, N = (d.mode & $e) !== Xe;
              tM(
                x,
                T,
                O,
                t,
                o,
                // TODO: Delete this argument when we remove the legacy root API.
                N
              );
              break;
            }
          }
      }
      return l;
    }
    function NM(e) {
      var t = e.memoizedState, o = t !== null ? t.dehydrated : null;
      if (!o)
        throw new Error("Expected to have a hydrated suspense instance. This error is likely caused by a bug in React. Please file an issue.");
      $k(o, e);
    }
    function kM(e) {
      var t = e.memoizedState, o = t !== null ? t.dehydrated : null;
      if (!o)
        throw new Error("Expected to have a hydrated suspense instance. This error is likely caused by a bug in React. Please file an issue.");
      return Qk(o);
    }
    function oT(e) {
      for (var t = e.return; t !== null && t.tag !== D && t.tag !== R && t.tag !== V; )
        t = t.return;
      wr = t;
    }
    function ky(e) {
      if (e !== wr)
        return !1;
      if (!Qi)
        return oT(e), Qi = !0, !1;
      if (e.tag !== R && (e.tag !== D || qk(e.type) && !GC(e.type, e.memoizedProps))) {
        var t = xi;
        if (t)
          if (a1(e))
            uT(e), i1();
          else
            for (; t; )
              rT(e, t), t = $v(t);
      }
      return oT(e), e.tag === V ? xi = kM(e) : xi = wr ? $v(e.stateNode) : null, !0;
    }
    function MM() {
      return Qi && xi !== null;
    }
    function uT(e) {
      for (var t = xi; t; )
        nT(e, t), t = $v(t);
    }
    function Rd() {
      wr = null, xi = null, Qi = !1, hc = !1;
    }
    function lT() {
      jl !== null && (tI(jl), jl = null);
    }
    function Tr() {
      return Qi;
    }
    function u1(e) {
      jl === null ? jl = [e] : jl.push(e);
    }
    var LM = u.ReactCurrentBatchConfig, FM = null;
    function BM() {
      return LM.transition;
    }
    var Ji = {
      recordUnsafeLifecycleWarnings: function(e, t) {
      },
      flushPendingUnsafeLifecycleWarnings: function() {
      },
      recordLegacyContextWarning: function(e, t) {
      },
      flushLegacyContextWarning: function() {
      },
      discardPendingWarnings: function() {
      }
    };
    {
      var VM = function(e) {
        for (var t = null, o = e; o !== null; )
          o.mode & xt && (t = o), o = o.return;
        return t;
      }, mc = function(e) {
        var t = [];
        return e.forEach(function(o) {
          t.push(o);
        }), t.sort().join(", ");
      }, qv = [], eh = [], th = [], nh = [], rh = [], ah = [], gc = /* @__PURE__ */ new Set();
      Ji.recordUnsafeLifecycleWarnings = function(e, t) {
        gc.has(e.type) || (typeof t.componentWillMount == "function" && // Don't warn about react-lifecycles-compat polyfilled components.
        t.componentWillMount.__suppressDeprecationWarning !== !0 && qv.push(e), e.mode & xt && typeof t.UNSAFE_componentWillMount == "function" && eh.push(e), typeof t.componentWillReceiveProps == "function" && t.componentWillReceiveProps.__suppressDeprecationWarning !== !0 && th.push(e), e.mode & xt && typeof t.UNSAFE_componentWillReceiveProps == "function" && nh.push(e), typeof t.componentWillUpdate == "function" && t.componentWillUpdate.__suppressDeprecationWarning !== !0 && rh.push(e), e.mode & xt && typeof t.UNSAFE_componentWillUpdate == "function" && ah.push(e));
      }, Ji.flushPendingUnsafeLifecycleWarnings = function() {
        var e = /* @__PURE__ */ new Set();
        qv.length > 0 && (qv.forEach(function(G) {
          e.add(bt(G) || "Component"), gc.add(G.type);
        }), qv = []);
        var t = /* @__PURE__ */ new Set();
        eh.length > 0 && (eh.forEach(function(G) {
          t.add(bt(G) || "Component"), gc.add(G.type);
        }), eh = []);
        var o = /* @__PURE__ */ new Set();
        th.length > 0 && (th.forEach(function(G) {
          o.add(bt(G) || "Component"), gc.add(G.type);
        }), th = []);
        var l = /* @__PURE__ */ new Set();
        nh.length > 0 && (nh.forEach(function(G) {
          l.add(bt(G) || "Component"), gc.add(G.type);
        }), nh = []);
        var d = /* @__PURE__ */ new Set();
        rh.length > 0 && (rh.forEach(function(G) {
          d.add(bt(G) || "Component"), gc.add(G.type);
        }), rh = []);
        var m = /* @__PURE__ */ new Set();
        if (ah.length > 0 && (ah.forEach(function(G) {
          m.add(bt(G) || "Component"), gc.add(G.type);
        }), ah = []), t.size > 0) {
          var y = mc(t);
          h(`Using UNSAFE_componentWillMount in strict mode is not recommended and may indicate bugs in your code. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move code with side effects to componentDidMount, and set initial state in the constructor.

Please update the following components: %s`, y);
        }
        if (l.size > 0) {
          var x = mc(l);
          h(`Using UNSAFE_componentWillReceiveProps in strict mode is not recommended and may indicate bugs in your code. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move data fetching code or side effects to componentDidUpdate.
* If you're updating state whenever props change, refactor your code to use memoization techniques or move it to static getDerivedStateFromProps. Learn more at: https://reactjs.org/link/derived-state

Please update the following components: %s`, x);
        }
        if (m.size > 0) {
          var T = mc(m);
          h(`Using UNSAFE_componentWillUpdate in strict mode is not recommended and may indicate bugs in your code. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move data fetching code or side effects to componentDidUpdate.

Please update the following components: %s`, T);
        }
        if (e.size > 0) {
          var O = mc(e);
          v(`componentWillMount has been renamed, and is not recommended for use. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move code with side effects to componentDidMount, and set initial state in the constructor.
* Rename componentWillMount to UNSAFE_componentWillMount to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run \`npx react-codemod rename-unsafe-lifecycles\` in your project source folder.

Please update the following components: %s`, O);
        }
        if (o.size > 0) {
          var N = mc(o);
          v(`componentWillReceiveProps has been renamed, and is not recommended for use. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move data fetching code or side effects to componentDidUpdate.
* If you're updating state whenever props change, refactor your code to use memoization techniques or move it to static getDerivedStateFromProps. Learn more at: https://reactjs.org/link/derived-state
* Rename componentWillReceiveProps to UNSAFE_componentWillReceiveProps to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run \`npx react-codemod rename-unsafe-lifecycles\` in your project source folder.

Please update the following components: %s`, N);
        }
        if (d.size > 0) {
          var j = mc(d);
          v(`componentWillUpdate has been renamed, and is not recommended for use. See https://reactjs.org/link/unsafe-component-lifecycles for details.

* Move data fetching code or side effects to componentDidUpdate.
* Rename componentWillUpdate to UNSAFE_componentWillUpdate to suppress this warning in non-strict mode. In React 18.x, only the UNSAFE_ name will work. To rename all deprecated lifecycles to their new names, you can run \`npx react-codemod rename-unsafe-lifecycles\` in your project source folder.

Please update the following components: %s`, j);
        }
      };
      var My = /* @__PURE__ */ new Map(), sT = /* @__PURE__ */ new Set();
      Ji.recordLegacyContextWarning = function(e, t) {
        var o = VM(e);
        if (o === null) {
          h("Expected to find a StrictMode component in a strict mode tree. This error is likely caused by a bug in React. Please file an issue.");
          return;
        }
        if (!sT.has(e.type)) {
          var l = My.get(o);
          (e.type.contextTypes != null || e.type.childContextTypes != null || t !== null && typeof t.getChildContext == "function") && (l === void 0 && (l = [], My.set(o, l)), l.push(e));
        }
      }, Ji.flushLegacyContextWarning = function() {
        My.forEach(function(e, t) {
          if (e.length !== 0) {
            var o = e[0], l = /* @__PURE__ */ new Set();
            e.forEach(function(m) {
              l.add(bt(m) || "Component"), sT.add(m.type);
            });
            var d = mc(l);
            try {
              Zt(o), h(`Legacy context API has been detected within a strict-mode tree.

The old API will be supported in all 16.x releases, but applications using it should migrate to the new version.

Please update the following components: %s

Learn more about this warning here: https://reactjs.org/link/legacy-context`, d);
            } finally {
              Vn();
            }
          }
        });
      }, Ji.discardPendingWarnings = function() {
        qv = [], eh = [], th = [], nh = [], rh = [], ah = [], My = /* @__PURE__ */ new Map();
      };
    }
    var l1, s1, c1, f1, d1, cT = function(e, t) {
    };
    l1 = !1, s1 = !1, c1 = {}, f1 = {}, d1 = {}, cT = function(e, t) {
      if (!(e === null || typeof e != "object") && !(!e._store || e._store.validated || e.key != null)) {
        if (typeof e._store != "object")
          throw new Error("React Component in warnForMissingKey should have a _store. This error is likely caused by a bug in React. Please file an issue.");
        e._store.validated = !0;
        var o = bt(t) || "Component";
        f1[o] || (f1[o] = !0, h('Each child in a list should have a unique "key" prop. See https://reactjs.org/link/warning-keys for more information.'));
      }
    };
    function PM(e) {
      return e.prototype && e.prototype.isReactComponent;
    }
    function ih(e, t, o) {
      var l = o.ref;
      if (l !== null && typeof l != "function" && typeof l != "object") {
        if ((e.mode & xt || Ae) && // We warn in ReactElement.js if owner and self are equal for string refs
        // because these cannot be automatically converted to an arrow function
        // using a codemod. Therefore, we don't have to warn about string refs again.
        !(o._owner && o._self && o._owner.stateNode !== o._self) && // Will already throw with "Function components cannot have string refs"
        !(o._owner && o._owner.tag !== E) && // Will already warn with "Function components cannot be given refs"
        !(typeof o.type == "function" && !PM(o.type)) && // Will already throw with "Element ref was specified as a string (someStringRef) but no owner was set"
        o._owner) {
          var d = bt(e) || "Component";
          c1[d] || (h('Component "%s" contains the string ref "%s". Support for string refs will be removed in a future major release. We recommend using useRef() or createRef() instead. Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref', d, l), c1[d] = !0);
        }
        if (o._owner) {
          var m = o._owner, y;
          if (m) {
            var x = m;
            if (x.tag !== E)
              throw new Error("Function components cannot have string refs. We recommend using useRef() instead. Learn more about using refs safely here: https://reactjs.org/link/strict-mode-string-ref");
            y = x.stateNode;
          }
          if (!y)
            throw new Error("Missing owner for string ref " + l + ". This error is likely caused by a bug in React. Please file an issue.");
          var T = y;
          Sr(l, "ref");
          var O = "" + l;
          if (t !== null && t.ref !== null && typeof t.ref == "function" && t.ref._stringRef === O)
            return t.ref;
          var N = function(j) {
            var G = T.refs;
            j === null ? delete G[O] : G[O] = j;
          };
          return N._stringRef = O, N;
        } else {
          if (typeof l != "string")
            throw new Error("Expected ref to be a function, a string, an object returned by React.createRef(), or null.");
          if (!o._owner)
            throw new Error("Element ref was specified as a string (" + l + `) but no owner was set. This could happen for one of the following reasons:
1. You may be adding a ref to a function component
2. You may be adding a ref to a component that was not created inside a component's render method
3. You have multiple copies of React loaded
See https://reactjs.org/link/refs-must-have-owner for more information.`);
        }
      }
      return l;
    }
    function Ly(e, t) {
      var o = Object.prototype.toString.call(t);
      throw new Error("Objects are not valid as a React child (found: " + (o === "[object Object]" ? "object with keys {" + Object.keys(t).join(", ") + "}" : o) + "). If you meant to render a collection of children, use an array instead.");
    }
    function Fy(e) {
      {
        var t = bt(e) || "Component";
        if (d1[t])
          return;
        d1[t] = !0, h("Functions are not valid as a React child. This may happen if you return a Component instead of <Component /> from render. Or maybe you meant to call this function rather than return it.");
      }
    }
    function fT(e) {
      var t = e._payload, o = e._init;
      return o(t);
    }
    function dT(e) {
      function t(K, oe) {
        if (e) {
          var q = K.deletions;
          q === null ? (K.deletions = [oe], K.flags |= Pt) : q.push(oe);
        }
      }
      function o(K, oe) {
        if (!e)
          return null;
        for (var q = oe; q !== null; )
          t(K, q), q = q.sibling;
        return null;
      }
      function l(K, oe) {
        for (var q = /* @__PURE__ */ new Map(), me = oe; me !== null; )
          me.key !== null ? q.set(me.key, me) : q.set(me.index, me), me = me.sibling;
        return q;
      }
      function d(K, oe) {
        var q = Rc(K, oe);
        return q.index = 0, q.sibling = null, q;
      }
      function m(K, oe, q) {
        if (K.index = q, !e)
          return K.flags |= iv, oe;
        var me = K.alternate;
        if (me !== null) {
          var Ve = me.index;
          return Ve < oe ? (K.flags |= nn, oe) : Ve;
        } else
          return K.flags |= nn, oe;
      }
      function y(K) {
        return e && K.alternate === null && (K.flags |= nn), K;
      }
      function x(K, oe, q, me) {
        if (oe === null || oe.tag !== L) {
          var Ve = uE(q, K.mode, me);
          return Ve.return = K, Ve;
        } else {
          var Le = d(oe, q);
          return Le.return = K, Le;
        }
      }
      function T(K, oe, q, me) {
        var Ve = q.type;
        if (Ve === ki)
          return N(K, oe, q.props.children, me, q.key);
        if (oe !== null && (oe.elementType === Ve || // Keep this check inline so it only runs on the false path:
        gI(oe, q) || // Lazy types should reconcile their resolved type.
        // We need to do this after the Hot Reloading check above,
        // because hot reloading has different semantics than prod because
        // it doesn't resuspend. So we can't let the call below suspend.
        typeof Ve == "object" && Ve !== null && Ve.$$typeof === Er && fT(Ve) === oe.type)) {
          var Le = d(oe, q.props);
          return Le.ref = ih(K, oe, q), Le.return = K, Le._debugSource = q._source, Le._debugOwner = q._owner, Le;
        }
        var ct = oE(q, K.mode, me);
        return ct.ref = ih(K, oe, q), ct.return = K, ct;
      }
      function O(K, oe, q, me) {
        if (oe === null || oe.tag !== A || oe.stateNode.containerInfo !== q.containerInfo || oe.stateNode.implementation !== q.implementation) {
          var Ve = lE(q, K.mode, me);
          return Ve.return = K, Ve;
        } else {
          var Le = d(oe, q.children || []);
          return Le.return = K, Le;
        }
      }
      function N(K, oe, q, me, Ve) {
        if (oe === null || oe.tag !== M) {
          var Le = es(q, K.mode, me, Ve);
          return Le.return = K, Le;
        } else {
          var ct = d(oe, q);
          return ct.return = K, ct;
        }
      }
      function j(K, oe, q) {
        if (typeof oe == "string" && oe !== "" || typeof oe == "number") {
          var me = uE("" + oe, K.mode, q);
          return me.return = K, me;
        }
        if (typeof oe == "object" && oe !== null) {
          switch (oe.$$typeof) {
            case Xu: {
              var Ve = oE(oe, K.mode, q);
              return Ve.ref = ih(K, null, oe), Ve.return = K, Ve;
            }
            case Qa: {
              var Le = lE(oe, K.mode, q);
              return Le.return = K, Le;
            }
            case Er: {
              var ct = oe._payload, mt = oe._init;
              return j(K, mt(ct), q);
            }
          }
          if (Wn(oe) || Mi(oe)) {
            var $t = es(oe, K.mode, q, null);
            return $t.return = K, $t;
          }
          Ly(K, oe);
        }
        return typeof oe == "function" && Fy(K), null;
      }
      function G(K, oe, q, me) {
        var Ve = oe !== null ? oe.key : null;
        if (typeof q == "string" && q !== "" || typeof q == "number")
          return Ve !== null ? null : x(K, oe, "" + q, me);
        if (typeof q == "object" && q !== null) {
          switch (q.$$typeof) {
            case Xu:
              return q.key === Ve ? T(K, oe, q, me) : null;
            case Qa:
              return q.key === Ve ? O(K, oe, q, me) : null;
            case Er: {
              var Le = q._payload, ct = q._init;
              return G(K, oe, ct(Le), me);
            }
          }
          if (Wn(q) || Mi(q))
            return Ve !== null ? null : N(K, oe, q, me, null);
          Ly(K, q);
        }
        return typeof q == "function" && Fy(K), null;
      }
      function te(K, oe, q, me, Ve) {
        if (typeof me == "string" && me !== "" || typeof me == "number") {
          var Le = K.get(q) || null;
          return x(oe, Le, "" + me, Ve);
        }
        if (typeof me == "object" && me !== null) {
          switch (me.$$typeof) {
            case Xu: {
              var ct = K.get(me.key === null ? q : me.key) || null;
              return T(oe, ct, me, Ve);
            }
            case Qa: {
              var mt = K.get(me.key === null ? q : me.key) || null;
              return O(oe, mt, me, Ve);
            }
            case Er:
              var $t = me._payload, Bt = me._init;
              return te(K, oe, q, Bt($t), Ve);
          }
          if (Wn(me) || Mi(me)) {
            var jn = K.get(q) || null;
            return N(oe, jn, me, Ve, null);
          }
          Ly(oe, me);
        }
        return typeof me == "function" && Fy(oe), null;
      }
      function re(K, oe, q) {
        {
          if (typeof K != "object" || K === null)
            return oe;
          switch (K.$$typeof) {
            case Xu:
            case Qa:
              cT(K, q);
              var me = K.key;
              if (typeof me != "string")
                break;
              if (oe === null) {
                oe = /* @__PURE__ */ new Set(), oe.add(me);
                break;
              }
              if (!oe.has(me)) {
                oe.add(me);
                break;
              }
              h("Encountered two children with the same key, `%s`. Keys should be unique so that components maintain their identity across updates. Non-unique keys may cause children to be duplicated and/or omitted — the behavior is unsupported and could change in a future version.", me);
              break;
            case Er:
              var Ve = K._payload, Le = K._init;
              re(Le(Ve), oe, q);
              break;
          }
        }
        return oe;
      }
      function ie(K, oe, q, me) {
        for (var Ve = null, Le = 0; Le < q.length; Le++) {
          var ct = q[Le];
          Ve = re(ct, Ve, K);
        }
        for (var mt = null, $t = null, Bt = oe, jn = 0, Vt = 0, Mn = null; Bt !== null && Vt < q.length; Vt++) {
          Bt.index > Vt ? (Mn = Bt, Bt = null) : Mn = Bt.sibling;
          var Jr = G(K, Bt, q[Vt], me);
          if (Jr === null) {
            Bt === null && (Bt = Mn);
            break;
          }
          e && Bt && Jr.alternate === null && t(K, Bt), jn = m(Jr, jn, Vt), $t === null ? mt = Jr : $t.sibling = Jr, $t = Jr, Bt = Mn;
        }
        if (Vt === q.length) {
          if (o(K, Bt), Tr()) {
            var kr = Vt;
            pc(K, kr);
          }
          return mt;
        }
        if (Bt === null) {
          for (; Vt < q.length; Vt++) {
            var oi = j(K, q[Vt], me);
            oi !== null && (jn = m(oi, jn, Vt), $t === null ? mt = oi : $t.sibling = oi, $t = oi);
          }
          if (Tr()) {
            var pa = Vt;
            pc(K, pa);
          }
          return mt;
        }
        for (var va = l(K, Bt); Vt < q.length; Vt++) {
          var Kr = te(va, K, Vt, q[Vt], me);
          Kr !== null && (e && Kr.alternate !== null && va.delete(Kr.key === null ? Vt : Kr.key), jn = m(Kr, jn, Vt), $t === null ? mt = Kr : $t.sibling = Kr, $t = Kr);
        }
        if (e && va.forEach(function(Wd) {
          return t(K, Wd);
        }), Tr()) {
          var Fu = Vt;
          pc(K, Fu);
        }
        return mt;
      }
      function Ne(K, oe, q, me) {
        var Ve = Mi(q);
        if (typeof Ve != "function")
          throw new Error("An object is not an iterable. This error is likely caused by a bug in React. Please file an issue.");
        {
          typeof Symbol == "function" && // $FlowFixMe Flow doesn't know about toStringTag
          q[Symbol.toStringTag] === "Generator" && (s1 || h("Using Generators as children is unsupported and will likely yield unexpected results because enumerating a generator mutates it. You may convert it to an array with `Array.from()` or the `[...spread]` operator before rendering. Keep in mind you might need to polyfill these features for older browsers."), s1 = !0), q.entries === Ve && (l1 || h("Using Maps as children is not supported. Use an array of keyed ReactElements instead."), l1 = !0);
          var Le = Ve.call(q);
          if (Le)
            for (var ct = null, mt = Le.next(); !mt.done; mt = Le.next()) {
              var $t = mt.value;
              ct = re($t, ct, K);
            }
        }
        var Bt = Ve.call(q);
        if (Bt == null)
          throw new Error("An iterable object provided no iterator.");
        for (var jn = null, Vt = null, Mn = oe, Jr = 0, kr = 0, oi = null, pa = Bt.next(); Mn !== null && !pa.done; kr++, pa = Bt.next()) {
          Mn.index > kr ? (oi = Mn, Mn = null) : oi = Mn.sibling;
          var va = G(K, Mn, pa.value, me);
          if (va === null) {
            Mn === null && (Mn = oi);
            break;
          }
          e && Mn && va.alternate === null && t(K, Mn), Jr = m(va, Jr, kr), Vt === null ? jn = va : Vt.sibling = va, Vt = va, Mn = oi;
        }
        if (pa.done) {
          if (o(K, Mn), Tr()) {
            var Kr = kr;
            pc(K, Kr);
          }
          return jn;
        }
        if (Mn === null) {
          for (; !pa.done; kr++, pa = Bt.next()) {
            var Fu = j(K, pa.value, me);
            Fu !== null && (Jr = m(Fu, Jr, kr), Vt === null ? jn = Fu : Vt.sibling = Fu, Vt = Fu);
          }
          if (Tr()) {
            var Wd = kr;
            pc(K, Wd);
          }
          return jn;
        }
        for (var Vh = l(K, Mn); !pa.done; kr++, pa = Bt.next()) {
          var zo = te(Vh, K, kr, pa.value, me);
          zo !== null && (e && zo.alternate !== null && Vh.delete(zo.key === null ? kr : zo.key), Jr = m(zo, Jr, kr), Vt === null ? jn = zo : Vt.sibling = zo, Vt = zo);
        }
        if (e && Vh.forEach(function(cB) {
          return t(K, cB);
        }), Tr()) {
          var sB = kr;
          pc(K, sB);
        }
        return jn;
      }
      function et(K, oe, q, me) {
        if (oe !== null && oe.tag === L) {
          o(K, oe.sibling);
          var Ve = d(oe, q);
          return Ve.return = K, Ve;
        }
        o(K, oe);
        var Le = uE(q, K.mode, me);
        return Le.return = K, Le;
      }
      function Qe(K, oe, q, me) {
        for (var Ve = q.key, Le = oe; Le !== null; ) {
          if (Le.key === Ve) {
            var ct = q.type;
            if (ct === ki) {
              if (Le.tag === M) {
                o(K, Le.sibling);
                var mt = d(Le, q.props.children);
                return mt.return = K, mt._debugSource = q._source, mt._debugOwner = q._owner, mt;
              }
            } else if (Le.elementType === ct || // Keep this check inline so it only runs on the false path:
            gI(Le, q) || // Lazy types should reconcile their resolved type.
            // We need to do this after the Hot Reloading check above,
            // because hot reloading has different semantics than prod because
            // it doesn't resuspend. So we can't let the call below suspend.
            typeof ct == "object" && ct !== null && ct.$$typeof === Er && fT(ct) === Le.type) {
              o(K, Le.sibling);
              var $t = d(Le, q.props);
              return $t.ref = ih(K, Le, q), $t.return = K, $t._debugSource = q._source, $t._debugOwner = q._owner, $t;
            }
            o(K, Le);
            break;
          } else
            t(K, Le);
          Le = Le.sibling;
        }
        if (q.type === ki) {
          var Bt = es(q.props.children, K.mode, me, q.key);
          return Bt.return = K, Bt;
        } else {
          var jn = oE(q, K.mode, me);
          return jn.ref = ih(K, oe, q), jn.return = K, jn;
        }
      }
      function Nt(K, oe, q, me) {
        for (var Ve = q.key, Le = oe; Le !== null; ) {
          if (Le.key === Ve)
            if (Le.tag === A && Le.stateNode.containerInfo === q.containerInfo && Le.stateNode.implementation === q.implementation) {
              o(K, Le.sibling);
              var ct = d(Le, q.children || []);
              return ct.return = K, ct;
            } else {
              o(K, Le);
              break;
            }
          else
            t(K, Le);
          Le = Le.sibling;
        }
        var mt = lE(q, K.mode, me);
        return mt.return = K, mt;
      }
      function It(K, oe, q, me) {
        var Ve = typeof q == "object" && q !== null && q.type === ki && q.key === null;
        if (Ve && (q = q.props.children), typeof q == "object" && q !== null) {
          switch (q.$$typeof) {
            case Xu:
              return y(Qe(K, oe, q, me));
            case Qa:
              return y(Nt(K, oe, q, me));
            case Er:
              var Le = q._payload, ct = q._init;
              return It(K, oe, ct(Le), me);
          }
          if (Wn(q))
            return ie(K, oe, q, me);
          if (Mi(q))
            return Ne(K, oe, q, me);
          Ly(K, q);
        }
        return typeof q == "string" && q !== "" || typeof q == "number" ? y(et(K, oe, "" + q, me)) : (typeof q == "function" && Fy(K), o(K, oe));
      }
      return It;
    }
    var Id = dT(!0), pT = dT(!1);
    function HM(e, t) {
      if (e !== null && t.child !== e.child)
        throw new Error("Resuming work not yet implemented.");
      if (t.child !== null) {
        var o = t.child, l = Rc(o, o.pendingProps);
        for (t.child = l, l.return = t; o.sibling !== null; )
          o = o.sibling, l = l.sibling = Rc(o, o.pendingProps), l.return = t;
        l.sibling = null;
      }
    }
    function zM(e, t) {
      for (var o = e.child; o !== null; )
        I4(o, t), o = o.sibling;
    }
    var p1 = zl(null), v1;
    v1 = {};
    var By = null, Ad = null, h1 = null, Vy = !1;
    function Py() {
      By = null, Ad = null, h1 = null, Vy = !1;
    }
    function vT() {
      Vy = !0;
    }
    function hT() {
      Vy = !1;
    }
    function mT(e, t, o) {
      $r(p1, t._currentValue, e), t._currentValue = o, t._currentRenderer !== void 0 && t._currentRenderer !== null && t._currentRenderer !== v1 && h("Detected multiple renderers concurrently rendering the same context provider. This is currently unsupported."), t._currentRenderer = v1;
    }
    function m1(e, t) {
      var o = p1.current;
      Yr(p1, t), e._currentValue = o;
    }
    function g1(e, t, o) {
      for (var l = e; l !== null; ) {
        var d = l.alternate;
        if (mu(l.childLanes, t) ? d !== null && !mu(d.childLanes, t) && (d.childLanes = Ct(d.childLanes, t)) : (l.childLanes = Ct(l.childLanes, t), d !== null && (d.childLanes = Ct(d.childLanes, t))), l === o)
          break;
        l = l.return;
      }
      l !== o && h("Expected to find the propagation root when scheduling context work. This error is likely caused by a bug in React. Please file an issue.");
    }
    function GM(e, t, o) {
      _M(e, t, o);
    }
    function _M(e, t, o) {
      var l = e.child;
      for (l !== null && (l.return = e); l !== null; ) {
        var d = void 0, m = l.dependencies;
        if (m !== null) {
          d = l.child;
          for (var y = m.firstContext; y !== null; ) {
            if (y.context === t) {
              if (l.tag === E) {
                var x = Sl(o), T = Ou(tn, x);
                T.tag = zy;
                var O = l.updateQueue;
                if (O !== null) {
                  var N = O.shared, j = N.pending;
                  j === null ? T.next = T : (T.next = j.next, j.next = T), N.pending = T;
                }
              }
              l.lanes = Ct(l.lanes, o);
              var G = l.alternate;
              G !== null && (G.lanes = Ct(G.lanes, o)), g1(l.return, o, e), m.lanes = Ct(m.lanes, o);
              break;
            }
            y = y.next;
          }
        } else if (l.tag === Y)
          d = l.type === e.type ? null : l.child;
        else if (l.tag === Se) {
          var te = l.return;
          if (te === null)
            throw new Error("We just came from a parent so we must have had a parent. This is a bug in React.");
          te.lanes = Ct(te.lanes, o);
          var re = te.alternate;
          re !== null && (re.lanes = Ct(re.lanes, o)), g1(te, o, e), d = l.sibling;
        } else
          d = l.child;
        if (d !== null)
          d.return = l;
        else
          for (d = l; d !== null; ) {
            if (d === e) {
              d = null;
              break;
            }
            var ie = d.sibling;
            if (ie !== null) {
              ie.return = d.return, d = ie;
              break;
            }
            d = d.return;
          }
        l = d;
      }
    }
    function Od(e, t) {
      By = e, Ad = null, h1 = null;
      var o = e.dependencies;
      if (o !== null) {
        var l = o.firstContext;
        l !== null && (Xr(o.lanes, t) && Ch(), o.firstContext = null);
      }
    }
    function Jn(e) {
      Vy && h("Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().");
      var t = e._currentValue;
      if (h1 !== e) {
        var o = {
          context: e,
          memoizedValue: t,
          next: null
        };
        if (Ad === null) {
          if (By === null)
            throw new Error("Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().");
          Ad = o, By.dependencies = {
            lanes: fe,
            firstContext: o
          };
        } else
          Ad = Ad.next = o;
      }
      return t;
    }
    var yc = null;
    function y1(e) {
      yc === null ? yc = [e] : yc.push(e);
    }
    function jM() {
      if (yc !== null) {
        for (var e = 0; e < yc.length; e++) {
          var t = yc[e], o = t.interleaved;
          if (o !== null) {
            t.interleaved = null;
            var l = o.next, d = t.pending;
            if (d !== null) {
              var m = d.next;
              d.next = l, o.next = m;
            }
            t.pending = o;
          }
        }
        yc = null;
      }
    }
    function gT(e, t, o, l) {
      var d = t.interleaved;
      return d === null ? (o.next = o, y1(t)) : (o.next = d.next, d.next = o), t.interleaved = o, Hy(e, l);
    }
    function UM(e, t, o, l) {
      var d = t.interleaved;
      d === null ? (o.next = o, y1(t)) : (o.next = d.next, d.next = o), t.interleaved = o;
    }
    function WM(e, t, o, l) {
      var d = t.interleaved;
      return d === null ? (o.next = o, y1(t)) : (o.next = d.next, d.next = o), t.interleaved = o, Hy(e, l);
    }
    function Va(e, t) {
      return Hy(e, t);
    }
    var XM = Hy;
    function Hy(e, t) {
      e.lanes = Ct(e.lanes, t);
      var o = e.alternate;
      o !== null && (o.lanes = Ct(o.lanes, t)), o === null && (e.flags & (nn | Ta)) !== qe && pI(e);
      for (var l = e, d = e.return; d !== null; )
        d.childLanes = Ct(d.childLanes, t), o = d.alternate, o !== null ? o.childLanes = Ct(o.childLanes, t) : (d.flags & (nn | Ta)) !== qe && pI(e), l = d, d = d.return;
      if (l.tag === R) {
        var m = l.stateNode;
        return m;
      } else
        return null;
    }
    var yT = 0, bT = 1, zy = 2, b1 = 3, Gy = !1, C1, _y;
    C1 = !1, _y = null;
    function S1(e) {
      var t = {
        baseState: e.memoizedState,
        firstBaseUpdate: null,
        lastBaseUpdate: null,
        shared: {
          pending: null,
          interleaved: null,
          lanes: fe
        },
        effects: null
      };
      e.updateQueue = t;
    }
    function CT(e, t) {
      var o = t.updateQueue, l = e.updateQueue;
      if (o === l) {
        var d = {
          baseState: l.baseState,
          firstBaseUpdate: l.firstBaseUpdate,
          lastBaseUpdate: l.lastBaseUpdate,
          shared: l.shared,
          effects: l.effects
        };
        t.updateQueue = d;
      }
    }
    function Ou(e, t) {
      var o = {
        eventTime: e,
        lane: t,
        tag: yT,
        payload: null,
        callback: null,
        next: null
      };
      return o;
    }
    function Ul(e, t, o) {
      var l = e.updateQueue;
      if (l === null)
        return null;
      var d = l.shared;
      if (_y === d && !C1 && (h("An update (setState, replaceState, or forceUpdate) was scheduled from inside an update function. Update functions should be pure, with zero side-effects. Consider using componentDidUpdate or a callback."), C1 = !0), UF()) {
        var m = d.pending;
        return m === null ? t.next = t : (t.next = m.next, m.next = t), d.pending = t, XM(e, o);
      } else
        return WM(e, d, t, o);
    }
    function jy(e, t, o) {
      var l = t.updateQueue;
      if (l !== null) {
        var d = l.shared;
        if (Cv(o)) {
          var m = d.lanes;
          m = Xf(m, e.pendingLanes);
          var y = Ct(m, o);
          d.lanes = y, Sv(e, y);
        }
      }
    }
    function E1(e, t) {
      var o = e.updateQueue, l = e.alternate;
      if (l !== null) {
        var d = l.updateQueue;
        if (o === d) {
          var m = null, y = null, x = o.firstBaseUpdate;
          if (x !== null) {
            var T = x;
            do {
              var O = {
                eventTime: T.eventTime,
                lane: T.lane,
                tag: T.tag,
                payload: T.payload,
                callback: T.callback,
                next: null
              };
              y === null ? m = y = O : (y.next = O, y = O), T = T.next;
            } while (T !== null);
            y === null ? m = y = t : (y.next = t, y = t);
          } else
            m = y = t;
          o = {
            baseState: d.baseState,
            firstBaseUpdate: m,
            lastBaseUpdate: y,
            shared: d.shared,
            effects: d.effects
          }, e.updateQueue = o;
          return;
        }
      }
      var N = o.lastBaseUpdate;
      N === null ? o.firstBaseUpdate = t : N.next = t, o.lastBaseUpdate = t;
    }
    function ZM(e, t, o, l, d, m) {
      switch (o.tag) {
        case bT: {
          var y = o.payload;
          if (typeof y == "function") {
            vT();
            var x = y.call(m, l, d);
            {
              if (e.mode & xt) {
                Nn(!0);
                try {
                  y.call(m, l, d);
                } finally {
                  Nn(!1);
                }
              }
              hT();
            }
            return x;
          }
          return y;
        }
        case b1:
          e.flags = e.flags & ~Zn | dt;
        case yT: {
          var T = o.payload, O;
          if (typeof T == "function") {
            vT(), O = T.call(m, l, d);
            {
              if (e.mode & xt) {
                Nn(!0);
                try {
                  T.call(m, l, d);
                } finally {
                  Nn(!1);
                }
              }
              hT();
            }
          } else
            O = T;
          return O == null ? l : Tt({}, l, O);
        }
        case zy:
          return Gy = !0, l;
      }
      return l;
    }
    function Uy(e, t, o, l) {
      var d = e.updateQueue;
      Gy = !1, _y = d.shared;
      var m = d.firstBaseUpdate, y = d.lastBaseUpdate, x = d.shared.pending;
      if (x !== null) {
        d.shared.pending = null;
        var T = x, O = T.next;
        T.next = null, y === null ? m = O : y.next = O, y = T;
        var N = e.alternate;
        if (N !== null) {
          var j = N.updateQueue, G = j.lastBaseUpdate;
          G !== y && (G === null ? j.firstBaseUpdate = O : G.next = O, j.lastBaseUpdate = T);
        }
      }
      if (m !== null) {
        var te = d.baseState, re = fe, ie = null, Ne = null, et = null, Qe = m;
        do {
          var Nt = Qe.lane, It = Qe.eventTime;
          if (mu(l, Nt)) {
            if (et !== null) {
              var oe = {
                eventTime: It,
                // This update is going to be committed so we never want uncommit
                // it. Using NoLane works because 0 is a subset of all bitmasks, so
                // this will never be skipped by the check above.
                lane: kn,
                tag: Qe.tag,
                payload: Qe.payload,
                callback: Qe.callback,
                next: null
              };
              et = et.next = oe;
            }
            te = ZM(e, d, Qe, te, t, o);
            var q = Qe.callback;
            if (q !== null && // If the update was already committed, we should not queue its
            // callback again.
            Qe.lane !== kn) {
              e.flags |= hi;
              var me = d.effects;
              me === null ? d.effects = [Qe] : me.push(Qe);
            }
          } else {
            var K = {
              eventTime: It,
              lane: Nt,
              tag: Qe.tag,
              payload: Qe.payload,
              callback: Qe.callback,
              next: null
            };
            et === null ? (Ne = et = K, ie = te) : et = et.next = K, re = Ct(re, Nt);
          }
          if (Qe = Qe.next, Qe === null) {
            if (x = d.shared.pending, x === null)
              break;
            var Ve = x, Le = Ve.next;
            Ve.next = null, Qe = Le, d.lastBaseUpdate = Ve, d.shared.pending = null;
          }
        } while (!0);
        et === null && (ie = te), d.baseState = ie, d.firstBaseUpdate = Ne, d.lastBaseUpdate = et;
        var ct = d.shared.interleaved;
        if (ct !== null) {
          var mt = ct;
          do
            re = Ct(re, mt.lane), mt = mt.next;
          while (mt !== ct);
        } else
          m === null && (d.shared.lanes = fe);
        kh(re), e.lanes = re, e.memoizedState = te;
      }
      _y = null;
    }
    function YM(e, t) {
      if (typeof e != "function")
        throw new Error("Invalid argument passed as callback. Expected a function. Instead " + ("received: " + e));
      e.call(t);
    }
    function ST() {
      Gy = !1;
    }
    function Wy() {
      return Gy;
    }
    function ET(e, t, o) {
      var l = t.effects;
      if (t.effects = null, l !== null)
        for (var d = 0; d < l.length; d++) {
          var m = l[d], y = m.callback;
          y !== null && (m.callback = null, YM(y, o));
        }
    }
    var oh = {}, Wl = zl(oh), uh = zl(oh), Xy = zl(oh);
    function Zy(e) {
      if (e === oh)
        throw new Error("Expected host context to exist. This error is likely caused by a bug in React. Please file an issue.");
      return e;
    }
    function xT() {
      var e = Zy(Xy.current);
      return e;
    }
    function x1(e, t) {
      $r(Xy, t, e), $r(uh, e, e), $r(Wl, oh, e);
      var o = fk(t);
      Yr(Wl, e), $r(Wl, o, e);
    }
    function Dd(e) {
      Yr(Wl, e), Yr(uh, e), Yr(Xy, e);
    }
    function w1() {
      var e = Zy(Wl.current);
      return e;
    }
    function wT(e) {
      Zy(Xy.current);
      var t = Zy(Wl.current), o = dk(t, e.type);
      t !== o && ($r(uh, e, e), $r(Wl, o, e));
    }
    function T1(e) {
      uh.current === e && (Yr(Wl, e), Yr(uh, e));
    }
    var $M = 0, TT = 1, RT = 1, lh = 2, Ki = zl($M);
    function R1(e, t) {
      return (e & t) !== 0;
    }
    function Nd(e) {
      return e & TT;
    }
    function I1(e, t) {
      return e & TT | t;
    }
    function QM(e, t) {
      return e | t;
    }
    function Xl(e, t) {
      $r(Ki, t, e);
    }
    function kd(e) {
      Yr(Ki, e);
    }
    function JM(e, t) {
      var o = e.memoizedState;
      return o !== null ? o.dehydrated !== null : (e.memoizedProps, !0);
    }
    function Yy(e) {
      for (var t = e; t !== null; ) {
        if (t.tag === V) {
          var o = t.memoizedState;
          if (o !== null) {
            var l = o.dehydrated;
            if (l === null || jw(l) || WC(l))
              return t;
          }
        } else if (t.tag === he && // revealOrder undefined can't be trusted because it don't
        // keep track of whether it suspended or not.
        t.memoizedProps.revealOrder !== void 0) {
          var d = (t.flags & dt) !== qe;
          if (d)
            return t;
        } else if (t.child !== null) {
          t.child.return = t, t = t.child;
          continue;
        }
        if (t === e)
          return null;
        for (; t.sibling === null; ) {
          if (t.return === null || t.return === e)
            return null;
          t = t.return;
        }
        t.sibling.return = t.return, t = t.sibling;
      }
      return null;
    }
    var Pa = (
      /*   */
      0
    ), ir = (
      /* */
      1
    ), Mo = (
      /*  */
      2
    ), or = (
      /*    */
      4
    ), Rr = (
      /*   */
      8
    ), A1 = [];
    function O1() {
      for (var e = 0; e < A1.length; e++) {
        var t = A1[e];
        t._workInProgressVersionPrimary = null;
      }
      A1.length = 0;
    }
    function KM(e, t) {
      var o = t._getVersion, l = o(t._source);
      e.mutableSourceEagerHydrationData == null ? e.mutableSourceEagerHydrationData = [t, l] : e.mutableSourceEagerHydrationData.push(t, l);
    }
    var Be = u.ReactCurrentDispatcher, sh = u.ReactCurrentBatchConfig, D1, Md;
    D1 = /* @__PURE__ */ new Set();
    var bc = fe, Yt = null, ur = null, lr = null, $y = !1, ch = !1, fh = 0, qM = 0, eL = 25, le = null, wi = null, Zl = -1, N1 = !1;
    function zt() {
      {
        var e = le;
        wi === null ? wi = [e] : wi.push(e);
      }
    }
    function Re() {
      {
        var e = le;
        wi !== null && (Zl++, wi[Zl] !== e && tL(e));
      }
    }
    function Ld(e) {
      e != null && !Wn(e) && h("%s received a final argument that is not an array (instead, received `%s`). When specified, the final argument must be an array.", le, typeof e);
    }
    function tL(e) {
      {
        var t = bt(Yt);
        if (!D1.has(t) && (D1.add(t), wi !== null)) {
          for (var o = "", l = 30, d = 0; d <= Zl; d++) {
            for (var m = wi[d], y = d === Zl ? e : m, x = d + 1 + ". " + m; x.length < l; )
              x += " ";
            x += y + `
`, o += x;
          }
          h(`React has detected a change in the order of Hooks called by %s. This will lead to bugs and errors if not fixed. For more information, read the Rules of Hooks: https://reactjs.org/link/rules-of-hooks

   Previous render            Next render
   ------------------------------------------------------
%s   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
`, t, o);
        }
      }
    }
    function Qr() {
      throw new Error(`Invalid hook call. Hooks can only be called inside of the body of a function component. This could happen for one of the following reasons:
1. You might have mismatching versions of React and the renderer (such as React DOM)
2. You might be breaking the Rules of Hooks
3. You might have more than one copy of React in the same app
See https://reactjs.org/link/invalid-hook-call for tips about how to debug and fix this problem.`);
    }
    function k1(e, t) {
      if (N1)
        return !1;
      if (t === null)
        return h("%s received a final argument during this render, but not during the previous render. Even though the final argument is optional, its type cannot change between renders.", le), !1;
      e.length !== t.length && h(`The final argument passed to %s changed size between renders. The order and size of this array must remain constant.

Previous: %s
Incoming: %s`, le, "[" + t.join(", ") + "]", "[" + e.join(", ") + "]");
      for (var o = 0; o < t.length && o < e.length; o++)
        if (!Ge(e[o], t[o]))
          return !1;
      return !0;
    }
    function Fd(e, t, o, l, d, m) {
      bc = m, Yt = t, wi = e !== null ? e._debugHookTypes : null, Zl = -1, N1 = e !== null && e.type !== t.type, t.memoizedState = null, t.updateQueue = null, t.lanes = fe, e !== null && e.memoizedState !== null ? Be.current = YT : wi !== null ? Be.current = ZT : Be.current = XT;
      var y = o(l, d);
      if (ch) {
        var x = 0;
        do {
          if (ch = !1, fh = 0, x >= eL)
            throw new Error("Too many re-renders. React limits the number of renders to prevent an infinite loop.");
          x += 1, N1 = !1, ur = null, lr = null, t.updateQueue = null, Zl = -1, Be.current = $T, y = o(l, d);
        } while (ch);
      }
      Be.current = l0, t._debugHookTypes = wi;
      var T = ur !== null && ur.next !== null;
      if (bc = fe, Yt = null, ur = null, lr = null, le = null, wi = null, Zl = -1, e !== null && (e.flags & nr) !== (t.flags & nr) && // Disable this warning in legacy mode, because legacy Suspense is weird
      // and creates false positives. To make this work in legacy mode, we'd
      // need to mark fibers that commit in an incomplete state, somehow. For
      // now I'll disable the warning that most of the bugs that would trigger
      // it are either exclusive to concurrent mode or exist in both.
      (e.mode & $e) !== Xe && h("Internal React error: Expected static flag was missing. Please notify the React team."), $y = !1, T)
        throw new Error("Rendered fewer hooks than expected. This may be caused by an accidental early return statement.");
      return y;
    }
    function Bd() {
      var e = fh !== 0;
      return fh = 0, e;
    }
    function IT(e, t, o) {
      t.updateQueue = e.updateQueue, (t.mode & la) !== Xe ? t.flags &= ~(uu | zr | un | St) : t.flags &= ~(un | St), e.lanes = Qs(e.lanes, o);
    }
    function AT() {
      if (Be.current = l0, $y) {
        for (var e = Yt.memoizedState; e !== null; ) {
          var t = e.queue;
          t !== null && (t.pending = null), e = e.next;
        }
        $y = !1;
      }
      bc = fe, Yt = null, ur = null, lr = null, wi = null, Zl = -1, le = null, GT = !1, ch = !1, fh = 0;
    }
    function Lo() {
      var e = {
        memoizedState: null,
        baseState: null,
        baseQueue: null,
        queue: null,
        next: null
      };
      return lr === null ? Yt.memoizedState = lr = e : lr = lr.next = e, lr;
    }
    function Ti() {
      var e;
      if (ur === null) {
        var t = Yt.alternate;
        t !== null ? e = t.memoizedState : e = null;
      } else
        e = ur.next;
      var o;
      if (lr === null ? o = Yt.memoizedState : o = lr.next, o !== null)
        lr = o, o = lr.next, ur = e;
      else {
        if (e === null)
          throw new Error("Rendered more hooks than during the previous render.");
        ur = e;
        var l = {
          memoizedState: ur.memoizedState,
          baseState: ur.baseState,
          baseQueue: ur.baseQueue,
          queue: ur.queue,
          next: null
        };
        lr === null ? Yt.memoizedState = lr = l : lr = lr.next = l;
      }
      return lr;
    }
    function OT() {
      return {
        lastEffect: null,
        stores: null
      };
    }
    function M1(e, t) {
      return typeof t == "function" ? t(e) : t;
    }
    function L1(e, t, o) {
      var l = Lo(), d;
      o !== void 0 ? d = o(t) : d = t, l.memoizedState = l.baseState = d;
      var m = {
        pending: null,
        interleaved: null,
        lanes: fe,
        dispatch: null,
        lastRenderedReducer: e,
        lastRenderedState: d
      };
      l.queue = m;
      var y = m.dispatch = iL.bind(null, Yt, m);
      return [l.memoizedState, y];
    }
    function F1(e, t, o) {
      var l = Ti(), d = l.queue;
      if (d === null)
        throw new Error("Should have a queue. This is likely a bug in React. Please file an issue.");
      d.lastRenderedReducer = e;
      var m = ur, y = m.baseQueue, x = d.pending;
      if (x !== null) {
        if (y !== null) {
          var T = y.next, O = x.next;
          y.next = O, x.next = T;
        }
        m.baseQueue !== y && h("Internal error: Expected work-in-progress queue to be a clone. This is a bug in React."), m.baseQueue = y = x, d.pending = null;
      }
      if (y !== null) {
        var N = y.next, j = m.baseState, G = null, te = null, re = null, ie = N;
        do {
          var Ne = ie.lane;
          if (mu(bc, Ne)) {
            if (re !== null) {
              var Qe = {
                // This update is going to be committed so we never want uncommit
                // it. Using NoLane works because 0 is a subset of all bitmasks, so
                // this will never be skipped by the check above.
                lane: kn,
                action: ie.action,
                hasEagerState: ie.hasEagerState,
                eagerState: ie.eagerState,
                next: null
              };
              re = re.next = Qe;
            }
            if (ie.hasEagerState)
              j = ie.eagerState;
            else {
              var Nt = ie.action;
              j = e(j, Nt);
            }
          } else {
            var et = {
              lane: Ne,
              action: ie.action,
              hasEagerState: ie.hasEagerState,
              eagerState: ie.eagerState,
              next: null
            };
            re === null ? (te = re = et, G = j) : re = re.next = et, Yt.lanes = Ct(Yt.lanes, Ne), kh(Ne);
          }
          ie = ie.next;
        } while (ie !== null && ie !== N);
        re === null ? G = j : re.next = te, Ge(j, l.memoizedState) || Ch(), l.memoizedState = j, l.baseState = G, l.baseQueue = re, d.lastRenderedState = j;
      }
      var It = d.interleaved;
      if (It !== null) {
        var K = It;
        do {
          var oe = K.lane;
          Yt.lanes = Ct(Yt.lanes, oe), kh(oe), K = K.next;
        } while (K !== It);
      } else
        y === null && (d.lanes = fe);
      var q = d.dispatch;
      return [l.memoizedState, q];
    }
    function B1(e, t, o) {
      var l = Ti(), d = l.queue;
      if (d === null)
        throw new Error("Should have a queue. This is likely a bug in React. Please file an issue.");
      d.lastRenderedReducer = e;
      var m = d.dispatch, y = d.pending, x = l.memoizedState;
      if (y !== null) {
        d.pending = null;
        var T = y.next, O = T;
        do {
          var N = O.action;
          x = e(x, N), O = O.next;
        } while (O !== T);
        Ge(x, l.memoizedState) || Ch(), l.memoizedState = x, l.baseQueue === null && (l.baseState = x), d.lastRenderedState = x;
      }
      return [x, m];
    }
    function E_(e, t, o) {
    }
    function x_(e, t, o) {
    }
    function V1(e, t, o) {
      var l = Yt, d = Lo(), m, y = Tr();
      if (y) {
        if (o === void 0)
          throw new Error("Missing getServerSnapshot, which is required for server-rendered content. Will revert to client rendering.");
        m = o(), Md || m !== o() && (h("The result of getServerSnapshot should be cached to avoid an infinite loop"), Md = !0);
      } else {
        if (m = t(), !Md) {
          var x = t();
          Ge(m, x) || (h("The result of getSnapshot should be cached to avoid an infinite loop"), Md = !0);
        }
        var T = I0();
        if (T === null)
          throw new Error("Expected a work-in-progress root. This is a bug in React. Please file an issue.");
        $s(T, bc) || DT(l, t, m);
      }
      d.memoizedState = m;
      var O = {
        value: m,
        getSnapshot: t
      };
      return d.queue = O, e0(kT.bind(null, l, O, e), [e]), l.flags |= un, dh(ir | Rr, NT.bind(null, l, O, m, t), void 0, null), m;
    }
    function Qy(e, t, o) {
      var l = Yt, d = Ti(), m = t();
      if (!Md) {
        var y = t();
        Ge(m, y) || (h("The result of getSnapshot should be cached to avoid an infinite loop"), Md = !0);
      }
      var x = d.memoizedState, T = !Ge(x, m);
      T && (d.memoizedState = m, Ch());
      var O = d.queue;
      if (vh(kT.bind(null, l, O, e), [e]), O.getSnapshot !== t || T || // Check if the susbcribe function changed. We can save some memory by
      // checking whether we scheduled a subscription effect above.
      lr !== null && lr.memoizedState.tag & ir) {
        l.flags |= un, dh(ir | Rr, NT.bind(null, l, O, m, t), void 0, null);
        var N = I0();
        if (N === null)
          throw new Error("Expected a work-in-progress root. This is a bug in React. Please file an issue.");
        $s(N, bc) || DT(l, t, m);
      }
      return m;
    }
    function DT(e, t, o) {
      e.flags |= Hs;
      var l = {
        getSnapshot: t,
        value: o
      }, d = Yt.updateQueue;
      if (d === null)
        d = OT(), Yt.updateQueue = d, d.stores = [l];
      else {
        var m = d.stores;
        m === null ? d.stores = [l] : m.push(l);
      }
    }
    function NT(e, t, o, l) {
      t.value = o, t.getSnapshot = l, MT(t) && LT(e);
    }
    function kT(e, t, o) {
      var l = function() {
        MT(t) && LT(e);
      };
      return o(l);
    }
    function MT(e) {
      var t = e.getSnapshot, o = e.value;
      try {
        var l = t();
        return !Ge(o, l);
      } catch {
        return !0;
      }
    }
    function LT(e) {
      var t = Va(e, Je);
      t !== null && dr(t, e, Je, tn);
    }
    function Jy(e) {
      var t = Lo();
      typeof e == "function" && (e = e()), t.memoizedState = t.baseState = e;
      var o = {
        pending: null,
        interleaved: null,
        lanes: fe,
        dispatch: null,
        lastRenderedReducer: M1,
        lastRenderedState: e
      };
      t.queue = o;
      var l = o.dispatch = oL.bind(null, Yt, o);
      return [t.memoizedState, l];
    }
    function P1(e) {
      return F1(M1);
    }
    function H1(e) {
      return B1(M1);
    }
    function dh(e, t, o, l) {
      var d = {
        tag: e,
        create: t,
        destroy: o,
        deps: l,
        // Circular
        next: null
      }, m = Yt.updateQueue;
      if (m === null)
        m = OT(), Yt.updateQueue = m, m.lastEffect = d.next = d;
      else {
        var y = m.lastEffect;
        if (y === null)
          m.lastEffect = d.next = d;
        else {
          var x = y.next;
          y.next = d, d.next = x, m.lastEffect = d;
        }
      }
      return d;
    }
    function z1(e) {
      var t = Lo();
      {
        var o = {
          current: e
        };
        return t.memoizedState = o, o;
      }
    }
    function Ky(e) {
      var t = Ti();
      return t.memoizedState;
    }
    function ph(e, t, o, l) {
      var d = Lo(), m = l === void 0 ? null : l;
      Yt.flags |= e, d.memoizedState = dh(ir | t, o, void 0, m);
    }
    function qy(e, t, o, l) {
      var d = Ti(), m = l === void 0 ? null : l, y = void 0;
      if (ur !== null) {
        var x = ur.memoizedState;
        if (y = x.destroy, m !== null) {
          var T = x.deps;
          if (k1(m, T)) {
            d.memoizedState = dh(t, o, y, m);
            return;
          }
        }
      }
      Yt.flags |= e, d.memoizedState = dh(ir | t, o, y, m);
    }
    function e0(e, t) {
      return (Yt.mode & la) !== Xe ? ph(uu | un | So, Rr, e, t) : ph(un | So, Rr, e, t);
    }
    function vh(e, t) {
      return qy(un, Rr, e, t);
    }
    function G1(e, t) {
      return ph(St, Mo, e, t);
    }
    function t0(e, t) {
      return qy(St, Mo, e, t);
    }
    function _1(e, t) {
      var o = St;
      return o |= Hr, (Yt.mode & la) !== Xe && (o |= zr), ph(o, or, e, t);
    }
    function n0(e, t) {
      return qy(St, or, e, t);
    }
    function FT(e, t) {
      if (typeof t == "function") {
        var o = t, l = e();
        return o(l), function() {
          o(null);
        };
      } else if (t != null) {
        var d = t;
        d.hasOwnProperty("current") || h("Expected useImperativeHandle() first argument to either be a ref callback or React.createRef() object. Instead received: %s.", "an object with keys {" + Object.keys(d).join(", ") + "}");
        var m = e();
        return d.current = m, function() {
          d.current = null;
        };
      }
    }
    function j1(e, t, o) {
      typeof t != "function" && h("Expected useImperativeHandle() second argument to be a function that creates a handle. Instead received: %s.", t !== null ? typeof t : "null");
      var l = o != null ? o.concat([e]) : null, d = St;
      return d |= Hr, (Yt.mode & la) !== Xe && (d |= zr), ph(d, or, FT.bind(null, t, e), l);
    }
    function r0(e, t, o) {
      typeof t != "function" && h("Expected useImperativeHandle() second argument to be a function that creates a handle. Instead received: %s.", t !== null ? typeof t : "null");
      var l = o != null ? o.concat([e]) : null;
      return qy(St, or, FT.bind(null, t, e), l);
    }
    function nL(e, t) {
    }
    var a0 = nL;
    function U1(e, t) {
      var o = Lo(), l = t === void 0 ? null : t;
      return o.memoizedState = [e, l], e;
    }
    function i0(e, t) {
      var o = Ti(), l = t === void 0 ? null : t, d = o.memoizedState;
      if (d !== null && l !== null) {
        var m = d[1];
        if (k1(l, m))
          return d[0];
      }
      return o.memoizedState = [e, l], e;
    }
    function W1(e, t) {
      var o = Lo(), l = t === void 0 ? null : t, d = e();
      return o.memoizedState = [d, l], d;
    }
    function o0(e, t) {
      var o = Ti(), l = t === void 0 ? null : t, d = o.memoizedState;
      if (d !== null && l !== null) {
        var m = d[1];
        if (k1(l, m))
          return d[0];
      }
      var y = e();
      return o.memoizedState = [y, l], y;
    }
    function X1(e) {
      var t = Lo();
      return t.memoizedState = e, e;
    }
    function BT(e) {
      var t = Ti(), o = ur, l = o.memoizedState;
      return PT(t, l, e);
    }
    function VT(e) {
      var t = Ti();
      if (ur === null)
        return t.memoizedState = e, e;
      var o = ur.memoizedState;
      return PT(t, o, e);
    }
    function PT(e, t, o) {
      var l = !Bg(bc);
      if (l) {
        if (!Ge(o, t)) {
          var d = Hg();
          Yt.lanes = Ct(Yt.lanes, d), kh(d), e.baseState = !0;
        }
        return t;
      } else
        return e.baseState && (e.baseState = !1, Ch()), e.memoizedState = o, o;
    }
    function rL(e, t, o) {
      var l = Na();
      yn(dC(l, Wi)), e(!0);
      var d = sh.transition;
      sh.transition = {};
      var m = sh.transition;
      sh.transition._updatedFibers = /* @__PURE__ */ new Set();
      try {
        e(!1), t();
      } finally {
        if (yn(l), sh.transition = d, d === null && m._updatedFibers) {
          var y = m._updatedFibers.size;
          y > 10 && v("Detected a large number of updates inside startTransition. If this is due to a subscription please re-write it to use React provided hooks. Otherwise concurrent mode guarantees are off the table."), m._updatedFibers.clear();
        }
      }
    }
    function Z1() {
      var e = Jy(!1), t = e[0], o = e[1], l = rL.bind(null, o), d = Lo();
      return d.memoizedState = l, [t, l];
    }
    function HT() {
      var e = P1(), t = e[0], o = Ti(), l = o.memoizedState;
      return [t, l];
    }
    function zT() {
      var e = H1(), t = e[0], o = Ti(), l = o.memoizedState;
      return [t, l];
    }
    var GT = !1;
    function aL() {
      return GT;
    }
    function Y1() {
      var e = Lo(), t = I0(), o = t.identifierPrefix, l;
      if (Tr()) {
        var d = SM();
        l = ":" + o + "R" + d;
        var m = fh++;
        m > 0 && (l += "H" + m.toString(32)), l += ":";
      } else {
        var y = qM++;
        l = ":" + o + "r" + y.toString(32) + ":";
      }
      return e.memoizedState = l, l;
    }
    function u0() {
      var e = Ti(), t = e.memoizedState;
      return t;
    }
    function iL(e, t, o) {
      typeof arguments[3] == "function" && h("State updates from the useState() and useReducer() Hooks don't support the second callback argument. To execute a side effect after rendering, declare it in the component body with useEffect().");
      var l = Kl(e), d = {
        lane: l,
        action: o,
        hasEagerState: !1,
        eagerState: null,
        next: null
      };
      if (_T(e))
        jT(t, d);
      else {
        var m = gT(e, t, d, l);
        if (m !== null) {
          var y = da();
          dr(m, e, l, y), UT(m, t, l);
        }
      }
      WT(e, l);
    }
    function oL(e, t, o) {
      typeof arguments[3] == "function" && h("State updates from the useState() and useReducer() Hooks don't support the second callback argument. To execute a side effect after rendering, declare it in the component body with useEffect().");
      var l = Kl(e), d = {
        lane: l,
        action: o,
        hasEagerState: !1,
        eagerState: null,
        next: null
      };
      if (_T(e))
        jT(t, d);
      else {
        var m = e.alternate;
        if (e.lanes === fe && (m === null || m.lanes === fe)) {
          var y = t.lastRenderedReducer;
          if (y !== null) {
            var x;
            x = Be.current, Be.current = qi;
            try {
              var T = t.lastRenderedState, O = y(T, o);
              if (d.hasEagerState = !0, d.eagerState = O, Ge(O, T)) {
                UM(e, t, d, l);
                return;
              }
            } catch {
            } finally {
              Be.current = x;
            }
          }
        }
        var N = gT(e, t, d, l);
        if (N !== null) {
          var j = da();
          dr(N, e, l, j), UT(N, t, l);
        }
      }
      WT(e, l);
    }
    function _T(e) {
      var t = e.alternate;
      return e === Yt || t !== null && t === Yt;
    }
    function jT(e, t) {
      ch = $y = !0;
      var o = e.pending;
      o === null ? t.next = t : (t.next = o.next, o.next = t), e.pending = t;
    }
    function UT(e, t, o) {
      if (Cv(o)) {
        var l = t.lanes;
        l = Xf(l, e.pendingLanes);
        var d = Ct(l, o);
        t.lanes = d, Sv(e, d);
      }
    }
    function WT(e, t, o) {
      js(e, t);
    }
    var l0 = {
      readContext: Jn,
      useCallback: Qr,
      useContext: Qr,
      useEffect: Qr,
      useImperativeHandle: Qr,
      useInsertionEffect: Qr,
      useLayoutEffect: Qr,
      useMemo: Qr,
      useReducer: Qr,
      useRef: Qr,
      useState: Qr,
      useDebugValue: Qr,
      useDeferredValue: Qr,
      useTransition: Qr,
      useMutableSource: Qr,
      useSyncExternalStore: Qr,
      useId: Qr,
      unstable_isNewReconciler: ue
    }, XT = null, ZT = null, YT = null, $T = null, Fo = null, qi = null, s0 = null;
    {
      var $1 = function() {
        h("Context can only be read while React is rendering. In classes, you can read it in the render method or getDerivedStateFromProps. In function components, you can read it directly in the function body, but not inside Hooks like useReducer() or useMemo().");
      }, ht = function() {
        h("Do not call Hooks inside useEffect(...), useMemo(...), or other built-in Hooks. You can only call Hooks at the top level of your React function. For more information, see https://reactjs.org/link/rules-of-hooks");
      };
      XT = {
        readContext: function(e) {
          return Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", zt(), Ld(t), U1(e, t);
        },
        useContext: function(e) {
          return le = "useContext", zt(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", zt(), Ld(t), e0(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", zt(), Ld(o), j1(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", zt(), Ld(t), G1(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", zt(), Ld(t), _1(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", zt(), Ld(t);
          var o = Be.current;
          Be.current = Fo;
          try {
            return W1(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", zt();
          var l = Be.current;
          Be.current = Fo;
          try {
            return L1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", zt(), z1(e);
        },
        useState: function(e) {
          le = "useState", zt();
          var t = Be.current;
          Be.current = Fo;
          try {
            return Jy(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", zt(), void 0;
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", zt(), X1(e);
        },
        useTransition: function() {
          return le = "useTransition", zt(), Z1();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", zt(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", zt(), V1(e, t, o);
        },
        useId: function() {
          return le = "useId", zt(), Y1();
        },
        unstable_isNewReconciler: ue
      }, ZT = {
        readContext: function(e) {
          return Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", Re(), U1(e, t);
        },
        useContext: function(e) {
          return le = "useContext", Re(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", Re(), e0(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", Re(), j1(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", Re(), G1(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", Re(), _1(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", Re();
          var o = Be.current;
          Be.current = Fo;
          try {
            return W1(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", Re();
          var l = Be.current;
          Be.current = Fo;
          try {
            return L1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", Re(), z1(e);
        },
        useState: function(e) {
          le = "useState", Re();
          var t = Be.current;
          Be.current = Fo;
          try {
            return Jy(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", Re(), void 0;
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", Re(), X1(e);
        },
        useTransition: function() {
          return le = "useTransition", Re(), Z1();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", Re(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", Re(), V1(e, t, o);
        },
        useId: function() {
          return le = "useId", Re(), Y1();
        },
        unstable_isNewReconciler: ue
      }, YT = {
        readContext: function(e) {
          return Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", Re(), i0(e, t);
        },
        useContext: function(e) {
          return le = "useContext", Re(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", Re(), vh(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", Re(), r0(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", Re(), t0(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", Re(), n0(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", Re();
          var o = Be.current;
          Be.current = qi;
          try {
            return o0(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", Re();
          var l = Be.current;
          Be.current = qi;
          try {
            return F1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", Re(), Ky();
        },
        useState: function(e) {
          le = "useState", Re();
          var t = Be.current;
          Be.current = qi;
          try {
            return P1(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", Re(), a0();
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", Re(), BT(e);
        },
        useTransition: function() {
          return le = "useTransition", Re(), HT();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", Re(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", Re(), Qy(e, t);
        },
        useId: function() {
          return le = "useId", Re(), u0();
        },
        unstable_isNewReconciler: ue
      }, $T = {
        readContext: function(e) {
          return Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", Re(), i0(e, t);
        },
        useContext: function(e) {
          return le = "useContext", Re(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", Re(), vh(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", Re(), r0(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", Re(), t0(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", Re(), n0(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", Re();
          var o = Be.current;
          Be.current = s0;
          try {
            return o0(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", Re();
          var l = Be.current;
          Be.current = s0;
          try {
            return B1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", Re(), Ky();
        },
        useState: function(e) {
          le = "useState", Re();
          var t = Be.current;
          Be.current = s0;
          try {
            return H1(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", Re(), a0();
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", Re(), VT(e);
        },
        useTransition: function() {
          return le = "useTransition", Re(), zT();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", Re(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", Re(), Qy(e, t);
        },
        useId: function() {
          return le = "useId", Re(), u0();
        },
        unstable_isNewReconciler: ue
      }, Fo = {
        readContext: function(e) {
          return $1(), Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", ht(), zt(), U1(e, t);
        },
        useContext: function(e) {
          return le = "useContext", ht(), zt(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", ht(), zt(), e0(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", ht(), zt(), j1(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", ht(), zt(), G1(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", ht(), zt(), _1(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", ht(), zt();
          var o = Be.current;
          Be.current = Fo;
          try {
            return W1(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", ht(), zt();
          var l = Be.current;
          Be.current = Fo;
          try {
            return L1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", ht(), zt(), z1(e);
        },
        useState: function(e) {
          le = "useState", ht(), zt();
          var t = Be.current;
          Be.current = Fo;
          try {
            return Jy(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", ht(), zt(), void 0;
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", ht(), zt(), X1(e);
        },
        useTransition: function() {
          return le = "useTransition", ht(), zt(), Z1();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", ht(), zt(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", ht(), zt(), V1(e, t, o);
        },
        useId: function() {
          return le = "useId", ht(), zt(), Y1();
        },
        unstable_isNewReconciler: ue
      }, qi = {
        readContext: function(e) {
          return $1(), Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", ht(), Re(), i0(e, t);
        },
        useContext: function(e) {
          return le = "useContext", ht(), Re(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", ht(), Re(), vh(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", ht(), Re(), r0(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", ht(), Re(), t0(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", ht(), Re(), n0(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", ht(), Re();
          var o = Be.current;
          Be.current = qi;
          try {
            return o0(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", ht(), Re();
          var l = Be.current;
          Be.current = qi;
          try {
            return F1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", ht(), Re(), Ky();
        },
        useState: function(e) {
          le = "useState", ht(), Re();
          var t = Be.current;
          Be.current = qi;
          try {
            return P1(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", ht(), Re(), a0();
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", ht(), Re(), BT(e);
        },
        useTransition: function() {
          return le = "useTransition", ht(), Re(), HT();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", ht(), Re(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", ht(), Re(), Qy(e, t);
        },
        useId: function() {
          return le = "useId", ht(), Re(), u0();
        },
        unstable_isNewReconciler: ue
      }, s0 = {
        readContext: function(e) {
          return $1(), Jn(e);
        },
        useCallback: function(e, t) {
          return le = "useCallback", ht(), Re(), i0(e, t);
        },
        useContext: function(e) {
          return le = "useContext", ht(), Re(), Jn(e);
        },
        useEffect: function(e, t) {
          return le = "useEffect", ht(), Re(), vh(e, t);
        },
        useImperativeHandle: function(e, t, o) {
          return le = "useImperativeHandle", ht(), Re(), r0(e, t, o);
        },
        useInsertionEffect: function(e, t) {
          return le = "useInsertionEffect", ht(), Re(), t0(e, t);
        },
        useLayoutEffect: function(e, t) {
          return le = "useLayoutEffect", ht(), Re(), n0(e, t);
        },
        useMemo: function(e, t) {
          le = "useMemo", ht(), Re();
          var o = Be.current;
          Be.current = qi;
          try {
            return o0(e, t);
          } finally {
            Be.current = o;
          }
        },
        useReducer: function(e, t, o) {
          le = "useReducer", ht(), Re();
          var l = Be.current;
          Be.current = qi;
          try {
            return B1(e, t, o);
          } finally {
            Be.current = l;
          }
        },
        useRef: function(e) {
          return le = "useRef", ht(), Re(), Ky();
        },
        useState: function(e) {
          le = "useState", ht(), Re();
          var t = Be.current;
          Be.current = qi;
          try {
            return H1(e);
          } finally {
            Be.current = t;
          }
        },
        useDebugValue: function(e, t) {
          return le = "useDebugValue", ht(), Re(), a0();
        },
        useDeferredValue: function(e) {
          return le = "useDeferredValue", ht(), Re(), VT(e);
        },
        useTransition: function() {
          return le = "useTransition", ht(), Re(), zT();
        },
        useMutableSource: function(e, t, o) {
          return le = "useMutableSource", ht(), Re(), void 0;
        },
        useSyncExternalStore: function(e, t, o) {
          return le = "useSyncExternalStore", ht(), Re(), Qy(e, t);
        },
        useId: function() {
          return le = "useId", ht(), Re(), u0();
        },
        unstable_isNewReconciler: ue
      };
    }
    var Yl = r.unstable_now, QT = 0, c0 = -1, hh = -1, f0 = -1, Q1 = !1, d0 = !1;
    function JT() {
      return Q1;
    }
    function uL() {
      d0 = !0;
    }
    function lL() {
      Q1 = !1, d0 = !1;
    }
    function sL() {
      Q1 = d0, d0 = !1;
    }
    function KT() {
      return QT;
    }
    function qT() {
      QT = Yl();
    }
    function J1(e) {
      hh = Yl(), e.actualStartTime < 0 && (e.actualStartTime = Yl());
    }
    function eR(e) {
      hh = -1;
    }
    function p0(e, t) {
      if (hh >= 0) {
        var o = Yl() - hh;
        e.actualDuration += o, t && (e.selfBaseDuration = o), hh = -1;
      }
    }
    function Bo(e) {
      if (c0 >= 0) {
        var t = Yl() - c0;
        c0 = -1;
        for (var o = e.return; o !== null; ) {
          switch (o.tag) {
            case R:
              var l = o.stateNode;
              l.effectDuration += t;
              return;
            case Z:
              var d = o.stateNode;
              d.effectDuration += t;
              return;
          }
          o = o.return;
        }
      }
    }
    function K1(e) {
      if (f0 >= 0) {
        var t = Yl() - f0;
        f0 = -1;
        for (var o = e.return; o !== null; ) {
          switch (o.tag) {
            case R:
              var l = o.stateNode;
              l !== null && (l.passiveEffectDuration += t);
              return;
            case Z:
              var d = o.stateNode;
              d !== null && (d.passiveEffectDuration += t);
              return;
          }
          o = o.return;
        }
      }
    }
    function Vo() {
      c0 = Yl();
    }
    function q1() {
      f0 = Yl();
    }
    function eS(e) {
      for (var t = e.child; t; )
        e.actualDuration += t.actualDuration, t = t.sibling;
    }
    function eo(e, t) {
      if (e && e.defaultProps) {
        var o = Tt({}, t), l = e.defaultProps;
        for (var d in l)
          o[d] === void 0 && (o[d] = l[d]);
        return o;
      }
      return t;
    }
    var tS = {}, nS, rS, aS, iS, oS, tR, v0, uS, lS, sS, mh;
    {
      nS = /* @__PURE__ */ new Set(), rS = /* @__PURE__ */ new Set(), aS = /* @__PURE__ */ new Set(), iS = /* @__PURE__ */ new Set(), uS = /* @__PURE__ */ new Set(), oS = /* @__PURE__ */ new Set(), lS = /* @__PURE__ */ new Set(), sS = /* @__PURE__ */ new Set(), mh = /* @__PURE__ */ new Set();
      var nR = /* @__PURE__ */ new Set();
      v0 = function(e, t) {
        if (!(e === null || typeof e == "function")) {
          var o = t + "_" + e;
          nR.has(o) || (nR.add(o), h("%s(...): Expected the last optional `callback` argument to be a function. Instead received: %s.", t, e));
        }
      }, tR = function(e, t) {
        if (t === void 0) {
          var o = Ut(e) || "Component";
          oS.has(o) || (oS.add(o), h("%s.getDerivedStateFromProps(): A valid state object (or null) must be returned. You have returned undefined.", o));
        }
      }, Object.defineProperty(tS, "_processChildContext", {
        enumerable: !1,
        value: function() {
          throw new Error("_processChildContext is not available in React 16+. This likely means you have multiple copies of React and are attempting to nest a React 15 tree inside a React 16 tree using unstable_renderSubtreeIntoContainer, which isn't supported. Try to make sure you have only one copy of React (and ideally, switch to ReactDOM.createPortal).");
        }
      }), Object.freeze(tS);
    }
    function cS(e, t, o, l) {
      var d = e.memoizedState, m = o(l, d);
      {
        if (e.mode & xt) {
          Nn(!0);
          try {
            m = o(l, d);
          } finally {
            Nn(!1);
          }
        }
        tR(t, m);
      }
      var y = m == null ? d : Tt({}, d, m);
      if (e.memoizedState = y, e.lanes === fe) {
        var x = e.updateQueue;
        x.baseState = y;
      }
    }
    var fS = {
      isMounted: oa,
      enqueueSetState: function(e, t, o) {
        var l = xa(e), d = da(), m = Kl(l), y = Ou(d, m);
        y.payload = t, o != null && (v0(o, "setState"), y.callback = o);
        var x = Ul(l, y, m);
        x !== null && (dr(x, l, m, d), jy(x, l, m)), js(l, m);
      },
      enqueueReplaceState: function(e, t, o) {
        var l = xa(e), d = da(), m = Kl(l), y = Ou(d, m);
        y.tag = bT, y.payload = t, o != null && (v0(o, "replaceState"), y.callback = o);
        var x = Ul(l, y, m);
        x !== null && (dr(x, l, m, d), jy(x, l, m)), js(l, m);
      },
      enqueueForceUpdate: function(e, t) {
        var o = xa(e), l = da(), d = Kl(o), m = Ou(l, d);
        m.tag = zy, t != null && (v0(t, "forceUpdate"), m.callback = t);
        var y = Ul(o, m, d);
        y !== null && (dr(y, o, d, l), jy(y, o, d)), Of(o, d);
      }
    };
    function rR(e, t, o, l, d, m, y) {
      var x = e.stateNode;
      if (typeof x.shouldComponentUpdate == "function") {
        var T = x.shouldComponentUpdate(l, m, y);
        {
          if (e.mode & xt) {
            Nn(!0);
            try {
              T = x.shouldComponentUpdate(l, m, y);
            } finally {
              Nn(!1);
            }
          }
          T === void 0 && h("%s.shouldComponentUpdate(): Returned undefined instead of a boolean value. Make sure to return true or false.", Ut(t) || "Component");
        }
        return T;
      }
      return t.prototype && t.prototype.isPureReactComponent ? !ot(o, l) || !ot(d, m) : !0;
    }
    function cL(e, t, o) {
      var l = e.stateNode;
      {
        var d = Ut(t) || "Component", m = l.render;
        m || (t.prototype && typeof t.prototype.render == "function" ? h("%s(...): No `render` method found on the returned component instance: did you accidentally return an object from the constructor?", d) : h("%s(...): No `render` method found on the returned component instance: you may have forgotten to define `render`.", d)), l.getInitialState && !l.getInitialState.isReactClassApproved && !l.state && h("getInitialState was defined on %s, a plain JavaScript class. This is only supported for classes created using React.createClass. Did you mean to define a state property instead?", d), l.getDefaultProps && !l.getDefaultProps.isReactClassApproved && h("getDefaultProps was defined on %s, a plain JavaScript class. This is only supported for classes created using React.createClass. Use a static property to define defaultProps instead.", d), l.propTypes && h("propTypes was defined as an instance property on %s. Use a static property to define propTypes instead.", d), l.contextType && h("contextType was defined as an instance property on %s. Use a static property to define contextType instead.", d), t.childContextTypes && !mh.has(t) && // Strict Mode has its own warning for legacy context, so we can skip
        // this one.
        (e.mode & xt) === Xe && (mh.add(t), h(`%s uses the legacy childContextTypes API which is no longer supported and will be removed in the next major release. Use React.createContext() instead

.Learn more about this warning here: https://reactjs.org/link/legacy-context`, d)), t.contextTypes && !mh.has(t) && // Strict Mode has its own warning for legacy context, so we can skip
        // this one.
        (e.mode & xt) === Xe && (mh.add(t), h(`%s uses the legacy contextTypes API which is no longer supported and will be removed in the next major release. Use React.createContext() with static contextType instead.

Learn more about this warning here: https://reactjs.org/link/legacy-context`, d)), l.contextTypes && h("contextTypes was defined as an instance property on %s. Use a static property to define contextTypes instead.", d), t.contextType && t.contextTypes && !lS.has(t) && (lS.add(t), h("%s declares both contextTypes and contextType static properties. The legacy contextTypes property will be ignored.", d)), typeof l.componentShouldUpdate == "function" && h("%s has a method called componentShouldUpdate(). Did you mean shouldComponentUpdate()? The name is phrased as a question because the function is expected to return a value.", d), t.prototype && t.prototype.isPureReactComponent && typeof l.shouldComponentUpdate < "u" && h("%s has a method called shouldComponentUpdate(). shouldComponentUpdate should not be used when extending React.PureComponent. Please extend React.Component if shouldComponentUpdate is used.", Ut(t) || "A pure component"), typeof l.componentDidUnmount == "function" && h("%s has a method called componentDidUnmount(). But there is no such lifecycle method. Did you mean componentWillUnmount()?", d), typeof l.componentDidReceiveProps == "function" && h("%s has a method called componentDidReceiveProps(). But there is no such lifecycle method. If you meant to update the state in response to changing props, use componentWillReceiveProps(). If you meant to fetch data or run side-effects or mutations after React has updated the UI, use componentDidUpdate().", d), typeof l.componentWillRecieveProps == "function" && h("%s has a method called componentWillRecieveProps(). Did you mean componentWillReceiveProps()?", d), typeof l.UNSAFE_componentWillRecieveProps == "function" && h("%s has a method called UNSAFE_componentWillRecieveProps(). Did you mean UNSAFE_componentWillReceiveProps()?", d);
        var y = l.props !== o;
        l.props !== void 0 && y && h("%s(...): When calling super() in `%s`, make sure to pass up the same props that your component's constructor was passed.", d, d), l.defaultProps && h("Setting defaultProps as an instance property on %s is not supported and will be ignored. Instead, define defaultProps as a static property on %s.", d, d), typeof l.getSnapshotBeforeUpdate == "function" && typeof l.componentDidUpdate != "function" && !aS.has(t) && (aS.add(t), h("%s: getSnapshotBeforeUpdate() should be used with componentDidUpdate(). This component defines getSnapshotBeforeUpdate() only.", Ut(t))), typeof l.getDerivedStateFromProps == "function" && h("%s: getDerivedStateFromProps() is defined as an instance method and will be ignored. Instead, declare it as a static method.", d), typeof l.getDerivedStateFromError == "function" && h("%s: getDerivedStateFromError() is defined as an instance method and will be ignored. Instead, declare it as a static method.", d), typeof t.getSnapshotBeforeUpdate == "function" && h("%s: getSnapshotBeforeUpdate() is defined as a static method and will be ignored. Instead, declare it as an instance method.", d);
        var x = l.state;
        x && (typeof x != "object" || Wn(x)) && h("%s.state: must be set to an object or null", d), typeof l.getChildContext == "function" && typeof t.childContextTypes != "object" && h("%s.getChildContext(): childContextTypes must be defined in order to use getChildContext().", d);
      }
    }
    function aR(e, t) {
      t.updater = fS, e.stateNode = t, ul(t, e), t._reactInternalInstance = tS;
    }
    function iR(e, t, o) {
      var l = !1, d = ai, m = ai, y = t.contextType;
      if ("contextType" in t) {
        var x = (
          // Allow null for conditional declaration
          y === null || y !== void 0 && y.$$typeof === Tp && y._context === void 0
        );
        if (!x && !sS.has(t)) {
          sS.add(t);
          var T = "";
          y === void 0 ? T = " However, it is set to undefined. This can be caused by a typo or by mixing up named and default imports. This can also happen due to a circular dependency, so try moving the createContext() call to a separate file." : typeof y != "object" ? T = " However, it is set to a " + typeof y + "." : y.$$typeof === wp ? T = " Did you accidentally pass the Context.Provider instead?" : y._context !== void 0 ? T = " Did you accidentally pass the Context.Consumer instead?" : T = " However, it is set to an object with keys {" + Object.keys(y).join(", ") + "}.", h("%s defines an invalid contextType. contextType should point to the Context object returned by React.createContext().%s", Ut(t) || "Component", T);
        }
      }
      if (typeof y == "object" && y !== null)
        m = Jn(y);
      else {
        d = Ed(e, t, !0);
        var O = t.contextTypes;
        l = O != null, m = l ? xd(e, d) : ai;
      }
      var N = new t(o, m);
      if (e.mode & xt) {
        Nn(!0);
        try {
          N = new t(o, m);
        } finally {
          Nn(!1);
        }
      }
      var j = e.memoizedState = N.state !== null && N.state !== void 0 ? N.state : null;
      aR(e, N);
      {
        if (typeof t.getDerivedStateFromProps == "function" && j === null) {
          var G = Ut(t) || "Component";
          rS.has(G) || (rS.add(G), h("`%s` uses `getDerivedStateFromProps` but its initial state is %s. This is not recommended. Instead, define the initial state by assigning an object to `this.state` in the constructor of `%s`. This ensures that `getDerivedStateFromProps` arguments have a consistent shape.", G, N.state === null ? "null" : "undefined", G));
        }
        if (typeof t.getDerivedStateFromProps == "function" || typeof N.getSnapshotBeforeUpdate == "function") {
          var te = null, re = null, ie = null;
          if (typeof N.componentWillMount == "function" && N.componentWillMount.__suppressDeprecationWarning !== !0 ? te = "componentWillMount" : typeof N.UNSAFE_componentWillMount == "function" && (te = "UNSAFE_componentWillMount"), typeof N.componentWillReceiveProps == "function" && N.componentWillReceiveProps.__suppressDeprecationWarning !== !0 ? re = "componentWillReceiveProps" : typeof N.UNSAFE_componentWillReceiveProps == "function" && (re = "UNSAFE_componentWillReceiveProps"), typeof N.componentWillUpdate == "function" && N.componentWillUpdate.__suppressDeprecationWarning !== !0 ? ie = "componentWillUpdate" : typeof N.UNSAFE_componentWillUpdate == "function" && (ie = "UNSAFE_componentWillUpdate"), te !== null || re !== null || ie !== null) {
            var Ne = Ut(t) || "Component", et = typeof t.getDerivedStateFromProps == "function" ? "getDerivedStateFromProps()" : "getSnapshotBeforeUpdate()";
            iS.has(Ne) || (iS.add(Ne), h(`Unsafe legacy lifecycles will not be called for components using new component APIs.

%s uses %s but also contains the following legacy lifecycles:%s%s%s

The above lifecycles should be removed. Learn more about this warning here:
https://reactjs.org/link/unsafe-component-lifecycles`, Ne, et, te !== null ? `
  ` + te : "", re !== null ? `
  ` + re : "", ie !== null ? `
  ` + ie : ""));
          }
        }
      }
      return l && Yw(e, d, m), N;
    }
    function fL(e, t) {
      var o = t.state;
      typeof t.componentWillMount == "function" && t.componentWillMount(), typeof t.UNSAFE_componentWillMount == "function" && t.UNSAFE_componentWillMount(), o !== t.state && (h("%s.componentWillMount(): Assigning directly to this.state is deprecated (except inside a component's constructor). Use setState instead.", bt(e) || "Component"), fS.enqueueReplaceState(t, t.state, null));
    }
    function oR(e, t, o, l) {
      var d = t.state;
      if (typeof t.componentWillReceiveProps == "function" && t.componentWillReceiveProps(o, l), typeof t.UNSAFE_componentWillReceiveProps == "function" && t.UNSAFE_componentWillReceiveProps(o, l), t.state !== d) {
        {
          var m = bt(e) || "Component";
          nS.has(m) || (nS.add(m), h("%s.componentWillReceiveProps(): Assigning directly to this.state is deprecated (except inside a component's constructor). Use setState instead.", m));
        }
        fS.enqueueReplaceState(t, t.state, null);
      }
    }
    function dS(e, t, o, l) {
      cL(e, t, o);
      var d = e.stateNode;
      d.props = o, d.state = e.memoizedState, d.refs = {}, S1(e);
      var m = t.contextType;
      if (typeof m == "object" && m !== null)
        d.context = Jn(m);
      else {
        var y = Ed(e, t, !0);
        d.context = xd(e, y);
      }
      {
        if (d.state === o) {
          var x = Ut(t) || "Component";
          uS.has(x) || (uS.add(x), h("%s: It is not recommended to assign props directly to state because updates to props won't be reflected in state. In most cases, it is better to use props directly.", x));
        }
        e.mode & xt && Ji.recordLegacyContextWarning(e, d), Ji.recordUnsafeLifecycleWarnings(e, d);
      }
      d.state = e.memoizedState;
      var T = t.getDerivedStateFromProps;
      if (typeof T == "function" && (cS(e, t, T, o), d.state = e.memoizedState), typeof t.getDerivedStateFromProps != "function" && typeof d.getSnapshotBeforeUpdate != "function" && (typeof d.UNSAFE_componentWillMount == "function" || typeof d.componentWillMount == "function") && (fL(e, d), Uy(e, o, d, l), d.state = e.memoizedState), typeof d.componentDidMount == "function") {
        var O = St;
        O |= Hr, (e.mode & la) !== Xe && (O |= zr), e.flags |= O;
      }
    }
    function dL(e, t, o, l) {
      var d = e.stateNode, m = e.memoizedProps;
      d.props = m;
      var y = d.context, x = t.contextType, T = ai;
      if (typeof x == "object" && x !== null)
        T = Jn(x);
      else {
        var O = Ed(e, t, !0);
        T = xd(e, O);
      }
      var N = t.getDerivedStateFromProps, j = typeof N == "function" || typeof d.getSnapshotBeforeUpdate == "function";
      !j && (typeof d.UNSAFE_componentWillReceiveProps == "function" || typeof d.componentWillReceiveProps == "function") && (m !== o || y !== T) && oR(e, d, o, T), ST();
      var G = e.memoizedState, te = d.state = G;
      if (Uy(e, o, d, l), te = e.memoizedState, m === o && G === te && !Ty() && !Wy()) {
        if (typeof d.componentDidMount == "function") {
          var re = St;
          re |= Hr, (e.mode & la) !== Xe && (re |= zr), e.flags |= re;
        }
        return !1;
      }
      typeof N == "function" && (cS(e, t, N, o), te = e.memoizedState);
      var ie = Wy() || rR(e, t, m, o, G, te, T);
      if (ie) {
        if (!j && (typeof d.UNSAFE_componentWillMount == "function" || typeof d.componentWillMount == "function") && (typeof d.componentWillMount == "function" && d.componentWillMount(), typeof d.UNSAFE_componentWillMount == "function" && d.UNSAFE_componentWillMount()), typeof d.componentDidMount == "function") {
          var Ne = St;
          Ne |= Hr, (e.mode & la) !== Xe && (Ne |= zr), e.flags |= Ne;
        }
      } else {
        if (typeof d.componentDidMount == "function") {
          var et = St;
          et |= Hr, (e.mode & la) !== Xe && (et |= zr), e.flags |= et;
        }
        e.memoizedProps = o, e.memoizedState = te;
      }
      return d.props = o, d.state = te, d.context = T, ie;
    }
    function pL(e, t, o, l, d) {
      var m = t.stateNode;
      CT(e, t);
      var y = t.memoizedProps, x = t.type === t.elementType ? y : eo(t.type, y);
      m.props = x;
      var T = t.pendingProps, O = m.context, N = o.contextType, j = ai;
      if (typeof N == "object" && N !== null)
        j = Jn(N);
      else {
        var G = Ed(t, o, !0);
        j = xd(t, G);
      }
      var te = o.getDerivedStateFromProps, re = typeof te == "function" || typeof m.getSnapshotBeforeUpdate == "function";
      !re && (typeof m.UNSAFE_componentWillReceiveProps == "function" || typeof m.componentWillReceiveProps == "function") && (y !== T || O !== j) && oR(t, m, l, j), ST();
      var ie = t.memoizedState, Ne = m.state = ie;
      if (Uy(t, l, m, d), Ne = t.memoizedState, y === T && ie === Ne && !Ty() && !Wy() && !we)
        return typeof m.componentDidUpdate == "function" && (y !== e.memoizedProps || ie !== e.memoizedState) && (t.flags |= St), typeof m.getSnapshotBeforeUpdate == "function" && (y !== e.memoizedProps || ie !== e.memoizedState) && (t.flags |= wa), !1;
      typeof te == "function" && (cS(t, o, te, l), Ne = t.memoizedState);
      var et = Wy() || rR(t, o, x, l, ie, Ne, j) || // TODO: In some cases, we'll end up checking if context has changed twice,
      // both before and after `shouldComponentUpdate` has been called. Not ideal,
      // but I'm loath to refactor this function. This only happens for memoized
      // components so it's not that common.
      we;
      return et ? (!re && (typeof m.UNSAFE_componentWillUpdate == "function" || typeof m.componentWillUpdate == "function") && (typeof m.componentWillUpdate == "function" && m.componentWillUpdate(l, Ne, j), typeof m.UNSAFE_componentWillUpdate == "function" && m.UNSAFE_componentWillUpdate(l, Ne, j)), typeof m.componentDidUpdate == "function" && (t.flags |= St), typeof m.getSnapshotBeforeUpdate == "function" && (t.flags |= wa)) : (typeof m.componentDidUpdate == "function" && (y !== e.memoizedProps || ie !== e.memoizedState) && (t.flags |= St), typeof m.getSnapshotBeforeUpdate == "function" && (y !== e.memoizedProps || ie !== e.memoizedState) && (t.flags |= wa), t.memoizedProps = l, t.memoizedState = Ne), m.props = l, m.state = Ne, m.context = j, et;
    }
    function Cc(e, t) {
      return {
        value: e,
        source: t,
        stack: Np(t),
        digest: null
      };
    }
    function pS(e, t, o) {
      return {
        value: e,
        source: null,
        stack: o ?? null,
        digest: t ?? null
      };
    }
    function vL(e, t) {
      return !0;
    }
    function vS(e, t) {
      try {
        var o = vL(e, t);
        if (o === !1)
          return;
        var l = t.value, d = t.source, m = t.stack, y = m !== null ? m : "";
        if (l != null && l._suppressLogging) {
          if (e.tag === E)
            return;
          console.error(l);
        }
        var x = d ? bt(d) : null, T = x ? "The above error occurred in the <" + x + "> component:" : "The above error occurred in one of your React components:", O;
        if (e.tag === R)
          O = `Consider adding an error boundary to your tree to customize error handling behavior.
Visit https://reactjs.org/link/error-boundaries to learn more about error boundaries.`;
        else {
          var N = bt(e) || "Anonymous";
          O = "React will try to recreate this component tree from scratch " + ("using the error boundary you provided, " + N + ".");
        }
        var j = T + `
` + y + `

` + ("" + O);
        console.error(j);
      } catch (G) {
        setTimeout(function() {
          throw G;
        });
      }
    }
    var hL = typeof WeakMap == "function" ? WeakMap : Map;
    function uR(e, t, o) {
      var l = Ou(tn, o);
      l.tag = b1, l.payload = {
        element: null
      };
      var d = t.value;
      return l.callback = function() {
        u4(d), vS(e, t);
      }, l;
    }
    function hS(e, t, o) {
      var l = Ou(tn, o);
      l.tag = b1;
      var d = e.type.getDerivedStateFromError;
      if (typeof d == "function") {
        var m = t.value;
        l.payload = function() {
          return d(m);
        }, l.callback = function() {
          yI(e), vS(e, t);
        };
      }
      var y = e.stateNode;
      return y !== null && typeof y.componentDidCatch == "function" && (l.callback = function() {
        yI(e), vS(e, t), typeof d != "function" && i4(this);
        var T = t.value, O = t.stack;
        this.componentDidCatch(T, {
          componentStack: O !== null ? O : ""
        }), typeof d != "function" && (Xr(e.lanes, Je) || h("%s: Error boundaries should implement getDerivedStateFromError(). In that method, return a state update to display an error message or fallback UI.", bt(e) || "Unknown"));
      }), l;
    }
    function lR(e, t, o) {
      var l = e.pingCache, d;
      if (l === null ? (l = e.pingCache = new hL(), d = /* @__PURE__ */ new Set(), l.set(t, d)) : (d = l.get(t), d === void 0 && (d = /* @__PURE__ */ new Set(), l.set(t, d))), !d.has(o)) {
        d.add(o);
        var m = l4.bind(null, e, t, o);
        ua && Mh(e, o), t.then(m, m);
      }
    }
    function mL(e, t, o, l) {
      var d = e.updateQueue;
      if (d === null) {
        var m = /* @__PURE__ */ new Set();
        m.add(o), e.updateQueue = m;
      } else
        d.add(o);
    }
    function gL(e, t) {
      var o = e.tag;
      if ((e.mode & $e) === Xe && (o === b || o === z || o === P)) {
        var l = e.alternate;
        l ? (e.updateQueue = l.updateQueue, e.memoizedState = l.memoizedState, e.lanes = l.lanes) : (e.updateQueue = null, e.memoizedState = null);
      }
    }
    function sR(e) {
      var t = e;
      do {
        if (t.tag === V && JM(t))
          return t;
        t = t.return;
      } while (t !== null);
      return null;
    }
    function cR(e, t, o, l, d) {
      if ((e.mode & $e) === Xe) {
        if (e === t)
          e.flags |= Zn;
        else {
          if (e.flags |= dt, o.flags |= zs, o.flags &= ~(gf | aa), o.tag === E) {
            var m = o.alternate;
            if (m === null)
              o.tag = se;
            else {
              var y = Ou(tn, Je);
              y.tag = zy, Ul(o, y, Je);
            }
          }
          o.lanes = Ct(o.lanes, Je);
        }
        return e;
      }
      return e.flags |= Zn, e.lanes = d, e;
    }
    function yL(e, t, o, l, d) {
      if (o.flags |= aa, ua && Mh(e, d), l !== null && typeof l == "object" && typeof l.then == "function") {
        var m = l;
        gL(o), Tr() && o.mode & $e && tT();
        var y = sR(t);
        if (y !== null) {
          y.flags &= ~gn, cR(y, t, o, e, d), y.mode & $e && lR(e, m, d), mL(y, e, m);
          return;
        } else {
          if (!bv(d)) {
            lR(e, m, d), YS();
            return;
          }
          var x = new Error("A component suspended while responding to synchronous input. This will cause the UI to be replaced with a loading indicator. To fix, updates that suspend should be wrapped with startTransition.");
          l = x;
        }
      } else if (Tr() && o.mode & $e) {
        tT();
        var T = sR(t);
        if (T !== null) {
          (T.flags & Zn) === qe && (T.flags |= gn), cR(T, t, o, e, d), u1(Cc(l, o));
          return;
        }
      }
      l = Cc(l, o), JF(l);
      var O = t;
      do {
        switch (O.tag) {
          case R: {
            var N = l;
            O.flags |= Zn;
            var j = Sl(d);
            O.lanes = Ct(O.lanes, j);
            var G = uR(O, N, j);
            E1(O, G);
            return;
          }
          case E:
            var te = l, re = O.type, ie = O.stateNode;
            if ((O.flags & dt) === qe && (typeof re.getDerivedStateFromError == "function" || ie !== null && typeof ie.componentDidCatch == "function" && !sI(ie))) {
              O.flags |= Zn;
              var Ne = Sl(d);
              O.lanes = Ct(O.lanes, Ne);
              var et = hS(O, te, Ne);
              E1(O, et);
              return;
            }
            break;
        }
        O = O.return;
      } while (O !== null);
    }
    function bL() {
      return null;
    }
    var gh = u.ReactCurrentOwner, to = !1, mS, yh, gS, yS, bS, Sc, CS, h0, bh;
    mS = {}, yh = {}, gS = {}, yS = {}, bS = {}, Sc = !1, CS = {}, h0 = {}, bh = {};
    function ca(e, t, o, l) {
      e === null ? t.child = pT(t, null, o, l) : t.child = Id(t, e.child, o, l);
    }
    function CL(e, t, o, l) {
      t.child = Id(t, e.child, null, l), t.child = Id(t, null, o, l);
    }
    function fR(e, t, o, l, d) {
      if (t.type !== t.elementType) {
        var m = o.propTypes;
        m && $i(
          m,
          l,
          // Resolved props
          "prop",
          Ut(o)
        );
      }
      var y = o.render, x = t.ref, T, O;
      Od(t, d), fl(t);
      {
        if (gh.current = t, qa(!0), T = Fd(e, t, y, l, x, d), O = Bd(), t.mode & xt) {
          Nn(!0);
          try {
            T = Fd(e, t, y, l, x, d), O = Bd();
          } finally {
            Nn(!1);
          }
        }
        qa(!1);
      }
      return jr(), e !== null && !to ? (IT(e, t, d), Du(e, t, d)) : (Tr() && O && t1(t), t.flags |= bo, ca(e, t, T, d), t.child);
    }
    function dR(e, t, o, l, d) {
      if (e === null) {
        var m = o.type;
        if (T4(m) && o.compare === null && // SimpleMemoComponent codepath doesn't resolve outer props either.
        o.defaultProps === void 0) {
          var y = m;
          return y = Ud(m), t.tag = P, t.type = y, xS(t, m), pR(e, t, y, l, d);
        }
        {
          var x = m.propTypes;
          if (x && $i(
            x,
            l,
            // Resolved props
            "prop",
            Ut(m)
          ), o.defaultProps !== void 0) {
            var T = Ut(m) || "Unknown";
            bh[T] || (h("%s: Support for defaultProps will be removed from memo components in a future major release. Use JavaScript default parameters instead.", T), bh[T] = !0);
          }
        }
        var O = iE(o.type, null, l, t, t.mode, d);
        return O.ref = t.ref, O.return = t, t.child = O, O;
      }
      {
        var N = o.type, j = N.propTypes;
        j && $i(
          j,
          l,
          // Resolved props
          "prop",
          Ut(N)
        );
      }
      var G = e.child, te = OS(e, d);
      if (!te) {
        var re = G.memoizedProps, ie = o.compare;
        if (ie = ie !== null ? ie : ot, ie(re, l) && e.ref === t.ref)
          return Du(e, t, d);
      }
      t.flags |= bo;
      var Ne = Rc(G, l);
      return Ne.ref = t.ref, Ne.return = t, t.child = Ne, Ne;
    }
    function pR(e, t, o, l, d) {
      if (t.type !== t.elementType) {
        var m = t.elementType;
        if (m.$$typeof === Er) {
          var y = m, x = y._payload, T = y._init;
          try {
            m = T(x);
          } catch {
            m = null;
          }
          var O = m && m.propTypes;
          O && $i(
            O,
            l,
            // Resolved (SimpleMemoComponent has no defaultProps)
            "prop",
            Ut(m)
          );
        }
      }
      if (e !== null) {
        var N = e.memoizedProps;
        if (ot(N, l) && e.ref === t.ref && // Prevent bailout if the implementation changed due to hot reload.
        t.type === e.type)
          if (to = !1, t.pendingProps = l = N, OS(e, d))
            (e.flags & zs) !== qe && (to = !0);
          else
            return t.lanes = e.lanes, Du(e, t, d);
      }
      return SS(e, t, o, l, d);
    }
    function vR(e, t, o) {
      var l = t.pendingProps, d = l.children, m = e !== null ? e.memoizedState : null;
      if (l.mode === "hidden" || W)
        if ((t.mode & $e) === Xe) {
          var y = {
            baseLanes: fe,
            cachePool: null,
            transitions: null
          };
          t.memoizedState = y, A0(t, o);
        } else if (Xr(o, gr)) {
          var j = {
            baseLanes: fe,
            cachePool: null,
            transitions: null
          };
          t.memoizedState = j;
          var G = m !== null ? m.baseLanes : o;
          A0(t, G);
        } else {
          var x = null, T;
          if (m !== null) {
            var O = m.baseLanes;
            T = Ct(O, o);
          } else
            T = o;
          t.lanes = t.childLanes = gr;
          var N = {
            baseLanes: T,
            cachePool: x,
            transitions: null
          };
          return t.memoizedState = N, t.updateQueue = null, A0(t, T), null;
        }
      else {
        var te;
        m !== null ? (te = Ct(m.baseLanes, o), t.memoizedState = null) : te = o, A0(t, te);
      }
      return ca(e, t, d, o), t.child;
    }
    function SL(e, t, o) {
      var l = t.pendingProps;
      return ca(e, t, l, o), t.child;
    }
    function EL(e, t, o) {
      var l = t.pendingProps.children;
      return ca(e, t, l, o), t.child;
    }
    function xL(e, t, o) {
      {
        t.flags |= St;
        {
          var l = t.stateNode;
          l.effectDuration = 0, l.passiveEffectDuration = 0;
        }
      }
      var d = t.pendingProps, m = d.children;
      return ca(e, t, m, o), t.child;
    }
    function hR(e, t) {
      var o = t.ref;
      (e === null && o !== null || e !== null && e.ref !== o) && (t.flags |= Pr, t.flags |= ov);
    }
    function SS(e, t, o, l, d) {
      if (t.type !== t.elementType) {
        var m = o.propTypes;
        m && $i(
          m,
          l,
          // Resolved props
          "prop",
          Ut(o)
        );
      }
      var y;
      {
        var x = Ed(t, o, !0);
        y = xd(t, x);
      }
      var T, O;
      Od(t, d), fl(t);
      {
        if (gh.current = t, qa(!0), T = Fd(e, t, o, l, y, d), O = Bd(), t.mode & xt) {
          Nn(!0);
          try {
            T = Fd(e, t, o, l, y, d), O = Bd();
          } finally {
            Nn(!1);
          }
        }
        qa(!1);
      }
      return jr(), e !== null && !to ? (IT(e, t, d), Du(e, t, d)) : (Tr() && O && t1(t), t.flags |= bo, ca(e, t, T, d), t.child);
    }
    function mR(e, t, o, l, d) {
      {
        switch (z4(t)) {
          case !1: {
            var m = t.stateNode, y = t.type, x = new y(t.memoizedProps, m.context), T = x.state;
            m.updater.enqueueSetState(m, T, null);
            break;
          }
          case !0: {
            t.flags |= dt, t.flags |= Zn;
            var O = new Error("Simulated error coming from DevTools"), N = Sl(d);
            t.lanes = Ct(t.lanes, N);
            var j = hS(t, Cc(O, t), N);
            E1(t, j);
            break;
          }
        }
        if (t.type !== t.elementType) {
          var G = o.propTypes;
          G && $i(
            G,
            l,
            // Resolved props
            "prop",
            Ut(o)
          );
        }
      }
      var te;
      ko(o) ? (te = !0, Iy(t)) : te = !1, Od(t, d);
      var re = t.stateNode, ie;
      re === null ? (g0(e, t), iR(t, o, l), dS(t, o, l, d), ie = !0) : e === null ? ie = dL(t, o, l, d) : ie = pL(e, t, o, l, d);
      var Ne = ES(e, t, o, ie, te, d);
      {
        var et = t.stateNode;
        ie && et.props !== l && (Sc || h("It looks like %s is reassigning its own `this.props` while rendering. This is not supported and can lead to confusing bugs.", bt(t) || "a component"), Sc = !0);
      }
      return Ne;
    }
    function ES(e, t, o, l, d, m) {
      hR(e, t);
      var y = (t.flags & dt) !== qe;
      if (!l && !y)
        return d && Jw(t, o, !1), Du(e, t, m);
      var x = t.stateNode;
      gh.current = t;
      var T;
      if (y && typeof o.getDerivedStateFromError != "function")
        T = null, eR();
      else {
        fl(t);
        {
          if (qa(!0), T = x.render(), t.mode & xt) {
            Nn(!0);
            try {
              x.render();
            } finally {
              Nn(!1);
            }
          }
          qa(!1);
        }
        jr();
      }
      return t.flags |= bo, e !== null && y ? CL(e, t, T, m) : ca(e, t, T, m), t.memoizedState = x.state, d && Jw(t, o, !0), t.child;
    }
    function gR(e) {
      var t = e.stateNode;
      t.pendingContext ? $w(e, t.pendingContext, t.pendingContext !== t.context) : t.context && $w(e, t.context, !1), x1(e, t.containerInfo);
    }
    function wL(e, t, o) {
      if (gR(t), e === null)
        throw new Error("Should have a current fiber. This is a bug in React.");
      var l = t.pendingProps, d = t.memoizedState, m = d.element;
      CT(e, t), Uy(t, l, null, o);
      var y = t.memoizedState;
      t.stateNode;
      var x = y.element;
      if (d.isDehydrated) {
        var T = {
          element: x,
          isDehydrated: !1,
          cache: y.cache,
          pendingSuspenseBoundaries: y.pendingSuspenseBoundaries,
          transitions: y.transitions
        }, O = t.updateQueue;
        if (O.baseState = T, t.memoizedState = T, t.flags & gn) {
          var N = Cc(new Error("There was an error while hydrating. Because the error happened outside of a Suspense boundary, the entire root will switch to client rendering."), t);
          return yR(e, t, x, o, N);
        } else if (x !== m) {
          var j = Cc(new Error("This root received an early update, before anything was able hydrate. Switched the entire root to client rendering."), t);
          return yR(e, t, x, o, j);
        } else {
          IM(t);
          var G = pT(t, null, x, o);
          t.child = G;
          for (var te = G; te; )
            te.flags = te.flags & ~nn | Ta, te = te.sibling;
        }
      } else {
        if (Rd(), x === m)
          return Du(e, t, o);
        ca(e, t, x, o);
      }
      return t.child;
    }
    function yR(e, t, o, l, d) {
      return Rd(), u1(d), t.flags |= gn, ca(e, t, o, l), t.child;
    }
    function TL(e, t, o) {
      wT(t), e === null && o1(t);
      var l = t.type, d = t.pendingProps, m = e !== null ? e.memoizedProps : null, y = d.children, x = GC(l, d);
      return x ? y = null : m !== null && GC(l, m) && (t.flags |= Ht), hR(e, t), ca(e, t, y, o), t.child;
    }
    function RL(e, t) {
      return e === null && o1(t), null;
    }
    function IL(e, t, o, l) {
      g0(e, t);
      var d = t.pendingProps, m = o, y = m._payload, x = m._init, T = x(y);
      t.type = T;
      var O = t.tag = R4(T), N = eo(T, d), j;
      switch (O) {
        case b:
          return xS(t, T), t.type = T = Ud(T), j = SS(null, t, T, N, l), j;
        case E:
          return t.type = T = qS(T), j = mR(null, t, T, N, l), j;
        case z:
          return t.type = T = eE(T), j = fR(null, t, T, N, l), j;
        case Q: {
          if (t.type !== t.elementType) {
            var G = T.propTypes;
            G && $i(
              G,
              N,
              // Resolved for outer only
              "prop",
              Ut(T)
            );
          }
          return j = dR(
            null,
            t,
            T,
            eo(T.type, N),
            // The inner type can have defaults too
            l
          ), j;
        }
      }
      var te = "";
      throw T !== null && typeof T == "object" && T.$$typeof === Er && (te = " Did you wrap a component in React.lazy() more than once?"), new Error("Element type is invalid. Received a promise that resolves to: " + T + ". " + ("Lazy element type must resolve to a class or function." + te));
    }
    function AL(e, t, o, l, d) {
      g0(e, t), t.tag = E;
      var m;
      return ko(o) ? (m = !0, Iy(t)) : m = !1, Od(t, d), iR(t, o, l), dS(t, o, l, d), ES(null, t, o, !0, m, d);
    }
    function OL(e, t, o, l) {
      g0(e, t);
      var d = t.pendingProps, m;
      {
        var y = Ed(t, o, !1);
        m = xd(t, y);
      }
      Od(t, l);
      var x, T;
      fl(t);
      {
        if (o.prototype && typeof o.prototype.render == "function") {
          var O = Ut(o) || "Unknown";
          mS[O] || (h("The <%s /> component appears to have a render method, but doesn't extend React.Component. This is likely to cause errors. Change %s to extend React.Component instead.", O, O), mS[O] = !0);
        }
        t.mode & xt && Ji.recordLegacyContextWarning(t, null), qa(!0), gh.current = t, x = Fd(null, t, o, d, m, l), T = Bd(), qa(!1);
      }
      if (jr(), t.flags |= bo, typeof x == "object" && x !== null && typeof x.render == "function" && x.$$typeof === void 0) {
        var N = Ut(o) || "Unknown";
        yh[N] || (h("The <%s /> component appears to be a function component that returns a class instance. Change %s to a class that extends React.Component instead. If you can't use a class try assigning the prototype on the function as a workaround. `%s.prototype = React.Component.prototype`. Don't use an arrow function since it cannot be called with `new` by React.", N, N, N), yh[N] = !0);
      }
      if (
        // Run these checks in production only if the flag is off.
        // Eventually we'll delete this branch altogether.
        typeof x == "object" && x !== null && typeof x.render == "function" && x.$$typeof === void 0
      ) {
        {
          var j = Ut(o) || "Unknown";
          yh[j] || (h("The <%s /> component appears to be a function component that returns a class instance. Change %s to a class that extends React.Component instead. If you can't use a class try assigning the prototype on the function as a workaround. `%s.prototype = React.Component.prototype`. Don't use an arrow function since it cannot be called with `new` by React.", j, j, j), yh[j] = !0);
        }
        t.tag = E, t.memoizedState = null, t.updateQueue = null;
        var G = !1;
        return ko(o) ? (G = !0, Iy(t)) : G = !1, t.memoizedState = x.state !== null && x.state !== void 0 ? x.state : null, S1(t), aR(t, x), dS(t, o, d, l), ES(null, t, o, !0, G, l);
      } else {
        if (t.tag = b, t.mode & xt) {
          Nn(!0);
          try {
            x = Fd(null, t, o, d, m, l), T = Bd();
          } finally {
            Nn(!1);
          }
        }
        return Tr() && T && t1(t), ca(null, t, x, l), xS(t, o), t.child;
      }
    }
    function xS(e, t) {
      {
        if (t && t.childContextTypes && h("%s(...): childContextTypes cannot be defined on a function component.", t.displayName || t.name || "Component"), e.ref !== null) {
          var o = "", l = Sa();
          l && (o += `

Check the render method of \`` + l + "`.");
          var d = l || "", m = e._debugSource;
          m && (d = m.fileName + ":" + m.lineNumber), bS[d] || (bS[d] = !0, h("Function components cannot be given refs. Attempts to access this ref will fail. Did you mean to use React.forwardRef()?%s", o));
        }
        if (t.defaultProps !== void 0) {
          var y = Ut(t) || "Unknown";
          bh[y] || (h("%s: Support for defaultProps will be removed from function components in a future major release. Use JavaScript default parameters instead.", y), bh[y] = !0);
        }
        if (typeof t.getDerivedStateFromProps == "function") {
          var x = Ut(t) || "Unknown";
          yS[x] || (h("%s: Function components do not support getDerivedStateFromProps.", x), yS[x] = !0);
        }
        if (typeof t.contextType == "object" && t.contextType !== null) {
          var T = Ut(t) || "Unknown";
          gS[T] || (h("%s: Function components do not support contextType.", T), gS[T] = !0);
        }
      }
    }
    var wS = {
      dehydrated: null,
      treeContext: null,
      retryLane: kn
    };
    function TS(e) {
      return {
        baseLanes: e,
        cachePool: bL(),
        transitions: null
      };
    }
    function DL(e, t) {
      var o = null;
      return {
        baseLanes: Ct(e.baseLanes, t),
        cachePool: o,
        transitions: e.transitions
      };
    }
    function NL(e, t, o, l) {
      if (t !== null) {
        var d = t.memoizedState;
        if (d === null)
          return !1;
      }
      return R1(e, lh);
    }
    function kL(e, t) {
      return Qs(e.childLanes, t);
    }
    function bR(e, t, o) {
      var l = t.pendingProps;
      G4(t) && (t.flags |= dt);
      var d = Ki.current, m = !1, y = (t.flags & dt) !== qe;
      if (y || NL(d, e) ? (m = !0, t.flags &= ~dt) : (e === null || e.memoizedState !== null) && (d = QM(d, RT)), d = Nd(d), Xl(t, d), e === null) {
        o1(t);
        var x = t.memoizedState;
        if (x !== null) {
          var T = x.dehydrated;
          if (T !== null)
            return VL(t, T);
        }
        var O = l.children, N = l.fallback;
        if (m) {
          var j = ML(t, O, N, o), G = t.child;
          return G.memoizedState = TS(o), t.memoizedState = wS, j;
        } else
          return RS(t, O);
      } else {
        var te = e.memoizedState;
        if (te !== null) {
          var re = te.dehydrated;
          if (re !== null)
            return PL(e, t, y, l, re, te, o);
        }
        if (m) {
          var ie = l.fallback, Ne = l.children, et = FL(e, t, Ne, ie, o), Qe = t.child, Nt = e.child.memoizedState;
          return Qe.memoizedState = Nt === null ? TS(o) : DL(Nt, o), Qe.childLanes = kL(e, o), t.memoizedState = wS, et;
        } else {
          var It = l.children, K = LL(e, t, It, o);
          return t.memoizedState = null, K;
        }
      }
    }
    function RS(e, t, o) {
      var l = e.mode, d = {
        mode: "visible",
        children: t
      }, m = IS(d, l);
      return m.return = e, e.child = m, m;
    }
    function ML(e, t, o, l) {
      var d = e.mode, m = e.child, y = {
        mode: "hidden",
        children: t
      }, x, T;
      return (d & $e) === Xe && m !== null ? (x = m, x.childLanes = fe, x.pendingProps = y, e.mode & pt && (x.actualDuration = 0, x.actualStartTime = -1, x.selfBaseDuration = 0, x.treeBaseDuration = 0), T = es(o, d, l, null)) : (x = IS(y, d), T = es(o, d, l, null)), x.return = e, T.return = e, x.sibling = T, e.child = x, T;
    }
    function IS(e, t, o) {
      return CI(e, t, fe, null);
    }
    function CR(e, t) {
      return Rc(e, t);
    }
    function LL(e, t, o, l) {
      var d = e.child, m = d.sibling, y = CR(d, {
        mode: "visible",
        children: o
      });
      if ((t.mode & $e) === Xe && (y.lanes = l), y.return = t, y.sibling = null, m !== null) {
        var x = t.deletions;
        x === null ? (t.deletions = [m], t.flags |= Pt) : x.push(m);
      }
      return t.child = y, y;
    }
    function FL(e, t, o, l, d) {
      var m = t.mode, y = e.child, x = y.sibling, T = {
        mode: "hidden",
        children: o
      }, O;
      if (
        // In legacy mode, we commit the primary tree as if it successfully
        // completed, even though it's in an inconsistent state.
        (m & $e) === Xe && // Make sure we're on the second pass, i.e. the primary child fragment was
        // already cloned. In legacy mode, the only case where this isn't true is
        // when DevTools forces us to display a fallback; we skip the first render
        // pass entirely and go straight to rendering the fallback. (In Concurrent
        // Mode, SuspenseList can also trigger this scenario, but this is a legacy-
        // only codepath.)
        t.child !== y
      ) {
        var N = t.child;
        O = N, O.childLanes = fe, O.pendingProps = T, t.mode & pt && (O.actualDuration = 0, O.actualStartTime = -1, O.selfBaseDuration = y.selfBaseDuration, O.treeBaseDuration = y.treeBaseDuration), t.deletions = null;
      } else
        O = CR(y, T), O.subtreeFlags = y.subtreeFlags & nr;
      var j;
      return x !== null ? j = Rc(x, l) : (j = es(l, m, d, null), j.flags |= nn), j.return = t, O.return = t, O.sibling = j, t.child = O, j;
    }
    function m0(e, t, o, l) {
      l !== null && u1(l), Id(t, e.child, null, o);
      var d = t.pendingProps, m = d.children, y = RS(t, m);
      return y.flags |= nn, t.memoizedState = null, y;
    }
    function BL(e, t, o, l, d) {
      var m = t.mode, y = {
        mode: "visible",
        children: o
      }, x = IS(y, m), T = es(l, m, d, null);
      return T.flags |= nn, x.return = t, T.return = t, x.sibling = T, t.child = x, (t.mode & $e) !== Xe && Id(t, e.child, null, d), T;
    }
    function VL(e, t, o) {
      return (e.mode & $e) === Xe ? (h("Cannot hydrate Suspense in legacy mode. Switch from ReactDOM.hydrate(element, container) to ReactDOMClient.hydrateRoot(container, <App />).render(element) or remove the Suspense components from the server rendered components."), e.lanes = Je) : WC(t) ? e.lanes = Ui : e.lanes = gr, null;
    }
    function PL(e, t, o, l, d, m, y) {
      if (o)
        if (t.flags & gn) {
          t.flags &= ~gn;
          var K = pS(new Error("There was an error while hydrating this Suspense boundary. Switched to client rendering."));
          return m0(e, t, y, K);
        } else {
          if (t.memoizedState !== null)
            return t.child = e.child, t.flags |= dt, null;
          var oe = l.children, q = l.fallback, me = BL(e, t, oe, q, y), Ve = t.child;
          return Ve.memoizedState = TS(y), t.memoizedState = wS, me;
        }
      else {
        if (TM(), (t.mode & $e) === Xe)
          return m0(
            e,
            t,
            y,
            // TODO: When we delete legacy mode, we should make this error argument
            // required — every concurrent mode path that causes hydration to
            // de-opt to client rendering should have an error message.
            null
          );
        if (WC(d)) {
          var x, T, O;
          {
            var N = _k(d);
            x = N.digest, T = N.message, O = N.stack;
          }
          var j;
          T ? j = new Error(T) : j = new Error("The server could not finish this Suspense boundary, likely due to an error during server rendering. Switched to client rendering.");
          var G = pS(j, x, O);
          return m0(e, t, y, G);
        }
        var te = Xr(y, e.childLanes);
        if (to || te) {
          var re = I0();
          if (re !== null) {
            var ie = _g(re, y);
            if (ie !== kn && ie !== m.retryLane) {
              m.retryLane = ie;
              var Ne = tn;
              Va(e, ie), dr(re, e, ie, Ne);
            }
          }
          YS();
          var et = pS(new Error("This Suspense boundary received an update before it finished hydrating. This caused the boundary to switch to client rendering. The usual way to fix this is to wrap the original update in startTransition."));
          return m0(e, t, y, et);
        } else if (jw(d)) {
          t.flags |= dt, t.child = e.child;
          var Qe = s4.bind(null, e);
          return jk(d, Qe), null;
        } else {
          AM(t, d, m.treeContext);
          var Nt = l.children, It = RS(t, Nt);
          return It.flags |= Ta, It;
        }
      }
    }
    function SR(e, t, o) {
      e.lanes = Ct(e.lanes, t);
      var l = e.alternate;
      l !== null && (l.lanes = Ct(l.lanes, t)), g1(e.return, t, o);
    }
    function HL(e, t, o) {
      for (var l = t; l !== null; ) {
        if (l.tag === V) {
          var d = l.memoizedState;
          d !== null && SR(l, o, e);
        } else if (l.tag === he)
          SR(l, o, e);
        else if (l.child !== null) {
          l.child.return = l, l = l.child;
          continue;
        }
        if (l === e)
          return;
        for (; l.sibling === null; ) {
          if (l.return === null || l.return === e)
            return;
          l = l.return;
        }
        l.sibling.return = l.return, l = l.sibling;
      }
    }
    function zL(e) {
      for (var t = e, o = null; t !== null; ) {
        var l = t.alternate;
        l !== null && Yy(l) === null && (o = t), t = t.sibling;
      }
      return o;
    }
    function GL(e) {
      if (e !== void 0 && e !== "forwards" && e !== "backwards" && e !== "together" && !CS[e])
        if (CS[e] = !0, typeof e == "string")
          switch (e.toLowerCase()) {
            case "together":
            case "forwards":
            case "backwards": {
              h('"%s" is not a valid value for revealOrder on <SuspenseList />. Use lowercase "%s" instead.', e, e.toLowerCase());
              break;
            }
            case "forward":
            case "backward": {
              h('"%s" is not a valid value for revealOrder on <SuspenseList />. React uses the -s suffix in the spelling. Use "%ss" instead.', e, e.toLowerCase());
              break;
            }
            default:
              h('"%s" is not a supported revealOrder on <SuspenseList />. Did you mean "together", "forwards" or "backwards"?', e);
              break;
          }
        else
          h('%s is not a supported value for revealOrder on <SuspenseList />. Did you mean "together", "forwards" or "backwards"?', e);
    }
    function _L(e, t) {
      e !== void 0 && !h0[e] && (e !== "collapsed" && e !== "hidden" ? (h0[e] = !0, h('"%s" is not a supported value for tail on <SuspenseList />. Did you mean "collapsed" or "hidden"?', e)) : t !== "forwards" && t !== "backwards" && (h0[e] = !0, h('<SuspenseList tail="%s" /> is only valid if revealOrder is "forwards" or "backwards". Did you mean to specify revealOrder="forwards"?', e)));
    }
    function ER(e, t) {
      {
        var o = Wn(e), l = !o && typeof Mi(e) == "function";
        if (o || l) {
          var d = o ? "array" : "iterable";
          return h("A nested %s was passed to row #%s in <SuspenseList />. Wrap it in an additional SuspenseList to configure its revealOrder: <SuspenseList revealOrder=...> ... <SuspenseList revealOrder=...>{%s}</SuspenseList> ... </SuspenseList>", d, t, d), !1;
        }
      }
      return !0;
    }
    function jL(e, t) {
      if ((t === "forwards" || t === "backwards") && e !== void 0 && e !== null && e !== !1)
        if (Wn(e)) {
          for (var o = 0; o < e.length; o++)
            if (!ER(e[o], o))
              return;
        } else {
          var l = Mi(e);
          if (typeof l == "function") {
            var d = l.call(e);
            if (d)
              for (var m = d.next(), y = 0; !m.done; m = d.next()) {
                if (!ER(m.value, y))
                  return;
                y++;
              }
          } else
            h('A single row was passed to a <SuspenseList revealOrder="%s" />. This is not useful since it needs multiple rows. Did you mean to pass multiple children or an array?', t);
        }
    }
    function AS(e, t, o, l, d) {
      var m = e.memoizedState;
      m === null ? e.memoizedState = {
        isBackwards: t,
        rendering: null,
        renderingStartTime: 0,
        last: l,
        tail: o,
        tailMode: d
      } : (m.isBackwards = t, m.rendering = null, m.renderingStartTime = 0, m.last = l, m.tail = o, m.tailMode = d);
    }
    function xR(e, t, o) {
      var l = t.pendingProps, d = l.revealOrder, m = l.tail, y = l.children;
      GL(d), _L(m, d), jL(y, d), ca(e, t, y, o);
      var x = Ki.current, T = R1(x, lh);
      if (T)
        x = I1(x, lh), t.flags |= dt;
      else {
        var O = e !== null && (e.flags & dt) !== qe;
        O && HL(t, t.child, o), x = Nd(x);
      }
      if (Xl(t, x), (t.mode & $e) === Xe)
        t.memoizedState = null;
      else
        switch (d) {
          case "forwards": {
            var N = zL(t.child), j;
            N === null ? (j = t.child, t.child = null) : (j = N.sibling, N.sibling = null), AS(
              t,
              !1,
              // isBackwards
              j,
              N,
              m
            );
            break;
          }
          case "backwards": {
            var G = null, te = t.child;
            for (t.child = null; te !== null; ) {
              var re = te.alternate;
              if (re !== null && Yy(re) === null) {
                t.child = te;
                break;
              }
              var ie = te.sibling;
              te.sibling = G, G = te, te = ie;
            }
            AS(
              t,
              !0,
              // isBackwards
              G,
              null,
              // last
              m
            );
            break;
          }
          case "together": {
            AS(
              t,
              !1,
              // isBackwards
              null,
              // tail
              null,
              // last
              void 0
            );
            break;
          }
          default:
            t.memoizedState = null;
        }
      return t.child;
    }
    function UL(e, t, o) {
      x1(t, t.stateNode.containerInfo);
      var l = t.pendingProps;
      return e === null ? t.child = Id(t, null, l, o) : ca(e, t, l, o), t.child;
    }
    var wR = !1;
    function WL(e, t, o) {
      var l = t.type, d = l._context, m = t.pendingProps, y = t.memoizedProps, x = m.value;
      {
        "value" in m || wR || (wR = !0, h("The `value` prop is required for the `<Context.Provider>`. Did you misspell it or forget to pass it?"));
        var T = t.type.propTypes;
        T && $i(T, m, "prop", "Context.Provider");
      }
      if (mT(t, d, x), y !== null) {
        var O = y.value;
        if (Ge(O, x)) {
          if (y.children === m.children && !Ty())
            return Du(e, t, o);
        } else
          GM(t, d, o);
      }
      var N = m.children;
      return ca(e, t, N, o), t.child;
    }
    var TR = !1;
    function XL(e, t, o) {
      var l = t.type;
      l._context === void 0 ? l !== l.Consumer && (TR || (TR = !0, h("Rendering <Context> directly is not supported and will be removed in a future major release. Did you mean to render <Context.Consumer> instead?"))) : l = l._context;
      var d = t.pendingProps, m = d.children;
      typeof m != "function" && h("A context consumer was rendered with multiple children, or a child that isn't a function. A context consumer expects a single child that is a function. If you did pass a function, make sure there is no trailing or leading whitespace around it."), Od(t, o);
      var y = Jn(l);
      fl(t);
      var x;
      return gh.current = t, qa(!0), x = m(y), qa(!1), jr(), t.flags |= bo, ca(e, t, x, o), t.child;
    }
    function Ch() {
      to = !0;
    }
    function g0(e, t) {
      (t.mode & $e) === Xe && e !== null && (e.alternate = null, t.alternate = null, t.flags |= nn);
    }
    function Du(e, t, o) {
      return e !== null && (t.dependencies = e.dependencies), eR(), kh(t.lanes), Xr(o, t.childLanes) ? (HM(e, t), t.child) : null;
    }
    function ZL(e, t, o) {
      {
        var l = t.return;
        if (l === null)
          throw new Error("Cannot swap the root fiber.");
        if (e.alternate = null, t.alternate = null, o.index = t.index, o.sibling = t.sibling, o.return = t.return, o.ref = t.ref, t === l.child)
          l.child = o;
        else {
          var d = l.child;
          if (d === null)
            throw new Error("Expected parent to have a child.");
          for (; d.sibling !== t; )
            if (d = d.sibling, d === null)
              throw new Error("Expected to find the previous sibling.");
          d.sibling = o;
        }
        var m = l.deletions;
        return m === null ? (l.deletions = [e], l.flags |= Pt) : m.push(e), o.flags |= nn, o;
      }
    }
    function OS(e, t) {
      var o = e.lanes;
      return !!Xr(o, t);
    }
    function YL(e, t, o) {
      switch (t.tag) {
        case R:
          gR(t), t.stateNode, Rd();
          break;
        case D:
          wT(t);
          break;
        case E: {
          var l = t.type;
          ko(l) && Iy(t);
          break;
        }
        case A:
          x1(t, t.stateNode.containerInfo);
          break;
        case Y: {
          var d = t.memoizedProps.value, m = t.type._context;
          mT(t, m, d);
          break;
        }
        case Z:
          {
            var y = Xr(o, t.childLanes);
            y && (t.flags |= St);
            {
              var x = t.stateNode;
              x.effectDuration = 0, x.passiveEffectDuration = 0;
            }
          }
          break;
        case V: {
          var T = t.memoizedState;
          if (T !== null) {
            if (T.dehydrated !== null)
              return Xl(t, Nd(Ki.current)), t.flags |= dt, null;
            var O = t.child, N = O.childLanes;
            if (Xr(o, N))
              return bR(e, t, o);
            Xl(t, Nd(Ki.current));
            var j = Du(e, t, o);
            return j !== null ? j.sibling : null;
          } else
            Xl(t, Nd(Ki.current));
          break;
        }
        case he: {
          var G = (e.flags & dt) !== qe, te = Xr(o, t.childLanes);
          if (G) {
            if (te)
              return xR(e, t, o);
            t.flags |= dt;
          }
          var re = t.memoizedState;
          if (re !== null && (re.rendering = null, re.tail = null, re.lastEffect = null), Xl(t, Ki.current), te)
            break;
          return null;
        }
        case ye:
        case Me:
          return t.lanes = fe, vR(e, t, o);
      }
      return Du(e, t, o);
    }
    function RR(e, t, o) {
      if (t._debugNeedsRemount && e !== null)
        return ZL(e, t, iE(t.type, t.key, t.pendingProps, t._debugOwner || null, t.mode, t.lanes));
      if (e !== null) {
        var l = e.memoizedProps, d = t.pendingProps;
        if (l !== d || Ty() || // Force a re-render if the implementation changed due to hot reload:
        t.type !== e.type)
          to = !0;
        else {
          var m = OS(e, o);
          if (!m && // If this is the second pass of an error or suspense boundary, there
          // may not be work scheduled on `current`, so we check for this flag.
          (t.flags & dt) === qe)
            return to = !1, YL(e, t, o);
          (e.flags & zs) !== qe ? to = !0 : to = !1;
        }
      } else if (to = !1, Tr() && bM(t)) {
        var y = t.index, x = CM();
        eT(t, x, y);
      }
      switch (t.lanes = fe, t.tag) {
        case w:
          return OL(e, t, t.type, o);
        case U: {
          var T = t.elementType;
          return IL(e, t, T, o);
        }
        case b: {
          var O = t.type, N = t.pendingProps, j = t.elementType === O ? N : eo(O, N);
          return SS(e, t, O, j, o);
        }
        case E: {
          var G = t.type, te = t.pendingProps, re = t.elementType === G ? te : eo(G, te);
          return mR(e, t, G, re, o);
        }
        case R:
          return wL(e, t, o);
        case D:
          return TL(e, t, o);
        case L:
          return RL(e, t);
        case V:
          return bR(e, t, o);
        case A:
          return UL(e, t, o);
        case z: {
          var ie = t.type, Ne = t.pendingProps, et = t.elementType === ie ? Ne : eo(ie, Ne);
          return fR(e, t, ie, et, o);
        }
        case M:
          return SL(e, t, o);
        case k:
          return EL(e, t, o);
        case Z:
          return xL(e, t, o);
        case Y:
          return WL(e, t, o);
        case B:
          return XL(e, t, o);
        case Q: {
          var Qe = t.type, Nt = t.pendingProps, It = eo(Qe, Nt);
          if (t.type !== t.elementType) {
            var K = Qe.propTypes;
            K && $i(
              K,
              It,
              // Resolved for outer only
              "prop",
              Ut(Qe)
            );
          }
          return It = eo(Qe.type, It), dR(e, t, Qe, It, o);
        }
        case P:
          return pR(e, t, t.type, t.pendingProps, o);
        case se: {
          var oe = t.type, q = t.pendingProps, me = t.elementType === oe ? q : eo(oe, q);
          return AL(e, t, oe, me, o);
        }
        case he:
          return xR(e, t, o);
        case xe:
          break;
        case ye:
          return vR(e, t, o);
      }
      throw new Error("Unknown unit of work tag (" + t.tag + "). This error is likely caused by a bug in React. Please file an issue.");
    }
    function Vd(e) {
      e.flags |= St;
    }
    function IR(e) {
      e.flags |= Pr, e.flags |= ov;
    }
    var AR, DS, OR, DR;
    AR = function(e, t, o, l) {
      for (var d = t.child; d !== null; ) {
        if (d.tag === D || d.tag === L)
          mk(e, d.stateNode);
        else if (d.tag !== A) {
          if (d.child !== null) {
            d.child.return = d, d = d.child;
            continue;
          }
        }
        if (d === t)
          return;
        for (; d.sibling === null; ) {
          if (d.return === null || d.return === t)
            return;
          d = d.return;
        }
        d.sibling.return = d.return, d = d.sibling;
      }
    }, DS = function(e, t) {
    }, OR = function(e, t, o, l, d) {
      var m = e.memoizedProps;
      if (m !== l) {
        var y = t.stateNode, x = w1(), T = yk(y, o, m, l, d, x);
        t.updateQueue = T, T && Vd(t);
      }
    }, DR = function(e, t, o, l) {
      o !== l && Vd(t);
    };
    function Sh(e, t) {
      if (!Tr())
        switch (e.tailMode) {
          case "hidden": {
            for (var o = e.tail, l = null; o !== null; )
              o.alternate !== null && (l = o), o = o.sibling;
            l === null ? e.tail = null : l.sibling = null;
            break;
          }
          case "collapsed": {
            for (var d = e.tail, m = null; d !== null; )
              d.alternate !== null && (m = d), d = d.sibling;
            m === null ? !t && e.tail !== null ? e.tail.sibling = null : e.tail = null : m.sibling = null;
            break;
          }
        }
    }
    function Ir(e) {
      var t = e.alternate !== null && e.alternate.child === e.child, o = fe, l = qe;
      if (t) {
        if ((e.mode & pt) !== Xe) {
          for (var T = e.selfBaseDuration, O = e.child; O !== null; )
            o = Ct(o, Ct(O.lanes, O.childLanes)), l |= O.subtreeFlags & nr, l |= O.flags & nr, T += O.treeBaseDuration, O = O.sibling;
          e.treeBaseDuration = T;
        } else
          for (var N = e.child; N !== null; )
            o = Ct(o, Ct(N.lanes, N.childLanes)), l |= N.subtreeFlags & nr, l |= N.flags & nr, N.return = e, N = N.sibling;
        e.subtreeFlags |= l;
      } else {
        if ((e.mode & pt) !== Xe) {
          for (var d = e.actualDuration, m = e.selfBaseDuration, y = e.child; y !== null; )
            o = Ct(o, Ct(y.lanes, y.childLanes)), l |= y.subtreeFlags, l |= y.flags, d += y.actualDuration, m += y.treeBaseDuration, y = y.sibling;
          e.actualDuration = d, e.treeBaseDuration = m;
        } else
          for (var x = e.child; x !== null; )
            o = Ct(o, Ct(x.lanes, x.childLanes)), l |= x.subtreeFlags, l |= x.flags, x.return = e, x = x.sibling;
        e.subtreeFlags |= l;
      }
      return e.childLanes = o, t;
    }
    function $L(e, t, o) {
      if (MM() && (t.mode & $e) !== Xe && (t.flags & dt) === qe)
        return uT(t), Rd(), t.flags |= gn | aa | Zn, !1;
      var l = ky(t);
      if (o !== null && o.dehydrated !== null)
        if (e === null) {
          if (!l)
            throw new Error("A dehydrated suspense component was completed without a hydrated node. This is probably a bug in React.");
          if (NM(t), Ir(t), (t.mode & pt) !== Xe) {
            var d = o !== null;
            if (d) {
              var m = t.child;
              m !== null && (t.treeBaseDuration -= m.treeBaseDuration);
            }
          }
          return !1;
        } else {
          if (Rd(), (t.flags & dt) === qe && (t.memoizedState = null), t.flags |= St, Ir(t), (t.mode & pt) !== Xe) {
            var y = o !== null;
            if (y) {
              var x = t.child;
              x !== null && (t.treeBaseDuration -= x.treeBaseDuration);
            }
          }
          return !1;
        }
      else
        return lT(), !0;
    }
    function NR(e, t, o) {
      var l = t.pendingProps;
      switch (n1(t), t.tag) {
        case w:
        case U:
        case P:
        case b:
        case z:
        case M:
        case k:
        case Z:
        case B:
        case Q:
          return Ir(t), null;
        case E: {
          var d = t.type;
          return ko(d) && Ry(t), Ir(t), null;
        }
        case R: {
          var m = t.stateNode;
          if (Dd(t), KC(t), O1(), m.pendingContext && (m.context = m.pendingContext, m.pendingContext = null), e === null || e.child === null) {
            var y = ky(t);
            if (y)
              Vd(t);
            else if (e !== null) {
              var x = e.memoizedState;
              // Check if this is a client root
              (!x.isDehydrated || // Check if we reverted to client rendering (e.g. due to an error)
              (t.flags & gn) !== qe) && (t.flags |= wa, lT());
            }
          }
          return DS(e, t), Ir(t), null;
        }
        case D: {
          T1(t);
          var T = xT(), O = t.type;
          if (e !== null && t.stateNode != null)
            OR(e, t, O, l, T), e.ref !== t.ref && IR(t);
          else {
            if (!l) {
              if (t.stateNode === null)
                throw new Error("We must have new props for new mounts. This error is likely caused by a bug in React. Please file an issue.");
              return Ir(t), null;
            }
            var N = w1(), j = ky(t);
            if (j)
              OM(t, T, N) && Vd(t);
            else {
              var G = hk(O, l, T, N, t);
              AR(G, t, !1, !1), t.stateNode = G, gk(G, O, l, T) && Vd(t);
            }
            t.ref !== null && IR(t);
          }
          return Ir(t), null;
        }
        case L: {
          var te = l;
          if (e && t.stateNode != null) {
            var re = e.memoizedProps;
            DR(e, t, re, te);
          } else {
            if (typeof te != "string" && t.stateNode === null)
              throw new Error("We must have new props for new mounts. This error is likely caused by a bug in React. Please file an issue.");
            var ie = xT(), Ne = w1(), et = ky(t);
            et ? DM(t) && Vd(t) : t.stateNode = bk(te, ie, Ne, t);
          }
          return Ir(t), null;
        }
        case V: {
          kd(t);
          var Qe = t.memoizedState;
          if (e === null || e.memoizedState !== null && e.memoizedState.dehydrated !== null) {
            var Nt = $L(e, t, Qe);
            if (!Nt)
              return t.flags & Zn ? t : null;
          }
          if ((t.flags & dt) !== qe)
            return t.lanes = o, (t.mode & pt) !== Xe && eS(t), t;
          var It = Qe !== null, K = e !== null && e.memoizedState !== null;
          if (It !== K && It) {
            var oe = t.child;
            if (oe.flags |= Co, (t.mode & $e) !== Xe) {
              var q = e === null && (t.memoizedProps.unstable_avoidThisFallback !== !0 || !ce);
              q || R1(Ki.current, RT) ? QF() : YS();
            }
          }
          var me = t.updateQueue;
          if (me !== null && (t.flags |= St), Ir(t), (t.mode & pt) !== Xe && It) {
            var Ve = t.child;
            Ve !== null && (t.treeBaseDuration -= Ve.treeBaseDuration);
          }
          return null;
        }
        case A:
          return Dd(t), DS(e, t), e === null && dM(t.stateNode.containerInfo), Ir(t), null;
        case Y:
          var Le = t.type._context;
          return m1(Le, t), Ir(t), null;
        case se: {
          var ct = t.type;
          return ko(ct) && Ry(t), Ir(t), null;
        }
        case he: {
          kd(t);
          var mt = t.memoizedState;
          if (mt === null)
            return Ir(t), null;
          var $t = (t.flags & dt) !== qe, Bt = mt.rendering;
          if (Bt === null)
            if ($t)
              Sh(mt, !1);
            else {
              var jn = KF() && (e === null || (e.flags & dt) === qe);
              if (!jn)
                for (var Vt = t.child; Vt !== null; ) {
                  var Mn = Yy(Vt);
                  if (Mn !== null) {
                    $t = !0, t.flags |= dt, Sh(mt, !1);
                    var Jr = Mn.updateQueue;
                    return Jr !== null && (t.updateQueue = Jr, t.flags |= St), t.subtreeFlags = qe, zM(t, o), Xl(t, I1(Ki.current, lh)), t.child;
                  }
                  Vt = Vt.sibling;
                }
              mt.tail !== null && cn() > KR() && (t.flags |= dt, $t = !0, Sh(mt, !1), t.lanes = Ng);
            }
          else {
            if (!$t) {
              var kr = Yy(Bt);
              if (kr !== null) {
                t.flags |= dt, $t = !0;
                var oi = kr.updateQueue;
                if (oi !== null && (t.updateQueue = oi, t.flags |= St), Sh(mt, !0), mt.tail === null && mt.tailMode === "hidden" && !Bt.alternate && !Tr())
                  return Ir(t), null;
              } else
                // The time it took to render last row is greater than the remaining
                // time we have to render. So rendering one more row would likely
                // exceed it.
                cn() * 2 - mt.renderingStartTime > KR() && o !== gr && (t.flags |= dt, $t = !0, Sh(mt, !1), t.lanes = Ng);
            }
            if (mt.isBackwards)
              Bt.sibling = t.child, t.child = Bt;
            else {
              var pa = mt.last;
              pa !== null ? pa.sibling = Bt : t.child = Bt, mt.last = Bt;
            }
          }
          if (mt.tail !== null) {
            var va = mt.tail;
            mt.rendering = va, mt.tail = va.sibling, mt.renderingStartTime = cn(), va.sibling = null;
            var Kr = Ki.current;
            return $t ? Kr = I1(Kr, lh) : Kr = Nd(Kr), Xl(t, Kr), va;
          }
          return Ir(t), null;
        }
        case xe:
          break;
        case ye:
        case Me: {
          ZS(t);
          var Fu = t.memoizedState, Wd = Fu !== null;
          if (e !== null) {
            var Vh = e.memoizedState, zo = Vh !== null;
            zo !== Wd && // LegacyHidden doesn't do any hiding — it only pre-renders.
            !W && (t.flags |= Co);
          }
          return !Wd || (t.mode & $e) === Xe ? Ir(t) : Xr(Ho, gr) && (Ir(t), t.subtreeFlags & (nn | St) && (t.flags |= Co)), null;
        }
        case We:
          return null;
        case Ee:
          return null;
      }
      throw new Error("Unknown unit of work tag (" + t.tag + "). This error is likely caused by a bug in React. Please file an issue.");
    }
    function QL(e, t, o) {
      switch (n1(t), t.tag) {
        case E: {
          var l = t.type;
          ko(l) && Ry(t);
          var d = t.flags;
          return d & Zn ? (t.flags = d & ~Zn | dt, (t.mode & pt) !== Xe && eS(t), t) : null;
        }
        case R: {
          t.stateNode, Dd(t), KC(t), O1();
          var m = t.flags;
          return (m & Zn) !== qe && (m & dt) === qe ? (t.flags = m & ~Zn | dt, t) : null;
        }
        case D:
          return T1(t), null;
        case V: {
          kd(t);
          var y = t.memoizedState;
          if (y !== null && y.dehydrated !== null) {
            if (t.alternate === null)
              throw new Error("Threw in newly mounted dehydrated component. This is likely a bug in React. Please file an issue.");
            Rd();
          }
          var x = t.flags;
          return x & Zn ? (t.flags = x & ~Zn | dt, (t.mode & pt) !== Xe && eS(t), t) : null;
        }
        case he:
          return kd(t), null;
        case A:
          return Dd(t), null;
        case Y:
          var T = t.type._context;
          return m1(T, t), null;
        case ye:
        case Me:
          return ZS(t), null;
        case We:
          return null;
        default:
          return null;
      }
    }
    function kR(e, t, o) {
      switch (n1(t), t.tag) {
        case E: {
          var l = t.type.childContextTypes;
          l != null && Ry(t);
          break;
        }
        case R: {
          t.stateNode, Dd(t), KC(t), O1();
          break;
        }
        case D: {
          T1(t);
          break;
        }
        case A:
          Dd(t);
          break;
        case V:
          kd(t);
          break;
        case he:
          kd(t);
          break;
        case Y:
          var d = t.type._context;
          m1(d, t);
          break;
        case ye:
        case Me:
          ZS(t);
          break;
      }
    }
    var MR = null;
    MR = /* @__PURE__ */ new Set();
    var y0 = !1, Ar = !1, JL = typeof WeakSet == "function" ? WeakSet : Set, _e = null, Pd = null, Hd = null;
    function KL(e) {
      ou(null, function() {
        throw e;
      }), av();
    }
    var qL = function(e, t) {
      if (t.props = e.memoizedProps, t.state = e.memoizedState, e.mode & pt)
        try {
          Vo(), t.componentWillUnmount();
        } finally {
          Bo(e);
        }
      else
        t.componentWillUnmount();
    };
    function LR(e, t) {
      try {
        $l(or, e);
      } catch (o) {
        on(e, t, o);
      }
    }
    function NS(e, t, o) {
      try {
        qL(e, o);
      } catch (l) {
        on(e, t, l);
      }
    }
    function eF(e, t, o) {
      try {
        o.componentDidMount();
      } catch (l) {
        on(e, t, l);
      }
    }
    function FR(e, t) {
      try {
        VR(e);
      } catch (o) {
        on(e, t, o);
      }
    }
    function zd(e, t) {
      var o = e.ref;
      if (o !== null)
        if (typeof o == "function") {
          var l;
          try {
            if (Oe && Ue && e.mode & pt)
              try {
                Vo(), l = o(null);
              } finally {
                Bo(e);
              }
            else
              l = o(null);
          } catch (d) {
            on(e, t, d);
          }
          typeof l == "function" && h("Unexpected return value from a callback ref in %s. A callback ref should not return a function.", bt(e));
        } else
          o.current = null;
    }
    function b0(e, t, o) {
      try {
        o();
      } catch (l) {
        on(e, t, l);
      }
    }
    var BR = !1;
    function tF(e, t) {
      pk(e.containerInfo), _e = t, nF();
      var o = BR;
      return BR = !1, o;
    }
    function nF() {
      for (; _e !== null; ) {
        var e = _e, t = e.child;
        (e.subtreeFlags & ll) !== qe && t !== null ? (t.return = e, _e = t) : rF();
      }
    }
    function rF() {
      for (; _e !== null; ) {
        var e = _e;
        Zt(e);
        try {
          aF(e);
        } catch (o) {
          on(e, e.return, o);
        }
        Vn();
        var t = e.sibling;
        if (t !== null) {
          t.return = e.return, _e = t;
          return;
        }
        _e = e.return;
      }
    }
    function aF(e) {
      var t = e.alternate, o = e.flags;
      if ((o & wa) !== qe) {
        switch (Zt(e), e.tag) {
          case b:
          case z:
          case P:
            break;
          case E: {
            if (t !== null) {
              var l = t.memoizedProps, d = t.memoizedState, m = e.stateNode;
              e.type === e.elementType && !Sc && (m.props !== e.memoizedProps && h("Expected %s props to match memoized props before getSnapshotBeforeUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.", bt(e) || "instance"), m.state !== e.memoizedState && h("Expected %s state to match memoized state before getSnapshotBeforeUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.", bt(e) || "instance"));
              var y = m.getSnapshotBeforeUpdate(e.elementType === e.type ? l : eo(e.type, l), d);
              {
                var x = MR;
                y === void 0 && !x.has(e.type) && (x.add(e.type), h("%s.getSnapshotBeforeUpdate(): A snapshot value (or null) must be returned. You have returned undefined.", bt(e)));
              }
              m.__reactInternalSnapshotBeforeUpdate = y;
            }
            break;
          }
          case R: {
            {
              var T = e.stateNode;
              Pk(T.containerInfo);
            }
            break;
          }
          case D:
          case L:
          case A:
          case se:
            break;
          default:
            throw new Error("This unit of work tag should not have side-effects. This error is likely caused by a bug in React. Please file an issue.");
        }
        Vn();
      }
    }
    function no(e, t, o) {
      var l = t.updateQueue, d = l !== null ? l.lastEffect : null;
      if (d !== null) {
        var m = d.next, y = m;
        do {
          if ((y.tag & e) === e) {
            var x = y.destroy;
            y.destroy = void 0, x !== void 0 && ((e & Rr) !== Pa ? Ag(t) : (e & or) !== Pa && ri(t), (e & Mo) !== Pa && Lh(!0), b0(t, o, x), (e & Mo) !== Pa && Lh(!1), (e & Rr) !== Pa ? Rf() : (e & or) !== Pa && dl());
          }
          y = y.next;
        } while (y !== m);
      }
    }
    function $l(e, t) {
      var o = t.updateQueue, l = o !== null ? o.lastEffect : null;
      if (l !== null) {
        var d = l.next, m = d;
        do {
          if ((m.tag & e) === e) {
            (e & Rr) !== Pa ? xo(t) : (e & or) !== Pa && Og(t);
            var y = m.create;
            (e & Mo) !== Pa && Lh(!0), m.destroy = y(), (e & Mo) !== Pa && Lh(!1), (e & Rr) !== Pa ? Tf() : (e & or) !== Pa && Gs();
            {
              var x = m.destroy;
              if (x !== void 0 && typeof x != "function") {
                var T = void 0;
                (m.tag & or) !== qe ? T = "useLayoutEffect" : (m.tag & Mo) !== qe ? T = "useInsertionEffect" : T = "useEffect";
                var O = void 0;
                x === null ? O = " You returned null. If your effect does not require clean up, return undefined (or nothing)." : typeof x.then == "function" ? O = `

It looks like you wrote ` + T + `(async () => ...) or returned a Promise. Instead, write the async function inside your effect and call it immediately:

` + T + `(() => {
  async function fetchData() {
    // You can await here
    const response = await MyAPI.getData(someId);
    // ...
  }
  fetchData();
}, [someId]); // Or [] if effect doesn't need props or state

Learn more about data fetching with Hooks: https://reactjs.org/link/hooks-data-fetching` : O = " You returned: " + x, h("%s must not return anything besides a function, which is used for clean-up.%s", T, O);
              }
            }
          }
          m = m.next;
        } while (m !== d);
      }
    }
    function iF(e, t) {
      if ((t.flags & St) !== qe)
        switch (t.tag) {
          case Z: {
            var o = t.stateNode.passiveEffectDuration, l = t.memoizedProps, d = l.id, m = l.onPostCommit, y = KT(), x = t.alternate === null ? "mount" : "update";
            JT() && (x = "nested-update"), typeof m == "function" && m(d, x, o, y);
            var T = t.return;
            e:
              for (; T !== null; ) {
                switch (T.tag) {
                  case R:
                    var O = T.stateNode;
                    O.passiveEffectDuration += o;
                    break e;
                  case Z:
                    var N = T.stateNode;
                    N.passiveEffectDuration += o;
                    break e;
                }
                T = T.return;
              }
            break;
          }
        }
    }
    function oF(e, t, o, l) {
      if ((o.flags & hr) !== qe)
        switch (o.tag) {
          case b:
          case z:
          case P: {
            if (!Ar)
              if (o.mode & pt)
                try {
                  Vo(), $l(or | ir, o);
                } finally {
                  Bo(o);
                }
              else
                $l(or | ir, o);
            break;
          }
          case E: {
            var d = o.stateNode;
            if (o.flags & St && !Ar)
              if (t === null)
                if (o.type === o.elementType && !Sc && (d.props !== o.memoizedProps && h("Expected %s props to match memoized props before componentDidMount. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.", bt(o) || "instance"), d.state !== o.memoizedState && h("Expected %s state to match memoized state before componentDidMount. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.", bt(o) || "instance")), o.mode & pt)
                  try {
                    Vo(), d.componentDidMount();
                  } finally {
                    Bo(o);
                  }
                else
                  d.componentDidMount();
              else {
                var m = o.elementType === o.type ? t.memoizedProps : eo(o.type, t.memoizedProps), y = t.memoizedState;
                if (o.type === o.elementType && !Sc && (d.props !== o.memoizedProps && h("Expected %s props to match memoized props before componentDidUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.", bt(o) || "instance"), d.state !== o.memoizedState && h("Expected %s state to match memoized state before componentDidUpdate. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.", bt(o) || "instance")), o.mode & pt)
                  try {
                    Vo(), d.componentDidUpdate(m, y, d.__reactInternalSnapshotBeforeUpdate);
                  } finally {
                    Bo(o);
                  }
                else
                  d.componentDidUpdate(m, y, d.__reactInternalSnapshotBeforeUpdate);
              }
            var x = o.updateQueue;
            x !== null && (o.type === o.elementType && !Sc && (d.props !== o.memoizedProps && h("Expected %s props to match memoized props before processing the update queue. This might either be because of a bug in React, or because a component reassigns its own `this.props`. Please file an issue.", bt(o) || "instance"), d.state !== o.memoizedState && h("Expected %s state to match memoized state before processing the update queue. This might either be because of a bug in React, or because a component reassigns its own `this.state`. Please file an issue.", bt(o) || "instance")), ET(o, x, d));
            break;
          }
          case R: {
            var T = o.updateQueue;
            if (T !== null) {
              var O = null;
              if (o.child !== null)
                switch (o.child.tag) {
                  case D:
                    O = o.child.stateNode;
                    break;
                  case E:
                    O = o.child.stateNode;
                    break;
                }
              ET(o, T, O);
            }
            break;
          }
          case D: {
            var N = o.stateNode;
            if (t === null && o.flags & St) {
              var j = o.type, G = o.memoizedProps;
              wk(N, j, G);
            }
            break;
          }
          case L:
            break;
          case A:
            break;
          case Z: {
            {
              var te = o.memoizedProps, re = te.onCommit, ie = te.onRender, Ne = o.stateNode.effectDuration, et = KT(), Qe = t === null ? "mount" : "update";
              JT() && (Qe = "nested-update"), typeof ie == "function" && ie(o.memoizedProps.id, Qe, o.actualDuration, o.treeBaseDuration, o.actualStartTime, et);
              {
                typeof re == "function" && re(o.memoizedProps.id, Qe, Ne, et), r4(o);
                var Nt = o.return;
                e:
                  for (; Nt !== null; ) {
                    switch (Nt.tag) {
                      case R:
                        var It = Nt.stateNode;
                        It.effectDuration += Ne;
                        break e;
                      case Z:
                        var K = Nt.stateNode;
                        K.effectDuration += Ne;
                        break e;
                    }
                    Nt = Nt.return;
                  }
              }
            }
            break;
          }
          case V: {
            vF(e, o);
            break;
          }
          case he:
          case se:
          case xe:
          case ye:
          case Me:
          case Ee:
            break;
          default:
            throw new Error("This unit of work tag should not have side-effects. This error is likely caused by a bug in React. Please file an issue.");
        }
      Ar || o.flags & Pr && VR(o);
    }
    function uF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          if (e.mode & pt)
            try {
              Vo(), LR(e, e.return);
            } finally {
              Bo(e);
            }
          else
            LR(e, e.return);
          break;
        }
        case E: {
          var t = e.stateNode;
          typeof t.componentDidMount == "function" && eF(e, e.return, t), FR(e, e.return);
          break;
        }
        case D: {
          FR(e, e.return);
          break;
        }
      }
    }
    function lF(e, t) {
      for (var o = null, l = e; ; ) {
        if (l.tag === D) {
          if (o === null) {
            o = l;
            try {
              var d = l.stateNode;
              t ? Lk(d) : Bk(l.stateNode, l.memoizedProps);
            } catch (y) {
              on(e, e.return, y);
            }
          }
        } else if (l.tag === L) {
          if (o === null)
            try {
              var m = l.stateNode;
              t ? Fk(m) : Vk(m, l.memoizedProps);
            } catch (y) {
              on(e, e.return, y);
            }
        } else if (!((l.tag === ye || l.tag === Me) && l.memoizedState !== null && l !== e)) {
          if (l.child !== null) {
            l.child.return = l, l = l.child;
            continue;
          }
        }
        if (l === e)
          return;
        for (; l.sibling === null; ) {
          if (l.return === null || l.return === e)
            return;
          o === l && (o = null), l = l.return;
        }
        o === l && (o = null), l.sibling.return = l.return, l = l.sibling;
      }
    }
    function VR(e) {
      var t = e.ref;
      if (t !== null) {
        var o = e.stateNode, l;
        switch (e.tag) {
          case D:
            l = o;
            break;
          default:
            l = o;
        }
        if (typeof t == "function") {
          var d;
          if (e.mode & pt)
            try {
              Vo(), d = t(l);
            } finally {
              Bo(e);
            }
          else
            d = t(l);
          typeof d == "function" && h("Unexpected return value from a callback ref in %s. A callback ref should not return a function.", bt(e));
        } else
          t.hasOwnProperty("current") || h("Unexpected ref object provided for %s. Use either a ref-setter function or React.createRef().", bt(e)), t.current = l;
      }
    }
    function sF(e) {
      var t = e.alternate;
      t !== null && (t.return = null), e.return = null;
    }
    function PR(e) {
      var t = e.alternate;
      t !== null && (e.alternate = null, PR(t));
      {
        if (e.child = null, e.deletions = null, e.sibling = null, e.tag === D) {
          var o = e.stateNode;
          o !== null && hM(o);
        }
        e.stateNode = null, e._debugOwner = null, e.return = null, e.dependencies = null, e.memoizedProps = null, e.memoizedState = null, e.pendingProps = null, e.stateNode = null, e.updateQueue = null;
      }
    }
    function cF(e) {
      for (var t = e.return; t !== null; ) {
        if (HR(t))
          return t;
        t = t.return;
      }
      throw new Error("Expected to find a host parent. This error is likely caused by a bug in React. Please file an issue.");
    }
    function HR(e) {
      return e.tag === D || e.tag === R || e.tag === A;
    }
    function zR(e) {
      var t = e;
      e:
        for (; ; ) {
          for (; t.sibling === null; ) {
            if (t.return === null || HR(t.return))
              return null;
            t = t.return;
          }
          for (t.sibling.return = t.return, t = t.sibling; t.tag !== D && t.tag !== L && t.tag !== Se; ) {
            if (t.flags & nn || t.child === null || t.tag === A)
              continue e;
            t.child.return = t, t = t.child;
          }
          if (!(t.flags & nn))
            return t.stateNode;
        }
    }
    function fF(e) {
      var t = cF(e);
      switch (t.tag) {
        case D: {
          var o = t.stateNode;
          t.flags & Ht && (_w(o), t.flags &= ~Ht);
          var l = zR(e);
          MS(e, l, o);
          break;
        }
        case R:
        case A: {
          var d = t.stateNode.containerInfo, m = zR(e);
          kS(e, m, d);
          break;
        }
        default:
          throw new Error("Invalid host parent fiber. This error is likely caused by a bug in React. Please file an issue.");
      }
    }
    function kS(e, t, o) {
      var l = e.tag, d = l === D || l === L;
      if (d) {
        var m = e.stateNode;
        t ? Dk(o, m, t) : Ak(o, m);
      } else if (l !== A) {
        var y = e.child;
        if (y !== null) {
          kS(y, t, o);
          for (var x = y.sibling; x !== null; )
            kS(x, t, o), x = x.sibling;
        }
      }
    }
    function MS(e, t, o) {
      var l = e.tag, d = l === D || l === L;
      if (d) {
        var m = e.stateNode;
        t ? Ok(o, m, t) : Ik(o, m);
      } else if (l !== A) {
        var y = e.child;
        if (y !== null) {
          MS(y, t, o);
          for (var x = y.sibling; x !== null; )
            MS(x, t, o), x = x.sibling;
        }
      }
    }
    var Or = null, ro = !1;
    function dF(e, t, o) {
      {
        var l = t;
        e:
          for (; l !== null; ) {
            switch (l.tag) {
              case D: {
                Or = l.stateNode, ro = !1;
                break e;
              }
              case R: {
                Or = l.stateNode.containerInfo, ro = !0;
                break e;
              }
              case A: {
                Or = l.stateNode.containerInfo, ro = !0;
                break e;
              }
            }
            l = l.return;
          }
        if (Or === null)
          throw new Error("Expected to find a host parent. This error is likely caused by a bug in React. Please file an issue.");
        GR(e, t, o), Or = null, ro = !1;
      }
      sF(o);
    }
    function Ql(e, t, o) {
      for (var l = o.child; l !== null; )
        GR(e, t, l), l = l.sibling;
    }
    function GR(e, t, o) {
      switch (su(o), o.tag) {
        case D:
          Ar || zd(o, t);
        case L: {
          {
            var l = Or, d = ro;
            Or = null, Ql(e, t, o), Or = l, ro = d, Or !== null && (ro ? kk(Or, o.stateNode) : Nk(Or, o.stateNode));
          }
          return;
        }
        case Se: {
          Or !== null && (ro ? Mk(Or, o.stateNode) : UC(Or, o.stateNode));
          return;
        }
        case A: {
          {
            var m = Or, y = ro;
            Or = o.stateNode.containerInfo, ro = !0, Ql(e, t, o), Or = m, ro = y;
          }
          return;
        }
        case b:
        case z:
        case Q:
        case P: {
          if (!Ar) {
            var x = o.updateQueue;
            if (x !== null) {
              var T = x.lastEffect;
              if (T !== null) {
                var O = T.next, N = O;
                do {
                  var j = N, G = j.destroy, te = j.tag;
                  G !== void 0 && ((te & Mo) !== Pa ? b0(o, t, G) : (te & or) !== Pa && (ri(o), o.mode & pt ? (Vo(), b0(o, t, G), Bo(o)) : b0(o, t, G), dl())), N = N.next;
                } while (N !== O);
              }
            }
          }
          Ql(e, t, o);
          return;
        }
        case E: {
          if (!Ar) {
            zd(o, t);
            var re = o.stateNode;
            typeof re.componentWillUnmount == "function" && NS(o, t, re);
          }
          Ql(e, t, o);
          return;
        }
        case xe: {
          Ql(e, t, o);
          return;
        }
        case ye: {
          if (
            // TODO: Remove this dead flag
            o.mode & $e
          ) {
            var ie = Ar;
            Ar = ie || o.memoizedState !== null, Ql(e, t, o), Ar = ie;
          } else
            Ql(e, t, o);
          break;
        }
        default: {
          Ql(e, t, o);
          return;
        }
      }
    }
    function pF(e) {
      e.memoizedState;
    }
    function vF(e, t) {
      var o = t.memoizedState;
      if (o === null) {
        var l = t.alternate;
        if (l !== null) {
          var d = l.memoizedState;
          if (d !== null) {
            var m = d.dehydrated;
            m !== null && Kk(m);
          }
        }
      }
    }
    function _R(e) {
      var t = e.updateQueue;
      if (t !== null) {
        e.updateQueue = null;
        var o = e.stateNode;
        o === null && (o = e.stateNode = new JL()), t.forEach(function(l) {
          var d = c4.bind(null, e, l);
          if (!o.has(l)) {
            if (o.add(l), ua)
              if (Pd !== null && Hd !== null)
                Mh(Hd, Pd);
              else
                throw Error("Expected finished root and lanes to be set. This is a bug in React.");
            l.then(d, d);
          }
        });
      }
    }
    function hF(e, t, o) {
      Pd = o, Hd = e, Zt(t), jR(t, e), Zt(t), Pd = null, Hd = null;
    }
    function ao(e, t, o) {
      var l = t.deletions;
      if (l !== null)
        for (var d = 0; d < l.length; d++) {
          var m = l[d];
          try {
            dF(e, t, m);
          } catch (T) {
            on(m, t, T);
          }
        }
      var y = Wb();
      if (t.subtreeFlags & Gr)
        for (var x = t.child; x !== null; )
          Zt(x), jR(x, e), x = x.sibling;
      Zt(y);
    }
    function jR(e, t, o) {
      var l = e.alternate, d = e.flags;
      switch (e.tag) {
        case b:
        case z:
        case Q:
        case P: {
          if (ao(t, e), Po(e), d & St) {
            try {
              no(Mo | ir, e, e.return), $l(Mo | ir, e);
            } catch (ct) {
              on(e, e.return, ct);
            }
            if (e.mode & pt) {
              try {
                Vo(), no(or | ir, e, e.return);
              } catch (ct) {
                on(e, e.return, ct);
              }
              Bo(e);
            } else
              try {
                no(or | ir, e, e.return);
              } catch (ct) {
                on(e, e.return, ct);
              }
          }
          return;
        }
        case E: {
          ao(t, e), Po(e), d & Pr && l !== null && zd(l, l.return);
          return;
        }
        case D: {
          ao(t, e), Po(e), d & Pr && l !== null && zd(l, l.return);
          {
            if (e.flags & Ht) {
              var m = e.stateNode;
              try {
                _w(m);
              } catch (ct) {
                on(e, e.return, ct);
              }
            }
            if (d & St) {
              var y = e.stateNode;
              if (y != null) {
                var x = e.memoizedProps, T = l !== null ? l.memoizedProps : x, O = e.type, N = e.updateQueue;
                if (e.updateQueue = null, N !== null)
                  try {
                    Tk(y, N, O, T, x, e);
                  } catch (ct) {
                    on(e, e.return, ct);
                  }
              }
            }
          }
          return;
        }
        case L: {
          if (ao(t, e), Po(e), d & St) {
            if (e.stateNode === null)
              throw new Error("This should have a text node initialized. This error is likely caused by a bug in React. Please file an issue.");
            var j = e.stateNode, G = e.memoizedProps, te = l !== null ? l.memoizedProps : G;
            try {
              Rk(j, te, G);
            } catch (ct) {
              on(e, e.return, ct);
            }
          }
          return;
        }
        case R: {
          if (ao(t, e), Po(e), d & St && l !== null) {
            var re = l.memoizedState;
            if (re.isDehydrated)
              try {
                Jk(t.containerInfo);
              } catch (ct) {
                on(e, e.return, ct);
              }
          }
          return;
        }
        case A: {
          ao(t, e), Po(e);
          return;
        }
        case V: {
          ao(t, e), Po(e);
          var ie = e.child;
          if (ie.flags & Co) {
            var Ne = ie.stateNode, et = ie.memoizedState, Qe = et !== null;
            if (Ne.isHidden = Qe, Qe) {
              var Nt = ie.alternate !== null && ie.alternate.memoizedState !== null;
              Nt || $F();
            }
          }
          if (d & St) {
            try {
              pF(e);
            } catch (ct) {
              on(e, e.return, ct);
            }
            _R(e);
          }
          return;
        }
        case ye: {
          var It = l !== null && l.memoizedState !== null;
          if (
            // TODO: Remove this dead flag
            e.mode & $e
          ) {
            var K = Ar;
            Ar = K || It, ao(t, e), Ar = K;
          } else
            ao(t, e);
          if (Po(e), d & Co) {
            var oe = e.stateNode, q = e.memoizedState, me = q !== null, Ve = e;
            if (oe.isHidden = me, me && !It && (Ve.mode & $e) !== Xe) {
              _e = Ve;
              for (var Le = Ve.child; Le !== null; )
                _e = Le, gF(Le), Le = Le.sibling;
            }
            lF(Ve, me);
          }
          return;
        }
        case he: {
          ao(t, e), Po(e), d & St && _R(e);
          return;
        }
        case xe:
          return;
        default: {
          ao(t, e), Po(e);
          return;
        }
      }
    }
    function Po(e) {
      var t = e.flags;
      if (t & nn) {
        try {
          fF(e);
        } catch (o) {
          on(e, e.return, o);
        }
        e.flags &= ~nn;
      }
      t & Ta && (e.flags &= ~Ta);
    }
    function mF(e, t, o) {
      Pd = o, Hd = t, _e = e, UR(e, t, o), Pd = null, Hd = null;
    }
    function UR(e, t, o) {
      for (var l = (e.mode & $e) !== Xe; _e !== null; ) {
        var d = _e, m = d.child;
        if (d.tag === ye && l) {
          var y = d.memoizedState !== null, x = y || y0;
          if (x) {
            LS(e, t, o);
            continue;
          } else {
            var T = d.alternate, O = T !== null && T.memoizedState !== null, N = O || Ar, j = y0, G = Ar;
            y0 = x, Ar = N, Ar && !G && (_e = d, yF(d));
            for (var te = m; te !== null; )
              _e = te, UR(
                te,
                // New root; bubble back up to here and stop.
                t,
                o
              ), te = te.sibling;
            _e = d, y0 = j, Ar = G, LS(e, t, o);
            continue;
          }
        }
        (d.subtreeFlags & hr) !== qe && m !== null ? (m.return = d, _e = m) : LS(e, t, o);
      }
    }
    function LS(e, t, o) {
      for (; _e !== null; ) {
        var l = _e;
        if ((l.flags & hr) !== qe) {
          var d = l.alternate;
          Zt(l);
          try {
            oF(t, d, l, o);
          } catch (y) {
            on(l, l.return, y);
          }
          Vn();
        }
        if (l === e) {
          _e = null;
          return;
        }
        var m = l.sibling;
        if (m !== null) {
          m.return = l.return, _e = m;
          return;
        }
        _e = l.return;
      }
    }
    function gF(e) {
      for (; _e !== null; ) {
        var t = _e, o = t.child;
        switch (t.tag) {
          case b:
          case z:
          case Q:
          case P: {
            if (t.mode & pt)
              try {
                Vo(), no(or, t, t.return);
              } finally {
                Bo(t);
              }
            else
              no(or, t, t.return);
            break;
          }
          case E: {
            zd(t, t.return);
            var l = t.stateNode;
            typeof l.componentWillUnmount == "function" && NS(t, t.return, l);
            break;
          }
          case D: {
            zd(t, t.return);
            break;
          }
          case ye: {
            var d = t.memoizedState !== null;
            if (d) {
              WR(e);
              continue;
            }
            break;
          }
        }
        o !== null ? (o.return = t, _e = o) : WR(e);
      }
    }
    function WR(e) {
      for (; _e !== null; ) {
        var t = _e;
        if (t === e) {
          _e = null;
          return;
        }
        var o = t.sibling;
        if (o !== null) {
          o.return = t.return, _e = o;
          return;
        }
        _e = t.return;
      }
    }
    function yF(e) {
      for (; _e !== null; ) {
        var t = _e, o = t.child;
        if (t.tag === ye) {
          var l = t.memoizedState !== null;
          if (l) {
            XR(e);
            continue;
          }
        }
        o !== null ? (o.return = t, _e = o) : XR(e);
      }
    }
    function XR(e) {
      for (; _e !== null; ) {
        var t = _e;
        Zt(t);
        try {
          uF(t);
        } catch (l) {
          on(t, t.return, l);
        }
        if (Vn(), t === e) {
          _e = null;
          return;
        }
        var o = t.sibling;
        if (o !== null) {
          o.return = t.return, _e = o;
          return;
        }
        _e = t.return;
      }
    }
    function bF(e, t, o, l) {
      _e = t, CF(t, e, o, l);
    }
    function CF(e, t, o, l) {
      for (; _e !== null; ) {
        var d = _e, m = d.child;
        (d.subtreeFlags & Ra) !== qe && m !== null ? (m.return = d, _e = m) : SF(e, t, o, l);
      }
    }
    function SF(e, t, o, l) {
      for (; _e !== null; ) {
        var d = _e;
        if ((d.flags & un) !== qe) {
          Zt(d);
          try {
            EF(t, d, o, l);
          } catch (y) {
            on(d, d.return, y);
          }
          Vn();
        }
        if (d === e) {
          _e = null;
          return;
        }
        var m = d.sibling;
        if (m !== null) {
          m.return = d.return, _e = m;
          return;
        }
        _e = d.return;
      }
    }
    function EF(e, t, o, l) {
      switch (t.tag) {
        case b:
        case z:
        case P: {
          if (t.mode & pt) {
            q1();
            try {
              $l(Rr | ir, t);
            } finally {
              K1(t);
            }
          } else
            $l(Rr | ir, t);
          break;
        }
      }
    }
    function xF(e) {
      _e = e, wF();
    }
    function wF() {
      for (; _e !== null; ) {
        var e = _e, t = e.child;
        if ((_e.flags & Pt) !== qe) {
          var o = e.deletions;
          if (o !== null) {
            for (var l = 0; l < o.length; l++) {
              var d = o[l];
              _e = d, IF(d, e);
            }
            {
              var m = e.alternate;
              if (m !== null) {
                var y = m.child;
                if (y !== null) {
                  m.child = null;
                  do {
                    var x = y.sibling;
                    y.sibling = null, y = x;
                  } while (y !== null);
                }
              }
            }
            _e = e;
          }
        }
        (e.subtreeFlags & Ra) !== qe && t !== null ? (t.return = e, _e = t) : TF();
      }
    }
    function TF() {
      for (; _e !== null; ) {
        var e = _e;
        (e.flags & un) !== qe && (Zt(e), RF(e), Vn());
        var t = e.sibling;
        if (t !== null) {
          t.return = e.return, _e = t;
          return;
        }
        _e = e.return;
      }
    }
    function RF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          e.mode & pt ? (q1(), no(Rr | ir, e, e.return), K1(e)) : no(Rr | ir, e, e.return);
          break;
        }
      }
    }
    function IF(e, t) {
      for (; _e !== null; ) {
        var o = _e;
        Zt(o), OF(o, t), Vn();
        var l = o.child;
        l !== null ? (l.return = o, _e = l) : AF(e);
      }
    }
    function AF(e) {
      for (; _e !== null; ) {
        var t = _e, o = t.sibling, l = t.return;
        if (PR(t), t === e) {
          _e = null;
          return;
        }
        if (o !== null) {
          o.return = l, _e = o;
          return;
        }
        _e = l;
      }
    }
    function OF(e, t) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          e.mode & pt ? (q1(), no(Rr, e, t), K1(e)) : no(Rr, e, t);
          break;
        }
      }
    }
    function DF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          try {
            $l(or | ir, e);
          } catch (o) {
            on(e, e.return, o);
          }
          break;
        }
        case E: {
          var t = e.stateNode;
          try {
            t.componentDidMount();
          } catch (o) {
            on(e, e.return, o);
          }
          break;
        }
      }
    }
    function NF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          try {
            $l(Rr | ir, e);
          } catch (t) {
            on(e, e.return, t);
          }
          break;
        }
      }
    }
    function kF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P: {
          try {
            no(or | ir, e, e.return);
          } catch (o) {
            on(e, e.return, o);
          }
          break;
        }
        case E: {
          var t = e.stateNode;
          typeof t.componentWillUnmount == "function" && NS(e, e.return, t);
          break;
        }
      }
    }
    function MF(e) {
      switch (e.tag) {
        case b:
        case z:
        case P:
          try {
            no(Rr | ir, e, e.return);
          } catch (t) {
            on(e, e.return, t);
          }
      }
    }
    if (typeof Symbol == "function" && Symbol.for) {
      var Eh = Symbol.for;
      Eh("selector.component"), Eh("selector.has_pseudo_class"), Eh("selector.role"), Eh("selector.test_id"), Eh("selector.text");
    }
    var LF = [];
    function FF() {
      LF.forEach(function(e) {
        return e();
      });
    }
    var BF = u.ReactCurrentActQueue;
    function VF(e) {
      {
        var t = (
          // $FlowExpectedError – Flow doesn't know about IS_REACT_ACT_ENVIRONMENT global
          typeof IS_REACT_ACT_ENVIRONMENT < "u" ? IS_REACT_ACT_ENVIRONMENT : void 0
        ), o = typeof jest < "u";
        return o && t !== !1;
      }
    }
    function ZR() {
      {
        var e = (
          // $FlowExpectedError – Flow doesn't know about IS_REACT_ACT_ENVIRONMENT global
          typeof IS_REACT_ACT_ENVIRONMENT < "u" ? IS_REACT_ACT_ENVIRONMENT : void 0
        );
        return !e && BF.current !== null && h("The current testing environment is not configured to support act(...)"), e;
      }
    }
    var PF = Math.ceil, FS = u.ReactCurrentDispatcher, BS = u.ReactCurrentOwner, Dr = u.ReactCurrentBatchConfig, io = u.ReactCurrentActQueue, sr = (
      /*             */
      0
    ), YR = (
      /*               */
      1
    ), Nr = (
      /*                */
      2
    ), Ri = (
      /*                */
      4
    ), Nu = 0, xh = 1, Ec = 2, C0 = 3, wh = 4, $R = 5, VS = 6, Dt = sr, fa = null, Rn = null, cr = fe, Ho = fe, PS = zl(fe), fr = Nu, Th = null, S0 = fe, Rh = fe, E0 = fe, Ih = null, Ha = null, HS = 0, QR = 500, JR = 1 / 0, HF = 500, ku = null;
    function Ah() {
      JR = cn() + HF;
    }
    function KR() {
      return JR;
    }
    var x0 = !1, zS = null, Gd = null, xc = !1, Jl = null, Oh = fe, GS = [], _S = null, zF = 50, Dh = 0, jS = null, US = !1, w0 = !1, GF = 50, _d = 0, T0 = null, Nh = tn, R0 = fe, qR = !1;
    function I0() {
      return fa;
    }
    function da() {
      return (Dt & (Nr | Ri)) !== sr ? cn() : (Nh !== tn || (Nh = cn()), Nh);
    }
    function Kl(e) {
      var t = e.mode;
      if ((t & $e) === Xe)
        return Je;
      if ((Dt & Nr) !== sr && cr !== fe)
        return Sl(cr);
      var o = BM() !== FM;
      if (o) {
        if (Dr.transition !== null) {
          var l = Dr.transition;
          l._updatedFibers || (l._updatedFibers = /* @__PURE__ */ new Set()), l._updatedFibers.add(e);
        }
        return R0 === kn && (R0 = Hg()), R0;
      }
      var d = Na();
      if (d !== kn)
        return d;
      var m = Ck();
      return m;
    }
    function _F(e) {
      var t = e.mode;
      return (t & $e) === Xe ? Je : Wr();
    }
    function dr(e, t, o, l) {
      d4(), qR && h("useInsertionEffect must not schedule updates."), US && (w0 = !0), gu(e, o, l), (Dt & Nr) !== fe && e === fa ? h4(t) : (ua && $f(e, t, o), m4(t), e === fa && ((Dt & Nr) === sr && (Rh = Ct(Rh, o)), fr === wh && ql(e, cr)), za(e, l), o === Je && Dt === sr && (t.mode & $e) === Xe && // Treat `act` as if it's inside `batchedUpdates`, even in legacy mode.
      !io.isBatchingLegacy && (Ah(), qw()));
    }
    function jF(e, t, o) {
      var l = e.current;
      l.lanes = t, gu(e, t, o), za(e, o);
    }
    function UF(e) {
      return (
        // TODO: Remove outdated deferRenderPhaseUpdateToNextBatch experiment. We
        // decided not to enable it.
        (Dt & Nr) !== sr
      );
    }
    function za(e, t) {
      var o = e.callbackNode;
      Lg(e, t);
      var l = hu(e, e === fa ? cr : fe);
      if (l === fe) {
        o !== null && hI(o), e.callbackNode = null, e.callbackPriority = kn;
        return;
      }
      var d = xn(l), m = e.callbackPriority;
      if (m === d && // Special case related to `act`. If the currently scheduled task is a
      // Scheduler task, rather than an `act` task, cancel it and re-scheduled
      // on the `act` queue.
      !(io.current !== null && o !== JS)) {
        o == null && m !== Je && h("Expected scheduled callback to exist. This error is likely caused by a bug in React. Please file an issue.");
        return;
      }
      o != null && hI(o);
      var y;
      if (d === Je)
        e.tag === Gl ? (io.isBatchingLegacy !== null && (io.didScheduleLegacyUpdate = !0), yM(nI.bind(null, e))) : Kw(nI.bind(null, e)), io.current !== null ? io.current.push(_l) : Ek(function() {
          (Dt & (Nr | Ri)) === sr && _l();
        }), y = null;
      else {
        var x;
        switch (ar(l)) {
          case wn:
            x = Ef;
            break;
          case Wi:
            x = lu;
            break;
          case gi:
            x = mi;
            break;
          case El:
            x = xf;
            break;
          default:
            x = mi;
            break;
        }
        y = KS(x, eI.bind(null, e));
      }
      e.callbackPriority = d, e.callbackNode = y;
    }
    function eI(e, t) {
      if (lL(), Nh = tn, R0 = fe, (Dt & (Nr | Ri)) !== sr)
        throw new Error("Should not already be working.");
      var o = e.callbackNode, l = Lu();
      if (l && e.callbackNode !== o)
        return null;
      var d = hu(e, e === fa ? cr : fe);
      if (d === fe)
        return null;
      var m = !$s(e, d) && !Pg(e, d) && !t, y = m ? e4(e, d) : O0(e, d);
      if (y !== Nu) {
        if (y === Ec) {
          var x = To(e);
          x !== fe && (d = x, y = WS(e, x));
        }
        if (y === xh) {
          var T = Th;
          throw wc(e, fe), ql(e, d), za(e, cn()), T;
        }
        if (y === VS)
          ql(e, d);
        else {
          var O = !$s(e, d), N = e.current.alternate;
          if (O && !XF(N)) {
            if (y = O0(e, d), y === Ec) {
              var j = To(e);
              j !== fe && (d = j, y = WS(e, j));
            }
            if (y === xh) {
              var G = Th;
              throw wc(e, fe), ql(e, d), za(e, cn()), G;
            }
          }
          e.finishedWork = N, e.finishedLanes = d, WF(e, y, d);
        }
      }
      return za(e, cn()), e.callbackNode === o ? eI.bind(null, e) : null;
    }
    function WS(e, t) {
      var o = Ih;
      if (Qf(e)) {
        var l = wc(e, t);
        l.flags |= gn, fM(e.containerInfo);
      }
      var d = O0(e, t);
      if (d !== Ec) {
        var m = Ha;
        Ha = o, m !== null && tI(m);
      }
      return d;
    }
    function tI(e) {
      Ha === null ? Ha = e : Ha.push.apply(Ha, e);
    }
    function WF(e, t, o) {
      switch (t) {
        case Nu:
        case xh:
          throw new Error("Root did not complete. This is a bug in React.");
        case Ec: {
          Tc(e, Ha, ku);
          break;
        }
        case C0: {
          if (ql(e, o), Fg(o) && // do not delay if we're inside an act() scope
          !mI()) {
            var l = HS + QR - cn();
            if (l > 10) {
              var d = hu(e, fe);
              if (d !== fe)
                break;
              var m = e.suspendedLanes;
              if (!mu(m, o)) {
                da(), Zf(e, m);
                break;
              }
              e.timeoutHandle = _C(Tc.bind(null, e, Ha, ku), l);
              break;
            }
          }
          Tc(e, Ha, ku);
          break;
        }
        case wh: {
          if (ql(e, o), Vg(o))
            break;
          if (!mI()) {
            var y = kg(e, o), x = y, T = cn() - x, O = f4(T) - T;
            if (O > 10) {
              e.timeoutHandle = _C(Tc.bind(null, e, Ha, ku), O);
              break;
            }
          }
          Tc(e, Ha, ku);
          break;
        }
        case $R: {
          Tc(e, Ha, ku);
          break;
        }
        default:
          throw new Error("Unknown root exit status.");
      }
    }
    function XF(e) {
      for (var t = e; ; ) {
        if (t.flags & Hs) {
          var o = t.updateQueue;
          if (o !== null) {
            var l = o.stores;
            if (l !== null)
              for (var d = 0; d < l.length; d++) {
                var m = l[d], y = m.getSnapshot, x = m.value;
                try {
                  if (!Ge(y(), x))
                    return !1;
                } catch {
                  return !1;
                }
              }
          }
        }
        var T = t.child;
        if (t.subtreeFlags & Hs && T !== null) {
          T.return = t, t = T;
          continue;
        }
        if (t === e)
          return !0;
        for (; t.sibling === null; ) {
          if (t.return === null || t.return === e)
            return !0;
          t = t.return;
        }
        t.sibling.return = t.return, t = t.sibling;
      }
      return !0;
    }
    function ql(e, t) {
      t = Qs(t, E0), t = Qs(t, Rh), Gg(e, t);
    }
    function nI(e) {
      if (sL(), (Dt & (Nr | Ri)) !== sr)
        throw new Error("Should not already be working.");
      Lu();
      var t = hu(e, fe);
      if (!Xr(t, Je))
        return za(e, cn()), null;
      var o = O0(e, t);
      if (e.tag !== Gl && o === Ec) {
        var l = To(e);
        l !== fe && (t = l, o = WS(e, l));
      }
      if (o === xh) {
        var d = Th;
        throw wc(e, fe), ql(e, t), za(e, cn()), d;
      }
      if (o === VS)
        throw new Error("Root did not complete. This is a bug in React.");
      var m = e.current.alternate;
      return e.finishedWork = m, e.finishedLanes = t, Tc(e, Ha, ku), za(e, cn()), null;
    }
    function ZF(e, t) {
      t !== fe && (Sv(e, Ct(t, Je)), za(e, cn()), (Dt & (Nr | Ri)) === sr && (Ah(), _l()));
    }
    function XS(e, t) {
      var o = Dt;
      Dt |= YR;
      try {
        return e(t);
      } finally {
        Dt = o, Dt === sr && // Treat `act` as if it's inside `batchedUpdates`, even in legacy mode.
        !io.isBatchingLegacy && (Ah(), qw());
      }
    }
    function YF(e, t, o, l, d) {
      var m = Na(), y = Dr.transition;
      try {
        return Dr.transition = null, yn(wn), e(t, o, l, d);
      } finally {
        yn(m), Dr.transition = y, Dt === sr && Ah();
      }
    }
    function Mu(e) {
      Jl !== null && Jl.tag === Gl && (Dt & (Nr | Ri)) === sr && Lu();
      var t = Dt;
      Dt |= YR;
      var o = Dr.transition, l = Na();
      try {
        return Dr.transition = null, yn(wn), e ? e() : void 0;
      } finally {
        yn(l), Dr.transition = o, Dt = t, (Dt & (Nr | Ri)) === sr && _l();
      }
    }
    function rI() {
      return (Dt & (Nr | Ri)) !== sr;
    }
    function A0(e, t) {
      $r(PS, Ho, e), Ho = Ct(Ho, t);
    }
    function ZS(e) {
      Ho = PS.current, Yr(PS, e);
    }
    function wc(e, t) {
      e.finishedWork = null, e.finishedLanes = fe;
      var o = e.timeoutHandle;
      if (o !== jC && (e.timeoutHandle = jC, Sk(o)), Rn !== null)
        for (var l = Rn.return; l !== null; ) {
          var d = l.alternate;
          kR(d, l), l = l.return;
        }
      fa = e;
      var m = Rc(e.current, null);
      return Rn = m, cr = Ho = t, fr = Nu, Th = null, S0 = fe, Rh = fe, E0 = fe, Ih = null, Ha = null, jM(), Ji.discardPendingWarnings(), m;
    }
    function aI(e, t) {
      do {
        var o = Rn;
        try {
          if (Py(), AT(), Vn(), BS.current = null, o === null || o.return === null) {
            fr = xh, Th = t, Rn = null;
            return;
          }
          if (Oe && o.mode & pt && p0(o, !0), De)
            if (jr(), t !== null && typeof t == "object" && typeof t.then == "function") {
              var l = t;
              cu(o, l, cr);
            } else
              _s(o, t, cr);
          yL(e, o.return, o, t, cr), lI(o);
        } catch (d) {
          t = d, Rn === o && o !== null ? (o = o.return, Rn = o) : o = Rn;
          continue;
        }
        return;
      } while (!0);
    }
    function iI() {
      var e = FS.current;
      return FS.current = l0, e === null ? l0 : e;
    }
    function oI(e) {
      FS.current = e;
    }
    function $F() {
      HS = cn();
    }
    function kh(e) {
      S0 = Ct(e, S0);
    }
    function QF() {
      fr === Nu && (fr = C0);
    }
    function YS() {
      (fr === Nu || fr === C0 || fr === Ec) && (fr = wh), fa !== null && (Ys(S0) || Ys(Rh)) && ql(fa, cr);
    }
    function JF(e) {
      fr !== wh && (fr = Ec), Ih === null ? Ih = [e] : Ih.push(e);
    }
    function KF() {
      return fr === Nu;
    }
    function O0(e, t) {
      var o = Dt;
      Dt |= Nr;
      var l = iI();
      if (fa !== e || cr !== t) {
        if (ua) {
          var d = e.memoizedUpdaters;
          d.size > 0 && (Mh(e, cr), d.clear()), Ev(e, t);
        }
        ku = Ks(), wc(e, t);
      }
      ln(t);
      do
        try {
          qF();
          break;
        } catch (m) {
          aI(e, m);
        }
      while (!0);
      if (Py(), Dt = o, oI(l), Rn !== null)
        throw new Error("Cannot commit an incomplete root. This error is likely caused by a bug in React. Please file an issue.");
      return Af(), fa = null, cr = fe, fr;
    }
    function qF() {
      for (; Rn !== null; )
        uI(Rn);
    }
    function e4(e, t) {
      var o = Dt;
      Dt |= Nr;
      var l = iI();
      if (fa !== e || cr !== t) {
        if (ua) {
          var d = e.memoizedUpdaters;
          d.size > 0 && (Mh(e, cr), d.clear()), Ev(e, t);
        }
        ku = Ks(), Ah(), wc(e, t);
      }
      ln(t);
      do
        try {
          t4();
          break;
        } catch (m) {
          aI(e, m);
        }
      while (!0);
      return Py(), oI(l), Dt = o, Rn !== null ? (If(), Nu) : (Af(), fa = null, cr = fe, fr);
    }
    function t4() {
      for (; Rn !== null && !Sf(); )
        uI(Rn);
    }
    function uI(e) {
      var t = e.alternate;
      Zt(e);
      var o;
      (e.mode & pt) !== Xe ? (J1(e), o = $S(t, e, Ho), p0(e, !0)) : o = $S(t, e, Ho), Vn(), e.memoizedProps = e.pendingProps, o === null ? lI(e) : Rn = o, BS.current = null;
    }
    function lI(e) {
      var t = e;
      do {
        var o = t.alternate, l = t.return;
        if ((t.flags & aa) === qe) {
          Zt(t);
          var d = void 0;
          if ((t.mode & pt) === Xe ? d = NR(o, t, Ho) : (J1(t), d = NR(o, t, Ho), p0(t, !1)), Vn(), d !== null) {
            Rn = d;
            return;
          }
        } else {
          var m = QL(o, t);
          if (m !== null) {
            m.flags &= Sg, Rn = m;
            return;
          }
          if ((t.mode & pt) !== Xe) {
            p0(t, !1);
            for (var y = t.actualDuration, x = t.child; x !== null; )
              y += x.actualDuration, x = x.sibling;
            t.actualDuration = y;
          }
          if (l !== null)
            l.flags |= aa, l.subtreeFlags = qe, l.deletions = null;
          else {
            fr = VS, Rn = null;
            return;
          }
        }
        var T = t.sibling;
        if (T !== null) {
          Rn = T;
          return;
        }
        t = l, Rn = t;
      } while (t !== null);
      fr === Nu && (fr = $R);
    }
    function Tc(e, t, o) {
      var l = Na(), d = Dr.transition;
      try {
        Dr.transition = null, yn(wn), n4(e, t, o, l);
      } finally {
        Dr.transition = d, yn(l);
      }
      return null;
    }
    function n4(e, t, o, l) {
      do
        Lu();
      while (Jl !== null);
      if (p4(), (Dt & (Nr | Ri)) !== sr)
        throw new Error("Should not already be working.");
      var d = e.finishedWork, m = e.finishedLanes;
      if (Eo(m), d === null)
        return wf(), null;
      if (m === fe && h("root.finishedLanes should not be empty during a commit. This is a bug in React."), e.finishedWork = null, e.finishedLanes = fe, d === e.current)
        throw new Error("Cannot commit the same tree as before. This error is likely caused by a bug in React. Please file an issue.");
      e.callbackNode = null, e.callbackPriority = kn;
      var y = Ct(d.lanes, d.childLanes);
      Yf(e, y), e === fa && (fa = null, Rn = null, cr = fe), ((d.subtreeFlags & Ra) !== qe || (d.flags & Ra) !== qe) && (xc || (xc = !0, _S = o, KS(mi, function() {
        return Lu(), null;
      })));
      var x = (d.subtreeFlags & (ll | Gr | hr | Ra)) !== qe, T = (d.flags & (ll | Gr | hr | Ra)) !== qe;
      if (x || T) {
        var O = Dr.transition;
        Dr.transition = null;
        var N = Na();
        yn(wn);
        var j = Dt;
        Dt |= Ri, BS.current = null, tF(e, d), qT(), hF(e, d, m), vk(e.containerInfo), e.current = d, pv(m), mF(d, e, m), pl(), wg(), Dt = j, yn(N), Dr.transition = O;
      } else
        e.current = d, qT();
      var G = xc;
      if (xc ? (xc = !1, Jl = e, Oh = m) : (_d = 0, T0 = null), y = e.pendingLanes, y === fe && (Gd = null), G || dI(e.current, !1), cl(d.stateNode, l), ua && e.memoizedUpdaters.clear(), FF(), za(e, cn()), t !== null)
        for (var te = e.onRecoverableError, re = 0; re < t.length; re++) {
          var ie = t[re], Ne = ie.stack, et = ie.digest;
          te(ie.value, {
            componentStack: Ne,
            digest: et
          });
        }
      if (x0) {
        x0 = !1;
        var Qe = zS;
        throw zS = null, Qe;
      }
      return Xr(Oh, Je) && e.tag !== Gl && Lu(), y = e.pendingLanes, Xr(y, Je) ? (uL(), e === jS ? Dh++ : (Dh = 0, jS = e)) : Dh = 0, _l(), wf(), null;
    }
    function Lu() {
      if (Jl !== null) {
        var e = ar(Oh), t = pC(gi, e), o = Dr.transition, l = Na();
        try {
          return Dr.transition = null, yn(t), a4();
        } finally {
          yn(l), Dr.transition = o;
        }
      }
      return !1;
    }
    function r4(e) {
      GS.push(e), xc || (xc = !0, KS(mi, function() {
        return Lu(), null;
      }));
    }
    function a4() {
      if (Jl === null)
        return !1;
      var e = _S;
      _S = null;
      var t = Jl, o = Oh;
      if (Jl = null, Oh = fe, (Dt & (Nr | Ri)) !== sr)
        throw new Error("Cannot flush passive effects while already rendering.");
      US = !0, w0 = !1, Dg(o);
      var l = Dt;
      Dt |= Ri, xF(t.current), bF(t, t.current, o, e);
      {
        var d = GS;
        GS = [];
        for (var m = 0; m < d.length; m++) {
          var y = d[m];
          iF(t, y);
        }
      }
      vv(), dI(t.current, !0), Dt = l, _l(), w0 ? t === T0 ? _d++ : (_d = 0, T0 = t) : _d = 0, US = !1, w0 = !1, Oa(t);
      {
        var x = t.current.stateNode;
        x.effectDuration = 0, x.passiveEffectDuration = 0;
      }
      return !0;
    }
    function sI(e) {
      return Gd !== null && Gd.has(e);
    }
    function i4(e) {
      Gd === null ? Gd = /* @__PURE__ */ new Set([e]) : Gd.add(e);
    }
    function o4(e) {
      x0 || (x0 = !0, zS = e);
    }
    var u4 = o4;
    function cI(e, t, o) {
      var l = Cc(o, t), d = uR(e, l, Je), m = Ul(e, d, Je), y = da();
      m !== null && (gu(m, Je, y), za(m, y));
    }
    function on(e, t, o) {
      if (KL(o), Lh(!1), e.tag === R) {
        cI(e, e, o);
        return;
      }
      var l = null;
      for (l = t; l !== null; ) {
        if (l.tag === R) {
          cI(l, e, o);
          return;
        } else if (l.tag === E) {
          var d = l.type, m = l.stateNode;
          if (typeof d.getDerivedStateFromError == "function" || typeof m.componentDidCatch == "function" && !sI(m)) {
            var y = Cc(o, e), x = hS(l, y, Je), T = Ul(l, x, Je), O = da();
            T !== null && (gu(T, Je, O), za(T, O));
            return;
          }
        }
        l = l.return;
      }
      h(`Internal React error: Attempted to capture a commit phase error inside a detached tree. This indicates a bug in React. Likely causes include deleting the same fiber more than once, committing an already-finished tree, or an inconsistent return pointer.

Error message:

%s`, o);
    }
    function l4(e, t, o) {
      var l = e.pingCache;
      l !== null && l.delete(t);
      var d = da();
      Zf(e, o), g4(e), fa === e && mu(cr, o) && (fr === wh || fr === C0 && Fg(cr) && cn() - HS < QR ? wc(e, fe) : E0 = Ct(E0, o)), za(e, d);
    }
    function fI(e, t) {
      t === kn && (t = _F(e));
      var o = da(), l = Va(e, t);
      l !== null && (gu(l, t, o), za(l, o));
    }
    function s4(e) {
      var t = e.memoizedState, o = kn;
      t !== null && (o = t.retryLane), fI(e, o);
    }
    function c4(e, t) {
      var o = kn, l;
      switch (e.tag) {
        case V:
          l = e.stateNode;
          var d = e.memoizedState;
          d !== null && (o = d.retryLane);
          break;
        case he:
          l = e.stateNode;
          break;
        default:
          throw new Error("Pinged unknown suspense boundary type. This is probably a bug in React.");
      }
      l !== null && l.delete(t), fI(e, o);
    }
    function f4(e) {
      return e < 120 ? 120 : e < 480 ? 480 : e < 1080 ? 1080 : e < 1920 ? 1920 : e < 3e3 ? 3e3 : e < 4320 ? 4320 : PF(e / 1960) * 1960;
    }
    function d4() {
      if (Dh > zF)
        throw Dh = 0, jS = null, new Error("Maximum update depth exceeded. This can happen when a component repeatedly calls setState inside componentWillUpdate or componentDidUpdate. React limits the number of nested updates to prevent infinite loops.");
      _d > GF && (_d = 0, T0 = null, h("Maximum update depth exceeded. This can happen when a component calls setState inside useEffect, but useEffect either doesn't have a dependency array, or one of the dependencies changes on every render."));
    }
    function p4() {
      Ji.flushLegacyContextWarning(), Ji.flushPendingUnsafeLifecycleWarnings();
    }
    function dI(e, t) {
      Zt(e), D0(e, zr, kF), t && D0(e, uu, MF), D0(e, zr, DF), t && D0(e, uu, NF), Vn();
    }
    function D0(e, t, o) {
      for (var l = e, d = null; l !== null; ) {
        var m = l.subtreeFlags & t;
        l !== d && l.child !== null && m !== qe ? l = l.child : ((l.flags & t) !== qe && o(l), l.sibling !== null ? l = l.sibling : l = d = l.return);
      }
    }
    var N0 = null;
    function pI(e) {
      {
        if ((Dt & Nr) !== sr || !(e.mode & $e))
          return;
        var t = e.tag;
        if (t !== w && t !== R && t !== E && t !== b && t !== z && t !== Q && t !== P)
          return;
        var o = bt(e) || "ReactComponent";
        if (N0 !== null) {
          if (N0.has(o))
            return;
          N0.add(o);
        } else
          N0 = /* @__PURE__ */ new Set([o]);
        var l = On;
        try {
          Zt(e), h("Can't perform a React state update on a component that hasn't mounted yet. This indicates that you have a side-effect in your render function that asynchronously later calls tries to update the component. Move this work to useEffect instead.");
        } finally {
          l ? Zt(e) : Vn();
        }
      }
    }
    var $S;
    {
      var v4 = null;
      $S = function(e, t, o) {
        var l = SI(v4, t);
        try {
          return RR(e, t, o);
        } catch (m) {
          if (RM() || m !== null && typeof m == "object" && typeof m.then == "function")
            throw m;
          if (Py(), AT(), kR(e, t), SI(t, l), t.mode & pt && J1(t), ou(null, RR, null, e, t, o), lC()) {
            var d = av();
            typeof d == "object" && d !== null && d._suppressLogging && typeof m == "object" && m !== null && !m._suppressLogging && (m._suppressLogging = !0);
          }
          throw m;
        }
      };
    }
    var vI = !1, QS;
    QS = /* @__PURE__ */ new Set();
    function h4(e) {
      if (Ca && !aL())
        switch (e.tag) {
          case b:
          case z:
          case P: {
            var t = Rn && bt(Rn) || "Unknown", o = t;
            if (!QS.has(o)) {
              QS.add(o);
              var l = bt(e) || "Unknown";
              h("Cannot update a component (`%s`) while rendering a different component (`%s`). To locate the bad setState() call inside `%s`, follow the stack trace as described in https://reactjs.org/link/setstate-in-render", l, t, t);
            }
            break;
          }
          case E: {
            vI || (h("Cannot update during an existing state transition (such as within `render`). Render methods should be a pure function of props and state."), vI = !0);
            break;
          }
        }
    }
    function Mh(e, t) {
      if (ua) {
        var o = e.memoizedUpdaters;
        o.forEach(function(l) {
          $f(e, l, t);
        });
      }
    }
    var JS = {};
    function KS(e, t) {
      {
        var o = io.current;
        return o !== null ? (o.push(t), JS) : Cf(e, t);
      }
    }
    function hI(e) {
      if (e !== JS)
        return xg(e);
    }
    function mI() {
      return io.current !== null;
    }
    function m4(e) {
      {
        if (e.mode & $e) {
          if (!ZR())
            return;
        } else if (!VF() || Dt !== sr || e.tag !== b && e.tag !== z && e.tag !== P)
          return;
        if (io.current === null) {
          var t = On;
          try {
            Zt(e), h(`An update to %s inside a test was not wrapped in act(...).

When testing, code that causes React state updates should be wrapped into act(...):

act(() => {
  /* fire events that update state */
});
/* assert on the output */

This ensures that you're testing the behavior the user would see in the browser. Learn more at https://reactjs.org/link/wrap-tests-with-act`, bt(e));
          } finally {
            t ? Zt(e) : Vn();
          }
        }
      }
    }
    function g4(e) {
      e.tag !== Gl && ZR() && io.current === null && h(`A suspended resource finished loading inside a test, but the event was not wrapped in act(...).

When testing, code that resolves suspended data should be wrapped into act(...):

act(() => {
  /* finish loading suspended data */
});
/* assert on the output */

This ensures that you're testing the behavior the user would see in the browser. Learn more at https://reactjs.org/link/wrap-tests-with-act`);
    }
    function Lh(e) {
      qR = e;
    }
    var Ii = null, jd = null, y4 = function(e) {
      Ii = e;
    };
    function Ud(e) {
      {
        if (Ii === null)
          return e;
        var t = Ii(e);
        return t === void 0 ? e : t.current;
      }
    }
    function qS(e) {
      return Ud(e);
    }
    function eE(e) {
      {
        if (Ii === null)
          return e;
        var t = Ii(e);
        if (t === void 0) {
          if (e != null && typeof e.render == "function") {
            var o = Ud(e.render);
            if (e.render !== o) {
              var l = {
                $$typeof: Yu,
                render: o
              };
              return e.displayName !== void 0 && (l.displayName = e.displayName), l;
            }
          }
          return e;
        }
        return t.current;
      }
    }
    function gI(e, t) {
      {
        if (Ii === null)
          return !1;
        var o = e.elementType, l = t.type, d = !1, m = typeof l == "object" && l !== null ? l.$$typeof : null;
        switch (e.tag) {
          case E: {
            typeof l == "function" && (d = !0);
            break;
          }
          case b: {
            (typeof l == "function" || m === Er) && (d = !0);
            break;
          }
          case z: {
            (m === Yu || m === Er) && (d = !0);
            break;
          }
          case Q:
          case P: {
            (m === $u || m === Er) && (d = !0);
            break;
          }
          default:
            return !1;
        }
        if (d) {
          var y = Ii(o);
          if (y !== void 0 && y === Ii(l))
            return !0;
        }
        return !1;
      }
    }
    function yI(e) {
      {
        if (Ii === null || typeof WeakSet != "function")
          return;
        jd === null && (jd = /* @__PURE__ */ new WeakSet()), jd.add(e);
      }
    }
    var b4 = function(e, t) {
      {
        if (Ii === null)
          return;
        var o = t.staleFamilies, l = t.updatedFamilies;
        Lu(), Mu(function() {
          tE(e.current, l, o);
        });
      }
    }, C4 = function(e, t) {
      {
        if (e.context !== ai)
          return;
        Lu(), Mu(function() {
          Fh(t, e, null, null);
        });
      }
    };
    function tE(e, t, o) {
      {
        var l = e.alternate, d = e.child, m = e.sibling, y = e.tag, x = e.type, T = null;
        switch (y) {
          case b:
          case P:
          case E:
            T = x;
            break;
          case z:
            T = x.render;
            break;
        }
        if (Ii === null)
          throw new Error("Expected resolveFamily to be set during hot reload.");
        var O = !1, N = !1;
        if (T !== null) {
          var j = Ii(T);
          j !== void 0 && (o.has(j) ? N = !0 : t.has(j) && (y === E ? N = !0 : O = !0));
        }
        if (jd !== null && (jd.has(e) || l !== null && jd.has(l)) && (N = !0), N && (e._debugNeedsRemount = !0), N || O) {
          var G = Va(e, Je);
          G !== null && dr(G, e, Je, tn);
        }
        d !== null && !N && tE(d, t, o), m !== null && tE(m, t, o);
      }
    }
    var S4 = function(e, t) {
      {
        var o = /* @__PURE__ */ new Set(), l = new Set(t.map(function(d) {
          return d.current;
        }));
        return nE(e.current, l, o), o;
      }
    };
    function nE(e, t, o) {
      {
        var l = e.child, d = e.sibling, m = e.tag, y = e.type, x = null;
        switch (m) {
          case b:
          case P:
          case E:
            x = y;
            break;
          case z:
            x = y.render;
            break;
        }
        var T = !1;
        x !== null && t.has(x) && (T = !0), T ? E4(e, o) : l !== null && nE(l, t, o), d !== null && nE(d, t, o);
      }
    }
    function E4(e, t) {
      {
        var o = x4(e, t);
        if (o)
          return;
        for (var l = e; ; ) {
          switch (l.tag) {
            case D:
              t.add(l.stateNode);
              return;
            case A:
              t.add(l.stateNode.containerInfo);
              return;
            case R:
              t.add(l.stateNode.containerInfo);
              return;
          }
          if (l.return === null)
            throw new Error("Expected to reach root first.");
          l = l.return;
        }
      }
    }
    function x4(e, t) {
      for (var o = e, l = !1; ; ) {
        if (o.tag === D)
          l = !0, t.add(o.stateNode);
        else if (o.child !== null) {
          o.child.return = o, o = o.child;
          continue;
        }
        if (o === e)
          return l;
        for (; o.sibling === null; ) {
          if (o.return === null || o.return === e)
            return l;
          o = o.return;
        }
        o.sibling.return = o.return, o = o.sibling;
      }
      return !1;
    }
    var rE;
    {
      rE = !1;
      try {
        var bI = Object.preventExtensions({});
      } catch {
        rE = !0;
      }
    }
    function w4(e, t, o, l) {
      this.tag = e, this.key = o, this.elementType = null, this.type = null, this.stateNode = null, this.return = null, this.child = null, this.sibling = null, this.index = 0, this.ref = null, this.pendingProps = t, this.memoizedProps = null, this.updateQueue = null, this.memoizedState = null, this.dependencies = null, this.mode = l, this.flags = qe, this.subtreeFlags = qe, this.deletions = null, this.lanes = fe, this.childLanes = fe, this.alternate = null, this.actualDuration = Number.NaN, this.actualStartTime = Number.NaN, this.selfBaseDuration = Number.NaN, this.treeBaseDuration = Number.NaN, this.actualDuration = 0, this.actualStartTime = -1, this.selfBaseDuration = 0, this.treeBaseDuration = 0, this._debugSource = null, this._debugOwner = null, this._debugNeedsRemount = !1, this._debugHookTypes = null, !rE && typeof Object.preventExtensions == "function" && Object.preventExtensions(this);
    }
    var ii = function(e, t, o, l) {
      return new w4(e, t, o, l);
    };
    function aE(e) {
      var t = e.prototype;
      return !!(t && t.isReactComponent);
    }
    function T4(e) {
      return typeof e == "function" && !aE(e) && e.defaultProps === void 0;
    }
    function R4(e) {
      if (typeof e == "function")
        return aE(e) ? E : b;
      if (e != null) {
        var t = e.$$typeof;
        if (t === Yu)
          return z;
        if (t === $u)
          return Q;
      }
      return w;
    }
    function Rc(e, t) {
      var o = e.alternate;
      o === null ? (o = ii(e.tag, t, e.key, e.mode), o.elementType = e.elementType, o.type = e.type, o.stateNode = e.stateNode, o._debugSource = e._debugSource, o._debugOwner = e._debugOwner, o._debugHookTypes = e._debugHookTypes, o.alternate = e, e.alternate = o) : (o.pendingProps = t, o.type = e.type, o.flags = qe, o.subtreeFlags = qe, o.deletions = null, o.actualDuration = 0, o.actualStartTime = -1), o.flags = e.flags & nr, o.childLanes = e.childLanes, o.lanes = e.lanes, o.child = e.child, o.memoizedProps = e.memoizedProps, o.memoizedState = e.memoizedState, o.updateQueue = e.updateQueue;
      var l = e.dependencies;
      switch (o.dependencies = l === null ? null : {
        lanes: l.lanes,
        firstContext: l.firstContext
      }, o.sibling = e.sibling, o.index = e.index, o.ref = e.ref, o.selfBaseDuration = e.selfBaseDuration, o.treeBaseDuration = e.treeBaseDuration, o._debugNeedsRemount = e._debugNeedsRemount, o.tag) {
        case w:
        case b:
        case P:
          o.type = Ud(e.type);
          break;
        case E:
          o.type = qS(e.type);
          break;
        case z:
          o.type = eE(e.type);
          break;
      }
      return o;
    }
    function I4(e, t) {
      e.flags &= nr | nn;
      var o = e.alternate;
      if (o === null)
        e.childLanes = fe, e.lanes = t, e.child = null, e.subtreeFlags = qe, e.memoizedProps = null, e.memoizedState = null, e.updateQueue = null, e.dependencies = null, e.stateNode = null, e.selfBaseDuration = 0, e.treeBaseDuration = 0;
      else {
        e.childLanes = o.childLanes, e.lanes = o.lanes, e.child = o.child, e.subtreeFlags = qe, e.deletions = null, e.memoizedProps = o.memoizedProps, e.memoizedState = o.memoizedState, e.updateQueue = o.updateQueue, e.type = o.type;
        var l = o.dependencies;
        e.dependencies = l === null ? null : {
          lanes: l.lanes,
          firstContext: l.firstContext
        }, e.selfBaseDuration = o.selfBaseDuration, e.treeBaseDuration = o.treeBaseDuration;
      }
      return e;
    }
    function A4(e, t, o) {
      var l;
      return e === Ay ? (l = $e, t === !0 && (l |= xt, l |= la)) : l = Xe, ua && (l |= pt), ii(R, null, null, l);
    }
    function iE(e, t, o, l, d, m) {
      var y = w, x = e;
      if (typeof e == "function")
        aE(e) ? (y = E, x = qS(x)) : x = Ud(x);
      else if (typeof e == "string")
        y = D;
      else
        e:
          switch (e) {
            case ki:
              return es(o.children, d, m, t);
            case Zu:
              y = k, d |= xt, (d & $e) !== Xe && (d |= la);
              break;
            case hs:
              return O4(o, d, m, t);
            case di:
              return D4(o, d, m, t);
            case ms:
              return N4(o, d, m, t);
            case Rp:
              return CI(o, d, m, t);
            case km:
            case Dm:
            case zb:
            case Gb:
            case Nm:
            default: {
              if (typeof e == "object" && e !== null)
                switch (e.$$typeof) {
                  case wp:
                    y = Y;
                    break e;
                  case Tp:
                    y = B;
                    break e;
                  case Yu:
                    y = z, x = eE(x);
                    break e;
                  case $u:
                    y = Q;
                    break e;
                  case Er:
                    y = U, x = null;
                    break e;
                }
              var T = "";
              {
                (e === void 0 || typeof e == "object" && e !== null && Object.keys(e).length === 0) && (T += " You likely forgot to export your component from the file it's defined in, or you might have mixed up default and named imports.");
                var O = l ? bt(l) : null;
                O && (T += `

Check the render method of \`` + O + "`.");
              }
              throw new Error("Element type is invalid: expected a string (for built-in components) or a class/function (for composite components) " + ("but got: " + (e == null ? e : typeof e) + "." + T));
            }
          }
      var N = ii(y, o, t, d);
      return N.elementType = e, N.type = x, N.lanes = m, N._debugOwner = l, N;
    }
    function oE(e, t, o) {
      var l = null;
      l = e._owner;
      var d = e.type, m = e.key, y = e.props, x = iE(d, m, y, l, t, o);
      return x._debugSource = e._source, x._debugOwner = e._owner, x;
    }
    function es(e, t, o, l) {
      var d = ii(M, e, l, t);
      return d.lanes = o, d;
    }
    function O4(e, t, o, l) {
      typeof e.id != "string" && h('Profiler must specify an "id" of type `string` as a prop. Received the type `%s` instead.', typeof e.id);
      var d = ii(Z, e, l, t | pt);
      return d.elementType = hs, d.lanes = o, d.stateNode = {
        effectDuration: 0,
        passiveEffectDuration: 0
      }, d;
    }
    function D4(e, t, o, l) {
      var d = ii(V, e, l, t);
      return d.elementType = di, d.lanes = o, d;
    }
    function N4(e, t, o, l) {
      var d = ii(he, e, l, t);
      return d.elementType = ms, d.lanes = o, d;
    }
    function CI(e, t, o, l) {
      var d = ii(ye, e, l, t);
      d.elementType = Rp, d.lanes = o;
      var m = {
        isHidden: !1
      };
      return d.stateNode = m, d;
    }
    function uE(e, t, o) {
      var l = ii(L, e, null, t);
      return l.lanes = o, l;
    }
    function k4() {
      var e = ii(D, null, null, Xe);
      return e.elementType = "DELETED", e;
    }
    function M4(e) {
      var t = ii(Se, null, null, Xe);
      return t.stateNode = e, t;
    }
    function lE(e, t, o) {
      var l = e.children !== null ? e.children : [], d = ii(A, l, e.key, t);
      return d.lanes = o, d.stateNode = {
        containerInfo: e.containerInfo,
        pendingChildren: null,
        // Used by persistent updates
        implementation: e.implementation
      }, d;
    }
    function SI(e, t) {
      return e === null && (e = ii(w, null, null, Xe)), e.tag = t.tag, e.key = t.key, e.elementType = t.elementType, e.type = t.type, e.stateNode = t.stateNode, e.return = t.return, e.child = t.child, e.sibling = t.sibling, e.index = t.index, e.ref = t.ref, e.pendingProps = t.pendingProps, e.memoizedProps = t.memoizedProps, e.updateQueue = t.updateQueue, e.memoizedState = t.memoizedState, e.dependencies = t.dependencies, e.mode = t.mode, e.flags = t.flags, e.subtreeFlags = t.subtreeFlags, e.deletions = t.deletions, e.lanes = t.lanes, e.childLanes = t.childLanes, e.alternate = t.alternate, e.actualDuration = t.actualDuration, e.actualStartTime = t.actualStartTime, e.selfBaseDuration = t.selfBaseDuration, e.treeBaseDuration = t.treeBaseDuration, e._debugSource = t._debugSource, e._debugOwner = t._debugOwner, e._debugNeedsRemount = t._debugNeedsRemount, e._debugHookTypes = t._debugHookTypes, e;
    }
    function L4(e, t, o, l, d) {
      this.tag = t, this.containerInfo = e, this.pendingChildren = null, this.current = null, this.pingCache = null, this.finishedWork = null, this.timeoutHandle = jC, this.context = null, this.pendingContext = null, this.callbackNode = null, this.callbackPriority = kn, this.eventTimes = Js(fe), this.expirationTimes = Js(tn), this.pendingLanes = fe, this.suspendedLanes = fe, this.pingedLanes = fe, this.expiredLanes = fe, this.mutableReadLanes = fe, this.finishedLanes = fe, this.entangledLanes = fe, this.entanglements = Js(fe), this.identifierPrefix = l, this.onRecoverableError = d, this.mutableSourceEagerHydrationData = null, this.effectDuration = 0, this.passiveEffectDuration = 0;
      {
        this.memoizedUpdaters = /* @__PURE__ */ new Set();
        for (var m = this.pendingUpdatersLaneMap = [], y = 0; y < Ws; y++)
          m.push(/* @__PURE__ */ new Set());
      }
      switch (t) {
        case Ay:
          this._debugRootType = o ? "hydrateRoot()" : "createRoot()";
          break;
        case Gl:
          this._debugRootType = o ? "hydrate()" : "render()";
          break;
      }
    }
    function EI(e, t, o, l, d, m, y, x, T, O) {
      var N = new L4(e, t, o, x, T), j = A4(t, m);
      N.current = j, j.stateNode = N;
      {
        var G = {
          element: l,
          isDehydrated: o,
          cache: null,
          // not enabled yet
          transitions: null,
          pendingSuspenseBoundaries: null
        };
        j.memoizedState = G;
      }
      return S1(j), N;
    }
    var sE = "18.3.1";
    function F4(e, t, o) {
      var l = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : null;
      return so(l), {
        // This tag allow us to uniquely identify this as a React Portal
        $$typeof: Qa,
        key: l == null ? null : "" + l,
        children: e,
        containerInfo: t,
        implementation: o
      };
    }
    var cE, fE;
    cE = !1, fE = {};
    function xI(e) {
      if (!e)
        return ai;
      var t = xa(e), o = gM(t);
      if (t.tag === E) {
        var l = t.type;
        if (ko(l))
          return Qw(t, l, o);
      }
      return o;
    }
    function B4(e, t) {
      {
        var o = xa(e);
        if (o === void 0) {
          if (typeof e.render == "function")
            throw new Error("Unable to find node on an unmounted component.");
          var l = Object.keys(e).join(",");
          throw new Error("Argument appears to not be a ReactComponent. Keys: " + l);
        }
        var d = Ia(o);
        if (d === null)
          return null;
        if (d.mode & xt) {
          var m = bt(o) || "Component";
          if (!fE[m]) {
            fE[m] = !0;
            var y = On;
            try {
              Zt(d), o.mode & xt ? h("%s is deprecated in StrictMode. %s was passed an instance of %s which is inside StrictMode. Instead, add a ref directly to the element you want to reference. Learn more about using refs safely here: https://reactjs.org/link/strict-mode-find-node", t, t, m) : h("%s is deprecated in StrictMode. %s was passed an instance of %s which renders StrictMode children. Instead, add a ref directly to the element you want to reference. Learn more about using refs safely here: https://reactjs.org/link/strict-mode-find-node", t, t, m);
            } finally {
              y ? Zt(y) : Vn();
            }
          }
        }
        return d.stateNode;
      }
    }
    function wI(e, t, o, l, d, m, y, x) {
      var T = !1, O = null;
      return EI(e, t, T, O, o, l, d, m, y);
    }
    function TI(e, t, o, l, d, m, y, x, T, O) {
      var N = !0, j = EI(o, l, N, e, d, m, y, x, T);
      j.context = xI(null);
      var G = j.current, te = da(), re = Kl(G), ie = Ou(te, re);
      return ie.callback = t ?? null, Ul(G, ie, re), jF(j, re, te), j;
    }
    function Fh(e, t, o, l) {
      dv(t, e);
      var d = t.current, m = da(), y = Kl(d);
      hv(y);
      var x = xI(o);
      t.context === null ? t.context = x : t.pendingContext = x, Ca && On !== null && !cE && (cE = !0, h(`Render methods should be a pure function of props and state; triggering nested component updates from render is not allowed. If necessary, trigger nested updates in componentDidUpdate.

Check the render method of %s.`, bt(On) || "Unknown"));
      var T = Ou(m, y);
      T.payload = {
        element: e
      }, l = l === void 0 ? null : l, l !== null && (typeof l != "function" && h("render(...): Expected the last optional `callback` argument to be a function. Instead received: %s.", l), T.callback = l);
      var O = Ul(d, T, y);
      return O !== null && (dr(O, d, y, m), jy(O, d, y)), y;
    }
    function k0(e) {
      var t = e.current;
      if (!t.child)
        return null;
      switch (t.child.tag) {
        case D:
          return t.child.stateNode;
        default:
          return t.child.stateNode;
      }
    }
    function V4(e) {
      switch (e.tag) {
        case R: {
          var t = e.stateNode;
          if (Qf(t)) {
            var o = yv(t);
            ZF(t, o);
          }
          break;
        }
        case V: {
          Mu(function() {
            var d = Va(e, Je);
            if (d !== null) {
              var m = da();
              dr(d, e, Je, m);
            }
          });
          var l = Je;
          dE(e, l);
          break;
        }
      }
    }
    function RI(e, t) {
      var o = e.memoizedState;
      o !== null && o.dehydrated !== null && (o.retryLane = zg(o.retryLane, t));
    }
    function dE(e, t) {
      RI(e, t);
      var o = e.alternate;
      o && RI(o, t);
    }
    function P4(e) {
      if (e.tag === V) {
        var t = Zs, o = Va(e, t);
        if (o !== null) {
          var l = da();
          dr(o, e, t, l);
        }
        dE(e, t);
      }
    }
    function H4(e) {
      if (e.tag === V) {
        var t = Kl(e), o = Va(e, t);
        if (o !== null) {
          var l = da();
          dr(o, e, t, l);
        }
        dE(e, t);
      }
    }
    function II(e) {
      var t = Eg(e);
      return t === null ? null : t.stateNode;
    }
    var AI = function(e) {
      return null;
    };
    function z4(e) {
      return AI(e);
    }
    var OI = function(e) {
      return !1;
    };
    function G4(e) {
      return OI(e);
    }
    var DI = null, NI = null, kI = null, MI = null, LI = null, FI = null, BI = null, VI = null, PI = null;
    {
      var HI = function(e, t, o) {
        var l = t[o], d = Wn(e) ? e.slice() : Tt({}, e);
        return o + 1 === t.length ? (Wn(d) ? d.splice(l, 1) : delete d[l], d) : (d[l] = HI(e[l], t, o + 1), d);
      }, zI = function(e, t) {
        return HI(e, t, 0);
      }, GI = function(e, t, o, l) {
        var d = t[l], m = Wn(e) ? e.slice() : Tt({}, e);
        if (l + 1 === t.length) {
          var y = o[l];
          m[y] = m[d], Wn(m) ? m.splice(d, 1) : delete m[d];
        } else
          m[d] = GI(
            // $FlowFixMe number or string is fine here
            e[d],
            t,
            o,
            l + 1
          );
        return m;
      }, _I = function(e, t, o) {
        if (t.length !== o.length) {
          v("copyWithRename() expects paths of the same length");
          return;
        } else
          for (var l = 0; l < o.length - 1; l++)
            if (t[l] !== o[l]) {
              v("copyWithRename() expects paths to be the same except for the deepest key");
              return;
            }
        return GI(e, t, o, 0);
      }, jI = function(e, t, o, l) {
        if (o >= t.length)
          return l;
        var d = t[o], m = Wn(e) ? e.slice() : Tt({}, e);
        return m[d] = jI(e[d], t, o + 1, l), m;
      }, UI = function(e, t, o) {
        return jI(e, t, 0, o);
      }, pE = function(e, t) {
        for (var o = e.memoizedState; o !== null && t > 0; )
          o = o.next, t--;
        return o;
      };
      DI = function(e, t, o, l) {
        var d = pE(e, t);
        if (d !== null) {
          var m = UI(d.memoizedState, o, l);
          d.memoizedState = m, d.baseState = m, e.memoizedProps = Tt({}, e.memoizedProps);
          var y = Va(e, Je);
          y !== null && dr(y, e, Je, tn);
        }
      }, NI = function(e, t, o) {
        var l = pE(e, t);
        if (l !== null) {
          var d = zI(l.memoizedState, o);
          l.memoizedState = d, l.baseState = d, e.memoizedProps = Tt({}, e.memoizedProps);
          var m = Va(e, Je);
          m !== null && dr(m, e, Je, tn);
        }
      }, kI = function(e, t, o, l) {
        var d = pE(e, t);
        if (d !== null) {
          var m = _I(d.memoizedState, o, l);
          d.memoizedState = m, d.baseState = m, e.memoizedProps = Tt({}, e.memoizedProps);
          var y = Va(e, Je);
          y !== null && dr(y, e, Je, tn);
        }
      }, MI = function(e, t, o) {
        e.pendingProps = UI(e.memoizedProps, t, o), e.alternate && (e.alternate.pendingProps = e.pendingProps);
        var l = Va(e, Je);
        l !== null && dr(l, e, Je, tn);
      }, LI = function(e, t) {
        e.pendingProps = zI(e.memoizedProps, t), e.alternate && (e.alternate.pendingProps = e.pendingProps);
        var o = Va(e, Je);
        o !== null && dr(o, e, Je, tn);
      }, FI = function(e, t, o) {
        e.pendingProps = _I(e.memoizedProps, t, o), e.alternate && (e.alternate.pendingProps = e.pendingProps);
        var l = Va(e, Je);
        l !== null && dr(l, e, Je, tn);
      }, BI = function(e) {
        var t = Va(e, Je);
        t !== null && dr(t, e, Je, tn);
      }, VI = function(e) {
        AI = e;
      }, PI = function(e) {
        OI = e;
      };
    }
    function _4(e) {
      var t = Ia(e);
      return t === null ? null : t.stateNode;
    }
    function j4(e) {
      return null;
    }
    function U4() {
      return On;
    }
    function W4(e) {
      var t = e.findFiberByHostInstance, o = u.ReactCurrentDispatcher;
      return fv({
        bundleType: e.bundleType,
        version: e.version,
        rendererPackageName: e.rendererPackageName,
        rendererConfig: e.rendererConfig,
        overrideHookState: DI,
        overrideHookStateDeletePath: NI,
        overrideHookStateRenamePath: kI,
        overrideProps: MI,
        overridePropsDeletePath: LI,
        overridePropsRenamePath: FI,
        setErrorHandler: VI,
        setSuspenseHandler: PI,
        scheduleUpdate: BI,
        currentDispatcherRef: o,
        findHostInstanceByFiber: _4,
        findFiberByHostInstance: t || j4,
        // React Refresh
        findHostInstancesForRefresh: S4,
        scheduleRefresh: b4,
        scheduleRoot: C4,
        setRefreshHandler: y4,
        // Enables DevTools to append owner stacks to error messages in DEV mode.
        getCurrentFiber: U4,
        // Enables DevTools to detect reconciler version rather than renderer version
        // which may not match for third party renderers.
        reconcilerVersion: sE
      });
    }
    var WI = typeof reportError == "function" ? (
      // In modern browsers, reportError will dispatch an error event,
      // emulating an uncaught JavaScript error.
      reportError
    ) : function(e) {
      console.error(e);
    };
    function vE(e) {
      this._internalRoot = e;
    }
    M0.prototype.render = vE.prototype.render = function(e) {
      var t = this._internalRoot;
      if (t === null)
        throw new Error("Cannot update an unmounted root.");
      {
        typeof arguments[1] == "function" ? h("render(...): does not support the second callback argument. To execute a side effect after rendering, declare it in a component body with useEffect().") : L0(arguments[1]) ? h("You passed a container to the second argument of root.render(...). You don't need to pass it again since you already passed it to create the root.") : typeof arguments[1] < "u" && h("You passed a second argument to root.render(...) but it only accepts one argument.");
        var o = t.containerInfo;
        if (o.nodeType !== Dn) {
          var l = II(t.current);
          l && l.parentNode !== o && h("render(...): It looks like the React-rendered content of the root container was removed without using React. This is not supported and will cause errors. Instead, call root.unmount() to empty a root's container.");
        }
      }
      Fh(e, t, null, null);
    }, M0.prototype.unmount = vE.prototype.unmount = function() {
      typeof arguments[0] == "function" && h("unmount(...): does not support a callback argument. To execute a side effect after rendering, declare it in a component body with useEffect().");
      var e = this._internalRoot;
      if (e !== null) {
        this._internalRoot = null;
        var t = e.containerInfo;
        rI() && h("Attempted to synchronously unmount a root while React was already rendering. React cannot finish unmounting the root until the current render has completed, which may lead to a race condition."), Mu(function() {
          Fh(null, e, null, null);
        }), Ww(t);
      }
    };
    function X4(e, t) {
      if (!L0(e))
        throw new Error("createRoot(...): Target container is not a DOM element.");
      XI(e);
      var o = !1, l = !1, d = "", m = WI;
      t != null && (t.hydrate ? v("hydrate through createRoot is deprecated. Use ReactDOMClient.hydrateRoot(container, <App />) instead.") : typeof t == "object" && t !== null && t.$$typeof === Xu && h(`You passed a JSX element to createRoot. You probably meant to call root.render instead. Example usage:

  let root = createRoot(domContainer);
  root.render(<App />);`), t.unstable_strictMode === !0 && (o = !0), t.identifierPrefix !== void 0 && (d = t.identifierPrefix), t.onRecoverableError !== void 0 && (m = t.onRecoverableError), t.transitionCallbacks !== void 0 && t.transitionCallbacks);
      var y = wI(e, Ay, null, o, l, d, m);
      Sy(y.current, e);
      var x = e.nodeType === Dn ? e.parentNode : e;
      return Gv(x), new vE(y);
    }
    function M0(e) {
      this._internalRoot = e;
    }
    function Z4(e) {
      e && mC(e);
    }
    M0.prototype.unstable_scheduleHydration = Z4;
    function Y4(e, t, o) {
      if (!L0(e))
        throw new Error("hydrateRoot(...): Target container is not a DOM element.");
      XI(e), t === void 0 && h("Must provide initial children as second argument to hydrateRoot. Example usage: hydrateRoot(domContainer, <App />)");
      var l = o ?? null, d = o != null && o.hydratedSources || null, m = !1, y = !1, x = "", T = WI;
      o != null && (o.unstable_strictMode === !0 && (m = !0), o.identifierPrefix !== void 0 && (x = o.identifierPrefix), o.onRecoverableError !== void 0 && (T = o.onRecoverableError));
      var O = TI(t, null, e, Ay, l, m, y, x, T);
      if (Sy(O.current, e), Gv(e), d)
        for (var N = 0; N < d.length; N++) {
          var j = d[N];
          KM(O, j);
        }
      return new M0(O);
    }
    function L0(e) {
      return !!(e && (e.nodeType === Vr || e.nodeType === ei || e.nodeType === eu || !ge));
    }
    function Bh(e) {
      return !!(e && (e.nodeType === Vr || e.nodeType === ei || e.nodeType === eu || e.nodeType === Dn && e.nodeValue === " react-mount-point-unstable "));
    }
    function XI(e) {
      e.nodeType === Vr && e.tagName && e.tagName.toUpperCase() === "BODY" && h("createRoot(): Creating roots directly with document.body is discouraged, since its children are often manipulated by third-party scripts and browser extensions. This may lead to subtle reconciliation issues. Try using a container element created for your app."), Kv(e) && (e._reactRootContainer ? h("You are calling ReactDOMClient.createRoot() on a container that was previously passed to ReactDOM.render(). This is not supported.") : h("You are calling ReactDOMClient.createRoot() on a container that has already been passed to createRoot() before. Instead, call root.render() on the existing root instead if you want to update it."));
    }
    var $4 = u.ReactCurrentOwner, ZI;
    ZI = function(e) {
      if (e._reactRootContainer && e.nodeType !== Dn) {
        var t = II(e._reactRootContainer.current);
        t && t.parentNode !== e && h("render(...): It looks like the React-rendered content of this container was removed without using React. This is not supported and will cause errors. Instead, call ReactDOM.unmountComponentAtNode to empty a container.");
      }
      var o = !!e._reactRootContainer, l = hE(e), d = !!(l && Hl(l));
      d && !o && h("render(...): Replacing React-rendered children with a new root component. If you intended to update the children of this node, you should instead have the existing children update their state and render the new components instead of calling ReactDOM.render."), e.nodeType === Vr && e.tagName && e.tagName.toUpperCase() === "BODY" && h("render(): Rendering components directly into document.body is discouraged, since its children are often manipulated by third-party scripts and browser extensions. This may lead to subtle reconciliation issues. Try rendering into a container element created for your app.");
    };
    function hE(e) {
      return e ? e.nodeType === ei ? e.documentElement : e.firstChild : null;
    }
    function YI() {
    }
    function Q4(e, t, o, l, d) {
      if (d) {
        if (typeof l == "function") {
          var m = l;
          l = function() {
            var G = k0(y);
            m.call(G);
          };
        }
        var y = TI(
          t,
          l,
          e,
          Gl,
          null,
          // hydrationCallbacks
          !1,
          // isStrictMode
          !1,
          // concurrentUpdatesByDefaultOverride,
          "",
          // identifierPrefix
          YI
        );
        e._reactRootContainer = y, Sy(y.current, e);
        var x = e.nodeType === Dn ? e.parentNode : e;
        return Gv(x), Mu(), y;
      } else {
        for (var T; T = e.lastChild; )
          e.removeChild(T);
        if (typeof l == "function") {
          var O = l;
          l = function() {
            var G = k0(N);
            O.call(G);
          };
        }
        var N = wI(
          e,
          Gl,
          null,
          // hydrationCallbacks
          !1,
          // isStrictMode
          !1,
          // concurrentUpdatesByDefaultOverride,
          "",
          // identifierPrefix
          YI
        );
        e._reactRootContainer = N, Sy(N.current, e);
        var j = e.nodeType === Dn ? e.parentNode : e;
        return Gv(j), Mu(function() {
          Fh(t, N, o, l);
        }), N;
      }
    }
    function J4(e, t) {
      e !== null && typeof e != "function" && h("%s(...): Expected the last optional `callback` argument to be a function. Instead received: %s.", t, e);
    }
    function F0(e, t, o, l, d) {
      ZI(o), J4(d === void 0 ? null : d, "render");
      var m = o._reactRootContainer, y;
      if (!m)
        y = Q4(o, t, e, d, l);
      else {
        if (y = m, typeof d == "function") {
          var x = d;
          d = function() {
            var T = k0(y);
            x.call(T);
          };
        }
        Fh(t, y, e, d);
      }
      return k0(y);
    }
    var $I = !1;
    function K4(e) {
      {
        $I || ($I = !0, h("findDOMNode is deprecated and will be removed in the next major release. Instead, add a ref directly to the element you want to reference. Learn more about using refs safely here: https://reactjs.org/link/strict-mode-find-node"));
        var t = $4.current;
        if (t !== null && t.stateNode !== null) {
          var o = t.stateNode._warnedAboutRefsInRender;
          o || h("%s is accessing findDOMNode inside its render(). render() should be a pure function of props and state. It should never access something that requires stale data from the previous render, such as refs. Move this logic to componentDidMount and componentDidUpdate instead.", Ut(t.type) || "A component"), t.stateNode._warnedAboutRefsInRender = !0;
        }
      }
      return e == null ? null : e.nodeType === Vr ? e : B4(e, "findDOMNode");
    }
    function q4(e, t, o) {
      if (h("ReactDOM.hydrate is no longer supported in React 18. Use hydrateRoot instead. Until you switch to the new API, your app will behave as if it's running React 17. Learn more: https://reactjs.org/link/switch-to-createroot"), !Bh(t))
        throw new Error("Target container is not a DOM element.");
      {
        var l = Kv(t) && t._reactRootContainer === void 0;
        l && h("You are calling ReactDOM.hydrate() on a container that was previously passed to ReactDOMClient.createRoot(). This is not supported. Did you mean to call hydrateRoot(container, element)?");
      }
      return F0(null, e, t, !0, o);
    }
    function eB(e, t, o) {
      if (h("ReactDOM.render is no longer supported in React 18. Use createRoot instead. Until you switch to the new API, your app will behave as if it's running React 17. Learn more: https://reactjs.org/link/switch-to-createroot"), !Bh(t))
        throw new Error("Target container is not a DOM element.");
      {
        var l = Kv(t) && t._reactRootContainer === void 0;
        l && h("You are calling ReactDOM.render() on a container that was previously passed to ReactDOMClient.createRoot(). This is not supported. Did you mean to call root.render(element)?");
      }
      return F0(null, e, t, !1, o);
    }
    function tB(e, t, o, l) {
      if (h("ReactDOM.unstable_renderSubtreeIntoContainer() is no longer supported in React 18. Consider using a portal instead. Until you switch to the createRoot API, your app will behave as if it's running React 17. Learn more: https://reactjs.org/link/switch-to-createroot"), !Bh(o))
        throw new Error("Target container is not a DOM element.");
      if (e == null || !Ps(e))
        throw new Error("parentComponent must be a valid React Component");
      return F0(e, t, o, !1, l);
    }
    var QI = !1;
    function nB(e) {
      if (QI || (QI = !0, h("unmountComponentAtNode is deprecated and will be removed in the next major release. Switch to the createRoot API. Learn more: https://reactjs.org/link/switch-to-createroot")), !Bh(e))
        throw new Error("unmountComponentAtNode(...): Target container is not a DOM element.");
      {
        var t = Kv(e) && e._reactRootContainer === void 0;
        t && h("You are calling ReactDOM.unmountComponentAtNode() on a container that was previously passed to ReactDOMClient.createRoot(). This is not supported. Did you mean to call root.unmount()?");
      }
      if (e._reactRootContainer) {
        {
          var o = hE(e), l = o && !Hl(o);
          l && h("unmountComponentAtNode(): The node you're attempting to unmount was rendered by another copy of React.");
        }
        return Mu(function() {
          F0(null, null, e, !1, function() {
            e._reactRootContainer = null, Ww(e);
          });
        }), !0;
      } else {
        {
          var d = hE(e), m = !!(d && Hl(d)), y = e.nodeType === Vr && Bh(e.parentNode) && !!e.parentNode._reactRootContainer;
          m && h("unmountComponentAtNode(): The node you're attempting to unmount was rendered by React and is not a top-level container. %s", y ? "You may have accidentally passed in a React root node instead of its container." : "Instead, have the parent component update its state and rerender in order to remove this component.");
        }
        return !1;
      }
    }
    Tl(V4), vC(P4), Kf(H4), jg(Na), Ug(yr), (typeof Map != "function" || // $FlowIssue Flow incorrectly thinks Map has no prototype
    Map.prototype == null || typeof Map.prototype.forEach != "function" || typeof Set != "function" || // $FlowIssue Flow incorrectly thinks Set has no prototype
    Set.prototype == null || typeof Set.prototype.clear != "function" || typeof Set.prototype.forEach != "function") && h("React depends on Map and Set built-in types. Make sure that you load a polyfill in older browsers. https://reactjs.org/link/react-polyfills"), bg(ak), hf(XS, YF, Mu);
    function rB(e, t) {
      var o = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : null;
      if (!L0(t))
        throw new Error("Target container is not a DOM element.");
      return F4(e, t, null, o);
    }
    function aB(e, t, o, l) {
      return tB(e, t, o, l);
    }
    var mE = {
      usingClientEntryPoint: !1,
      // Keep in sync with ReactTestUtils.js.
      // This is an array for better minification.
      Events: [Hl, Sd, Ey, vf, Fs, XS]
    };
    function iB(e, t) {
      return mE.usingClientEntryPoint || h('You are importing createRoot from "react-dom" which is not supported. You should instead import it from "react-dom/client".'), X4(e, t);
    }
    function oB(e, t, o) {
      return mE.usingClientEntryPoint || h('You are importing hydrateRoot from "react-dom" which is not supported. You should instead import it from "react-dom/client".'), Y4(e, t, o);
    }
    function uB(e) {
      return rI() && h("flushSync was called from inside a lifecycle method. React cannot flush when React is already rendering. Consider moving this call to a scheduler task or micro task."), Mu(e);
    }
    var lB = W4({
      findFiberByHostInstance: fc,
      bundleType: 1,
      version: sE,
      rendererPackageName: "react-dom"
    });
    if (!lB && Ot && window.top === window.self && (navigator.userAgent.indexOf("Chrome") > -1 && navigator.userAgent.indexOf("Edge") === -1 || navigator.userAgent.indexOf("Firefox") > -1)) {
      var JI = window.location.protocol;
      /^(https?|file):$/.test(JI) && console.info("%cDownload the React DevTools for a better development experience: https://reactjs.org/link/react-devtools" + (JI === "file:" ? `
You might need to use a local HTTP server (instead of file://): https://reactjs.org/link/react-devtools-faq` : ""), "font-weight:bold");
    }
    _a.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED = mE, _a.createPortal = rB, _a.createRoot = iB, _a.findDOMNode = K4, _a.flushSync = uB, _a.hydrate = q4, _a.hydrateRoot = oB, _a.render = eB, _a.unmountComponentAtNode = nB, _a.unstable_batchedUpdates = XS, _a.unstable_renderSubtreeIntoContainer = aB, _a.version = sE, typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ < "u" && typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop == "function" && __REACT_DEVTOOLS_GLOBAL_HOOK__.registerInternalModuleStop(new Error());
  }()), _a;
}
function oD() {
  if (!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__ > "u" || typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE != "function")) {
    if (true)
      throw new Error("^_^");
    try {
      __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(oD);
    } catch (a) {
      console.error(a);
    }
  }
}
 false ? (0) : ZE.exports = tV();
var uD = ZE.exports;
const $d = /* @__PURE__ */ ym(uD), vA = {
  disabled: !1
};
var nV =  true ? gt.oneOfType([gt.number, gt.shape({
  enter: gt.number,
  exit: gt.number,
  appear: gt.number
}).isRequired]) : 0;
 true && gt.oneOfType([gt.string, gt.shape({
  enter: gt.string,
  exit: gt.string,
  active: gt.string
}), gt.shape({
  enter: gt.string,
  enterDone: gt.string,
  enterActive: gt.string,
  exit: gt.string,
  exitDone: gt.string,
  exitActive: gt.string
})]);
const lD = react__WEBPACK_IMPORTED_MODULE_1___default().createContext(null);
var rV = function(r) {
  return r.scrollTop;
}, jh = "unmounted", ns = "exited", uo = "entering", Vu = "entered", Jh = "exiting", Wu = /* @__PURE__ */ function(a) {
  zB(r, a);
  function r(f, c) {
    var v;
    v = a.call(this, f, c) || this;
    var h = c, C = h && !h.isMounting ? f.enter : f.appear, b;
    return v.appearStatus = null, f.in ? C ? (b = ns, v.appearStatus = uo) : b = Vu : f.unmountOnExit || f.mountOnEnter ? b = jh : b = ns, v.state = {
      status: b
    }, v.nextCallback = null, v;
  }
  r.getDerivedStateFromProps = function(c, v) {
    var h = c.in;
    return h && v.status === jh ? {
      status: ns
    } : null;
  };
  var u = r.prototype;
  return u.componentDidMount = function() {
    this.updateStatus(!0, this.appearStatus);
  }, u.componentDidUpdate = function(c) {
    var v = null;
    if (c !== this.props) {
      var h = this.state.status;
      this.props.in ? h !== uo && h !== Vu && (v = uo) : (h === uo || h === Vu) && (v = Jh);
    }
    this.updateStatus(!1, v);
  }, u.componentWillUnmount = function() {
    this.cancelNextCallback();
  }, u.getTimeouts = function() {
    var c = this.props.timeout, v, h, C;
    return v = h = C = c, c != null && typeof c != "number" && (v = c.exit, h = c.enter, C = c.appear !== void 0 ? c.appear : h), {
      exit: v,
      enter: h,
      appear: C
    };
  }, u.updateStatus = function(c, v) {
    if (c === void 0 && (c = !1), v !== null)
      if (this.cancelNextCallback(), v === uo) {
        if (this.props.unmountOnExit || this.props.mountOnEnter) {
          var h = this.props.nodeRef ? this.props.nodeRef.current : $d.findDOMNode(this);
          h && rV(h);
        }
        this.performEnter(c);
      } else
        this.performExit();
    else
      this.props.unmountOnExit && this.state.status === ns && this.setState({
        status: jh
      });
  }, u.performEnter = function(c) {
    var v = this, h = this.props.enter, C = this.context ? this.context.isMounting : c, b = this.props.nodeRef ? [C] : [$d.findDOMNode(this), C], E = b[0], w = b[1], R = this.getTimeouts(), A = C ? R.appear : R.enter;
    if (!c && !h || vA.disabled) {
      this.safeSetState({
        status: Vu
      }, function() {
        v.props.onEntered(E);
      });
      return;
    }
    this.props.onEnter(E, w), this.safeSetState({
      status: uo
    }, function() {
      v.props.onEntering(E, w), v.onTransitionEnd(A, function() {
        v.safeSetState({
          status: Vu
        }, function() {
          v.props.onEntered(E, w);
        });
      });
    });
  }, u.performExit = function() {
    var c = this, v = this.props.exit, h = this.getTimeouts(), C = this.props.nodeRef ? void 0 : $d.findDOMNode(this);
    if (!v || vA.disabled) {
      this.safeSetState({
        status: ns
      }, function() {
        c.props.onExited(C);
      });
      return;
    }
    this.props.onExit(C), this.safeSetState({
      status: Jh
    }, function() {
      c.props.onExiting(C), c.onTransitionEnd(h.exit, function() {
        c.safeSetState({
          status: ns
        }, function() {
          c.props.onExited(C);
        });
      });
    });
  }, u.cancelNextCallback = function() {
    this.nextCallback !== null && (this.nextCallback.cancel(), this.nextCallback = null);
  }, u.safeSetState = function(c, v) {
    v = this.setNextCallback(v), this.setState(c, v);
  }, u.setNextCallback = function(c) {
    var v = this, h = !0;
    return this.nextCallback = function(C) {
      h && (h = !1, v.nextCallback = null, c(C));
    }, this.nextCallback.cancel = function() {
      h = !1;
    }, this.nextCallback;
  }, u.onTransitionEnd = function(c, v) {
    this.setNextCallback(v);
    var h = this.props.nodeRef ? this.props.nodeRef.current : $d.findDOMNode(this), C = c == null && !this.props.addEndListener;
    if (!h || C) {
      setTimeout(this.nextCallback, 0);
      return;
    }
    if (this.props.addEndListener) {
      var b = this.props.nodeRef ? [this.nextCallback] : [h, this.nextCallback], E = b[0], w = b[1];
      this.props.addEndListener(E, w);
    }
    c != null && setTimeout(this.nextCallback, c);
  }, u.render = function() {
    var c = this.state.status;
    if (c === jh)
      return null;
    var v = this.props, h = v.children;
    v.in, v.mountOnEnter, v.unmountOnExit, v.appear, v.enter, v.exit, v.timeout, v.addEndListener, v.onEnter, v.onEntering, v.onEntered, v.onExit, v.onExiting, v.onExited, v.nodeRef;
    var C = Ax(v, ["children", "in", "mountOnEnter", "unmountOnExit", "appear", "enter", "exit", "timeout", "addEndListener", "onEnter", "onEntering", "onEntered", "onExit", "onExiting", "onExited", "nodeRef"]);
    return (
      // allows for nested Transitions
      /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().createElement(lD.Provider, {
        value: null
      }, typeof h == "function" ? h(c, C) : react__WEBPACK_IMPORTED_MODULE_1___default().cloneElement(react__WEBPACK_IMPORTED_MODULE_1___default().Children.only(h), C))
    );
  }, r;
}((react__WEBPACK_IMPORTED_MODULE_1___default().Component));
Wu.contextType = lD;
Wu.propTypes =  true ? {
  /**
   * A React reference to DOM element that need to transition:
   * https://stackoverflow.com/a/51127130/4671932
   *
   *   - When `nodeRef` prop is used, `node` is not passed to callback functions
   *      (e.g. `onEnter`) because user already has direct access to the node.
   *   - When changing `key` prop of `Transition` in a `TransitionGroup` a new
   *     `nodeRef` need to be provided to `Transition` with changed `key` prop
   *     (see
   *     [test/CSSTransition-test.js](https://github.com/reactjs/react-transition-group/blob/13435f897b3ab71f6e19d724f145596f5910581c/test/CSSTransition-test.js#L362-L437)).
   */
  nodeRef: gt.shape({
    current: typeof Element > "u" ? gt.any : function(a, r, u, f, c, v) {
      var h = a[r];
      return gt.instanceOf(h && "ownerDocument" in h ? h.ownerDocument.defaultView.Element : Element)(a, r, u, f, c, v);
    }
  }),
  /**
   * A `function` child can be used instead of a React element. This function is
   * called with the current transition status (`'entering'`, `'entered'`,
   * `'exiting'`, `'exited'`), which can be used to apply context
   * specific props to a component.
   *
   * ```jsx
   * <Transition in={this.state.in} timeout={150}>
   *   {state => (
   *     <MyComponent className={`fade fade-${state}`} />
   *   )}
   * </Transition>
   * ```
   */
  children: gt.oneOfType([gt.func.isRequired, gt.element.isRequired]).isRequired,
  /**
   * Show the component; triggers the enter or exit states
   */
  in: gt.bool,
  /**
   * By default the child component is mounted immediately along with
   * the parent `Transition` component. If you want to "lazy mount" the component on the
   * first `in={true}` you can set `mountOnEnter`. After the first enter transition the component will stay
   * mounted, even on "exited", unless you also specify `unmountOnExit`.
   */
  mountOnEnter: gt.bool,
  /**
   * By default the child component stays mounted after it reaches the `'exited'` state.
   * Set `unmountOnExit` if you'd prefer to unmount the component after it finishes exiting.
   */
  unmountOnExit: gt.bool,
  /**
   * By default the child component does not perform the enter transition when
   * it first mounts, regardless of the value of `in`. If you want this
   * behavior, set both `appear` and `in` to `true`.
   *
   * > **Note**: there are no special appear states like `appearing`/`appeared`, this prop
   * > only adds an additional enter transition. However, in the
   * > `<CSSTransition>` component that first enter transition does result in
   * > additional `.appear-*` classes, that way you can choose to style it
   * > differently.
   */
  appear: gt.bool,
  /**
   * Enable or disable enter transitions.
   */
  enter: gt.bool,
  /**
   * Enable or disable exit transitions.
   */
  exit: gt.bool,
  /**
   * The duration of the transition, in milliseconds.
   * Required unless `addEndListener` is provided.
   *
   * You may specify a single timeout for all transitions:
   *
   * ```jsx
   * timeout={500}
   * ```
   *
   * or individually:
   *
   * ```jsx
   * timeout={{
   *  appear: 500,
   *  enter: 300,
   *  exit: 500,
   * }}
   * ```
   *
   * - `appear` defaults to the value of `enter`
   * - `enter` defaults to `0`
   * - `exit` defaults to `0`
   *
   * @type {number | { enter?: number, exit?: number, appear?: number }}
   */
  timeout: function(r) {
    var u = nV;
    r.addEndListener || (u = u.isRequired);
    for (var f = arguments.length, c = new Array(f > 1 ? f - 1 : 0), v = 1; v < f; v++)
      c[v - 1] = arguments[v];
    return u.apply(void 0, [r].concat(c));
  },
  /**
   * Add a custom transition end trigger. Called with the transitioning
   * DOM node and a `done` callback. Allows for more fine grained transition end
   * logic. Timeouts are still used as a fallback if provided.
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * ```jsx
   * addEndListener={(node, done) => {
   *   // use the css transitionend event to mark the finish of a transition
   *   node.addEventListener('transitionend', done, false);
   * }}
   * ```
   */
  addEndListener: gt.func,
  /**
   * Callback fired before the "entering" status is applied. An extra parameter
   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * @type Function(node: HtmlElement, isAppearing: bool) -> void
   */
  onEnter: gt.func,
  /**
   * Callback fired after the "entering" status is applied. An extra parameter
   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * @type Function(node: HtmlElement, isAppearing: bool)
   */
  onEntering: gt.func,
  /**
   * Callback fired after the "entered" status is applied. An extra parameter
   * `isAppearing` is supplied to indicate if the enter stage is occurring on the initial mount
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * @type Function(node: HtmlElement, isAppearing: bool) -> void
   */
  onEntered: gt.func,
  /**
   * Callback fired before the "exiting" status is applied.
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * @type Function(node: HtmlElement) -> void
   */
  onExit: gt.func,
  /**
   * Callback fired after the "exiting" status is applied.
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed.
   *
   * @type Function(node: HtmlElement) -> void
   */
  onExiting: gt.func,
  /**
   * Callback fired after the "exited" status is applied.
   *
   * **Note**: when `nodeRef` prop is passed, `node` is not passed
   *
   * @type Function(node: HtmlElement) -> void
   */
  onExited: gt.func
} : 0;
function Xd() {
}
Wu.defaultProps = {
  in: !1,
  mountOnEnter: !1,
  unmountOnExit: !1,
  appear: !1,
  enter: !0,
  exit: !0,
  onEnter: Xd,
  onEntering: Xd,
  onEntered: Xd,
  onExit: Xd,
  onExiting: Xd,
  onExited: Xd
};
Wu.UNMOUNTED = jh;
Wu.EXITED = ns;
Wu.ENTERING = uo;
Wu.ENTERED = Vu;
Wu.EXITING = Jh;
function Ib(a) {
  return a && a.ownerDocument || document;
}
function aV(a) {
  var r = Ib(a);
  return r && r.defaultView || window;
}
function iV(a, r) {
  return aV(a).getComputedStyle(a, r);
}
var oV = /([A-Z])/g;
function uV(a) {
  return a.replace(oV, "-$1").toLowerCase();
}
var lV = /^ms-/;
function H0(a) {
  return uV(a).replace(lV, "-ms-");
}
var sV = /^((translate|rotate|scale)(X|Y|Z|3d)?|matrix(3d)?|perspective|skew(X|Y)?)$/i;
function cV(a) {
  return !!(a && sV.test(a));
}
function Pu(a, r) {
  var u = "", f = "";
  if (typeof r == "string")
    return a.style.getPropertyValue(H0(r)) || iV(a).getPropertyValue(H0(r));
  Object.keys(r).forEach(function(c) {
    var v = r[c];
    !v && v !== 0 ? a.style.removeProperty(H0(c)) : cV(c) ? f += c + "(" + v + ") " : u += H0(c) + ": " + v + ";";
  }), f && (u += "transform: " + f + ";"), a.style.cssText += ";" + u;
}
const Ab = !!(typeof window < "u" && window.document && window.document.createElement);
var YE = !1, $E = !1;
try {
  var RE = {
    get passive() {
      return YE = !0;
    },
    get once() {
      return $E = YE = !0;
    }
  };
  Ab && (window.addEventListener("test", RE, RE), window.removeEventListener("test", RE, !0));
} catch {
}
function sD(a, r, u, f) {
  if (f && typeof f != "boolean" && !$E) {
    var c = f.once, v = f.capture, h = u;
    !$E && c && (h = u.__once || function C(b) {
      this.removeEventListener(r, C, v), u.call(this, b);
    }, u.__once = h), a.addEventListener(r, h, YE ? f : v);
  }
  a.addEventListener(r, u, f);
}
function fV(a, r, u, f) {
  var c = f && typeof f != "boolean" ? f.capture : f;
  a.removeEventListener(r, u, c), u.__once && a.removeEventListener(r, u.__once, c);
}
function cs(a, r, u, f) {
  return sD(a, r, u, f), function() {
    fV(a, r, u, f);
  };
}
function dV(a, r, u, f) {
  if (f === void 0 && (f = !0), a) {
    var c = document.createEvent("HTMLEvents");
    c.initEvent(r, u, f), a.dispatchEvent(c);
  }
}
function pV(a) {
  var r = Pu(a, "transitionDuration") || "", u = r.indexOf("ms") === -1 ? 1e3 : 1;
  return parseFloat(r) * u;
}
function vV(a, r, u) {
  u === void 0 && (u = 5);
  var f = !1, c = setTimeout(function() {
    f || dV(a, "transitionend", !0);
  }, r + u), v = cs(a, "transitionend", function() {
    f = !0;
  }, {
    once: !0
  });
  return function() {
    clearTimeout(c), v();
  };
}
function hV(a, r, u, f) {
  u == null && (u = pV(a) || 0);
  var c = vV(a, u, f), v = cs(a, "transitionend", r);
  return function() {
    c(), v();
  };
}
function hA(a, r) {
  const u = Pu(a, r) || "", f = u.indexOf("ms") === -1 ? 1e3 : 1;
  return parseFloat(u) * f;
}
function kx(a, r) {
  const u = hA(a, "transitionDuration"), f = hA(a, "transitionDelay"), c = hV(a, (v) => {
    v.target === a && (c(), r(v));
  }, u + f);
}
function cD(a) {
  a.offsetHeight;
}
const mA = (a) => !a || typeof a == "function" ? a : (r) => {
  a.current = r;
};
function mV(a, r) {
  const u = mA(a), f = mA(r);
  return (c) => {
    u && u(c), f && f(c);
  };
}
function jc(a, r) {
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => mV(a, r), [a, r]);
}
function gV(a) {
  return a && "setState" in a ? $d.findDOMNode(a) : a ?? null;
}
const Mx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().forwardRef(({
  onEnter: a,
  onEntering: r,
  onEntered: u,
  onExit: f,
  onExiting: c,
  onExited: v,
  addEndListener: h,
  children: C,
  childRef: b,
  ...E
}, w) => {
  const R = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), A = jc(R, b), D = (Q) => {
    A(gV(Q));
  }, L = (Q) => (P) => {
    Q && R.current && Q(R.current, P);
  }, M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(a), [a]), k = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(r), [r]), B = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(u), [u]), Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(f), [f]), z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(c), [c]), Z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(v), [v]), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(h), [h]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Wu, {
    ref: w,
    ...E,
    onEnter: M,
    onEntered: B,
    onEntering: k,
    onExit: Y,
    onExited: Z,
    onExiting: z,
    addEndListener: V,
    nodeRef: R,
    children: typeof C == "function" ? (Q, P) => (
      // TODO: Types for RTG missing innerProps, so need to cast.
      C(Q, {
        ...P,
        ref: D
      })
    ) : /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().cloneElement(C, {
      ref: D
    })
  });
}), yV = {
  [uo]: "show",
  [Vu]: "show"
}, ab = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  children: r,
  transitionClasses: u = {},
  onEnter: f,
  ...c
}, v) => {
  const h = {
    in: !1,
    timeout: 300,
    mountOnEnter: !1,
    unmountOnExit: !1,
    appear: !1,
    ...c
  }, C = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((b, E) => {
    cD(b), f == null || f(b, E);
  }, [f]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Mx, {
    ref: v,
    addEndListener: kx,
    ...h,
    onEnter: C,
    childRef: r.ref,
    children: (b, E) => /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.cloneElement(r, {
      ...E,
      className: Ke("fade", a, r.props.className, yV[b], u[b])
    })
  });
});
ab.displayName = "Fade";
const bV = {
  /** An accessible label indicating the relevant information about the Close Button. */
  "aria-label": gt.string,
  /** A callback fired after the Close Button is clicked. */
  onClick: gt.func,
  /**
   * Render different color variant for the button.
   *
   * Omitting this will render the default dark color.
   */
  variant: gt.oneOf(["white"])
}, Ob = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  variant: r,
  "aria-label": u = "Close",
  ...f
}, c) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("button", {
  ref: c,
  type: "button",
  className: Ke("btn-close", r && `btn-close-${r}`, a),
  "aria-label": u,
  ...f
}));
Ob.displayName = "CloseButton";
Ob.propTypes = bV;
const fD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  const {
    bsPrefix: u,
    show: f = !0,
    closeLabel: c = "Close alert",
    closeVariant: v,
    className: h,
    children: C,
    variant: b = "primary",
    onClose: E,
    dismissible: w,
    transition: R = ab,
    ...A
  } = Ox(a, {
    show: "onClose"
  }), D = ft(u, "alert"), L = Un((B) => {
    E && E(!1, B);
  }), M = R === !0 ? ab : R, k = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)("div", {
    role: "alert",
    ...M ? void 0 : A,
    ref: r,
    className: Ke(h, D, b && `${D}-${b}`, w && `${D}-dismissible`),
    children: [w && /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ob, {
      onClick: L,
      "aria-label": c,
      variant: v
    }), C]
  });
  return M ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(M, {
    unmountOnExit: !0,
    ...A,
    ref: void 0,
    in: f,
    children: k
  }) : f ? k : null;
});
fD.displayName = "Alert";
const Z0 = Object.assign(fD, {
  Link: aD,
  Heading: tD
}), zc = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  fluid: r = !1,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: u = "div",
  className: f,
  ...c
}, v) => {
  const h = ft(a, "container"), C = typeof r == "string" ? `-${r}` : "-fluid";
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
    ref: v,
    ...c,
    className: Ke(f, r ? `${h}${C}` : h)
  });
});
zc.displayName = "Container";
const Lx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: u = "div",
  ...f
}, c) => {
  const v = ft(a, "row"), h = Ex(), C = xx(), b = `${v}-cols`, E = [];
  return h.forEach((w) => {
    const R = f[w];
    delete f[w];
    let A;
    R != null && typeof R == "object" ? {
      cols: A
    } = R : A = R;
    const D = w !== C ? `-${w}` : "";
    A != null && E.push(`${b}${D}-${A}`);
  }), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
    ref: c,
    ...f,
    className: Ke(r, v, ...E)
  });
});
Lx.displayName = "Row";
function CV(a, r = _O, u = jO) {
  const f = [];
  return Object.entries(a).forEach(([c, v]) => {
    v != null && (typeof v == "object" ? r.forEach((h) => {
      const C = v[h];
      if (C != null) {
        const b = h !== u ? `-${h}` : "";
        f.push(`${c}${b}-${C}`);
      }
    }) : f.push(`${c}-${v}`));
  }), f;
}
const Hu = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  as: a = "div",
  bsPrefix: r,
  className: u,
  direction: f,
  gap: c,
  ...v
}, h) => {
  r = ft(r, f === "horizontal" ? "hstack" : "vstack");
  const C = Ex(), b = xx();
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(a, {
    ...v,
    ref: h,
    className: Ke(u, r, ...CV({
      gap: c
    }, C, b))
  });
});
Hu.displayName = "Stack";
const zu = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  as: a,
  bsPrefix: r,
  variant: u = "primary",
  size: f,
  active: c = !1,
  disabled: v = !1,
  className: h,
  ...C
}, b) => {
  const E = ft(r, "btn"), [w, {
    tagName: R
  }] = Dx({
    tagName: a,
    disabled: v,
    ...C
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(R, {
    ...w,
    ...C,
    ref: b,
    disabled: v,
    className: Ke(h, E, c && "active", u && `${E}-${u}`, f && `${E}-${f}`, C.href && v && "disabled")
  });
});
zu.displayName = "Button";
const Fx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  ...f
}, c) => (r = ft(r, "card-body"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
Fx.displayName = "CardBody";
const dD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  ...f
}, c) => (r = ft(r, "card-footer"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
dD.displayName = "CardFooter";
const pD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null);
pD.displayName = "CardHeaderContext";
const vD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: u = "div",
  ...f
}, c) => {
  const v = ft(a, "card-header"), h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    cardHeaderBsPrefix: v
  }), [v]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(pD.Provider, {
    value: h,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
      ref: c,
      ...f,
      className: Ke(r, v)
    })
  });
});
vD.displayName = "CardHeader";
const hD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  ({
    bsPrefix: a,
    className: r,
    variant: u,
    as: f = "img",
    ...c
  }, v) => {
    const h = ft(a, "card-img");
    return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(f, {
      ref: v,
      className: Ke(u ? `${h}-${u}` : h, r),
      ...c
    });
  }
);
hD.displayName = "CardImg";
const mD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  ...f
}, c) => (r = ft(r, "card-img-overlay"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
mD.displayName = "CardImgOverlay";
const gD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "a",
  ...f
}, c) => (r = ft(r, "card-link"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
gD.displayName = "CardLink";
const SV = Rb("h6"), yD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = SV,
  ...f
}, c) => (r = ft(r, "card-subtitle"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
yD.displayName = "CardSubtitle";
const bD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "p",
  ...f
}, c) => (r = ft(r, "card-text"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
bD.displayName = "CardText";
const EV = Rb("h5"), CD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = EV,
  ...f
}, c) => (r = ft(r, "card-title"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
CD.displayName = "CardTitle";
const SD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  bg: u,
  text: f,
  border: c,
  body: v = !1,
  children: h,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: C = "div",
  ...b
}, E) => {
  const w = ft(a, "card");
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(C, {
    ref: E,
    ...b,
    className: Ke(r, w, u && `bg-${u}`, f && `text-${f}`, c && `border-${c}`),
    children: v ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Fx, {
      children: h
    }) : h
  });
});
SD.displayName = "Card";
const Ic = Object.assign(SD, {
  Img: hD,
  Title: CD,
  Subtitle: yD,
  Body: Fx,
  Link: gD,
  Text: bD,
  Header: vD,
  Footer: dD,
  ImgOverlay: mD
}), Bx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  active: a = !1,
  disabled: r = !1,
  className: u,
  style: f,
  activeLabel: c = "(current)",
  children: v,
  linkStyle: h,
  linkClassName: C,
  as: b = bm,
  ...E
}, w) => {
  const R = a || r ? "span" : b;
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("li", {
    ref: w,
    style: f,
    className: Ke(u, "page-item", {
      active: a,
      disabled: r
    }),
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(R, {
      className: Ke("page-link", C),
      style: h,
      ...E,
      children: [v, a && c && /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", {
        className: "visually-hidden",
        children: c
      })]
    })
  });
});
Bx.displayName = "PageItem";
const xV = Bx;
function Cm(a, r, u = a) {
  const f = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
    children: c,
    ...v
  }, h) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Bx, {
    ...v,
    ref: h,
    children: [/* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", {
      "aria-hidden": "true",
      children: c || r
    }), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", {
      className: "visually-hidden",
      children: u
    })]
  }));
  return f.displayName = a, f;
}
const wV = Cm("First", "«"), TV = Cm("Prev", "‹", "Previous"), RV = Cm("Ellipsis", "…", "More"), IV = Cm("Next", "›"), AV = Cm("Last", "»"), ED = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  size: u,
  ...f
}, c) => {
  const v = ft(a, "pagination");
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("ul", {
    ref: c,
    ...f,
    className: Ke(r, v, u && `${v}-${u}`)
  });
});
ED.displayName = "Pagination";
const z0 = Object.assign(ED, {
  First: wV,
  Prev: TV,
  Ellipsis: RV,
  Item: xV,
  Next: IV,
  Last: AV
});
var OV = { fieldSeparator: ",", decimalSeparator: ".", quoteStrings: !0, quoteCharacter: '"', showTitle: !1, title: "My Generated Report", filename: "generated", showColumnHeaders: !0, useTextFile: !1, useBom: !0, columnHeaders: [], useKeysAsHeaders: !1, boolDisplay: { true: "TRUE", false: "FALSE" }, replaceUndefinedWith: "" }, DV = `\r
`, NV = "\uFEFF", Db = (a) => Object.assign({}, OV, a);
class kV extends Error {
  constructor(r) {
    super(r), this.name = "CsvGenerationError";
  }
}
class MV extends Error {
  constructor(r) {
    super(r), this.name = "EmptyHeadersError";
  }
}
class LV extends Error {
  constructor(r) {
    super(r), this.name = "CsvDownloadEnvironmentError";
  }
}
class FV extends Error {
  constructor(r) {
    super(r), this.name = "UnsupportedDataFormatError";
  }
}
var Sp = (a) => a, Xa = (a) => a, Zh = Sp, Nb = Sp, vp = Sp, gA = Sp, yA = Sp, BV = function(a, r) {
  return r == '"' && a.indexOf('"') > -1 ? a.replace(/"/g, '""') : a;
}, VV = (a) => gA(typeof a == "object" ? a.key : a), PV = (a) => yA(typeof a == "object" ? a.displayLabel : a), HV = (a, ...r) => r.reduce((u, f) => f(u), a), zV = (a) => (r) => a.useBom ? Nb(Xa(r) + NV) : r, GV = (a) => (r) => a.showTitle ? Vx(Nb(Xa(r) + a.title))(vp("")) : r, Vx = (a) => (r) => Nb(Xa(a) + Xa(r) + DV), xD = (a) => (r, u) => _V(a)(vp(Xa(r) + Xa(u))), _V = (a) => (r) => Sp(Xa(r) + a.fieldSeparator), jV = (a, r) => (u) => {
  if (!a.showColumnHeaders)
    return u;
  if (r.length < 1)
    throw new MV("Option to show headers but none supplied. Make sure there are keys in your collection or that you've supplied headers through the config options.");
  let f = vp("");
  for (let c = 0; c < r.length; c++) {
    const v = PV(r[c]);
    f = xD(a)(f, wD(a, Xa(v)));
  }
  return f = vp(Xa(f).slice(0, -1)), Vx(u)(f);
}, UV = (a, r, u) => (f) => {
  let c = f;
  for (var v = 0; v < u.length; v++) {
    let h = vp("");
    for (let C = 0; C < r.length; C++) {
      const b = VV(r[C]), E = u[v][Xa(b)];
      h = xD(a)(h, wD(a, E));
    }
    h = vp(Xa(h).slice(0, -1)), c = Vx(c)(h);
  }
  return c;
}, WV = Xa, XV = (a) => +a === a && (!isFinite(a) || !!(a % 1)), ZV = (a, r) => {
  if (XV(r)) {
    if (a.decimalSeparator === "locale")
      return Zh(r.toLocaleString());
    if (a.decimalSeparator)
      return Zh(r.toString().replace(".", a.decimalSeparator));
  }
  return Zh(r.toString());
}, Y0 = (a, r) => {
  let u = r;
  return (a.quoteStrings || a.fieldSeparator && r.indexOf(a.fieldSeparator) > -1 || a.quoteCharacter && r.indexOf(a.quoteCharacter) > -1 || r.indexOf(`
`) > -1 || r.indexOf("\r") > -1) && (u = a.quoteCharacter + BV(r, a.quoteCharacter) + a.quoteCharacter), Zh(u);
}, YV = (a, r) => {
  const u = r ? "true" : "false";
  return Zh(a.boolDisplay[u]);
}, $V = (a, r) => typeof r > "u" && a.replaceUndefinedWith !== void 0 ? Y0(a, a.replaceUndefinedWith + "") : r === null ? Y0(a, "null") : Y0(a, ""), wD = (a, r) => {
  if (typeof r == "number")
    return ZV(a, r);
  if (typeof r == "string")
    return Y0(a, r);
  if (typeof r == "boolean" && a.boolDisplay)
    return YV(a, r);
  if (r === null || typeof r > "u")
    return $V(a, r);
  throw new FV(`
    typeof ${typeof r} isn't supported. Only number, string, boolean, null and undefined are supported.
    Please convert the data in your object to one of those before generating the CSV.
    `);
}, QV = (a) => (r) => {
  const u = Db(a), f = u.useKeysAsHeaders ? Object.keys(r[0]) : u.columnHeaders;
  let c = HV(Nb(""), zV(u), GV(u), jV(u, f), UV(u, f, r));
  if (Xa(c).length < 1)
    throw new kV("Output is empty. Is your data formatted correctly?");
  return c;
}, JV = (a) => (r) => {
  const u = Db(a), f = Xa(r), c = u.useTextFile ? "plain" : "csv";
  return new Blob([f], { type: `text/${c};charset=utf8;` });
}, KV = (a) => (r) => {
  if (!window)
    throw new LV("Downloading only supported in a browser environment.");
  const u = JV(a)(r), f = Db(a), c = f.useTextFile ? "txt" : "csv", v = `${f.filename}.${c}`, h = document.createElement("a");
  h.download = v, h.href = URL.createObjectURL(u), h.setAttribute("visibility", "hidden"), document.body.appendChild(h), h.click(), document.body.removeChild(h);
}, Sm = class {
  constructor() {
    this.listeners = /* @__PURE__ */ new Set(), this.subscribe = this.subscribe.bind(this);
  }
  subscribe(a) {
    return this.listeners.add(a), this.onSubscribe(), () => {
      this.listeners.delete(a), this.onUnsubscribe();
    };
  }
  hasListeners() {
    return this.listeners.size > 0;
  }
  onSubscribe() {
  }
  onUnsubscribe() {
  }
}, hp = typeof window > "u" || "Deno" in globalThis;
function Oi() {
}
function qV(a, r) {
  return typeof a == "function" ? a(r) : a;
}
function QE(a) {
  return typeof a == "number" && a >= 0 && a !== 1 / 0;
}
function TD(a, r) {
  return Math.max(a + (r || 0) - Date.now(), 0);
}
function Kd(a, r) {
  return typeof a == "function" ? a(r) : a;
}
function bA(a, r) {
  const {
    type: u = "all",
    exact: f,
    fetchStatus: c,
    predicate: v,
    queryKey: h,
    stale: C
  } = a;
  if (h) {
    if (f) {
      if (r.queryHash !== Px(h, r.options))
        return !1;
    } else if (!qh(r.queryKey, h))
      return !1;
  }
  if (u !== "all") {
    const b = r.isActive();
    if (u === "active" && !b || u === "inactive" && b)
      return !1;
  }
  return !(typeof C == "boolean" && r.isStale() !== C || c && c !== r.state.fetchStatus || v && !v(r));
}
function CA(a, r) {
  const { exact: u, status: f, predicate: c, mutationKey: v } = a;
  if (v) {
    if (!r.options.mutationKey)
      return !1;
    if (u) {
      if (Kh(r.options.mutationKey) !== Kh(v))
        return !1;
    } else if (!qh(r.options.mutationKey, v))
      return !1;
  }
  return !(f && r.state.status !== f || c && !c(r));
}
function Px(a, r) {
  return ((r == null ? void 0 : r.queryKeyHashFn) || Kh)(a);
}
function Kh(a) {
  return JSON.stringify(
    a,
    (r, u) => KE(u) ? Object.keys(u).sort().reduce((f, c) => (f[c] = u[c], f), {}) : u
  );
}
function qh(a, r) {
  return a === r ? !0 : typeof a != typeof r ? !1 : a && r && typeof a == "object" && typeof r == "object" ? !Object.keys(r).some((u) => !qh(a[u], r[u])) : !1;
}
function RD(a, r) {
  if (a === r)
    return a;
  const u = SA(a) && SA(r);
  if (u || KE(a) && KE(r)) {
    const f = u ? a : Object.keys(a), c = f.length, v = u ? r : Object.keys(r), h = v.length, C = u ? [] : {};
    let b = 0;
    for (let E = 0; E < h; E++) {
      const w = u ? E : v[E];
      (!u && f.includes(w) || u) && a[w] === void 0 && r[w] === void 0 ? (C[w] = void 0, b++) : (C[w] = RD(a[w], r[w]), C[w] === a[w] && a[w] !== void 0 && b++);
    }
    return c === h && b === c ? a : C;
  }
  return r;
}
function JE(a, r) {
  if (!r || Object.keys(a).length !== Object.keys(r).length)
    return !1;
  for (const u in a)
    if (a[u] !== r[u])
      return !1;
  return !0;
}
function SA(a) {
  return Array.isArray(a) && a.length === Object.keys(a).length;
}
function KE(a) {
  if (!EA(a))
    return !1;
  const r = a.constructor;
  if (r === void 0)
    return !0;
  const u = r.prototype;
  return !(!EA(u) || !u.hasOwnProperty("isPrototypeOf") || Object.getPrototypeOf(a) !== Object.prototype);
}
function EA(a) {
  return Object.prototype.toString.call(a) === "[object Object]";
}
function e9(a) {
  return new Promise((r) => {
    setTimeout(r, a);
  });
}
function qE(a, r, u) {
  return typeof u.structuralSharing == "function" ? u.structuralSharing(a, r) : u.structuralSharing !== !1 ? RD(a, r) : r;
}
function t9(a, r, u = 0) {
  const f = [...a, r];
  return u && f.length > u ? f.slice(1) : f;
}
function n9(a, r, u = 0) {
  const f = [r, ...a];
  return u && f.length > u ? f.slice(0, -1) : f;
}
var ex = Symbol(), ID = (a, r) => ( true && a.queryFn === ex && console.error(
  `Attempted to invoke queryFn when set to skipToken. This is likely a configuration error. Query hash: '${a.queryHash}'`
), !a.queryFn && (r != null && r.initialPromise) ? () => r.initialPromise : !a.queryFn || a.queryFn === ex ? () => Promise.reject(new Error(`Missing queryFn: '${a.queryHash}'`)) : a.queryFn), Oc, as, tp, DO, r9 = (DO = class extends Sm {
  constructor() {
    super();
    ut(this, Oc, void 0);
    ut(this, as, void 0);
    ut(this, tp, void 0);
    Ye(this, tp, (r) => {
      if (!hp && window.addEventListener) {
        const u = () => r();
        return window.addEventListener("visibilitychange", u, !1), () => {
          window.removeEventListener("visibilitychange", u);
        };
      }
    });
  }
  onSubscribe() {
    $(this, as) || this.setEventListener($(this, tp));
  }
  onUnsubscribe() {
    var r;
    this.hasListeners() || ((r = $(this, as)) == null || r.call(this), Ye(this, as, void 0));
  }
  setEventListener(r) {
    var u;
    Ye(this, tp, r), (u = $(this, as)) == null || u.call(this), Ye(this, as, r((f) => {
      typeof f == "boolean" ? this.setFocused(f) : this.onFocus();
    }));
  }
  setFocused(r) {
    $(this, Oc) !== r && (Ye(this, Oc, r), this.onFocus());
  }
  onFocus() {
    const r = this.isFocused();
    this.listeners.forEach((u) => {
      u(r);
    });
  }
  isFocused() {
    var r;
    return typeof $(this, Oc) == "boolean" ? $(this, Oc) : ((r = globalThis.document) == null ? void 0 : r.visibilityState) !== "hidden";
  }
}, Oc = new WeakMap(), as = new WeakMap(), tp = new WeakMap(), DO), Hx = new r9(), np, is, rp, NO, a9 = (NO = class extends Sm {
  constructor() {
    super();
    ut(this, np, !0);
    ut(this, is, void 0);
    ut(this, rp, void 0);
    Ye(this, rp, (r) => {
      if (!hp && window.addEventListener) {
        const u = () => r(!0), f = () => r(!1);
        return window.addEventListener("online", u, !1), window.addEventListener("offline", f, !1), () => {
          window.removeEventListener("online", u), window.removeEventListener("offline", f);
        };
      }
    });
  }
  onSubscribe() {
    $(this, is) || this.setEventListener($(this, rp));
  }
  onUnsubscribe() {
    var r;
    this.hasListeners() || ((r = $(this, is)) == null || r.call(this), Ye(this, is, void 0));
  }
  setEventListener(r) {
    var u;
    Ye(this, rp, r), (u = $(this, is)) == null || u.call(this), Ye(this, is, r(this.setOnline.bind(this)));
  }
  setOnline(r) {
    $(this, np) !== r && (Ye(this, np, r), this.listeners.forEach((f) => {
      f(r);
    }));
  }
  isOnline() {
    return $(this, np);
  }
}, np = new WeakMap(), is = new WeakMap(), rp = new WeakMap(), NO), ib = new a9();
function i9(a) {
  return Math.min(1e3 * 2 ** a, 3e4);
}
function AD(a) {
  return (a ?? "online") === "online" ? ib.isOnline() : !0;
}
var OD = class {
  constructor(a) {
    this.revert = a == null ? void 0 : a.revert, this.silent = a == null ? void 0 : a.silent;
  }
};
function IE(a) {
  return a instanceof OD;
}
function DD(a) {
  let r = !1, u = 0, f = !1, c, v, h;
  const C = new Promise((B, Y) => {
    v = B, h = Y;
  }), b = (B) => {
    var Y;
    f || (L(new OD(B)), (Y = a.abort) == null || Y.call(a));
  }, E = () => {
    r = !0;
  }, w = () => {
    r = !1;
  }, R = () => Hx.isFocused() && (a.networkMode === "always" || ib.isOnline()) && a.canRun(), A = () => AD(a.networkMode) && a.canRun(), D = (B) => {
    var Y;
    f || (f = !0, (Y = a.onSuccess) == null || Y.call(a, B), c == null || c(), v(B));
  }, L = (B) => {
    var Y;
    f || (f = !0, (Y = a.onError) == null || Y.call(a, B), c == null || c(), h(B));
  }, M = () => new Promise((B) => {
    var Y;
    c = (z) => {
      (f || R()) && B(z);
    }, (Y = a.onPause) == null || Y.call(a);
  }).then(() => {
    var B;
    c = void 0, f || (B = a.onContinue) == null || B.call(a);
  }), k = () => {
    if (f)
      return;
    let B;
    const Y = u === 0 ? a.initialPromise : void 0;
    try {
      B = Y ?? a.fn();
    } catch (z) {
      B = Promise.reject(z);
    }
    Promise.resolve(B).then(D).catch((z) => {
      var U;
      if (f)
        return;
      const Z = a.retry ?? (hp ? 0 : 3), V = a.retryDelay ?? i9, Q = typeof V == "function" ? V(u, z) : V, P = Z === !0 || typeof Z == "number" && u < Z || typeof Z == "function" && Z(u, z);
      if (r || !P) {
        L(z);
        return;
      }
      u++, (U = a.onFail) == null || U.call(a, u, z), e9(Q).then(() => R() ? void 0 : M()).then(() => {
        r ? L(z) : k();
      });
    });
  };
  return {
    promise: C,
    cancel: b,
    continue: () => (c == null || c(), C),
    cancelRetry: E,
    continueRetry: w,
    canStart: A,
    start: () => (A() ? k() : M().then(k), C)
  };
}
function o9() {
  let a = [], r = 0, u = (A) => {
    A();
  }, f = (A) => {
    A();
  }, c = (A) => setTimeout(A, 0);
  const v = (A) => {
    c = A;
  }, h = (A) => {
    let D;
    r++;
    try {
      D = A();
    } finally {
      r--, r || E();
    }
    return D;
  }, C = (A) => {
    r ? a.push(A) : c(() => {
      u(A);
    });
  }, b = (A) => (...D) => {
    C(() => {
      A(...D);
    });
  }, E = () => {
    const A = a;
    a = [], A.length && c(() => {
      f(() => {
        A.forEach((D) => {
          u(D);
        });
      });
    });
  };
  return {
    batch: h,
    batchCalls: b,
    schedule: C,
    setNotifyFunction: (A) => {
      u = A;
    },
    setBatchNotifyFunction: (A) => {
      f = A;
    },
    setScheduler: v
  };
}
var Lr = o9(), Dc, kO, ND = (kO = class {
  constructor() {
    ut(this, Dc, void 0);
  }
  destroy() {
    this.clearGcTimeout();
  }
  scheduleGc() {
    this.clearGcTimeout(), QE(this.gcTime) && Ye(this, Dc, setTimeout(() => {
      this.optionalRemove();
    }, this.gcTime));
  }
  updateGcTime(a) {
    this.gcTime = Math.max(
      this.gcTime || 0,
      a ?? (hp ? 1 / 0 : 5 * 60 * 1e3)
    );
  }
  clearGcTimeout() {
    $(this, Dc) && (clearTimeout($(this, Dc)), Ye(this, Dc, void 0));
  }
}, Dc = new WeakMap(), kO), ap, ip, Ai, ea, um, Nc, oo, Bu, MO, u9 = (MO = class extends ND {
  constructor(r) {
    super();
    ut(this, oo);
    ut(this, ap, void 0);
    ut(this, ip, void 0);
    ut(this, Ai, void 0);
    ut(this, ea, void 0);
    ut(this, um, void 0);
    ut(this, Nc, void 0);
    Ye(this, Nc, !1), Ye(this, um, r.defaultOptions), this.setOptions(r.options), this.observers = [], Ye(this, Ai, r.cache), this.queryKey = r.queryKey, this.queryHash = r.queryHash, Ye(this, ap, r.state || l9(this.options)), this.state = $(this, ap), this.scheduleGc();
  }
  get meta() {
    return this.options.meta;
  }
  get promise() {
    var r;
    return (r = $(this, ea)) == null ? void 0 : r.promise;
  }
  setOptions(r) {
    this.options = { ...$(this, um), ...r }, this.updateGcTime(this.options.gcTime);
  }
  optionalRemove() {
    !this.observers.length && this.state.fetchStatus === "idle" && $(this, Ai).remove(this);
  }
  setData(r, u) {
    const f = qE(this.state.data, r, this.options);
    return Mt(this, oo, Bu).call(this, {
      data: f,
      type: "success",
      dataUpdatedAt: u == null ? void 0 : u.updatedAt,
      manual: u == null ? void 0 : u.manual
    }), f;
  }
  setState(r, u) {
    Mt(this, oo, Bu).call(this, { type: "setState", state: r, setStateOptions: u });
  }
  cancel(r) {
    var f, c;
    const u = (f = $(this, ea)) == null ? void 0 : f.promise;
    return (c = $(this, ea)) == null || c.cancel(r), u ? u.then(Oi).catch(Oi) : Promise.resolve();
  }
  destroy() {
    super.destroy(), this.cancel({ silent: !0 });
  }
  reset() {
    this.destroy(), this.setState($(this, ap));
  }
  isActive() {
    return this.observers.some((r) => r.options.enabled !== !1);
  }
  isDisabled() {
    return this.getObserversCount() > 0 && !this.isActive();
  }
  isStale() {
    return this.state.isInvalidated ? !0 : this.getObserversCount() > 0 ? this.observers.some(
      (r) => r.getCurrentResult().isStale
    ) : this.state.data === void 0;
  }
  isStaleByTime(r = 0) {
    return this.state.isInvalidated || this.state.data === void 0 || !TD(this.state.dataUpdatedAt, r);
  }
  onFocus() {
    var u;
    const r = this.observers.find((f) => f.shouldFetchOnWindowFocus());
    r == null || r.refetch({ cancelRefetch: !1 }), (u = $(this, ea)) == null || u.continue();
  }
  onOnline() {
    var u;
    const r = this.observers.find((f) => f.shouldFetchOnReconnect());
    r == null || r.refetch({ cancelRefetch: !1 }), (u = $(this, ea)) == null || u.continue();
  }
  addObserver(r) {
    this.observers.includes(r) || (this.observers.push(r), this.clearGcTimeout(), $(this, Ai).notify({ type: "observerAdded", query: this, observer: r }));
  }
  removeObserver(r) {
    this.observers.includes(r) && (this.observers = this.observers.filter((u) => u !== r), this.observers.length || ($(this, ea) && ($(this, Nc) ? $(this, ea).cancel({ revert: !0 }) : $(this, ea).cancelRetry()), this.scheduleGc()), $(this, Ai).notify({ type: "observerRemoved", query: this, observer: r }));
  }
  getObserversCount() {
    return this.observers.length;
  }
  invalidate() {
    this.state.isInvalidated || Mt(this, oo, Bu).call(this, { type: "invalidate" });
  }
  fetch(r, u) {
    var b, E, w;
    if (this.state.fetchStatus !== "idle") {
      if (this.state.data !== void 0 && (u != null && u.cancelRefetch))
        this.cancel({ silent: !0 });
      else if ($(this, ea))
        return $(this, ea).continueRetry(), $(this, ea).promise;
    }
    if (r && this.setOptions(r), !this.options.queryFn) {
      const R = this.observers.find((A) => A.options.queryFn);
      R && this.setOptions(R.options);
    }
     true && (Array.isArray(this.options.queryKey) || console.error(
      "As of v4, queryKey needs to be an Array. If you are using a string like 'repoData', please change it to an Array, e.g. ['repoData']"
    ));
    const f = new AbortController(), c = (R) => {
      Object.defineProperty(R, "signal", {
        enumerable: !0,
        get: () => (Ye(this, Nc, !0), f.signal)
      });
    }, v = () => {
      const R = ID(this.options, u), A = {
        queryKey: this.queryKey,
        meta: this.meta
      };
      return c(A), Ye(this, Nc, !1), this.options.persister ? this.options.persister(
        R,
        A,
        this
      ) : R(A);
    }, h = {
      fetchOptions: u,
      options: this.options,
      queryKey: this.queryKey,
      state: this.state,
      fetchFn: v
    };
    c(h), (b = this.options.behavior) == null || b.onFetch(
      h,
      this
    ), Ye(this, ip, this.state), (this.state.fetchStatus === "idle" || this.state.fetchMeta !== ((E = h.fetchOptions) == null ? void 0 : E.meta)) && Mt(this, oo, Bu).call(this, { type: "fetch", meta: (w = h.fetchOptions) == null ? void 0 : w.meta });
    const C = (R) => {
      var A, D, L, M;
      IE(R) && R.silent || Mt(this, oo, Bu).call(this, {
        type: "error",
        error: R
      }), IE(R) || ((D = (A = $(this, Ai).config).onError) == null || D.call(
        A,
        R,
        this
      ), (M = (L = $(this, Ai).config).onSettled) == null || M.call(
        L,
        this.state.data,
        R,
        this
      )), this.isFetchingOptimistic || this.scheduleGc(), this.isFetchingOptimistic = !1;
    };
    return Ye(this, ea, DD({
      initialPromise: u == null ? void 0 : u.initialPromise,
      fn: h.fetchFn,
      abort: f.abort.bind(f),
      onSuccess: (R) => {
        var A, D, L, M;
        if (R === void 0) {
           true && console.error(
            `Query data cannot be undefined. Please make sure to return a value other than undefined from your query function. Affected query key: ${this.queryHash}`
          ), C(new Error(`${this.queryHash} data is undefined`));
          return;
        }
        this.setData(R), (D = (A = $(this, Ai).config).onSuccess) == null || D.call(A, R, this), (M = (L = $(this, Ai).config).onSettled) == null || M.call(
          L,
          R,
          this.state.error,
          this
        ), this.isFetchingOptimistic || this.scheduleGc(), this.isFetchingOptimistic = !1;
      },
      onError: C,
      onFail: (R, A) => {
        Mt(this, oo, Bu).call(this, { type: "failed", failureCount: R, error: A });
      },
      onPause: () => {
        Mt(this, oo, Bu).call(this, { type: "pause" });
      },
      onContinue: () => {
        Mt(this, oo, Bu).call(this, { type: "continue" });
      },
      retry: h.options.retry,
      retryDelay: h.options.retryDelay,
      networkMode: h.options.networkMode,
      canRun: () => !0
    })), $(this, ea).start();
  }
}, ap = new WeakMap(), ip = new WeakMap(), Ai = new WeakMap(), ea = new WeakMap(), um = new WeakMap(), Nc = new WeakMap(), oo = new WeakSet(), Bu = function(r) {
  const u = (f) => {
    switch (r.type) {
      case "failed":
        return {
          ...f,
          fetchFailureCount: r.failureCount,
          fetchFailureReason: r.error
        };
      case "pause":
        return {
          ...f,
          fetchStatus: "paused"
        };
      case "continue":
        return {
          ...f,
          fetchStatus: "fetching"
        };
      case "fetch":
        return {
          ...f,
          ...kD(f.data, this.options),
          fetchMeta: r.meta ?? null
        };
      case "success":
        return {
          ...f,
          data: r.data,
          dataUpdateCount: f.dataUpdateCount + 1,
          dataUpdatedAt: r.dataUpdatedAt ?? Date.now(),
          error: null,
          isInvalidated: !1,
          status: "success",
          ...!r.manual && {
            fetchStatus: "idle",
            fetchFailureCount: 0,
            fetchFailureReason: null
          }
        };
      case "error":
        const c = r.error;
        return IE(c) && c.revert && $(this, ip) ? { ...$(this, ip), fetchStatus: "idle" } : {
          ...f,
          error: c,
          errorUpdateCount: f.errorUpdateCount + 1,
          errorUpdatedAt: Date.now(),
          fetchFailureCount: f.fetchFailureCount + 1,
          fetchFailureReason: c,
          fetchStatus: "idle",
          status: "error"
        };
      case "invalidate":
        return {
          ...f,
          isInvalidated: !0
        };
      case "setState":
        return {
          ...f,
          ...r.state
        };
    }
  };
  this.state = u(this.state), Lr.batch(() => {
    this.observers.forEach((f) => {
      f.onQueryUpdate();
    }), $(this, Ai).notify({ query: this, type: "updated", action: r });
  });
}, MO);
function kD(a, r) {
  return {
    fetchFailureCount: 0,
    fetchFailureReason: null,
    fetchStatus: AD(r.networkMode) ? "fetching" : "paused",
    ...a === void 0 && {
      error: null,
      status: "pending"
    }
  };
}
function l9(a) {
  const r = typeof a.initialData == "function" ? a.initialData() : a.initialData, u = r !== void 0, f = u ? typeof a.initialDataUpdatedAt == "function" ? a.initialDataUpdatedAt() : a.initialDataUpdatedAt : 0;
  return {
    data: r,
    dataUpdateCount: 0,
    dataUpdatedAt: u ? f ?? Date.now() : 0,
    error: null,
    errorUpdateCount: 0,
    errorUpdatedAt: 0,
    fetchFailureCount: 0,
    fetchFailureReason: null,
    fetchMeta: null,
    isInvalidated: !1,
    status: u ? "success" : "pending",
    fetchStatus: "idle"
  };
}
var Go, LO, s9 = (LO = class extends Sm {
  constructor(r = {}) {
    super();
    ut(this, Go, void 0);
    this.config = r, Ye(this, Go, /* @__PURE__ */ new Map());
  }
  build(r, u, f) {
    const c = u.queryKey, v = u.queryHash ?? Px(c, u);
    let h = this.get(v);
    return h || (h = new u9({
      cache: this,
      queryKey: c,
      queryHash: v,
      options: r.defaultQueryOptions(u),
      state: f,
      defaultOptions: r.getQueryDefaults(c)
    }), this.add(h)), h;
  }
  add(r) {
    $(this, Go).has(r.queryHash) || ($(this, Go).set(r.queryHash, r), this.notify({
      type: "added",
      query: r
    }));
  }
  remove(r) {
    const u = $(this, Go).get(r.queryHash);
    u && (r.destroy(), u === r && $(this, Go).delete(r.queryHash), this.notify({ type: "removed", query: r }));
  }
  clear() {
    Lr.batch(() => {
      this.getAll().forEach((r) => {
        this.remove(r);
      });
    });
  }
  get(r) {
    return $(this, Go).get(r);
  }
  getAll() {
    return [...$(this, Go).values()];
  }
  find(r) {
    const u = { exact: !0, ...r };
    return this.getAll().find(
      (f) => bA(u, f)
    );
  }
  findAll(r = {}) {
    const u = this.getAll();
    return Object.keys(r).length > 0 ? u.filter((f) => bA(r, f)) : u;
  }
  notify(r) {
    Lr.batch(() => {
      this.listeners.forEach((u) => {
        u(r);
      });
    });
  }
  onFocus() {
    Lr.batch(() => {
      this.getAll().forEach((r) => {
        r.onFocus();
      });
    });
  }
  onOnline() {
    Lr.batch(() => {
      this.getAll().forEach((r) => {
        r.onOnline();
      });
    });
  }
}, Go = new WeakMap(), LO), _o, ha, kc, jo, ts, FO, c9 = (FO = class extends ND {
  constructor(r) {
    super();
    ut(this, jo);
    ut(this, _o, void 0);
    ut(this, ha, void 0);
    ut(this, kc, void 0);
    this.mutationId = r.mutationId, Ye(this, ha, r.mutationCache), Ye(this, _o, []), this.state = r.state || f9(), this.setOptions(r.options), this.scheduleGc();
  }
  setOptions(r) {
    this.options = r, this.updateGcTime(this.options.gcTime);
  }
  get meta() {
    return this.options.meta;
  }
  addObserver(r) {
    $(this, _o).includes(r) || ($(this, _o).push(r), this.clearGcTimeout(), $(this, ha).notify({
      type: "observerAdded",
      mutation: this,
      observer: r
    }));
  }
  removeObserver(r) {
    Ye(this, _o, $(this, _o).filter((u) => u !== r)), this.scheduleGc(), $(this, ha).notify({
      type: "observerRemoved",
      mutation: this,
      observer: r
    });
  }
  optionalRemove() {
    $(this, _o).length || (this.state.status === "pending" ? this.scheduleGc() : $(this, ha).remove(this));
  }
  continue() {
    var r;
    return ((r = $(this, kc)) == null ? void 0 : r.continue()) ?? // continuing a mutation assumes that variables are set, mutation must have been dehydrated before
    this.execute(this.state.variables);
  }
  async execute(r) {
    var c, v, h, C, b, E, w, R, A, D, L, M, k, B, Y, z, Z, V, Q, P;
    Ye(this, kc, DD({
      fn: () => this.options.mutationFn ? this.options.mutationFn(r) : Promise.reject(new Error("No mutationFn found")),
      onFail: (U, se) => {
        Mt(this, jo, ts).call(this, { type: "failed", failureCount: U, error: se });
      },
      onPause: () => {
        Mt(this, jo, ts).call(this, { type: "pause" });
      },
      onContinue: () => {
        Mt(this, jo, ts).call(this, { type: "continue" });
      },
      retry: this.options.retry ?? 0,
      retryDelay: this.options.retryDelay,
      networkMode: this.options.networkMode,
      canRun: () => $(this, ha).canRun(this)
    }));
    const u = this.state.status === "pending", f = !$(this, kc).canStart();
    try {
      if (!u) {
        Mt(this, jo, ts).call(this, { type: "pending", variables: r, isPaused: f }), await ((v = (c = $(this, ha).config).onMutate) == null ? void 0 : v.call(
          c,
          r,
          this
        ));
        const se = await ((C = (h = this.options).onMutate) == null ? void 0 : C.call(h, r));
        se !== this.state.context && Mt(this, jo, ts).call(this, {
          type: "pending",
          context: se,
          variables: r,
          isPaused: f
        });
      }
      const U = await $(this, kc).start();
      return await ((E = (b = $(this, ha).config).onSuccess) == null ? void 0 : E.call(
        b,
        U,
        r,
        this.state.context,
        this
      )), await ((R = (w = this.options).onSuccess) == null ? void 0 : R.call(w, U, r, this.state.context)), await ((D = (A = $(this, ha).config).onSettled) == null ? void 0 : D.call(
        A,
        U,
        null,
        this.state.variables,
        this.state.context,
        this
      )), await ((M = (L = this.options).onSettled) == null ? void 0 : M.call(L, U, null, r, this.state.context)), Mt(this, jo, ts).call(this, { type: "success", data: U }), U;
    } catch (U) {
      try {
        throw await ((B = (k = $(this, ha).config).onError) == null ? void 0 : B.call(
          k,
          U,
          r,
          this.state.context,
          this
        )), await ((z = (Y = this.options).onError) == null ? void 0 : z.call(
          Y,
          U,
          r,
          this.state.context
        )), await ((V = (Z = $(this, ha).config).onSettled) == null ? void 0 : V.call(
          Z,
          void 0,
          U,
          this.state.variables,
          this.state.context,
          this
        )), await ((P = (Q = this.options).onSettled) == null ? void 0 : P.call(
          Q,
          void 0,
          U,
          r,
          this.state.context
        )), U;
      } finally {
        Mt(this, jo, ts).call(this, { type: "error", error: U });
      }
    } finally {
      $(this, ha).runNext(this);
    }
  }
}, _o = new WeakMap(), ha = new WeakMap(), kc = new WeakMap(), jo = new WeakSet(), ts = function(r) {
  const u = (f) => {
    switch (r.type) {
      case "failed":
        return {
          ...f,
          failureCount: r.failureCount,
          failureReason: r.error
        };
      case "pause":
        return {
          ...f,
          isPaused: !0
        };
      case "continue":
        return {
          ...f,
          isPaused: !1
        };
      case "pending":
        return {
          ...f,
          context: r.context,
          data: void 0,
          failureCount: 0,
          failureReason: null,
          error: null,
          isPaused: r.isPaused,
          status: "pending",
          variables: r.variables,
          submittedAt: Date.now()
        };
      case "success":
        return {
          ...f,
          data: r.data,
          failureCount: 0,
          failureReason: null,
          error: null,
          status: "success",
          isPaused: !1
        };
      case "error":
        return {
          ...f,
          data: void 0,
          error: r.error,
          failureCount: f.failureCount + 1,
          failureReason: r.error,
          isPaused: !1,
          status: "error"
        };
    }
  };
  this.state = u(this.state), Lr.batch(() => {
    $(this, _o).forEach((f) => {
      f.onMutationUpdate(r);
    }), $(this, ha).notify({
      mutation: this,
      type: "updated",
      action: r
    });
  });
}, FO);
function f9() {
  return {
    context: void 0,
    data: void 0,
    error: null,
    failureCount: 0,
    failureReason: null,
    isPaused: !1,
    status: "idle",
    variables: void 0,
    submittedAt: 0
  };
}
var ui, lm, BO, d9 = (BO = class extends Sm {
  constructor(r = {}) {
    super();
    ut(this, ui, void 0);
    ut(this, lm, void 0);
    this.config = r, Ye(this, ui, /* @__PURE__ */ new Map()), Ye(this, lm, Date.now());
  }
  build(r, u, f) {
    const c = new c9({
      mutationCache: this,
      mutationId: ++B0(this, lm)._,
      options: r.defaultMutationOptions(u),
      state: f
    });
    return this.add(c), c;
  }
  add(r) {
    const u = G0(r), f = $(this, ui).get(u) ?? [];
    f.push(r), $(this, ui).set(u, f), this.notify({ type: "added", mutation: r });
  }
  remove(r) {
    var f;
    const u = G0(r);
    if ($(this, ui).has(u)) {
      const c = (f = $(this, ui).get(u)) == null ? void 0 : f.filter((v) => v !== r);
      c && (c.length === 0 ? $(this, ui).delete(u) : $(this, ui).set(u, c));
    }
    this.notify({ type: "removed", mutation: r });
  }
  canRun(r) {
    var f;
    const u = (f = $(this, ui).get(G0(r))) == null ? void 0 : f.find((c) => c.state.status === "pending");
    return !u || u === r;
  }
  runNext(r) {
    var f;
    const u = (f = $(this, ui).get(G0(r))) == null ? void 0 : f.find((c) => c !== r && c.state.isPaused);
    return (u == null ? void 0 : u.continue()) ?? Promise.resolve();
  }
  clear() {
    Lr.batch(() => {
      this.getAll().forEach((r) => {
        this.remove(r);
      });
    });
  }
  getAll() {
    return [...$(this, ui).values()].flat();
  }
  find(r) {
    const u = { exact: !0, ...r };
    return this.getAll().find(
      (f) => CA(u, f)
    );
  }
  findAll(r = {}) {
    return this.getAll().filter((u) => CA(r, u));
  }
  notify(r) {
    Lr.batch(() => {
      this.listeners.forEach((u) => {
        u(r);
      });
    });
  }
  resumePausedMutations() {
    const r = this.getAll().filter((u) => u.state.isPaused);
    return Lr.batch(
      () => Promise.all(
        r.map((u) => u.continue().catch(Oi))
      )
    );
  }
}, ui = new WeakMap(), lm = new WeakMap(), BO);
function G0(a) {
  var r;
  return ((r = a.options.scope) == null ? void 0 : r.id) ?? String(a.mutationId);
}
function p9(a) {
  return {
    onFetch: (r, u) => {
      const f = async () => {
        var L, M, k, B, Y;
        const c = r.options, v = (k = (M = (L = r.fetchOptions) == null ? void 0 : L.meta) == null ? void 0 : M.fetchMore) == null ? void 0 : k.direction, h = ((B = r.state.data) == null ? void 0 : B.pages) || [], C = ((Y = r.state.data) == null ? void 0 : Y.pageParams) || [], b = { pages: [], pageParams: [] };
        let E = !1;
        const w = (z) => {
          Object.defineProperty(z, "signal", {
            enumerable: !0,
            get: () => (r.signal.aborted ? E = !0 : r.signal.addEventListener("abort", () => {
              E = !0;
            }), r.signal)
          });
        }, R = ID(r.options, r.fetchOptions), A = async (z, Z, V) => {
          if (E)
            return Promise.reject();
          if (Z == null && z.pages.length)
            return Promise.resolve(z);
          const Q = {
            queryKey: r.queryKey,
            pageParam: Z,
            direction: V ? "backward" : "forward",
            meta: r.options.meta
          };
          w(Q);
          const P = await R(
            Q
          ), { maxPages: U } = r.options, se = V ? n9 : t9;
          return {
            pages: se(z.pages, P, U),
            pageParams: se(z.pageParams, Z, U)
          };
        };
        let D;
        if (v && h.length) {
          const z = v === "backward", Z = z ? v9 : xA, V = {
            pages: h,
            pageParams: C
          }, Q = Z(c, V);
          D = await A(V, Q, z);
        } else {
          D = await A(
            b,
            C[0] ?? c.initialPageParam
          );
          const z = a ?? h.length;
          for (let Z = 1; Z < z; Z++) {
            const V = xA(c, D);
            D = await A(D, V);
          }
        }
        return D;
      };
      r.options.persister ? r.fetchFn = () => {
        var c, v;
        return (v = (c = r.options).persister) == null ? void 0 : v.call(
          c,
          f,
          {
            queryKey: r.queryKey,
            meta: r.options.meta,
            signal: r.signal
          },
          u
        );
      } : r.fetchFn = f;
    }
  };
}
function xA(a, { pages: r, pageParams: u }) {
  const f = r.length - 1;
  return a.getNextPageParam(
    r[f],
    r,
    u[f],
    u
  );
}
function v9(a, { pages: r, pageParams: u }) {
  var f;
  return (f = a.getPreviousPageParam) == null ? void 0 : f.call(
    a,
    r[0],
    r,
    u[0],
    u
  );
}
var Ln, os, us, op, up, ls, lp, sp, VO, h9 = (VO = class {
  constructor(a = {}) {
    ut(this, Ln, void 0);
    ut(this, os, void 0);
    ut(this, us, void 0);
    ut(this, op, void 0);
    ut(this, up, void 0);
    ut(this, ls, void 0);
    ut(this, lp, void 0);
    ut(this, sp, void 0);
    Ye(this, Ln, a.queryCache || new s9()), Ye(this, os, a.mutationCache || new d9()), Ye(this, us, a.defaultOptions || {}), Ye(this, op, /* @__PURE__ */ new Map()), Ye(this, up, /* @__PURE__ */ new Map()), Ye(this, ls, 0);
  }
  mount() {
    B0(this, ls)._++, $(this, ls) === 1 && (Ye(this, lp, Hx.subscribe(async (a) => {
      a && (await this.resumePausedMutations(), $(this, Ln).onFocus());
    })), Ye(this, sp, ib.subscribe(async (a) => {
      a && (await this.resumePausedMutations(), $(this, Ln).onOnline());
    })));
  }
  unmount() {
    var a, r;
    B0(this, ls)._--, $(this, ls) === 0 && ((a = $(this, lp)) == null || a.call(this), Ye(this, lp, void 0), (r = $(this, sp)) == null || r.call(this), Ye(this, sp, void 0));
  }
  isFetching(a) {
    return $(this, Ln).findAll({ ...a, fetchStatus: "fetching" }).length;
  }
  isMutating(a) {
    return $(this, os).findAll({ ...a, status: "pending" }).length;
  }
  getQueryData(a) {
    var u;
    const r = this.defaultQueryOptions({ queryKey: a });
    return (u = $(this, Ln).get(r.queryHash)) == null ? void 0 : u.state.data;
  }
  ensureQueryData(a) {
    const r = this.getQueryData(a.queryKey);
    if (r === void 0)
      return this.fetchQuery(a);
    {
      const u = this.defaultQueryOptions(a), f = $(this, Ln).build(this, u);
      return a.revalidateIfStale && f.isStaleByTime(Kd(u.staleTime, f)) && this.prefetchQuery(u), Promise.resolve(r);
    }
  }
  getQueriesData(a) {
    return $(this, Ln).findAll(a).map(({ queryKey: r, state: u }) => {
      const f = u.data;
      return [r, f];
    });
  }
  setQueryData(a, r, u) {
    const f = this.defaultQueryOptions({ queryKey: a }), c = $(this, Ln).get(
      f.queryHash
    ), v = c == null ? void 0 : c.state.data, h = qV(r, v);
    if (h !== void 0)
      return $(this, Ln).build(this, f).setData(h, { ...u, manual: !0 });
  }
  setQueriesData(a, r, u) {
    return Lr.batch(
      () => $(this, Ln).findAll(a).map(({ queryKey: f }) => [
        f,
        this.setQueryData(f, r, u)
      ])
    );
  }
  getQueryState(a) {
    var u;
    const r = this.defaultQueryOptions({ queryKey: a });
    return (u = $(this, Ln).get(r.queryHash)) == null ? void 0 : u.state;
  }
  removeQueries(a) {
    const r = $(this, Ln);
    Lr.batch(() => {
      r.findAll(a).forEach((u) => {
        r.remove(u);
      });
    });
  }
  resetQueries(a, r) {
    const u = $(this, Ln), f = {
      type: "active",
      ...a
    };
    return Lr.batch(() => (u.findAll(a).forEach((c) => {
      c.reset();
    }), this.refetchQueries(f, r)));
  }
  cancelQueries(a = {}, r = {}) {
    const u = { revert: !0, ...r }, f = Lr.batch(
      () => $(this, Ln).findAll(a).map((c) => c.cancel(u))
    );
    return Promise.all(f).then(Oi).catch(Oi);
  }
  invalidateQueries(a = {}, r = {}) {
    return Lr.batch(() => {
      if ($(this, Ln).findAll(a).forEach((f) => {
        f.invalidate();
      }), a.refetchType === "none")
        return Promise.resolve();
      const u = {
        ...a,
        type: a.refetchType ?? a.type ?? "active"
      };
      return this.refetchQueries(u, r);
    });
  }
  refetchQueries(a = {}, r) {
    const u = {
      ...r,
      cancelRefetch: (r == null ? void 0 : r.cancelRefetch) ?? !0
    }, f = Lr.batch(
      () => $(this, Ln).findAll(a).filter((c) => !c.isDisabled()).map((c) => {
        let v = c.fetch(void 0, u);
        return u.throwOnError || (v = v.catch(Oi)), c.state.fetchStatus === "paused" ? Promise.resolve() : v;
      })
    );
    return Promise.all(f).then(Oi);
  }
  fetchQuery(a) {
    const r = this.defaultQueryOptions(a);
    r.retry === void 0 && (r.retry = !1);
    const u = $(this, Ln).build(this, r);
    return u.isStaleByTime(
      Kd(r.staleTime, u)
    ) ? u.fetch(r) : Promise.resolve(u.state.data);
  }
  prefetchQuery(a) {
    return this.fetchQuery(a).then(Oi).catch(Oi);
  }
  fetchInfiniteQuery(a) {
    return a.behavior = p9(a.pages), this.fetchQuery(a);
  }
  prefetchInfiniteQuery(a) {
    return this.fetchInfiniteQuery(a).then(Oi).catch(Oi);
  }
  resumePausedMutations() {
    return ib.isOnline() ? $(this, os).resumePausedMutations() : Promise.resolve();
  }
  getQueryCache() {
    return $(this, Ln);
  }
  getMutationCache() {
    return $(this, os);
  }
  getDefaultOptions() {
    return $(this, us);
  }
  setDefaultOptions(a) {
    Ye(this, us, a);
  }
  setQueryDefaults(a, r) {
    $(this, op).set(Kh(a), {
      queryKey: a,
      defaultOptions: r
    });
  }
  getQueryDefaults(a) {
    const r = [...$(this, op).values()];
    let u = {};
    return r.forEach((f) => {
      qh(a, f.queryKey) && (u = { ...u, ...f.defaultOptions });
    }), u;
  }
  setMutationDefaults(a, r) {
    $(this, up).set(Kh(a), {
      mutationKey: a,
      defaultOptions: r
    });
  }
  getMutationDefaults(a) {
    const r = [...$(this, up).values()];
    let u = {};
    return r.forEach((f) => {
      qh(a, f.mutationKey) && (u = { ...u, ...f.defaultOptions });
    }), u;
  }
  defaultQueryOptions(a) {
    if (a._defaulted)
      return a;
    const r = {
      ...$(this, us).queries,
      ...this.getQueryDefaults(a.queryKey),
      ...a,
      _defaulted: !0
    };
    return r.queryHash || (r.queryHash = Px(
      r.queryKey,
      r
    )), r.refetchOnReconnect === void 0 && (r.refetchOnReconnect = r.networkMode !== "always"), r.throwOnError === void 0 && (r.throwOnError = !!r.suspense), !r.networkMode && r.persister && (r.networkMode = "offlineFirst"), r.enabled !== !0 && r.queryFn === ex && (r.enabled = !1), r;
  }
  defaultMutationOptions(a) {
    return a != null && a._defaulted ? a : {
      ...$(this, us).mutations,
      ...(a == null ? void 0 : a.mutationKey) && this.getMutationDefaults(a.mutationKey),
      ...a,
      _defaulted: !0
    };
  }
  clear() {
    $(this, Ln).clear(), $(this, os).clear();
  }
}, Ln = new WeakMap(), os = new WeakMap(), us = new WeakMap(), op = new WeakMap(), up = new WeakMap(), ls = new WeakMap(), lp = new WeakMap(), sp = new WeakMap(), VO), ja, en, sm, ma, Mc, cp, Uo, cm, fp, dp, Lc, Fc, ss, pp, Bc, Uh, fm, tx, dm, nx, pm, rx, vm, ax, hm, ix, mm, ox, gm, ux, Cb, MD, PO, m9 = (PO = class extends Sm {
  constructor(r, u) {
    super();
    ut(this, Bc);
    ut(this, fm);
    ut(this, dm);
    ut(this, pm);
    ut(this, vm);
    ut(this, hm);
    ut(this, mm);
    ut(this, gm);
    ut(this, Cb);
    ut(this, ja, void 0);
    ut(this, en, void 0);
    ut(this, sm, void 0);
    ut(this, ma, void 0);
    ut(this, Mc, void 0);
    ut(this, cp, void 0);
    ut(this, Uo, void 0);
    ut(this, cm, void 0);
    ut(this, fp, void 0);
    // This property keeps track of the last query with defined data.
    // It will be used to pass the previous data and query to the placeholder function between renders.
    ut(this, dp, void 0);
    ut(this, Lc, void 0);
    ut(this, Fc, void 0);
    ut(this, ss, void 0);
    ut(this, pp, /* @__PURE__ */ new Set());
    this.options = u, Ye(this, ja, r), Ye(this, Uo, null), this.bindMethods(), this.setOptions(u);
  }
  bindMethods() {
    this.refetch = this.refetch.bind(this);
  }
  onSubscribe() {
    this.listeners.size === 1 && ($(this, en).addObserver(this), wA($(this, en), this.options) ? Mt(this, Bc, Uh).call(this) : this.updateResult(), Mt(this, vm, ax).call(this));
  }
  onUnsubscribe() {
    this.hasListeners() || this.destroy();
  }
  shouldFetchOnReconnect() {
    return lx(
      $(this, en),
      this.options,
      this.options.refetchOnReconnect
    );
  }
  shouldFetchOnWindowFocus() {
    return lx(
      $(this, en),
      this.options,
      this.options.refetchOnWindowFocus
    );
  }
  destroy() {
    this.listeners = /* @__PURE__ */ new Set(), Mt(this, hm, ix).call(this), Mt(this, mm, ox).call(this), $(this, en).removeObserver(this);
  }
  setOptions(r, u) {
    const f = this.options, c = $(this, en);
    if (this.options = $(this, ja).defaultQueryOptions(r), this.options.enabled !== void 0 && typeof this.options.enabled != "boolean")
      throw new Error("Expected enabled to be a boolean");
    Mt(this, gm, ux).call(this), $(this, en).setOptions(this.options), f._defaulted && !JE(this.options, f) && $(this, ja).getQueryCache().notify({
      type: "observerOptionsUpdated",
      query: $(this, en),
      observer: this
    });
    const v = this.hasListeners();
    v && TA(
      $(this, en),
      c,
      this.options,
      f
    ) && Mt(this, Bc, Uh).call(this), this.updateResult(u), v && ($(this, en) !== c || this.options.enabled !== f.enabled || Kd(this.options.staleTime, $(this, en)) !== Kd(f.staleTime, $(this, en))) && Mt(this, fm, tx).call(this);
    const h = Mt(this, dm, nx).call(this);
    v && ($(this, en) !== c || this.options.enabled !== f.enabled || h !== $(this, ss)) && Mt(this, pm, rx).call(this, h);
  }
  getOptimisticResult(r) {
    const u = $(this, ja).getQueryCache().build($(this, ja), r), f = this.createResult(u, r);
    return y9(this, f) && (Ye(this, ma, f), Ye(this, cp, this.options), Ye(this, Mc, $(this, en).state)), f;
  }
  getCurrentResult() {
    return $(this, ma);
  }
  trackResult(r, u) {
    const f = {};
    return Object.keys(r).forEach((c) => {
      Object.defineProperty(f, c, {
        configurable: !1,
        enumerable: !0,
        get: () => (this.trackProp(c), u == null || u(c), r[c])
      });
    }), f;
  }
  trackProp(r) {
    $(this, pp).add(r);
  }
  getCurrentQuery() {
    return $(this, en);
  }
  refetch({ ...r } = {}) {
    return this.fetch({
      ...r
    });
  }
  fetchOptimistic(r) {
    const u = $(this, ja).defaultQueryOptions(r), f = $(this, ja).getQueryCache().build($(this, ja), u);
    return f.isFetchingOptimistic = !0, f.fetch().then(() => this.createResult(f, u));
  }
  fetch(r) {
    return Mt(this, Bc, Uh).call(this, {
      ...r,
      cancelRefetch: r.cancelRefetch ?? !0
    }).then(() => (this.updateResult(), $(this, ma)));
  }
  createResult(r, u) {
    var P;
    const f = $(this, en), c = this.options, v = $(this, ma), h = $(this, Mc), C = $(this, cp), E = r !== f ? r.state : $(this, sm), { state: w } = r;
    let R = { ...w }, A = !1, D;
    if (u._optimisticResults) {
      const U = this.hasListeners(), se = !U && wA(r, u), Se = U && TA(r, f, u, c);
      (se || Se) && (R = {
        ...R,
        ...kD(w.data, r.options)
      }), u._optimisticResults === "isRestoring" && (R.fetchStatus = "idle");
    }
    let { error: L, errorUpdatedAt: M, status: k } = R;
    if (u.select && R.data !== void 0)
      if (v && R.data === (h == null ? void 0 : h.data) && u.select === $(this, cm))
        D = $(this, fp);
      else
        try {
          Ye(this, cm, u.select), D = u.select(R.data), D = qE(v == null ? void 0 : v.data, D, u), Ye(this, fp, D), Ye(this, Uo, null);
        } catch (U) {
          Ye(this, Uo, U);
        }
    else
      D = R.data;
    if (u.placeholderData !== void 0 && D === void 0 && k === "pending") {
      let U;
      if (v != null && v.isPlaceholderData && u.placeholderData === (C == null ? void 0 : C.placeholderData))
        U = v.data;
      else if (U = typeof u.placeholderData == "function" ? u.placeholderData(
        (P = $(this, dp)) == null ? void 0 : P.state.data,
        $(this, dp)
      ) : u.placeholderData, u.select && U !== void 0)
        try {
          U = u.select(U), Ye(this, Uo, null);
        } catch (se) {
          Ye(this, Uo, se);
        }
      U !== void 0 && (k = "success", D = qE(
        v == null ? void 0 : v.data,
        U,
        u
      ), A = !0);
    }
    $(this, Uo) && (L = $(this, Uo), D = $(this, fp), M = Date.now(), k = "error");
    const B = R.fetchStatus === "fetching", Y = k === "pending", z = k === "error", Z = Y && B, V = D !== void 0;
    return {
      status: k,
      fetchStatus: R.fetchStatus,
      isPending: Y,
      isSuccess: k === "success",
      isError: z,
      isInitialLoading: Z,
      isLoading: Z,
      data: D,
      dataUpdatedAt: R.dataUpdatedAt,
      error: L,
      errorUpdatedAt: M,
      failureCount: R.fetchFailureCount,
      failureReason: R.fetchFailureReason,
      errorUpdateCount: R.errorUpdateCount,
      isFetched: R.dataUpdateCount > 0 || R.errorUpdateCount > 0,
      isFetchedAfterMount: R.dataUpdateCount > E.dataUpdateCount || R.errorUpdateCount > E.errorUpdateCount,
      isFetching: B,
      isRefetching: B && !Y,
      isLoadingError: z && !V,
      isPaused: R.fetchStatus === "paused",
      isPlaceholderData: A,
      isRefetchError: z && V,
      isStale: zx(r, u),
      refetch: this.refetch
    };
  }
  updateResult(r) {
    const u = $(this, ma), f = this.createResult($(this, en), this.options);
    if (Ye(this, Mc, $(this, en).state), Ye(this, cp, this.options), $(this, Mc).data !== void 0 && Ye(this, dp, $(this, en)), JE(f, u))
      return;
    Ye(this, ma, f);
    const c = {}, v = () => {
      if (!u)
        return !0;
      const { notifyOnChangeProps: h } = this.options, C = typeof h == "function" ? h() : h;
      if (C === "all" || !C && !$(this, pp).size)
        return !0;
      const b = new Set(
        C ?? $(this, pp)
      );
      return this.options.throwOnError && b.add("error"), Object.keys($(this, ma)).some((E) => {
        const w = E;
        return $(this, ma)[w] !== u[w] && b.has(w);
      });
    };
    (r == null ? void 0 : r.listeners) !== !1 && v() && (c.listeners = !0), Mt(this, Cb, MD).call(this, { ...c, ...r });
  }
  onQueryUpdate() {
    this.updateResult(), this.hasListeners() && Mt(this, vm, ax).call(this);
  }
}, ja = new WeakMap(), en = new WeakMap(), sm = new WeakMap(), ma = new WeakMap(), Mc = new WeakMap(), cp = new WeakMap(), Uo = new WeakMap(), cm = new WeakMap(), fp = new WeakMap(), dp = new WeakMap(), Lc = new WeakMap(), Fc = new WeakMap(), ss = new WeakMap(), pp = new WeakMap(), Bc = new WeakSet(), Uh = function(r) {
  Mt(this, gm, ux).call(this);
  let u = $(this, en).fetch(
    this.options,
    r
  );
  return r != null && r.throwOnError || (u = u.catch(Oi)), u;
}, fm = new WeakSet(), tx = function() {
  Mt(this, hm, ix).call(this);
  const r = Kd(
    this.options.staleTime,
    $(this, en)
  );
  if (hp || $(this, ma).isStale || !QE(r))
    return;
  const f = TD($(this, ma).dataUpdatedAt, r) + 1;
  Ye(this, Lc, setTimeout(() => {
    $(this, ma).isStale || this.updateResult();
  }, f));
}, dm = new WeakSet(), nx = function() {
  return (typeof this.options.refetchInterval == "function" ? this.options.refetchInterval($(this, en)) : this.options.refetchInterval) ?? !1;
}, pm = new WeakSet(), rx = function(r) {
  Mt(this, mm, ox).call(this), Ye(this, ss, r), !(hp || this.options.enabled === !1 || !QE($(this, ss)) || $(this, ss) === 0) && Ye(this, Fc, setInterval(() => {
    (this.options.refetchIntervalInBackground || Hx.isFocused()) && Mt(this, Bc, Uh).call(this);
  }, $(this, ss)));
}, vm = new WeakSet(), ax = function() {
  Mt(this, fm, tx).call(this), Mt(this, pm, rx).call(this, Mt(this, dm, nx).call(this));
}, hm = new WeakSet(), ix = function() {
  $(this, Lc) && (clearTimeout($(this, Lc)), Ye(this, Lc, void 0));
}, mm = new WeakSet(), ox = function() {
  $(this, Fc) && (clearInterval($(this, Fc)), Ye(this, Fc, void 0));
}, gm = new WeakSet(), ux = function() {
  const r = $(this, ja).getQueryCache().build($(this, ja), this.options);
  if (r === $(this, en))
    return;
  const u = $(this, en);
  Ye(this, en, r), Ye(this, sm, r.state), this.hasListeners() && (u == null || u.removeObserver(this), r.addObserver(this));
}, Cb = new WeakSet(), MD = function(r) {
  Lr.batch(() => {
    r.listeners && this.listeners.forEach((u) => {
      u($(this, ma));
    }), $(this, ja).getQueryCache().notify({
      query: $(this, en),
      type: "observerResultsUpdated"
    });
  });
}, PO);
function g9(a, r) {
  return r.enabled !== !1 && a.state.data === void 0 && !(a.state.status === "error" && r.retryOnMount === !1);
}
function wA(a, r) {
  return g9(a, r) || a.state.data !== void 0 && lx(a, r, r.refetchOnMount);
}
function lx(a, r, u) {
  if (r.enabled !== !1) {
    const f = typeof u == "function" ? u(a) : u;
    return f === "always" || f !== !1 && zx(a, r);
  }
  return !1;
}
function TA(a, r, u, f) {
  return (a !== r || f.enabled === !1) && (!u.suspense || a.state.status !== "error") && zx(a, u);
}
function zx(a, r) {
  return r.enabled !== !1 && a.isStaleByTime(Kd(r.staleTime, a));
}
function y9(a, r) {
  return !JE(a.getCurrentResult(), r);
}
var LD = react__WEBPACK_IMPORTED_MODULE_1__.createContext(
  void 0
), b9 = (a) => {
  const r = react__WEBPACK_IMPORTED_MODULE_1__.useContext(LD);
  if (!r)
    throw new Error("No QueryClient set, use QueryClientProvider to set one");
  return r;
}, C9 = ({
  client: a,
  children: r
}) => (react__WEBPACK_IMPORTED_MODULE_1__.useEffect(() => (a.mount(), () => {
  a.unmount();
}), [a]), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LD.Provider, { value: a, children: r })), FD = react__WEBPACK_IMPORTED_MODULE_1__.createContext(!1), S9 = () => react__WEBPACK_IMPORTED_MODULE_1__.useContext(FD);
FD.Provider;
function E9() {
  let a = !1;
  return {
    clearReset: () => {
      a = !1;
    },
    reset: () => {
      a = !0;
    },
    isReset: () => a
  };
}
var x9 = react__WEBPACK_IMPORTED_MODULE_1__.createContext(E9()), w9 = () => react__WEBPACK_IMPORTED_MODULE_1__.useContext(x9);
function T9(a, r) {
  return typeof a == "function" ? a(...r) : !!a;
}
var R9 = (a, r) => {
  (a.suspense || a.throwOnError) && (r.isReset() || (a.retryOnMount = !1));
}, I9 = (a) => {
  react__WEBPACK_IMPORTED_MODULE_1__.useEffect(() => {
    a.clearReset();
  }, [a]);
}, A9 = ({
  result: a,
  errorResetBoundary: r,
  throwOnError: u,
  query: f
}) => a.isError && !r.isReset() && !a.isFetching && f && T9(u, [a.error, f]), O9 = (a) => {
  a.suspense && typeof a.staleTime != "number" && (a.staleTime = 1e3);
}, D9 = (a, r) => (a == null ? void 0 : a.suspense) && r.isPending, N9 = (a, r, u) => r.fetchOptimistic(a).catch(() => {
  u.clearReset();
});
function k9(a, r, u) {
  if ( true && (typeof a != "object" || Array.isArray(a)))
    throw new Error(
      'Bad argument type. Starting with v5, only the "Object" form is allowed when calling query related functions. Please use the error stack to find the culprit call. More info here: https://tanstack.com/query/latest/docs/react/guides/migrating-to-v5#supports-a-single-signature-one-object'
    );
  const f = b9(), c = S9(), v = w9(), h = f.defaultQueryOptions(a);
  h._optimisticResults = c ? "isRestoring" : "optimistic", O9(h), R9(h, v), I9(v);
  const [C] = react__WEBPACK_IMPORTED_MODULE_1__.useState(
    () => new r(
      f,
      h
    )
  ), b = C.getOptimisticResult(h);
  if (react__WEBPACK_IMPORTED_MODULE_1__.useSyncExternalStore(
    react__WEBPACK_IMPORTED_MODULE_1__.useCallback(
      (E) => {
        const w = c ? () => {
        } : C.subscribe(Lr.batchCalls(E));
        return C.updateResult(), w;
      },
      [C, c]
    ),
    () => C.getCurrentResult(),
    () => C.getCurrentResult()
  ), react__WEBPACK_IMPORTED_MODULE_1__.useEffect(() => {
    C.setOptions(h, { listeners: !1 });
  }, [h, C]), D9(h, b))
    throw N9(h, C, v);
  if (A9({
    result: b,
    errorResetBoundary: v,
    throwOnError: h.throwOnError,
    query: f.getQueryCache().get(h.queryHash)
  }))
    throw b.error;
  return h.notifyOnChangeProps ? b : C.trackResult(b);
}
function Qd(a, r) {
  return k9(a, m9);
}
const em = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null), ob = (a, r = null) => a != null ? String(a) : r || null, BD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  as: u,
  ...f
}, c) => {
  a = ft(a, "navbar-brand");
  const v = u || (f.href ? "a" : "span");
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(v, {
    ...f,
    ref: c,
    className: Ke(r, a)
  });
});
BD.displayName = "NavbarBrand";
function Ph(...a) {
  return a.filter((r) => r != null).reduce((r, u) => {
    if (typeof u != "function")
      throw new Error("Invalid Argument Type, must only provide functions, undefined, or null.");
    return r === null ? u : function(...c) {
      r.apply(this, c), u.apply(this, c);
    };
  }, null);
}
const M9 = {
  height: ["marginTop", "marginBottom"],
  width: ["marginLeft", "marginRight"]
};
function L9(a, r) {
  const u = `offset${a[0].toUpperCase()}${a.slice(1)}`, f = r[u], c = M9[a];
  return f + // @ts-ignore
  parseInt(Pu(r, c[0]), 10) + // @ts-ignore
  parseInt(Pu(r, c[1]), 10);
}
const F9 = {
  [ns]: "collapse",
  [Jh]: "collapsing",
  [uo]: "collapsing",
  [Vu]: "collapse show"
}, B9 = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().forwardRef(({
  onEnter: a,
  onEntering: r,
  onEntered: u,
  onExit: f,
  onExiting: c,
  className: v,
  children: h,
  dimension: C = "height",
  in: b = !1,
  timeout: E = 300,
  mountOnEnter: w = !1,
  unmountOnExit: R = !1,
  appear: A = !1,
  getDimensionValue: D = L9,
  ...L
}, M) => {
  const k = typeof C == "function" ? C() : C, B = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Ph((Q) => {
    Q.style[k] = "0";
  }, a), [k, a]), Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Ph((Q) => {
    const P = `scroll${k[0].toUpperCase()}${k.slice(1)}`;
    Q.style[k] = `${Q[P]}px`;
  }, r), [k, r]), z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Ph((Q) => {
    Q.style[k] = null;
  }, u), [k, u]), Z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Ph((Q) => {
    Q.style[k] = `${D(k, Q)}px`, cD(Q);
  }, f), [f, D, k]), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Ph((Q) => {
    Q.style[k] = null;
  }, c), [k, c]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Mx, {
    ref: M,
    addEndListener: kx,
    ...L,
    "aria-expanded": L.role ? b : null,
    onEnter: B,
    onEntering: Y,
    onEntered: z,
    onExit: Z,
    onExiting: V,
    childRef: h.ref,
    in: b,
    timeout: E,
    mountOnEnter: w,
    unmountOnExit: R,
    appear: A,
    children: (Q, P) => /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().cloneElement(h, {
      ...P,
      className: Ke(v, h.props.className, F9[Q], k === "width" && "collapse-horizontal")
    })
  });
}), Uc = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null);
Uc.displayName = "NavbarContext";
const VD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  children: a,
  bsPrefix: r,
  ...u
}, f) => {
  r = ft(r, "navbar-collapse");
  const c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Uc);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(B9, {
    in: !!(c && c.expanded),
    ...u,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
      ref: f,
      className: r,
      children: a
    })
  });
});
VD.displayName = "NavbarCollapse";
const PD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  children: u,
  label: f = "Toggle navigation",
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: c = "button",
  onClick: v,
  ...h
}, C) => {
  a = ft(a, "navbar-toggler");
  const {
    onToggle: b,
    expanded: E
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Uc) || {}, w = Un((R) => {
    v && v(R), b && b();
  });
  return c === "button" && (h.type = "button"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(c, {
    ...h,
    ref: C,
    onClick: w,
    "aria-label": f,
    className: Ke(r, a, !E && "collapsed"),
    children: u || /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", {
      className: `${a}-icon`
    })
  });
});
PD.displayName = "NavbarToggle";
const sx = /* @__PURE__ */ new WeakMap(), RA = (a, r) => {
  if (!a || !r)
    return;
  const u = sx.get(r) || /* @__PURE__ */ new Map();
  sx.set(r, u);
  let f = u.get(a);
  return f || (f = r.matchMedia(a), f.refCount = 0, u.set(f.media, f)), f;
};
function V9(a, r = typeof window > "u" ? void 0 : window) {
  const u = RA(a, r), [f, c] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(() => u ? u.matches : !1);
  return rb(() => {
    let v = RA(a, r);
    if (!v)
      return c(!1);
    let h = sx.get(r);
    const C = () => {
      c(v.matches);
    };
    return v.refCount++, v.addListener(C), C(), () => {
      v.removeListener(C), v.refCount--, v.refCount <= 0 && (h == null || h.delete(v.media)), v = void 0;
    };
  }, [a]), f;
}
function P9(a) {
  const r = Object.keys(a);
  function u(C, b) {
    return C === b ? b : C ? `${C} and ${b}` : b;
  }
  function f(C) {
    return r[Math.min(r.indexOf(C) + 1, r.length - 1)];
  }
  function c(C) {
    const b = f(C);
    let E = a[b];
    return typeof E == "number" ? E = `${E - 0.2}px` : E = `calc(${E} - 0.2px)`, `(max-width: ${E})`;
  }
  function v(C) {
    let b = a[C];
    return typeof b == "number" && (b = `${b}px`), `(min-width: ${b})`;
  }
  function h(C, b, E) {
    let w;
    typeof C == "object" ? (w = C, E = b, b = !0) : (b = b || !0, w = {
      [C]: b
    });
    let R = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => Object.entries(w).reduce((A, [D, L]) => ((L === "up" || L === !0) && (A = u(A, v(D))), (L === "down" || L === !0) && (A = u(A, c(D))), A), ""), [JSON.stringify(w)]);
    return V9(R, E);
  }
  return h;
}
const H9 = P9({
  xs: 0,
  sm: 576,
  md: 768,
  lg: 992,
  xl: 1200,
  xxl: 1400
});
function AE(a) {
  a === void 0 && (a = Ib());
  try {
    var r = a.activeElement;
    return !r || !r.nodeName ? null : r;
  } catch {
    return a.body;
  }
}
function ub(a, r) {
  if (a.contains)
    return a.contains(r);
  if (a.compareDocumentPosition)
    return a === r || !!(a.compareDocumentPosition(r) & 16);
}
function z9(a) {
  const r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(a);
  return r.current = a, r;
}
function G9(a) {
  const r = z9(a);
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => () => r.current(), []);
}
const _9 = "data-rr-ui-";
function Em(a) {
  return `${_9}${a}`;
}
function j9(a = document) {
  const r = a.defaultView;
  return Math.abs(r.innerWidth - a.documentElement.clientWidth);
}
const IA = Em("modal-open");
class Gx {
  constructor({
    ownerDocument: r,
    handleContainerOverflow: u = !0,
    isRTL: f = !1
  } = {}) {
    this.handleContainerOverflow = u, this.isRTL = f, this.modals = [], this.ownerDocument = r;
  }
  getScrollbarWidth() {
    return j9(this.ownerDocument);
  }
  getElement() {
    return (this.ownerDocument || document).body;
  }
  setModalAttributes(r) {
  }
  removeModalAttributes(r) {
  }
  setContainerStyle(r) {
    const u = {
      overflow: "hidden"
    }, f = this.isRTL ? "paddingLeft" : "paddingRight", c = this.getElement();
    r.style = {
      overflow: c.style.overflow,
      [f]: c.style[f]
    }, r.scrollBarWidth && (u[f] = `${parseInt(Pu(c, f) || "0", 10) + r.scrollBarWidth}px`), c.setAttribute(IA, ""), Pu(c, u);
  }
  reset() {
    [...this.modals].forEach((r) => this.remove(r));
  }
  removeContainerStyle(r) {
    const u = this.getElement();
    u.removeAttribute(IA), Object.assign(u.style, r.style);
  }
  add(r) {
    let u = this.modals.indexOf(r);
    return u !== -1 || (u = this.modals.length, this.modals.push(r), this.setModalAttributes(r), u !== 0) || (this.state = {
      scrollBarWidth: this.getScrollbarWidth(),
      style: {}
    }, this.handleContainerOverflow && this.setContainerStyle(this.state)), u;
  }
  remove(r) {
    const u = this.modals.indexOf(r);
    u !== -1 && (this.modals.splice(u, 1), !this.modals.length && this.handleContainerOverflow && this.removeContainerStyle(this.state), this.removeModalAttributes(r));
  }
  isTopModal(r) {
    return !!this.modals.length && this.modals[this.modals.length - 1] === r;
  }
}
const HD = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.createContext)(Ab ? window : void 0);
HD.Provider;
function kb() {
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(HD);
}
const OE = (a, r) => Ab ? a == null ? (r || Ib()).body : (typeof a == "function" && (a = a()), a && "current" in a && (a = a.current), a && ("nodeType" in a || a.getBoundingClientRect) ? a : null) : null;
function U9(a, r) {
  const u = kb(), [f, c] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(() => OE(a, u == null ? void 0 : u.document));
  if (!f) {
    const v = OE(a);
    v && c(v);
  }
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
  }, [r, f]), (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    const v = OE(a);
    v !== f && c(v);
  }, [a, f]), f;
}
function W9({
  children: a,
  in: r,
  onExited: u,
  mountOnEnter: f,
  unmountOnExit: c
}) {
  const v = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(r), C = Un(u);
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    r ? h.current = !0 : C(v.current);
  }, [r, C]);
  const b = jc(v, a.ref), E = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.cloneElement)(a, {
    ref: b
  });
  return r ? E : c || !h.current && f ? null : E;
}
function X9(a) {
  return a.code === "Escape" || a.keyCode === 27;
}
function Z9() {
  const a = react__WEBPACK_IMPORTED_MODULE_1__.version.split(".");
  return {
    major: +a[0],
    minor: +a[1],
    patch: +a[2]
  };
}
const Y9 = ["onEnter", "onEntering", "onEntered", "onExit", "onExiting", "onExited", "addEndListener", "children"];
function $9(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
function Q9(a) {
  let {
    onEnter: r,
    onEntering: u,
    onEntered: f,
    onExit: c,
    onExiting: v,
    onExited: h,
    addEndListener: C,
    children: b
  } = a, E = $9(a, Y9);
  const {
    major: w
  } = Z9(), R = w >= 19 ? b.props.ref : b.ref, A = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), D = jc(A, typeof b == "function" ? null : R), L = (Q) => (P) => {
    Q && A.current && Q(A.current, P);
  }, M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(r), [r]), k = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(u), [u]), B = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(f), [f]), Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(c), [c]), z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(v), [v]), Z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(h), [h]), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(L(C), [C]);
  return Object.assign({}, E, {
    nodeRef: A
  }, r && {
    onEnter: M
  }, u && {
    onEntering: k
  }, f && {
    onEntered: B
  }, c && {
    onExit: Y
  }, v && {
    onExiting: z
  }, h && {
    onExited: Z
  }, C && {
    addEndListener: V
  }, {
    children: typeof b == "function" ? (Q, P) => (
      // TODO: Types for RTG missing innerProps, so need to cast.
      b(Q, Object.assign({}, P, {
        ref: D
      }))
    ) : /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.cloneElement)(b, {
      ref: D
    })
  });
}
const J9 = ["component"];
function K9(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
const q9 = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  let {
    component: u
  } = a, f = K9(a, J9);
  const c = Q9(f);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, Object.assign({
    ref: r
  }, c));
});
function e3({
  in: a,
  onTransition: r
}) {
  const u = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), f = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!0), c = Un(r);
  return rb(() => {
    if (!u.current)
      return;
    let v = !1;
    return c({
      in: a,
      element: u.current,
      initial: f.current,
      isStale: () => v
    }), () => {
      v = !0;
    };
  }, [a, c]), rb(() => (f.current = !1, () => {
    f.current = !0;
  }), []), u;
}
function t3({
  children: a,
  in: r,
  onExited: u,
  onEntered: f,
  transition: c
}) {
  const [v, h] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(!r);
  r && v && h(!1);
  const C = e3({
    in: !!r,
    onTransition: (E) => {
      const w = () => {
        E.isStale() || (E.in ? f == null || f(E.element, E.initial) : (h(!0), u == null || u(E.element)));
      };
      Promise.resolve(c(E)).then(w, (R) => {
        throw E.in || h(!0), R;
      });
    }
  }), b = jc(C, a.ref);
  return v && !r ? null : /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.cloneElement)(a, {
    ref: b
  });
}
function AA(a, r, u) {
  return a ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(q9, Object.assign({}, u, {
    component: a
  })) : r ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(t3, Object.assign({}, u, {
    transition: r
  })) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(W9, Object.assign({}, u));
}
const n3 = ["show", "role", "className", "style", "children", "backdrop", "keyboard", "onBackdropClick", "onEscapeKeyDown", "transition", "runTransition", "backdropTransition", "runBackdropTransition", "autoFocus", "enforceFocus", "restoreFocus", "restoreFocusOptions", "renderDialog", "renderBackdrop", "manager", "container", "onShow", "onHide", "onExit", "onExited", "onExiting", "onEnter", "onEntering", "onEntered"];
function r3(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
let DE;
function a3(a) {
  return DE || (DE = new Gx({
    ownerDocument: a == null ? void 0 : a.document
  })), DE;
}
function i3(a) {
  const r = kb(), u = a || a3(r), f = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)({
    dialog: null,
    backdrop: null
  });
  return Object.assign(f.current, {
    add: () => u.add(f.current),
    remove: () => u.remove(f.current),
    isTopModal: () => u.isTopModal(f.current),
    setDialogRef: (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((c) => {
      f.current.dialog = c;
    }, []),
    setBackdropRef: (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((c) => {
      f.current.backdrop = c;
    }, [])
  });
}
const zD = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.forwardRef)((a, r) => {
  let {
    show: u = !1,
    role: f = "dialog",
    className: c,
    style: v,
    children: h,
    backdrop: C = !0,
    keyboard: b = !0,
    onBackdropClick: E,
    onEscapeKeyDown: w,
    transition: R,
    runTransition: A,
    backdropTransition: D,
    runBackdropTransition: L,
    autoFocus: M = !0,
    enforceFocus: k = !0,
    restoreFocus: B = !0,
    restoreFocusOptions: Y,
    renderDialog: z,
    renderBackdrop: Z = (Et) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", Object.assign({}, Et)),
    manager: V,
    container: Q,
    onShow: P,
    onHide: U = () => {
    },
    onExit: se,
    onExited: Se,
    onExiting: he,
    onEnter: xe,
    onEntering: ye,
    onEntered: Me
  } = a, We = r3(a, n3);
  const Ee = kb(), tt = U9(Q), ue = i3(V), we = nD(), W = rD(u), [ce, ge] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(!u), Ie = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null);
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useImperativeHandle)(r, () => ue, [ue]), Ab && !W && u && (Ie.current = AE(Ee == null ? void 0 : Ee.document)), u && ce && ge(!1);
  const Ae = Un(() => {
    if (ue.add(), vt.current = cs(document, "keydown", He), nt.current = cs(
      document,
      "focus",
      // the timeout is necessary b/c this will run before the new modal is mounted
      // and so steals focus from it
      () => setTimeout(Oe),
      !0
    ), P && P(), M) {
      var Et, er;
      const In = AE((Et = (er = ue.dialog) == null ? void 0 : er.ownerDocument) != null ? Et : Ee == null ? void 0 : Ee.document);
      ue.dialog && In && !ub(ue.dialog, In) && (Ie.current = In, ue.dialog.focus());
    }
  }), De = Un(() => {
    if (ue.remove(), vt.current == null || vt.current(), nt.current == null || nt.current(), B) {
      var Et;
      (Et = Ie.current) == null || Et.focus == null || Et.focus(Y), Ie.current = null;
    }
  });
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    !u || !tt || Ae();
  }, [
    u,
    tt,
    /* should never change: */
    Ae
  ]), (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    ce && De();
  }, [ce, De]), G9(() => {
    De();
  });
  const Oe = Un(() => {
    if (!k || !we() || !ue.isTopModal())
      return;
    const Et = AE(Ee == null ? void 0 : Ee.document);
    ue.dialog && Et && !ub(ue.dialog, Et) && ue.dialog.focus();
  }), Ue = Un((Et) => {
    Et.target === Et.currentTarget && (E == null || E(Et), C === !0 && U());
  }), He = Un((Et) => {
    b && X9(Et) && ue.isTopModal() && (w == null || w(Et), Et.defaultPrevented || U());
  }), nt = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(), vt = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(), At = (...Et) => {
    ge(!0), Se == null || Se(...Et);
  };
  if (!tt)
    return null;
  const Ce = Object.assign({
    role: f,
    ref: ue.setDialogRef,
    // apparently only works on the dialog role element
    "aria-modal": f === "dialog" ? !0 : void 0
  }, We, {
    style: v,
    className: c,
    tabIndex: -1
  });
  let Ot = z ? z(Ce) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", Object.assign({}, Ce, {
    children: /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.cloneElement(h, {
      role: "document"
    })
  }));
  Ot = AA(R, A, {
    unmountOnExit: !0,
    mountOnEnter: !0,
    appear: !0,
    in: !!u,
    onExit: se,
    onExiting: he,
    onExited: At,
    onEnter: xe,
    onEntering: ye,
    onEntered: Me,
    children: Ot
  });
  let _t = null;
  return C && (_t = Z({
    ref: ue.setBackdropRef,
    onClick: Ue
  }), _t = AA(D, L, {
    in: !!u,
    appear: !0,
    mountOnEnter: !0,
    unmountOnExit: !0,
    children: _t
  })), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
    children: /* @__PURE__ */ $d.createPortal(/* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
      children: [_t, Ot]
    }), tt)
  });
});
zD.displayName = "Modal";
const o3 = Object.assign(zD, {
  Manager: Gx
}), GD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  ...f
}, c) => (r = ft(r, "offcanvas-body"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
GD.displayName = "OffcanvasBody";
const u3 = {
  [uo]: "show",
  [Vu]: "show"
}, _D = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  children: u,
  in: f = !1,
  mountOnEnter: c = !1,
  unmountOnExit: v = !1,
  appear: h = !1,
  ...C
}, b) => (a = ft(a, "offcanvas"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Mx, {
  ref: b,
  addEndListener: kx,
  in: f,
  mountOnEnter: c,
  unmountOnExit: v,
  appear: h,
  ...C,
  childRef: u.ref,
  children: (E, w) => /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.cloneElement(u, {
    ...w,
    className: Ke(r, u.props.className, (E === uo || E === Jh) && `${a}-toggling`, u3[E])
  })
})));
_D.displayName = "OffcanvasToggling";
const jD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext({
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  onHide() {
  }
}), l3 = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  closeLabel: a = "Close",
  closeVariant: r,
  closeButton: u = !1,
  onHide: f,
  children: c,
  ...v
}, h) => {
  const C = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(jD), b = Un(() => {
    C == null || C.onHide(), f == null || f();
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)("div", {
    ref: h,
    ...v,
    children: [c, u && /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ob, {
      "aria-label": a,
      variant: r,
      onClick: b
    })]
  });
}), UD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  closeLabel: u = "Close",
  closeButton: f = !1,
  ...c
}, v) => (a = ft(a, "offcanvas-header"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(l3, {
  ref: v,
  ...c,
  className: Ke(r, a),
  closeLabel: u,
  closeButton: f
})));
UD.displayName = "OffcanvasHeader";
const s3 = Rb("h5"), WD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = s3,
  ...f
}, c) => (r = ft(r, "offcanvas-title"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
WD.displayName = "OffcanvasTitle";
function c3(a, r) {
  return a.classList ? a.classList.contains(r) : (" " + (a.className.baseVal || a.className) + " ").indexOf(" " + r + " ") !== -1;
}
function f3(a, r) {
  a.classList ? a.classList.add(r) : c3(a, r) || (typeof a.className == "string" ? a.className = a.className + " " + r : a.setAttribute("class", (a.className && a.className.baseVal || "") + " " + r));
}
var d3 = Function.prototype.bind.call(Function.prototype.call, [].slice);
function rs(a, r) {
  return d3(a.querySelectorAll(r));
}
function OA(a, r) {
  return a.replace(new RegExp("(^|\\s)" + r + "(?:\\s|$)", "g"), "$1").replace(/\s+/g, " ").replace(/^\s*|\s*$/g, "");
}
function p3(a, r) {
  a.classList ? a.classList.remove(r) : typeof a.className == "string" ? a.className = OA(a.className, r) : a.setAttribute("class", OA(a.className && a.className.baseVal || "", r));
}
const Zd = {
  FIXED_CONTENT: ".fixed-top, .fixed-bottom, .is-fixed, .sticky-top",
  STICKY_CONTENT: ".sticky-top",
  NAVBAR_TOGGLER: ".navbar-toggler"
};
class XD extends Gx {
  adjustAndStore(r, u, f) {
    const c = u.style[r];
    u.dataset[r] = c, Pu(u, {
      [r]: `${parseFloat(Pu(u, r)) + f}px`
    });
  }
  restore(r, u) {
    const f = u.dataset[r];
    f !== void 0 && (delete u.dataset[r], Pu(u, {
      [r]: f
    }));
  }
  setContainerStyle(r) {
    super.setContainerStyle(r);
    const u = this.getElement();
    if (f3(u, "modal-open"), !r.scrollBarWidth)
      return;
    const f = this.isRTL ? "paddingLeft" : "paddingRight", c = this.isRTL ? "marginLeft" : "marginRight";
    rs(u, Zd.FIXED_CONTENT).forEach((v) => this.adjustAndStore(f, v, r.scrollBarWidth)), rs(u, Zd.STICKY_CONTENT).forEach((v) => this.adjustAndStore(c, v, -r.scrollBarWidth)), rs(u, Zd.NAVBAR_TOGGLER).forEach((v) => this.adjustAndStore(c, v, r.scrollBarWidth));
  }
  removeContainerStyle(r) {
    super.removeContainerStyle(r);
    const u = this.getElement();
    p3(u, "modal-open");
    const f = this.isRTL ? "paddingLeft" : "paddingRight", c = this.isRTL ? "marginLeft" : "marginRight";
    rs(u, Zd.FIXED_CONTENT).forEach((v) => this.restore(f, v)), rs(u, Zd.STICKY_CONTENT).forEach((v) => this.restore(c, v)), rs(u, Zd.NAVBAR_TOGGLER).forEach((v) => this.restore(c, v));
  }
}
let NE;
function v3(a) {
  return NE || (NE = new XD(a)), NE;
}
function h3(a) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(_D, {
    ...a
  });
}
function m3(a) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(ab, {
    ...a
  });
}
const ZD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  children: u,
  "aria-labelledby": f,
  placement: c = "start",
  responsive: v,
  /* BaseModal props */
  show: h = !1,
  backdrop: C = !0,
  keyboard: b = !0,
  scroll: E = !1,
  onEscapeKeyDown: w,
  onShow: R,
  onHide: A,
  container: D,
  autoFocus: L = !0,
  enforceFocus: M = !0,
  restoreFocus: k = !0,
  restoreFocusOptions: B,
  onEntered: Y,
  onExit: z,
  onExiting: Z,
  onEnter: V,
  onEntering: Q,
  onExited: P,
  backdropClassName: U,
  manager: se,
  renderStaticNode: Se = !1,
  ...he
}, xe) => {
  const ye = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)();
  a = ft(a, "offcanvas");
  const {
    onToggle: Me
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Uc) || {}, [We, Ee] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(!1), tt = H9(v || "xs", "up");
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    Ee(v ? h && !tt : h);
  }, [h, v, tt]);
  const ue = Un(() => {
    Me == null || Me(), A == null || A();
  }), we = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    onHide: ue
  }), [ue]);
  function W() {
    return se || (E ? (ye.current || (ye.current = new XD({
      handleContainerOverflow: !1
    })), ye.current) : v3());
  }
  const ce = (De, ...Oe) => {
    De && (De.style.visibility = "visible"), V == null || V(De, ...Oe);
  }, ge = (De, ...Oe) => {
    De && (De.style.visibility = ""), P == null || P(...Oe);
  }, Ie = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((De) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
    ...De,
    className: Ke(`${a}-backdrop`, U)
  }), [U, a]), Ae = (De) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
    ...De,
    ...he,
    className: Ke(r, v ? `${a}-${v}` : a, `${a}-${c}`),
    "aria-labelledby": f,
    children: u
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
    children: [!We && (v || Se) && Ae({}), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(jD.Provider, {
      value: we,
      children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(o3, {
        show: We,
        ref: xe,
        backdrop: C,
        container: D,
        keyboard: b,
        autoFocus: L,
        enforceFocus: M && !E,
        restoreFocus: k,
        restoreFocusOptions: B,
        onEscapeKeyDown: w,
        onShow: R,
        onHide: ue,
        onEnter: ce,
        onEntering: Q,
        onEntered: Y,
        onExit: z,
        onExiting: Z,
        onExited: ge,
        manager: W(),
        transition: h3,
        backdropTransition: m3,
        renderBackdrop: Ie,
        renderDialog: Ae
      })
    })]
  });
});
ZD.displayName = "Offcanvas";
const g3 = Object.assign(ZD, {
  Body: GD,
  Header: UD,
  Title: WD
}), YD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  const u = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Uc);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(g3, {
    ref: r,
    show: !!(u != null && u.expanded),
    ...a,
    renderStaticNode: !0
  });
});
YD.displayName = "NavbarOffcanvas";
const $D = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "span",
  ...f
}, c) => (r = ft(r, "navbar-text"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
$D.displayName = "NavbarText";
const QD = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  const {
    bsPrefix: u,
    expand: f = !0,
    variant: c = "light",
    bg: v,
    fixed: h,
    sticky: C,
    className: b,
    // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
    as: E = "nav",
    expanded: w,
    onToggle: R,
    onSelect: A,
    collapseOnSelect: D = !1,
    ...L
  } = Ox(a, {
    expanded: "onToggle"
  }), M = ft(u, "navbar"), k = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((...z) => {
    A == null || A(...z), D && w && (R == null || R(!1));
  }, [A, D, w, R]);
  L.role === void 0 && E !== "nav" && (L.role = "navigation");
  let B = `${M}-expand`;
  typeof f == "string" && (B = `${B}-${f}`);
  const Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    onToggle: () => R == null ? void 0 : R(!w),
    bsPrefix: M,
    expanded: !!w,
    expand: f
  }), [M, w, f, R]);
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Uc.Provider, {
    value: Y,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(em.Provider, {
      value: k,
      children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(E, {
        ref: r,
        ...L,
        className: Ke(b, M, f && B, c && `${M}-${c}`, v && `bg-${v}`, C && `sticky-${C}`, h && `fixed-${h}`)
      })
    })
  });
});
QD.displayName = "Navbar";
const Jd = Object.assign(QD, {
  Brand: BD,
  Collapse: VD,
  Offcanvas: YD,
  Text: $D,
  Toggle: PD
});
function y3(a, r, u) {
  const f = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(a !== void 0), [c, v] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(r), h = a !== void 0, C = f.current;
  return f.current = h, !h && C && c !== r && v(r), [h ? a : c, (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((...b) => {
    const [E, ...w] = b;
    let R = u == null ? void 0 : u(E, ...w);
    return v(E), R;
  }, [u])];
}
function b3() {
  const [, a] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useReducer)((r) => !r, !1);
  return a;
}
const Mb = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null);
var DA = Object.prototype.hasOwnProperty;
function NA(a, r, u) {
  for (u of a.keys())
    if (Yh(u, r))
      return u;
}
function Yh(a, r) {
  var u, f, c;
  if (a === r)
    return !0;
  if (a && r && (u = a.constructor) === r.constructor) {
    if (u === Date)
      return a.getTime() === r.getTime();
    if (u === RegExp)
      return a.toString() === r.toString();
    if (u === Array) {
      if ((f = a.length) === r.length)
        for (; f-- && Yh(a[f], r[f]); )
          ;
      return f === -1;
    }
    if (u === Set) {
      if (a.size !== r.size)
        return !1;
      for (f of a)
        if (c = f, c && typeof c == "object" && (c = NA(r, c), !c) || !r.has(c))
          return !1;
      return !0;
    }
    if (u === Map) {
      if (a.size !== r.size)
        return !1;
      for (f of a)
        if (c = f[0], c && typeof c == "object" && (c = NA(r, c), !c) || !Yh(f[1], r.get(c)))
          return !1;
      return !0;
    }
    if (u === ArrayBuffer)
      a = new Uint8Array(a), r = new Uint8Array(r);
    else if (u === DataView) {
      if ((f = a.byteLength) === r.byteLength)
        for (; f-- && a.getInt8(f) === r.getInt8(f); )
          ;
      return f === -1;
    }
    if (ArrayBuffer.isView(a)) {
      if ((f = a.byteLength) === r.byteLength)
        for (; f-- && a[f] === r[f]; )
          ;
      return f === -1;
    }
    if (!u || typeof a == "object") {
      f = 0;
      for (u in a)
        if (DA.call(a, u) && ++f && !DA.call(r, u) || !(u in r) || !Yh(a[u], r[u]))
          return !1;
      return Object.keys(r).length === f;
    }
  }
  return a !== a && r !== r;
}
function C3(a) {
  const r = nD();
  return [a[0], (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((u) => {
    if (r())
      return a[1](u);
  }, [r, a[1]])];
}
var Ua = "top", Di = "bottom", Ni = "right", Wa = "left", _x = "auto", xm = [Ua, Di, Ni, Wa], mp = "start", tm = "end", S3 = "clippingParents", JD = "viewport", Hh = "popper", E3 = "reference", kA = /* @__PURE__ */ xm.reduce(function(a, r) {
  return a.concat([r + "-" + mp, r + "-" + tm]);
}, []), KD = /* @__PURE__ */ [].concat(xm, [_x]).reduce(function(a, r) {
  return a.concat([r, r + "-" + mp, r + "-" + tm]);
}, []), x3 = "beforeRead", w3 = "read", T3 = "afterRead", R3 = "beforeMain", I3 = "main", A3 = "afterMain", O3 = "beforeWrite", D3 = "write", N3 = "afterWrite", k3 = [x3, w3, T3, R3, I3, A3, O3, D3, N3];
function Xo(a) {
  return a.split("-")[0];
}
function si(a) {
  if (a == null)
    return window;
  if (a.toString() !== "[object Window]") {
    var r = a.ownerDocument;
    return r && r.defaultView || window;
  }
  return a;
}
function Gc(a) {
  var r = si(a).Element;
  return a instanceof r || a instanceof Element;
}
function Zo(a) {
  var r = si(a).HTMLElement;
  return a instanceof r || a instanceof HTMLElement;
}
function jx(a) {
  if (typeof ShadowRoot > "u")
    return !1;
  var r = si(a).ShadowRoot;
  return a instanceof r || a instanceof ShadowRoot;
}
var Pc = Math.max, lb = Math.min, gp = Math.round;
function cx() {
  var a = navigator.userAgentData;
  return a != null && a.brands && Array.isArray(a.brands) ? a.brands.map(function(r) {
    return r.brand + "/" + r.version;
  }).join(" ") : navigator.userAgent;
}
function qD() {
  return !/^((?!chrome|android).)*safari/i.test(cx());
}
function yp(a, r, u) {
  r === void 0 && (r = !1), u === void 0 && (u = !1);
  var f = a.getBoundingClientRect(), c = 1, v = 1;
  r && Zo(a) && (c = a.offsetWidth > 0 && gp(f.width) / a.offsetWidth || 1, v = a.offsetHeight > 0 && gp(f.height) / a.offsetHeight || 1);
  var h = Gc(a) ? si(a) : window, C = h.visualViewport, b = !qD() && u, E = (f.left + (b && C ? C.offsetLeft : 0)) / c, w = (f.top + (b && C ? C.offsetTop : 0)) / v, R = f.width / c, A = f.height / v;
  return {
    width: R,
    height: A,
    top: w,
    right: E + R,
    bottom: w + A,
    left: E,
    x: E,
    y: w
  };
}
function Ux(a) {
  var r = yp(a), u = a.offsetWidth, f = a.offsetHeight;
  return Math.abs(r.width - u) <= 1 && (u = r.width), Math.abs(r.height - f) <= 1 && (f = r.height), {
    x: a.offsetLeft,
    y: a.offsetTop,
    width: u,
    height: f
  };
}
function eN(a, r) {
  var u = r.getRootNode && r.getRootNode();
  if (a.contains(r))
    return !0;
  if (u && jx(u)) {
    var f = r;
    do {
      if (f && a.isSameNode(f))
        return !0;
      f = f.parentNode || f.host;
    } while (f);
  }
  return !1;
}
function fs(a) {
  return a ? (a.nodeName || "").toLowerCase() : null;
}
function ju(a) {
  return si(a).getComputedStyle(a);
}
function M3(a) {
  return ["table", "td", "th"].indexOf(fs(a)) >= 0;
}
function ds(a) {
  return ((Gc(a) ? a.ownerDocument : (
    // $FlowFixMe[prop-missing]
    a.document
  )) || window.document).documentElement;
}
function Lb(a) {
  return fs(a) === "html" ? a : (
    // this is a quicker (but less type safe) way to save quite some bytes from the bundle
    // $FlowFixMe[incompatible-return]
    // $FlowFixMe[prop-missing]
    a.assignedSlot || // step into the shadow DOM of the parent of a slotted node
    a.parentNode || // DOM Element detected
    (jx(a) ? a.host : null) || // ShadowRoot detected
    // $FlowFixMe[incompatible-call]: HTMLElement is a Node
    ds(a)
  );
}
function MA(a) {
  return !Zo(a) || // https://github.com/popperjs/popper-core/issues/837
  ju(a).position === "fixed" ? null : a.offsetParent;
}
function L3(a) {
  var r = /firefox/i.test(cx()), u = /Trident/i.test(cx());
  if (u && Zo(a)) {
    var f = ju(a);
    if (f.position === "fixed")
      return null;
  }
  var c = Lb(a);
  for (jx(c) && (c = c.host); Zo(c) && ["html", "body"].indexOf(fs(c)) < 0; ) {
    var v = ju(c);
    if (v.transform !== "none" || v.perspective !== "none" || v.contain === "paint" || ["transform", "perspective"].indexOf(v.willChange) !== -1 || r && v.willChange === "filter" || r && v.filter && v.filter !== "none")
      return c;
    c = c.parentNode;
  }
  return null;
}
function wm(a) {
  for (var r = si(a), u = MA(a); u && M3(u) && ju(u).position === "static"; )
    u = MA(u);
  return u && (fs(u) === "html" || fs(u) === "body" && ju(u).position === "static") ? r : u || L3(a) || r;
}
function Wx(a) {
  return ["top", "bottom"].indexOf(a) >= 0 ? "x" : "y";
}
function $h(a, r, u) {
  return Pc(a, lb(r, u));
}
function F3(a, r, u) {
  var f = $h(a, r, u);
  return f > u ? u : f;
}
function tN() {
  return {
    top: 0,
    right: 0,
    bottom: 0,
    left: 0
  };
}
function nN(a) {
  return Object.assign({}, tN(), a);
}
function rN(a, r) {
  return r.reduce(function(u, f) {
    return u[f] = a, u;
  }, {});
}
var B3 = function(r, u) {
  return r = typeof r == "function" ? r(Object.assign({}, u.rects, {
    placement: u.placement
  })) : r, nN(typeof r != "number" ? r : rN(r, xm));
};
function V3(a) {
  var r, u = a.state, f = a.name, c = a.options, v = u.elements.arrow, h = u.modifiersData.popperOffsets, C = Xo(u.placement), b = Wx(C), E = [Wa, Ni].indexOf(C) >= 0, w = E ? "height" : "width";
  if (!(!v || !h)) {
    var R = B3(c.padding, u), A = Ux(v), D = b === "y" ? Ua : Wa, L = b === "y" ? Di : Ni, M = u.rects.reference[w] + u.rects.reference[b] - h[b] - u.rects.popper[w], k = h[b] - u.rects.reference[b], B = wm(v), Y = B ? b === "y" ? B.clientHeight || 0 : B.clientWidth || 0 : 0, z = M / 2 - k / 2, Z = R[D], V = Y - A[w] - R[L], Q = Y / 2 - A[w] / 2 + z, P = $h(Z, Q, V), U = b;
    u.modifiersData[f] = (r = {}, r[U] = P, r.centerOffset = P - Q, r);
  }
}
function P3(a) {
  var r = a.state, u = a.options, f = u.element, c = f === void 0 ? "[data-popper-arrow]" : f;
  c != null && (typeof c == "string" && (c = r.elements.popper.querySelector(c), !c) || eN(r.elements.popper, c) && (r.elements.arrow = c));
}
const H3 = {
  name: "arrow",
  enabled: !0,
  phase: "main",
  fn: V3,
  effect: P3,
  requires: ["popperOffsets"],
  requiresIfExists: ["preventOverflow"]
};
function bp(a) {
  return a.split("-")[1];
}
var z3 = {
  top: "auto",
  right: "auto",
  bottom: "auto",
  left: "auto"
};
function G3(a, r) {
  var u = a.x, f = a.y, c = r.devicePixelRatio || 1;
  return {
    x: gp(u * c) / c || 0,
    y: gp(f * c) / c || 0
  };
}
function LA(a) {
  var r, u = a.popper, f = a.popperRect, c = a.placement, v = a.variation, h = a.offsets, C = a.position, b = a.gpuAcceleration, E = a.adaptive, w = a.roundOffsets, R = a.isFixed, A = h.x, D = A === void 0 ? 0 : A, L = h.y, M = L === void 0 ? 0 : L, k = typeof w == "function" ? w({
    x: D,
    y: M
  }) : {
    x: D,
    y: M
  };
  D = k.x, M = k.y;
  var B = h.hasOwnProperty("x"), Y = h.hasOwnProperty("y"), z = Wa, Z = Ua, V = window;
  if (E) {
    var Q = wm(u), P = "clientHeight", U = "clientWidth";
    if (Q === si(u) && (Q = ds(u), ju(Q).position !== "static" && C === "absolute" && (P = "scrollHeight", U = "scrollWidth")), Q = Q, c === Ua || (c === Wa || c === Ni) && v === tm) {
      Z = Di;
      var se = R && Q === V && V.visualViewport ? V.visualViewport.height : (
        // $FlowFixMe[prop-missing]
        Q[P]
      );
      M -= se - f.height, M *= b ? 1 : -1;
    }
    if (c === Wa || (c === Ua || c === Di) && v === tm) {
      z = Ni;
      var Se = R && Q === V && V.visualViewport ? V.visualViewport.width : (
        // $FlowFixMe[prop-missing]
        Q[U]
      );
      D -= Se - f.width, D *= b ? 1 : -1;
    }
  }
  var he = Object.assign({
    position: C
  }, E && z3), xe = w === !0 ? G3({
    x: D,
    y: M
  }, si(u)) : {
    x: D,
    y: M
  };
  if (D = xe.x, M = xe.y, b) {
    var ye;
    return Object.assign({}, he, (ye = {}, ye[Z] = Y ? "0" : "", ye[z] = B ? "0" : "", ye.transform = (V.devicePixelRatio || 1) <= 1 ? "translate(" + D + "px, " + M + "px)" : "translate3d(" + D + "px, " + M + "px, 0)", ye));
  }
  return Object.assign({}, he, (r = {}, r[Z] = Y ? M + "px" : "", r[z] = B ? D + "px" : "", r.transform = "", r));
}
function _3(a) {
  var r = a.state, u = a.options, f = u.gpuAcceleration, c = f === void 0 ? !0 : f, v = u.adaptive, h = v === void 0 ? !0 : v, C = u.roundOffsets, b = C === void 0 ? !0 : C, E = {
    placement: Xo(r.placement),
    variation: bp(r.placement),
    popper: r.elements.popper,
    popperRect: r.rects.popper,
    gpuAcceleration: c,
    isFixed: r.options.strategy === "fixed"
  };
  r.modifiersData.popperOffsets != null && (r.styles.popper = Object.assign({}, r.styles.popper, LA(Object.assign({}, E, {
    offsets: r.modifiersData.popperOffsets,
    position: r.options.strategy,
    adaptive: h,
    roundOffsets: b
  })))), r.modifiersData.arrow != null && (r.styles.arrow = Object.assign({}, r.styles.arrow, LA(Object.assign({}, E, {
    offsets: r.modifiersData.arrow,
    position: "absolute",
    adaptive: !1,
    roundOffsets: b
  })))), r.attributes.popper = Object.assign({}, r.attributes.popper, {
    "data-popper-placement": r.placement
  });
}
const j3 = {
  name: "computeStyles",
  enabled: !0,
  phase: "beforeWrite",
  fn: _3,
  data: {}
};
var _0 = {
  passive: !0
};
function U3(a) {
  var r = a.state, u = a.instance, f = a.options, c = f.scroll, v = c === void 0 ? !0 : c, h = f.resize, C = h === void 0 ? !0 : h, b = si(r.elements.popper), E = [].concat(r.scrollParents.reference, r.scrollParents.popper);
  return v && E.forEach(function(w) {
    w.addEventListener("scroll", u.update, _0);
  }), C && b.addEventListener("resize", u.update, _0), function() {
    v && E.forEach(function(w) {
      w.removeEventListener("scroll", u.update, _0);
    }), C && b.removeEventListener("resize", u.update, _0);
  };
}
const W3 = {
  name: "eventListeners",
  enabled: !0,
  phase: "write",
  fn: function() {
  },
  effect: U3,
  data: {}
};
var X3 = {
  left: "right",
  right: "left",
  bottom: "top",
  top: "bottom"
};
function $0(a) {
  return a.replace(/left|right|bottom|top/g, function(r) {
    return X3[r];
  });
}
var Z3 = {
  start: "end",
  end: "start"
};
function FA(a) {
  return a.replace(/start|end/g, function(r) {
    return Z3[r];
  });
}
function Xx(a) {
  var r = si(a), u = r.pageXOffset, f = r.pageYOffset;
  return {
    scrollLeft: u,
    scrollTop: f
  };
}
function Zx(a) {
  return yp(ds(a)).left + Xx(a).scrollLeft;
}
function Y3(a, r) {
  var u = si(a), f = ds(a), c = u.visualViewport, v = f.clientWidth, h = f.clientHeight, C = 0, b = 0;
  if (c) {
    v = c.width, h = c.height;
    var E = qD();
    (E || !E && r === "fixed") && (C = c.offsetLeft, b = c.offsetTop);
  }
  return {
    width: v,
    height: h,
    x: C + Zx(a),
    y: b
  };
}
function $3(a) {
  var r, u = ds(a), f = Xx(a), c = (r = a.ownerDocument) == null ? void 0 : r.body, v = Pc(u.scrollWidth, u.clientWidth, c ? c.scrollWidth : 0, c ? c.clientWidth : 0), h = Pc(u.scrollHeight, u.clientHeight, c ? c.scrollHeight : 0, c ? c.clientHeight : 0), C = -f.scrollLeft + Zx(a), b = -f.scrollTop;
  return ju(c || u).direction === "rtl" && (C += Pc(u.clientWidth, c ? c.clientWidth : 0) - v), {
    width: v,
    height: h,
    x: C,
    y: b
  };
}
function Yx(a) {
  var r = ju(a), u = r.overflow, f = r.overflowX, c = r.overflowY;
  return /auto|scroll|overlay|hidden/.test(u + c + f);
}
function aN(a) {
  return ["html", "body", "#document"].indexOf(fs(a)) >= 0 ? a.ownerDocument.body : Zo(a) && Yx(a) ? a : aN(Lb(a));
}
function Qh(a, r) {
  var u;
  r === void 0 && (r = []);
  var f = aN(a), c = f === ((u = a.ownerDocument) == null ? void 0 : u.body), v = si(f), h = c ? [v].concat(v.visualViewport || [], Yx(f) ? f : []) : f, C = r.concat(h);
  return c ? C : (
    // $FlowFixMe[incompatible-call]: isBody tells us target will be an HTMLElement here
    C.concat(Qh(Lb(h)))
  );
}
function fx(a) {
  return Object.assign({}, a, {
    left: a.x,
    top: a.y,
    right: a.x + a.width,
    bottom: a.y + a.height
  });
}
function Q3(a, r) {
  var u = yp(a, !1, r === "fixed");
  return u.top = u.top + a.clientTop, u.left = u.left + a.clientLeft, u.bottom = u.top + a.clientHeight, u.right = u.left + a.clientWidth, u.width = a.clientWidth, u.height = a.clientHeight, u.x = u.left, u.y = u.top, u;
}
function BA(a, r, u) {
  return r === JD ? fx(Y3(a, u)) : Gc(r) ? Q3(r, u) : fx($3(ds(a)));
}
function J3(a) {
  var r = Qh(Lb(a)), u = ["absolute", "fixed"].indexOf(ju(a).position) >= 0, f = u && Zo(a) ? wm(a) : a;
  return Gc(f) ? r.filter(function(c) {
    return Gc(c) && eN(c, f) && fs(c) !== "body";
  }) : [];
}
function K3(a, r, u, f) {
  var c = r === "clippingParents" ? J3(a) : [].concat(r), v = [].concat(c, [u]), h = v[0], C = v.reduce(function(b, E) {
    var w = BA(a, E, f);
    return b.top = Pc(w.top, b.top), b.right = lb(w.right, b.right), b.bottom = lb(w.bottom, b.bottom), b.left = Pc(w.left, b.left), b;
  }, BA(a, h, f));
  return C.width = C.right - C.left, C.height = C.bottom - C.top, C.x = C.left, C.y = C.top, C;
}
function iN(a) {
  var r = a.reference, u = a.element, f = a.placement, c = f ? Xo(f) : null, v = f ? bp(f) : null, h = r.x + r.width / 2 - u.width / 2, C = r.y + r.height / 2 - u.height / 2, b;
  switch (c) {
    case Ua:
      b = {
        x: h,
        y: r.y - u.height
      };
      break;
    case Di:
      b = {
        x: h,
        y: r.y + r.height
      };
      break;
    case Ni:
      b = {
        x: r.x + r.width,
        y: C
      };
      break;
    case Wa:
      b = {
        x: r.x - u.width,
        y: C
      };
      break;
    default:
      b = {
        x: r.x,
        y: r.y
      };
  }
  var E = c ? Wx(c) : null;
  if (E != null) {
    var w = E === "y" ? "height" : "width";
    switch (v) {
      case mp:
        b[E] = b[E] - (r[w] / 2 - u[w] / 2);
        break;
      case tm:
        b[E] = b[E] + (r[w] / 2 - u[w] / 2);
        break;
    }
  }
  return b;
}
function nm(a, r) {
  r === void 0 && (r = {});
  var u = r, f = u.placement, c = f === void 0 ? a.placement : f, v = u.strategy, h = v === void 0 ? a.strategy : v, C = u.boundary, b = C === void 0 ? S3 : C, E = u.rootBoundary, w = E === void 0 ? JD : E, R = u.elementContext, A = R === void 0 ? Hh : R, D = u.altBoundary, L = D === void 0 ? !1 : D, M = u.padding, k = M === void 0 ? 0 : M, B = nN(typeof k != "number" ? k : rN(k, xm)), Y = A === Hh ? E3 : Hh, z = a.rects.popper, Z = a.elements[L ? Y : A], V = K3(Gc(Z) ? Z : Z.contextElement || ds(a.elements.popper), b, w, h), Q = yp(a.elements.reference), P = iN({
    reference: Q,
    element: z,
    strategy: "absolute",
    placement: c
  }), U = fx(Object.assign({}, z, P)), se = A === Hh ? U : Q, Se = {
    top: V.top - se.top + B.top,
    bottom: se.bottom - V.bottom + B.bottom,
    left: V.left - se.left + B.left,
    right: se.right - V.right + B.right
  }, he = a.modifiersData.offset;
  if (A === Hh && he) {
    var xe = he[c];
    Object.keys(Se).forEach(function(ye) {
      var Me = [Ni, Di].indexOf(ye) >= 0 ? 1 : -1, We = [Ua, Di].indexOf(ye) >= 0 ? "y" : "x";
      Se[ye] += xe[We] * Me;
    });
  }
  return Se;
}
function q3(a, r) {
  r === void 0 && (r = {});
  var u = r, f = u.placement, c = u.boundary, v = u.rootBoundary, h = u.padding, C = u.flipVariations, b = u.allowedAutoPlacements, E = b === void 0 ? KD : b, w = bp(f), R = w ? C ? kA : kA.filter(function(L) {
    return bp(L) === w;
  }) : xm, A = R.filter(function(L) {
    return E.indexOf(L) >= 0;
  });
  A.length === 0 && (A = R);
  var D = A.reduce(function(L, M) {
    return L[M] = nm(a, {
      placement: M,
      boundary: c,
      rootBoundary: v,
      padding: h
    })[Xo(M)], L;
  }, {});
  return Object.keys(D).sort(function(L, M) {
    return D[L] - D[M];
  });
}
function eP(a) {
  if (Xo(a) === _x)
    return [];
  var r = $0(a);
  return [FA(a), r, FA(r)];
}
function tP(a) {
  var r = a.state, u = a.options, f = a.name;
  if (!r.modifiersData[f]._skip) {
    for (var c = u.mainAxis, v = c === void 0 ? !0 : c, h = u.altAxis, C = h === void 0 ? !0 : h, b = u.fallbackPlacements, E = u.padding, w = u.boundary, R = u.rootBoundary, A = u.altBoundary, D = u.flipVariations, L = D === void 0 ? !0 : D, M = u.allowedAutoPlacements, k = r.options.placement, B = Xo(k), Y = B === k, z = b || (Y || !L ? [$0(k)] : eP(k)), Z = [k].concat(z).reduce(function(Ae, De) {
      return Ae.concat(Xo(De) === _x ? q3(r, {
        placement: De,
        boundary: w,
        rootBoundary: R,
        padding: E,
        flipVariations: L,
        allowedAutoPlacements: M
      }) : De);
    }, []), V = r.rects.reference, Q = r.rects.popper, P = /* @__PURE__ */ new Map(), U = !0, se = Z[0], Se = 0; Se < Z.length; Se++) {
      var he = Z[Se], xe = Xo(he), ye = bp(he) === mp, Me = [Ua, Di].indexOf(xe) >= 0, We = Me ? "width" : "height", Ee = nm(r, {
        placement: he,
        boundary: w,
        rootBoundary: R,
        altBoundary: A,
        padding: E
      }), tt = Me ? ye ? Ni : Wa : ye ? Di : Ua;
      V[We] > Q[We] && (tt = $0(tt));
      var ue = $0(tt), we = [];
      if (v && we.push(Ee[xe] <= 0), C && we.push(Ee[tt] <= 0, Ee[ue] <= 0), we.every(function(Ae) {
        return Ae;
      })) {
        se = he, U = !1;
        break;
      }
      P.set(he, we);
    }
    if (U)
      for (var W = L ? 3 : 1, ce = function(De) {
        var Oe = Z.find(function(Ue) {
          var He = P.get(Ue);
          if (He)
            return He.slice(0, De).every(function(nt) {
              return nt;
            });
        });
        if (Oe)
          return se = Oe, "break";
      }, ge = W; ge > 0; ge--) {
        var Ie = ce(ge);
        if (Ie === "break")
          break;
      }
    r.placement !== se && (r.modifiersData[f]._skip = !0, r.placement = se, r.reset = !0);
  }
}
const nP = {
  name: "flip",
  enabled: !0,
  phase: "main",
  fn: tP,
  requiresIfExists: ["offset"],
  data: {
    _skip: !1
  }
};
function VA(a, r, u) {
  return u === void 0 && (u = {
    x: 0,
    y: 0
  }), {
    top: a.top - r.height - u.y,
    right: a.right - r.width + u.x,
    bottom: a.bottom - r.height + u.y,
    left: a.left - r.width - u.x
  };
}
function PA(a) {
  return [Ua, Ni, Di, Wa].some(function(r) {
    return a[r] >= 0;
  });
}
function rP(a) {
  var r = a.state, u = a.name, f = r.rects.reference, c = r.rects.popper, v = r.modifiersData.preventOverflow, h = nm(r, {
    elementContext: "reference"
  }), C = nm(r, {
    altBoundary: !0
  }), b = VA(h, f), E = VA(C, c, v), w = PA(b), R = PA(E);
  r.modifiersData[u] = {
    referenceClippingOffsets: b,
    popperEscapeOffsets: E,
    isReferenceHidden: w,
    hasPopperEscaped: R
  }, r.attributes.popper = Object.assign({}, r.attributes.popper, {
    "data-popper-reference-hidden": w,
    "data-popper-escaped": R
  });
}
const aP = {
  name: "hide",
  enabled: !0,
  phase: "main",
  requiresIfExists: ["preventOverflow"],
  fn: rP
};
function iP(a, r, u) {
  var f = Xo(a), c = [Wa, Ua].indexOf(f) >= 0 ? -1 : 1, v = typeof u == "function" ? u(Object.assign({}, r, {
    placement: a
  })) : u, h = v[0], C = v[1];
  return h = h || 0, C = (C || 0) * c, [Wa, Ni].indexOf(f) >= 0 ? {
    x: C,
    y: h
  } : {
    x: h,
    y: C
  };
}
function oP(a) {
  var r = a.state, u = a.options, f = a.name, c = u.offset, v = c === void 0 ? [0, 0] : c, h = KD.reduce(function(w, R) {
    return w[R] = iP(R, r.rects, v), w;
  }, {}), C = h[r.placement], b = C.x, E = C.y;
  r.modifiersData.popperOffsets != null && (r.modifiersData.popperOffsets.x += b, r.modifiersData.popperOffsets.y += E), r.modifiersData[f] = h;
}
const uP = {
  name: "offset",
  enabled: !0,
  phase: "main",
  requires: ["popperOffsets"],
  fn: oP
};
function lP(a) {
  var r = a.state, u = a.name;
  r.modifiersData[u] = iN({
    reference: r.rects.reference,
    element: r.rects.popper,
    strategy: "absolute",
    placement: r.placement
  });
}
const sP = {
  name: "popperOffsets",
  enabled: !0,
  phase: "read",
  fn: lP,
  data: {}
};
function cP(a) {
  return a === "x" ? "y" : "x";
}
function fP(a) {
  var r = a.state, u = a.options, f = a.name, c = u.mainAxis, v = c === void 0 ? !0 : c, h = u.altAxis, C = h === void 0 ? !1 : h, b = u.boundary, E = u.rootBoundary, w = u.altBoundary, R = u.padding, A = u.tether, D = A === void 0 ? !0 : A, L = u.tetherOffset, M = L === void 0 ? 0 : L, k = nm(r, {
    boundary: b,
    rootBoundary: E,
    padding: R,
    altBoundary: w
  }), B = Xo(r.placement), Y = bp(r.placement), z = !Y, Z = Wx(B), V = cP(Z), Q = r.modifiersData.popperOffsets, P = r.rects.reference, U = r.rects.popper, se = typeof M == "function" ? M(Object.assign({}, r.rects, {
    placement: r.placement
  })) : M, Se = typeof se == "number" ? {
    mainAxis: se,
    altAxis: se
  } : Object.assign({
    mainAxis: 0,
    altAxis: 0
  }, se), he = r.modifiersData.offset ? r.modifiersData.offset[r.placement] : null, xe = {
    x: 0,
    y: 0
  };
  if (Q) {
    if (v) {
      var ye, Me = Z === "y" ? Ua : Wa, We = Z === "y" ? Di : Ni, Ee = Z === "y" ? "height" : "width", tt = Q[Z], ue = tt + k[Me], we = tt - k[We], W = D ? -U[Ee] / 2 : 0, ce = Y === mp ? P[Ee] : U[Ee], ge = Y === mp ? -U[Ee] : -P[Ee], Ie = r.elements.arrow, Ae = D && Ie ? Ux(Ie) : {
        width: 0,
        height: 0
      }, De = r.modifiersData["arrow#persistent"] ? r.modifiersData["arrow#persistent"].padding : tN(), Oe = De[Me], Ue = De[We], He = $h(0, P[Ee], Ae[Ee]), nt = z ? P[Ee] / 2 - W - He - Oe - Se.mainAxis : ce - He - Oe - Se.mainAxis, vt = z ? -P[Ee] / 2 + W + He + Ue + Se.mainAxis : ge + He + Ue + Se.mainAxis, At = r.elements.arrow && wm(r.elements.arrow), Ce = At ? Z === "y" ? At.clientTop || 0 : At.clientLeft || 0 : 0, Ot = (ye = he == null ? void 0 : he[Z]) != null ? ye : 0, _t = tt + nt - Ot - Ce, Et = tt + vt - Ot, er = $h(D ? lb(ue, _t) : ue, tt, D ? Pc(we, Et) : we);
      Q[Z] = er, xe[Z] = er - tt;
    }
    if (C) {
      var In, Ya = Z === "x" ? Ua : Wa, so = Z === "x" ? Di : Ni, Sr = Q[V], ta = V === "y" ? "height" : "width", ci = Sr + k[Ya], Fr = Sr - k[so], ga = [Ua, Wa].indexOf(B) !== -1, na = (In = he == null ? void 0 : he[V]) != null ? In : 0, $a = ga ? ci : Sr - P[ta] - U[ta] - na + Se.altAxis, tr = ga ? Sr + P[ta] + U[ta] - na - Se.altAxis : Fr, Br = D && ga ? F3($a, Sr, tr) : $h(D ? $a : ci, Sr, D ? tr : Fr);
      Q[V] = Br, xe[V] = Br - Sr;
    }
    r.modifiersData[f] = xe;
  }
}
const dP = {
  name: "preventOverflow",
  enabled: !0,
  phase: "main",
  fn: fP,
  requiresIfExists: ["offset"]
};
function pP(a) {
  return {
    scrollLeft: a.scrollLeft,
    scrollTop: a.scrollTop
  };
}
function vP(a) {
  return a === si(a) || !Zo(a) ? Xx(a) : pP(a);
}
function hP(a) {
  var r = a.getBoundingClientRect(), u = gp(r.width) / a.offsetWidth || 1, f = gp(r.height) / a.offsetHeight || 1;
  return u !== 1 || f !== 1;
}
function mP(a, r, u) {
  u === void 0 && (u = !1);
  var f = Zo(r), c = Zo(r) && hP(r), v = ds(r), h = yp(a, c, u), C = {
    scrollLeft: 0,
    scrollTop: 0
  }, b = {
    x: 0,
    y: 0
  };
  return (f || !f && !u) && ((fs(r) !== "body" || // https://github.com/popperjs/popper-core/issues/1078
  Yx(v)) && (C = vP(r)), Zo(r) ? (b = yp(r, !0), b.x += r.clientLeft, b.y += r.clientTop) : v && (b.x = Zx(v))), {
    x: h.left + C.scrollLeft - b.x,
    y: h.top + C.scrollTop - b.y,
    width: h.width,
    height: h.height
  };
}
function gP(a) {
  var r = /* @__PURE__ */ new Map(), u = /* @__PURE__ */ new Set(), f = [];
  a.forEach(function(v) {
    r.set(v.name, v);
  });
  function c(v) {
    u.add(v.name);
    var h = [].concat(v.requires || [], v.requiresIfExists || []);
    h.forEach(function(C) {
      if (!u.has(C)) {
        var b = r.get(C);
        b && c(b);
      }
    }), f.push(v);
  }
  return a.forEach(function(v) {
    u.has(v.name) || c(v);
  }), f;
}
function yP(a) {
  var r = gP(a);
  return k3.reduce(function(u, f) {
    return u.concat(r.filter(function(c) {
      return c.phase === f;
    }));
  }, []);
}
function bP(a) {
  var r;
  return function() {
    return r || (r = new Promise(function(u) {
      Promise.resolve().then(function() {
        r = void 0, u(a());
      });
    })), r;
  };
}
function CP(a) {
  var r = a.reduce(function(u, f) {
    var c = u[f.name];
    return u[f.name] = c ? Object.assign({}, c, f, {
      options: Object.assign({}, c.options, f.options),
      data: Object.assign({}, c.data, f.data)
    }) : f, u;
  }, {});
  return Object.keys(r).map(function(u) {
    return r[u];
  });
}
var HA = {
  placement: "bottom",
  modifiers: [],
  strategy: "absolute"
};
function zA() {
  for (var a = arguments.length, r = new Array(a), u = 0; u < a; u++)
    r[u] = arguments[u];
  return !r.some(function(f) {
    return !(f && typeof f.getBoundingClientRect == "function");
  });
}
function SP(a) {
  a === void 0 && (a = {});
  var r = a, u = r.defaultModifiers, f = u === void 0 ? [] : u, c = r.defaultOptions, v = c === void 0 ? HA : c;
  return function(C, b, E) {
    E === void 0 && (E = v);
    var w = {
      placement: "bottom",
      orderedModifiers: [],
      options: Object.assign({}, HA, v),
      modifiersData: {},
      elements: {
        reference: C,
        popper: b
      },
      attributes: {},
      styles: {}
    }, R = [], A = !1, D = {
      state: w,
      setOptions: function(B) {
        var Y = typeof B == "function" ? B(w.options) : B;
        M(), w.options = Object.assign({}, v, w.options, Y), w.scrollParents = {
          reference: Gc(C) ? Qh(C) : C.contextElement ? Qh(C.contextElement) : [],
          popper: Qh(b)
        };
        var z = yP(CP([].concat(f, w.options.modifiers)));
        return w.orderedModifiers = z.filter(function(Z) {
          return Z.enabled;
        }), L(), D.update();
      },
      // Sync update – it will always be executed, even if not necessary. This
      // is useful for low frequency updates where sync behavior simplifies the
      // logic.
      // For high frequency updates (e.g. `resize` and `scroll` events), always
      // prefer the async Popper#update method
      forceUpdate: function() {
        if (!A) {
          var B = w.elements, Y = B.reference, z = B.popper;
          if (zA(Y, z)) {
            w.rects = {
              reference: mP(Y, wm(z), w.options.strategy === "fixed"),
              popper: Ux(z)
            }, w.reset = !1, w.placement = w.options.placement, w.orderedModifiers.forEach(function(Se) {
              return w.modifiersData[Se.name] = Object.assign({}, Se.data);
            });
            for (var Z = 0; Z < w.orderedModifiers.length; Z++) {
              if (w.reset === !0) {
                w.reset = !1, Z = -1;
                continue;
              }
              var V = w.orderedModifiers[Z], Q = V.fn, P = V.options, U = P === void 0 ? {} : P, se = V.name;
              typeof Q == "function" && (w = Q({
                state: w,
                options: U,
                name: se,
                instance: D
              }) || w);
            }
          }
        }
      },
      // Async and optimistically optimized update – it will not be executed if
      // not necessary (debounced to run at most once-per-tick)
      update: bP(function() {
        return new Promise(function(k) {
          D.forceUpdate(), k(w);
        });
      }),
      destroy: function() {
        M(), A = !0;
      }
    };
    if (!zA(C, b))
      return D;
    D.setOptions(E).then(function(k) {
      !A && E.onFirstUpdate && E.onFirstUpdate(k);
    });
    function L() {
      w.orderedModifiers.forEach(function(k) {
        var B = k.name, Y = k.options, z = Y === void 0 ? {} : Y, Z = k.effect;
        if (typeof Z == "function") {
          var V = Z({
            state: w,
            name: B,
            instance: D,
            options: z
          }), Q = function() {
          };
          R.push(V || Q);
        }
      });
    }
    function M() {
      R.forEach(function(k) {
        return k();
      }), R = [];
    }
    return D;
  };
}
const EP = SP({
  defaultModifiers: [aP, sP, j3, W3, uP, nP, dP, H3]
}), xP = ["enabled", "placement", "strategy", "modifiers"];
function wP(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
const TP = {
  name: "applyStyles",
  enabled: !1,
  phase: "afterWrite",
  fn: () => {
  }
}, RP = {
  name: "ariaDescribedBy",
  enabled: !0,
  phase: "afterWrite",
  effect: ({
    state: a
  }) => () => {
    const {
      reference: r,
      popper: u
    } = a.elements;
    if ("removeAttribute" in r) {
      const f = (r.getAttribute("aria-describedby") || "").split(",").filter((c) => c.trim() !== u.id);
      f.length ? r.setAttribute("aria-describedby", f.join(",")) : r.removeAttribute("aria-describedby");
    }
  },
  fn: ({
    state: a
  }) => {
    var r;
    const {
      popper: u,
      reference: f
    } = a.elements, c = (r = u.getAttribute("role")) == null ? void 0 : r.toLowerCase();
    if (u.id && c === "tooltip" && "setAttribute" in f) {
      const v = f.getAttribute("aria-describedby");
      if (v && v.split(",").indexOf(u.id) !== -1)
        return;
      f.setAttribute("aria-describedby", v ? `${v},${u.id}` : u.id);
    }
  }
}, IP = [];
function AP(a, r, u = {}) {
  let {
    enabled: f = !0,
    placement: c = "bottom",
    strategy: v = "absolute",
    modifiers: h = IP
  } = u, C = wP(u, xP);
  const b = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(h), E = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(), w = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(() => {
    var k;
    (k = E.current) == null || k.update();
  }, []), R = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(() => {
    var k;
    (k = E.current) == null || k.forceUpdate();
  }, []), [A, D] = C3((0,react__WEBPACK_IMPORTED_MODULE_1__.useState)({
    placement: c,
    update: w,
    forceUpdate: R,
    attributes: {},
    styles: {
      popper: {},
      arrow: {}
    }
  })), L = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    name: "updateStateModifier",
    enabled: !0,
    phase: "write",
    requires: ["computeStyles"],
    fn: ({
      state: k
    }) => {
      const B = {}, Y = {};
      Object.keys(k.elements).forEach((z) => {
        B[z] = k.styles[z], Y[z] = k.attributes[z];
      }), D({
        state: k,
        styles: B,
        attributes: Y,
        update: w,
        forceUpdate: R,
        placement: k.placement
      });
    }
  }), [w, R, D]), M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => (Yh(b.current, h) || (b.current = h), b.current), [h]);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    !E.current || !f || E.current.setOptions({
      placement: c,
      strategy: v,
      modifiers: [...M, L, TP]
    });
  }, [v, c, L, f, M]), (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    if (!(!f || a == null || r == null))
      return E.current = EP(a, r, Object.assign({}, C, {
        placement: c,
        strategy: v,
        modifiers: [...M, RP, L]
      })), () => {
        E.current != null && (E.current.destroy(), E.current = void 0, D((k) => Object.assign({}, k, {
          attributes: {},
          styles: {
            popper: {}
          }
        })));
      };
  }, [f, a, r]), A;
}
const GA = () => {
};
function OP(a) {
  return a.button === 0;
}
function DP(a) {
  return !!(a.metaKey || a.altKey || a.ctrlKey || a.shiftKey);
}
const kE = (a) => a && ("current" in a ? a.current : a), _A = {
  click: "mousedown",
  mouseup: "mousedown",
  pointerup: "pointerdown"
};
function NP(a, r = GA, {
  disabled: u,
  clickTrigger: f = "click"
} = {}) {
  const c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), v = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((E) => {
    const w = kE(a);
    Tb(!!w, "ClickOutside captured a close event but does not have a ref to compare it to. useClickOutside(), should be passed a ref that resolves to a DOM node"), c.current = !w || DP(E) || !OP(E) || !!ub(w, E.target) || v.current, v.current = !1;
  }, [a]), C = Un((E) => {
    const w = kE(a);
    w && ub(w, E.target) && (v.current = !0);
  }), b = Un((E) => {
    c.current || r(E);
  });
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    var E, w;
    if (u || a == null)
      return;
    const R = Ib(kE(a)), A = R.defaultView || window;
    let D = (E = A.event) != null ? E : (w = A.parent) == null ? void 0 : w.event, L = null;
    _A[f] && (L = cs(R, _A[f], C, !0));
    const M = cs(R, f, h, !0), k = cs(R, f, (Y) => {
      if (Y === D) {
        D = void 0;
        return;
      }
      b(Y);
    });
    let B = [];
    return "ontouchstart" in R.documentElement && (B = [].slice.call(R.body.children).map((Y) => cs(Y, "mousemove", GA))), () => {
      L == null || L(), M(), k(), B.forEach((Y) => Y());
    };
  }, [a, u, f, h, C, b]);
}
function kP(a) {
  const r = {};
  return Array.isArray(a) ? (a == null || a.forEach((u) => {
    r[u.name] = u;
  }), r) : a || r;
}
function MP(a = {}) {
  return Array.isArray(a) ? a : Object.keys(a).map((r) => (a[r].name = r, a[r]));
}
function LP({
  enabled: a,
  enableEvents: r,
  placement: u,
  flip: f,
  offset: c,
  fixed: v,
  containerPadding: h,
  arrowElement: C,
  popperConfig: b = {}
}) {
  var E, w, R, A, D;
  const L = kP(b.modifiers);
  return Object.assign({}, b, {
    placement: u,
    enabled: a,
    strategy: v ? "fixed" : b.strategy,
    modifiers: MP(Object.assign({}, L, {
      eventListeners: {
        enabled: r,
        options: (E = L.eventListeners) == null ? void 0 : E.options
      },
      preventOverflow: Object.assign({}, L.preventOverflow, {
        options: h ? Object.assign({
          padding: h
        }, (w = L.preventOverflow) == null ? void 0 : w.options) : (R = L.preventOverflow) == null ? void 0 : R.options
      }),
      offset: {
        options: Object.assign({
          offset: c
        }, (A = L.offset) == null ? void 0 : A.options)
      },
      arrow: Object.assign({}, L.arrow, {
        enabled: !!C,
        options: Object.assign({}, (D = L.arrow) == null ? void 0 : D.options, {
          element: C
        })
      }),
      flip: Object.assign({
        enabled: !!f
      }, L.flip)
    }))
  });
}
const FP = ["children", "usePopper"];
function BP(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
const VP = () => {
};
function oN(a = {}) {
  const r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Mb), [u, f] = _B(), c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), {
    flip: v,
    offset: h,
    rootCloseEvent: C,
    fixed: b = !1,
    placement: E,
    popperConfig: w = {},
    enableEventListeners: R = !0,
    usePopper: A = !!r
  } = a, D = (r == null ? void 0 : r.show) == null ? !!a.show : r.show;
  D && !c.current && (c.current = !0);
  const L = (Q) => {
    r == null || r.toggle(!1, Q);
  }, {
    placement: M,
    setMenu: k,
    menuElement: B,
    toggleElement: Y
  } = r || {}, z = AP(Y, B, LP({
    placement: E || M || "bottom-start",
    enabled: A,
    enableEvents: R ?? D,
    offset: h,
    flip: v,
    fixed: b,
    arrowElement: u,
    popperConfig: w
  })), Z = Object.assign({
    ref: k || VP,
    "aria-labelledby": Y == null ? void 0 : Y.id
  }, z.attributes.popper, {
    style: z.styles.popper
  }), V = {
    show: D,
    placement: M,
    hasShown: c.current,
    toggle: r == null ? void 0 : r.toggle,
    popper: A ? z : null,
    arrowProps: A ? Object.assign({
      ref: f
    }, z.attributes.arrow, {
      style: z.styles.arrow
    }) : {}
  };
  return NP(B, L, {
    clickTrigger: C,
    disabled: !D
  }), [Z, V];
}
function uN(a) {
  let {
    children: r,
    usePopper: u = !0
  } = a, f = BP(a, FP);
  const [c, v] = oN(Object.assign({}, f, {
    usePopper: u
  }));
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
    children: r(c, v)
  });
}
uN.displayName = "DropdownMenu";
const sb = {
  prefix: String(Math.round(Math.random() * 1e10)),
  current: 0
}, lN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().createContext(sb), PP = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1___default().createContext(!1);
let HP = !!(typeof window < "u" && window.document && window.document.createElement), ME = /* @__PURE__ */ new WeakMap();
function zP(a = !1) {
  let r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(lN), u = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null);
  if (u.current === null && !a) {
    var f, c;
    let v = (c = (react__WEBPACK_IMPORTED_MODULE_1___default().__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED)) === null || c === void 0 || (f = c.ReactCurrentOwner) === null || f === void 0 ? void 0 : f.current;
    if (v) {
      let h = ME.get(v);
      h == null ? ME.set(v, {
        id: r.current,
        state: v.memoizedState
      }) : v.memoizedState !== h.state && (r.current = h.id, ME.delete(v));
    }
    u.current = ++r.current;
  }
  return u.current;
}
function GP(a) {
  let r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(lN);
  r === sb && !HP && console.warn("When server rendering, you must wrap your application in an <SSRProvider> to ensure consistent ids are generated between the client and server.");
  let u = zP(!!a), f = r === sb && "development" === "test" ? 0 : `react-aria${r.prefix}`;
  return a || `${f}-${u}`;
}
function _P(a) {
  let r = react__WEBPACK_IMPORTED_MODULE_1___default().useId(), [u] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(ZP()), f = u || "development" === "test" ? "react-aria" : `react-aria${sb.prefix}`;
  return a || `${f}-${r}`;
}
const jP = typeof (react__WEBPACK_IMPORTED_MODULE_1___default().useId) == "function" ? _P : GP;
function UP() {
  return !1;
}
function WP() {
  return !0;
}
function XP(a) {
  return () => {
  };
}
function ZP() {
  return typeof (react__WEBPACK_IMPORTED_MODULE_1___default().useSyncExternalStore) == "function" ? react__WEBPACK_IMPORTED_MODULE_1___default().useSyncExternalStore(XP, UP, WP) : (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(PP);
}
const sN = (a) => {
  var r;
  return ((r = a.getAttribute("role")) == null ? void 0 : r.toLowerCase()) === "menu";
}, jA = () => {
};
function cN() {
  const a = jP(), {
    show: r = !1,
    toggle: u = jA,
    setToggle: f,
    menuElement: c
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Mb) || {}, v = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((C) => {
    u(!r, C);
  }, [r, u]), h = {
    id: a,
    ref: f || jA,
    onClick: v,
    "aria-expanded": !!r
  };
  return c && sN(c) && (h["aria-haspopup"] = !0), [h, {
    show: r,
    toggle: u
  }];
}
function fN({
  children: a
}) {
  const [r, u] = cN();
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, {
    children: a(r, u)
  });
}
fN.displayName = "DropdownToggle";
const $x = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null);
$x.displayName = "NavContext";
const YP = ["eventKey", "disabled", "onClick", "active", "as"];
function $P(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
function dN({
  key: a,
  href: r,
  active: u,
  disabled: f,
  onClick: c
}) {
  const v = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(em), h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)($x), {
    activeKey: C
  } = h || {}, b = ob(a, r), E = u == null && a != null ? ob(C) === b : u;
  return [{
    onClick: Un((R) => {
      f || (c == null || c(R), v && !R.isPropagationStopped() && v(b, R));
    }),
    "aria-disabled": f || void 0,
    "aria-selected": E,
    [Em("dropdown-item")]: ""
  }, {
    isActive: E
  }];
}
const pN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  let {
    eventKey: u,
    disabled: f,
    onClick: c,
    active: v,
    as: h = Nx
  } = a, C = $P(a, YP);
  const [b] = dN({
    key: u,
    href: C.href,
    disabled: f,
    onClick: c,
    active: v
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(h, Object.assign({}, C, {
    ref: r
  }, b));
});
pN.displayName = "DropdownItem";
function UA() {
  const a = b3(), r = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), u = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((f) => {
    r.current = f, a();
  }, [a]);
  return [r, u];
}
function Tm({
  defaultShow: a,
  show: r,
  onSelect: u,
  onToggle: f,
  itemSelector: c = `* [${Em("dropdown-item")}]`,
  focusFirstItemOnShow: v,
  placement: h = "bottom-start",
  children: C
}) {
  const b = kb(), [E, w] = y3(r, a, f), [R, A] = UA(), D = R.current, [L, M] = UA(), k = L.current, B = rD(E), Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), Z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(em), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((he, xe, ye = xe == null ? void 0 : xe.type) => {
    w(he, {
      originalEvent: xe,
      source: ye
    });
  }, [w]), Q = Un((he, xe) => {
    u == null || u(he, xe), V(!1, xe, "select"), xe.isPropagationStopped() || Z == null || Z(he, xe);
  }), P = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    toggle: V,
    placement: h,
    show: E,
    menuElement: D,
    toggleElement: k,
    setMenu: A,
    setToggle: M
  }), [V, h, E, D, k, A, M]);
  D && B && !E && (z.current = D.contains(D.ownerDocument.activeElement));
  const U = Un(() => {
    k && k.focus && k.focus();
  }), se = Un(() => {
    const he = Y.current;
    let xe = v;
    if (xe == null && (xe = R.current && sN(R.current) ? "keyboard" : !1), xe === !1 || xe === "keyboard" && !/^key.+$/.test(he))
      return;
    const ye = rs(R.current, c)[0];
    ye && ye.focus && ye.focus();
  });
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    E ? se() : z.current && (z.current = !1, U());
  }, [E, z, U, se]), (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    Y.current = null;
  });
  const Se = (he, xe) => {
    if (!R.current)
      return null;
    const ye = rs(R.current, c);
    let Me = ye.indexOf(he) + xe;
    return Me = Math.max(0, Math.min(Me, ye.length)), ye[Me];
  };
  return jB((0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(() => b.document, [b]), "keydown", (he) => {
    var xe, ye;
    const {
      key: Me
    } = he, We = he.target, Ee = (xe = R.current) == null ? void 0 : xe.contains(We), tt = (ye = L.current) == null ? void 0 : ye.contains(We);
    if (/input|textarea/i.test(We.tagName) && (Me === " " || Me !== "Escape" && Ee || Me === "Escape" && We.type === "search") || !Ee && !tt || Me === "Tab" && (!R.current || !E))
      return;
    Y.current = he.type;
    const we = {
      originalEvent: he,
      source: he.type
    };
    switch (Me) {
      case "ArrowUp": {
        const W = Se(We, -1);
        W && W.focus && W.focus(), he.preventDefault();
        return;
      }
      case "ArrowDown":
        if (he.preventDefault(), !E)
          w(!0, we);
        else {
          const W = Se(We, 1);
          W && W.focus && W.focus();
        }
        return;
      case "Tab":
        sD(We.ownerDocument, "keyup", (W) => {
          var ce;
          (W.key === "Tab" && !W.target || !((ce = R.current) != null && ce.contains(W.target))) && w(!1, we);
        }, {
          once: !0
        });
        break;
      case "Escape":
        Me === "Escape" && (he.preventDefault(), he.stopPropagation()), w(!1, we);
        break;
    }
  }), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(em.Provider, {
    value: Q,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Mb.Provider, {
      value: P,
      children: C
    })
  });
}
Tm.displayName = "Dropdown";
Tm.Menu = uN;
Tm.Toggle = fN;
Tm.Item = pN;
const Qx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext({});
Qx.displayName = "DropdownContext";
const vN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "hr",
  role: f = "separator",
  ...c
}, v) => (r = ft(r, "dropdown-divider"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: v,
  className: Ke(a, r),
  role: f,
  ...c
})));
vN.displayName = "DropdownDivider";
const hN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "div",
  role: f = "heading",
  ...c
}, v) => (r = ft(r, "dropdown-header"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: v,
  className: Ke(a, r),
  role: f,
  ...c
})));
hN.displayName = "DropdownHeader";
const mN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  eventKey: u,
  disabled: f = !1,
  onClick: c,
  active: v,
  as: h = bm,
  ...C
}, b) => {
  const E = ft(a, "dropdown-item"), [w, R] = dN({
    key: u,
    href: C.href,
    disabled: f,
    onClick: c,
    active: v
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(h, {
    ...C,
    ...w,
    ref: b,
    className: Ke(r, E, R.isActive && "active", f && "disabled")
  });
});
mN.displayName = "DropdownItem";
const gN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  className: a,
  bsPrefix: r,
  as: u = "span",
  ...f
}, c) => (r = ft(r, "dropdown-item-text"), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
  ref: c,
  className: Ke(a, r),
  ...f
})));
gN.displayName = "DropdownItemText";
const Jx = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null);
Jx.displayName = "InputGroupContext";
function yN(a, r) {
  if (false)
    {}
  const u = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)((f) => {
    f == null || !f.isReactComponent || ( true ? uA(!1, `${r} injected a ref to a provided \`as\` component that resolved to a component instance instead of a DOM element. Use \`React.forwardRef\` to provide the injected ref to the class component as a prop in order to pass it directly to a DOM element`) : 0);
  }, [r]);
  return jc(u, a);
}
function bN(a, r, u) {
  const f = u ? "top-end" : "top-start", c = u ? "top-start" : "top-end", v = u ? "bottom-end" : "bottom-start", h = u ? "bottom-start" : "bottom-end", C = u ? "right-start" : "left-start", b = u ? "right-end" : "left-end", E = u ? "left-start" : "right-start", w = u ? "left-end" : "right-end";
  let R = a ? h : v;
  return r === "up" ? R = a ? c : f : r === "end" ? R = a ? w : E : r === "start" ? R = a ? b : C : r === "down-centered" ? R = "bottom" : r === "up-centered" && (R = "top"), R;
}
const CN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  align: u,
  rootCloseEvent: f,
  flip: c = !0,
  show: v,
  renderOnMount: h,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: C = "div",
  popperConfig: b,
  variant: E,
  ...w
}, R) => {
  let A = !1;
  const D = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Uc), L = ft(a, "dropdown-menu"), {
    align: M,
    drop: k,
    isRTL: B
  } = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Qx);
  u = u || M;
  const Y = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Jx), z = [];
  if (u)
    if (typeof u == "object") {
      const he = Object.keys(u);
      if ( true && Tb(he.length === 1, "There should only be 1 breakpoint when passing an object to `align`"), he.length) {
        const xe = he[0], ye = u[xe];
        A = ye === "start", z.push(`${L}-${xe}-${ye}`);
      }
    } else
      u === "end" && (A = !0);
  const Z = bN(A, k, B), [V, {
    hasShown: Q,
    popper: P,
    show: U,
    toggle: se
  }] = oN({
    flip: c,
    rootCloseEvent: f,
    show: v,
    usePopper: !D && z.length === 0,
    offset: [0, 2],
    popperConfig: b,
    placement: Z
  });
  if (V.ref = jc(yN(R, "DropdownMenu"), V.ref), rb(() => {
    U && (P == null || P.update());
  }, [U]), !Q && !h && !Y)
    return null;
  typeof C != "string" && (V.show = U, V.close = () => se == null ? void 0 : se(!1), V.align = u);
  let Se = w.style;
  return P != null && P.placement && (Se = {
    ...w.style,
    ...V.style
  }, w["x-placement"] = P.placement), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(C, {
    ...w,
    ...V,
    style: Se,
    ...(z.length || D) && {
      "data-bs-popper": "static"
    },
    className: Ke(r, L, U && "show", A && `${L}-end`, E && `${L}-${E}`, ...z)
  });
});
CN.displayName = "DropdownMenu";
const SN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  split: r,
  className: u,
  childBsPrefix: f,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: c = zu,
  ...v
}, h) => {
  const C = ft(a, "dropdown-toggle"), b = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Mb);
  f !== void 0 && (v.bsPrefix = f);
  const [E] = cN();
  return E.ref = jc(E.ref, yN(h, "DropdownToggle")), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(c, {
    className: Ke(u, C, r && `${C}-split`, (b == null ? void 0 : b.show) && "show"),
    ...E,
    ...v
  });
});
SN.displayName = "DropdownToggle";
const EN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  const {
    bsPrefix: u,
    drop: f = "down",
    show: c,
    className: v,
    align: h = "start",
    onSelect: C,
    onToggle: b,
    focusFirstItemOnShow: E,
    // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
    as: w = "div",
    navbar: R,
    autoClose: A = !0,
    ...D
  } = Ox(a, {
    show: "onToggle"
  }), L = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(Jx), M = ft(u, "dropdown"), k = RB(), B = (P) => A === !1 ? P === "click" : A === "inside" ? P !== "rootClose" : A === "outside" ? P !== "select" : !0, Y = Un((P, U) => {
    var se, Se;
    !((se = U.originalEvent) == null || (Se = se.target) == null) && Se.classList.contains("dropdown-toggle") && U.source === "mousedown" || (U.originalEvent.currentTarget === document && (U.source !== "keydown" || U.originalEvent.key === "Escape") && (U.source = "rootClose"), B(U.source) && (b == null || b(P, U)));
  }), Z = bN(h === "end", f, k), V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(() => ({
    align: h,
    drop: f,
    isRTL: k
  }), [h, f, k]), Q = {
    down: M,
    "down-centered": `${M}-center`,
    up: "dropup",
    "up-centered": "dropup-center dropup",
    end: "dropend",
    start: "dropstart"
  };
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Qx.Provider, {
    value: V,
    children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Tm, {
      placement: Z,
      show: c,
      onSelect: C,
      onToggle: Y,
      focusFirstItemOnShow: E,
      itemSelector: `.${M}-item:not(.disabled):not(:disabled)`,
      children: L ? D.children : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(w, {
        ...D,
        ref: r,
        className: Ke(v, c && "show", Q[f])
      })
    })
  });
});
EN.displayName = "Dropdown";
const Ac = Object.assign(EN, {
  Toggle: SN,
  Menu: CN,
  Item: mN,
  ItemText: gN,
  Divider: vN,
  Header: hN
}), QP = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(null), JP = ["as", "active", "eventKey"];
function KP(a, r) {
  if (a == null)
    return {};
  var u = {}, f = Object.keys(a), c, v;
  for (v = 0; v < f.length; v++)
    c = f[v], !(r.indexOf(c) >= 0) && (u[c] = a[c]);
  return u;
}
function xN({
  key: a,
  onClick: r,
  active: u,
  id: f,
  role: c,
  disabled: v
}) {
  const h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(em), C = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)($x), b = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(QP);
  let E = u;
  const w = {
    role: c
  };
  if (C) {
    !c && C.role === "tablist" && (w.role = "tab");
    const R = C.getControllerId(a ?? null), A = C.getControlledId(a ?? null);
    w[Em("event-key")] = a, w.id = R || f, E = u == null && a != null ? C.activeKey === a : u, (E || !(b != null && b.unmountOnExit) && !(b != null && b.mountOnEnter)) && (w["aria-controls"] = A);
  }
  return w.role === "tab" && (w["aria-selected"] = E, E || (w.tabIndex = -1), v && (w.tabIndex = -1, w["aria-disabled"] = !0)), w.onClick = Un((R) => {
    v || (r == null || r(R), a != null && h && !R.isPropagationStopped() && h(a, R));
  }), [w, {
    isActive: E
  }];
}
const qP = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef((a, r) => {
  let {
    as: u = Nx,
    active: f,
    eventKey: c
  } = a, v = KP(a, JP);
  const [h, C] = xN(Object.assign({
    key: ob(c, v.href),
    active: f
  }, v));
  return h[Em("active")] = C.isActive, /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, Object.assign({}, v, h, {
    ref: r
  }));
});
qP.displayName = "NavItem";
const wN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  as: u = bm,
  active: f,
  eventKey: c,
  disabled: v = !1,
  ...h
}, C) => {
  a = ft(a, "nav-link");
  const [b, E] = xN({
    key: ob(c, h.href),
    active: f,
    disabled: v,
    ...h
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(u, {
    ...h,
    ...b,
    ref: C,
    disabled: v,
    className: Ke(r, a, v && "disabled", E.isActive && "active")
  });
});
wN.displayName = "NavLink";
const TN = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  id: a,
  title: r,
  children: u,
  bsPrefix: f,
  className: c,
  rootCloseEvent: v,
  menuRole: h,
  disabled: C,
  active: b,
  renderMenuOnMount: E,
  menuVariant: w,
  ...R
}, A) => {
  const D = ft(void 0, "nav-item");
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ac, {
    ref: A,
    ...R,
    className: Ke(c, D),
    children: [/* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ac.Toggle, {
      id: a,
      eventKey: null,
      active: b,
      disabled: C,
      childBsPrefix: f,
      as: wN,
      children: r
    }), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ac.Menu, {
      role: h,
      renderOnMount: E,
      rootCloseEvent: v,
      variant: w,
      children: u
    })]
  });
});
TN.displayName = "NavDropdown";
const WA = Object.assign(TN, {
  Item: Ac.Item,
  ItemText: Ac.ItemText,
  Divider: Ac.Divider,
  Header: Ac.Header
});
function LE({ label: a, value: r }) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Jd.Text, { children: [
    a,
    ": ",
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { className: "text-light", children: r || "None" })
  ] });
}
function XA({
  label: a,
  version: r
}) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Jd.Text, { children: [
    a,
    ":",
    " ",
    r ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("code", { className: "text-success", children: `v${r}` }) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { className: "text-light", children: "None" })
  ] });
}
function eH(a) {
  const { data: { username: r, site: u } = { username: "", site: "" } } = Qd({
    queryKey: ["profile"],
    queryFn: async () => a.httpPathHandler("accounts/profile").then((f) => f.json()).then((f) => ({ username: f.data.username, site: f.data.site }))
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Jd, { bg: "dark", variant: "dark", collapseOnSelect: !0, expand: "sm", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(zc, { fluid: !0, children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Jd.Brand, { children: "Onyx" }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Jd.Collapse, { id: "responsive-navbar-nav", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { direction: "horizontal", gap: 3, children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        WA,
        {
          title: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LE, { label: "Project", value: a.projectName }),
          id: "collapsible-nav-dropdown",
          style: { color: "white" },
          children: a.projectList.map((f) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
            WA.Item,
            {
              onClick: () => a.handleProjectChange(f),
              children: f
            },
            f
          ))
        }
      ),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LE, { label: "User", value: r }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(LE, { label: "Site", value: u }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(XA, { label: "GUI", version: a.guiVersion }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(XA, { label: "Extension", version: a.extVersion })
    ] }) }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Jd.Toggle, { "aria-controls": "responsive-navbar-nav" }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
      Ix.Check,
      {
        type: "switch",
        id: "theme-switch",
        label: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { className: "text-light", children: "Switch Theme" }),
        onChange: a.handleThemeChange
      }
    )
  ] }) });
}
function _c(a) {
  "@babel/helpers - typeof";
  return _c = typeof Symbol == "function" && typeof Symbol.iterator == "symbol" ? function(r) {
    return typeof r;
  } : function(r) {
    return r && typeof Symbol == "function" && r.constructor === Symbol && r !== Symbol.prototype ? "symbol" : typeof r;
  }, _c(a);
}
function tH(a, r) {
  if (_c(a) != "object" || !a)
    return a;
  var u = a[Symbol.toPrimitive];
  if (u !== void 0) {
    var f = u.call(a, r || "default");
    if (_c(f) != "object")
      return f;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (r === "string" ? String : Number)(a);
}
function RN(a) {
  var r = tH(a, "string");
  return _c(r) == "symbol" ? r : r + "";
}
function Wh(a, r, u) {
  return r = RN(r), r in a ? Object.defineProperty(a, r, {
    value: u,
    enumerable: !0,
    configurable: !0,
    writable: !0
  }) : a[r] = u, a;
}
function ZA(a, r) {
  var u = Object.keys(a);
  if (Object.getOwnPropertySymbols) {
    var f = Object.getOwnPropertySymbols(a);
    r && (f = f.filter(function(c) {
      return Object.getOwnPropertyDescriptor(a, c).enumerable;
    })), u.push.apply(u, f);
  }
  return u;
}
function it(a) {
  for (var r = 1; r < arguments.length; r++) {
    var u = arguments[r] != null ? arguments[r] : {};
    r % 2 ? ZA(Object(u), !0).forEach(function(f) {
      Wh(a, f, u[f]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(a, Object.getOwnPropertyDescriptors(u)) : ZA(Object(u)).forEach(function(f) {
      Object.defineProperty(a, f, Object.getOwnPropertyDescriptor(u, f));
    });
  }
  return a;
}
function nH(a) {
  if (Array.isArray(a))
    return a;
}
function rH(a, r) {
  var u = a == null ? null : typeof Symbol < "u" && a[Symbol.iterator] || a["@@iterator"];
  if (u != null) {
    var f, c, v, h, C = [], b = !0, E = !1;
    try {
      if (v = (u = u.call(a)).next, r === 0) {
        if (Object(u) !== u)
          return;
        b = !1;
      } else
        for (; !(b = (f = v.call(u)).done) && (C.push(f.value), C.length !== r); b = !0)
          ;
    } catch (w) {
      E = !0, c = w;
    } finally {
      try {
        if (!b && u.return != null && (h = u.return(), Object(h) !== h))
          return;
      } finally {
        if (E)
          throw c;
      }
    }
    return C;
  }
}
function dx(a, r) {
  (r == null || r > a.length) && (r = a.length);
  for (var u = 0, f = new Array(r); u < r; u++)
    f[u] = a[u];
  return f;
}
function IN(a, r) {
  if (a) {
    if (typeof a == "string")
      return dx(a, r);
    var u = Object.prototype.toString.call(a).slice(8, -1);
    if (u === "Object" && a.constructor && (u = a.constructor.name), u === "Map" || u === "Set")
      return Array.from(a);
    if (u === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(u))
      return dx(a, r);
  }
}
function aH() {
  throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
}
function Gu(a, r) {
  return nH(a) || rH(a, r) || IN(a, r) || aH();
}
function $o(a, r) {
  if (a == null)
    return {};
  var u = Ax(a, r), f, c;
  if (Object.getOwnPropertySymbols) {
    var v = Object.getOwnPropertySymbols(a);
    for (c = 0; c < v.length; c++)
      f = v[c], !(r.indexOf(f) >= 0) && Object.prototype.propertyIsEnumerable.call(a, f) && (u[f] = a[f]);
  }
  return u;
}
var iH = ["defaultInputValue", "defaultMenuIsOpen", "defaultValue", "inputValue", "menuIsOpen", "onChange", "onInputChange", "onMenuClose", "onMenuOpen", "value"];
function AN(a) {
  var r = a.defaultInputValue, u = r === void 0 ? "" : r, f = a.defaultMenuIsOpen, c = f === void 0 ? !1 : f, v = a.defaultValue, h = v === void 0 ? null : v, C = a.inputValue, b = a.menuIsOpen, E = a.onChange, w = a.onInputChange, R = a.onMenuClose, A = a.onMenuOpen, D = a.value, L = $o(a, iH), M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(C !== void 0 ? C : u), k = Gu(M, 2), B = k[0], Y = k[1], z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(b !== void 0 ? b : c), Z = Gu(z, 2), V = Z[0], Q = Z[1], P = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(D !== void 0 ? D : h), U = Gu(P, 2), se = U[0], Se = U[1], he = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(ue, we) {
    typeof E == "function" && E(ue, we), Se(ue);
  }, [E]), xe = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(ue, we) {
    var W;
    typeof w == "function" && (W = w(ue, we)), Y(W !== void 0 ? W : ue);
  }, [w]), ye = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function() {
    typeof A == "function" && A(), Q(!0);
  }, [A]), Me = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function() {
    typeof R == "function" && R(), Q(!1);
  }, [R]), We = C !== void 0 ? C : B, Ee = b !== void 0 ? b : V, tt = D !== void 0 ? D : se;
  return it(it({}, L), {}, {
    inputValue: We,
    menuIsOpen: Ee,
    onChange: he,
    onInputChange: xe,
    onMenuClose: Me,
    onMenuOpen: ye,
    value: tt
  });
}
function oH(a, r) {
  if (!(a instanceof r))
    throw new TypeError("Cannot call a class as a function");
}
function YA(a, r) {
  for (var u = 0; u < r.length; u++) {
    var f = r[u];
    f.enumerable = f.enumerable || !1, f.configurable = !0, "value" in f && (f.writable = !0), Object.defineProperty(a, RN(f.key), f);
  }
}
function uH(a, r, u) {
  return r && YA(a.prototype, r), u && YA(a, u), Object.defineProperty(a, "prototype", {
    writable: !1
  }), a;
}
function lH(a, r) {
  if (typeof r != "function" && r !== null)
    throw new TypeError("Super expression must either be null or a function");
  a.prototype = Object.create(r && r.prototype, {
    constructor: {
      value: a,
      writable: !0,
      configurable: !0
    }
  }), Object.defineProperty(a, "prototype", {
    writable: !1
  }), r && nb(a, r);
}
function cb(a) {
  return cb = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(u) {
    return u.__proto__ || Object.getPrototypeOf(u);
  }, cb(a);
}
function ON() {
  try {
    var a = !Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {
    }));
  } catch {
  }
  return (ON = function() {
    return !!a;
  })();
}
function sH(a) {
  if (a === void 0)
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  return a;
}
function cH(a, r) {
  if (r && (_c(r) === "object" || typeof r == "function"))
    return r;
  if (r !== void 0)
    throw new TypeError("Derived constructors may only return object or undefined");
  return sH(a);
}
function fH(a) {
  var r = ON();
  return function() {
    var f = cb(a), c;
    if (r) {
      var v = cb(this).constructor;
      c = Reflect.construct(f, arguments, v);
    } else
      c = f.apply(this, arguments);
    return cH(this, c);
  };
}
function dH(a) {
  if (Array.isArray(a))
    return dx(a);
}
function pH(a) {
  if (typeof Symbol < "u" && a[Symbol.iterator] != null || a["@@iterator"] != null)
    return Array.from(a);
}
function vH() {
  throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`);
}
function qd(a) {
  return dH(a) || pH(a) || IN(a) || vH();
}
function hH(a) {
  if (a.sheet)
    return a.sheet;
  for (var r = 0; r < document.styleSheets.length; r++)
    if (document.styleSheets[r].ownerNode === a)
      return document.styleSheets[r];
}
function mH(a) {
  var r = document.createElement("style");
  return r.setAttribute("data-emotion", a.key), a.nonce !== void 0 && r.setAttribute("nonce", a.nonce), r.appendChild(document.createTextNode("")), r.setAttribute("data-s", ""), r;
}
var gH = /* @__PURE__ */ function() {
  function a(u) {
    var f = this;
    this._insertTag = function(c) {
      var v;
      f.tags.length === 0 ? f.insertionPoint ? v = f.insertionPoint.nextSibling : f.prepend ? v = f.container.firstChild : v = f.before : v = f.tags[f.tags.length - 1].nextSibling, f.container.insertBefore(c, v), f.tags.push(c);
    }, this.isSpeedy = u.speedy === void 0 ? "development" === "production" : u.speedy, this.tags = [], this.ctr = 0, this.nonce = u.nonce, this.key = u.key, this.container = u.container, this.prepend = u.prepend, this.insertionPoint = u.insertionPoint, this.before = null;
  }
  var r = a.prototype;
  return r.hydrate = function(f) {
    f.forEach(this._insertTag);
  }, r.insert = function(f) {
    this.ctr % (this.isSpeedy ? 65e3 : 1) === 0 && this._insertTag(mH(this));
    var c = this.tags[this.tags.length - 1];
    if (true) {
      var v = f.charCodeAt(0) === 64 && f.charCodeAt(1) === 105;
      v && this._alreadyInsertedOrderInsensitiveRule && console.error(`You're attempting to insert the following rule:
` + f + "\n\n`@import` rules must be before all other types of rules in a stylesheet but other rules have already been inserted. Please ensure that `@import` rules are before all other rules."), this._alreadyInsertedOrderInsensitiveRule = this._alreadyInsertedOrderInsensitiveRule || !v;
    }
    if (this.isSpeedy) {
      var h = hH(c);
      try {
        h.insertRule(f, h.cssRules.length);
      } catch (C) {
         true && !/:(-moz-placeholder|-moz-focus-inner|-moz-focusring|-ms-input-placeholder|-moz-read-write|-moz-read-only|-ms-clear|-ms-expand|-ms-reveal){/.test(f) && console.error('There was a problem inserting the following rule: "' + f + '"', C);
      }
    } else
      c.appendChild(document.createTextNode(f));
    this.ctr++;
  }, r.flush = function() {
    this.tags.forEach(function(f) {
      return f.parentNode && f.parentNode.removeChild(f);
    }), this.tags = [], this.ctr = 0,  true && (this._alreadyInsertedOrderInsensitiveRule = !1);
  }, a;
}(), qr = "-ms-", fb = "-moz-", Wt = "-webkit-", Kx = "comm", qx = "rule", ew = "decl", yH = "@import", DN = "@keyframes", bH = "@layer", CH = Math.abs, Fb = String.fromCharCode, SH = Object.assign;
function EH(a, r) {
  return Mr(a, 0) ^ 45 ? (((r << 2 ^ Mr(a, 0)) << 2 ^ Mr(a, 1)) << 2 ^ Mr(a, 2)) << 2 ^ Mr(a, 3) : 0;
}
function NN(a) {
  return a.trim();
}
function xH(a, r) {
  return (a = r.exec(a)) ? a[0] : a;
}
function Xt(a, r, u) {
  return a.replace(r, u);
}
function px(a, r) {
  return a.indexOf(r);
}
function Mr(a, r) {
  return a.charCodeAt(r) | 0;
}
function rm(a, r, u) {
  return a.slice(r, u);
}
function Wo(a) {
  return a.length;
}
function tw(a) {
  return a.length;
}
function j0(a, r) {
  return r.push(a), a;
}
function wH(a, r) {
  return a.map(r).join("");
}
var Bb = 1, Cp = 1, kN = 0, Za = 0, Kn = 0, Ep = "";
function Vb(a, r, u, f, c, v, h) {
  return { value: a, root: r, parent: u, type: f, props: c, children: v, line: Bb, column: Cp, length: h, return: "" };
}
function zh(a, r) {
  return SH(Vb("", null, null, "", null, null, 0), a, { length: -a.length }, r);
}
function TH() {
  return Kn;
}
function RH() {
  return Kn = Za > 0 ? Mr(Ep, --Za) : 0, Cp--, Kn === 10 && (Cp = 1, Bb--), Kn;
}
function li() {
  return Kn = Za < kN ? Mr(Ep, Za++) : 0, Cp++, Kn === 10 && (Cp = 1, Bb++), Kn;
}
function Yo() {
  return Mr(Ep, Za);
}
function Q0() {
  return Za;
}
function Rm(a, r) {
  return rm(Ep, a, r);
}
function am(a) {
  switch (a) {
    case 0:
    case 9:
    case 10:
    case 13:
    case 32:
      return 5;
    case 33:
    case 43:
    case 44:
    case 47:
    case 62:
    case 64:
    case 126:
    case 59:
    case 123:
    case 125:
      return 4;
    case 58:
      return 3;
    case 34:
    case 39:
    case 40:
    case 91:
      return 2;
    case 41:
    case 93:
      return 1;
  }
  return 0;
}
function MN(a) {
  return Bb = Cp = 1, kN = Wo(Ep = a), Za = 0, [];
}
function LN(a) {
  return Ep = "", a;
}
function J0(a) {
  return NN(Rm(Za - 1, vx(a === 91 ? a + 2 : a === 40 ? a + 1 : a)));
}
function IH(a) {
  for (; (Kn = Yo()) && Kn < 33; )
    li();
  return am(a) > 2 || am(Kn) > 3 ? "" : " ";
}
function AH(a, r) {
  for (; --r && li() && !(Kn < 48 || Kn > 102 || Kn > 57 && Kn < 65 || Kn > 70 && Kn < 97); )
    ;
  return Rm(a, Q0() + (r < 6 && Yo() == 32 && li() == 32));
}
function vx(a) {
  for (; li(); )
    switch (Kn) {
      case a:
        return Za;
      case 34:
      case 39:
        a !== 34 && a !== 39 && vx(Kn);
        break;
      case 40:
        a === 41 && vx(a);
        break;
      case 92:
        li();
        break;
    }
  return Za;
}
function OH(a, r) {
  for (; li() && a + Kn !== 57; )
    if (a + Kn === 84 && Yo() === 47)
      break;
  return "/*" + Rm(r, Za - 1) + "*" + Fb(a === 47 ? a : li());
}
function DH(a) {
  for (; !am(Yo()); )
    li();
  return Rm(a, Za);
}
function NH(a) {
  return LN(K0("", null, null, null, [""], a = MN(a), 0, [0], a));
}
function K0(a, r, u, f, c, v, h, C, b) {
  for (var E = 0, w = 0, R = h, A = 0, D = 0, L = 0, M = 1, k = 1, B = 1, Y = 0, z = "", Z = c, V = v, Q = f, P = z; k; )
    switch (L = Y, Y = li()) {
      case 40:
        if (L != 108 && Mr(P, R - 1) == 58) {
          px(P += Xt(J0(Y), "&", "&\f"), "&\f") != -1 && (B = -1);
          break;
        }
      case 34:
      case 39:
      case 91:
        P += J0(Y);
        break;
      case 9:
      case 10:
      case 13:
      case 32:
        P += IH(L);
        break;
      case 92:
        P += AH(Q0() - 1, 7);
        continue;
      case 47:
        switch (Yo()) {
          case 42:
          case 47:
            j0(kH(OH(li(), Q0()), r, u), b);
            break;
          default:
            P += "/";
        }
        break;
      case 123 * M:
        C[E++] = Wo(P) * B;
      case 125 * M:
      case 59:
      case 0:
        switch (Y) {
          case 0:
          case 125:
            k = 0;
          case 59 + w:
            B == -1 && (P = Xt(P, /\f/g, "")), D > 0 && Wo(P) - R && j0(D > 32 ? QA(P + ";", f, u, R - 1) : QA(Xt(P, " ", "") + ";", f, u, R - 2), b);
            break;
          case 59:
            P += ";";
          default:
            if (j0(Q = $A(P, r, u, E, w, c, C, z, Z = [], V = [], R), v), Y === 123)
              if (w === 0)
                K0(P, r, Q, Q, Z, v, R, C, V);
              else
                switch (A === 99 && Mr(P, 3) === 110 ? 100 : A) {
                  case 100:
                  case 108:
                  case 109:
                  case 115:
                    K0(a, Q, Q, f && j0($A(a, Q, Q, 0, 0, c, C, z, c, Z = [], R), V), c, V, R, C, f ? Z : V);
                    break;
                  default:
                    K0(P, Q, Q, Q, [""], V, 0, C, V);
                }
        }
        E = w = D = 0, M = B = 1, z = P = "", R = h;
        break;
      case 58:
        R = 1 + Wo(P), D = L;
      default:
        if (M < 1) {
          if (Y == 123)
            --M;
          else if (Y == 125 && M++ == 0 && RH() == 125)
            continue;
        }
        switch (P += Fb(Y), Y * M) {
          case 38:
            B = w > 0 ? 1 : (P += "\f", -1);
            break;
          case 44:
            C[E++] = (Wo(P) - 1) * B, B = 1;
            break;
          case 64:
            Yo() === 45 && (P += J0(li())), A = Yo(), w = R = Wo(z = P += DH(Q0())), Y++;
            break;
          case 45:
            L === 45 && Wo(P) == 2 && (M = 0);
        }
    }
  return v;
}
function $A(a, r, u, f, c, v, h, C, b, E, w) {
  for (var R = c - 1, A = c === 0 ? v : [""], D = tw(A), L = 0, M = 0, k = 0; L < f; ++L)
    for (var B = 0, Y = rm(a, R + 1, R = CH(M = h[L])), z = a; B < D; ++B)
      (z = NN(M > 0 ? A[B] + " " + Y : Xt(Y, /&\f/g, A[B]))) && (b[k++] = z);
  return Vb(a, r, u, c === 0 ? qx : C, b, E, w);
}
function kH(a, r, u) {
  return Vb(a, r, u, Kx, Fb(TH()), rm(a, 2, -2), 0);
}
function QA(a, r, u, f) {
  return Vb(a, r, u, ew, rm(a, 0, f), rm(a, f + 1, -1), f);
}
function ep(a, r) {
  for (var u = "", f = tw(a), c = 0; c < f; c++)
    u += r(a[c], c, a, r) || "";
  return u;
}
function MH(a, r, u, f) {
  switch (a.type) {
    case bH:
      if (a.children.length)
        break;
    case yH:
    case ew:
      return a.return = a.return || a.value;
    case Kx:
      return "";
    case DN:
      return a.return = a.value + "{" + ep(a.children, f) + "}";
    case qx:
      a.value = a.props.join(",");
  }
  return Wo(u = ep(a.children, f)) ? a.return = a.value + "{" + u + "}" : "";
}
function LH(a) {
  var r = tw(a);
  return function(u, f, c, v) {
    for (var h = "", C = 0; C < r; C++)
      h += a[C](u, f, c, v) || "";
    return h;
  };
}
function FH(a) {
  return function(r) {
    r.root || (r = r.return) && a(r);
  };
}
function BH(a) {
  var r = /* @__PURE__ */ Object.create(null);
  return function(u) {
    return r[u] === void 0 && (r[u] = a(u)), r[u];
  };
}
var VH = function(r, u, f) {
  for (var c = 0, v = 0; c = v, v = Yo(), c === 38 && v === 12 && (u[f] = 1), !am(v); )
    li();
  return Rm(r, Za);
}, PH = function(r, u) {
  var f = -1, c = 44;
  do
    switch (am(c)) {
      case 0:
        c === 38 && Yo() === 12 && (u[f] = 1), r[f] += VH(Za - 1, u, f);
        break;
      case 2:
        r[f] += J0(c);
        break;
      case 4:
        if (c === 44) {
          r[++f] = Yo() === 58 ? "&\f" : "", u[f] = r[f].length;
          break;
        }
      default:
        r[f] += Fb(c);
    }
  while (c = li());
  return r;
}, HH = function(r, u) {
  return LN(PH(MN(r), u));
}, JA = /* @__PURE__ */ new WeakMap(), zH = function(r) {
  if (!(r.type !== "rule" || !r.parent || // positive .length indicates that this rule contains pseudo
  // negative .length indicates that this rule has been already prefixed
  r.length < 1)) {
    for (var u = r.value, f = r.parent, c = r.column === f.column && r.line === f.line; f.type !== "rule"; )
      if (f = f.parent, !f)
        return;
    if (!(r.props.length === 1 && u.charCodeAt(0) !== 58 && !JA.get(f)) && !c) {
      JA.set(r, !0);
      for (var v = [], h = HH(u, v), C = f.props, b = 0, E = 0; b < h.length; b++)
        for (var w = 0; w < C.length; w++, E++)
          r.props[E] = v[b] ? h[b].replace(/&\f/g, C[w]) : C[w] + " " + h[b];
    }
  }
}, GH = function(r) {
  if (r.type === "decl") {
    var u = r.value;
    // charcode for l
    u.charCodeAt(0) === 108 && // charcode for b
    u.charCodeAt(2) === 98 && (r.return = "", r.value = "");
  }
}, _H = "emotion-disable-server-rendering-unsafe-selector-warning-please-do-not-use-this-the-warning-exists-for-a-reason", jH = function(r) {
  return r.type === "comm" && r.children.indexOf(_H) > -1;
}, UH = function(r) {
  return function(u, f, c) {
    if (!(u.type !== "rule" || r.compat)) {
      var v = u.value.match(/(:first|:nth|:nth-last)-child/g);
      if (v) {
        for (var h = !!u.parent, C = h ? u.parent.children : (
          // global rule at the root level
          c
        ), b = C.length - 1; b >= 0; b--) {
          var E = C[b];
          if (E.line < u.line)
            break;
          if (E.column < u.column) {
            if (jH(E))
              return;
            break;
          }
        }
        v.forEach(function(w) {
          console.error('The pseudo class "' + w + '" is potentially unsafe when doing server-side rendering. Try changing it to "' + w.split("-child")[0] + '-of-type".');
        });
      }
    }
  };
}, FN = function(r) {
  return r.type.charCodeAt(1) === 105 && r.type.charCodeAt(0) === 64;
}, WH = function(r, u) {
  for (var f = r - 1; f >= 0; f--)
    if (!FN(u[f]))
      return !0;
  return !1;
}, KA = function(r) {
  r.type = "", r.value = "", r.return = "", r.children = "", r.props = "";
}, XH = function(r, u, f) {
  FN(r) && (r.parent ? (console.error("`@import` rules can't be nested inside other rules. Please move it to the top level and put it before regular rules. Keep in mind that they can only be used within global styles."), KA(r)) : WH(u, f) && (console.error("`@import` rules can't be after other rules. Please put your `@import` rules before your other rules."), KA(r)));
};
function BN(a, r) {
  switch (EH(a, r)) {
    case 5103:
      return Wt + "print-" + a + a;
    case 5737:
    case 4201:
    case 3177:
    case 3433:
    case 1641:
    case 4457:
    case 2921:
    case 5572:
    case 6356:
    case 5844:
    case 3191:
    case 6645:
    case 3005:
    case 6391:
    case 5879:
    case 5623:
    case 6135:
    case 4599:
    case 4855:
    case 4215:
    case 6389:
    case 5109:
    case 5365:
    case 5621:
    case 3829:
      return Wt + a + a;
    case 5349:
    case 4246:
    case 4810:
    case 6968:
    case 2756:
      return Wt + a + fb + a + qr + a + a;
    case 6828:
    case 4268:
      return Wt + a + qr + a + a;
    case 6165:
      return Wt + a + qr + "flex-" + a + a;
    case 5187:
      return Wt + a + Xt(a, /(\w+).+(:[^]+)/, Wt + "box-$1$2" + qr + "flex-$1$2") + a;
    case 5443:
      return Wt + a + qr + "flex-item-" + Xt(a, /flex-|-self/, "") + a;
    case 4675:
      return Wt + a + qr + "flex-line-pack" + Xt(a, /align-content|flex-|-self/, "") + a;
    case 5548:
      return Wt + a + qr + Xt(a, "shrink", "negative") + a;
    case 5292:
      return Wt + a + qr + Xt(a, "basis", "preferred-size") + a;
    case 6060:
      return Wt + "box-" + Xt(a, "-grow", "") + Wt + a + qr + Xt(a, "grow", "positive") + a;
    case 4554:
      return Wt + Xt(a, /([^-])(transform)/g, "$1" + Wt + "$2") + a;
    case 6187:
      return Xt(Xt(Xt(a, /(zoom-|grab)/, Wt + "$1"), /(image-set)/, Wt + "$1"), a, "") + a;
    case 5495:
    case 3959:
      return Xt(a, /(image-set\([^]*)/, Wt + "$1$`$1");
    case 4968:
      return Xt(Xt(a, /(.+:)(flex-)?(.*)/, Wt + "box-pack:$3" + qr + "flex-pack:$3"), /s.+-b[^;]+/, "justify") + Wt + a + a;
    case 4095:
    case 3583:
    case 4068:
    case 2532:
      return Xt(a, /(.+)-inline(.+)/, Wt + "$1$2") + a;
    case 8116:
    case 7059:
    case 5753:
    case 5535:
    case 5445:
    case 5701:
    case 4933:
    case 4677:
    case 5533:
    case 5789:
    case 5021:
    case 4765:
      if (Wo(a) - 1 - r > 6)
        switch (Mr(a, r + 1)) {
          case 109:
            if (Mr(a, r + 4) !== 45)
              break;
          case 102:
            return Xt(a, /(.+:)(.+)-([^]+)/, "$1" + Wt + "$2-$3$1" + fb + (Mr(a, r + 3) == 108 ? "$3" : "$2-$3")) + a;
          case 115:
            return ~px(a, "stretch") ? BN(Xt(a, "stretch", "fill-available"), r) + a : a;
        }
      break;
    case 4949:
      if (Mr(a, r + 1) !== 115)
        break;
    case 6444:
      switch (Mr(a, Wo(a) - 3 - (~px(a, "!important") && 10))) {
        case 107:
          return Xt(a, ":", ":" + Wt) + a;
        case 101:
          return Xt(a, /(.+:)([^;!]+)(;|!.+)?/, "$1" + Wt + (Mr(a, 14) === 45 ? "inline-" : "") + "box$3$1" + Wt + "$2$3$1" + qr + "$2box$3") + a;
      }
      break;
    case 5936:
      switch (Mr(a, r + 11)) {
        case 114:
          return Wt + a + qr + Xt(a, /[svh]\w+-[tblr]{2}/, "tb") + a;
        case 108:
          return Wt + a + qr + Xt(a, /[svh]\w+-[tblr]{2}/, "tb-rl") + a;
        case 45:
          return Wt + a + qr + Xt(a, /[svh]\w+-[tblr]{2}/, "lr") + a;
      }
      return Wt + a + qr + a + a;
  }
  return a;
}
var ZH = function(r, u, f, c) {
  if (r.length > -1 && !r.return)
    switch (r.type) {
      case ew:
        r.return = BN(r.value, r.length);
        break;
      case DN:
        return ep([zh(r, {
          value: Xt(r.value, "@", "@" + Wt)
        })], c);
      case qx:
        if (r.length)
          return wH(r.props, function(v) {
            switch (xH(v, /(::plac\w+|:read-\w+)/)) {
              case ":read-only":
              case ":read-write":
                return ep([zh(r, {
                  props: [Xt(v, /:(read-\w+)/, ":" + fb + "$1")]
                })], c);
              case "::placeholder":
                return ep([zh(r, {
                  props: [Xt(v, /:(plac\w+)/, ":" + Wt + "input-$1")]
                }), zh(r, {
                  props: [Xt(v, /:(plac\w+)/, ":" + fb + "$1")]
                }), zh(r, {
                  props: [Xt(v, /:(plac\w+)/, qr + "input-$1")]
                })], c);
            }
            return "";
          });
    }
}, YH = [ZH], $H = function(r) {
  var u = r.key;
  if ( true && !u)
    throw new Error(`You have to configure \`key\` for your cache. Please make sure it's unique (and not equal to 'css') as it's used for linking styles to your cache.
If multiple caches share the same key they might "fight" for each other's style elements.`);
  if (u === "css") {
    var f = document.querySelectorAll("style[data-emotion]:not([data-s])");
    Array.prototype.forEach.call(f, function(M) {
      var k = M.getAttribute("data-emotion");
      k.indexOf(" ") !== -1 && (document.head.appendChild(M), M.setAttribute("data-s", ""));
    });
  }
  var c = r.stylisPlugins || YH;
  if ( true && /[^a-z-]/.test(u))
    throw new Error('Emotion key must only contain lower case alphabetical characters and - but "' + u + '" was passed');
  var v = {}, h, C = [];
  h = r.container || document.head, Array.prototype.forEach.call(
    // this means we will ignore elements which don't have a space in them which
    // means that the style elements we're looking at are only Emotion 11 server-rendered style elements
    document.querySelectorAll('style[data-emotion^="' + u + ' "]'),
    function(M) {
      for (var k = M.getAttribute("data-emotion").split(" "), B = 1; B < k.length; B++)
        v[k[B]] = !0;
      C.push(M);
    }
  );
  var b, E = [zH, GH];
   true && E.push(UH({
    get compat() {
      return L.compat;
    }
  }), XH);
  {
    var w, R = [MH,  true ? function(M) {
      M.root || (M.return ? w.insert(M.return) : M.value && M.type !== Kx && w.insert(M.value + "{}"));
    } : 0], A = LH(E.concat(c, R)), D = function(k) {
      return ep(NH(k), A);
    };
    b = function(k, B, Y, z) {
      w = Y,  true && B.map !== void 0 && (w = {
        insert: function(V) {
          Y.insert(V + B.map);
        }
      }), D(k ? k + "{" + B.styles + "}" : B.styles), z && (L.inserted[B.name] = !0);
    };
  }
  var L = {
    key: u,
    sheet: new gH({
      key: u,
      container: h,
      nonce: r.nonce,
      speedy: r.speedy,
      prepend: r.prepend,
      insertionPoint: r.insertionPoint
    }),
    nonce: r.nonce,
    inserted: v,
    registered: {},
    insert: b
  };
  return L.sheet.hydrate(C), L;
}, VN = Cx(), QH = {
  $$typeof: !0,
  render: !0,
  defaultProps: !0,
  displayName: !0,
  propTypes: !0
}, JH = {
  $$typeof: !0,
  compare: !0,
  defaultProps: !0,
  displayName: !0,
  propTypes: !0,
  type: !0
}, PN = {};
PN[VN.ForwardRef] = QH;
PN[VN.Memo] = JH;
var KH = !0;
function HN(a, r, u) {
  var f = "";
  return u.split(" ").forEach(function(c) {
    a[c] !== void 0 ? r.push(a[c] + ";") : f += c + " ";
  }), f;
}
var nw = function(r, u, f) {
  var c = r.key + "-" + u.name;
  // we only need to add the styles to the registered cache if the
  // class name could be used further down
  // the tree but if it's a string tag, we know it won't
  // so we don't have to add it to registered cache.
  // this improves memory usage since we can avoid storing the whole style string
  (f === !1 || // we need to always store it if we're in compat mode and
  // in node since emotion-server relies on whether a style is in
  // the registered cache to know whether a style is global or not
  // also, note that this check will be dead code eliminated in the browser
  KH === !1) && r.registered[c] === void 0 && (r.registered[c] = u.styles);
}, rw = function(r, u, f) {
  nw(r, u, f);
  var c = r.key + "-" + u.name;
  if (r.inserted[u.name] === void 0) {
    var v = u;
    do
      r.insert(u === v ? "." + c : "", v, r.sheet, !0), v = v.next;
    while (v !== void 0);
  }
};
function qH(a) {
  for (var r = 0, u, f = 0, c = a.length; c >= 4; ++f, c -= 4)
    u = a.charCodeAt(f) & 255 | (a.charCodeAt(++f) & 255) << 8 | (a.charCodeAt(++f) & 255) << 16 | (a.charCodeAt(++f) & 255) << 24, u = /* Math.imul(k, m): */
    (u & 65535) * 1540483477 + ((u >>> 16) * 59797 << 16), u ^= /* k >>> r: */
    u >>> 24, r = /* Math.imul(k, m): */
    (u & 65535) * 1540483477 + ((u >>> 16) * 59797 << 16) ^ /* Math.imul(h, m): */
    (r & 65535) * 1540483477 + ((r >>> 16) * 59797 << 16);
  switch (c) {
    case 3:
      r ^= (a.charCodeAt(f + 2) & 255) << 16;
    case 2:
      r ^= (a.charCodeAt(f + 1) & 255) << 8;
    case 1:
      r ^= a.charCodeAt(f) & 255, r = /* Math.imul(h, m): */
      (r & 65535) * 1540483477 + ((r >>> 16) * 59797 << 16);
  }
  return r ^= r >>> 13, r = /* Math.imul(h, m): */
  (r & 65535) * 1540483477 + ((r >>> 16) * 59797 << 16), ((r ^ r >>> 15) >>> 0).toString(36);
}
var ez = {
  animationIterationCount: 1,
  aspectRatio: 1,
  borderImageOutset: 1,
  borderImageSlice: 1,
  borderImageWidth: 1,
  boxFlex: 1,
  boxFlexGroup: 1,
  boxOrdinalGroup: 1,
  columnCount: 1,
  columns: 1,
  flex: 1,
  flexGrow: 1,
  flexPositive: 1,
  flexShrink: 1,
  flexNegative: 1,
  flexOrder: 1,
  gridRow: 1,
  gridRowEnd: 1,
  gridRowSpan: 1,
  gridRowStart: 1,
  gridColumn: 1,
  gridColumnEnd: 1,
  gridColumnSpan: 1,
  gridColumnStart: 1,
  msGridRow: 1,
  msGridRowSpan: 1,
  msGridColumn: 1,
  msGridColumnSpan: 1,
  fontWeight: 1,
  lineHeight: 1,
  opacity: 1,
  order: 1,
  orphans: 1,
  tabSize: 1,
  widows: 1,
  zIndex: 1,
  zoom: 1,
  WebkitLineClamp: 1,
  // SVG-related properties
  fillOpacity: 1,
  floodOpacity: 1,
  stopOpacity: 1,
  strokeDasharray: 1,
  strokeDashoffset: 1,
  strokeMiterlimit: 1,
  strokeOpacity: 1,
  strokeWidth: 1
}, qA = `You have illegal escape sequence in your template literal, most likely inside content's property value.
Because you write your CSS inside a JavaScript string you actually have to do double escaping, so for example "content: '\\00d7';" should become "content: '\\\\00d7';".
You can read more about this here:
https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Template_literals#ES2018_revision_of_illegal_escape_sequences`, tz = "You have passed in falsy value as style object's key (can happen when in example you pass unexported component as computed key).", nz = /[A-Z]|^ms/g, zN = /_EMO_([^_]+?)_([^]*?)_EMO_/g, aw = function(r) {
  return r.charCodeAt(1) === 45;
}, eO = function(r) {
  return r != null && typeof r != "boolean";
}, FE = /* @__PURE__ */ BH(function(a) {
  return aw(a) ? a : a.replace(nz, "-$&").toLowerCase();
}), db = function(r, u) {
  switch (r) {
    case "animation":
    case "animationName":
      if (typeof u == "string")
        return u.replace(zN, function(f, c, v) {
          return lo = {
            name: c,
            styles: v,
            next: lo
          }, c;
        });
  }
  return ez[r] !== 1 && !aw(r) && typeof u == "number" && u !== 0 ? u + "px" : u;
};
if (true) {
  var rz = /(var|attr|counters?|url|element|(((repeating-)?(linear|radial))|conic)-gradient)\(|(no-)?(open|close)-quote/, az = ["normal", "none", "initial", "inherit", "unset"], iz = db, oz = /^-ms-/, uz = /-(.)/g, tO = {};
  db = function(r, u) {
    if (r === "content" && (typeof u != "string" || az.indexOf(u) === -1 && !rz.test(u) && (u.charAt(0) !== u.charAt(u.length - 1) || u.charAt(0) !== '"' && u.charAt(0) !== "'")))
      throw new Error("You seem to be using a value for 'content' without quotes, try replacing it with `content: '\"" + u + "\"'`");
    var f = iz(r, u);
    return f !== "" && !aw(r) && r.indexOf("-") !== -1 && tO[r] === void 0 && (tO[r] = !0, console.error("Using kebab-case for css properties in objects is not supported. Did you mean " + r.replace(oz, "ms-").replace(uz, function(c, v) {
      return v.toUpperCase();
    }) + "?")), f;
  };
}
var GN = "Component selectors can only be used in conjunction with @emotion/babel-plugin, the swc Emotion plugin, or another Emotion-aware compiler transform.";
function im(a, r, u) {
  if (u == null)
    return "";
  if (u.__emotion_styles !== void 0) {
    if ( true && u.toString() === "NO_COMPONENT_SELECTOR")
      throw new Error(GN);
    return u;
  }
  switch (typeof u) {
    case "boolean":
      return "";
    case "object": {
      if (u.anim === 1)
        return lo = {
          name: u.name,
          styles: u.styles,
          next: lo
        }, u.name;
      if (u.styles !== void 0) {
        var f = u.next;
        if (f !== void 0)
          for (; f !== void 0; )
            lo = {
              name: f.name,
              styles: f.styles,
              next: lo
            }, f = f.next;
        var c = u.styles + ";";
        return  true && u.map !== void 0 && (c += u.map), c;
      }
      return lz(a, r, u);
    }
    case "function": {
      if (a !== void 0) {
        var v = lo, h = u(a);
        return lo = v, im(a, r, h);
      } else
         true && console.error("Functions that are interpolated in css calls will be stringified.\nIf you want to have a css call based on props, create a function that returns a css call like this\nlet dynamicStyle = (props) => css`color: ${props.color}`\nIt can be called directly with props or interpolated in a styled call like this\nlet SomeComponent = styled('div')`${dynamicStyle}`");
      break;
    }
    case "string":
      if (true) {
        var C = [], b = u.replace(zN, function(w, R, A) {
          var D = "animation" + C.length;
          return C.push("const " + D + " = keyframes`" + A.replace(/^@keyframes animation-\w+/, "") + "`"), "${" + D + "}";
        });
        C.length && console.error("`keyframes` output got interpolated into plain string, please wrap it with `css`.\n\nInstead of doing this:\n\n" + [].concat(C, ["`" + b + "`"]).join(`
`) + `

You should wrap it with \`css\` like this:

` + ("css`" + b + "`"));
      }
      break;
  }
  if (r == null)
    return u;
  var E = r[u];
  return E !== void 0 ? E : u;
}
function lz(a, r, u) {
  var f = "";
  if (Array.isArray(u))
    for (var c = 0; c < u.length; c++)
      f += im(a, r, u[c]) + ";";
  else
    for (var v in u) {
      var h = u[v];
      if (typeof h != "object")
        r != null && r[h] !== void 0 ? f += v + "{" + r[h] + "}" : eO(h) && (f += FE(v) + ":" + db(v, h) + ";");
      else {
        if (v === "NO_COMPONENT_SELECTOR" && "development" !== "production")
          throw new Error(GN);
        if (Array.isArray(h) && typeof h[0] == "string" && (r == null || r[h[0]] === void 0))
          for (var C = 0; C < h.length; C++)
            eO(h[C]) && (f += FE(v) + ":" + db(v, h[C]) + ";");
        else {
          var b = im(a, r, h);
          switch (v) {
            case "animation":
            case "animationName": {
              f += FE(v) + ":" + b + ";";
              break;
            }
            default:
               true && v === "undefined" && console.error(tz), f += v + "{" + b + "}";
          }
        }
      }
    }
  return f;
}
var nO = /label:\s*([^\s;\n{]+)\s*(;|$)/g, _N;
 true && (_N = /\/\*#\ssourceMappingURL=data:application\/json;\S+\s+\*\//g);
var lo, om = function(r, u, f) {
  if (r.length === 1 && typeof r[0] == "object" && r[0] !== null && r[0].styles !== void 0)
    return r[0];
  var c = !0, v = "";
  lo = void 0;
  var h = r[0];
  h == null || h.raw === void 0 ? (c = !1, v += im(f, u, h)) : ( true && h[0] === void 0 && console.error(qA), v += h[0]);
  for (var C = 1; C < r.length; C++)
    v += im(f, u, r[C]), c && ( true && h[C] === void 0 && console.error(qA), v += h[C]);
  var b;
   true && (v = v.replace(_N, function(A) {
    return b = A, "";
  })), nO.lastIndex = 0;
  for (var E = "", w; (w = nO.exec(v)) !== null; )
    E += "-" + // $FlowFixMe we know it's not null
    w[1];
  var R = qH(v) + E;
  return  true ? {
    name: R,
    styles: v,
    map: b,
    next: lo,
    toString: function() {
      return "You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).";
    }
  } : 0;
}, sz = function(r) {
  return r();
}, jN = react__WEBPACK_IMPORTED_MODULE_1__.useInsertionEffect ? react__WEBPACK_IMPORTED_MODULE_1__.useInsertionEffect : !1, UN = jN || sz, rO = jN || react__WEBPACK_IMPORTED_MODULE_1__.useLayoutEffect, iw = {}.hasOwnProperty, ow = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext(
  // we're doing this to avoid preconstruct's dead code elimination in this one case
  // because this module is primarily intended for the browser and node
  // but it's also required in react native and similar environments sometimes
  // and we could have a special build just for that
  // but this is much easier and the native packages
  // might use a different theme context in the future anyway
  typeof HTMLElement < "u" ? /* @__PURE__ */ $H({
    key: "css"
  }) : null
);
 true && (ow.displayName = "EmotionCacheContext");
ow.Provider;
var uw = function(r) {
  return /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.forwardRef)(function(u, f) {
    var c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(ow);
    return r(u, c, f);
  });
}, Pb = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createContext({});
 true && (Pb.displayName = "EmotionThemeContext");
var aO = function(r) {
  var u = r.split(".");
  return u[u.length - 1];
}, cz = function(r) {
  var u = /^\s+at\s+([A-Za-z0-9$.]+)\s/.exec(r);
  if (u || (u = /^([A-Za-z0-9$.]+)@/.exec(r), u))
    return aO(u[1]);
}, fz = /* @__PURE__ */ new Set(["renderWithHooks", "processChild", "finishClassComponent", "renderToString"]), dz = function(r) {
  return r.replace(/\$/g, "-");
}, pz = function(r) {
  if (r)
    for (var u = r.split(`
`), f = 0; f < u.length; f++) {
      var c = cz(u[f]);
      if (c) {
        if (fz.has(c))
          break;
        if (/^[A-Z]/.test(c))
          return dz(c);
      }
    }
}, hx = "__EMOTION_TYPE_PLEASE_DO_NOT_USE__", mx = "__EMOTION_LABEL_PLEASE_DO_NOT_USE__", vz = function(r, u) {
  if ( true && typeof u.css == "string" && // check if there is a css declaration
  u.css.indexOf(":") !== -1)
    throw new Error("Strings are not allowed as css prop values, please wrap it in a css template literal from '@emotion/react' like this: css`" + u.css + "`");
  var f = {};
  for (var c in u)
    iw.call(u, c) && (f[c] = u[c]);
  if (f[hx] = r,  true && u.css && (typeof u.css != "object" || typeof u.css.name != "string" || u.css.name.indexOf("-") === -1)) {
    var v = pz(new Error().stack);
    v && (f[mx] = v);
  }
  return f;
}, hz = function(r) {
  var u = r.cache, f = r.serialized, c = r.isStringTag;
  return nw(u, f, c), UN(function() {
    return rw(u, f, c);
  }), null;
}, WN = /* @__PURE__ */ uw(function(a, r, u) {
  var f = a.css;
  typeof f == "string" && r.registered[f] !== void 0 && (f = r.registered[f]);
  var c = a[hx], v = [f], h = "";
  typeof a.className == "string" ? h = HN(r.registered, v, a.className) : a.className != null && (h = a.className + " ");
  var C = om(v, void 0, react__WEBPACK_IMPORTED_MODULE_1__.useContext(Pb));
  if ( true && C.name.indexOf("-") === -1) {
    var b = a[mx];
    b && (C = om([C, "label:" + b + ";"]));
  }
  h += r.key + "-" + C.name;
  var E = {};
  for (var w in a)
    iw.call(a, w) && w !== "css" && w !== hx && ( false || w !== mx) && (E[w] = a[w]);
  return E.ref = u, E.className = h, /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(hz, {
    cache: r,
    serialized: C,
    isStringTag: typeof c == "string"
  }), /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(c, E));
});
 true && (WN.displayName = "EmotionCssPropInternal");
var mz = WN, gz = {
  name: "@emotion/react",
  version: "11.11.4",
  main: "dist/emotion-react.cjs.js",
  module: "dist/emotion-react.esm.js",
  browser: {
    "./dist/emotion-react.esm.js": "./dist/emotion-react.browser.esm.js"
  },
  exports: {
    ".": {
      module: {
        worker: "./dist/emotion-react.worker.esm.js",
        browser: "./dist/emotion-react.browser.esm.js",
        default: "./dist/emotion-react.esm.js"
      },
      import: "./dist/emotion-react.cjs.mjs",
      default: "./dist/emotion-react.cjs.js"
    },
    "./jsx-runtime": {
      module: {
        worker: "./jsx-runtime/dist/emotion-react-jsx-runtime.worker.esm.js",
        browser: "./jsx-runtime/dist/emotion-react-jsx-runtime.browser.esm.js",
        default: "./jsx-runtime/dist/emotion-react-jsx-runtime.esm.js"
      },
      import: "./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.mjs",
      default: "./jsx-runtime/dist/emotion-react-jsx-runtime.cjs.js"
    },
    "./_isolated-hnrs": {
      module: {
        worker: "./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.worker.esm.js",
        browser: "./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.browser.esm.js",
        default: "./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.esm.js"
      },
      import: "./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.mjs",
      default: "./_isolated-hnrs/dist/emotion-react-_isolated-hnrs.cjs.js"
    },
    "./jsx-dev-runtime": {
      module: {
        worker: "./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.worker.esm.js",
        browser: "./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.browser.esm.js",
        default: "./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.esm.js"
      },
      import: "./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.mjs",
      default: "./jsx-dev-runtime/dist/emotion-react-jsx-dev-runtime.cjs.js"
    },
    "./package.json": "./package.json",
    "./types/css-prop": "./types/css-prop.d.ts",
    "./macro": {
      types: {
        import: "./macro.d.mts",
        default: "./macro.d.ts"
      },
      default: "./macro.js"
    }
  },
  types: "types/index.d.ts",
  files: [
    "src",
    "dist",
    "jsx-runtime",
    "jsx-dev-runtime",
    "_isolated-hnrs",
    "types/*.d.ts",
    "macro.*"
  ],
  sideEffects: !1,
  author: "Emotion Contributors",
  license: "MIT",
  scripts: {
    "test:typescript": "dtslint types"
  },
  dependencies: {
    "@babel/runtime": "^7.18.3",
    "@emotion/babel-plugin": "^11.11.0",
    "@emotion/cache": "^11.11.0",
    "@emotion/serialize": "^1.1.3",
    "@emotion/use-insertion-effect-with-fallbacks": "^1.0.1",
    "@emotion/utils": "^1.2.1",
    "@emotion/weak-memoize": "^0.3.1",
    "hoist-non-react-statics": "^3.3.1"
  },
  peerDependencies: {
    react: ">=16.8.0"
  },
  peerDependenciesMeta: {
    "@types/react": {
      optional: !0
    }
  },
  devDependencies: {
    "@definitelytyped/dtslint": "0.0.112",
    "@emotion/css": "11.11.2",
    "@emotion/css-prettifier": "1.1.3",
    "@emotion/server": "11.11.0",
    "@emotion/styled": "11.11.0",
    "html-tag-names": "^1.1.2",
    react: "16.14.0",
    "svg-tag-names": "^1.1.1",
    typescript: "^4.5.5"
  },
  repository: "https://github.com/emotion-js/emotion/tree/main/packages/react",
  publishConfig: {
    access: "public"
  },
  "umd:main": "dist/emotion-react.umd.min.js",
  preconstruct: {
    entrypoints: [
      "./index.js",
      "./jsx-runtime.js",
      "./jsx-dev-runtime.js",
      "./_isolated-hnrs.js"
    ],
    umdName: "emotionReact",
    exports: {
      envConditions: [
        "browser",
        "worker"
      ],
      extra: {
        "./types/css-prop": "./types/css-prop.d.ts",
        "./macro": {
          types: {
            import: "./macro.d.mts",
            default: "./macro.d.ts"
          },
          default: "./macro.js"
        }
      }
    }
  }
}, lt = function(r, u) {
  var f = arguments;
  if (u == null || !iw.call(u, "css"))
    return react__WEBPACK_IMPORTED_MODULE_1__.createElement.apply(void 0, f);
  var c = f.length, v = new Array(c);
  v[0] = mz, v[1] = vz(r, u);
  for (var h = 2; h < c; h++)
    v[h] = f[h];
  return react__WEBPACK_IMPORTED_MODULE_1__.createElement.apply(null, v);
}, iO = !1, yz = /* @__PURE__ */ uw(function(a, r) {
   true && !iO && // check for className as well since the user is
  // probably using the custom createElement which
  // means it will be turned into a className prop
  // $FlowFixMe I don't really want to add it to the type since it shouldn't be used
  (a.className || a.css) && (console.error("It looks like you're using the css prop on Global, did you mean to use the styles prop instead?"), iO = !0);
  var u = a.styles, f = om([u], void 0, react__WEBPACK_IMPORTED_MODULE_1__.useContext(Pb)), c = react__WEBPACK_IMPORTED_MODULE_1__.useRef();
  return rO(function() {
    var v = r.key + "-global", h = new r.sheet.constructor({
      key: v,
      nonce: r.sheet.nonce,
      container: r.sheet.container,
      speedy: r.sheet.isSpeedy
    }), C = !1, b = document.querySelector('style[data-emotion="' + v + " " + f.name + '"]');
    return r.sheet.tags.length && (h.before = r.sheet.tags[0]), b !== null && (C = !0, b.setAttribute("data-emotion", v), h.hydrate([b])), c.current = [h, C], function() {
      h.flush();
    };
  }, [r]), rO(function() {
    var v = c.current, h = v[0], C = v[1];
    if (C) {
      v[1] = !1;
      return;
    }
    if (f.next !== void 0 && rw(r, f.next, !0), h.tags.length) {
      var b = h.tags[h.tags.length - 1].nextElementSibling;
      h.before = b, h.flush();
    }
    r.insert("", f, h, !1);
  }, [r, f.name]), null;
});
 true && (yz.displayName = "EmotionGlobal");
function lw() {
  for (var a = arguments.length, r = new Array(a), u = 0; u < a; u++)
    r[u] = arguments[u];
  return om(r);
}
var bz = function() {
  var r = lw.apply(void 0, arguments), u = "animation-" + r.name;
  return {
    name: u,
    styles: "@keyframes " + u + "{" + r.styles + "}",
    anim: 1,
    toString: function() {
      return "_EMO_" + this.name + "_" + this.styles + "_EMO_";
    }
  };
}, Cz = function a(r) {
  for (var u = r.length, f = 0, c = ""; f < u; f++) {
    var v = r[f];
    if (v != null) {
      var h = void 0;
      switch (typeof v) {
        case "boolean":
          break;
        case "object": {
          if (Array.isArray(v))
            h = a(v);
          else {
             true && v.styles !== void 0 && v.name !== void 0 && console.error("You have passed styles created with `css` from `@emotion/react` package to the `cx`.\n`cx` is meant to compose class names (strings) so you should convert those styles to a class name by passing them to the `css` received from <ClassNames/> component."), h = "";
            for (var C in v)
              v[C] && C && (h && (h += " "), h += C);
          }
          break;
        }
        default:
          h = v;
      }
      h && (c && (c += " "), c += h);
    }
  }
  return c;
};
function Sz(a, r, u) {
  var f = [], c = HN(a, f, u);
  return f.length < 2 ? u : c + r(f);
}
var Ez = function(r) {
  var u = r.cache, f = r.serializedArr;
  return UN(function() {
    for (var c = 0; c < f.length; c++)
      rw(u, f[c], !1);
  }), null;
}, xz = /* @__PURE__ */ uw(function(a, r) {
  var u = !1, f = [], c = function() {
    if (u && "development" !== "production")
      throw new Error("css can only be used during render");
    for (var E = arguments.length, w = new Array(E), R = 0; R < E; R++)
      w[R] = arguments[R];
    var A = om(w, r.registered);
    return f.push(A), nw(r, A, !1), r.key + "-" + A.name;
  }, v = function() {
    if (u && "development" !== "production")
      throw new Error("cx can only be used during render");
    for (var E = arguments.length, w = new Array(E), R = 0; R < E; R++)
      w[R] = arguments[R];
    return Sz(r.registered, c, Cz(w));
  }, h = {
    css: c,
    cx: v,
    theme: react__WEBPACK_IMPORTED_MODULE_1__.useContext(Pb)
  }, C = a.children(h);
  return u = !0, /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(Ez, {
    cache: r,
    serializedArr: f
  }), C);
});
 true && (xz.displayName = "EmotionClassNames");
if (true) {
  var oO = !0, wz = typeof jest < "u" || typeof vi < "u";
  if (oO && !wz) {
    var uO = (
      // $FlowIgnore
      typeof globalThis < "u" ? globalThis : oO ? window : __webpack_require__.g
    ), lO = "__EMOTION_REACT_" + gz.version.split(".")[0] + "__";
    uO[lO] && console.warn("You are loading @emotion/react when it is already loaded. Running multiple instances may cause problems. This can happen if multiple versions are used, or if multiple builds of the same version are used."), uO[lO] = !0;
  }
}
function Tz(a, r) {
  return r || (r = a.slice(0)), Object.freeze(Object.defineProperties(a, {
    raw: {
      value: Object.freeze(r)
    }
  }));
}
const Rz = Math.min, Iz = Math.max, pb = Math.round, U0 = Math.floor, vb = (a) => ({
  x: a,
  y: a
});
function Az(a) {
  const {
    x: r,
    y: u,
    width: f,
    height: c
  } = a;
  return {
    width: f,
    height: c,
    top: u,
    left: r,
    right: r + f,
    bottom: u + c,
    x: r,
    y: u
  };
}
function XN(a) {
  return YN(a) ? (a.nodeName || "").toLowerCase() : "#document";
}
function Uu(a) {
  var r;
  return (a == null || (r = a.ownerDocument) == null ? void 0 : r.defaultView) || window;
}
function ZN(a) {
  var r;
  return (r = (YN(a) ? a.ownerDocument : a.document) || window.document) == null ? void 0 : r.documentElement;
}
function YN(a) {
  return a instanceof Node || a instanceof Uu(a).Node;
}
function Oz(a) {
  return a instanceof Element || a instanceof Uu(a).Element;
}
function sw(a) {
  return a instanceof HTMLElement || a instanceof Uu(a).HTMLElement;
}
function sO(a) {
  return typeof ShadowRoot > "u" ? !1 : a instanceof ShadowRoot || a instanceof Uu(a).ShadowRoot;
}
function $N(a) {
  const {
    overflow: r,
    overflowX: u,
    overflowY: f,
    display: c
  } = cw(a);
  return /auto|scroll|overlay|hidden|clip/.test(r + f + u) && !["inline", "contents"].includes(c);
}
function Dz() {
  return typeof CSS > "u" || !CSS.supports ? !1 : CSS.supports("-webkit-backdrop-filter", "none");
}
function Nz(a) {
  return ["html", "body", "#document"].includes(XN(a));
}
function cw(a) {
  return Uu(a).getComputedStyle(a);
}
function kz(a) {
  if (XN(a) === "html")
    return a;
  const r = (
    // Step into the shadow DOM of the parent of a slotted node.
    a.assignedSlot || // DOM Element detected.
    a.parentNode || // ShadowRoot detected.
    sO(a) && a.host || // Fallback.
    ZN(a)
  );
  return sO(r) ? r.host : r;
}
function QN(a) {
  const r = kz(a);
  return Nz(r) ? a.ownerDocument ? a.ownerDocument.body : a.body : sw(r) && $N(r) ? r : QN(r);
}
function hb(a, r, u) {
  var f;
  r === void 0 && (r = []), u === void 0 && (u = !0);
  const c = QN(a), v = c === ((f = a.ownerDocument) == null ? void 0 : f.body), h = Uu(c);
  return v ? r.concat(h, h.visualViewport || [], $N(c) ? c : [], h.frameElement && u ? hb(h.frameElement) : []) : r.concat(c, hb(c, [], u));
}
function Mz(a) {
  const r = cw(a);
  let u = parseFloat(r.width) || 0, f = parseFloat(r.height) || 0;
  const c = sw(a), v = c ? a.offsetWidth : u, h = c ? a.offsetHeight : f, C = pb(u) !== v || pb(f) !== h;
  return C && (u = v, f = h), {
    width: u,
    height: f,
    $: C
  };
}
function fw(a) {
  return Oz(a) ? a : a.contextElement;
}
function cO(a) {
  const r = fw(a);
  if (!sw(r))
    return vb(1);
  const u = r.getBoundingClientRect(), {
    width: f,
    height: c,
    $: v
  } = Mz(r);
  let h = (v ? pb(u.width) : u.width) / f, C = (v ? pb(u.height) : u.height) / c;
  return (!h || !Number.isFinite(h)) && (h = 1), (!C || !Number.isFinite(C)) && (C = 1), {
    x: h,
    y: C
  };
}
const Lz = /* @__PURE__ */ vb(0);
function Fz(a) {
  const r = Uu(a);
  return !Dz() || !r.visualViewport ? Lz : {
    x: r.visualViewport.offsetLeft,
    y: r.visualViewport.offsetTop
  };
}
function Bz(a, r, u) {
  return !1;
}
function fO(a, r, u, f) {
  r === void 0 && (r = !1);
  const c = a.getBoundingClientRect(), v = fw(a);
  let h = vb(1);
  r && (h = cO(a));
  const C = Bz() ? Fz(v) : vb(0);
  let b = (c.left + C.x) / h.x, E = (c.top + C.y) / h.y, w = c.width / h.x, R = c.height / h.y;
  if (v) {
    const A = Uu(v), D = f;
    let L = A, M = L.frameElement;
    for (; M && f && D !== L; ) {
      const k = cO(M), B = M.getBoundingClientRect(), Y = cw(M), z = B.left + (M.clientLeft + parseFloat(Y.paddingLeft)) * k.x, Z = B.top + (M.clientTop + parseFloat(Y.paddingTop)) * k.y;
      b *= k.x, E *= k.y, w *= k.x, R *= k.y, b += z, E += Z, L = Uu(M), M = L.frameElement;
    }
  }
  return Az({
    width: w,
    height: R,
    x: b,
    y: E
  });
}
function Vz(a, r) {
  let u = null, f;
  const c = ZN(a);
  function v() {
    var C;
    clearTimeout(f), (C = u) == null || C.disconnect(), u = null;
  }
  function h(C, b) {
    C === void 0 && (C = !1), b === void 0 && (b = 1), v();
    const {
      left: E,
      top: w,
      width: R,
      height: A
    } = a.getBoundingClientRect();
    if (C || r(), !R || !A)
      return;
    const D = U0(w), L = U0(c.clientWidth - (E + R)), M = U0(c.clientHeight - (w + A)), k = U0(E), Y = {
      rootMargin: -D + "px " + -L + "px " + -M + "px " + -k + "px",
      threshold: Iz(0, Rz(1, b)) || 1
    };
    let z = !0;
    function Z(V) {
      const Q = V[0].intersectionRatio;
      if (Q !== b) {
        if (!z)
          return h();
        Q ? h(!1, Q) : f = setTimeout(() => {
          h(!1, 1e-7);
        }, 1e3);
      }
      z = !1;
    }
    try {
      u = new IntersectionObserver(Z, {
        ...Y,
        // Handle <iframe>s
        root: c.ownerDocument
      });
    } catch {
      u = new IntersectionObserver(Z, Y);
    }
    u.observe(a);
  }
  return h(!0), v;
}
function Pz(a, r, u, f) {
  f === void 0 && (f = {});
  const {
    ancestorScroll: c = !0,
    ancestorResize: v = !0,
    elementResize: h = typeof ResizeObserver == "function",
    layoutShift: C = typeof IntersectionObserver == "function",
    animationFrame: b = !1
  } = f, E = fw(a), w = c || v ? [...E ? hb(E) : [], ...hb(r)] : [];
  w.forEach((B) => {
    c && B.addEventListener("scroll", u, {
      passive: !0
    }), v && B.addEventListener("resize", u);
  });
  const R = E && C ? Vz(E, u) : null;
  let A = -1, D = null;
  h && (D = new ResizeObserver((B) => {
    let [Y] = B;
    Y && Y.target === E && D && (D.unobserve(r), cancelAnimationFrame(A), A = requestAnimationFrame(() => {
      var z;
      (z = D) == null || z.observe(r);
    })), u();
  }), E && !b && D.observe(E), D.observe(r));
  let L, M = b ? fO(a) : null;
  b && k();
  function k() {
    const B = fO(a);
    M && (B.x !== M.x || B.y !== M.y || B.width !== M.width || B.height !== M.height) && u(), M = B, L = requestAnimationFrame(k);
  }
  return u(), () => {
    var B;
    w.forEach((Y) => {
      c && Y.removeEventListener("scroll", u), v && Y.removeEventListener("resize", u);
    }), R == null || R(), (B = D) == null || B.disconnect(), D = null, b && cancelAnimationFrame(L);
  };
}
var gx = react__WEBPACK_IMPORTED_MODULE_1__.useLayoutEffect, Hz = ["className", "clearValue", "cx", "getStyles", "getClassNames", "getValue", "hasValue", "isMulti", "isRtl", "options", "selectOption", "selectProps", "setValue", "theme"], mb = function() {
};
function zz(a, r) {
  return r ? r[0] === "-" ? a + r : a + "__" + r : a;
}
function Gz(a, r) {
  for (var u = arguments.length, f = new Array(u > 2 ? u - 2 : 0), c = 2; c < u; c++)
    f[c - 2] = arguments[c];
  var v = [].concat(f);
  if (r && a)
    for (var h in r)
      r.hasOwnProperty(h) && r[h] && v.push("".concat(zz(a, h)));
  return v.filter(function(C) {
    return C;
  }).map(function(C) {
    return String(C).trim();
  }).join(" ");
}
var gb = function(r) {
  return Qz(r) ? r.filter(Boolean) : _c(r) === "object" && r !== null ? [r] : [];
}, JN = function(r) {
  r.className, r.clearValue, r.cx, r.getStyles, r.getClassNames, r.getValue, r.hasValue, r.isMulti, r.isRtl, r.options, r.selectOption, r.selectProps, r.setValue, r.theme;
  var u = $o(r, Hz);
  return it({}, u);
}, Fn = function(r, u, f) {
  var c = r.cx, v = r.getStyles, h = r.getClassNames, C = r.className;
  return {
    css: v(u, r),
    className: c(f ?? {}, h(u, r), C)
  };
};
function Hb(a) {
  return [document.documentElement, document.body, window].indexOf(a) > -1;
}
function _z(a) {
  return Hb(a) ? window.innerHeight : a.clientHeight;
}
function KN(a) {
  return Hb(a) ? window.pageYOffset : a.scrollTop;
}
function yb(a, r) {
  if (Hb(a)) {
    window.scrollTo(0, r);
    return;
  }
  a.scrollTop = r;
}
function jz(a) {
  var r = getComputedStyle(a), u = r.position === "absolute", f = /(auto|scroll)/;
  if (r.position === "fixed")
    return document.documentElement;
  for (var c = a; c = c.parentElement; )
    if (r = getComputedStyle(c), !(u && r.position === "static") && f.test(r.overflow + r.overflowY + r.overflowX))
      return c;
  return document.documentElement;
}
function Uz(a, r, u, f) {
  return u * ((a = a / f - 1) * a * a + 1) + r;
}
function W0(a, r) {
  var u = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : 200, f = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : mb, c = KN(a), v = r - c, h = 10, C = 0;
  function b() {
    C += h;
    var E = Uz(C, c, v, u);
    yb(a, E), C < u ? window.requestAnimationFrame(b) : f(a);
  }
  b();
}
function dO(a, r) {
  var u = a.getBoundingClientRect(), f = r.getBoundingClientRect(), c = r.offsetHeight / 3;
  f.bottom + c > u.bottom ? yb(a, Math.min(r.offsetTop + r.clientHeight - a.offsetHeight + c, a.scrollHeight)) : f.top - c < u.top && yb(a, Math.max(r.offsetTop - c, 0));
}
function Wz(a) {
  var r = a.getBoundingClientRect();
  return {
    bottom: r.bottom,
    height: r.height,
    left: r.left,
    right: r.right,
    top: r.top,
    width: r.width
  };
}
function pO() {
  try {
    return document.createEvent("TouchEvent"), !0;
  } catch {
    return !1;
  }
}
function Xz() {
  try {
    return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
  } catch {
    return !1;
  }
}
var qN = !1, Zz = {
  get passive() {
    return qN = !0;
  }
}, X0 = typeof window < "u" ? window : {};
X0.addEventListener && X0.removeEventListener && (X0.addEventListener("p", mb, Zz), X0.removeEventListener("p", mb, !1));
var Yz = qN;
function $z(a) {
  return a != null;
}
function Qz(a) {
  return Array.isArray(a);
}
function Xh(a, r, u) {
  return a ? r : u;
}
var Jz = function(r) {
  for (var u = arguments.length, f = new Array(u > 1 ? u - 1 : 0), c = 1; c < u; c++)
    f[c - 1] = arguments[c];
  var v = Object.entries(r).filter(function(h) {
    var C = Gu(h, 1), b = C[0];
    return !f.includes(b);
  });
  return v.reduce(function(h, C) {
    var b = Gu(C, 2), E = b[0], w = b[1];
    return h[E] = w, h;
  }, {});
}, Kz = ["children", "innerProps"], qz = ["children", "innerProps"];
function eG(a) {
  var r = a.maxHeight, u = a.menuEl, f = a.minHeight, c = a.placement, v = a.shouldScroll, h = a.isFixedPosition, C = a.controlHeight, b = jz(u), E = {
    placement: "bottom",
    maxHeight: r
  };
  if (!u || !u.offsetParent)
    return E;
  var w = b.getBoundingClientRect(), R = w.height, A = u.getBoundingClientRect(), D = A.bottom, L = A.height, M = A.top, k = u.offsetParent.getBoundingClientRect(), B = k.top, Y = h ? window.innerHeight : _z(b), z = KN(b), Z = parseInt(getComputedStyle(u).marginBottom, 10), V = parseInt(getComputedStyle(u).marginTop, 10), Q = B - V, P = Y - M, U = Q + z, se = R - z - M, Se = D - Y + z + Z, he = z + M - V, xe = 160;
  switch (c) {
    case "auto":
    case "bottom":
      if (P >= L)
        return {
          placement: "bottom",
          maxHeight: r
        };
      if (se >= L && !h)
        return v && W0(b, Se, xe), {
          placement: "bottom",
          maxHeight: r
        };
      if (!h && se >= f || h && P >= f) {
        v && W0(b, Se, xe);
        var ye = h ? P - Z : se - Z;
        return {
          placement: "bottom",
          maxHeight: ye
        };
      }
      if (c === "auto" || h) {
        var Me = r, We = h ? Q : U;
        return We >= f && (Me = Math.min(We - Z - C, r)), {
          placement: "top",
          maxHeight: Me
        };
      }
      if (c === "bottom")
        return v && yb(b, Se), {
          placement: "bottom",
          maxHeight: r
        };
      break;
    case "top":
      if (Q >= L)
        return {
          placement: "top",
          maxHeight: r
        };
      if (U >= L && !h)
        return v && W0(b, he, xe), {
          placement: "top",
          maxHeight: r
        };
      if (!h && U >= f || h && Q >= f) {
        var Ee = r;
        return (!h && U >= f || h && Q >= f) && (Ee = h ? Q - V : U - V), v && W0(b, he, xe), {
          placement: "top",
          maxHeight: Ee
        };
      }
      return {
        placement: "bottom",
        maxHeight: r
      };
    default:
      throw new Error('Invalid placement provided "'.concat(c, '".'));
  }
  return E;
}
function tG(a) {
  var r = {
    bottom: "top",
    top: "bottom"
  };
  return a ? r[a] : "bottom";
}
var e2 = function(r) {
  return r === "auto" ? "bottom" : r;
}, nG = function(r, u) {
  var f, c = r.placement, v = r.theme, h = v.borderRadius, C = v.spacing, b = v.colors;
  return it((f = {
    label: "menu"
  }, Wh(f, tG(c), "100%"), Wh(f, "position", "absolute"), Wh(f, "width", "100%"), Wh(f, "zIndex", 1), f), u ? {} : {
    backgroundColor: b.neutral0,
    borderRadius: h,
    boxShadow: "0 0 0 1px hsla(0, 0%, 0%, 0.1), 0 4px 11px hsla(0, 0%, 0%, 0.1)",
    marginBottom: C.menuGutter,
    marginTop: C.menuGutter
  });
}, t2 = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.createContext)(null), rG = function(r) {
  var u = r.children, f = r.minMenuHeight, c = r.maxMenuHeight, v = r.menuPlacement, h = r.menuPosition, C = r.menuShouldScrollIntoView, b = r.theme, E = (0,react__WEBPACK_IMPORTED_MODULE_1__.useContext)(t2) || {}, w = E.setPortalPlacement, R = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), A = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(c), D = Gu(A, 2), L = D[0], M = D[1], k = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(null), B = Gu(k, 2), Y = B[0], z = B[1], Z = b.spacing.controlHeight;
  return gx(function() {
    var V = R.current;
    if (V) {
      var Q = h === "fixed", P = C && !Q, U = eG({
        maxHeight: c,
        menuEl: V,
        minHeight: f,
        placement: v,
        shouldScroll: P,
        isFixedPosition: Q,
        controlHeight: Z
      });
      M(U.maxHeight), z(U.placement), w == null || w(U.placement);
    }
  }, [c, v, h, C, f, w, Z]), u({
    ref: R,
    placerProps: it(it({}, r), {}, {
      placement: Y || e2(v),
      maxHeight: L
    })
  });
}, aG = function(r) {
  var u = r.children, f = r.innerRef, c = r.innerProps;
  return lt("div", st({}, Fn(r, "menu", {
    menu: !0
  }), {
    ref: f
  }, c), u);
}, iG = aG, oG = function(r, u) {
  var f = r.maxHeight, c = r.theme.spacing.baseUnit;
  return it({
    maxHeight: f,
    overflowY: "auto",
    position: "relative",
    // required for offset[Height, Top] > keyboard scroll
    WebkitOverflowScrolling: "touch"
  }, u ? {} : {
    paddingBottom: c,
    paddingTop: c
  });
}, uG = function(r) {
  var u = r.children, f = r.innerProps, c = r.innerRef, v = r.isMulti;
  return lt("div", st({}, Fn(r, "menuList", {
    "menu-list": !0,
    "menu-list--is-multi": v
  }), {
    ref: c
  }, f), u);
}, n2 = function(r, u) {
  var f = r.theme, c = f.spacing.baseUnit, v = f.colors;
  return it({
    textAlign: "center"
  }, u ? {} : {
    color: v.neutral40,
    padding: "".concat(c * 2, "px ").concat(c * 3, "px")
  });
}, lG = n2, sG = n2, cG = function(r) {
  var u = r.children, f = u === void 0 ? "No options" : u, c = r.innerProps, v = $o(r, Kz);
  return lt("div", st({}, Fn(it(it({}, v), {}, {
    children: f,
    innerProps: c
  }), "noOptionsMessage", {
    "menu-notice": !0,
    "menu-notice--no-options": !0
  }), c), f);
}, fG = function(r) {
  var u = r.children, f = u === void 0 ? "Loading..." : u, c = r.innerProps, v = $o(r, qz);
  return lt("div", st({}, Fn(it(it({}, v), {}, {
    children: f,
    innerProps: c
  }), "loadingMessage", {
    "menu-notice": !0,
    "menu-notice--loading": !0
  }), c), f);
}, dG = function(r) {
  var u = r.rect, f = r.offset, c = r.position;
  return {
    left: u.left,
    position: c,
    top: f,
    width: u.width,
    zIndex: 1
  };
}, pG = function(r) {
  var u = r.appendTo, f = r.children, c = r.controlElement, v = r.innerProps, h = r.menuPlacement, C = r.menuPosition, b = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), E = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), w = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(e2(h)), R = Gu(w, 2), A = R[0], D = R[1], L = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    return {
      setPortalPlacement: D
    };
  }, []), M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(null), k = Gu(M, 2), B = k[0], Y = k[1], z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function() {
    if (c) {
      var P = Wz(c), U = C === "fixed" ? 0 : window.pageYOffset, se = P[A] + U;
      (se !== (B == null ? void 0 : B.offset) || P.left !== (B == null ? void 0 : B.rect.left) || P.width !== (B == null ? void 0 : B.rect.width)) && Y({
        offset: se,
        rect: P
      });
    }
  }, [c, C, A, B == null ? void 0 : B.offset, B == null ? void 0 : B.rect.left, B == null ? void 0 : B.rect.width]);
  gx(function() {
    z();
  }, [z]);
  var Z = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function() {
    typeof E.current == "function" && (E.current(), E.current = null), c && b.current && (E.current = Pz(c, b.current, z, {
      elementResize: "ResizeObserver" in window
    }));
  }, [c, z]);
  gx(function() {
    Z();
  }, [Z]);
  var V = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(P) {
    b.current = P, Z();
  }, [Z]);
  if (!u && C !== "fixed" || !B)
    return null;
  var Q = lt("div", st({
    ref: V
  }, Fn(it(it({}, r), {}, {
    offset: B.offset,
    position: C,
    rect: B.rect
  }), "menuPortal", {
    "menu-portal": !0
  }), v), f);
  return lt(t2.Provider, {
    value: L
  }, u ? /* @__PURE__ */ uD.createPortal(Q, u) : Q);
}, vG = function(r) {
  var u = r.isDisabled, f = r.isRtl;
  return {
    label: "container",
    direction: f ? "rtl" : void 0,
    pointerEvents: u ? "none" : void 0,
    // cancel mouse events when disabled
    position: "relative"
  };
}, hG = function(r) {
  var u = r.children, f = r.innerProps, c = r.isDisabled, v = r.isRtl;
  return lt("div", st({}, Fn(r, "container", {
    "--is-disabled": c,
    "--is-rtl": v
  }), f), u);
}, mG = function(r, u) {
  var f = r.theme.spacing, c = r.isMulti, v = r.hasValue, h = r.selectProps.controlShouldRenderValue;
  return it({
    alignItems: "center",
    display: c && v && h ? "flex" : "grid",
    flex: 1,
    flexWrap: "wrap",
    WebkitOverflowScrolling: "touch",
    position: "relative",
    overflow: "hidden"
  }, u ? {} : {
    padding: "".concat(f.baseUnit / 2, "px ").concat(f.baseUnit * 2, "px")
  });
}, gG = function(r) {
  var u = r.children, f = r.innerProps, c = r.isMulti, v = r.hasValue;
  return lt("div", st({}, Fn(r, "valueContainer", {
    "value-container": !0,
    "value-container--is-multi": c,
    "value-container--has-value": v
  }), f), u);
}, yG = function() {
  return {
    alignItems: "center",
    alignSelf: "stretch",
    display: "flex",
    flexShrink: 0
  };
}, bG = function(r) {
  var u = r.children, f = r.innerProps;
  return lt("div", st({}, Fn(r, "indicatorsContainer", {
    indicators: !0
  }), f), u);
}, vO, CG = ["size"], SG = ["innerProps", "isRtl", "size"];
function EG() {
  return "You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).";
}
var xG =  false ? 0 : {
  name: "tj5bde-Svg",
  styles: "display:inline-block;fill:currentColor;line-height:1;stroke:currentColor;stroke-width:0;label:Svg;",
  map: "/*# sourceMappingURL=data:application/json;charset=utf-8;base64,  */",
  toString: EG
}, r2 = function(r) {
  var u = r.size, f = $o(r, CG);
  return lt("svg", st({
    height: u,
    width: u,
    viewBox: "0 0 20 20",
    "aria-hidden": "true",
    focusable: "false",
    css: xG
  }, f));
}, dw = function(r) {
  return lt(r2, st({
    size: 20
  }, r), lt("path", {
    d: "M14.348 14.849c-0.469 0.469-1.229 0.469-1.697 0l-2.651-3.030-2.651 3.029c-0.469 0.469-1.229 0.469-1.697 0-0.469-0.469-0.469-1.229 0-1.697l2.758-3.15-2.759-3.152c-0.469-0.469-0.469-1.228 0-1.697s1.228-0.469 1.697 0l2.652 3.031 2.651-3.031c0.469-0.469 1.228-0.469 1.697 0s0.469 1.229 0 1.697l-2.758 3.152 2.758 3.15c0.469 0.469 0.469 1.229 0 1.698z"
  }));
}, a2 = function(r) {
  return lt(r2, st({
    size: 20
  }, r), lt("path", {
    d: "M4.516 7.548c0.436-0.446 1.043-0.481 1.576 0l3.908 3.747 3.908-3.747c0.533-0.481 1.141-0.446 1.574 0 0.436 0.445 0.408 1.197 0 1.615-0.406 0.418-4.695 4.502-4.695 4.502-0.217 0.223-0.502 0.335-0.787 0.335s-0.57-0.112-0.789-0.335c0 0-4.287-4.084-4.695-4.502s-0.436-1.17 0-1.615z"
  }));
}, i2 = function(r, u) {
  var f = r.isFocused, c = r.theme, v = c.spacing.baseUnit, h = c.colors;
  return it({
    label: "indicatorContainer",
    display: "flex",
    transition: "color 150ms"
  }, u ? {} : {
    color: f ? h.neutral60 : h.neutral20,
    padding: v * 2,
    ":hover": {
      color: f ? h.neutral80 : h.neutral40
    }
  });
}, wG = i2, TG = function(r) {
  var u = r.children, f = r.innerProps;
  return lt("div", st({}, Fn(r, "dropdownIndicator", {
    indicator: !0,
    "dropdown-indicator": !0
  }), f), u || lt(a2, null));
}, RG = i2, IG = function(r) {
  var u = r.children, f = r.innerProps;
  return lt("div", st({}, Fn(r, "clearIndicator", {
    indicator: !0,
    "clear-indicator": !0
  }), f), u || lt(dw, null));
}, AG = function(r, u) {
  var f = r.isDisabled, c = r.theme, v = c.spacing.baseUnit, h = c.colors;
  return it({
    label: "indicatorSeparator",
    alignSelf: "stretch",
    width: 1
  }, u ? {} : {
    backgroundColor: f ? h.neutral10 : h.neutral20,
    marginBottom: v * 2,
    marginTop: v * 2
  });
}, OG = function(r) {
  var u = r.innerProps;
  return lt("span", st({}, u, Fn(r, "indicatorSeparator", {
    "indicator-separator": !0
  })));
}, DG = bz(vO || (vO = Tz([`
  0%, 80%, 100% { opacity: 0; }
  40% { opacity: 1; }
`]))), NG = function(r, u) {
  var f = r.isFocused, c = r.size, v = r.theme, h = v.colors, C = v.spacing.baseUnit;
  return it({
    label: "loadingIndicator",
    display: "flex",
    transition: "color 150ms",
    alignSelf: "center",
    fontSize: c,
    lineHeight: 1,
    marginRight: c,
    textAlign: "center",
    verticalAlign: "middle"
  }, u ? {} : {
    color: f ? h.neutral60 : h.neutral20,
    padding: C * 2
  });
}, BE = function(r) {
  var u = r.delay, f = r.offset;
  return lt("span", {
    css: /* @__PURE__ */ lw({
      animation: "".concat(DG, " 1s ease-in-out ").concat(u, "ms infinite;"),
      backgroundColor: "currentColor",
      borderRadius: "1em",
      display: "inline-block",
      marginLeft: f ? "1em" : void 0,
      height: "1em",
      verticalAlign: "top",
      width: "1em"
    },  false ? 0 : ";label:LoadingDot;",  false ? 0 : "/*# sourceMappingURL=data:application/json;charset=utf-8;base64,  */")
  });
}, kG = function(r) {
  var u = r.innerProps, f = r.isRtl, c = r.size, v = c === void 0 ? 4 : c, h = $o(r, SG);
  return lt("div", st({}, Fn(it(it({}, h), {}, {
    innerProps: u,
    isRtl: f,
    size: v
  }), "loadingIndicator", {
    indicator: !0,
    "loading-indicator": !0
  }), u), lt(BE, {
    delay: 0,
    offset: f
  }), lt(BE, {
    delay: 160,
    offset: !0
  }), lt(BE, {
    delay: 320,
    offset: !f
  }));
}, MG = function(r, u) {
  var f = r.isDisabled, c = r.isFocused, v = r.theme, h = v.colors, C = v.borderRadius, b = v.spacing;
  return it({
    label: "control",
    alignItems: "center",
    cursor: "default",
    display: "flex",
    flexWrap: "wrap",
    justifyContent: "space-between",
    minHeight: b.controlHeight,
    outline: "0 !important",
    position: "relative",
    transition: "all 100ms"
  }, u ? {} : {
    backgroundColor: f ? h.neutral5 : h.neutral0,
    borderColor: f ? h.neutral10 : c ? h.primary : h.neutral20,
    borderRadius: C,
    borderStyle: "solid",
    borderWidth: 1,
    boxShadow: c ? "0 0 0 1px ".concat(h.primary) : void 0,
    "&:hover": {
      borderColor: c ? h.primary : h.neutral30
    }
  });
}, LG = function(r) {
  var u = r.children, f = r.isDisabled, c = r.isFocused, v = r.innerRef, h = r.innerProps, C = r.menuIsOpen;
  return lt("div", st({
    ref: v
  }, Fn(r, "control", {
    control: !0,
    "control--is-disabled": f,
    "control--is-focused": c,
    "control--menu-is-open": C
  }), h, {
    "aria-disabled": f || void 0
  }), u);
}, FG = LG, BG = ["data"], VG = function(r, u) {
  var f = r.theme.spacing;
  return u ? {} : {
    paddingBottom: f.baseUnit * 2,
    paddingTop: f.baseUnit * 2
  };
}, PG = function(r) {
  var u = r.children, f = r.cx, c = r.getStyles, v = r.getClassNames, h = r.Heading, C = r.headingProps, b = r.innerProps, E = r.label, w = r.theme, R = r.selectProps;
  return lt("div", st({}, Fn(r, "group", {
    group: !0
  }), b), lt(h, st({}, C, {
    selectProps: R,
    theme: w,
    getStyles: c,
    getClassNames: v,
    cx: f
  }), E), lt("div", null, u));
}, HG = function(r, u) {
  var f = r.theme, c = f.colors, v = f.spacing;
  return it({
    label: "group",
    cursor: "default",
    display: "block"
  }, u ? {} : {
    color: c.neutral40,
    fontSize: "75%",
    fontWeight: 500,
    marginBottom: "0.25em",
    paddingLeft: v.baseUnit * 3,
    paddingRight: v.baseUnit * 3,
    textTransform: "uppercase"
  });
}, zG = function(r) {
  var u = JN(r);
  u.data;
  var f = $o(u, BG);
  return lt("div", st({}, Fn(r, "groupHeading", {
    "group-heading": !0
  }), f));
}, GG = PG, _G = ["innerRef", "isDisabled", "isHidden", "inputClassName"], jG = function(r, u) {
  var f = r.isDisabled, c = r.value, v = r.theme, h = v.spacing, C = v.colors;
  return it(it({
    visibility: f ? "hidden" : "visible",
    // force css to recompute when value change due to @emotion bug.
    // We can remove it whenever the bug is fixed.
    transform: c ? "translateZ(0)" : ""
  }, UG), u ? {} : {
    margin: h.baseUnit / 2,
    paddingBottom: h.baseUnit / 2,
    paddingTop: h.baseUnit / 2,
    color: C.neutral80
  });
}, o2 = {
  gridArea: "1 / 2",
  font: "inherit",
  minWidth: "2px",
  border: 0,
  margin: 0,
  outline: 0,
  padding: 0
}, UG = {
  flex: "1 1 auto",
  display: "inline-grid",
  gridArea: "1 / 1 / 2 / 3",
  gridTemplateColumns: "0 min-content",
  "&:after": it({
    content: 'attr(data-value) " "',
    visibility: "hidden",
    whiteSpace: "pre"
  }, o2)
}, WG = function(r) {
  return it({
    label: "input",
    color: "inherit",
    background: 0,
    opacity: r ? 0 : 1,
    width: "100%"
  }, o2);
}, XG = function(r) {
  var u = r.cx, f = r.value, c = JN(r), v = c.innerRef, h = c.isDisabled, C = c.isHidden, b = c.inputClassName, E = $o(c, _G);
  return lt("div", st({}, Fn(r, "input", {
    "input-container": !0
  }), {
    "data-value": f || ""
  }), lt("input", st({
    className: u({
      input: !0
    }, b),
    ref: v,
    style: WG(C),
    disabled: h
  }, E)));
}, ZG = XG, YG = function(r, u) {
  var f = r.theme, c = f.spacing, v = f.borderRadius, h = f.colors;
  return it({
    label: "multiValue",
    display: "flex",
    minWidth: 0
  }, u ? {} : {
    backgroundColor: h.neutral10,
    borderRadius: v / 2,
    margin: c.baseUnit / 2
  });
}, $G = function(r, u) {
  var f = r.theme, c = f.borderRadius, v = f.colors, h = r.cropWithEllipsis;
  return it({
    overflow: "hidden",
    textOverflow: h || h === void 0 ? "ellipsis" : void 0,
    whiteSpace: "nowrap"
  }, u ? {} : {
    borderRadius: c / 2,
    color: v.neutral80,
    fontSize: "85%",
    padding: 3,
    paddingLeft: 6
  });
}, QG = function(r, u) {
  var f = r.theme, c = f.spacing, v = f.borderRadius, h = f.colors, C = r.isFocused;
  return it({
    alignItems: "center",
    display: "flex"
  }, u ? {} : {
    borderRadius: v / 2,
    backgroundColor: C ? h.dangerLight : void 0,
    paddingLeft: c.baseUnit,
    paddingRight: c.baseUnit,
    ":hover": {
      backgroundColor: h.dangerLight,
      color: h.danger
    }
  });
}, u2 = function(r) {
  var u = r.children, f = r.innerProps;
  return lt("div", f, u);
}, JG = u2, KG = u2;
function qG(a) {
  var r = a.children, u = a.innerProps;
  return lt("div", st({
    role: "button"
  }, u), r || lt(dw, {
    size: 14
  }));
}
var e5 = function(r) {
  var u = r.children, f = r.components, c = r.data, v = r.innerProps, h = r.isDisabled, C = r.removeProps, b = r.selectProps, E = f.Container, w = f.Label, R = f.Remove;
  return lt(E, {
    data: c,
    innerProps: it(it({}, Fn(r, "multiValue", {
      "multi-value": !0,
      "multi-value--is-disabled": h
    })), v),
    selectProps: b
  }, lt(w, {
    data: c,
    innerProps: it({}, Fn(r, "multiValueLabel", {
      "multi-value__label": !0
    })),
    selectProps: b
  }, u), lt(R, {
    data: c,
    innerProps: it(it({}, Fn(r, "multiValueRemove", {
      "multi-value__remove": !0
    })), {}, {
      "aria-label": "Remove ".concat(u || "option")
    }, C),
    selectProps: b
  }));
}, t5 = e5, n5 = function(r, u) {
  var f = r.isDisabled, c = r.isFocused, v = r.isSelected, h = r.theme, C = h.spacing, b = h.colors;
  return it({
    label: "option",
    cursor: "default",
    display: "block",
    fontSize: "inherit",
    width: "100%",
    userSelect: "none",
    WebkitTapHighlightColor: "rgba(0, 0, 0, 0)"
  }, u ? {} : {
    backgroundColor: v ? b.primary : c ? b.primary25 : "transparent",
    color: f ? b.neutral20 : v ? b.neutral0 : "inherit",
    padding: "".concat(C.baseUnit * 2, "px ").concat(C.baseUnit * 3, "px"),
    // provide some affordance on touch devices
    ":active": {
      backgroundColor: f ? void 0 : v ? b.primary : b.primary50
    }
  });
}, r5 = function(r) {
  var u = r.children, f = r.isDisabled, c = r.isFocused, v = r.isSelected, h = r.innerRef, C = r.innerProps;
  return lt("div", st({}, Fn(r, "option", {
    option: !0,
    "option--is-disabled": f,
    "option--is-focused": c,
    "option--is-selected": v
  }), {
    ref: h,
    "aria-disabled": f
  }, C), u);
}, a5 = r5, i5 = function(r, u) {
  var f = r.theme, c = f.spacing, v = f.colors;
  return it({
    label: "placeholder",
    gridArea: "1 / 1 / 2 / 3"
  }, u ? {} : {
    color: v.neutral50,
    marginLeft: c.baseUnit / 2,
    marginRight: c.baseUnit / 2
  });
}, o5 = function(r) {
  var u = r.children, f = r.innerProps;
  return lt("div", st({}, Fn(r, "placeholder", {
    placeholder: !0
  }), f), u);
}, u5 = o5, l5 = function(r, u) {
  var f = r.isDisabled, c = r.theme, v = c.spacing, h = c.colors;
  return it({
    label: "singleValue",
    gridArea: "1 / 1 / 2 / 3",
    maxWidth: "100%",
    overflow: "hidden",
    textOverflow: "ellipsis",
    whiteSpace: "nowrap"
  }, u ? {} : {
    color: f ? h.neutral40 : h.neutral80,
    marginLeft: v.baseUnit / 2,
    marginRight: v.baseUnit / 2
  });
}, s5 = function(r) {
  var u = r.children, f = r.isDisabled, c = r.innerProps;
  return lt("div", st({}, Fn(r, "singleValue", {
    "single-value": !0,
    "single-value--is-disabled": f
  }), c), u);
}, c5 = s5, pw = {
  ClearIndicator: IG,
  Control: FG,
  DropdownIndicator: TG,
  DownChevron: a2,
  CrossIcon: dw,
  Group: GG,
  GroupHeading: zG,
  IndicatorsContainer: bG,
  IndicatorSeparator: OG,
  Input: ZG,
  LoadingIndicator: kG,
  Menu: iG,
  MenuList: uG,
  MenuPortal: pG,
  LoadingMessage: fG,
  NoOptionsMessage: cG,
  MultiValue: t5,
  MultiValueContainer: JG,
  MultiValueLabel: KG,
  MultiValueRemove: qG,
  Option: a5,
  Placeholder: u5,
  SelectContainer: hG,
  SingleValue: c5,
  ValueContainer: gG
}, f5 = function(r) {
  return it(it({}, pw), r.components);
}, hO = Number.isNaN || function(r) {
  return typeof r == "number" && r !== r;
};
function d5(a, r) {
  return !!(a === r || hO(a) && hO(r));
}
function p5(a, r) {
  if (a.length !== r.length)
    return !1;
  for (var u = 0; u < a.length; u++)
    if (!d5(a[u], r[u]))
      return !1;
  return !0;
}
function v5(a, r) {
  r === void 0 && (r = p5);
  var u = null;
  function f() {
    for (var c = [], v = 0; v < arguments.length; v++)
      c[v] = arguments[v];
    if (u && u.lastThis === this && r(c, u.lastArgs))
      return u.lastResult;
    var h = a.apply(this, c);
    return u = {
      lastResult: h,
      lastArgs: c,
      lastThis: this
    }, h;
  }
  return f.clear = function() {
    u = null;
  }, f;
}
function h5() {
  return "You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).";
}
var m5 =  false ? 0 : {
  name: "1f43avz-a11yText-A11yText",
  styles: "label:a11yText;z-index:9999;border:0;clip:rect(1px, 1px, 1px, 1px);height:1px;width:1px;position:absolute;overflow:hidden;padding:0;white-space:nowrap;label:A11yText;",
  map: "/*# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJzb3VyY2VzIjpbIkExMXlUZXh0LnRzeCJdLCJuYW1lcyI6W10sIm1hcHBpbmdzIjoiQUFNSSIsImZpbGUiOiJBMTF5VGV4dC50c3giLCJzb3VyY2VzQ29udGVudCI6WyIvKiogQGpzeCBqc3ggKi9cbmltcG9ydCB7IGpzeCB9IGZyb20gJ0BlbW90aW9uL3JlYWN0JztcblxuLy8gQXNzaXN0aXZlIHRleHQgdG8gZGVzY3JpYmUgdmlzdWFsIGVsZW1lbnRzLiBIaWRkZW4gZm9yIHNpZ2h0ZWQgdXNlcnMuXG5jb25zdCBBMTF5VGV4dCA9IChwcm9wczogSlNYLkludHJpbnNpY0VsZW1lbnRzWydzcGFuJ10pID0+IChcbiAgPHNwYW5cbiAgICBjc3M9e3tcbiAgICAgIGxhYmVsOiAnYTExeVRleHQnLFxuICAgICAgekluZGV4OiA5OTk5LFxuICAgICAgYm9yZGVyOiAwLFxuICAgICAgY2xpcDogJ3JlY3QoMXB4LCAxcHgsIDFweCwgMXB4KScsXG4gICAgICBoZWlnaHQ6IDEsXG4gICAgICB3aWR0aDogMSxcbiAgICAgIHBvc2l0aW9uOiAnYWJzb2x1dGUnLFxuICAgICAgb3ZlcmZsb3c6ICdoaWRkZW4nLFxuICAgICAgcGFkZGluZzogMCxcbiAgICAgIHdoaXRlU3BhY2U6ICdub3dyYXAnLFxuICAgIH19XG4gICAgey4uLnByb3BzfVxuICAvPlxuKTtcblxuZXhwb3J0IGRlZmF1bHQgQTExeVRleHQ7XG4iXX0= */",
  toString: h5
}, g5 = function(r) {
  return lt("span", st({
    css: m5
  }, r));
}, mO = g5, y5 = {
  guidance: function(r) {
    var u = r.isSearchable, f = r.isMulti, c = r.tabSelectsValue, v = r.context, h = r.isInitialFocus;
    switch (v) {
      case "menu":
        return "Use Up and Down to choose options, press Enter to select the currently focused option, press Escape to exit the menu".concat(c ? ", press Tab to select the option and exit the menu" : "", ".");
      case "input":
        return h ? "".concat(r["aria-label"] || "Select", " is focused ").concat(u ? ",type to refine list" : "", ", press Down to open the menu, ").concat(f ? " press left to focus selected values" : "") : "";
      case "value":
        return "Use left and right to toggle between focused values, press Backspace to remove the currently focused value";
      default:
        return "";
    }
  },
  onChange: function(r) {
    var u = r.action, f = r.label, c = f === void 0 ? "" : f, v = r.labels, h = r.isDisabled;
    switch (u) {
      case "deselect-option":
      case "pop-value":
      case "remove-value":
        return "option ".concat(c, ", deselected.");
      case "clear":
        return "All selected options have been cleared.";
      case "initial-input-focus":
        return "option".concat(v.length > 1 ? "s" : "", " ").concat(v.join(","), ", selected.");
      case "select-option":
        return h ? "option ".concat(c, " is disabled. Select another option.") : "option ".concat(c, ", selected.");
      default:
        return "";
    }
  },
  onFocus: function(r) {
    var u = r.context, f = r.focused, c = r.options, v = r.label, h = v === void 0 ? "" : v, C = r.selectValue, b = r.isDisabled, E = r.isSelected, w = r.isAppleDevice, R = function(M, k) {
      return M && M.length ? "".concat(M.indexOf(k) + 1, " of ").concat(M.length) : "";
    };
    if (u === "value" && C)
      return "value ".concat(h, " focused, ").concat(R(C, f), ".");
    if (u === "menu" && w) {
      var A = b ? " disabled" : "", D = "".concat(E ? " selected" : "").concat(A);
      return "".concat(h).concat(D, ", ").concat(R(c, f), ".");
    }
    return "";
  },
  onFilter: function(r) {
    var u = r.inputValue, f = r.resultsMessage;
    return "".concat(f).concat(u ? " for search term " + u : "", ".");
  }
}, b5 = function(r) {
  var u = r.ariaSelection, f = r.focusedOption, c = r.focusedValue, v = r.focusableOptions, h = r.isFocused, C = r.selectValue, b = r.selectProps, E = r.id, w = r.isAppleDevice, R = b.ariaLiveMessages, A = b.getOptionLabel, D = b.inputValue, L = b.isMulti, M = b.isOptionDisabled, k = b.isSearchable, B = b.menuIsOpen, Y = b.options, z = b.screenReaderStatus, Z = b.tabSelectsValue, V = b.isLoading, Q = b["aria-label"], P = b["aria-live"], U = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    return it(it({}, y5), R || {});
  }, [R]), se = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    var We = "";
    if (u && U.onChange) {
      var Ee = u.option, tt = u.options, ue = u.removedValue, we = u.removedValues, W = u.value, ce = function(He) {
        return Array.isArray(He) ? null : He;
      }, ge = ue || Ee || ce(W), Ie = ge ? A(ge) : "", Ae = tt || we || void 0, De = Ae ? Ae.map(A) : [], Oe = it({
        // multiSelected items are usually items that have already been selected
        // or set by the user as a default value so we assume they are not disabled
        isDisabled: ge && M(ge, C),
        label: Ie,
        labels: De
      }, u);
      We = U.onChange(Oe);
    }
    return We;
  }, [u, U, M, C, A]), Se = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    var We = "", Ee = f || c, tt = !!(f && C && C.includes(f));
    if (Ee && U.onFocus) {
      var ue = {
        focused: Ee,
        label: A(Ee),
        isDisabled: M(Ee, C),
        isSelected: tt,
        options: v,
        context: Ee === f ? "menu" : "value",
        selectValue: C,
        isAppleDevice: w
      };
      We = U.onFocus(ue);
    }
    return We;
  }, [f, c, A, M, U, v, C, w]), he = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    var We = "";
    if (B && Y.length && !V && U.onFilter) {
      var Ee = z({
        count: v.length
      });
      We = U.onFilter({
        inputValue: D,
        resultsMessage: Ee
      });
    }
    return We;
  }, [v, D, B, U, Y, z, V]), xe = (u == null ? void 0 : u.action) === "initial-input-focus", ye = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    var We = "";
    if (U.guidance) {
      var Ee = c ? "value" : B ? "menu" : "input";
      We = U.guidance({
        "aria-label": Q,
        context: Ee,
        isDisabled: f && M(f, C),
        isMulti: L,
        isSearchable: k,
        tabSelectsValue: Z,
        isInitialFocus: xe
      });
    }
    return We;
  }, [Q, f, c, L, M, k, B, U, C, Z, xe]), Me = lt(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, lt("span", {
    id: "aria-selection"
  }, se), lt("span", {
    id: "aria-focused"
  }, Se), lt("span", {
    id: "aria-results"
  }, he), lt("span", {
    id: "aria-guidance"
  }, ye));
  return lt(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, lt(mO, {
    id: E
  }, xe && Me), lt(mO, {
    "aria-live": P,
    "aria-atomic": "false",
    "aria-relevant": "additions text",
    role: "log"
  }, h && !xe && Me));
}, C5 = b5, yx = [{
  base: "A",
  letters: "AⒶＡÀÁÂẦẤẪẨÃĀĂẰẮẴẲȦǠÄǞẢÅǺǍȀȂẠẬẶḀĄȺⱯ"
}, {
  base: "AA",
  letters: "Ꜳ"
}, {
  base: "AE",
  letters: "ÆǼǢ"
}, {
  base: "AO",
  letters: "Ꜵ"
}, {
  base: "AU",
  letters: "Ꜷ"
}, {
  base: "AV",
  letters: "ꜸꜺ"
}, {
  base: "AY",
  letters: "Ꜽ"
}, {
  base: "B",
  letters: "BⒷＢḂḄḆɃƂƁ"
}, {
  base: "C",
  letters: "CⒸＣĆĈĊČÇḈƇȻꜾ"
}, {
  base: "D",
  letters: "DⒹＤḊĎḌḐḒḎĐƋƊƉꝹ"
}, {
  base: "DZ",
  letters: "ǱǄ"
}, {
  base: "Dz",
  letters: "ǲǅ"
}, {
  base: "E",
  letters: "EⒺＥÈÉÊỀẾỄỂẼĒḔḖĔĖËẺĚȄȆẸỆȨḜĘḘḚƐƎ"
}, {
  base: "F",
  letters: "FⒻＦḞƑꝻ"
}, {
  base: "G",
  letters: "GⒼＧǴĜḠĞĠǦĢǤƓꞠꝽꝾ"
}, {
  base: "H",
  letters: "HⒽＨĤḢḦȞḤḨḪĦⱧⱵꞍ"
}, {
  base: "I",
  letters: "IⒾＩÌÍÎĨĪĬİÏḮỈǏȈȊỊĮḬƗ"
}, {
  base: "J",
  letters: "JⒿＪĴɈ"
}, {
  base: "K",
  letters: "KⓀＫḰǨḲĶḴƘⱩꝀꝂꝄꞢ"
}, {
  base: "L",
  letters: "LⓁＬĿĹĽḶḸĻḼḺŁȽⱢⱠꝈꝆꞀ"
}, {
  base: "LJ",
  letters: "Ǉ"
}, {
  base: "Lj",
  letters: "ǈ"
}, {
  base: "M",
  letters: "MⓂＭḾṀṂⱮƜ"
}, {
  base: "N",
  letters: "NⓃＮǸŃÑṄŇṆŅṊṈȠƝꞐꞤ"
}, {
  base: "NJ",
  letters: "Ǌ"
}, {
  base: "Nj",
  letters: "ǋ"
}, {
  base: "O",
  letters: "OⓄＯÒÓÔỒỐỖỔÕṌȬṎŌṐṒŎȮȰÖȪỎŐǑȌȎƠỜỚỠỞỢỌỘǪǬØǾƆƟꝊꝌ"
}, {
  base: "OI",
  letters: "Ƣ"
}, {
  base: "OO",
  letters: "Ꝏ"
}, {
  base: "OU",
  letters: "Ȣ"
}, {
  base: "P",
  letters: "PⓅＰṔṖƤⱣꝐꝒꝔ"
}, {
  base: "Q",
  letters: "QⓆＱꝖꝘɊ"
}, {
  base: "R",
  letters: "RⓇＲŔṘŘȐȒṚṜŖṞɌⱤꝚꞦꞂ"
}, {
  base: "S",
  letters: "SⓈＳẞŚṤŜṠŠṦṢṨȘŞⱾꞨꞄ"
}, {
  base: "T",
  letters: "TⓉＴṪŤṬȚŢṰṮŦƬƮȾꞆ"
}, {
  base: "TZ",
  letters: "Ꜩ"
}, {
  base: "U",
  letters: "UⓊＵÙÚÛŨṸŪṺŬÜǛǗǕǙỦŮŰǓȔȖƯỪỨỮỬỰỤṲŲṶṴɄ"
}, {
  base: "V",
  letters: "VⓋＶṼṾƲꝞɅ"
}, {
  base: "VY",
  letters: "Ꝡ"
}, {
  base: "W",
  letters: "WⓌＷẀẂŴẆẄẈⱲ"
}, {
  base: "X",
  letters: "XⓍＸẊẌ"
}, {
  base: "Y",
  letters: "YⓎＹỲÝŶỸȲẎŸỶỴƳɎỾ"
}, {
  base: "Z",
  letters: "ZⓏＺŹẐŻŽẒẔƵȤⱿⱫꝢ"
}, {
  base: "a",
  letters: "aⓐａẚàáâầấẫẩãāăằắẵẳȧǡäǟảåǻǎȁȃạậặḁąⱥɐ"
}, {
  base: "aa",
  letters: "ꜳ"
}, {
  base: "ae",
  letters: "æǽǣ"
}, {
  base: "ao",
  letters: "ꜵ"
}, {
  base: "au",
  letters: "ꜷ"
}, {
  base: "av",
  letters: "ꜹꜻ"
}, {
  base: "ay",
  letters: "ꜽ"
}, {
  base: "b",
  letters: "bⓑｂḃḅḇƀƃɓ"
}, {
  base: "c",
  letters: "cⓒｃćĉċčçḉƈȼꜿↄ"
}, {
  base: "d",
  letters: "dⓓｄḋďḍḑḓḏđƌɖɗꝺ"
}, {
  base: "dz",
  letters: "ǳǆ"
}, {
  base: "e",
  letters: "eⓔｅèéêềếễểẽēḕḗĕėëẻěȅȇẹệȩḝęḙḛɇɛǝ"
}, {
  base: "f",
  letters: "fⓕｆḟƒꝼ"
}, {
  base: "g",
  letters: "gⓖｇǵĝḡğġǧģǥɠꞡᵹꝿ"
}, {
  base: "h",
  letters: "hⓗｈĥḣḧȟḥḩḫẖħⱨⱶɥ"
}, {
  base: "hv",
  letters: "ƕ"
}, {
  base: "i",
  letters: "iⓘｉìíîĩīĭïḯỉǐȉȋịįḭɨı"
}, {
  base: "j",
  letters: "jⓙｊĵǰɉ"
}, {
  base: "k",
  letters: "kⓚｋḱǩḳķḵƙⱪꝁꝃꝅꞣ"
}, {
  base: "l",
  letters: "lⓛｌŀĺľḷḹļḽḻſłƚɫⱡꝉꞁꝇ"
}, {
  base: "lj",
  letters: "ǉ"
}, {
  base: "m",
  letters: "mⓜｍḿṁṃɱɯ"
}, {
  base: "n",
  letters: "nⓝｎǹńñṅňṇņṋṉƞɲŉꞑꞥ"
}, {
  base: "nj",
  letters: "ǌ"
}, {
  base: "o",
  letters: "oⓞｏòóôồốỗổõṍȭṏōṑṓŏȯȱöȫỏőǒȍȏơờớỡởợọộǫǭøǿɔꝋꝍɵ"
}, {
  base: "oi",
  letters: "ƣ"
}, {
  base: "ou",
  letters: "ȣ"
}, {
  base: "oo",
  letters: "ꝏ"
}, {
  base: "p",
  letters: "pⓟｐṕṗƥᵽꝑꝓꝕ"
}, {
  base: "q",
  letters: "qⓠｑɋꝗꝙ"
}, {
  base: "r",
  letters: "rⓡｒŕṙřȑȓṛṝŗṟɍɽꝛꞧꞃ"
}, {
  base: "s",
  letters: "sⓢｓßśṥŝṡšṧṣṩșşȿꞩꞅẛ"
}, {
  base: "t",
  letters: "tⓣｔṫẗťṭțţṱṯŧƭʈⱦꞇ"
}, {
  base: "tz",
  letters: "ꜩ"
}, {
  base: "u",
  letters: "uⓤｕùúûũṹūṻŭüǜǘǖǚủůűǔȕȗưừứữửựụṳųṷṵʉ"
}, {
  base: "v",
  letters: "vⓥｖṽṿʋꝟʌ"
}, {
  base: "vy",
  letters: "ꝡ"
}, {
  base: "w",
  letters: "wⓦｗẁẃŵẇẅẘẉⱳ"
}, {
  base: "x",
  letters: "xⓧｘẋẍ"
}, {
  base: "y",
  letters: "yⓨｙỳýŷỹȳẏÿỷẙỵƴɏỿ"
}, {
  base: "z",
  letters: "zⓩｚźẑżžẓẕƶȥɀⱬꝣ"
}], S5 = new RegExp("[" + yx.map(function(a) {
  return a.letters;
}).join("") + "]", "g"), l2 = {};
for (var VE = 0; VE < yx.length; VE++)
  for (var PE = yx[VE], HE = 0; HE < PE.letters.length; HE++)
    l2[PE.letters[HE]] = PE.base;
var s2 = function(r) {
  return r.replace(S5, function(u) {
    return l2[u];
  });
}, E5 = v5(s2), gO = function(r) {
  return r.replace(/^\s+|\s+$/g, "");
}, x5 = function(r) {
  return "".concat(r.label, " ").concat(r.value);
}, w5 = function(r) {
  return function(u, f) {
    if (u.data.__isNew__)
      return !0;
    var c = it({
      ignoreCase: !0,
      ignoreAccents: !0,
      stringify: x5,
      trim: !0,
      matchFrom: "any"
    }, r), v = c.ignoreCase, h = c.ignoreAccents, C = c.stringify, b = c.trim, E = c.matchFrom, w = b ? gO(f) : f, R = b ? gO(C(u)) : C(u);
    return v && (w = w.toLowerCase(), R = R.toLowerCase()), h && (w = E5(w), R = s2(R)), E === "start" ? R.substr(0, w.length) === w : R.indexOf(w) > -1;
  };
}, T5 = ["innerRef"];
function R5(a) {
  var r = a.innerRef, u = $o(a, T5), f = Jz(u, "onExited", "in", "enter", "exit", "appear");
  return lt("input", st({
    ref: r
  }, f, {
    css: /* @__PURE__ */ lw({
      label: "dummyInput",
      // get rid of any default styles
      background: 0,
      border: 0,
      // important! this hides the flashing cursor
      caretColor: "transparent",
      fontSize: "inherit",
      gridArea: "1 / 1 / 2 / 3",
      outline: 0,
      padding: 0,
      // important! without `width` browsers won't allow focus
      width: 1,
      // remove cursor on desktop
      color: "transparent",
      // remove cursor on mobile whilst maintaining "scroll into view" behaviour
      left: -100,
      opacity: 0,
      position: "relative",
      transform: "scale(.01)"
    },  false ? 0 : ";label:DummyInput;",  false ? 0 : "/*# sourceMappingURL=data:application/json;charset=utf-8;base64,eyJ2ZXJzaW9uIjozLCJzb3VyY2VzIjpbIkR1bW15SW5wdXQudHN4Il0sIm5hbWVzIjpbXSwibWFwcGluZ3MiOiJBQXlCTSIsImZpbGUiOiJEdW1teUlucHV0LnRzeCIsInNvdXJjZXNDb250ZW50IjpbIi8qKiBAanN4IGpzeCAqL1xuaW1wb3J0IHsgUmVmIH0gZnJvbSAncmVhY3QnO1xuaW1wb3J0IHsganN4IH0gZnJvbSAnQGVtb3Rpb24vcmVhY3QnO1xuaW1wb3J0IHsgcmVtb3ZlUHJvcHMgfSBmcm9tICcuLi91dGlscyc7XG5cbmV4cG9ydCBkZWZhdWx0IGZ1bmN0aW9uIER1bW15SW5wdXQoe1xuICBpbm5lclJlZixcbiAgLi4ucHJvcHNcbn06IEpTWC5JbnRyaW5zaWNFbGVtZW50c1snaW5wdXQnXSAmIHtcbiAgcmVhZG9ubHkgaW5uZXJSZWY6IFJlZjxIVE1MSW5wdXRFbGVtZW50Pjtcbn0pIHtcbiAgLy8gUmVtb3ZlIGFuaW1hdGlvbiBwcm9wcyBub3QgbWVhbnQgZm9yIEhUTUwgZWxlbWVudHNcbiAgY29uc3QgZmlsdGVyZWRQcm9wcyA9IHJlbW92ZVByb3BzKFxuICAgIHByb3BzLFxuICAgICdvbkV4aXRlZCcsXG4gICAgJ2luJyxcbiAgICAnZW50ZXInLFxuICAgICdleGl0JyxcbiAgICAnYXBwZWFyJ1xuICApO1xuXG4gIHJldHVybiAoXG4gICAgPGlucHV0XG4gICAgICByZWY9e2lubmVyUmVmfVxuICAgICAgey4uLmZpbHRlcmVkUHJvcHN9XG4gICAgICBjc3M9e3tcbiAgICAgICAgbGFiZWw6ICdkdW1teUlucHV0JyxcbiAgICAgICAgLy8gZ2V0IHJpZCBvZiBhbnkgZGVmYXVsdCBzdHlsZXNcbiAgICAgICAgYmFja2dyb3VuZDogMCxcbiAgICAgICAgYm9yZGVyOiAwLFxuICAgICAgICAvLyBpbXBvcnRhbnQhIHRoaXMgaGlkZXMgdGhlIGZsYXNoaW5nIGN1cnNvclxuICAgICAgICBjYXJldENvbG9yOiAndHJhbnNwYXJlbnQnLFxuICAgICAgICBmb250U2l6ZTogJ2luaGVyaXQnLFxuICAgICAgICBncmlkQXJlYTogJzEgLyAxIC8gMiAvIDMnLFxuICAgICAgICBvdXRsaW5lOiAwLFxuICAgICAgICBwYWRkaW5nOiAwLFxuICAgICAgICAvLyBpbXBvcnRhbnQhIHdpdGhvdXQgYHdpZHRoYCBicm93c2VycyB3b24ndCBhbGxvdyBmb2N1c1xuICAgICAgICB3aWR0aDogMSxcblxuICAgICAgICAvLyByZW1vdmUgY3Vyc29yIG9uIGRlc2t0b3BcbiAgICAgICAgY29sb3I6ICd0cmFuc3BhcmVudCcsXG5cbiAgICAgICAgLy8gcmVtb3ZlIGN1cnNvciBvbiBtb2JpbGUgd2hpbHN0IG1haW50YWluaW5nIFwic2Nyb2xsIGludG8gdmlld1wiIGJlaGF2aW91clxuICAgICAgICBsZWZ0OiAtMTAwLFxuICAgICAgICBvcGFjaXR5OiAwLFxuICAgICAgICBwb3NpdGlvbjogJ3JlbGF0aXZlJyxcbiAgICAgICAgdHJhbnNmb3JtOiAnc2NhbGUoLjAxKScsXG4gICAgICB9fVxuICAgIC8+XG4gICk7XG59XG4iXX0= */")
  }));
}
var I5 = function(r) {
  r.cancelable && r.preventDefault(), r.stopPropagation();
};
function A5(a) {
  var r = a.isEnabled, u = a.onBottomArrive, f = a.onBottomLeave, c = a.onTopArrive, v = a.onTopLeave, h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), C = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(!1), b = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(0), E = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), w = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k, B) {
    if (E.current !== null) {
      var Y = E.current, z = Y.scrollTop, Z = Y.scrollHeight, V = Y.clientHeight, Q = E.current, P = B > 0, U = Z - V - z, se = !1;
      U > B && h.current && (f && f(k), h.current = !1), P && C.current && (v && v(k), C.current = !1), P && B > U ? (u && !h.current && u(k), Q.scrollTop = Z, se = !0, h.current = !0) : !P && -B > z && (c && !C.current && c(k), Q.scrollTop = 0, se = !0, C.current = !0), se && I5(k);
    }
  }, [u, f, c, v]), R = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k) {
    w(k, k.deltaY);
  }, [w]), A = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k) {
    b.current = k.changedTouches[0].clientY;
  }, []), D = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k) {
    var B = b.current - k.changedTouches[0].clientY;
    w(k, B);
  }, [w]), L = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k) {
    if (k) {
      var B = Yz ? {
        passive: !1
      } : !1;
      k.addEventListener("wheel", R, B), k.addEventListener("touchstart", A, B), k.addEventListener("touchmove", D, B);
    }
  }, [D, A, R]), M = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(k) {
    k && (k.removeEventListener("wheel", R, !1), k.removeEventListener("touchstart", A, !1), k.removeEventListener("touchmove", D, !1));
  }, [D, A, R]);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(function() {
    if (r) {
      var k = E.current;
      return L(k), function() {
        M(k);
      };
    }
  }, [r, L, M]), function(k) {
    E.current = k;
  };
}
var yO = ["boxSizing", "height", "overflow", "paddingRight", "position"], bO = {
  boxSizing: "border-box",
  // account for possible declaration `width: 100%;` on body
  overflow: "hidden",
  position: "relative",
  height: "100%"
};
function CO(a) {
  a.preventDefault();
}
function SO(a) {
  a.stopPropagation();
}
function EO() {
  var a = this.scrollTop, r = this.scrollHeight, u = a + this.offsetHeight;
  a === 0 ? this.scrollTop = 1 : u === r && (this.scrollTop = a - 1);
}
function xO() {
  return "ontouchstart" in window || navigator.maxTouchPoints;
}
var wO = !!(typeof window < "u" && window.document && window.document.createElement), Gh = 0, Yd = {
  capture: !1,
  passive: !1
};
function O5(a) {
  var r = a.isEnabled, u = a.accountForScrollbars, f = u === void 0 ? !0 : u, c = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)({}), v = (0,react__WEBPACK_IMPORTED_MODULE_1__.useRef)(null), h = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(b) {
    if (wO) {
      var E = document.body, w = E && E.style;
      if (f && yO.forEach(function(L) {
        var M = w && w[L];
        c.current[L] = M;
      }), f && Gh < 1) {
        var R = parseInt(c.current.paddingRight, 10) || 0, A = document.body ? document.body.clientWidth : 0, D = window.innerWidth - A + R || 0;
        Object.keys(bO).forEach(function(L) {
          var M = bO[L];
          w && (w[L] = M);
        }), w && (w.paddingRight = "".concat(D, "px"));
      }
      E && xO() && (E.addEventListener("touchmove", CO, Yd), b && (b.addEventListener("touchstart", EO, Yd), b.addEventListener("touchmove", SO, Yd))), Gh += 1;
    }
  }, [f]), C = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(b) {
    if (wO) {
      var E = document.body, w = E && E.style;
      Gh = Math.max(Gh - 1, 0), f && Gh < 1 && yO.forEach(function(R) {
        var A = c.current[R];
        w && (w[R] = A);
      }), E && xO() && (E.removeEventListener("touchmove", CO, Yd), b && (b.removeEventListener("touchstart", EO, Yd), b.removeEventListener("touchmove", SO, Yd)));
    }
  }, [f]);
  return (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(function() {
    if (r) {
      var b = v.current;
      return h(b), function() {
        C(b);
      };
    }
  }, [r, h, C]), function(b) {
    v.current = b;
  };
}
function D5() {
  return "You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).";
}
var N5 = function(r) {
  var u = r.target;
  return u.ownerDocument.activeElement && u.ownerDocument.activeElement.blur();
}, k5 =  false ? 0 : {
  name: "bp8cua-ScrollManager",
  styles: "position:fixed;left:0;bottom:0;right:0;top:0;label:ScrollManager;",
  map: "/*# sourceMappingURL=data:application/json;charset=utf-8;base64,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 */",
  toString: D5
};
function M5(a) {
  var r = a.children, u = a.lockEnabled, f = a.captureEnabled, c = f === void 0 ? !0 : f, v = a.onBottomArrive, h = a.onBottomLeave, C = a.onTopArrive, b = a.onTopLeave, E = A5({
    isEnabled: c,
    onBottomArrive: v,
    onBottomLeave: h,
    onTopArrive: C,
    onTopLeave: b
  }), w = O5({
    isEnabled: u
  }), R = function(D) {
    E(D), w(D);
  };
  return lt(react__WEBPACK_IMPORTED_MODULE_1__.Fragment, null, u && lt("div", {
    onClick: N5,
    css: k5
  }), r(R));
}
function L5() {
  return "You have tried to stringify object returned from `css` function. It isn't supposed to be used directly (e.g. as value of the `className` prop), but rather handed to emotion so it can handle it (e.g. as value of `css` prop).";
}
var F5 =  false ? 0 : {
  name: "5kkxb2-requiredInput-RequiredInput",
  styles: "label:requiredInput;opacity:0;pointer-events:none;position:absolute;bottom:0;left:0;right:0;width:100%;label:RequiredInput;",
  map: "",
  toString: L5
}, B5 = function(r) {
  var u = r.name, f = r.onFocus;
  return lt("input", {
    required: !0,
    name: u,
    tabIndex: -1,
    "aria-hidden": "true",
    onFocus: f,
    css: F5,
    value: "",
    onChange: function() {
    }
  });
}, V5 = B5;
function vw(a) {
  var r;
  return typeof window < "u" && window.navigator != null ? a.test(((r = window.navigator.userAgentData) === null || r === void 0 ? void 0 : r.platform) || window.navigator.platform) : !1;
}
function P5() {
  return vw(/^iPhone/i);
}
function c2() {
  return vw(/^Mac/i);
}
function H5() {
  return vw(/^iPad/i) || // iPadOS 13 lies and says it's a Mac, but we can distinguish by detecting touch support.
  c2() && navigator.maxTouchPoints > 1;
}
function z5() {
  return P5() || H5();
}
function G5() {
  return c2() || z5();
}
var _5 = function(r) {
  return r.label;
}, f2 = function(r) {
  return r.label;
}, d2 = function(r) {
  return r.value;
}, j5 = function(r) {
  return !!r.isDisabled;
}, U5 = {
  clearIndicator: RG,
  container: vG,
  control: MG,
  dropdownIndicator: wG,
  group: VG,
  groupHeading: HG,
  indicatorsContainer: yG,
  indicatorSeparator: AG,
  input: jG,
  loadingIndicator: NG,
  loadingMessage: sG,
  menu: nG,
  menuList: oG,
  menuPortal: dG,
  multiValue: YG,
  multiValueLabel: $G,
  multiValueRemove: QG,
  noOptionsMessage: lG,
  option: n5,
  placeholder: i5,
  singleValue: l5,
  valueContainer: mG
}, W5 = {
  primary: "#2684FF",
  primary75: "#4C9AFF",
  primary50: "#B2D4FF",
  primary25: "#DEEBFF",
  danger: "#DE350B",
  dangerLight: "#FFBDAD",
  neutral0: "hsl(0, 0%, 100%)",
  neutral5: "hsl(0, 0%, 95%)",
  neutral10: "hsl(0, 0%, 90%)",
  neutral20: "hsl(0, 0%, 80%)",
  neutral30: "hsl(0, 0%, 70%)",
  neutral40: "hsl(0, 0%, 60%)",
  neutral50: "hsl(0, 0%, 50%)",
  neutral60: "hsl(0, 0%, 40%)",
  neutral70: "hsl(0, 0%, 30%)",
  neutral80: "hsl(0, 0%, 20%)",
  neutral90: "hsl(0, 0%, 10%)"
}, X5 = 4, p2 = 4, Z5 = 38, Y5 = p2 * 2, $5 = {
  baseUnit: p2,
  controlHeight: Z5,
  menuGutter: Y5
}, zE = {
  borderRadius: X5,
  colors: W5,
  spacing: $5
}, Q5 = {
  "aria-live": "polite",
  backspaceRemovesValue: !0,
  blurInputOnSelect: pO(),
  captureMenuScroll: !pO(),
  classNames: {},
  closeMenuOnSelect: !0,
  closeMenuOnScroll: !1,
  components: {},
  controlShouldRenderValue: !0,
  escapeClearsValue: !1,
  filterOption: w5(),
  formatGroupLabel: _5,
  getOptionLabel: f2,
  getOptionValue: d2,
  isDisabled: !1,
  isLoading: !1,
  isMulti: !1,
  isRtl: !1,
  isSearchable: !0,
  isOptionDisabled: j5,
  loadingMessage: function() {
    return "Loading...";
  },
  maxMenuHeight: 300,
  minMenuHeight: 140,
  menuIsOpen: !1,
  menuPlacement: "bottom",
  menuPosition: "absolute",
  menuShouldBlockScroll: !1,
  menuShouldScrollIntoView: !Xz(),
  noOptionsMessage: function() {
    return "No options";
  },
  openMenuOnFocus: !1,
  openMenuOnClick: !0,
  options: [],
  pageSize: 5,
  placeholder: "Select...",
  screenReaderStatus: function(r) {
    var u = r.count;
    return "".concat(u, " result").concat(u !== 1 ? "s" : "", " available");
  },
  styles: {},
  tabIndex: 0,
  tabSelectsValue: !0,
  unstyled: !1
};
function TO(a, r, u, f) {
  var c = m2(a, r, u), v = g2(a, r, u), h = h2(a, r), C = bb(a, r);
  return {
    type: "option",
    data: r,
    isDisabled: c,
    isSelected: v,
    label: h,
    value: C,
    index: f
  };
}
function q0(a, r) {
  return a.options.map(function(u, f) {
    if ("options" in u) {
      var c = u.options.map(function(h, C) {
        return TO(a, h, r, C);
      }).filter(function(h) {
        return IO(a, h);
      });
      return c.length > 0 ? {
        type: "group",
        data: u,
        options: c,
        index: f
      } : void 0;
    }
    var v = TO(a, u, r, f);
    return IO(a, v) ? v : void 0;
  }).filter($z);
}
function v2(a) {
  return a.reduce(function(r, u) {
    return u.type === "group" ? r.push.apply(r, qd(u.options.map(function(f) {
      return f.data;
    }))) : r.push(u.data), r;
  }, []);
}
function RO(a, r) {
  return a.reduce(function(u, f) {
    return f.type === "group" ? u.push.apply(u, qd(f.options.map(function(c) {
      return {
        data: c.data,
        id: "".concat(r, "-").concat(f.index, "-").concat(c.index)
      };
    }))) : u.push({
      data: f.data,
      id: "".concat(r, "-").concat(f.index)
    }), u;
  }, []);
}
function J5(a, r) {
  return v2(q0(a, r));
}
function IO(a, r) {
  var u = a.inputValue, f = u === void 0 ? "" : u, c = r.data, v = r.isSelected, h = r.label, C = r.value;
  return (!b2(a) || !v) && y2(a, {
    label: h,
    value: C,
    data: c
  }, f);
}
function K5(a, r) {
  var u = a.focusedValue, f = a.selectValue, c = f.indexOf(u);
  if (c > -1) {
    var v = r.indexOf(u);
    if (v > -1)
      return u;
    if (c < r.length)
      return r[c];
  }
  return null;
}
function q5(a, r) {
  var u = a.focusedOption;
  return u && r.indexOf(u) > -1 ? u : r[0];
}
var GE = function(r, u) {
  var f, c = (f = r.find(function(v) {
    return v.data === u;
  })) === null || f === void 0 ? void 0 : f.id;
  return c || null;
}, h2 = function(r, u) {
  return r.getOptionLabel(u);
}, bb = function(r, u) {
  return r.getOptionValue(u);
};
function m2(a, r, u) {
  return typeof a.isOptionDisabled == "function" ? a.isOptionDisabled(r, u) : !1;
}
function g2(a, r, u) {
  if (u.indexOf(r) > -1)
    return !0;
  if (typeof a.isOptionSelected == "function")
    return a.isOptionSelected(r, u);
  var f = bb(a, r);
  return u.some(function(c) {
    return bb(a, c) === f;
  });
}
function y2(a, r, u) {
  return a.filterOption ? a.filterOption(r, u) : !0;
}
var b2 = function(r) {
  var u = r.hideSelectedOptions, f = r.isMulti;
  return u === void 0 ? f : u;
}, e_ = 1, hw = /* @__PURE__ */ function(a) {
  lH(u, a);
  var r = fH(u);
  function u(f) {
    var c;
    if (oH(this, u), c = r.call(this, f), c.state = {
      ariaSelection: null,
      focusedOption: null,
      focusedOptionId: null,
      focusableOptionsWithIds: [],
      focusedValue: null,
      inputIsHidden: !1,
      isFocused: !1,
      selectValue: [],
      clearFocusValueOnUpdate: !1,
      prevWasFocused: !1,
      inputIsHiddenAfterUpdate: void 0,
      prevProps: void 0,
      instancePrefix: ""
    }, c.blockOptionHover = !1, c.isComposing = !1, c.commonProps = void 0, c.initialTouchX = 0, c.initialTouchY = 0, c.openAfterFocus = !1, c.scrollToFocusedOptionOnUpdate = !1, c.userIsDragging = void 0, c.isAppleDevice = G5(), c.controlRef = null, c.getControlRef = function(b) {
      c.controlRef = b;
    }, c.focusedOptionRef = null, c.getFocusedOptionRef = function(b) {
      c.focusedOptionRef = b;
    }, c.menuListRef = null, c.getMenuListRef = function(b) {
      c.menuListRef = b;
    }, c.inputRef = null, c.getInputRef = function(b) {
      c.inputRef = b;
    }, c.focus = c.focusInput, c.blur = c.blurInput, c.onChange = function(b, E) {
      var w = c.props, R = w.onChange, A = w.name;
      E.name = A, c.ariaOnChange(b, E), R(b, E);
    }, c.setValue = function(b, E, w) {
      var R = c.props, A = R.closeMenuOnSelect, D = R.isMulti, L = R.inputValue;
      c.onInputChange("", {
        action: "set-value",
        prevInputValue: L
      }), A && (c.setState({
        inputIsHiddenAfterUpdate: !D
      }), c.onMenuClose()), c.setState({
        clearFocusValueOnUpdate: !0
      }), c.onChange(b, {
        action: E,
        option: w
      });
    }, c.selectOption = function(b) {
      var E = c.props, w = E.blurInputOnSelect, R = E.isMulti, A = E.name, D = c.state.selectValue, L = R && c.isOptionSelected(b, D), M = c.isOptionDisabled(b, D);
      if (L) {
        var k = c.getOptionValue(b);
        c.setValue(D.filter(function(B) {
          return c.getOptionValue(B) !== k;
        }), "deselect-option", b);
      } else if (!M)
        R ? c.setValue([].concat(qd(D), [b]), "select-option", b) : c.setValue(b, "select-option");
      else {
        c.ariaOnChange(b, {
          action: "select-option",
          option: b,
          name: A
        });
        return;
      }
      w && c.blurInput();
    }, c.removeValue = function(b) {
      var E = c.props.isMulti, w = c.state.selectValue, R = c.getOptionValue(b), A = w.filter(function(L) {
        return c.getOptionValue(L) !== R;
      }), D = Xh(E, A, A[0] || null);
      c.onChange(D, {
        action: "remove-value",
        removedValue: b
      }), c.focusInput();
    }, c.clearValue = function() {
      var b = c.state.selectValue;
      c.onChange(Xh(c.props.isMulti, [], null), {
        action: "clear",
        removedValues: b
      });
    }, c.popValue = function() {
      var b = c.props.isMulti, E = c.state.selectValue, w = E[E.length - 1], R = E.slice(0, E.length - 1), A = Xh(b, R, R[0] || null);
      c.onChange(A, {
        action: "pop-value",
        removedValue: w
      });
    }, c.getFocusedOptionId = function(b) {
      return GE(c.state.focusableOptionsWithIds, b);
    }, c.getFocusableOptionsWithIds = function() {
      return RO(q0(c.props, c.state.selectValue), c.getElementId("option"));
    }, c.getValue = function() {
      return c.state.selectValue;
    }, c.cx = function() {
      for (var b = arguments.length, E = new Array(b), w = 0; w < b; w++)
        E[w] = arguments[w];
      return Gz.apply(void 0, [c.props.classNamePrefix].concat(E));
    }, c.getOptionLabel = function(b) {
      return h2(c.props, b);
    }, c.getOptionValue = function(b) {
      return bb(c.props, b);
    }, c.getStyles = function(b, E) {
      var w = c.props.unstyled, R = U5[b](E, w);
      R.boxSizing = "border-box";
      var A = c.props.styles[b];
      return A ? A(R, E) : R;
    }, c.getClassNames = function(b, E) {
      var w, R;
      return (w = (R = c.props.classNames)[b]) === null || w === void 0 ? void 0 : w.call(R, E);
    }, c.getElementId = function(b) {
      return "".concat(c.state.instancePrefix, "-").concat(b);
    }, c.getComponents = function() {
      return f5(c.props);
    }, c.buildCategorizedOptions = function() {
      return q0(c.props, c.state.selectValue);
    }, c.getCategorizedOptions = function() {
      return c.props.menuIsOpen ? c.buildCategorizedOptions() : [];
    }, c.buildFocusableOptions = function() {
      return v2(c.buildCategorizedOptions());
    }, c.getFocusableOptions = function() {
      return c.props.menuIsOpen ? c.buildFocusableOptions() : [];
    }, c.ariaOnChange = function(b, E) {
      c.setState({
        ariaSelection: it({
          value: b
        }, E)
      });
    }, c.onMenuMouseDown = function(b) {
      b.button === 0 && (b.stopPropagation(), b.preventDefault(), c.focusInput());
    }, c.onMenuMouseMove = function(b) {
      c.blockOptionHover = !1;
    }, c.onControlMouseDown = function(b) {
      if (!b.defaultPrevented) {
        var E = c.props.openMenuOnClick;
        c.state.isFocused ? c.props.menuIsOpen ? b.target.tagName !== "INPUT" && b.target.tagName !== "TEXTAREA" && c.onMenuClose() : E && c.openMenu("first") : (E && (c.openAfterFocus = !0), c.focusInput()), b.target.tagName !== "INPUT" && b.target.tagName !== "TEXTAREA" && b.preventDefault();
      }
    }, c.onDropdownIndicatorMouseDown = function(b) {
      if (!(b && b.type === "mousedown" && b.button !== 0) && !c.props.isDisabled) {
        var E = c.props, w = E.isMulti, R = E.menuIsOpen;
        c.focusInput(), R ? (c.setState({
          inputIsHiddenAfterUpdate: !w
        }), c.onMenuClose()) : c.openMenu("first"), b.preventDefault();
      }
    }, c.onClearIndicatorMouseDown = function(b) {
      b && b.type === "mousedown" && b.button !== 0 || (c.clearValue(), b.preventDefault(), c.openAfterFocus = !1, b.type === "touchend" ? c.focusInput() : setTimeout(function() {
        return c.focusInput();
      }));
    }, c.onScroll = function(b) {
      typeof c.props.closeMenuOnScroll == "boolean" ? b.target instanceof HTMLElement && Hb(b.target) && c.props.onMenuClose() : typeof c.props.closeMenuOnScroll == "function" && c.props.closeMenuOnScroll(b) && c.props.onMenuClose();
    }, c.onCompositionStart = function() {
      c.isComposing = !0;
    }, c.onCompositionEnd = function() {
      c.isComposing = !1;
    }, c.onTouchStart = function(b) {
      var E = b.touches, w = E && E.item(0);
      w && (c.initialTouchX = w.clientX, c.initialTouchY = w.clientY, c.userIsDragging = !1);
    }, c.onTouchMove = function(b) {
      var E = b.touches, w = E && E.item(0);
      if (w) {
        var R = Math.abs(w.clientX - c.initialTouchX), A = Math.abs(w.clientY - c.initialTouchY), D = 5;
        c.userIsDragging = R > D || A > D;
      }
    }, c.onTouchEnd = function(b) {
      c.userIsDragging || (c.controlRef && !c.controlRef.contains(b.target) && c.menuListRef && !c.menuListRef.contains(b.target) && c.blurInput(), c.initialTouchX = 0, c.initialTouchY = 0);
    }, c.onControlTouchEnd = function(b) {
      c.userIsDragging || c.onControlMouseDown(b);
    }, c.onClearIndicatorTouchEnd = function(b) {
      c.userIsDragging || c.onClearIndicatorMouseDown(b);
    }, c.onDropdownIndicatorTouchEnd = function(b) {
      c.userIsDragging || c.onDropdownIndicatorMouseDown(b);
    }, c.handleInputChange = function(b) {
      var E = c.props.inputValue, w = b.currentTarget.value;
      c.setState({
        inputIsHiddenAfterUpdate: !1
      }), c.onInputChange(w, {
        action: "input-change",
        prevInputValue: E
      }), c.props.menuIsOpen || c.onMenuOpen();
    }, c.onInputFocus = function(b) {
      c.props.onFocus && c.props.onFocus(b), c.setState({
        inputIsHiddenAfterUpdate: !1,
        isFocused: !0
      }), (c.openAfterFocus || c.props.openMenuOnFocus) && c.openMenu("first"), c.openAfterFocus = !1;
    }, c.onInputBlur = function(b) {
      var E = c.props.inputValue;
      if (c.menuListRef && c.menuListRef.contains(document.activeElement)) {
        c.inputRef.focus();
        return;
      }
      c.props.onBlur && c.props.onBlur(b), c.onInputChange("", {
        action: "input-blur",
        prevInputValue: E
      }), c.onMenuClose(), c.setState({
        focusedValue: null,
        isFocused: !1
      });
    }, c.onOptionHover = function(b) {
      if (!(c.blockOptionHover || c.state.focusedOption === b)) {
        var E = c.getFocusableOptions(), w = E.indexOf(b);
        c.setState({
          focusedOption: b,
          focusedOptionId: w > -1 ? c.getFocusedOptionId(b) : null
        });
      }
    }, c.shouldHideSelectedOptions = function() {
      return b2(c.props);
    }, c.onValueInputFocus = function(b) {
      b.preventDefault(), b.stopPropagation(), c.focus();
    }, c.onKeyDown = function(b) {
      var E = c.props, w = E.isMulti, R = E.backspaceRemovesValue, A = E.escapeClearsValue, D = E.inputValue, L = E.isClearable, M = E.isDisabled, k = E.menuIsOpen, B = E.onKeyDown, Y = E.tabSelectsValue, z = E.openMenuOnFocus, Z = c.state, V = Z.focusedOption, Q = Z.focusedValue, P = Z.selectValue;
      if (!M && !(typeof B == "function" && (B(b), b.defaultPrevented))) {
        switch (c.blockOptionHover = !0, b.key) {
          case "ArrowLeft":
            if (!w || D)
              return;
            c.focusValue("previous");
            break;
          case "ArrowRight":
            if (!w || D)
              return;
            c.focusValue("next");
            break;
          case "Delete":
          case "Backspace":
            if (D)
              return;
            if (Q)
              c.removeValue(Q);
            else {
              if (!R)
                return;
              w ? c.popValue() : L && c.clearValue();
            }
            break;
          case "Tab":
            if (c.isComposing || b.shiftKey || !k || !Y || !V || // don't capture the event if the menu opens on focus and the focused
            // option is already selected; it breaks the flow of navigation
            z && c.isOptionSelected(V, P))
              return;
            c.selectOption(V);
            break;
          case "Enter":
            if (b.keyCode === 229)
              break;
            if (k) {
              if (!V || c.isComposing)
                return;
              c.selectOption(V);
              break;
            }
            return;
          case "Escape":
            k ? (c.setState({
              inputIsHiddenAfterUpdate: !1
            }), c.onInputChange("", {
              action: "menu-close",
              prevInputValue: D
            }), c.onMenuClose()) : L && A && c.clearValue();
            break;
          case " ":
            if (D)
              return;
            if (!k) {
              c.openMenu("first");
              break;
            }
            if (!V)
              return;
            c.selectOption(V);
            break;
          case "ArrowUp":
            k ? c.focusOption("up") : c.openMenu("last");
            break;
          case "ArrowDown":
            k ? c.focusOption("down") : c.openMenu("first");
            break;
          case "PageUp":
            if (!k)
              return;
            c.focusOption("pageup");
            break;
          case "PageDown":
            if (!k)
              return;
            c.focusOption("pagedown");
            break;
          case "Home":
            if (!k)
              return;
            c.focusOption("first");
            break;
          case "End":
            if (!k)
              return;
            c.focusOption("last");
            break;
          default:
            return;
        }
        b.preventDefault();
      }
    }, c.state.instancePrefix = "react-select-" + (c.props.instanceId || ++e_), c.state.selectValue = gb(f.value), f.menuIsOpen && c.state.selectValue.length) {
      var v = c.getFocusableOptionsWithIds(), h = c.buildFocusableOptions(), C = h.indexOf(c.state.selectValue[0]);
      c.state.focusableOptionsWithIds = v, c.state.focusedOption = h[C], c.state.focusedOptionId = GE(v, h[C]);
    }
    return c;
  }
  return uH(u, [{
    key: "componentDidMount",
    value: function() {
      this.startListeningComposition(), this.startListeningToTouch(), this.props.closeMenuOnScroll && document && document.addEventListener && document.addEventListener("scroll", this.onScroll, !0), this.props.autoFocus && this.focusInput(), this.props.menuIsOpen && this.state.focusedOption && this.menuListRef && this.focusedOptionRef && dO(this.menuListRef, this.focusedOptionRef);
    }
  }, {
    key: "componentDidUpdate",
    value: function(c) {
      var v = this.props, h = v.isDisabled, C = v.menuIsOpen, b = this.state.isFocused;
      // ensure focus is restored correctly when the control becomes enabled
      (b && !h && c.isDisabled || // ensure focus is on the Input when the menu opens
      b && C && !c.menuIsOpen) && this.focusInput(), b && h && !c.isDisabled ? this.setState({
        isFocused: !1
      }, this.onMenuClose) : !b && !h && c.isDisabled && this.inputRef === document.activeElement && this.setState({
        isFocused: !0
      }), this.menuListRef && this.focusedOptionRef && this.scrollToFocusedOptionOnUpdate && (dO(this.menuListRef, this.focusedOptionRef), this.scrollToFocusedOptionOnUpdate = !1);
    }
  }, {
    key: "componentWillUnmount",
    value: function() {
      this.stopListeningComposition(), this.stopListeningToTouch(), document.removeEventListener("scroll", this.onScroll, !0);
    }
    // ==============================
    // Consumer Handlers
    // ==============================
  }, {
    key: "onMenuOpen",
    value: function() {
      this.props.onMenuOpen();
    }
  }, {
    key: "onMenuClose",
    value: function() {
      this.onInputChange("", {
        action: "menu-close",
        prevInputValue: this.props.inputValue
      }), this.props.onMenuClose();
    }
  }, {
    key: "onInputChange",
    value: function(c, v) {
      this.props.onInputChange(c, v);
    }
    // ==============================
    // Methods
    // ==============================
  }, {
    key: "focusInput",
    value: function() {
      this.inputRef && this.inputRef.focus();
    }
  }, {
    key: "blurInput",
    value: function() {
      this.inputRef && this.inputRef.blur();
    }
    // aliased for consumers
  }, {
    key: "openMenu",
    value: function(c) {
      var v = this, h = this.state, C = h.selectValue, b = h.isFocused, E = this.buildFocusableOptions(), w = c === "first" ? 0 : E.length - 1;
      if (!this.props.isMulti) {
        var R = E.indexOf(C[0]);
        R > -1 && (w = R);
      }
      this.scrollToFocusedOptionOnUpdate = !(b && this.menuListRef), this.setState({
        inputIsHiddenAfterUpdate: !1,
        focusedValue: null,
        focusedOption: E[w],
        focusedOptionId: this.getFocusedOptionId(E[w])
      }, function() {
        return v.onMenuOpen();
      });
    }
  }, {
    key: "focusValue",
    value: function(c) {
      var v = this.state, h = v.selectValue, C = v.focusedValue;
      if (this.props.isMulti) {
        this.setState({
          focusedOption: null
        });
        var b = h.indexOf(C);
        C || (b = -1);
        var E = h.length - 1, w = -1;
        if (h.length) {
          switch (c) {
            case "previous":
              b === 0 ? w = 0 : b === -1 ? w = E : w = b - 1;
              break;
            case "next":
              b > -1 && b < E && (w = b + 1);
              break;
          }
          this.setState({
            inputIsHidden: w !== -1,
            focusedValue: h[w]
          });
        }
      }
    }
  }, {
    key: "focusOption",
    value: function() {
      var c = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : "first", v = this.props.pageSize, h = this.state.focusedOption, C = this.getFocusableOptions();
      if (C.length) {
        var b = 0, E = C.indexOf(h);
        h || (E = -1), c === "up" ? b = E > 0 ? E - 1 : C.length - 1 : c === "down" ? b = (E + 1) % C.length : c === "pageup" ? (b = E - v, b < 0 && (b = 0)) : c === "pagedown" ? (b = E + v, b > C.length - 1 && (b = C.length - 1)) : c === "last" && (b = C.length - 1), this.scrollToFocusedOptionOnUpdate = !0, this.setState({
          focusedOption: C[b],
          focusedValue: null,
          focusedOptionId: this.getFocusedOptionId(C[b])
        });
      }
    }
  }, {
    key: "getTheme",
    value: (
      // ==============================
      // Getters
      // ==============================
      function() {
        return this.props.theme ? typeof this.props.theme == "function" ? this.props.theme(zE) : it(it({}, zE), this.props.theme) : zE;
      }
    )
  }, {
    key: "getCommonProps",
    value: function() {
      var c = this.clearValue, v = this.cx, h = this.getStyles, C = this.getClassNames, b = this.getValue, E = this.selectOption, w = this.setValue, R = this.props, A = R.isMulti, D = R.isRtl, L = R.options, M = this.hasValue();
      return {
        clearValue: c,
        cx: v,
        getStyles: h,
        getClassNames: C,
        getValue: b,
        hasValue: M,
        isMulti: A,
        isRtl: D,
        options: L,
        selectOption: E,
        selectProps: R,
        setValue: w,
        theme: this.getTheme()
      };
    }
  }, {
    key: "hasValue",
    value: function() {
      var c = this.state.selectValue;
      return c.length > 0;
    }
  }, {
    key: "hasOptions",
    value: function() {
      return !!this.getFocusableOptions().length;
    }
  }, {
    key: "isClearable",
    value: function() {
      var c = this.props, v = c.isClearable, h = c.isMulti;
      return v === void 0 ? h : v;
    }
  }, {
    key: "isOptionDisabled",
    value: function(c, v) {
      return m2(this.props, c, v);
    }
  }, {
    key: "isOptionSelected",
    value: function(c, v) {
      return g2(this.props, c, v);
    }
  }, {
    key: "filterOption",
    value: function(c, v) {
      return y2(this.props, c, v);
    }
  }, {
    key: "formatOptionLabel",
    value: function(c, v) {
      if (typeof this.props.formatOptionLabel == "function") {
        var h = this.props.inputValue, C = this.state.selectValue;
        return this.props.formatOptionLabel(c, {
          context: v,
          inputValue: h,
          selectValue: C
        });
      } else
        return this.getOptionLabel(c);
    }
  }, {
    key: "formatGroupLabel",
    value: function(c) {
      return this.props.formatGroupLabel(c);
    }
    // ==============================
    // Mouse Handlers
    // ==============================
  }, {
    key: "startListeningComposition",
    value: (
      // ==============================
      // Composition Handlers
      // ==============================
      function() {
        document && document.addEventListener && (document.addEventListener("compositionstart", this.onCompositionStart, !1), document.addEventListener("compositionend", this.onCompositionEnd, !1));
      }
    )
  }, {
    key: "stopListeningComposition",
    value: function() {
      document && document.removeEventListener && (document.removeEventListener("compositionstart", this.onCompositionStart), document.removeEventListener("compositionend", this.onCompositionEnd));
    }
  }, {
    key: "startListeningToTouch",
    value: (
      // ==============================
      // Touch Handlers
      // ==============================
      function() {
        document && document.addEventListener && (document.addEventListener("touchstart", this.onTouchStart, !1), document.addEventListener("touchmove", this.onTouchMove, !1), document.addEventListener("touchend", this.onTouchEnd, !1));
      }
    )
  }, {
    key: "stopListeningToTouch",
    value: function() {
      document && document.removeEventListener && (document.removeEventListener("touchstart", this.onTouchStart), document.removeEventListener("touchmove", this.onTouchMove), document.removeEventListener("touchend", this.onTouchEnd));
    }
  }, {
    key: "renderInput",
    value: (
      // ==============================
      // Renderers
      // ==============================
      function() {
        var c = this.props, v = c.isDisabled, h = c.isSearchable, C = c.inputId, b = c.inputValue, E = c.tabIndex, w = c.form, R = c.menuIsOpen, A = c.required, D = this.getComponents(), L = D.Input, M = this.state, k = M.inputIsHidden, B = M.ariaSelection, Y = this.commonProps, z = C || this.getElementId("input"), Z = it(it(it({
          "aria-autocomplete": "list",
          "aria-expanded": R,
          "aria-haspopup": !0,
          "aria-errormessage": this.props["aria-errormessage"],
          "aria-invalid": this.props["aria-invalid"],
          "aria-label": this.props["aria-label"],
          "aria-labelledby": this.props["aria-labelledby"],
          "aria-required": A,
          role: "combobox",
          "aria-activedescendant": this.isAppleDevice ? void 0 : this.state.focusedOptionId || ""
        }, R && {
          "aria-controls": this.getElementId("listbox")
        }), !h && {
          "aria-readonly": !0
        }), this.hasValue() ? (B == null ? void 0 : B.action) === "initial-input-focus" && {
          "aria-describedby": this.getElementId("live-region")
        } : {
          "aria-describedby": this.getElementId("placeholder")
        });
        return h ? /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(L, st({}, Y, {
          autoCapitalize: "none",
          autoComplete: "off",
          autoCorrect: "off",
          id: z,
          innerRef: this.getInputRef,
          isDisabled: v,
          isHidden: k,
          onBlur: this.onInputBlur,
          onChange: this.handleInputChange,
          onFocus: this.onInputFocus,
          spellCheck: "false",
          tabIndex: E,
          form: w,
          type: "text",
          value: b
        }, Z)) : /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(R5, st({
          id: z,
          innerRef: this.getInputRef,
          onBlur: this.onInputBlur,
          onChange: mb,
          onFocus: this.onInputFocus,
          disabled: v,
          tabIndex: E,
          inputMode: "none",
          form: w,
          value: ""
        }, Z));
      }
    )
  }, {
    key: "renderPlaceholderOrValue",
    value: function() {
      var c = this, v = this.getComponents(), h = v.MultiValue, C = v.MultiValueContainer, b = v.MultiValueLabel, E = v.MultiValueRemove, w = v.SingleValue, R = v.Placeholder, A = this.commonProps, D = this.props, L = D.controlShouldRenderValue, M = D.isDisabled, k = D.isMulti, B = D.inputValue, Y = D.placeholder, z = this.state, Z = z.selectValue, V = z.focusedValue, Q = z.isFocused;
      if (!this.hasValue() || !L)
        return B ? null : /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(R, st({}, A, {
          key: "placeholder",
          isDisabled: M,
          isFocused: Q,
          innerProps: {
            id: this.getElementId("placeholder")
          }
        }), Y);
      if (k)
        return Z.map(function(U, se) {
          var Se = U === V, he = "".concat(c.getOptionLabel(U), "-").concat(c.getOptionValue(U));
          return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(h, st({}, A, {
            components: {
              Container: C,
              Label: b,
              Remove: E
            },
            isFocused: Se,
            isDisabled: M,
            key: he,
            index: se,
            removeProps: {
              onClick: function() {
                return c.removeValue(U);
              },
              onTouchEnd: function() {
                return c.removeValue(U);
              },
              onMouseDown: function(ye) {
                ye.preventDefault();
              }
            },
            data: U
          }), c.formatOptionLabel(U, "value"));
        });
      if (B)
        return null;
      var P = Z[0];
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(w, st({}, A, {
        data: P,
        isDisabled: M
      }), this.formatOptionLabel(P, "value"));
    }
  }, {
    key: "renderClearIndicator",
    value: function() {
      var c = this.getComponents(), v = c.ClearIndicator, h = this.commonProps, C = this.props, b = C.isDisabled, E = C.isLoading, w = this.state.isFocused;
      if (!this.isClearable() || !v || b || !this.hasValue() || E)
        return null;
      var R = {
        onMouseDown: this.onClearIndicatorMouseDown,
        onTouchEnd: this.onClearIndicatorTouchEnd,
        "aria-hidden": "true"
      };
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(v, st({}, h, {
        innerProps: R,
        isFocused: w
      }));
    }
  }, {
    key: "renderLoadingIndicator",
    value: function() {
      var c = this.getComponents(), v = c.LoadingIndicator, h = this.commonProps, C = this.props, b = C.isDisabled, E = C.isLoading, w = this.state.isFocused;
      if (!v || !E)
        return null;
      var R = {
        "aria-hidden": "true"
      };
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(v, st({}, h, {
        innerProps: R,
        isDisabled: b,
        isFocused: w
      }));
    }
  }, {
    key: "renderIndicatorSeparator",
    value: function() {
      var c = this.getComponents(), v = c.DropdownIndicator, h = c.IndicatorSeparator;
      if (!v || !h)
        return null;
      var C = this.commonProps, b = this.props.isDisabled, E = this.state.isFocused;
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(h, st({}, C, {
        isDisabled: b,
        isFocused: E
      }));
    }
  }, {
    key: "renderDropdownIndicator",
    value: function() {
      var c = this.getComponents(), v = c.DropdownIndicator;
      if (!v)
        return null;
      var h = this.commonProps, C = this.props.isDisabled, b = this.state.isFocused, E = {
        onMouseDown: this.onDropdownIndicatorMouseDown,
        onTouchEnd: this.onDropdownIndicatorTouchEnd,
        "aria-hidden": "true"
      };
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(v, st({}, h, {
        innerProps: E,
        isDisabled: C,
        isFocused: b
      }));
    }
  }, {
    key: "renderMenu",
    value: function() {
      var c = this, v = this.getComponents(), h = v.Group, C = v.GroupHeading, b = v.Menu, E = v.MenuList, w = v.MenuPortal, R = v.LoadingMessage, A = v.NoOptionsMessage, D = v.Option, L = this.commonProps, M = this.state.focusedOption, k = this.props, B = k.captureMenuScroll, Y = k.inputValue, z = k.isLoading, Z = k.loadingMessage, V = k.minMenuHeight, Q = k.maxMenuHeight, P = k.menuIsOpen, U = k.menuPlacement, se = k.menuPosition, Se = k.menuPortalTarget, he = k.menuShouldBlockScroll, xe = k.menuShouldScrollIntoView, ye = k.noOptionsMessage, Me = k.onMenuScrollToTop, We = k.onMenuScrollToBottom;
      if (!P)
        return null;
      var Ee = function(Ie, Ae) {
        var De = Ie.type, Oe = Ie.data, Ue = Ie.isDisabled, He = Ie.isSelected, nt = Ie.label, vt = Ie.value, At = M === Oe, Ce = Ue ? void 0 : function() {
          return c.onOptionHover(Oe);
        }, Ot = Ue ? void 0 : function() {
          return c.selectOption(Oe);
        }, _t = "".concat(c.getElementId("option"), "-").concat(Ae), Et = {
          id: _t,
          onClick: Ot,
          onMouseMove: Ce,
          onMouseOver: Ce,
          tabIndex: -1,
          role: "option",
          "aria-selected": c.isAppleDevice ? void 0 : He
          // is not supported on Apple devices
        };
        return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(D, st({}, L, {
          innerProps: Et,
          data: Oe,
          isDisabled: Ue,
          isSelected: He,
          key: _t,
          label: nt,
          type: De,
          value: vt,
          isFocused: At,
          innerRef: At ? c.getFocusedOptionRef : void 0
        }), c.formatOptionLabel(Ie.data, "menu"));
      }, tt;
      if (this.hasOptions())
        tt = this.getCategorizedOptions().map(function(ge) {
          if (ge.type === "group") {
            var Ie = ge.data, Ae = ge.options, De = ge.index, Oe = "".concat(c.getElementId("group"), "-").concat(De), Ue = "".concat(Oe, "-heading");
            return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(h, st({}, L, {
              key: Oe,
              data: Ie,
              options: Ae,
              Heading: C,
              headingProps: {
                id: Ue,
                data: ge.data
              },
              label: c.formatGroupLabel(ge.data)
            }), ge.options.map(function(He) {
              return Ee(He, "".concat(De, "-").concat(He.index));
            }));
          } else if (ge.type === "option")
            return Ee(ge, "".concat(ge.index));
        });
      else if (z) {
        var ue = Z({
          inputValue: Y
        });
        if (ue === null)
          return null;
        tt = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(R, L, ue);
      } else {
        var we = ye({
          inputValue: Y
        });
        if (we === null)
          return null;
        tt = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(A, L, we);
      }
      var W = {
        minMenuHeight: V,
        maxMenuHeight: Q,
        menuPlacement: U,
        menuPosition: se,
        menuShouldScrollIntoView: xe
      }, ce = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(rG, st({}, L, W), function(ge) {
        var Ie = ge.ref, Ae = ge.placerProps, De = Ae.placement, Oe = Ae.maxHeight;
        return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(b, st({}, L, W, {
          innerRef: Ie,
          innerProps: {
            onMouseDown: c.onMenuMouseDown,
            onMouseMove: c.onMenuMouseMove
          },
          isLoading: z,
          placement: De
        }), /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(M5, {
          captureEnabled: B,
          onTopArrive: Me,
          onBottomArrive: We,
          lockEnabled: he
        }, function(Ue) {
          return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(E, st({}, L, {
            innerRef: function(nt) {
              c.getMenuListRef(nt), Ue(nt);
            },
            innerProps: {
              role: "listbox",
              "aria-multiselectable": L.isMulti,
              id: c.getElementId("listbox")
            },
            isLoading: z,
            maxHeight: Oe,
            focusedOption: M
          }), tt);
        }));
      });
      return Se || se === "fixed" ? /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(w, st({}, L, {
        appendTo: Se,
        controlElement: this.controlRef,
        menuPlacement: U,
        menuPosition: se
      }), ce) : ce;
    }
  }, {
    key: "renderFormField",
    value: function() {
      var c = this, v = this.props, h = v.delimiter, C = v.isDisabled, b = v.isMulti, E = v.name, w = v.required, R = this.state.selectValue;
      if (w && !this.hasValue() && !C)
        return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(V5, {
          name: E,
          onFocus: this.onValueInputFocus
        });
      if (!(!E || C))
        if (b)
          if (h) {
            var A = R.map(function(M) {
              return c.getOptionValue(M);
            }).join(h);
            return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement("input", {
              name: E,
              type: "hidden",
              value: A
            });
          } else {
            var D = R.length > 0 ? R.map(function(M, k) {
              return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement("input", {
                key: "i-".concat(k),
                name: E,
                type: "hidden",
                value: c.getOptionValue(M)
              });
            }) : /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement("input", {
              name: E,
              type: "hidden",
              value: ""
            });
            return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement("div", null, D);
          }
        else {
          var L = R[0] ? this.getOptionValue(R[0]) : "";
          return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement("input", {
            name: E,
            type: "hidden",
            value: L
          });
        }
    }
  }, {
    key: "renderLiveRegion",
    value: function() {
      var c = this.commonProps, v = this.state, h = v.ariaSelection, C = v.focusedOption, b = v.focusedValue, E = v.isFocused, w = v.selectValue, R = this.getFocusableOptions();
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(C5, st({}, c, {
        id: this.getElementId("live-region"),
        ariaSelection: h,
        focusedOption: C,
        focusedValue: b,
        isFocused: E,
        selectValue: w,
        focusableOptions: R,
        isAppleDevice: this.isAppleDevice
      }));
    }
  }, {
    key: "render",
    value: function() {
      var c = this.getComponents(), v = c.Control, h = c.IndicatorsContainer, C = c.SelectContainer, b = c.ValueContainer, E = this.props, w = E.className, R = E.id, A = E.isDisabled, D = E.menuIsOpen, L = this.state.isFocused, M = this.commonProps = this.getCommonProps();
      return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(C, st({}, M, {
        className: w,
        innerProps: {
          id: R,
          onKeyDown: this.onKeyDown
        },
        isDisabled: A,
        isFocused: L
      }), this.renderLiveRegion(), /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(v, st({}, M, {
        innerRef: this.getControlRef,
        innerProps: {
          onMouseDown: this.onControlMouseDown,
          onTouchEnd: this.onControlTouchEnd
        },
        isDisabled: A,
        isFocused: L,
        menuIsOpen: D
      }), /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(b, st({}, M, {
        isDisabled: A
      }), this.renderPlaceholderOrValue(), this.renderInput()), /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(h, st({}, M, {
        isDisabled: A
      }), this.renderClearIndicator(), this.renderLoadingIndicator(), this.renderIndicatorSeparator(), this.renderDropdownIndicator())), this.renderMenu(), this.renderFormField());
    }
  }], [{
    key: "getDerivedStateFromProps",
    value: function(c, v) {
      var h = v.prevProps, C = v.clearFocusValueOnUpdate, b = v.inputIsHiddenAfterUpdate, E = v.ariaSelection, w = v.isFocused, R = v.prevWasFocused, A = v.instancePrefix, D = c.options, L = c.value, M = c.menuIsOpen, k = c.inputValue, B = c.isMulti, Y = gb(L), z = {};
      if (h && (L !== h.value || D !== h.options || M !== h.menuIsOpen || k !== h.inputValue)) {
        var Z = M ? J5(c, Y) : [], V = M ? RO(q0(c, Y), "".concat(A, "-option")) : [], Q = C ? K5(v, Y) : null, P = q5(v, Z), U = GE(V, P);
        z = {
          selectValue: Y,
          focusedOption: P,
          focusedOptionId: U,
          focusableOptionsWithIds: V,
          focusedValue: Q,
          clearFocusValueOnUpdate: !1
        };
      }
      var se = b != null && c !== h ? {
        inputIsHidden: b,
        inputIsHiddenAfterUpdate: void 0
      } : {}, Se = E, he = w && R;
      return w && !he && (Se = {
        value: Xh(B, Y, Y[0] || null),
        options: Y,
        action: "initial-input-focus"
      }, he = !R), (E == null ? void 0 : E.action) === "initial-input-focus" && (Se = null), it(it(it({}, z), se), {}, {
        prevProps: c,
        ariaSelection: Se,
        prevWasFocused: he
      });
    }
  }]), u;
}(react__WEBPACK_IMPORTED_MODULE_1__.Component);
hw.defaultProps = Q5;
var t_ = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.forwardRef)(function(a, r) {
  var u = AN(a);
  return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(hw, st({
    ref: r
  }, u));
}), C2 = t_;
const n_ = "var(--bs-secondary-color)", _E = "var(--bs-gray-300)", jE = "var(--bs-gray-700)", r_ = "var(--bs-gray-900)", a_ = "var(--bs-red)", mw = (a) => ({
  control: (r) => ({
    ...r,
    borderColor: a ? jE : _E,
    backgroundColor: a ? r_ : "white"
  }),
  menuPortal: (r) => ({ ...r, zIndex: 9999 }),
  menu: (r) => ({
    ...r,
    width: "max-content",
    minWidth: "100%",
    backgroundColor: a ? "black" : "white"
  }),
  option: (r, u) => ({
    ...r,
    color: a ? "white" : "black",
    backgroundColor: u.isFocused ? a ? jE : _E : a ? "black" : "white"
  }),
  singleValue: (r) => ({ ...r, color: a ? "white" : "black" }),
  input: (r) => ({ ...r, color: a ? "white" : "black" }),
  multiValue: (r) => ({
    ...r,
    backgroundColor: a ? jE : _E
  }),
  multiValueLabel: (r) => ({
    ...r,
    color: a ? "white" : "black"
  }),
  multiValueRemove: (r) => ({
    ...r,
    ":hover": {
      backgroundColor: a_,
      color: "white"
    }
  }),
  placeholder: (r) => ({
    ...r,
    color: n_
  })
});
function _h({
  isClearable: a = !1,
  options: r,
  titles: u,
  value: f,
  placeholder: c,
  onChange: v,
  darkMode: h
}) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    C2,
    {
      isClearable: a,
      components: { Option: (b) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        "div",
        {
          "data-toggle": "tooltip",
          "data-placement": "top",
          title: u == null ? void 0 : u.get(b.label),
          children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(pw.Option, { ...b })
        }
      ) },
      menuPortalTarget: document.body,
      styles: mw(h),
      options: r.map((b) => ({
        value: b,
        label: b
      })),
      value: f ? {
        value: f,
        label: f
      } : null,
      onChange: (b) => v(b ? {
        target: {
          value: b.value
        }
      } : {
        target: {
          value: ""
        }
      }),
      placeholder: c || "Select value..."
    }
  );
}
function S2({
  options: a,
  titles: r,
  value: u,
  placeholder: f,
  onChange: c,
  darkMode: v
}) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    C2,
    {
      isMulti: !0,
      closeMenuOnSelect: !1,
      components: { Option: (C) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        "div",
        {
          "data-toggle": "tooltip",
          "data-placement": "top",
          title: r == null ? void 0 : r.get(C.label),
          children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(pw.Option, { ...C })
        }
      ) },
      menuPortalTarget: document.body,
      styles: mw(v),
      options: a.map((C) => ({
        value: C,
        label: C
      })),
      value: u.map((C) => ({
        value: C,
        label: C
      })),
      delimiter: ",",
      onChange: (C) => c({
        target: {
          value: C.map((b) => b.value).join(",")
        }
      }),
      placeholder: f || "Select values..."
    }
  );
}
var i_ = ["allowCreateWhileLoading", "createOptionPosition", "formatCreateLabel", "isValidNewOption", "getNewOptionData", "onCreateOption", "options", "onChange"], AO = function() {
  var r = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : "", u = arguments.length > 1 ? arguments[1] : void 0, f = arguments.length > 2 ? arguments[2] : void 0, c = String(r).toLowerCase(), v = String(f.getOptionValue(u)).toLowerCase(), h = String(f.getOptionLabel(u)).toLowerCase();
  return v === c || h === c;
}, UE = {
  formatCreateLabel: function(r) {
    return 'Create "'.concat(r, '"');
  },
  isValidNewOption: function(r, u, f, c) {
    return !(!r || u.some(function(v) {
      return AO(r, v, c);
    }) || f.some(function(v) {
      return AO(r, v, c);
    }));
  },
  getNewOptionData: function(r, u) {
    return {
      label: u,
      value: r,
      __isNew__: !0
    };
  }
};
function o_(a) {
  var r = a.allowCreateWhileLoading, u = r === void 0 ? !1 : r, f = a.createOptionPosition, c = f === void 0 ? "last" : f, v = a.formatCreateLabel, h = v === void 0 ? UE.formatCreateLabel : v, C = a.isValidNewOption, b = C === void 0 ? UE.isValidNewOption : C, E = a.getNewOptionData, w = E === void 0 ? UE.getNewOptionData : E, R = a.onCreateOption, A = a.options, D = A === void 0 ? [] : A, L = a.onChange, M = $o(a, i_), k = M.getOptionValue, B = k === void 0 ? d2 : k, Y = M.getOptionLabel, z = Y === void 0 ? f2 : Y, Z = M.inputValue, V = M.isLoading, Q = M.isMulti, P = M.value, U = M.name, se = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    return b(Z, gb(P), D, {
      getOptionValue: B,
      getOptionLabel: z
    }) ? w(Z, h(Z)) : void 0;
  }, [h, w, z, B, Z, b, D, P]), Se = (0,react__WEBPACK_IMPORTED_MODULE_1__.useMemo)(function() {
    return (u || !V) && se ? c === "first" ? [se].concat(qd(D)) : [].concat(qd(D), [se]) : D;
  }, [u, c, V, se, D]), he = (0,react__WEBPACK_IMPORTED_MODULE_1__.useCallback)(function(xe, ye) {
    if (ye.action !== "select-option")
      return L(xe, ye);
    var Me = Array.isArray(xe) ? xe : [xe];
    if (Me[Me.length - 1] === se) {
      if (R)
        R(Z);
      else {
        var We = w(Z, Z), Ee = {
          action: "create-option",
          name: U,
          option: We
        };
        L(Xh(Q, [].concat(qd(gb(P)), [We]), We), Ee);
      }
      return;
    }
    L(xe, ye);
  }, [w, Z, Q, U, se, R, L, P]);
  return it(it({}, M), {}, {
    options: Se,
    onChange: he
  });
}
var u_ = /* @__PURE__ */ (0,react__WEBPACK_IMPORTED_MODULE_1__.forwardRef)(function(a, r) {
  var u = AN(a), f = o_(u);
  return /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.createElement(hw, st({
    ref: r
  }, f));
}), l_ = u_;
function s_({
  value: a,
  placeholder: r,
  onChange: u
}) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    Ix.Control,
    {
      value: a,
      placeholder: r || "Enter value...",
      onChange: u
    }
  );
}
function OO({
  value: a,
  placeholder: r,
  limit: u,
  onChange: f,
  darkMode: c
}) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    l_,
    {
      isMulti: !0,
      menuPortalTarget: document.body,
      styles: mw(c),
      value: a.map((v) => ({
        value: v,
        label: v
      })),
      delimiter: ",",
      onChange: (v) => f({
        target: {
          value: v.map((h) => h.value).join(",")
        }
      }),
      isOptionDisabled: () => !(u === void 0 || a.length < u),
      noOptionsMessage: () => "Create a new value...",
      placeholder: r || "Enter values..."
    }
  );
}
function c_(a) {
  var f, c, v, h, C;
  let r;
  const u = (b) => b ? b.split(",") : [];
  return a.filter.lookup === "isnull" ? r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    _h,
    {
      options: ["true", "false"],
      value: a.filter.value,
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : ((f = a.projectFields.get(a.filter.field)) == null ? void 0 : f.type) === "choice" ? a.filter.lookup.endsWith("in") ? r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    S2,
    {
      options: ((c = a.projectFields.get(a.filter.field)) == null ? void 0 : c.values) || [],
      value: u(a.filter.value),
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    _h,
    {
      isClearable: !0,
      options: ((v = a.projectFields.get(a.filter.field)) == null ? void 0 : v.values) || [],
      value: a.filter.value,
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : a.filter.lookup.endsWith("in") ? r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    OO,
    {
      value: u(a.filter.value),
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : a.filter.lookup.endsWith("range") ? r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    OO,
    {
      value: u(a.filter.value),
      limit: 2,
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : ((h = a.projectFields.get(a.filter.field)) == null ? void 0 : h.type) === "bool" ? r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
    _h,
    {
      isClearable: !0,
      options: ["true", "false"],
      value: a.filter.value,
      onChange: a.handleValueChange,
      darkMode: a.darkMode
    }
  ) : r = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(s_, { value: a.filter.value, onChange: a.handleValueChange }), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { direction: "horizontal", gap: 1, children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zc, { fluid: !0, className: "g-0", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Lx, { className: "g-1", children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, { sm: 4, children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        _h,
        {
          options: a.fieldList,
          titles: a.fieldDescriptions,
          value: a.filter.field,
          placeholder: "Select field...",
          onChange: a.handleFieldChange,
          darkMode: a.darkMode
        }
      ) }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, { sm: 4, children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        _h,
        {
          options: a.typeLookups.get(
            ((C = a.projectFields.get(a.filter.field)) == null ? void 0 : C.type) || ""
          ) || [],
          titles: a.lookupDescriptions,
          value: a.filter.lookup,
          placeholder: "Select lookup...",
          onChange: a.handleLookupChange,
          darkMode: a.darkMode
        }
      ) }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, { sm: 4, children: r })
    ] }) }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zu, { variant: "primary", onClick: a.handleFilterAdd, children: "+" }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zu, { variant: "danger", onClick: a.handleFilterRemove, children: "-" })
  ] });
}
const f_ = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  className: r,
  striped: u,
  bordered: f,
  borderless: c,
  hover: v,
  size: h,
  variant: C,
  responsive: b,
  ...E
}, w) => {
  const R = ft(a, "table"), A = Ke(r, R, C && `${R}-${C}`, h && `${R}-${h}`, u && `${R}-${typeof u == "string" ? `striped-${u}` : "striped"}`, f && `${R}-bordered`, c && `${R}-borderless`, v && `${R}-hover`), D = /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("table", {
    ...E,
    className: A,
    ref: w
  });
  if (b) {
    let L = `${R}-responsive`;
    return typeof b == "string" && (L = `${L}-${b}`), /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {
      className: L,
      children: D
    });
  }
  return D;
}), d_ = (0,react__WEBPACK_IMPORTED_MODULE_1__.memo)(function({
  data: r,
  titles: u,
  s3PathHandler: f
}) {
  const c = () => r.length > 0 ? Object.keys(r[0]) : [], v = r.map(
    (h) => Object.values(h).map((C) => (C == null ? void 0 : C.toString().trim()) || "")
  );
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(f_, { striped: !0, bordered: !0, hover: !0, responsive: !0, size: "sm", children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("thead", { children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("tr", { children: c().map((h) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("th", { title: u == null ? void 0 : u.get(h), children: h }, h)) }) }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("tbody", { children: v.map((h, C) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("tr", { children: h.map(
      (b, E) => f && b.startsWith("s3://") && b.endsWith(".html") ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("td", { children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zu, { variant: "link", onClick: () => f(b), children: b }) }, E) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("td", { children: b }, E)
    ) }, C)) })
  ] });
}), E2 = /* @__PURE__ */ react__WEBPACK_IMPORTED_MODULE_1__.forwardRef(({
  bsPrefix: a,
  variant: r,
  animation: u = "border",
  size: f,
  // Need to define the default "as" during prop destructuring to be compatible with styled-components github.com/react-bootstrap/react-bootstrap/issues/3595
  as: c = "div",
  className: v,
  ...h
}, C) => {
  a = ft(a, "spinner");
  const b = `${a}-${u}`;
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(c, {
    ref: C,
    ...h,
    className: Ke(v, b, f && `${b}-${f}`, r && `text-${r}`)
  });
});
E2.displayName = "Spinner";
function p_() {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Z0, { variant: "light", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { direction: "horizontal", gap: 2, children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(E2, {}),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { children: "Loading..." })
  ] }) });
}
const v_ = "0.10.0";
function h_(a) {
  const [r, u] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(new Array()), [f, c] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(new Array()), [v, h] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(new Array()), [C, b] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(new Array()), [E, w] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(""), R = Array.from(a.projectFields.entries()).filter(([, P]) => P.actions.includes("filter")).map(([P]) => P), A = Array.from(a.projectFields.entries()).filter(([, P]) => P.actions.includes("list")).map(([P]) => P);
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useLayoutEffect)(() => {
    u([]), c([]), h([]), b([]), w("");
  }, [a.project]);
  const D = (P, U) => {
    var he;
    const se = [...r], Se = a.projectFields.get(P.target.value);
    se[U].field = P.target.value, se[U].lookup = ((he = a.typeLookups.get((Se == null ? void 0 : Se.type) || "")) == null ? void 0 : he[0]) || "", se[U].lookup === "isnull" ? se[U].value = "true" : se[U].value = "", u(se);
  }, L = (P, U) => {
    const se = [...r];
    se[U].lookup = P.target.value, se[U].lookup === "isnull" ? se[U].value = "true" : se[U].value = "", u(se);
  }, M = (P, U) => {
    const se = [...r];
    se[U].value = P.target.value, u(se);
  }, k = (P) => {
    u([
      ...r.slice(0, P),
      { field: "", lookup: "", value: "" },
      ...r.slice(P)
    ]);
  }, B = (P) => {
    const U = [...r];
    U.splice(P, 1), u(U);
  }, Y = () => {
    u([]);
  }, z = (P) => {
    c(P.target.value ? P.target.value.split(",") : []), h([]), b([]);
  }, Z = (P) => {
    c([]), h(P.target.value ? P.target.value.split(",") : []);
  }, V = (P) => {
    c([]), b(P.target.value ? P.target.value.split(",") : []);
  }, Q = () => {
    const P = new URLSearchParams(
      r.filter((U) => U.field).map((U) => U.lookup ? [U.field + "__" + U.lookup, U.value] : [U.field, U.value]).concat(
        v.filter((U) => U).map((U) => ["include", U])
      ).concat(
        C.filter((U) => U).map((U) => ["exclude", U])
      ).concat(
        f.filter((U) => U).map((U) => ["summarise", U])
      ).concat(
        [E].filter((U) => U).map((U) => ["search", U])
      )
    );
    a.handleSearch(P.toString());
  };
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.Fragment, { children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { direction: "horizontal", gap: 2, children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        Ix.Control,
        {
          value: E,
          placeholder: "Search records...",
          onChange: (P) => w(P.target.value),
          onKeyUp: (P) => {
            P.key === "Enter" && Q();
          }
        }
      ),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        zu,
        {
          variant: "primary",
          disabled: !a.project,
          onClick: Q,
          children: "Search"
        }
      )
    ] }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Lx, { className: "g-2", children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, { xl: 6, children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ic, { children: [
        /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ic.Header, { children: [
          /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { children: "Filter" }),
          /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { direction: "horizontal", gap: 1, className: "float-end", children: [
            /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
              zu,
              {
                size: "sm",
                variant: "primary",
                onClick: () => k(r.length),
                children: "Add Filter"
              }
            ),
            /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zu, { size: "sm", variant: "danger", onClick: Y, children: "Clear Filters" })
          ] })
        ] }),
        /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zc, { fluid: !0, className: "panel p-2", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Hu, { gap: 1, children: r.map((P, U) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", { children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
          c_,
          {
            filter: P,
            fieldList: R,
            projectFields: a.projectFields,
            typeLookups: a.typeLookups,
            fieldDescriptions: a.fieldDescriptions,
            lookupDescriptions: a.lookupDescriptions,
            handleFieldChange: (se) => D(se, U),
            handleLookupChange: (se) => L(se, U),
            handleValueChange: (se) => M(se, U),
            handleFilterAdd: () => k(U + 1),
            handleFilterRemove: () => B(U),
            darkMode: a.darkMode
          }
        ) }, U)) }) })
      ] }) }),
      [
        {
          title: "Summarise",
          options: R,
          value: f,
          onChange: z
        },
        {
          title: "Include",
          options: A,
          value: v,
          onChange: Z
        },
        {
          title: "Exclude",
          options: A,
          value: C,
          onChange: V
        }
      ].map(({ title: P, options: U, value: se, onChange: Se }) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Vc, { md: 4, xl: 2, children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ic, { children: [
        /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ic.Header, { children: P }),
        /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zc, { fluid: !0, className: "panel p-2", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
          S2,
          {
            options: U,
            titles: a.fieldDescriptions,
            value: se,
            placeholder: "Select fields...",
            onChange: Se,
            darkMode: a.darkMode
          }
        ) })
      ] }) }, P))
    ] })
  ] });
}
function m_(a) {
  var f;
  const r = Db({
    filename: a.project,
    useKeysAsHeaders: !0
  }), u = () => {
    const c = QV(r)(a.resultData.data || []);
    a.fileWriter ? a.fileWriter(a.project + ".csv", WV(c)) : KV(r)(c);
  };
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ic, { children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Ic.Header, { children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("span", { children: "Results" }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        zu,
        {
          className: "float-end",
          size: "sm",
          variant: "success",
          onClick: u,
          children: "Export Page to CSV"
        }
      )
    ] }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zc, { fluid: !0, className: "table-panel p-2", children: a.resultPending ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(p_, {}) : a.resultError ? /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Z0, { variant: "danger", children: [
      "Error: ",
      a.resultError.message
    ] }) : a.resultData.messages ? Object.entries(a.resultData.messages).map(
      ([c, v]) => Array.isArray(v) ? v.map((h) => /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Z0, { variant: "danger", children: [
        c,
        ": ",
        h
      ] }, c)) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Z0, { variant: "danger", children: [
        c,
        ": ",
        v
      ] }, c)
    ) : /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
      d_,
      {
        data: a.resultData.data || [],
        titles: a.fieldDescriptions,
        s3PathHandler: a.s3PathHandler
      }
    ) }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(Ic.Footer, { children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(z0, { size: "sm", children: [
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        z0.Prev,
        {
          disabled: !a.resultData.previous,
          onClick: () => {
            var c;
            a.handleSearch(
              ((c = a.resultData.previous) == null ? void 0 : c.split("?", 2)[1]) || ""
            );
          }
        }
      ),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(z0.Item, { children: a.resultPending ? "Loading..." : `Showing ${((f = a.resultData.data) == null ? void 0 : f.length) || 0} results` }),
      /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
        z0.Next,
        {
          disabled: !a.resultData.next,
          onClick: () => {
            var c, v;
            a.handleSearch(
              ((v = (c = a.resultData) == null ? void 0 : c.next) == null ? void 0 : v.split("?", 2)[1]) || ""
            );
          }
        }
      )
    ] }) })
  ] });
}
function g_(a) {
  const [r, u] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)("");
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useLayoutEffect)(() => {
    u("");
  }, [a.project]);
  const {
    isFetching: f,
    error: c,
    data: v = {},
    refetch: h
  } = Qd({
    queryKey: ["results", a.project, r],
    queryFn: async () => a.httpPathHandler(`projects/${a.project}/?${r}`).then((b) => b.json()),
    enabled: !!a.project
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(zc, { fluid: !0, className: "g-2", children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { gap: 2, children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(h_, { ...a, handleSearch: (b) => {
      r === b && !f && h(), u(b);
    } }),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
      m_,
      {
        ...a,
        handleSearch: u,
        resultPending: f,
        resultError: c,
        resultData: v
      }
    )
  ] }) });
}
function y_(a) {
  const r = {}, u = (f, c = "") => {
    for (const [v, h] of Object.entries(f))
      r[c + v] = h, h.type === "relation" && u(
        h.fields,
        c + v + "__"
      );
  };
  return u(a), r;
}
function b_(a) {
  const [r, u] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(!1), [f, c] = (0,react__WEBPACK_IMPORTED_MODULE_1__.useState)(""), { data: v = [] } = Qd({
    queryKey: ["projects"],
    queryFn: async () => a.httpPathHandler("projects").then((A) => A.json()).then((A) => [
      ...new Set(
        A.data.map(
          (D) => D.project
        )
      )
    ])
  });
  (0,react__WEBPACK_IMPORTED_MODULE_1__.useEffect)(() => {
    !f && v && c(v[0]);
  }, [f, v]);
  const { data: h = /* @__PURE__ */ new Map() } = Qd({
    queryKey: ["types"],
    queryFn: async () => a.httpPathHandler("projects/types").then((A) => A.json()).then((A) => new Map(
      A.data.map((D) => [
        D.type,
        D.lookups
      ])
    ))
  }), { data: C = /* @__PURE__ */ new Map() } = Qd({
    queryKey: ["lookups"],
    queryFn: async () => a.httpPathHandler("projects/lookups").then((A) => A.json()).then((A) => new Map(
      A.data.map((D) => [
        D.lookup,
        D.description
      ])
    ))
  }), {
    data: { projectName: b, projectFields: E, fieldDescriptions: w } = {
      projectName: "",
      projectFields: /* @__PURE__ */ new Map(),
      fieldDescriptions: /* @__PURE__ */ new Map()
    }
  } = Qd({
    queryKey: ["fields", f],
    queryFn: async () => a.httpPathHandler("projects/" + f + "/fields").then((A) => A.json()).then((A) => {
      const D = y_(A.data.fields), L = A.data.name, M = new Map(
        Object.keys(D).map((B) => [
          B,
          {
            type: D[B].type,
            description: D[B].description,
            actions: D[B].actions,
            values: D[B].values
          }
        ])
      ), k = new Map(
        Array.from(M.entries()).map(([B, Y]) => [
          B,
          Y.description
        ])
      );
      return { projectName: L, projectFields: M, fieldDescriptions: k };
    }),
    enabled: !!f
  });
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsxs)(Hu, { gap: 2, className: "Onyx", children: [
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
      eH,
      {
        ...a,
        projectName: b,
        projectList: v,
        handleProjectChange: c,
        handleThemeChange: () => {
          const A = document.querySelector("html");
          A == null || A.setAttribute("data-bs-theme", r ? "light" : "dark"), u(!r);
        },
        guiVersion: v_
      }
    ),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(
      g_,
      {
        ...a,
        project: f,
        projectFields: E,
        typeLookups: h,
        fieldDescriptions: w,
        lookupDescriptions: C,
        darkMode: r
      }
    ),
    /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)("div", {})
  ] });
}
const C_ = new h9({
  defaultOptions: {
    queries: {
      refetchOnWindowFocus: !1
    }
  }
});
function I_(a) {
  return /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(C9, { client: C_, children: /* @__PURE__ */ (0,react_jsx_runtime__WEBPACK_IMPORTED_MODULE_0__.jsx)(b_, { ...a }) });
}



/***/ }),

/***/ "./node_modules/react/cjs/react-jsx-runtime.development.js":
/*!*****************************************************************!*\
  !*** ./node_modules/react/cjs/react-jsx-runtime.development.js ***!
  \*****************************************************************/
/***/ ((__unused_webpack_module, exports, __webpack_require__) => {

/**
 * @license React
 * react-jsx-runtime.development.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */



if (true) {
  (function() {
'use strict';

var React = __webpack_require__(/*! react */ "webpack/sharing/consume/default/react");

// ATTENTION
// When adding new symbols to this file,
// Please consider also adding to 'react-devtools-shared/src/backend/ReactSymbols'
// The Symbol used to tag the ReactElement-like types.
var REACT_ELEMENT_TYPE = Symbol.for('react.element');
var REACT_PORTAL_TYPE = Symbol.for('react.portal');
var REACT_FRAGMENT_TYPE = Symbol.for('react.fragment');
var REACT_STRICT_MODE_TYPE = Symbol.for('react.strict_mode');
var REACT_PROFILER_TYPE = Symbol.for('react.profiler');
var REACT_PROVIDER_TYPE = Symbol.for('react.provider');
var REACT_CONTEXT_TYPE = Symbol.for('react.context');
var REACT_FORWARD_REF_TYPE = Symbol.for('react.forward_ref');
var REACT_SUSPENSE_TYPE = Symbol.for('react.suspense');
var REACT_SUSPENSE_LIST_TYPE = Symbol.for('react.suspense_list');
var REACT_MEMO_TYPE = Symbol.for('react.memo');
var REACT_LAZY_TYPE = Symbol.for('react.lazy');
var REACT_OFFSCREEN_TYPE = Symbol.for('react.offscreen');
var MAYBE_ITERATOR_SYMBOL = Symbol.iterator;
var FAUX_ITERATOR_SYMBOL = '@@iterator';
function getIteratorFn(maybeIterable) {
  if (maybeIterable === null || typeof maybeIterable !== 'object') {
    return null;
  }

  var maybeIterator = MAYBE_ITERATOR_SYMBOL && maybeIterable[MAYBE_ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL];

  if (typeof maybeIterator === 'function') {
    return maybeIterator;
  }

  return null;
}

var ReactSharedInternals = React.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;

function error(format) {
  {
    {
      for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
        args[_key2 - 1] = arguments[_key2];
      }

      printWarning('error', format, args);
    }
  }
}

function printWarning(level, format, args) {
  // When changing this logic, you might want to also
  // update consoleWithStackDev.www.js as well.
  {
    var ReactDebugCurrentFrame = ReactSharedInternals.ReactDebugCurrentFrame;
    var stack = ReactDebugCurrentFrame.getStackAddendum();

    if (stack !== '') {
      format += '%s';
      args = args.concat([stack]);
    } // eslint-disable-next-line react-internal/safe-string-coercion


    var argsWithFormat = args.map(function (item) {
      return String(item);
    }); // Careful: RN currently depends on this prefix

    argsWithFormat.unshift('Warning: ' + format); // We intentionally don't use spread (or .apply) directly because it
    // breaks IE9: https://github.com/facebook/react/issues/13610
    // eslint-disable-next-line react-internal/no-production-logging

    Function.prototype.apply.call(console[level], console, argsWithFormat);
  }
}

// -----------------------------------------------------------------------------

var enableScopeAPI = false; // Experimental Create Event Handle API.
var enableCacheElement = false;
var enableTransitionTracing = false; // No known bugs, but needs performance testing

var enableLegacyHidden = false; // Enables unstable_avoidThisFallback feature in Fiber
// stuff. Intended to enable React core members to more easily debug scheduling
// issues in DEV builds.

var enableDebugTracing = false; // Track which Fiber(s) schedule render work.

var REACT_MODULE_REFERENCE;

{
  REACT_MODULE_REFERENCE = Symbol.for('react.module.reference');
}

function isValidElementType(type) {
  if (typeof type === 'string' || typeof type === 'function') {
    return true;
  } // Note: typeof might be other than 'symbol' or 'number' (e.g. if it's a polyfill).


  if (type === REACT_FRAGMENT_TYPE || type === REACT_PROFILER_TYPE || enableDebugTracing  || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || enableLegacyHidden  || type === REACT_OFFSCREEN_TYPE || enableScopeAPI  || enableCacheElement  || enableTransitionTracing ) {
    return true;
  }

  if (typeof type === 'object' && type !== null) {
    if (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || // This needs to include all possible module reference object
    // types supported by any Flight configuration anywhere since
    // we don't know which Flight build this will end up being used
    // with.
    type.$$typeof === REACT_MODULE_REFERENCE || type.getModuleId !== undefined) {
      return true;
    }
  }

  return false;
}

function getWrappedName(outerType, innerType, wrapperName) {
  var displayName = outerType.displayName;

  if (displayName) {
    return displayName;
  }

  var functionName = innerType.displayName || innerType.name || '';
  return functionName !== '' ? wrapperName + "(" + functionName + ")" : wrapperName;
} // Keep in sync with react-reconciler/getComponentNameFromFiber


function getContextName(type) {
  return type.displayName || 'Context';
} // Note that the reconciler package should generally prefer to use getComponentNameFromFiber() instead.


function getComponentNameFromType(type) {
  if (type == null) {
    // Host root, text node or just invalid type.
    return null;
  }

  {
    if (typeof type.tag === 'number') {
      error('Received an unexpected object in getComponentNameFromType(). ' + 'This is likely a bug in React. Please file an issue.');
    }
  }

  if (typeof type === 'function') {
    return type.displayName || type.name || null;
  }

  if (typeof type === 'string') {
    return type;
  }

  switch (type) {
    case REACT_FRAGMENT_TYPE:
      return 'Fragment';

    case REACT_PORTAL_TYPE:
      return 'Portal';

    case REACT_PROFILER_TYPE:
      return 'Profiler';

    case REACT_STRICT_MODE_TYPE:
      return 'StrictMode';

    case REACT_SUSPENSE_TYPE:
      return 'Suspense';

    case REACT_SUSPENSE_LIST_TYPE:
      return 'SuspenseList';

  }

  if (typeof type === 'object') {
    switch (type.$$typeof) {
      case REACT_CONTEXT_TYPE:
        var context = type;
        return getContextName(context) + '.Consumer';

      case REACT_PROVIDER_TYPE:
        var provider = type;
        return getContextName(provider._context) + '.Provider';

      case REACT_FORWARD_REF_TYPE:
        return getWrappedName(type, type.render, 'ForwardRef');

      case REACT_MEMO_TYPE:
        var outerName = type.displayName || null;

        if (outerName !== null) {
          return outerName;
        }

        return getComponentNameFromType(type.type) || 'Memo';

      case REACT_LAZY_TYPE:
        {
          var lazyComponent = type;
          var payload = lazyComponent._payload;
          var init = lazyComponent._init;

          try {
            return getComponentNameFromType(init(payload));
          } catch (x) {
            return null;
          }
        }

      // eslint-disable-next-line no-fallthrough
    }
  }

  return null;
}

var assign = Object.assign;

// Helpers to patch console.logs to avoid logging during side-effect free
// replaying on render function. This currently only patches the object
// lazily which won't cover if the log function was extracted eagerly.
// We could also eagerly patch the method.
var disabledDepth = 0;
var prevLog;
var prevInfo;
var prevWarn;
var prevError;
var prevGroup;
var prevGroupCollapsed;
var prevGroupEnd;

function disabledLog() {}

disabledLog.__reactDisabledLog = true;
function disableLogs() {
  {
    if (disabledDepth === 0) {
      /* eslint-disable react-internal/no-production-logging */
      prevLog = console.log;
      prevInfo = console.info;
      prevWarn = console.warn;
      prevError = console.error;
      prevGroup = console.group;
      prevGroupCollapsed = console.groupCollapsed;
      prevGroupEnd = console.groupEnd; // https://github.com/facebook/react/issues/19099

      var props = {
        configurable: true,
        enumerable: true,
        value: disabledLog,
        writable: true
      }; // $FlowFixMe Flow thinks console is immutable.

      Object.defineProperties(console, {
        info: props,
        log: props,
        warn: props,
        error: props,
        group: props,
        groupCollapsed: props,
        groupEnd: props
      });
      /* eslint-enable react-internal/no-production-logging */
    }

    disabledDepth++;
  }
}
function reenableLogs() {
  {
    disabledDepth--;

    if (disabledDepth === 0) {
      /* eslint-disable react-internal/no-production-logging */
      var props = {
        configurable: true,
        enumerable: true,
        writable: true
      }; // $FlowFixMe Flow thinks console is immutable.

      Object.defineProperties(console, {
        log: assign({}, props, {
          value: prevLog
        }),
        info: assign({}, props, {
          value: prevInfo
        }),
        warn: assign({}, props, {
          value: prevWarn
        }),
        error: assign({}, props, {
          value: prevError
        }),
        group: assign({}, props, {
          value: prevGroup
        }),
        groupCollapsed: assign({}, props, {
          value: prevGroupCollapsed
        }),
        groupEnd: assign({}, props, {
          value: prevGroupEnd
        })
      });
      /* eslint-enable react-internal/no-production-logging */
    }

    if (disabledDepth < 0) {
      error('disabledDepth fell below zero. ' + 'This is a bug in React. Please file an issue.');
    }
  }
}

var ReactCurrentDispatcher = ReactSharedInternals.ReactCurrentDispatcher;
var prefix;
function describeBuiltInComponentFrame(name, source, ownerFn) {
  {
    if (prefix === undefined) {
      // Extract the VM specific prefix used by each line.
      try {
        throw Error();
      } catch (x) {
        var match = x.stack.trim().match(/\n( *(at )?)/);
        prefix = match && match[1] || '';
      }
    } // We use the prefix to ensure our stacks line up with native stack frames.


    return '\n' + prefix + name;
  }
}
var reentry = false;
var componentFrameCache;

{
  var PossiblyWeakMap = typeof WeakMap === 'function' ? WeakMap : Map;
  componentFrameCache = new PossiblyWeakMap();
}

function describeNativeComponentFrame(fn, construct) {
  // If something asked for a stack inside a fake render, it should get ignored.
  if ( !fn || reentry) {
    return '';
  }

  {
    var frame = componentFrameCache.get(fn);

    if (frame !== undefined) {
      return frame;
    }
  }

  var control;
  reentry = true;
  var previousPrepareStackTrace = Error.prepareStackTrace; // $FlowFixMe It does accept undefined.

  Error.prepareStackTrace = undefined;
  var previousDispatcher;

  {
    previousDispatcher = ReactCurrentDispatcher.current; // Set the dispatcher in DEV because this might be call in the render function
    // for warnings.

    ReactCurrentDispatcher.current = null;
    disableLogs();
  }

  try {
    // This should throw.
    if (construct) {
      // Something should be setting the props in the constructor.
      var Fake = function () {
        throw Error();
      }; // $FlowFixMe


      Object.defineProperty(Fake.prototype, 'props', {
        set: function () {
          // We use a throwing setter instead of frozen or non-writable props
          // because that won't throw in a non-strict mode function.
          throw Error();
        }
      });

      if (typeof Reflect === 'object' && Reflect.construct) {
        // We construct a different control for this case to include any extra
        // frames added by the construct call.
        try {
          Reflect.construct(Fake, []);
        } catch (x) {
          control = x;
        }

        Reflect.construct(fn, [], Fake);
      } else {
        try {
          Fake.call();
        } catch (x) {
          control = x;
        }

        fn.call(Fake.prototype);
      }
    } else {
      try {
        throw Error();
      } catch (x) {
        control = x;
      }

      fn();
    }
  } catch (sample) {
    // This is inlined manually because closure doesn't do it for us.
    if (sample && control && typeof sample.stack === 'string') {
      // This extracts the first frame from the sample that isn't also in the control.
      // Skipping one frame that we assume is the frame that calls the two.
      var sampleLines = sample.stack.split('\n');
      var controlLines = control.stack.split('\n');
      var s = sampleLines.length - 1;
      var c = controlLines.length - 1;

      while (s >= 1 && c >= 0 && sampleLines[s] !== controlLines[c]) {
        // We expect at least one stack frame to be shared.
        // Typically this will be the root most one. However, stack frames may be
        // cut off due to maximum stack limits. In this case, one maybe cut off
        // earlier than the other. We assume that the sample is longer or the same
        // and there for cut off earlier. So we should find the root most frame in
        // the sample somewhere in the control.
        c--;
      }

      for (; s >= 1 && c >= 0; s--, c--) {
        // Next we find the first one that isn't the same which should be the
        // frame that called our sample function and the control.
        if (sampleLines[s] !== controlLines[c]) {
          // In V8, the first line is describing the message but other VMs don't.
          // If we're about to return the first line, and the control is also on the same
          // line, that's a pretty good indicator that our sample threw at same line as
          // the control. I.e. before we entered the sample frame. So we ignore this result.
          // This can happen if you passed a class to function component, or non-function.
          if (s !== 1 || c !== 1) {
            do {
              s--;
              c--; // We may still have similar intermediate frames from the construct call.
              // The next one that isn't the same should be our match though.

              if (c < 0 || sampleLines[s] !== controlLines[c]) {
                // V8 adds a "new" prefix for native classes. Let's remove it to make it prettier.
                var _frame = '\n' + sampleLines[s].replace(' at new ', ' at '); // If our component frame is labeled "<anonymous>"
                // but we have a user-provided "displayName"
                // splice it in to make the stack more readable.


                if (fn.displayName && _frame.includes('<anonymous>')) {
                  _frame = _frame.replace('<anonymous>', fn.displayName);
                }

                {
                  if (typeof fn === 'function') {
                    componentFrameCache.set(fn, _frame);
                  }
                } // Return the line we found.


                return _frame;
              }
            } while (s >= 1 && c >= 0);
          }

          break;
        }
      }
    }
  } finally {
    reentry = false;

    {
      ReactCurrentDispatcher.current = previousDispatcher;
      reenableLogs();
    }

    Error.prepareStackTrace = previousPrepareStackTrace;
  } // Fallback to just using the name if we couldn't make it throw.


  var name = fn ? fn.displayName || fn.name : '';
  var syntheticFrame = name ? describeBuiltInComponentFrame(name) : '';

  {
    if (typeof fn === 'function') {
      componentFrameCache.set(fn, syntheticFrame);
    }
  }

  return syntheticFrame;
}
function describeFunctionComponentFrame(fn, source, ownerFn) {
  {
    return describeNativeComponentFrame(fn, false);
  }
}

function shouldConstruct(Component) {
  var prototype = Component.prototype;
  return !!(prototype && prototype.isReactComponent);
}

function describeUnknownElementTypeFrameInDEV(type, source, ownerFn) {

  if (type == null) {
    return '';
  }

  if (typeof type === 'function') {
    {
      return describeNativeComponentFrame(type, shouldConstruct(type));
    }
  }

  if (typeof type === 'string') {
    return describeBuiltInComponentFrame(type);
  }

  switch (type) {
    case REACT_SUSPENSE_TYPE:
      return describeBuiltInComponentFrame('Suspense');

    case REACT_SUSPENSE_LIST_TYPE:
      return describeBuiltInComponentFrame('SuspenseList');
  }

  if (typeof type === 'object') {
    switch (type.$$typeof) {
      case REACT_FORWARD_REF_TYPE:
        return describeFunctionComponentFrame(type.render);

      case REACT_MEMO_TYPE:
        // Memo may contain any component type so we recursively resolve it.
        return describeUnknownElementTypeFrameInDEV(type.type, source, ownerFn);

      case REACT_LAZY_TYPE:
        {
          var lazyComponent = type;
          var payload = lazyComponent._payload;
          var init = lazyComponent._init;

          try {
            // Lazy may contain any component type so we recursively resolve it.
            return describeUnknownElementTypeFrameInDEV(init(payload), source, ownerFn);
          } catch (x) {}
        }
    }
  }

  return '';
}

var hasOwnProperty = Object.prototype.hasOwnProperty;

var loggedTypeFailures = {};
var ReactDebugCurrentFrame = ReactSharedInternals.ReactDebugCurrentFrame;

function setCurrentlyValidatingElement(element) {
  {
    if (element) {
      var owner = element._owner;
      var stack = describeUnknownElementTypeFrameInDEV(element.type, element._source, owner ? owner.type : null);
      ReactDebugCurrentFrame.setExtraStackFrame(stack);
    } else {
      ReactDebugCurrentFrame.setExtraStackFrame(null);
    }
  }
}

function checkPropTypes(typeSpecs, values, location, componentName, element) {
  {
    // $FlowFixMe This is okay but Flow doesn't know it.
    var has = Function.call.bind(hasOwnProperty);

    for (var typeSpecName in typeSpecs) {
      if (has(typeSpecs, typeSpecName)) {
        var error$1 = void 0; // Prop type validation may throw. In case they do, we don't want to
        // fail the render phase where it didn't fail before. So we log it.
        // After these have been cleaned up, we'll let them throw.

        try {
          // This is intentionally an invariant that gets caught. It's the same
          // behavior as without this statement except with a better message.
          if (typeof typeSpecs[typeSpecName] !== 'function') {
            // eslint-disable-next-line react-internal/prod-error-codes
            var err = Error((componentName || 'React class') + ': ' + location + ' type `' + typeSpecName + '` is invalid; ' + 'it must be a function, usually from the `prop-types` package, but received `' + typeof typeSpecs[typeSpecName] + '`.' + 'This often happens because of typos such as `PropTypes.function` instead of `PropTypes.func`.');
            err.name = 'Invariant Violation';
            throw err;
          }

          error$1 = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED');
        } catch (ex) {
          error$1 = ex;
        }

        if (error$1 && !(error$1 instanceof Error)) {
          setCurrentlyValidatingElement(element);

          error('%s: type specification of %s' + ' `%s` is invalid; the type checker ' + 'function must return `null` or an `Error` but returned a %s. ' + 'You may have forgotten to pass an argument to the type checker ' + 'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' + 'shape all require an argument).', componentName || 'React class', location, typeSpecName, typeof error$1);

          setCurrentlyValidatingElement(null);
        }

        if (error$1 instanceof Error && !(error$1.message in loggedTypeFailures)) {
          // Only monitor this failure once because there tends to be a lot of the
          // same error.
          loggedTypeFailures[error$1.message] = true;
          setCurrentlyValidatingElement(element);

          error('Failed %s type: %s', location, error$1.message);

          setCurrentlyValidatingElement(null);
        }
      }
    }
  }
}

var isArrayImpl = Array.isArray; // eslint-disable-next-line no-redeclare

function isArray(a) {
  return isArrayImpl(a);
}

/*
 * The `'' + value` pattern (used in in perf-sensitive code) throws for Symbol
 * and Temporal.* types. See https://github.com/facebook/react/pull/22064.
 *
 * The functions in this module will throw an easier-to-understand,
 * easier-to-debug exception with a clear errors message message explaining the
 * problem. (Instead of a confusing exception thrown inside the implementation
 * of the `value` object).
 */
// $FlowFixMe only called in DEV, so void return is not possible.
function typeName(value) {
  {
    // toStringTag is needed for namespaced types like Temporal.Instant
    var hasToStringTag = typeof Symbol === 'function' && Symbol.toStringTag;
    var type = hasToStringTag && value[Symbol.toStringTag] || value.constructor.name || 'Object';
    return type;
  }
} // $FlowFixMe only called in DEV, so void return is not possible.


function willCoercionThrow(value) {
  {
    try {
      testStringCoercion(value);
      return false;
    } catch (e) {
      return true;
    }
  }
}

function testStringCoercion(value) {
  // If you ended up here by following an exception call stack, here's what's
  // happened: you supplied an object or symbol value to React (as a prop, key,
  // DOM attribute, CSS property, string ref, etc.) and when React tried to
  // coerce it to a string using `'' + value`, an exception was thrown.
  //
  // The most common types that will cause this exception are `Symbol` instances
  // and Temporal objects like `Temporal.Instant`. But any object that has a
  // `valueOf` or `[Symbol.toPrimitive]` method that throws will also cause this
  // exception. (Library authors do this to prevent users from using built-in
  // numeric operators like `+` or comparison operators like `>=` because custom
  // methods are needed to perform accurate arithmetic or comparison.)
  //
  // To fix the problem, coerce this object or symbol value to a string before
  // passing it to React. The most reliable way is usually `String(value)`.
  //
  // To find which value is throwing, check the browser or debugger console.
  // Before this exception was thrown, there should be `console.error` output
  // that shows the type (Symbol, Temporal.PlainDate, etc.) that caused the
  // problem and how that type was used: key, atrribute, input value prop, etc.
  // In most cases, this console output also shows the component and its
  // ancestor components where the exception happened.
  //
  // eslint-disable-next-line react-internal/safe-string-coercion
  return '' + value;
}
function checkKeyStringCoercion(value) {
  {
    if (willCoercionThrow(value)) {
      error('The provided key is an unsupported type %s.' + ' This value must be coerced to a string before before using it here.', typeName(value));

      return testStringCoercion(value); // throw (to help callers find troubleshooting comments)
    }
  }
}

var ReactCurrentOwner = ReactSharedInternals.ReactCurrentOwner;
var RESERVED_PROPS = {
  key: true,
  ref: true,
  __self: true,
  __source: true
};
var specialPropKeyWarningShown;
var specialPropRefWarningShown;
var didWarnAboutStringRefs;

{
  didWarnAboutStringRefs = {};
}

function hasValidRef(config) {
  {
    if (hasOwnProperty.call(config, 'ref')) {
      var getter = Object.getOwnPropertyDescriptor(config, 'ref').get;

      if (getter && getter.isReactWarning) {
        return false;
      }
    }
  }

  return config.ref !== undefined;
}

function hasValidKey(config) {
  {
    if (hasOwnProperty.call(config, 'key')) {
      var getter = Object.getOwnPropertyDescriptor(config, 'key').get;

      if (getter && getter.isReactWarning) {
        return false;
      }
    }
  }

  return config.key !== undefined;
}

function warnIfStringRefCannotBeAutoConverted(config, self) {
  {
    if (typeof config.ref === 'string' && ReactCurrentOwner.current && self && ReactCurrentOwner.current.stateNode !== self) {
      var componentName = getComponentNameFromType(ReactCurrentOwner.current.type);

      if (!didWarnAboutStringRefs[componentName]) {
        error('Component "%s" contains the string ref "%s". ' + 'Support for string refs will be removed in a future major release. ' + 'This case cannot be automatically converted to an arrow function. ' + 'We ask you to manually fix this case by using useRef() or createRef() instead. ' + 'Learn more about using refs safely here: ' + 'https://reactjs.org/link/strict-mode-string-ref', getComponentNameFromType(ReactCurrentOwner.current.type), config.ref);

        didWarnAboutStringRefs[componentName] = true;
      }
    }
  }
}

function defineKeyPropWarningGetter(props, displayName) {
  {
    var warnAboutAccessingKey = function () {
      if (!specialPropKeyWarningShown) {
        specialPropKeyWarningShown = true;

        error('%s: `key` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://reactjs.org/link/special-props)', displayName);
      }
    };

    warnAboutAccessingKey.isReactWarning = true;
    Object.defineProperty(props, 'key', {
      get: warnAboutAccessingKey,
      configurable: true
    });
  }
}

function defineRefPropWarningGetter(props, displayName) {
  {
    var warnAboutAccessingRef = function () {
      if (!specialPropRefWarningShown) {
        specialPropRefWarningShown = true;

        error('%s: `ref` is not a prop. Trying to access it will result ' + 'in `undefined` being returned. If you need to access the same ' + 'value within the child component, you should pass it as a different ' + 'prop. (https://reactjs.org/link/special-props)', displayName);
      }
    };

    warnAboutAccessingRef.isReactWarning = true;
    Object.defineProperty(props, 'ref', {
      get: warnAboutAccessingRef,
      configurable: true
    });
  }
}
/**
 * Factory method to create a new React element. This no longer adheres to
 * the class pattern, so do not use new to call it. Also, instanceof check
 * will not work. Instead test $$typeof field against Symbol.for('react.element') to check
 * if something is a React Element.
 *
 * @param {*} type
 * @param {*} props
 * @param {*} key
 * @param {string|object} ref
 * @param {*} owner
 * @param {*} self A *temporary* helper to detect places where `this` is
 * different from the `owner` when React.createElement is called, so that we
 * can warn. We want to get rid of owner and replace string `ref`s with arrow
 * functions, and as long as `this` and owner are the same, there will be no
 * change in behavior.
 * @param {*} source An annotation object (added by a transpiler or otherwise)
 * indicating filename, line number, and/or other information.
 * @internal
 */


var ReactElement = function (type, key, ref, self, source, owner, props) {
  var element = {
    // This tag allows us to uniquely identify this as a React Element
    $$typeof: REACT_ELEMENT_TYPE,
    // Built-in properties that belong on the element
    type: type,
    key: key,
    ref: ref,
    props: props,
    // Record the component responsible for creating this element.
    _owner: owner
  };

  {
    // The validation flag is currently mutative. We put it on
    // an external backing store so that we can freeze the whole object.
    // This can be replaced with a WeakMap once they are implemented in
    // commonly used development environments.
    element._store = {}; // To make comparing ReactElements easier for testing purposes, we make
    // the validation flag non-enumerable (where possible, which should
    // include every environment we run tests in), so the test framework
    // ignores it.

    Object.defineProperty(element._store, 'validated', {
      configurable: false,
      enumerable: false,
      writable: true,
      value: false
    }); // self and source are DEV only properties.

    Object.defineProperty(element, '_self', {
      configurable: false,
      enumerable: false,
      writable: false,
      value: self
    }); // Two elements created in two different places should be considered
    // equal for testing purposes and therefore we hide it from enumeration.

    Object.defineProperty(element, '_source', {
      configurable: false,
      enumerable: false,
      writable: false,
      value: source
    });

    if (Object.freeze) {
      Object.freeze(element.props);
      Object.freeze(element);
    }
  }

  return element;
};
/**
 * https://github.com/reactjs/rfcs/pull/107
 * @param {*} type
 * @param {object} props
 * @param {string} key
 */

function jsxDEV(type, config, maybeKey, source, self) {
  {
    var propName; // Reserved names are extracted

    var props = {};
    var key = null;
    var ref = null; // Currently, key can be spread in as a prop. This causes a potential
    // issue if key is also explicitly declared (ie. <div {...props} key="Hi" />
    // or <div key="Hi" {...props} /> ). We want to deprecate key spread,
    // but as an intermediary step, we will use jsxDEV for everything except
    // <div {...props} key="Hi" />, because we aren't currently able to tell if
    // key is explicitly declared to be undefined or not.

    if (maybeKey !== undefined) {
      {
        checkKeyStringCoercion(maybeKey);
      }

      key = '' + maybeKey;
    }

    if (hasValidKey(config)) {
      {
        checkKeyStringCoercion(config.key);
      }

      key = '' + config.key;
    }

    if (hasValidRef(config)) {
      ref = config.ref;
      warnIfStringRefCannotBeAutoConverted(config, self);
    } // Remaining properties are added to a new props object


    for (propName in config) {
      if (hasOwnProperty.call(config, propName) && !RESERVED_PROPS.hasOwnProperty(propName)) {
        props[propName] = config[propName];
      }
    } // Resolve default props


    if (type && type.defaultProps) {
      var defaultProps = type.defaultProps;

      for (propName in defaultProps) {
        if (props[propName] === undefined) {
          props[propName] = defaultProps[propName];
        }
      }
    }

    if (key || ref) {
      var displayName = typeof type === 'function' ? type.displayName || type.name || 'Unknown' : type;

      if (key) {
        defineKeyPropWarningGetter(props, displayName);
      }

      if (ref) {
        defineRefPropWarningGetter(props, displayName);
      }
    }

    return ReactElement(type, key, ref, self, source, ReactCurrentOwner.current, props);
  }
}

var ReactCurrentOwner$1 = ReactSharedInternals.ReactCurrentOwner;
var ReactDebugCurrentFrame$1 = ReactSharedInternals.ReactDebugCurrentFrame;

function setCurrentlyValidatingElement$1(element) {
  {
    if (element) {
      var owner = element._owner;
      var stack = describeUnknownElementTypeFrameInDEV(element.type, element._source, owner ? owner.type : null);
      ReactDebugCurrentFrame$1.setExtraStackFrame(stack);
    } else {
      ReactDebugCurrentFrame$1.setExtraStackFrame(null);
    }
  }
}

var propTypesMisspellWarningShown;

{
  propTypesMisspellWarningShown = false;
}
/**
 * Verifies the object is a ReactElement.
 * See https://reactjs.org/docs/react-api.html#isvalidelement
 * @param {?object} object
 * @return {boolean} True if `object` is a ReactElement.
 * @final
 */


function isValidElement(object) {
  {
    return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
  }
}

function getDeclarationErrorAddendum() {
  {
    if (ReactCurrentOwner$1.current) {
      var name = getComponentNameFromType(ReactCurrentOwner$1.current.type);

      if (name) {
        return '\n\nCheck the render method of `' + name + '`.';
      }
    }

    return '';
  }
}

function getSourceInfoErrorAddendum(source) {
  {
    if (source !== undefined) {
      var fileName = source.fileName.replace(/^.*[\\\/]/, '');
      var lineNumber = source.lineNumber;
      return '\n\nCheck your code at ' + fileName + ':' + lineNumber + '.';
    }

    return '';
  }
}
/**
 * Warn if there's no key explicitly set on dynamic arrays of children or
 * object keys are not valid. This allows us to keep track of children between
 * updates.
 */


var ownerHasKeyUseWarning = {};

function getCurrentComponentErrorInfo(parentType) {
  {
    var info = getDeclarationErrorAddendum();

    if (!info) {
      var parentName = typeof parentType === 'string' ? parentType : parentType.displayName || parentType.name;

      if (parentName) {
        info = "\n\nCheck the top-level render call using <" + parentName + ">.";
      }
    }

    return info;
  }
}
/**
 * Warn if the element doesn't have an explicit key assigned to it.
 * This element is in an array. The array could grow and shrink or be
 * reordered. All children that haven't already been validated are required to
 * have a "key" property assigned to it. Error statuses are cached so a warning
 * will only be shown once.
 *
 * @internal
 * @param {ReactElement} element Element that requires a key.
 * @param {*} parentType element's parent's type.
 */


function validateExplicitKey(element, parentType) {
  {
    if (!element._store || element._store.validated || element.key != null) {
      return;
    }

    element._store.validated = true;
    var currentComponentErrorInfo = getCurrentComponentErrorInfo(parentType);

    if (ownerHasKeyUseWarning[currentComponentErrorInfo]) {
      return;
    }

    ownerHasKeyUseWarning[currentComponentErrorInfo] = true; // Usually the current owner is the offender, but if it accepts children as a
    // property, it may be the creator of the child that's responsible for
    // assigning it a key.

    var childOwner = '';

    if (element && element._owner && element._owner !== ReactCurrentOwner$1.current) {
      // Give the component that originally created this child.
      childOwner = " It was passed a child from " + getComponentNameFromType(element._owner.type) + ".";
    }

    setCurrentlyValidatingElement$1(element);

    error('Each child in a list should have a unique "key" prop.' + '%s%s See https://reactjs.org/link/warning-keys for more information.', currentComponentErrorInfo, childOwner);

    setCurrentlyValidatingElement$1(null);
  }
}
/**
 * Ensure that every element either is passed in a static location, in an
 * array with an explicit keys property defined, or in an object literal
 * with valid key property.
 *
 * @internal
 * @param {ReactNode} node Statically passed child of any type.
 * @param {*} parentType node's parent's type.
 */


function validateChildKeys(node, parentType) {
  {
    if (typeof node !== 'object') {
      return;
    }

    if (isArray(node)) {
      for (var i = 0; i < node.length; i++) {
        var child = node[i];

        if (isValidElement(child)) {
          validateExplicitKey(child, parentType);
        }
      }
    } else if (isValidElement(node)) {
      // This element was passed in a valid location.
      if (node._store) {
        node._store.validated = true;
      }
    } else if (node) {
      var iteratorFn = getIteratorFn(node);

      if (typeof iteratorFn === 'function') {
        // Entry iterators used to provide implicit keys,
        // but now we print a separate warning for them later.
        if (iteratorFn !== node.entries) {
          var iterator = iteratorFn.call(node);
          var step;

          while (!(step = iterator.next()).done) {
            if (isValidElement(step.value)) {
              validateExplicitKey(step.value, parentType);
            }
          }
        }
      }
    }
  }
}
/**
 * Given an element, validate that its props follow the propTypes definition,
 * provided by the type.
 *
 * @param {ReactElement} element
 */


function validatePropTypes(element) {
  {
    var type = element.type;

    if (type === null || type === undefined || typeof type === 'string') {
      return;
    }

    var propTypes;

    if (typeof type === 'function') {
      propTypes = type.propTypes;
    } else if (typeof type === 'object' && (type.$$typeof === REACT_FORWARD_REF_TYPE || // Note: Memo only checks outer props here.
    // Inner props are checked in the reconciler.
    type.$$typeof === REACT_MEMO_TYPE)) {
      propTypes = type.propTypes;
    } else {
      return;
    }

    if (propTypes) {
      // Intentionally inside to avoid triggering lazy initializers:
      var name = getComponentNameFromType(type);
      checkPropTypes(propTypes, element.props, 'prop', name, element);
    } else if (type.PropTypes !== undefined && !propTypesMisspellWarningShown) {
      propTypesMisspellWarningShown = true; // Intentionally inside to avoid triggering lazy initializers:

      var _name = getComponentNameFromType(type);

      error('Component %s declared `PropTypes` instead of `propTypes`. Did you misspell the property assignment?', _name || 'Unknown');
    }

    if (typeof type.getDefaultProps === 'function' && !type.getDefaultProps.isReactClassApproved) {
      error('getDefaultProps is only used on classic React.createClass ' + 'definitions. Use a static property named `defaultProps` instead.');
    }
  }
}
/**
 * Given a fragment, validate that it can only be provided with fragment props
 * @param {ReactElement} fragment
 */


function validateFragmentProps(fragment) {
  {
    var keys = Object.keys(fragment.props);

    for (var i = 0; i < keys.length; i++) {
      var key = keys[i];

      if (key !== 'children' && key !== 'key') {
        setCurrentlyValidatingElement$1(fragment);

        error('Invalid prop `%s` supplied to `React.Fragment`. ' + 'React.Fragment can only have `key` and `children` props.', key);

        setCurrentlyValidatingElement$1(null);
        break;
      }
    }

    if (fragment.ref !== null) {
      setCurrentlyValidatingElement$1(fragment);

      error('Invalid attribute `ref` supplied to `React.Fragment`.');

      setCurrentlyValidatingElement$1(null);
    }
  }
}

var didWarnAboutKeySpread = {};
function jsxWithValidation(type, props, key, isStaticChildren, source, self) {
  {
    var validType = isValidElementType(type); // We warn in this case but don't throw. We expect the element creation to
    // succeed and there will likely be errors in render.

    if (!validType) {
      var info = '';

      if (type === undefined || typeof type === 'object' && type !== null && Object.keys(type).length === 0) {
        info += ' You likely forgot to export your component from the file ' + "it's defined in, or you might have mixed up default and named imports.";
      }

      var sourceInfo = getSourceInfoErrorAddendum(source);

      if (sourceInfo) {
        info += sourceInfo;
      } else {
        info += getDeclarationErrorAddendum();
      }

      var typeString;

      if (type === null) {
        typeString = 'null';
      } else if (isArray(type)) {
        typeString = 'array';
      } else if (type !== undefined && type.$$typeof === REACT_ELEMENT_TYPE) {
        typeString = "<" + (getComponentNameFromType(type.type) || 'Unknown') + " />";
        info = ' Did you accidentally export a JSX literal instead of a component?';
      } else {
        typeString = typeof type;
      }

      error('React.jsx: type is invalid -- expected a string (for ' + 'built-in components) or a class/function (for composite ' + 'components) but got: %s.%s', typeString, info);
    }

    var element = jsxDEV(type, props, key, source, self); // The result can be nullish if a mock or a custom function is used.
    // TODO: Drop this when these are no longer allowed as the type argument.

    if (element == null) {
      return element;
    } // Skip key warning if the type isn't valid since our key validation logic
    // doesn't expect a non-string/function type and can throw confusing errors.
    // We don't want exception behavior to differ between dev and prod.
    // (Rendering will throw with a helpful message and as soon as the type is
    // fixed, the key warnings will appear.)


    if (validType) {
      var children = props.children;

      if (children !== undefined) {
        if (isStaticChildren) {
          if (isArray(children)) {
            for (var i = 0; i < children.length; i++) {
              validateChildKeys(children[i], type);
            }

            if (Object.freeze) {
              Object.freeze(children);
            }
          } else {
            error('React.jsx: Static children should always be an array. ' + 'You are likely explicitly calling React.jsxs or React.jsxDEV. ' + 'Use the Babel transform instead.');
          }
        } else {
          validateChildKeys(children, type);
        }
      }
    }

    {
      if (hasOwnProperty.call(props, 'key')) {
        var componentName = getComponentNameFromType(type);
        var keys = Object.keys(props).filter(function (k) {
          return k !== 'key';
        });
        var beforeExample = keys.length > 0 ? '{key: someKey, ' + keys.join(': ..., ') + ': ...}' : '{key: someKey}';

        if (!didWarnAboutKeySpread[componentName + beforeExample]) {
          var afterExample = keys.length > 0 ? '{' + keys.join(': ..., ') + ': ...}' : '{}';

          error('A props object containing a "key" prop is being spread into JSX:\n' + '  let props = %s;\n' + '  <%s {...props} />\n' + 'React keys must be passed directly to JSX without using spread:\n' + '  let props = %s;\n' + '  <%s key={someKey} {...props} />', beforeExample, componentName, afterExample, componentName);

          didWarnAboutKeySpread[componentName + beforeExample] = true;
        }
      }
    }

    if (type === REACT_FRAGMENT_TYPE) {
      validateFragmentProps(element);
    } else {
      validatePropTypes(element);
    }

    return element;
  }
} // These two functions exist to still get child warnings in dev
// even with the prod transform. This means that jsxDEV is purely
// opt-in behavior for better messages but that we won't stop
// giving you warnings if you use production apis.

function jsxWithValidationStatic(type, props, key) {
  {
    return jsxWithValidation(type, props, key, true);
  }
}
function jsxWithValidationDynamic(type, props, key) {
  {
    return jsxWithValidation(type, props, key, false);
  }
}

var jsx =  jsxWithValidationDynamic ; // we may want to special case jsxs internally to take advantage of static children.
// for now we can ship identical prod functions

var jsxs =  jsxWithValidationStatic ;

exports.Fragment = REACT_FRAGMENT_TYPE;
exports.jsx = jsx;
exports.jsxs = jsxs;
  })();
}


/***/ }),

/***/ "./node_modules/react/jsx-runtime.js":
/*!*******************************************!*\
  !*** ./node_modules/react/jsx-runtime.js ***!
  \*******************************************/
/***/ ((module, __unused_webpack_exports, __webpack_require__) => {



if (false) {} else {
  module.exports = __webpack_require__(/*! ./cjs/react-jsx-runtime.development.js */ "./node_modules/react/cjs/react-jsx-runtime.development.js");
}


/***/ })

}]);
//# sourceMappingURL=vendors-node_modules_climb-onyx-gui_dist_climb-onyx-gui_js.dca7103240439ca13317.js.map